;**********************************************************************;
;*                           M A K R O K E Y                          *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : demonstriert die Definition von Makrotasten    *;
;*                     durch Programmierung eines eigenen Handlers    *;
;*                     fr den BIOS-Tastatur-Interrupt 16h            *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  :  3.01.1992                                     *;
;*    letztes Update :  3.01.1992                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM MAKROKEY                                  *;
;*                     LINK MAKROKEY                                  *;
;*                     EXE2BIN MAKROKEY MAKROKEY.COM                  *;
;*                                                                    *;
;*                     oder                                           *;
;*                                                                    *;
;*                     TASM MAKROKEY                                  *;
;*                     TLINK /T MAKROKEY                              *;
;*--------------------------------------------------------------------*;
;*    Aufruf         : MAKROKEY                                       *;
;**********************************************************************;

;== hier beginnt das eigentliche Programm ==============================

code      segment para 'CODE'     ;Definition des CODE-Segments

          org 100h

          assume cs:code, ds:code, es:code, ss:code

start:    jmp  mkinit             ;Aufruf der Initialisierungs-Routine

;== Daten (bleiben im Speicher) ========================================

alterint  equ this dword          ;alter Interruptvektor 16h
intaltofs dw (?)                  ;Offsetadresse Interruptvektor 16h
intaltseg dw (?)                  ;Segmentadresse Interruptvektor 16h

mkey      dw 3100h                ;Makrotaste: ALT + N
mstart    db "Walter Zabel"       ;der Makrotext
mend      equ this byte

mofs      dw 0FFFFh               ;Zeiger auf nchstes Zeichen aus dem
                                  ;Makrotext

;== dies ist der neue BIOS-Tastatur-Interrupt 16h (bleibt im Speicher) =

neui16    proc far

          assume cs:code, ds:nothing, es:nothing, ss:nothing

          sti                     ;Interrupts weiter erlauben
          jmp  short ni1

          db "MT"                 ;Identifikation des Programms

ni1:      ;-- zunchst feststellen, ob der neue Handler fr die auf-
          ;-- gerufene Funktion zustndig ist

          or   ah,ah              ;Funktion 00h oder 10h?
          je   fkt0
          cmp  ah,10h
          je   fkt0

          cmp  ah,01h             ;Funktion 01h oder 11h?
          je   fkt1
          cmp  ah,11h
          je   fkt1

          jmp  cs:[alterint]      ;auch nicht, dann Funktion vom
                                  ;alten Handler ausfhren lassen

fkt0:     ;-- Funktion 00h/10h -----------------------------------------

          cmp  mofs,offset mend   ;wird Makro bereits abgespielt?
          jae  check0             ;nein auf Taste berprfen

          ;-- Makro wird bereits abgespielt, nchstes Zeichen liefern --

fkt0p:    push bx                 ;BX wird verndert
          mov  bx,mofs            ;Ja, nchstes Zeichen weitergeben
          mov  al,cs:[bx]         ;ASCII-Code laden
          xor  ah,ah              ;0 als Scan-Code
          inc  mofs               ;MOFS auf das nchste Zeichen
          pop  bx                 ;BX wieder zurck

niret0:   iret                    ;zurck zum Aufrufer

check0:   ;-- berprfen, ob Makrotaste eingegeben wurde ---------------

          pushf                   ;alten Handler aufrufen
          call cs:[alterint]
          cmp  ax,mkey            ;Makrotaste eingegeben?
          jne  niret0             ;Nein, zurck zum Aufrufer

          ;-- Makrotaste entdeckt, Makro abspielen ---------------------

          mov  mofs,offset mstart ;Makro-Zeiger auf Makro-Start
          jmp  fkt0p              ;erstes Zeichen zurckliefern

fkt1:     ;-- Funktion 01h/11h -----------------------------------------

          cmp  mofs,offset mend   ;wird Makro bereits abgespielt?
          jae  check1             ;nein auf Taste berprfen

          ;-- Makro wird bereits abgespielt, nchstes Zeichen liefern --

fkt1p:    push bx                 ;BX wird verndert
          mov  bx,mofs            ;Ja, nchstes Zeichen weitergeben
          mov  al,cs:[bx]         ;ASCII-Code laden
          xor  ah,ah              ;0 als Scan-Code
          pop  bx                 ;BX wieder zurck
          cmp  ah,1               ;Zero-Flag auf 0: Taste verfgbar

niret1:   ret 2                   ;zurck zum Aufrufer, aber Flags
                                  ;vom Stack putzen

check1:   ;-- berprfen, ob Makrotaste eingegeben wurde ---------------

          pushf                   ;alten Handler aufrufen
          call cs:[alterint]
          je   niret1             ;keine Taste im Tastaturpuffer

          cmp  ax,mkey            ;Makrotaste eingegeben?
          je   check1a            ;Ja, Abspielen starten

          cmp  ax,0               ;Zero-Flag auf 0: Zeichen verfgbar
          ret 2                   ;zurck zum Aufrufer, aber Flags
                                  ;vom Stack putzen

check1a:  ;-- Makrotaste entdeckt, Makro abspielen ---------------------

          xor  ah,ah              ;Makro-Taste zunchst entfernen
          pushf
          call cs:[alterint]

          mov  mofs,offset mstart ;Makro-Zeiger auf Makro-Start
          jmp  fkt1p              ;erstes Zeichen zurckliefern

neui16    endp

;== hier endet der Teil, der im Speicher resident bleibt ===============

instend   equ this byte

;== Daten (knnen vom DOS berschrieben werden) ========================

installm  db "MAKROKEY  -  (c) 1992 by Michael Tischer", 13, 10, 13, 10
          db "Das Programm wurde installiert und kann durch erneuten"
          db 13,10, "Aufruf wieder deaktiviert werden", 13, 10, "$"

entfernt  db "MAKROKEY wurde deaktiviert.", 13, 10, "$"

;== Programm (kann vom DOS berschrieben werden) =======================

mkinit    label near              ;Initialisierung

          assume cs:code, ds:code, es:code, ss:code

          ;-- zunchst feststellen, ob Programm bereits installiert ----

          mov  ax,3516h           ;Inhalt des Interruptvektors 16h holen
          int  21h                ;DOS-Funktion aufrufen
          cmp  word ptr es:[bx+3],"TM" ;testen ob MAKROKEY
          jne  mkinstall          ;nicht stalliert --> aber jetzt

          ;-- Programm bereits installiert, wieder deaktivieren und ----
          ;-- aus dem Speicher entfernen

          mov  dx,es:intaltofs    ;Offsetadresse des Interrupts 16h
          mov  ax,es:intaltseg    ;Segmentadresse des Interrupts 16h
          mov  ds,ax              ;nach DS
          mov  ax,2516h           ;Inhalt des Interruptvektors 16h auf
          int  21h                ;alte Routine zurcklegen

          ;-- Speicher des alten MAKROKEY wieder freigeben -------------

          mov  bx,es              ;Segmentadresse des Programms merken
          mov  es,es:[2Ch]        ;Segadr. des Environment aus PSP holen
          mov  ah,49h             ;den Speicher des alten Environment
          int  21h                ;wieder freigeben

          mov  es,bx              ;den Speicher des alten PRUM
          mov  ah,49h             ;ber die DOS-Funktoin 49h
          int  21h                ;wieder freigeben

          push cs                 ;CS auf Stack speichern
          pop  ds                 ;als DS zurckholen

          mov  ah,09h             ;Meldung: Programm enfernt
          mov  dx,offset entfernt
          int  21h

          mov  ax,4C00h           ;Programm normal beenden
          int  21h

mkinstall:  ;-- das Programm installieren ------------------------------

          mov  intaltseg,es       ;Segment- und Offsetadresse des Inter-
          mov  intaltofs,bx       ;ruptvektors 16h merken

          mov  dx,offset neui16   ;Offsetadresse neue Interrupt-Routine
          mov  ax,2516h           ;Inhalt des Interruptvektors 16h auf
          int  21h                ;eigene Routine verbiegen

          mov  dx,offset installm ;Meldung: Programm installiert
          mov  ah,09h             ;Funktionsnummer fr String ausgeben
          int  21h                ;DOS-Funktion aufrufen

          ;-- nur der PSP, die neue Interrupt-Routine und die ----------
          ;-- dazugehrigen Daten mssen resident bleiben.

          mov  dx,offset instend  ;Anzahl der Paragraphen (je 16 Byte)
          add  dx,15              ;berechnen, ber die das Programm
          mov  cl,4               ;verfgen soll
          shr  dx,cl
          mov  ax,3100h           ;Programm mit Ende-Code 0 (o.k) been-
          int  21h                ;den aber resident bleiben

;== Ende ===============================================================

code      ends                    ;Ende des CODE-Segments
          end  start
