;**********************************************************************;
;*                               V C O L                              *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : stellt einige elementare Funktionen zum Zu-    *;
;*                     griff auf die Color-Bildschirmkarte zu Verf-  *;
;*                     gung                                           *;
;*--------------------------------------------------------------------*;
;*    Info           : alle Funktionen unterteilen den Bildschirm     *;
;*                     in die Spalten 0 bis 79 und Zeilen 0 bis 24    *;
;*                     im Textmodus und in die Spalten 0 bis 639 und  *;
;*                     die Zeilen 0 bis 199 im Grafikmodus            *;
;*                     der 40 Spalten-Textmodus wird nicht unterst.!  *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  : 13.08.1987                                     *;
;*    letztes Update : 10.10.1988                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM VCOL;                                     *;
;*                     LINK VCOL;                                     *;
;*--------------------------------------------------------------------*;
;*    Aufruf         : VCOL                                           *;
;**********************************************************************;
                                                                        
;== Konstanten =========================================================
                                                                        
STEUER_REG  = 03D8h               ;Portadr. des Steuerregisters
FWAHL_REG   = 03D9h               ;Portadresse des Farbauswahl-Register
ADRESS_6845 = 03D4h               ;Adresseregister des 6845
DATA_6845   = 03D5h               ;Datenregister des 6845
VIO_SEG     = 0B800h              ;Segmentadresse des Video-RAM
CUR_START   = 10                  ;Registernr. CRTC: Startzeile Cursor
CUR_END     = 11                  ;Registernr. CRTC: Endzeile Cursor
AKTP_HI     = 12                  ;HI-Byte der Seitenadresse
AKTP_LO     = 13                  ;LO-Byte der Seitenadresse
CURPOS_HI   = 14                  ;Registernr. CRTC: Cursorpos. Hi-Byte
CURPOS_LO   = 15                  ;Registernr. CRTC: Cursorpos. Lo-Byte
                                                                        
DELAY       = 20000               ;Schleifenzhler fr Warteschleife
                                                                        
;== Makros =============================================================
                                                                        
;-- SETMODE: Macro zum Setzen des Bildschirmsteuerregister -------------
                                                                        
setmode   macro modus
                                                                        
          mov  dx,STEUER_REG      ;Adresse des Bildschirmsteuerregisters
          mov  al,modus           ;neuen Modus in das AL-Register
          out  dx,al              ;Modus an Steuerregister senden
                                                                        
          endm
                                                                        
;-- WAITRET: wartet, bis Bildschirmaufbau beendet ----------------------
                                                                        
waitret   macro
local     wr1                     ;lokales Label
                                                                        
          mov  dx,3DAh            ;Adresse des Bildschirmstatusregisters
wr1:      in   al,dx              ;Inhalt holen
          test al,8               ;vertikaler Retrace?
          je   wr1                ;NEIN --> warten
                                                                        
          endm
                                                                        
;== Stack ==============================================================
                                                                        
stack     segment para stack      ;Definition des Stack-Segments
                                                                        
          dw 256 dup (?)          ;der Stack ist 256 Words gro
                                                                        
stack     ends                    ;Ende des Stack-Segments
                                                                        
;== Daten ==============================================================
                                                                        
data      segment para 'DATA'     ;Definition des Daten-Segments
                                                                        
;== die Daten fr das Demo-Programm ====================================
                                                                        
initm   db 13,10
        db " VCOL (c) 1987,1988 by Michael Tischer "
        db 13,10,13,10
        db "Dieses Demonstrationsprogramm luft nur in Verbindung",13,10
        db "mit  einer Color-Bildschirmkarte  (CGA). Wenn Ihr  PC",13,10
        db "ber  eine andere  Art von  Bildschirmkarte  verfgt,",13,10
        db "dann  geben Sie jetzt  bitte <S> ein, um das Programm",13,10
        db "zu stoppen.",13,10
        db "Ansonsten  bettigen Sie bitte eine beliebige  andere",13,10
        db "Taste, um das Programm zu starten ...",13,10,"$"
                                                                        
str1    db 1,0
                                                                        
;== Tabelle mit den Offsetadressen der Zeilenanfnge ===================
                                                                        
zeilen  dw  0*160, 1*160, 2*160, 3*160, 4*160, 5*160, 6*160
        dw  7*160, 8*160, 9*160,10*160,11*160,12*160,13*160
        dw 14*160,15*160,16*160,17*160,18*160,19*160,20*160
        dw 21*160,22*160,23*160,24*160
                                                                        
grafikt db 38h, 28h, 2Dh, 0Ah, 7Fh, 06h  ;Register-Werte fr die
        db 64h, 70h, 02h, 01h, 06h, 07h  ;Grafik-Modi
                                                                        
textt   db 71h, 50h, 5Ah, 0Ah, 1Fh, 06h  ;Register-Werte fr den
        db 19h, 1Ch, 02h, 07h, 06h, 07h  ;Grafik-Modus
                                                                        
wait      db 0                    ;wird TRUE (<>0), wenn der Aufrufer
                                  ;den /F Switch angibt
                                                                        
data      ends                    ;Ende des Daten-Segments
                                                                        
;== Code ===============================================================
                                                                        
code      segment para 'CODE'     ;Definition des CODE-Segments
                                                                        
          assume cs:code, ds:data, es:data, ss:stack
                                                                        
;== dies ist nur das Demo-Programm =====================================
                                                                        
demo      proc far
                                                                        
          ;-- in der Befehlszeile nach "/F" suchen ---------------------
                                                                        
          mov  cl,ds:128          ;Anzahl Bytes in der K.Zeile holen
          or   cl,cl              ;keine Parameter eingegeben?
          je   switch1            ;NEIN --> Auswertung umgehen
          mov  bx,129             ;BX zeigt auf erstes Byte in K.Zeile
          mov  ch,bh              ;HI-Byte des Schleifenz. auf 0
                                                                        
switch:   cmp  [bx],"F/"          ;Switch an dieser Position?
          je   switch1            ;JA --> Switch gefunden
          cmp  [bx],"f/"          ;Switch an dieser Position?
          je   switch1            ;JA --> Switch gefunden
          inc  bl                 ;BX auf nchstes Zeichen setzen
          loop switch             ;das nchste Zeichen untersuchen
                                                                        
switch1:  mov  ax,data            ;Segmentadr. des Datensegments holen
          mov  ds,ax              ;und nach DS laden
          mov  es,ax              ;ebenfalls nach ES
                                                                        
          mov  wait,cl            ;WAIT-Flag setzen
                                                                        
          ;-- Init-Meldung ausgeben und auf Einabe warten --------------
                                                                        
          mov  ah,9               ;Funktionsnummer fr String ausgeben
          mov  dx,offset initm    ;Adresse der Meldung
          int  21h                ;DOS-Interrupt aufrufen
                                                                        
          xor  ah,ah              ;Funktionsnummer fr Taste holen
          int  16h                ;BIOS-Tastatur-Interrupt aufrufen
          cmp  al,"s"             ;wurde <s> eingegeben?
          je   ende               ;JA --> Programm wieder beenden
          cmp  al,"S"             ;wurde <S> eingeben
          jne  startdemo          ;NEIN --> Demo starten
                                                                        
ende:     mov  ax,4C00h           ;Funktionsnr. fr Programm beenden
          int  21h                ;DOS-Interrupt 21(h) aufrufen
                                                                        
          ;-- Demo-Programm, demonstriert Aufruf der Funktionen --------
                                                                        
startdemo label near
                                                                        
          call grafhi             ;320*200-Punkte-Grafik anschalten
          xor  al,al
          call cgr                ;Grafikbildschirm lschen
                                                                        
          xor  bx,bx              ;Spalte 0
          xor  dx,dx              ;Zeile 0
          mov  ax,199             ;Punkte vertikal
          mov  cx,639             ;Punkte horizontal
gr1:      push cx                 ;Punkte horizontal merken
          mov  cx,ax              ;Punkte vertikal in Counter
          push ax                 ;Punkte vertikal auf Stack merken
          mov  al,1
gr2:      call pixhi              ;Punkt setzen
          inc  dx                 ;Zeile inkrementieren
          loop gr2                ;Linie durchziehen
          pop  ax                 ;Punkte vertikal vom Stack holen
          sub  ax,3               ;nchste Linie 3 Punkte weniger
          pop  cx                 ;Punkte horizontal vom Stack holen
          push cx                 ;Punkte horizontal merken
          push ax                 ;Punkte vertikal auf Stack merken
          mov  al,1
gr3:      call pixhi              ;Punkt setzen
          inc  bx                 ;Spalte inkrementieren
          loop gr3                ;Linie durchziehen
          pop  ax                 ;Punkte vertikal vom Stack holen
          pop  cx                 ;Punkte horizontal vom Stack holen
          sub  cx,6               ;nchste Linie 6 Punkte weniger
          push cx                 ;Punkte horizontal merken
          mov  cx,ax              ;Punkte vertikal in Counter
          push ax                 ;Punkte vertikal auf Stack merken
          mov  al,1
gr4:      call pixhi              ;Punkt setzen
          dec  dx                 ;Zeile dekrementieren
          loop gr4                ;Linie durchziehen
          pop  ax                 ;Punkte vertikal vom Stack holen
          sub  ax,3               ;nchste Linie 3 Punkte weniger
          pop  cx                 ;Punkte horizontal vom Stack holen
          push cx                 ;Punkte horizontal merken
          push ax                 ;Punkte vertikal auf Stack merken
          mov  al,1
gr5:      call pixhi              ;Punkt setzen
          dec  bx                 ;Spalte inkrementieren
          loop gr5                ;Linie durchziehen
          pop  ax                 ;Punkte vertikal vom Stack holen
          pop  cx                 ;Punkte horizontal vom Stack holen
          sub  cx,6               ;nchste Linie 6 Punkte weniger
          cmp  ax,5               ;ist die Vertik.Linie lnger als 5
          ja   gr1                ;JA --> weitermachen
                                                                        
          xor  ah,ah              ;Funktionsnr. fr auf Taste warten
          int  16h                ;BIOS-Tastatur-Interrupt aufrufen
                                                                        
          call text               ;80*25-Zeichen-Textmodus anschalten
          xor  bp,bp              ;zunchst Bildschirmseite 0 bearbeiten
demo1:    mov  al,30h             ;Ascii-Code "0"
          or   ax,bp              ;Seintennummer nach ASCII wandeln
          mov  str1,al            ;in String abspeichern
          call setcol             ;Farbe setzen
          call seite              ;Bildschirmseite in BP aktivieren
          call cls                ;Bildschirmseite lschen
          xor  bx,bx              ;in der oberen linken Bildschirm-
          call calo               ;ecke mit der Ausgabe beginnen
          mov  cx,2000            ;2000 Zeichen enthlt eine Seite
          xor  ah,ah              ;mit Farbcode 0 beginnen
          mov  si,offset str1     ;Offsetadresse des String 1
demo2:    inc  ah                 ;Farbwert inkrementieren
          call print              ;String 1 ausgeben
          loop demo2              ;wiederholen bis der Bilds. voll ist
                                                                        
          xor  ah,ah              ;auf Taste warten
          int  16h                ;BIOS-Tastatur-Interrupt aufrufen
          inc  bp                 ;Seitennummer inkrementieren
          cmp  bp,4               ;alle 4 Seiten bearbeitet?
          jne  demo1              ;NEIN --> dann nchste Seite
                                                                        
          xor  bp,bp              ;wieder die Seite 0 aktivieren
          call seite
          jmp  ende               ;zum Programmende
                                                                        
demo      endp
                                                                        
;== jetzt folgen die eigentlichen Funktionen ===========================
                                                                        
;-- TEXT: schaltet die Textdarstellung an ------------------------------
;-- Eingabe  : keine
;-- Ausgabe  : keine
;-- Register : AX, SI, BH, DX und FLAGS werden verndert
                                                                        
text      proc near
                                                                        
          mov  si,offset textt    ;Offsetadresse der Register-Tabelle
          mov  bl,00100001b       ;80*25 Textmodus,blinken
          jmp  short vcprog       ;Video-Controller neu programmieren
                                                                        
text      endp
                                                                        
;-- GRAFHI: schaltet den 640*200 Punkte Grafikmodus an -----------------
;-- Eingabe  : keine
;-- Ausgabe  : keine
;-- Register : AX, SI, BH, DX und FLAGS werden verndert
                                                                        
grafhi    proc near
                                                                        
          mov  bl,00010010b       ;Grafikmodus mit 640*200 Punkten
          jmp  short grafik       ;Video-Controller neu programmieren
                                                                        
grafhi    endp
                                                                        
;-- GRAFLO: schaltet den 320*200 Punkte Grafikmodus an------------------
;-- Eingabe  : keine
;-- Ausgabe  : keine
;-- Register : AX, SI, BH, DX und FLAGS werden verndert
                                                                        
graflo    proc near
                                                                        
          mov  bl,00100010b       ;Grafikmodus mit 320*200 Punkten
grafik:   mov  si,offset grafikt  ;Offsetadresse der Register-Tabelle
                                                                        
graflo    endp
                                                                        
;-- VCPROG: programmiert den Video-Controller --------------------------
;-- Eingabe  : SI = Adresse einer Register-Tabelle
;--            BL = Wert fr Bildschirmsteuerregister
;-- Ausgabe  : keine
;-- Register : AX, SI, BH, DX und FLAGS werden verndert
                                                                        
vcprog    proc near
                                                                        
          setmode bl              ;Bit 3 = 0: Bildschirm aus
                                                                        
          mov  cx,12              ;12 Register werden gesetzt
          xor  bh,bh              ;mit Register 0 beginnen
vcp1:     lodsb                   ;Register-Wert aus Tabelle holen
          mov  ah,al              ;Register-Wert nach AH
          mov  al,bh              ;Nummer des Register nach AL
          call setvk              ;Wert an Controller bermitteln
          inc  bh                 ;nchstes Register ansprechen
          loop vcp1               ;weitere Register setzen
                                                                        
          or   bl,8               ;Bit 3 = 1: Bildschirm an
          setmode bl              ;neuen Modus setzen
          ret                     ;zurck zum Aufrufer
                                                                        
vcprog    endp
                                                                        
;-- SETCOL: setzt die Farbe des Bildschirmrahmens u. Hintergrunds ------
;-- Eingabe  : AL = Farbwert
;-- Ausgabe  : keine
;-- Register : AX und DX werden verndert
;-- Info     : im Textmodus geben die untersten 4 Bit die Rahmenfarbe an
;--            im Grafikmodus geben die untersten 4 Bit die Rahmen-
;--            und Hintergrundfarbe an, Bit 5 whlt die Farbpalette aus
                                                                        
setcol    proc near
                                                                        
          mov  dx,FWAHL_REG       ;Adresse des Farbauswahlregister
          out  dx,al              ;Farbwert ausgeben
          ret                     ;zurck zum Aufrufer
                                                                        
setcol    endp
                                                                        
;-- CDEF: setzt die Start- und Endzeile des Cursor ---------------------
;-- Eingabe  : CL = Startzeile
;--            CH = Endzeile
;-- Ausgabe  : keine
;-- Register : AX und DX werden verndert
                                                                        
cdef      proc near
                                                                        
          mov  al,CUR_START       ;Register: Startzeile
          mov  ah,cl              ;Startzeile nach AH
          call setvk              ;an Video-Controller bermitteln
          mov  al,CUR_END         ;Register: Endzeile
          mov  ah,ch              ;Endzeile nach AH
          jmp  short setvk        ;an Video-Controller bermitteln
                                                                        
cdef      endp
                                                                        
;-- SEITE: setzt die angezeigte Bildschirmseite ------------------------
;-- Eingabe  : BP = Nummer der Bildschirmseite (0 bis 3)
;-- Ausgabe  : keine
;-- Register : BX, AX, CX und DX werden verndert
;-- Info     : in den Grafikmodi trgt die erste Bildschirmseite die
;--            Nummer 0, die zweite die Nummer 2
                                                                        
seite     proc near
                                                                        
          mov  bx,bp              ;Bildschirmseite nach BX
          mov  cl,5               ;mit 2048 multiplizieren
          ror  bx,cl
          mov  al,AKTP_HI         ;Register: Hi-Byte Seitenadresse
          mov  ah,bh              ;Hi-Byte der Bildschirmseite nach AH
          call setvk              ;an Video-Controller bermitteln
          mov  al,AKTP_LO         ;Register: Lo-Byte Seitenadresse
          mov  ah,bl              ;Lo-Byte der Bildschirmseite nach AH
          jmp  short setvk        ;an Video-Controller bermitteln
                                                                        
seite     endp
                                                                        
;-- SETBLINK: setzt den blinkenden Bildschirmcurosr --------------------
;-- Eingabe  : DI = Offsetadresse des Cursor
;-- Ausgabe  : keine
;-- Register : BX, AX und DX werden verndert
                                                                        
setblink  proc near
                                                                        
          mov  bx,di              ;Offset nach BX bertragen
          mov  al,14              ;Register 15: Hi-Byte des Cursoroffset
          mov  ah,bh              ;HI-Byte des Offset
          call setvk              ;an Video-Controller bermitteln
          mov  al,15              ;Register 15: Lo-Byte des Cursoroffset
          mov  ah,bl              ;Lo-Byte des Offset
                                                                        
          ;-- SETVK wird automatisch aufgerufen ------------------------
                                                                        
setblink  endp
                                                                        
;-- SETVK: setzt ein Byte in einem der Register des Video-Controllers --
;-- Eingabe  : AL = Nummer des Registers
;--            AH = neuer Inhalt des Registers
;-- Ausgabe  : keine
;-- Register : DX und AL werden verndert
                                                                        
setvk     proc near
                                                                        
          mov  dx,ADRESS_6845     ;Adresse des Indexregisters
          out  dx,al              ;Nummer des Register senden
          jmp  short $+2          ;kleine I/O Pause
          inc  dx                 ;Adresse des Indexregister
          mov  al,ah              ;Inhalt nach AL
          out  dx,al              ;neuen Inhalt setzen
          ret                     ;zurck zum Aufrufer
                                                                        
setvk     endp
                                                                        
;-- GETVK: liest ein Byte aus einem der Register des Video-Controllers -
;-- Eingabe  : AL = Nummer des Registers
;-- Ausgabe  : AL = Inhalt des Registers
;-- Register : DX und AL werden verndert
                                                                        
getvk     proc near
                                                                        
          mov  dx,ADRESS_6845     ;Adresse des Indexregisters
          out  dx,al              ;Nummer des Registers senden
          inc  dx                 ;Adresse des Indexregisters
          jmp  short $+2          ;kleine I/O Pause
          in   al,dx              ;Inhalt nach AL lesen
          ret                     ;zurck zum Aufrufer
                                                                        
getvk     endp
                                                                        
;-- SCROLLUP: scrollt ein Fenster um N Zeilen nach oben ----------------
;-- Eingabe  : BL = Zeile oben links
;--            BH = Spalte oben links
;--            DL = Zeile unten rechts
;--            DH = Spalte unten rechts
;--            CL = Anzahl der Zeilen, um die gescrollt wird
;--          : BP = Nummer der Bildschirmseite (0 bis 3)
;-- Ausgabe  : keine
;-- Register : nur FLAGS werden verndert
;-- Info     : die frei werdenden Bildschirmzeilen werden gelscht
                                                                        
scrollup  proc near
                                                                        
          cld                     ;bei Stringbefehlen aufwrts zhlen
                                                                        
          push ax                 ;alle vernderten Register auf dem
          push bx                 ;Stack sichern
          push di                 ;in diesem Fall mu die Reihenfolge
          push si                 ;beachtet werden!
                                                                        
          push bx                 ;diese drei Register werden schor vor
          push cx                 ;dem Ende der Routine wieder vom Stack
          push dx                 ;zurckgeholt
          sub  dl,bl              ;Anzahl Zeilen berechnen
          inc  dl
          sub  dl,cl              ;Anzahl der zu scr. Zeilen abziehen
          sub  dh,bh              ;Anzahl Spalten berechnen
          inc  dh
          call calo               ;oben links in Offset umrechnen
          mov  si,di              ;Adresse in SI merken
          add  bl,cl              ;erste Zeile im gescrollten Fenster
          call calo               ;erste Zeile in Offset umrechnen
          xchg si,di              ;SI und DI vertauschen
                                                                        
          cmp  wait,0             ;Flackern unterdrcken?
          je   sup0               ;NEIN --> SUP0
                                                                        
          waitret                 ;JA, Retrace abwarten
          setmode 00100101b       ;Bildschirm ausschalten
                                                                        
sup0:     push ds                 ;Segmentregister auf dem Stack
          push es                 ;sichern
          mov  ax,VIO_SEG         ;Segmentadresse des Video-RAM
          mov  ds,ax              ;nach DS
          mov  es,ax              ;und ES bringen
                                                                        
                                                                        
sup1:     mov  ax,di              ;DI in AX merken
          mov  bx,si              ;SI in BX merken
          mov  cl,dh              ;Anzahl Spalten in Counter
          rep movsw               ;eine Zeile verschieben
          mov  di,ax              ;DI aus AX zurckholen
          mov  si,bx              ;SI aus BX zurckholen
          add  di,160             ;jeweils in nchste Zeile setzen
          add  si,160
          dec  dl                 ;alle Zeilen verarbeitet?
          jne  sup1               ;NEIN --> noch eine Zeile verschieben
                                                                        
          pop  es                 ;Segmentregister wieder vom Stack
          pop  ds                 ;holen
                                                                        
          cmp  wait,0             ;Flackern unterdrcken?
          je   sup2               ;NEIN --> SUP2
                                                                        
          setmode 00101101b       ;JA, Bildschirm wieder anschalten
                                                                        
sup2:     pop  dx                 ;untere rechte Ecke zurckholen
          pop  cx                 ;Anzahl Zeilen zurckholen
          pop  bx                 ;obere linke Ecke zurckholen
          mov  bl,dl              ;untere Zeile nach BL
          sub  bl,cl              ;Anzahl Zeilen abziehen
          inc  bl
          mov  ah,07h             ;Farbe : schwarz auf wei
          call clear              ;frei gewordenen Zeilen lschen
                                                                        
          pop  si                 ;CX und DX sind bereits her-
          pop  di                 ;untergeholt worden
          pop  bx
          pop  ax
                                                                        
          ret                     ;zurck zum Aufrufer
                                                                        
scrollup  endp
                                                                        
;-- SCROLLDN: scrollt ein Fenster um N Zeilen nach unten ---------------
;-- Eingabe  : BL = Zeile oben links
;--            BH = Spalte oben links
;--            DL = Zeile unten rechts
;--            DH = Spalte unten rechts
;--            CL = Anzahl der Zeilen, um die gescrollt wird
;--          : BP = Nummer der Bildschirmseite (0 bis 3)
;-- Ausgabe  : keine
;-- Register : nur FLAGS werden verndert
;-- Info     : die frei werdenden Bildschirmzeilen werden gelscht
                                                                        
scrolldn  proc near
                                                                        
          cld                     ;bei Stringbefehlen aufwrts zhlen
                                                                        
          push ax                 ;alle vernderten Register auf dem
          push bx                 ;Stack sichern
          push di                 ;in diesem Fall mu die Reihenfolge
          push si                 ;beachtet werden!
                                                                        
          push bx                 ;diese drei Register werden schon vor
          push cx                 ;dem Ende der Routine wieder vom Stack
          push dx                 ;zurckgeholt
                                                                        
          sub  dh,bh              ;Anzahl Spalten berechnen
          inc  dh
          mov  al,bl              ;Zeile oben links in AL merken
          mov  bl,dl              ;Zeile u.r. nach Zeile u.l.
          call calo               ;oben links in Offset umrechnen
          mov  si,di              ;Adresse in SI merken
          sub  bl,cl              ;Anzahl zu scrollende Z. abziehen
          call calo               ;oben links in Offset umrechnen
          xchg si,di              ;SI und DI vertauschen
          sub  dl,al              ;Anzahl Zeilen berechnen
          inc  dl
          sub  dl,cl              ;Anzahl der zu scr. Zeilen abziehen
                                                                        
          cmp  wait,0             ;Flackern unterdrcken?
          je   sdn0               ;NEIN --> SDN0
                                                                        
          waitret                 ;JA, Retrace abwarten
          setmode 00100101b       ;Bildschirm ausschalten
                                                                        
sdn0:     push ds                 ;Segmentregister auf dem Stack
          push es                 ;sichern
          mov  ax,VIO_SEG         ;Segmentadresse des Video-RAM
          mov  ds,ax              ;nach DS
          mov  es,ax              ;und ES bringen
                                                                        
sdn1:     mov  ax,di              ;DI in AX merken
          mov  bx,si              ;SI in BX merken
          mov  cl,dh              ;Anzahl Spalten in Counter
          rep movsw               ;eine Zeile verschieben
          mov  di,ax              ;DI aus AX zurckholen
          mov  si,bx              ;SI aus BX zurckholen
          sub  di,160             ;jeweils in nchste Zeile setzen
          sub  si,160
          dec  dl                 ;alle Zeilen verarbeitet?
          jne  sdn1               ;NEIN --> noch eine Zeile verschieben
                                                                        
          pop  es                 ;Segmentregister wieder vom Stack
          pop  ds                 ;holen
                                                                        
          cmp  wait,0             ;Flackern unterdrcken?
          je   sdn2               ;NEIN --> SDN2
                                                                        
          setmode 00101101b       ;JA, Bildschirm wieder anschalten
                                                                        
sdn2:     pop  dx                 ;untere rechte Ecke zurckholen
          pop  cx                 ;Anzahl Zeilen zurckholen
          pop  bx                 ;obere linke Ecke zurckholen
          mov  dl,bl              ;obere Zeile nach DL
          add  dl,cl              ;Anzahl Zeilen addieren
          dec  dl
          mov  ah,07h             ;Farbe : schwarz auf wei
          call clear              ;frei gewordenen Zeilen lschen
                                                                        
          pop  si                 ;CX und DX sind bereits wieder her-
          pop  di                 ;untergeholt worden
          pop  bx
          pop  ax
                                                                        
          ret                     ;zurck zum Aufrufer
                                                                        
scrolldn  endp
                                                                        
;-- CLS: den gesamten Bildschirm lschen -------------------------------
;-- Eingabe  : BP = Nummer der Bildschirmseite (0 oder 1)
;-- Ausgabe  : keine
;-- Register : nur FLAGS werden verndert
                                                                        
cls       proc near
                                                                        
          mov  ah,07h             ;Farbe ist wei auf schwarz
          xor  bx,bx              ;oben links ist (0/0)
          mov  dx,4F18h           ;unten rechts ist (79/24)
                                                                        
          ;-- Clear ausfhren ------------------------------------------
                                                                        
cls       endp
                                                                        
;-- CLEAR: fllt einen bestimmten Bildschirmbereich mit Leerzeichen ----
;-- Eingabe  : AH = Attribut/Farbe
;--            BL = Zeile oben links
;--            BH = Spalte oben links
;--            DL = Zeile unten rechts
;--            DH = Spalte unten rechts
;--            BP = Nummer der Bildschirmseite (0 bis 3)
;-- Ausgabe  : keine
;-- Register : nur FLAGS werden verndert
                                                                        
clear     proc near
                                                                        
          cld                     ;bei Stringbefehlen aufwrts zhlen
          push cx                 ;alle, im nachfolgenden vernderten,
          push dx                 ;Register auf dem Stack sichern
          push si
          push di
          push es
          sub  dl,bl              ;Anzahl Zeilen berechnen
          inc  dl
          sub  dh,bh              ;Anzahl Spalten berechnen
          inc  dh
          call calo               ;Offsetadresse der oberen linken Ecke
          mov  cx,VIO_SEG         ;Segmentadresse des Video-RAM
          mov  es,cx              ;nach ES
          xor  ch,ch              ;Hi-Bytes des Counter auf 0
          mov  al," "             ;Leerzeichen
                                                                        
          cmp  wait,0             ;Flackern unterdrcken?
          je   clear1             ;NEIN --> CLEAR1
                                                                        
          push dx                 ;DX auf dem Stack sichern
          waitret                 ;Retrace abwarten
          setmode 00100101b       ;Bildschirm ausschalten
          pop  dx                 ;DX vom Stack zurckholen
                                                                        
                                                                        
clear1:   mov  si,di              ;DI in SI merken
          mov  cl,dh              ;Anzahl Spalten in Counter
          rep stosw               ;Leerzeichen speichern
          mov  di,si              ;DI aus SI zurckholen
          add  di,160             ;in nchste Zeile setzen
          dec  dl                 ;alle Zeilen verarbeitet?
          jne  clear1             ;NEIN --> noch eine Zeile lschen
                                                                        
          cmp  wait,0             ;Flackern unterdrcken?
          je   clear2             ;NEIN --> CLEAR2
                                                                        
          setmode 00101101b       ;Bildschirm wieder anschalten
                                                                        
clear2:   pop  es                 ;die gesicherten Register wieder
          pop  di                 ;vom Stack holen
          pop  si
          pop  dx
          pop  cx
          ret                     ;zurck zum Aufrufer
                                                                        
clear     endp
                                                                        
;-- PRINT: gibt einen String auf dem Bildschirm aus --------------------
;-- Eingabe  : AH = Attribut/Farbe
;--            DI = Offsetadresse des ersten Zeichen
;--            SI = Offsetadresse des Strings zu DS
;--            BP = Nummer der Bildschirmseite (0 bis 3)
;-- Ausgabe  : DI zeigt hinter das letzte ausgegebene Zeichen
;-- Register : AL, DI und FLAGS werden verndert
;-- Info     : der String mu durch das NUL-Zeichen beendet werden.
;--            andere Steuerzeichen werden nicht erkannt
                                                                        
print     proc near
                                                                        
          cld                     ;bei Stringbefehlen aufwrts zhlen
          push si                 ;SI, CX, DX und ES auf dem 
          push es                 ;Stack sichern
          push cx
          push dx
          mov  dx,VIO_SEG         ;Segmentadresse des Video-RAM erst
          mov  cl,wait            ;WAIT-Flag holen
          mov  es,dx              ;nach DX und dann nach ES
                                                                        
          jmp  short print3       ;Zeichen holen und auswerten
                                                                        
print1    label near
                                                                        
          or   cl,cl              ;Flackern unterdrcken?
          je   print2             ;NEIN --> PRINT2
                                                                        
          push ax                 ;Zeichen und Farbe merken
          mov  dx,3DAh            ;Adresse des Bildschirmstatusregisters
hr1:      in   al,dx              ;Inhalt holen
          test al,1               ;horizontaler Retrace?
          jne  hr1                ;NEIN --> warten
          cli                     ;keine weiteren Interrupts erlauben
hr2:      in   al,dx              ;Inhalt holen
          test al,1               ;horizontaler Retrace?
          je   hr2                ;JA --> warten
          pop  ax                 ;Zeichen und Farbe zurckholen
          sti                     ;Interrupts nicht mehr unterdrcken
                                                                        
print2:   stosw                   ;Attribut und Farbe in V-RAM speichern
print3:   lodsb                   ;nchstes Zeichen aus dem String holen
          or   al,al              ;ist es NUL
          jne  print1             ;NEIN --> ausgeben
                                                                        
printe:   pop  dx                 ;SI, CX, DX und ES wieder
          pop  cx                 ;vom Stack holen
          pop  es
          pop  si
          ret                     ;zurck zum Aufrufer
                                                                        
print     endp
                                                                        
;-- CALO: rechnet Zeile und Spalte in Offsetadresse um -----------------
;-- Eingabe  : BL = Zeile
;--            BH = Spalte
;--            BP = Nummer der Bildschirmseite (0 bis 3)
;-- Ausgabe  : DI = die Offsetadresse
;-- Register : DI und FLAGS werden verndert
                                                                        
calo      proc near
                                                                        
          push ax                 ;AX auf dem Stack sichern
          push bx                 ;BX auf dem Stack sichern
                                                                        
          shl  bx,1               ;Spalte und Zeile mal 2
          mov  al,bh              ;Spalte nach AL
          xor  bh,bh              ;Hi-Byte
          mov  di,[zeilen+bx]     ;Offsetadresse der Zeile holen
          xor  ah,ah              ;HI-Byte fr Spaltenoffset
          add  di,ax              ;Zeilen- und Spaltenoffset addieren
          mov  bx,bp              ;Bildschirmseite nach BX
          mov  cl,4               ;mit 4096 multiplizieren
          ror  bx,cl
          add  di,bx              ;Beginn der Bs. zu Offset addieren
          pop  bx                 ;BX wieder vom Stack holen
          pop  ax                 ;AX wieder vom Stack holen
          ret                     ;zurck zum Aufrufer
                                                                        
calo      endp
                                                                        
;-- CGR: den gesamten Grafikbildschirm lschen -------------------------
;-- Eingabe  : AL = 00(h) : alle Punkte lschen
;--                 FF(h) : alle Punkte setzen
;-- Ausgabe  : keine
;-- Register : AH, BX, CX, DI und FLAGS werden verndert
;-- Info     : diese Funktion lscht den Grafikbildschirm in beiden
;--            Grafikmodi
                                                                        
cgr       proc near
                                                                        
          push es                 ;ES auf dem Stack sichern
          cbw                     ;AL auf AH ausdehnen
          xor  di,di              ;Offsetadresse in Video-RAM
          mov  bx,VIO_SEG         ;Segmentadresse Bildschirmseite
          mov  es,bx              ;Segmentadresse ins Segmentregister
          mov  cx,2000h           ;eine Seite sind 8KB-Words
          rep stosw               ;Seite fllen
          pop  es                 ;ES vom Stack zurckholen
          ret                     ;zurck zum Aufrufer
                                                                        
cgr       endp
                                                                        
;-- PIXLO: setzt einen Punkt im 320*200-Punkte-Grafikmodus -------------
;-- Eingabe  : BP = Nummer der Bildschirmseite (0 oder 1)
;--            BX = Spalte (0 bis 319)
;--            DX = Zeile  (0 bis 199)
;--            AL = Farbe des Punktes (0 bis 3)
;-- Ausgabe  : keine
;-- Register : AX, DI und FLAGS werden verndert
                                                                        
pixlo     proc near
                                                                        
          push ax                 ;AX auf dem Stack sichern
          push bx                 ;BX auf dem Stack merken
          push cx                 ;CX auf dem Stack merken
          mov  cl,7
          mov  ah,bl              ;Spalte nach AH bertragen
          and  ah,11b             ;Spalte mod 4
          shl  ah,1               ;Spalte * 2
          sub  cl,ah              ;7 - 2 * (Spalte mod 4)
          mov  ah,11              ;Bit-Wert zum Ausblenden
          shl  ax,cl              ;zur Punktposition verschieben
          not  ah                 ;AH zum Ausblenden umdrehen
          shr  bx,1               ;BX durch zweimaliges rechtsshiften
          shr  bx,1               ;durch 4 teilen
          jmp  short spix         ;Punkt setzen
                                                                        
pixlo     endp
                                                                        
;-- PIXHI: setzt einen Punkt im 640*200-Punkte-Grafikmodus -------------
;-- Eingabe  : BP = Nummer der Bildschirmseite (0 oder 1)
;--            BX = Spalte (0 bis 639)
;--            DX = Zeile  (0 bis 199)
;--            AL = Farbe des Punktes (0 oder 1)
;-- Ausgabe  : keine
;-- Register : AX, DI und FLAGS werden verndert
                                                                        
pixhi     proc near
                                                                        
          push ax                 ;AX auf dem Stack sichern
          push bx                 ;BX auf dem Stack merken
          push cx                 ;CX auf dem Stack merken
          mov  cl,7
          mov  ah,bl              ;Spalte nach AH bertragen
          and  ah,111b            ;Spalte mod 8
          sub  cl,ah              ;7 - Spalte mod 8
          mov  ah,1               ;Bit-Wert zum Ausblenden
          shl  ax,cl              ;zur Punktposition verschieben
          not  ah                 ;AH zum Ausblenden umdrehen
          mov  cl,3               ;3 Verschiebungen
          shr  bx,cl              ;BX durch 8 teilen
                                                                        
          ;-- Punkt setzen ---------------------------------------------
                                                                        
pixhi          endp
                                                                        
;-- SPIX: setzt einen Punkt im Grafikbildschirm ------------------------
;-- Eingabe  : BX = Spaltenoffset
;--            DX = Zeile  (0 bis 199)
;--            AH = Wert zum Ausblenden der alten Bits
;--            AL = neuer Bitwert
;-- Ausgabe  : keine
;-- Register : AX, DI und FLAGS werden verndert
                                                                        
spix      proc near
                                                                        
          push es                 ;ES auf dem Stack sichern
          push dx                 ;DX auf dem Stack sichern
          push ax                 ;AX auf dem Stack sichern
                                                                        
          xor  di,di              ;Offsetadresse in Video-RAM
          mov  cx,VIO_SEG         ;Segmentadresse Bildschirmseite
          mov  es,cx              ;Segmentadresse ins Segmentregister
          mov  ax,dx              ;Zeile nach AX holen
          shr  ax,1               ;Zeile durch 2 teilen
          mov  cl,80              ;der Faktor ist 90
          mul  cl                 ;Zeile mit 80 multiplizieren
          and  dx,1               ;Zeile mod 2
          mov  cl,3               ;3 Verschiebungen
          ror  dx,cl              ;rechts herum rotieren (* 2000(h))
          mov  di,ax              ;80 * int(Zeile/2)
          add  di,dx              ;+ 2000(h) * (Zeile mod 4)
          add  di,bx              ;Spaltenoffset hinzuaddieren
          pop  ax                 ;AX wieder vom Stack holen
          mov  bl,es:[di]         ;Punkt holen
          and  bl,ah              ;Bits ausblenden
          or   bl,al              ;Punkt einblenden
          mov  es:[di],bl         ;Punkt zurckschreiben;
                                                                        
          pop  dx                 ;DX wieder vom Stack holen
          pop  es                 ;ES wieder vom Stack holen
          pop  cx                 ;CX wieder vom Stack merken
          pop  bx                 ;BX wieder vom Stack holen
          pop  ax                 ;AX wieder vom Stack holen
                                                                        
          ret                     ;zurck zum Aufrufer
                                                                        
spix      endp
                                                                        
                                                                        
;== Ende ===============================================================
                                                                        
code      ends                    ;Ende des CODE-Segments
          end  demo               ;Programmausfhrung mit DEMO beginnen
