'*****************************************************************************
'*                              FIXPARTB.BAS                                 *
'-----------------------------------------------------------------------------
'*   Aufgabe        : Zeigt die Partitionen einer Festplatte an              *
'-----------------------------------------------------------------------------
'*   Autor          : Michael Tischer                                        *
'*   entwickelt am  : 21.05.1991                                             *
'*   letztes Update : 12.01.1992                                             *
'-----------------------------------------------------------------------------
'*   Aufruf         : FIXPARTB [Laufwerksnummer]                             *
'*                    Defaulteinstellung ist Laufwerk 0 ("C")                *
'*****************************************************************************
'
DECLARE SUB GetSecZyl (SecZyl AS LONG, Sektor AS INTEGER, Zylinder AS INTEGER)
DECLARE FUNCTION ReadPartSec% (LW%, Kopf%, SecZyl%, PartEintrag() AS ANY)
DECLARE SUB Showpartition (LW AS INTEGER)
DECLARE FUNCTION MLNG& (Zahl AS INTEGER)
										
'$INCLUDE: 'QB.BI'               'Includedatei einbinden fr Interruptaufrufe
										
CONST TRUE = -1                                    'Wahrheitswerte definieren
CONST FALSE = NOT TRUE
										
TYPE SecPos                            'beschreibt die Position eines Sektors
      Kopf AS INTEGER                                       'Schreib/Lesekopf
      SecZyl AS LONG                               'Sektor und Zylindernummer
END TYPE
										
TYPE PartEntry                               'Eintrag n der Partitionstabelle
      Status AS INTEGER                                    'Partitions-Status
      StartSec AS SecPos                                       'erster Sektor
      PartType AS INTEGER                                      'Partitionstyp
      EndSec AS SecPos                                        'letzter Sektor
      SecOfs AS LONG                                 'Offset des Boot-Sektors
      SecAnz AS LONG                                     'Anzahl der Sektoren
END TYPE
										
DIM Laufwerk AS INTEGER   'Variabele zur Umrechnung des bergebenen Arguments
										
CLS
PRINT " FIXPARTB - (C)";
PRINT " 1991, 92 by Michael Tischer "
Laufwerk = 0                                'Default ist die erste Festplatte
IF COMMAND$ <> "" THEN                        'anderes Laufwerk ist angegeben
  Laufwerk = VAL(COMMAND$)                                'ASCII nach Dezimal
END IF
CALL Showpartition(Laufwerk)                       'Partitionssektor anzeigen
END

'*****************************************************************************
'* GetSecZyl : ermittelt aus der kombinierten Sektor/Zylindercodierung       *
'*             des BIOS die Sektor- und Zylindernummer                       *
'* Eingaben  : Sektorvariabele in BIOS-Codierung                             *
'* Ausgabe   : Sektor und ZylinderVariabele ber Referenz aus Variabele      *
'*****************************************************************************
SUB GetSecZyl (SecZyl AS LONG, Sektor AS INTEGER, Zylinder AS INTEGER)
									   
 Sektor = SecZyl AND 63                              'Bits 6 und 7 ausblenden
 Zylinder = SecZyl \ 256 + ((SecZyl MOD 256) AND 192) * 4
END SUB

'****************************************************************************
'*  MLNG    : Wandelt eine Integerzahl in die sie reprsentierende Langzahl *                  
'*            um                                                            *
'*  Eingabe : Eine Integerzahl                                              *
'*  Ausgabe : Die Langzahl, die dem Wert des Registerinhaltes entspricht    *
'****************************************************************************
FUNCTION MLNG& (Zahl AS INTEGER)

IF Zahl >= 0 THEN                                   'Bit 16 ist nicht gesetzt
   MLNG = Zahl                         'Integerzahl entspricht Registerinhalt
ELSE                                                      'Bit 16 ist gesetzt
   MLNG = 65536 + Zahl              'diese Zahl entspricht dem Registerinhalt
END IF

END FUNCTION

'*****************************************************************************
'* ReadPartSec : Liest den Partitionssektor einer Festplatte aus             *
'* Eingaben    : LW          = BIOS-Code des Laufwerks                       *
'*               Kopf        = Nummer des Schreib/Lesekopfs                  *
'*               SecZyl      = Sektor- und Zylindernummer im BIOS-Format     *
'* Ausgaben    : PartEntry() = Die 4 Partitionstabellen im Partitionssektor  *
'*****************************************************************************
'
FUNCTION ReadPartSec% (LW%, Kopf%, SecZyl%, PartEintr() AS PartEntry)

DIM Register AS RegTypeX           'Prozessorregister fr den Interruptaufruf
DIM PartSektor AS STRING * 512                'nimmt den Partitionssektor auf
DIM IdCode AS LONG                    'nimmt den Partitionserkennungscode auf
DIM i AS INTEGER                                             'Schleifenzhler
DIM j AS INTEGER                                             'Schleifenzhler
										
Register.ax = &H201                     'Funkt.Nr fr "READ" im ersten Sektor
Register.cx = SecZyl%             'Sektor und Zylinder der aktuellen Partition
Register.dx = Kopf% * 256 + LW%                      ' Kopf und Laufwerksnummer
Register.es = VARSEG(PartSektor)       'Segmentaddresse des Partitionssektors
Register.bx = VARPTR(PartSektor)         'OffsetAdresse des Partitionssektors
CALL INTERRUPTX(&H13, Register, Register)     'Festplatten-Interrupt aufrufen
DEF SEG = VARSEG(PartSektor)     'Segmentadresse festlegen fr PEEK-Anweisung
Pointer = VARPTR(PartSektor) + &H1BD     'Poiter auf ersten Partitionseintrag
offset = 0                              'Offset des ersten Partitionseintrags
FOR i = 1 TO 4                               'Alle 4 Partitionseintrge lesen
  PartEintr(i).Status = PEEK(Pointer + offset + 1)
  PartEintr(i).StartSec.Kopf = PEEK(Pointer + offset + 2)
  PartEintr(i).StartSec.SecZyl = PEEK(Pointer + offset + 4) * 256& + PEEK(Pointer + offset + 3)
  PartEintr(i).PartType = PEEK(Pointer + offset + 5)
  PartEintr(i).EndSec.Kopf = PEEK(Pointer + offset + 6)
  PartEintr(i).EndSec.SecZyl = PEEK(Pointer + offset + 8) * 256& + PEEK(Pointer + offset + 7)
  PartEintr(i).SecOfs = 0
  FOR j = 0 TO 3
     PartEintr(i).SecOfs = PartEintr(i).SecOfs * 256& + PEEK(Pointer + offset + 12 - j)
  NEXT
  PartEintr(i).SecAnz = 0
  FOR j = 0 TO 3
     PartEintr(i).SecAnz = PartEintr(i).SecAnz * 256& + PEEK(Pointer + offset + 16 - j)
  NEXT
  offset = offset + 16      'Offset auf den nchsten Partitionseintrag setzen
NEXT
IdCode = PEEK(Pointer + offset + 2) * 256& + PEEK(Pointer + offset + 1)
IF (Register.flags AND 1) = 0 THEN
  ReadPartSec = TRUE
END IF

END FUNCTION

'*****************************************************************************
'* ShowPartition : Zeigt die Partionierung einer Festplatte auf dem          *
'*                 Bildschirm an                                             *
'* Eingabe       : Nummer des angesprochenen Festplattenlaufwerks            *
'* Ausgabe       : keine                                                     *
'*****************************************************************************
SUB Showpartition (LW AS INTEGER)
										
DIM Kopf AS INTEGER                             'Kopf der aktuellen Partition
DIM SecZyl AS INTEGER            'Sektor und Zylinder der aktuellen Partition
DIM Entry AS INTEGER                                         'Schleifenzhler
DIM Sektor AS INTEGER                                           'Sektornummer
DIM Zylinder AS INTEGER                                       'Zylindernummer
DIM PartEintrag(4) AS PartEntry               'der aktuelle Partitionseintrag
DIM Register AS RegTypeX               'Prozessorregister zum Interruptaufruf
										
PRINT ;
LW = LW + &H80                          'Laufwerksnummer fr BIOS aufbereiten
IF ReadPartSec(LW, 0, 1, PartEintrag()) THEN          'Partitionssektor lesen
  Register.ax = &H800          ' AH = Fkt.Nr Kenndaten des Laufwerks erfragen
  Register.dx = LW                                           'Laufwerksnummer
  CALL INTERRUPTX(&H13, Register, Register)   'Festplatten-Interrupt aufrufen
  CALL GetSecZyl(MLNG(Register.cx), Sektor, Zylinder) 'Sekt.u. Zyl.zahl best.
  PRINT "";
  PRINT "ͻ"
  PRINT USING " Laufwerk # :  ## Kpfe mit "; LW - &H80; Register.dx \ 256 + 1;
  PRINT USING "jeweils ###### Zylindern zu je "; Zylinder;
  PRINT USING "### Sektoren       "; Sektor
  PRINT " Partitionstabelle im Partitionssektor     ";
  PRINT "                                   "
  PRINT "";
  PRINT "͹"
  PRINT "                               Start    ";
  PRINT "     Ende      Entf. v          "
  PRINT "NrBoot Typ                Kopf ";
  PRINT "Zyl. Sek.Kopf Zyl. Sek. BootSek   Anzahl "
  PRINT "";
  PRINT "͹"
  FOR Entry = 1 TO 4                                'Die Eintrge durchlaufen
    PRINT USING "##"; Entry;
    IF PartEintrag(Entry).Status = &H80 THEN
      PRINT " Ja ";
    ELSE
      PRINT "Nein";
    END IF
    SELECT CASE PartEintrag(Entry).PartType
       CASE 0
	 PRINT "nicht belegt        ";
       CASE 1
	 PRINT "DOS, 12-Bit-FAT     ";
       CASE 2 OR 3
	 PRINT "Xenix               ";
       CASE 4
	 PRINT "DOS, 16-Bit-FAT     ";
       CASE 5
	 PRINT "DOS, extended Part. ";
       CASE 6
	 PRINT "DOS 4.0 > 32 MB     ";
       CASE &HDB
	 PRINT "concurrent DOS,     ";
       CASE ELSE
	 PRINT USING "unbekannt    (###)  "; PartEintrag(Entry).PartType
     END SELECT
     CALL GetSecZyl(PartEintrag(Entry).StartSec.SecZyl, Sektor, Zylinder)
     PRINT USING "###  ###  "; PartEintrag(Entry).StartSec.Kopf; Zylinder;
     PRINT USING "### "; Sektor;
     CALL GetSecZyl(PartEintrag(Entry).EndSec.SecZyl, Sektor, Zylinder)
     PRINT USING "###  ###  ### "; PartEintrag(Entry).EndSec.Kopf; Zylinder; Sektor;
										
     PRINT USING " ####### "; PartEintrag(Entry).SecOfs;
     PRINT USING " ####### "; PartEintrag(Entry).SecAnz
   NEXT
   PRINT "";
   PRINT "ͼ"
ELSE
  PRINT "Fehler beim Zugriff auf Bootsektor"
END IF

END SUB

