'*****************************************************************************
'*                                  LEDB                                     *
'*---------------------------------------------------------------------------*
'*   Aufgabe        : setzt die verschiedenen Bits im Tastatur-Status-Byte   *
'*                    des BIOS und lt dadurch die LEDs auf der AT-Tastatur *
'*                    aufleuchten                                            *
'*---------------------------------------------------------------------------*
'*   Autor          : MICHAEL TISCHER                                        *
'*   entwickelt am  : 08.06.1991                                             *
'*   letztes Update : 03.01.1992                                             *
'*****************************************************************************
'
DECLARE SUB SetFlag (Flag AS INTEGER)
DECLARE SUB Delay (Pause AS INTEGER)
DECLARE SUB ClrFlag (Flag AS INTEGER)
										
'$INCLUDE: 'QB.BI'              'Include Datei fr Interruptaufrufe einbinden

CONST SCRL = 16                                              'Scroll-Lock-Bit
CONST NUML = 32                                                 'Num-Lock-Bit
CONST CAPL = 64                                                'Caps-Lock-Bit
										
DIM Zaehler AS INTEGER                                       'Schleifenzhler
										
CLS                                                       'Bildschirm lschen
PRINT " LEDB  -  (c) 1988, 92 by Michael Tischer"
PRINT
PRINT " Achten sie auf die LEDs Ihrer Tastatur"
										
FOR Zaehler = 1 TO 10                            'Schleife 10 mal durchlaufen
  SetFlag (CAPL)                                             'CAPS anschalten
  Delay (100)                                       '100 Millisekunden warten
  ClrFlag (CAPL)                                     'CAPS wieder ausschalten
  SetFlag (NUML)                                              'NUM anschalten
  Delay (100)                                       '100 Millisekunden warten
  ClrFlag (NUML)                                     'NUML wieder ausschalten
  SetFlag (SCRL)                                      'Scroll-Lock anschalten
  Delay (100)                                       '100 Millisekunden warten
  ClrFlag (SCRL)                              'Scroll-Lock wieder ausschalten
NEXT

FOR Zaehler = 1 TO 10                            'Schleife 10 mal durchlaufen
  SetFlag (CAPL OR SCRL OR NUML)                  'Alle drei Flags anschalten
  Delay (500)                                       '200 Millisekunden warten
  ClrFlag (CAPL OR SCRL OR NUML)                  'Alle drei Flags wieder aus
  Delay (500)                                       '200 Millisekunden warten
NEXT
END

'*****************************************************************************
'*  ClrFLAG : lscht eines der Flags im BIOS-Status-Byte                     *
'*  Eingabe : das zu lschende Flag (siehe Konstanten)                       *
'*  Ausgabe : keine                                                          *
'*****************************************************************************
'
SUB ClrFlag (Flag AS INTEGER)
										
DIM Register AS RegType                                    'Prozessorregister
										
DEF SEG = &H40                  'Segmentadresse des BIOS-Tastatur-Statusbytes
POKE &H17, (PEEK(&H17) AND (NOT Flag))      'Das Tastatur-Status-Byte lschen
Register.ax = 1 * 256               'AH = Funktionsnummer: Zeichen vorhanden?
CALL INTERRUPT(&H16, Register, Register)    'BIOS-Tastatur-Interrupt aufrufen

END SUB

'*****************************************************************************
'* Delay   : Wartet die Zeit des Argumentes in Millisekunden ab              *
'* Eingabe : Pause = Anzahl der zu wartenden Millisekunden                   *
'* Ausgabe : keine                                                           *
'*****************************************************************************
'
SUB Delay (Pause AS INTEGER)
										
DIM Register AS RegType                      'Prozessorregister fr Interrupt
DIM Time AS LONG                    'Nimmt den zu erreichenden Timer-Wert auf
										
Register.ax = 0                            'FunktionsNr. ZeitZaehler auslesen
CALL INTERRUPT(&H1A, Register, Register)       'BIOS-Timer-Interrupt aufrufen
Time = Register.dx + (Register.cx * 32768)     'Den zu erreichenden Time-Wert
Time = Time + (Pause * 18 + ((Pause * 2) / 10)) / 1000             'berechnen
DO
  CALL INTERRUPT(&H1A, Register, Register)          'Timer-Interrupt aufrufen
LOOP WHILE (Register.dx + (Register.cx * 32768)) <= Time   'Bis Time erreicht

END SUB

'*****************************************************************************
'*  SETFLAG : Setz eines der Flags im BIOS-Status-Byte                       *
'*  Eingabe : das zu setzende Flag (siehe Konstanten)                        *
'*  Ausgabe : keine                                                          *
'*****************************************************************************
'
SUB SetFlag (Flag AS INTEGER)
										
DIM Register AS RegType                'Prozessorregister fr Interruptaufruf
										
DEF SEG = &H40                  'Segmentadresse des BIOS-Tastatur-Statusbytes
POKE &H17, (PEEK(&H17) OR Flag)            'Das Flag in Statusbyte einblenden
Register.ax = 1 * 256               'AH = Funktionsnummer: Zeichen Vorhanden?
CALL INTERRUPT(&H16, Register, Register)    'BIOS-Tastatur-Interrupt aufrufen

END SUB

