/**********************************************************************/
/*                   C D D E V . C                                    */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Utils fr CD-Device-Treiber Programmierung       */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 08.04.1994                                       */
/*  letztes Update : 18.04.1994                                       */
/*                                                                    */
/* Achtung: Struct-Member Alignment mu auf 1 Byte gesetzt werden, da */
/*          Treiber nicht auf logischen Strukturen, sondern auf       */
/*          physischen BYTE-Offsets operiert!                         */
/*          CDDEV.C funktioniert nicht ohne CDUTIL.C/OBJ!             */
/**********************************************************************/
#ifndef __CDDEV_C               /* CDDEV.C kann auch #Included werden */
#define __CDDEV_C

#include <stdio.h>
#include <conio.h>

#include "types.h"
#include "cddev.h"
#include "cdutil.h"
           /* Klartext fr Device-Status-Bits */
static char *DevStat [13][2] = {
{ "Door is open", "Door is closed" },
{ "Door is unlocked", "Door is locked" },
{ "Drive reads COOKED and RAW", "Drive reads only COOKED" },
{ "Drive reads and writes", "Drive only reads" },
{ "Drive reads DATA and plays AUDIO/VIDEO tracks",
  "Drive reads only DATA tracks" },
{ "Drive supports ISO 9600 interleave",
  "Drive supports no interleave"},
{ "reserved", "reserved" },
{ "Drive supports prefetching requests",
  "Drive doesn't support prefetching requests"},
{ "Drive supports audio-channel manipulation",
  "Drive doesn't support audio-channel manipulation" },
{ "Drive supports HSG and REDBOOK adressing",
  "Drive supports HSG Adressing only"},
{ "reserved", "reserved" },
{ "No disk in drive", "Disk in drive"},
{ "Drive supports R/W sub-channel-info",
  "Drive doesn't support R/W sub-channel-info" } };

/**********************************************************************/
/* _CallStrategy: Sende Device-Request an Device-Treiber              */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Nummer des Laufwerks, an dessen       */
/*                              Device-Treiber ein Device-Request ge- */
/*                              sendet werden soll. (0=A, 1=B, etc.)  */
/*           lpReqHdr         - Adresse des Device-Request-Headers    */
/* Ausgabe : Status des Device nach Operation                         */
/*           (s. RequestHeader.wStatus)                               */
/**------------------------------------------------------------------**/
/* Info: Diese Funktion ruft zuerst die Strategy, dann die Interrupt- */
/*       Funktion des Device-Treibers auf, dessen Header und SubUnit  */
/*       durch den bergebenen Laufwerksbuchstaben bestimmt werden.   */
/*       Diese Funktion wird aufgerufen, wenn die Versionsnummer von  */
/*       MSCDEX < 2.10 ist.                                           */
/**********************************************************************/
INT _CallStrategy( INT iCD_Drive_Letter, LPRequestHeader lpReqHdr )
{ static VOID (_FP *Strategy)( VOID );           /* Funktionsadressen */
  static VOID (_FP *Interrupt)( VOID );
  static INT S, O;                                 /* Segment, Offset */
  static INT i;
  static INT         iNumUnits = -1;
  static CHAR        cCDLetters[ 26 ]; /* Alle CD-Laufwerksbuchstaben */
  static DevElement  DevElements[ 26 ];

  if( iNumUnits == -1 )  /* Alle statischen Variablen initialisiert ? */
  { static INT iStart, iVersion;
    MSCDEX_GetNumberOfDriveLetters( &iNumUnits, &iStart );
    MSCDEX_GetCDRomDriveDeviceList( DevElements );

    if( iVersion >= 0x200 )      /* Existiert GetCDRomDriveLetters()? */
        MSCDEX200_GetCDRomDriveLetters( cCDLetters );
    else         /* ansonsten aufeinanderfolgende Laufwerksbuchstaben */
    for( i = 0; i < iNumUnits; i++ )  cCDLetters[ i ] =
                                      ( char ) ( iStart + i );
  }

  for( i = 0; i < iNumUnits; i++ )
    if( cCDLetters[ i ] == iCD_Drive_Letter )    /* Welches Laufwerk? */
    { LPDeviceHeader lpDevHdr;
      lpDevHdr = DevElements[ i ].lpDeviceHeader;  /* DeviceHader und */
      lpReqHdr->bSubUnit = DevElements[ i ].bSubUnit;      /* SubUnit */
      if( FP_OFF( lpDevHdr ) ) return -1;        /* Immer an Offset 0 */
      Strategy = ( VOID (_FP *)( VOID ) ) MK_FP( FP_SEG( lpDevHdr ),
                                          lpDevHdr->wStrategy );
      Interrupt = ( VOID (_FP *)( VOID ) ) MK_FP( FP_SEG( lpDevHdr ),
                                           lpDevHdr->wInterrupt );
      S = FP_SEG( lpReqHdr );  /* Segment und Offset ermitteln, bevor */
      O = FP_OFF( lpReqHdr );  /* asm-Anweisungen ausgefhrt werden!  */

      _asm mov es,S                                   /* ES:BX setzen */
      _asm mov bx,O
      Strategy();          /* Zuerst Interne Bufferadressen setzen... */
      Interrupt();                          /* ...dann Code ausfhren */
      return lpReqHdr->wStatus;
    }
  return 0xFFFF;
}

/**********************************************************************/
/* cd_IsError : Beschreibt Status-Wort einen Fehler?                  */
/**------------------------------------------------------------------**/
/* Eingabe : wStatus - zu testendes Status-Wort                       */
/* Ausgabe : TRUE  - Status-Wort beschreibt Fehler                    */
/*           FALSE - Status-Wort beschreibt keinen Fehler             */
/**------------------------------------------------------------------**/
/* Info    : Diese Funktion stellt fest, ob das Fehler-Bit (Bit15)    */
/*           gesetzt ist.                                             */
/**********************************************************************/
INT cd_IsError( WORD wStatus )
{
  return ( wStatus & DEV_ERROR ) ? TRUE : FALSE;
}

/**********************************************************************/
/* cd_GetReqHdrError : Ermittle Fehler der letzten Device-            */
/*                     Kommunikation                                  */
/**------------------------------------------------------------------**/
/* Eingabe : lpReqHdr - Adresse des Request-Header, der auf Fehler    */
/*                      untersucht werden soll.                       */
/* Ausgabe :  > 0 - FehlerCode + 1                                    */
/*           == 0 - Kein Fehler                                       */
/**********************************************************************/
INT cd_GetReqHdrError( LPRequestHeader lpReqHdr )
{
  return cd_IsError( lpReqHdr->wStatus ) ?           /* Bit 15 testen */
                ( lpReqHdr->wStatus & 0x00FF ) + 1 : 0;
}

/**********************************************************************/
/* cd_IsReqHdrBusy : Ist Device noch mit Ausfhrung von Kommandos     */
/*                   beschftigt ?                                    */
/**------------------------------------------------------------------**/
/* Eingabe : lpReqHdr - Adresse des Device-Headers                    */
/* Ausgabe : TRUE  - Device noch mit alten Kommandos beschftigt      */
/*           FALSE - Device nicht mehr beschftigt                    */
/**********************************************************************/
INT cd_IsReqHdrBusy( LPRequestHeader lpReqHdr )
{
  return ( lpReqHdr->wStatus & DEV_BUSY ) ? TRUE : FALSE;    /* Bit 9 */
}

/**********************************************************************/
/* cd_IsReqHdrDone : Ist letztes Device-Kommando vollstndig          */
/*                   ausgefhrt worden ?                              */
/**------------------------------------------------------------------**/
/* Eingabe : lpReqHdr - Adresse des Device-Headers                    */
/* Ausgabe : TRUE  - Device-Kommando wurde ausgefhrt                 */
/*           FALSE - Device noch BUSY                                 */
/**********************************************************************/
INT cd_IsReqHdrDone( LPRequestHeader lpReqHdr )
{
  return ( lpReqHdr->wStatus & DEV_DONE ) ? TRUE : FALSE;    /* Bit 8 */
}

/*- Wrapper-Funktionen fr MSCDEX-Device Kommandos -------------------*/

/**********************************************************************/
/* cd_GetDeviceHeaderAdress : Ermittle Adresse des Device-Headers     */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpRA             - Raddr-ControlBlock                    */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpRA->lpDeviceHeaderAdress enthlt nach Aufruf die          */
/*        Adresse des Headers des fr das angegebene CD-Laufwerk      */
/*        zustndigen Device-Treibers                                 */
/**********************************************************************/
INT cd_GetDeviceHeaderAdress( INT            iCD_Drive_Letter,
                              LPMSCDEX_Raddr lpRA )
{
  lpRA->bControlBlockCode = IOCTLI_GET_DEVHDR_ADRESS;          /* = 0 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpRA, sizeof( LPMSCDEX_Raddr ) );
}

/**********************************************************************/
/* cd_GetLocationOfHead : Ermittle Position des (Schreib-)/Lesekopfes */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpLH             - LocHead-ControlBlock                  */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpLH->bAdressingMode mu vor dem Aufruf Adressierungs-      */
/*        Modus enthalten (HSG(0) oder REDBOOK(1)). Nach dem Aufruf   */
/*        befindet sich dann in lpLH->lLocation die aktuelle Position */
/*        des (Schreib-)/Lesekopfes.                                  */
/**********************************************************************/
INT cd_GetLocationOfHead( INT iCD_Drive_Letter, LPMSCDEX_LocHead lpLH )
{
  lpLH->bControlBlockCode = IOCTLI_GET_HEAD_LOCATION;          /* = 1 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpLH, sizeof( LPMSCDEX_Raddr ) );
}

/**********************************************************************/
/* cd_GetAudioChannelInfo : Ermittle Zuordnung der CD-Kanle zu       */
/*                          CD-Ausgngen                              */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpAI             - AudInfo-ControlBlock                  */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpAI->Output0-4 und lpAI->Volume0-4 enthalten Zuordnung und */
/*         Lauststrke der 4 mglichen Kanle                         */
/**********************************************************************/
INT cd_GetAudioChannelInfo( INT              iCD_Drive_Letter,
                            LPMSCDEX_AudInfo lpAI )
{
  lpAI->bControlBlockCode = IOCTLI_GET_AUDIO_CHANNEL_INFO;     /* = 4 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpAI, sizeof( LPMSCDEX_AudInfo ) );
}

/**********************************************************************/
/* cd_ReadDriveBytes : Lies Laufwerk-Spezifische Daten                */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpDB             - DrvBytes-ControlBlock                 */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpDB->bNumReadBytes enthlt vor dem Aufruf die Anzahl der   */
/*        zu lesenden Bytes. Nach dem Aufruf stehen in                */
/*        lpDB->bReadBuffer die gelesenen Bytes. Sollen mehr als 128  */
/*        Bytes gelesen werden werden die vorherigen 128Byte-Blcke   */
/*        verworfen, um den durch INT(NumReadBytes/128) bestimmten    */
/*        Block zu bergeben.                                         */
/**********************************************************************/
INT cd_ReadDriveBytes( INT iCD_Drive_Letter, LPMSCDEX_DrvBytes lpDB )
{
  lpDB->bControlBlockCode = IOCTLI_READ_DRIVE_BYTES;           /* = 5 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpDB, sizeof( LPMSCDEX_DrvBytes ) );
}

/**********************************************************************/
/* cd_GetDevStat : Lese Device-Status (Laufwerkszustand )             */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpDS             - DevStat-ControlBlock                  */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpDS->lDeviceStatus enthlt nach Aufruf den Device-Status   */
/**********************************************************************/
INT cd_GetDevStat( INT iCD_Drive_Letter, LPMSCDEX_DevStat lpDS )
{
  lpDS->bControlBlockCode = IOCTLI_GET_DEV_STAT;               /* = 6 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpDS, sizeof( MSCDEX_DevStat ) );
}

/**********************************************************************/
/* cd_PrintDevStat : Device-Status ausgeben                           */
/**------------------------------------------------------------------**/
/* Eingabe : lpDS - zuvor mittels GetDevStat ermittelter Device-Status*/
/**********************************************************************/
VOID cd_PrintDevStat( LPMSCDEX_DevStat lpDS )
{ INT i;
     /* Alle Bits durchgehen und Meldung fr gesetztes/gelschtes Bit */
     /* ausgeben                                                      */
  for( i = 0; i < 13; i++ )
    printf("%s\n", DevStat[i][(lpDS->lDeviceStatus&(1<<i))?0:1]);
}

/**********************************************************************/
/* cd_GetSectorSize : Ermittle Gre eines Sektors                    */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpSec            - SectSize-ControlBlock                 */
/*           iReadMode        - COOKED ( 0 ) oder RAW ( 1 )           */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpSec->wSectorSize enthlt nach Aufruf Sektor-Gre (Bytes) */
/**********************************************************************/
INT cd_GetSectorSize( INT               iCD_Drive_Letter,
                      LPMSCDEX_SectSize lpSEC,
                      INT               iReadMode )
{
  lpSEC->bControlBlockCode = IOCTLI_SECTOR_SIZE;               /* = 7 */
  lpSEC->bReadMode = ( BYTE ) iReadMode;
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpSEC, sizeof( MSCDEX_SectSize ) );
}

/**********************************************************************/
/* cd_GetVolumeSize : Ermittle Anzahl der Sektoren einer CD           */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpVS             - VolSize-ControlBlock                  */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpVS->lVolumeSize enthlt nach Aufruf Anzahl der Sektoren   */
/*        der eingelegten CD.                                         */
/**********************************************************************/
INT cd_GetVolumeSize( INT iCD_Drive_Letter, LPMSCDEX_VolSize lpVS )
{
  lpVS->bControlBlockCode = IOCTLI_VOLUME_SIZE;                /* = 8 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpVS, sizeof( LPMSCDEX_VolSize ) );
}

/**********************************************************************/
/* cd_GetMediaChanged : Wurde CD gewechselt?                          */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpMC             - MedChng-ControlBlock                  */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpMC->bMediaChanged enthlt nach Aufruf Status des Mediums  */
/*        1   : CD seit letzem Aufruf nicht gewechselt.               */
/*        0   : Wei nicht!?!                                         */
/*        255 : CD wurde seit letztem Aufruf gewechselt.              */
/**********************************************************************/
INT cd_GetMediaChanged( INT iCD_Drive_Letter, LPMSCDEX_MedChng lpMC )
{
  lpMC->bControlBlockCode = IOCTLI_MEDIA_CHANGED;              /* = 9 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpMC, sizeof( LPMSCDEX_MedChng ) );
}

/**********************************************************************/
/* cd_GetAudioDiskInfo : Ermittle Anzahl der Titel (Stcke, Tracks)   */
/*                       einer Audio-CD.                              */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpDI             - DiskInfo-ControlBlock                 */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpDI->bLowestTrack, lpDI->bHighestTrack enthalten nach      */
/*        Aufruf Nummer des ersten bzw. letzten Liedes.               */
/*        lpDI->bLeadOutTrack enthlt Adresse des ersten unbenutzen   */
/*        Sektors (nach letzem Titel)                                 */
/**********************************************************************/
INT cd_GetAudioDiskInfo( INT iCD_Drive_Letter, LPMSCDEX_DiskInfo lpDI )
{
  lpDI->bControlBlockCode = IOCTLI_AUDIO_DISK_INFO;           /* = 10 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpDI, sizeof( LPMSCDEX_DiskInfo ) );
}

/**********************************************************************/
/* cd_GetAudioTrackInfo : Ermittle Daten zu einem Titel               */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           iTrackNo         - Nummer des Tracks, fr den Daten      */
/*                              ermittelt werden sollen.              */
/*           lpTI             - TackInfo-ControlBlock                 */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpTI->lStartingPoINT    - REDBOOK Adresse des Titelstarts   */
/*        lpTI->bTrackControlInfo - Informationen ber Track-Typ      */
/**********************************************************************/
INT cd_GetAudioTrackInfo( INT              iCD_Drive_Letter,
                          INT              iTrackNo,
                          LPMSCDEX_TnoInfo lpTI )
{
  lpTI->bControlBlockCode = IOCTLI_AUDIO_TRACK_INFO;          /* = 11 */
  lpTI->bTrackNo = ( BYTE ) iTrackNo;
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpTI, sizeof( LPMSCDEX_TnoInfo ) );
}

/**********************************************************************/
/* cd_IsDataTrack : Ist ein Track ein reiner Daten Track, oder        */
/*                  enthlt er Audio-Informationen?                   */
/**------------------------------------------------------------------**/
/* Eingabe : lpTI - Adresse eines TrackInfo-ControlBlocks             */
/*                  (s. GetAudioTrackInfo)                            */
/* Ausgabe : TRUE  - lpTI beschreibt Daten-Track                      */
/*           FALSE - Track enthlt Audio-Daten                        */
/**********************************************************************/
INT cd_IsDataTrack( LPMSCDEX_TnoInfo lpTI )
{
  return ( ( lpTI->bTrackControlInfo & TCI_TRACK_MASK ) ==
             TCI_DATA_TRACK ) ? TRUE : FALSE;
}

/**********************************************************************/
/* cd_GetTrackLen : Ermittle Gre eines Tracks (Anzahl Sektoren bzw.  */
/*                  Frames)                                           */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           iTrackNo         - Nummer des Tracks, fr den Daten      */
/*                              ermittelt werden sollen.              */
/* Ausgabe : Anzahl der Frames des angegebenen Tracks                 */
/**------------------------------------------------------------------**/
/* Info : Beim letzten Track mu der Lead-Out-Track als Terminator    */
/*        verwendet werden.                                           */
/**********************************************************************/
LONG cd_GetTrackLen( INT iCD_Drive_Letter, INT iTno )
{ MSCDEX_DiskInfo DI;
  MSCDEX_TnoInfo  TI;
  MSCDEX_VolSize  VS;
  LONG            lStart;
  LONG            lEnd;

  cd_GetAudioDiskInfo( iCD_Drive_Letter, &DI );
  if( ( iTno < DI.bLowestTrack ) || ( iTno > DI.bHighestTrack ) )
    return 0L;                                 /* kein gltiger Track */

  cd_GetAudioTrackInfo( iCD_Drive_Letter, iTno, &TI );
  lStart = REDBOOK2HSG( TI.lStartingPoint );
  if( iTno < DI.bHighestTrack )  /* Differenz aufeinanderfolg. Tracks */
  {
    cd_GetAudioTrackInfo( iCD_Drive_Letter, iTno + 1, &TI );
    lEnd = REDBOOK2HSG( TI.lStartingPoint );
  }
  else                     /* Differenz [Ende der CD - letzter Track] */
  {
    cd_GetVolumeSize( iCD_Drive_Letter, &VS );
    lEnd = VS.lVolumeSize - 150L;          /* Frames in HSG umrechnen */
  }
  return lEnd - lStart;
}

/**********************************************************************/
/* cd_QueryAudioChannel : Ermittle Online-Informationen ber zur Zeit */
/*                        gespielten Audio-Titel                      */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpQI             - QInfo-ControlBlock                    */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpQI->bTrackNo/bIndex/bTrackMin/bTrackSec/bTrackFrame       */
/*           Informationen ber aktuell abgespieltes Lied             */
/*           (Nummer und Spielzeit)                                   */
/*        lpQI->bDiskMin / bDiskSec / bDiskFrame                      */
/*           bisherige Gesammtspiezeit                                */
/*        Diese Funktion kann laufend aufgerufen werden, um die       */
/*        aktuellen Spielzeiten zu erhalten                           */
/**********************************************************************/
INT cd_QueryAudioChannel( INT iCD_Drive_Letter, LPMSCDEX_QInfo lpQI )
{
  lpQI->bControlBlockCode = IOCTLI_AUDIO_QUERY_CHANNEL;       /* = 12 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpQI, sizeof( LPMSCDEX_QInfo ) );
}

/**********************************************************************/
/* cd_GetAudioSubChannelInfo : Ermittle Sub-Channel-Informationen     */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpSCI            - SubChanInfo-ControlBlock              */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info :    In der in lpSCI->lpTransferAdress angegebenen Adresse    */
/*           finden sich nach Aufruf dieser Funktion die              */
/*           Sub-Channel-Inforationen der angebenenen Sektoren        */
/*           (lpSCI->lStartingSektor, lpSCI->lNumberOfSectorsToRead)  */
/*           Fr jeden Sektoe mu ein 96Bytes groer Buffer bereit-   */
/*           gestellt werden.                                         */
/**********************************************************************/
INT cd_GetAudioSubChannel( INT                  iCD_Drive_Letter,
                           LPMSCDEX_SubChanInfo lpSCI )
{
  lpSCI->bControlBlockCode = IOCTLI_AUDIO_SUB_CHANNEL_INFO;   /* = 13 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpSCI, sizeof( MSCDEX_SubChanInfo ) );
}

/**********************************************************************/
/* cd_GetUPCode : Ermittle Universal-Product-Code einer CD            */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpUPC            - UPCode-ControlBlock                   */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info :    Nach dem Aufruf befindet sich in  lpUPC->bUPC_EAN eine   */
/*           13 stellige BCD-Zahl, die den UP-Code der aktuellen CD   */
/*           angibt.                                                  */
/**********************************************************************/
INT cd_GetUPCode( INT iCD_Drive_Letter, LPMSCDEX_UPCode lpUPC )
{
  lpUPC->bControlBlockCode = IOCTLI_UPCODE;                   /* = 14 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpUPC, sizeof( MSCDEX_UPCode ) );
}

/**********************************************************************/
/* cd_PrintUPCode : Gib UPC BCD-Zahl aus.                             */
/**------------------------------------------------------------------**/
/* Eingabe : lpUPC - UPCode-ControlBlock                              */
/**------------------------------------------------------------------**/
/* Info : Letztes Nibble des bUPC_EAN-Arrays wird nicht verwendet     */
/**********************************************************************/
VOID cd_PrintUPCode( LPMSCDEX_UPCode lpUPC )
{ INT i;
  printf(" UPC: ");
  for( i = 0; i < 6; i++ )                   /* die ersten 12 Ziffern */
    printf("%c%c", ( ( lpUPC->bUPC_EAN[ i ] >>  4 ) & 0x0F ) + '0',
                   ( ( lpUPC->bUPC_EAN[ i ] ) & 0x0F ) + '0' );
  printf("%c\n", ( ( lpUPC->bUPC_EAN[ i ] >>  4 ) & 0x0F ) + '0');
}

/**********************************************************************/
/* cd_GetAudioStatusInfo : Ermittle aktuellen Audio Zustand           */
/*                         (Play/Pause)                               */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpAS             - AudStat-ControlBlock                  */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info :    Nach dem Aufruf gibt Bit 0 der Variablen                 */
/*           lpAS->wAudioStatus Auskunft ber den Spielmodus des      */
/*           CD-Laufwerks. (Gesetzt = Pause)                          */
/*           lpAS->lResumeStart enthlt die als nchstes abzu-        */
/*           spielende Position bei einem RESUME. lpAS->lResumeEnd    */
/*           gibt an, wann das mit dem Abspielen des Liedes auf-      */
/*           gehrt werden soll. (s. STOP_AUDIO / PLAY_AUDIO )        */
/**********************************************************************/
INT cd_GetAudioStatusInfo( INT iCD_Drive_Letter, LPMSCDEX_AudStat lpAS )
{
  lpAS->bControlBlockCode = IOCTLI_AUDIO_STATUS_INFO;         /* = 15 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_READ,
                              lpAS, sizeof( LPMSCDEX_AudStat ) );
}

/**********************************************************************/
/* cd_Eject : Lade ffnen (ausfahren)                                 */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : Vor dem ffnen der Lade, wird diese entriegelt              */
/**********************************************************************/
INT cd_Eject( INT iCD_Drive_Letter )
{ MSCDEX_Eject E;
  E.bControlBlockCode = IOCTLO_EJECT_DISK;                     /* = 0 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_WRITE,
                             ( LPVOID )&E, sizeof( MSCDEX_Eject ) );
}

/**********************************************************************/
/* cd_LockDoor : Lade ver-/entriegeln                                 */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           iLock            - == 0 : Lade entriegeln                */
/*                              <> 0 : Lade verriegeln                */
/* Ausgabe : Device-Status                                            */
/**********************************************************************/
INT cd_LockDoor( INT iCD_Drive_Letter, INT iLock  )
{ MSCDEX_LockDoor LD;
  LD.bControlBlockCode = IOCTLO_LOCK_DOOR;                     /* = 1 */
  LD.bLock = ( BYTE ) ( iLock ? 1 : 0 );
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_WRITE,
                              &LD, sizeof( MSCDEX_LockDoor ) );
}

/**********************************************************************/
/* cd_Reset : Laufwerk zurcksetzen                                   */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/* Ausgabe : Device-Status                                            */
/**********************************************************************/
INT cd_ResetDrive( INT iCD_Drive_Letter )
{ MSCDEX_Reset R;
  R.bControlBlockCode = IOCTLO_RESET_DRIVE;                    /* = 2 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_WRITE,
                             ( LPVOID )&R, sizeof( MSCDEX_Reset ) );
}

/**********************************************************************/
/* cd_SetAudioChannelInfo : Setze Zuordnung der CD-Kanle zu          */
/*                          CD-Ausgngen                              */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpAI             - AudInfo-ControlBlock                  */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : lpAI->Output0-4 und lpAI->Volume0-4 enthalten Zuordnung und */
/*        Lauststrke der 4 mglichen Kanle                          */
/**********************************************************************/
INT cd_SetAudioChannelInfo( INT              iCD_Drive_Letter,
                            LPMSCDEX_AudInfo lpAI )
{
  lpAI->bControlBlockCode = IOCTLO_SET_AUDIO_CHANNEL_INFO;     /* = 3 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_WRITE,
                              lpAI, sizeof( LPMSCDEX_AudInfo ) );
}

/**********************************************************************/
/* cd_WriteDriveBytes : bergibt Laufwerk-Spezifische Daten           */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           lpDB             - DrvBytes-ControlBlock                 */
/* Ausgabe : Device-Status                                            */
/**********************************************************************/
INT cd_WriteDriveBytes( INT iCD_Drive_Letter, LPMSCDEX_DrvBytes lpDB )
{
  lpDB->bControlBlockCode = IOCTLO_WRITE_DRIVE_BYTES;           /* = 4 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_WRITE,
                              lpDB, sizeof( LPMSCDEX_DrvBytes ) );
}

/**********************************************************************/
/* cd_CloseTray : Lade verschliessen (einfahren)                      */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter : Laufwerkskennung( 0=A, etc )          */
/* Ausgabe : Device-Status                                            */
/**********************************************************************/
INT cd_CloseTray( INT iCD_Drive_Letter )
{ MSCDEX_CloseTray CT;
  CT.bControlBlockCode = IOCTLO_CLOSE_TRAY;                    /* = 5 */
  return MSCDEX_ReadWriteReq( iCD_Drive_Letter, DEVCMD_IOCTL_WRITE,
                       ( LPVOID )&CT, sizeof( MSCDEX_CloseTray ) );
}

/**********************************************************************/
/* cd_PlayAudio : Spielt Musikdaten ab                                */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           iAdressMode      - HSG(0) oder REDBOOK(1)-Adressen?      */
/*           lStart           - Adresse ab der mit dem Abspielen      */
/*                              begonnen werden soll.                 */
/*           lLen             - Anzahl der abzuspielenden Sektoren    */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info    : Durch dieses Kommando werden BUSY und PAUSE-Bit (Audio-  */
/*           Stat) aufdatiert. Ist der CD-Player schon mit dem Ab-    */
/*           spielen von Musikdaten beschftigt, mu vor einem neuen  */
/*           PLAY_AUDIO-Request erst ein STOP_AUDIO-Request           */
/*           abgesendet werden. Untersttzt das Laufwerk kein Audio   */
/*           wird dieser Befehl ignoriert.                            */
/**********************************************************************/
INT cd_PlayAudio( INT  iCD_Drive_Letter,
                  INT  iAdressMode,
                  LONG lStart,
                  LONG lLen )
{ MSCDEX_PlayReq PR;

  lLen = lLen > 0 ? lLen : 0L;                       /* Immer Positiv */

  PR.ReqHdr.bLength  = sizeof( RequestHeader );
  PR.ReqHdr.bSubUnit = 0;
  PR.ReqHdr.bCommand = ( BYTE )DEVCMD_PLAY_AUDIO;
  PR.bAdressingMode  = ( BYTE ) iAdressMode;
  PR.lStartSector    = lStart;
  PR.lSectorCount    = lLen;
  return MSCDEX_SendDeviceRequest( iCD_Drive_Letter, &PR.ReqHdr );
}

/**********************************************************************/
/* cd_StopAudio : Stoppt Musikausgabe                                 */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info    : Durch das Play-Kommando abzuspielende Musikdaten werden  */
/*           solange abgespielt, bis alle zu spielenden Sektoren aus- */
/*           gegegeben wurden. Um die Musikausgabe aber vorzeitig zu  */
/*           unterbrechen, mu das Stop-Kommando gesendet werden.     */
/*           Wird ein Play-Request durch einen einmaligen Stop-Aufruf */
/*           unterbrochen, kann dieser durch ein Resume wieder        */
/*           gestartet werden. Es wird ab der letzten Position        */
/*           weitergespielt. Nach zwei aufeinanderfolgenden Stop-     */
/*           Requests kann die Audioausgabe nur durch einen neuen     */
/*           Play-Request gestartet werden.                           */
/**********************************************************************/
INT cd_StopAudio( INT iCD_Drive_Letter )
{ RequestHeader ReqHdr;

  ReqHdr.bLength  = sizeof( RequestHeader );
  ReqHdr.bSubUnit = 0;
  ReqHdr.bCommand = ( BYTE )DEVCMD_STOP_AUDIO;

  return MSCDEX_SendDeviceRequest( iCD_Drive_Letter, &ReqHdr );
}

/**********************************************************************/
/* cd_ResumeAudio : Startet einmalig unterbrochene Musikausgabe       */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/* Ausgabe : Device-Status                                            */
/**********************************************************************/
INT cd_ResumeAudio(  INT iCD_Drive_Letter )
{ RequestHeader ReqHdr;

  ReqHdr.bLength = sizeof( RequestHeader );
  ReqHdr.bSubUnit = 0;
  ReqHdr.bCommand = ( BYTE )DEVCMD_RESUME_AUDIO;

  return MSCDEX_SendDeviceRequest( iCD_Drive_Letter, &ReqHdr );
}

/**********************************************************************/
/* cd_Seek : Position (Schreib-)/Lesekopf                             */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           iAdressingMode   - HSG(0) oder REDBOOK(1)                */
/*           lSector          - Anzufahrender Sektor                  */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : Die Positionierung des (Schreib-)/Lesekopf unterscheidet    */
/*        sich von der Positionierung des Dateizeiger innerhalb eines */
/*        Files. Beim CD-Laufwerk kann Seek Dateibergreifend (auch   */
/*        bei der Audio-Ausgabe) verwendet werden! Bei der Audio-     */
/*        Ausgabe kann ein Seek allerdings zur Positionierung ber    */
/*        das Ende der Ausgabe (Anzahl abzuspielender Sektoren)       */
/*        bewegt werden.                                              */
/**********************************************************************/
INT cd_Seek( INT iCD_Drive_Letter, INT iAdressingMode, LONG lSector )
{ MSCDEX_SeekReq SR;

  SR.ReqHdr.bLength   = sizeof( RequestHeader );
  SR.ReqHdr.bSubUnit  = 0;
  SR.ReqHdr.bCommand  = ( BYTE )DEVCMD_SEEK;

  SR.bAdressingMode   = ( BYTE ) iAdressingMode;
  SR.lpTransferAdress = NULL;
  SR.wNumSec          = 0;
  SR.lStartingSector  = lSector;
  return MSCDEX_SendDeviceRequest( iCD_Drive_Letter, &SR.ReqHdr );
}

/**********************************************************************/
/* cd_ReadLong : Liest Anzahl von Sektoren (s. AbsoluteDiskRead)      */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           iAdressingMode   - HSG(0) oder REDBOOK(1)                */
/*           lStart           - Erster zu lesender Sektor             */
/*           iNumSec          - Anzahl der zu lesenden Sektoren       */
/*           lpReadData       - Adresse des Lesebuffers               */
/*           iDataMode        - RAW(1) oder COOKED(0)                 */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : Ein RAW-Sektor ist 2352 Bytes gro. COOKED-Sektoren         */
/*        hingegen nur 2048 Bytes.                                    */
/**********************************************************************/
INT cd_ReadLong( INT       iCD_Drive_Letter,
                 INT       iAdressMode,
                 LONG      lStart,
                 INT       iNumSec,
                 LPVOID    lpReadData,
                 INT       iDataMode )
{ MSCDEX_ReadWriteL RWL;

  RWL.ReqHdr.bLength        = sizeof( RequestHeader );
  RWL.ReqHdr.bSubUnit       = 0;
  RWL.ReqHdr.bCommand       = ( BYTE )DEVCMD_READLONG;

  RWL.bAdressingMode        = ( BYTE ) iAdressMode;
  RWL.lpTransferAdress      = lpReadData;
  RWL.wNumSec               = iNumSec;
  RWL.lStartingSector       = lStart;
  RWL.bDataReadWriteMode    = ( BYTE ) iDataMode;
  RWL.bInterleaveSize       = 1;
  RWL.bInterleaveSkipFactor = 1;

  return MSCDEX_SendDeviceRequest( iCD_Drive_Letter, &RWL.ReqHdr );
}

/**********************************************************************/
/* cd_ReadLongPrefetch : Teilt Laufwerk den nchsten zu lesenden      */
/*                       Sektor mit.                                  */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           iAdressingMode   - HSG(0) oder REDBOOK(1)                */
/*           lStart           - Erster zu lesender Sektor             */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : Um die langsame Geschwindigkeit der CD's ein wenig auszu-   */
/*        gleichen, kann mittels Prefetching ein vereinfachtes Cache  */
/*        System (ein Sektor gro) aufgebaut werden. Dieser Befehl    */
/*        bermittelt dem Device-Treiber dazu den nchsten zu         */
/*        lesenden Sektor, der im "vorbeigehen" gelesen wird.         */
/**********************************************************************/
INT cd_ReadLongPrefetch( INT  iCD_Drive_Letter,
                         INT  iAdressMode,
                         LONG lStart )
{ MSCDEX_ReadWriteL RWL;

  RWL.ReqHdr.bLength        = sizeof( RequestHeader );
  RWL.ReqHdr.bSubUnit       = 0;
  RWL.ReqHdr.bCommand       = ( BYTE )DEVCMD_READLONG_PREFETCH;

  RWL.bAdressingMode        = ( BYTE ) iAdressMode;
  RWL.lpTransferAdress      = NULL;
  RWL.wNumSec               = 1;
  RWL.lStartingSector       = lStart;
  RWL.bDataReadWriteMode    = 0;
  RWL.bInterleaveSize       = 0;
  RWL.bInterleaveSkipFactor = 0;

  return MSCDEX_SendDeviceRequest( iCD_Drive_Letter, &RWL.ReqHdr );
}

/**********************************************************************/
/* cd_WriteLong : Schreibt Anzahl von Sektoren (s. AbsoluteDiskWrite) */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           iAdressingMode   - HSG(0) oder REDBOOK(1)                */
/*           lStart           - Erster zu schreibender Sektor         */
/*           iNumSec          - Anzahl der zu schreibenden Sektoren   */
/*           lpWriteData      - Adresse des Schreibbuffers            */
/*           iDataMode        - Mode0(0), Mode1(1),                   */
/*                              Mode2Form1(2), Mode2Form2(3)          */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : Mode0 -      Sektoren werden mit Nullen beschriben          */
/*        Mode1 -      COOKED-Write (2048 Byte groe Sektoren)        */
/*        Mode2Form1 - COOKED-Write (2048 Byte groe Sektoren)        */
/*        Mode2Form2 - RAW-Write (2352 Byte groe Sektoren)           */
/**********************************************************************/
INT cd_WriteLong( INT       iCD_Drive_Letter,
                  INT       iAdressMode,
                  LONG      lStart,
                  INT       iNumSec,
                  LPVOID    lpWriteData,
                  INT       iDataMode )
{ MSCDEX_ReadWriteL RWL;

  RWL.ReqHdr.bLength        = sizeof( RequestHeader );
  RWL.ReqHdr.bSubUnit       = 0;
  RWL.ReqHdr.bCommand       = ( BYTE )DEVCMD_WRITELONG;

  RWL.bAdressingMode        = ( BYTE ) iAdressMode;
  RWL.lpTransferAdress      = lpWriteData;
  RWL.wNumSec               = iNumSec;
  RWL.lStartingSector       = lStart;
  RWL.bDataReadWriteMode    = ( BYTE )iDataMode;
  RWL.bInterleaveSize       = 0;
  RWL.bInterleaveSkipFactor = 0;

  return MSCDEX_SendDeviceRequest( iCD_Drive_Letter, &RWL.ReqHdr );
}

/**********************************************************************/
/* cd_WriteLongVerify : Schreibt unf Prft Daten auf CD mit Daten     */
/*                      im Speicher                                   */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/*           iAdressingMode   - HSG(0) oder REDBOOK(1)                */
/*           lStart           - Erster zu berprfenden Sektor        */
/*           iNumSec          - Anzahl der zu berprfenden Sektoren  */
/*           lpVerifyData     - Adresse des Prfbuffers               */
/*           iDataMode        - Mode1(1),                             */
/*                              Mode2Form1(2), Mode2Form2(3)          */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info : Mode1:      COOKED-Verify (2048 Byte groe Sektoren)        */
/*        Mode2Form1: COOKED-Verify (2048 Byte groe Sektoren)        */
/*        Mode2Form2: RAW-Verify (2352 Byte groe Sektoren)           */
/**********************************************************************/
INT cd_WriteLongVerify( INT       iCD_Drive_Letter,
                        INT       iAdressMode,
                        LONG      lStart,
                        INT       iNumSec,
                        LPVOID    lpVerifyData,
                        INT       iDataMode )
{ MSCDEX_ReadWriteL RWL;

  RWL.ReqHdr.bLength        = sizeof( RequestHeader );
  RWL.ReqHdr.bSubUnit       = 0;
  RWL.ReqHdr.bCommand       = ( BYTE )DEVCMD_WRITELONG_VERIFY;

  RWL.bAdressingMode        = ( BYTE )iAdressMode;
  RWL.lpTransferAdress      = lpVerifyData;
  RWL.wNumSec               = iNumSec;
  RWL.lStartingSector       = lStart;
  RWL.bDataReadWriteMode    = ( BYTE )iDataMode;
  RWL.bInterleaveSize       = 0;
  RWL.bInterleaveSkipFactor = 0;

  return MSCDEX_SendDeviceRequest( iCD_Drive_Letter, &RWL.ReqHdr );
}

/**********************************************************************/
/* cd_PrintDiskTracks : Zeige alle Titel und zugehrige Spielzeiten   */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/**********************************************************************/
VOID cd_PrintDiskTracks( INT iCD_Drive_Letter )
{ INT             i;
  MSCDEX_DiskInfo DI;
  MSCDEX_TnoInfo  TI;

  if( !cd_IsError( cd_GetAudioDiskInfo( iCD_Drive_Letter, &DI ) ) )
    for( i = DI.bLowestTrack; i <= DI.bHighestTrack; i++ )
    { static INT iMin, iSec, iFrame;
      if( !cd_IsError( cd_GetAudioTrackInfo( iCD_Drive_Letter,
                                             i, &TI ) ) )
      { LONG lLen;
        REDBOOK2Time( TI.lStartingPoint, &iMin, &iSec, &iFrame );
        printf(" %2d: %2d:%02d.%02d   ",i, iMin, iSec, iFrame );
        lLen = cd_GetTrackLen(iCD_Drive_Letter, i );
        Frame2Time( lLen, &iMin, &iSec, &iFrame );
        printf("  Lnge: %2d: %2d:%02d.%02d",i, iMin, iSec, iFrame );
        printf(" Typ :" );
        switch( TI.bTrackControlInfo & TCI_TRACK_MASK )
        {
          case TCI_DATA_TRACK: printf("Data "); break;
          case TCI_4AUDIO_CHANNELS: printf("Audio/Quadro "); break;
          default: printf("Audio/Stereo ");
        }
        if( TI.bTrackControlInfo & TCI_PRE_EMPHASIS )
            printf("Pre-Emphasis ");
        if( TI.bTrackControlInfo & TCI_DIGITAL_COPY_PROHIBITED )
            printf("Digital Copy prohibited ");
        printf("\n");
      }
    }
}

/**********************************************************************/
/* cd_FastForward : Aktuelle Musikausgabe 2 Sekunden vorlaufen lassen */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/**********************************************************************/
VOID cd_FastForward( INT iCD_Drive_Letter )
{ MSCDEX_AudStat AS;
  LONG           lHSG;

  cd_GetAudioStatusInfo( iCD_Drive_Letter, &AS );
  cd_StopAudio( iCD_Drive_Letter );
  lHSG = AS.lResumeStart + Time2Frame( 0, 2, 0 );
  cd_PlayAudio( iCD_Drive_Letter,
             REDBOOK,
             HSG2REDBOOK( lHSG ),
             REDBOOK2HSG( AS.lResumeEnd ) - lHSG );
}

/**********************************************************************/
/* cd_PrintActPlay : Zeige Spielzeiten und Titel das aktuell abge-    */
/*                   spielten Titel an.                               */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung( 0=A, etc )          */
/* Ausgabe : TRUE             - CD-Player noch mit Audio-Ausgabe be-  */
/*                              schftigt                             */
/*           FALSE            - CD-Player mit abspielen fertig        */
/**********************************************************************/
INT cd_PrintActPlay( INT iCD_Drive_Letter )
{ INT             iStat;
  MSCDEX_QInfo    QI;

  iStat = cd_QueryAudioChannel( iCD_Drive_Letter, &QI );
  printf(" Track: %01d%01d Index: %02d  ",
           ( INT ) ( ( QI.bTrackNo & 0xF0 ) >> 4 ),  
           ( INT ) ( QI.bTrackNo & 0x0F ), 
           ( INT ) QI.bIndex );
  printf(" Track: %02d:%02d.%02d", ( INT ) QI.bTrackMin,
                                   ( INT ) QI.bTrackSec,
                                   ( INT ) QI.bTrackFrame );
  printf(" Disk: %02d:%02d.%02d\r", ( INT ) QI.bDiskMin,
                                    ( INT ) QI.bDiskSec,
                                    ( INT ) QI.bDiskFrame );
  return ( iStat & DEV_BUSY ) ? TRUE : FALSE;
}

/**********************************************************************/
/* cd_PrintSektor : Stelle Inhalt eines Sektors in Zeichen / Hex dar. */
/**------------------------------------------------------------------**/
/* Eingabe : lpSektor - Adresse der Sektor-Daten                      */
/*           iSize    - Gre des Sektors                             */
/*           iCols    - Anzahl darzustellender Spalten                */
/*           iRows    - Anzahl anzuzeigender Zeilen, nach denen auf   */
/*                      Tastendruck gewartet werden soll.             */
/*                      0 - Nicht warten                              */
/**********************************************************************/
VOID cd_PrintSektor( LPVOID lpSektor, INT iSize, INT iCols, INT iRows )
{ LPBYTE lpS;
  INT i, j, k;

  lpS = lpSektor;
  k = 0;
  for( i = 0; i < iSize; )
  {
    for( j = 0; j < iCols; j++ )
      if( i + j < iSize )
         printf("%c",isprint( lpS[ i + j ] ) ? lpS[ i+ j ] : '.' );
      else printf(" ");

    printf("  ");
    for( j = 0; j < iCols; j++ )
      if( i + j < iSize )   printf("%02X ", lpS[ i + j ] );
      else printf("   ");

    printf("\n");
    i += iCols;

    if( iRows > 0 ) k++;
    if( ( k == iRows ) && ( i < iSize ) )
    {
      printf("< Taste >\r");
      _getch();
      k = 0;
    }
  }
}

/**********************************************************************/
/* cd_PrintDirEntry : Zeige Elemente einer DirEntry-Struktur an       */
/**------------------------------------------------------------------**/
/* Eingabe : lpDirEntry - Adresse der anzuzeigenden Struktur          */
/**********************************************************************/
VOID cd_PrintDirEntry( LPDIR_ENTRY lpDirEntry )
{
  printf(" XAR-Lnge                 : %d\n",
        ( INT )lpDirEntry->XAR_len );
  printf(" Start-LBN                 : %ld\n",
         lpDirEntry->loc_extent );
  printf(" LBN-Gre                 : %d\n",
         lpDirEntry->lb_size );
  printf(" Datei-Lnge               : %ld\n",
         lpDirEntry->data_len );
  /* printf(" Datum/Zeit                : ",
     lpDirEntry->record_time ); */
  printf(" Flags                     : %d\n",
         lpDirEntry->file_flags );
  printf(" Interleave-Size           : %d\n",
         (INT) lpDirEntry->il_size );
  printf(" Interleave-Skip           : %d\n",
         (INT) lpDirEntry->il_skip );
  printf(" Volume Set Sequence Number: %d\n",
         lpDirEntry->VSSN );
  printf(" Lnge des Dateinamen      : %d\n",
         lpDirEntry->len_fi );
  printf(" DateiName                 : %s\n",
         lpDirEntry->file_id );
  printf(" Version                   : %d.%02d\n",
         (lpDirEntry->file_version >> 8),
          lpDirEntry->file_version & 0x00FF );
  printf(" Systemdaten Lnge         : %d\n",
         (INT) lpDirEntry->len_su );
}

#endif

