/**********************************************************************/
/*                   C D D E V . H                                    */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Header-Datei fr CDDEV.C                         */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 08.04.1994                                       */
/*  letztes Update : 18.04.1994                                       */
/**********************************************************************/

#ifndef _CDDEV_H
#define _CDDEV_H

/*- Device Treiber Requests ------------------------------------------*/
/*- Device-Header ----------------------------------------------------*/
typedef struct tagDeviceHeader
{ LPVOID    lpNextHdr;        /* Adresse des nchsten Device-Treibers */
  WORD      wDevAttributes;                      /* Treiber-Attribute */
  WORD      wStrategy;                 /* Offset der Strategy-Routine */
  WORD      wInterrupt;               /* Offset der Interrupt-Routine */
  BYTE      bName[ 8 ];                          /* Name des Treibers */
  WORD      wReserved;
  BYTE      bDriveLetter;                       /* Laufwerksbuchstabe */
  BYTE      bNumberOfSubUnits;           /* Anzahl der Untereinheiten */
} DeviceHeader;
typedef DeviceHeader _FP *LPDeviceHeader;

/*- Request-Header ---------------------------------------------------*/
typedef struct tagRequestHeader
{ BYTE bLength;                         /* Lnge der Request-Struktur */
  BYTE bSubUnit;                           /* Nummer der Untereinheit */
  BYTE bCommand;                           /* auszufhrendes Kommando */
  WORD wStatus;                                       /* Fehlerstatus */
  BYTE bReserved[ 8 ];         /* werden je nach Kommando umdefiniert */
} RequestHeader;
typedef RequestHeader _FP *LPRequestHeader;

/*- RequestHeader.wStatus Bits : -------------------------------------*/
#define DEV_ERROR ( 1 << 15 )     /* Gesetzt, dann in Bits 0-7 Fehler */
#define DEV_BUSY  ( 1 << 9 )                /* Device ist beschftigt */
#define DEV_DONE  ( 1 << 8 )   /* Device hat letzte Operation beendet */

/*- Bei gesetztem Fehlerbit folgende Fehlermeldungen in Low-Byte -----*/
#define DEV_WRITE_PROTECT_VIOLATION       0
#define DEV_UNKNOWN_UNIT                  1
#define DEV_DRIVE_NOT_READY               2
#define DEV_UNKNOWN_COMMAND               3
#define DEV_CRC_ERROR                     4
#define DEV_BAD_DRIVE_REQUEST_LEN         5
#define DEV_SEEK_ERROR                    6
#define DEV_UNKNOWN_MEDIA                 7
#define DEV_SECTOR_NOT_FOUND              8
#define DEV_PRINTER_OUT_OF_PAPER          9
#define DEV_WRITE_FAULT                   10
#define DEV_READ_FAULT                    11
#define DEV_GENERAL_FAILURE               12
#define DEV_RESERVED1                     13
#define DEV_RESERVED2                     14
#define DEV_INVALID_DISK_CHANGE           15

/*- Device-Kommandos -------------------------------------------------*/
#define DEVCMD_INIT 0
  /* INIT-Kommando wird nur einmal whrend des Bootens aufgerufen,    */
  /* danach nicht mehr.                                               */
  typedef struct tagMSCDEX_Init
  { RequestHeader ReqHdr;
    BYTE          bNumberOfUnits;
    LPVOID        lpEndAdress;
    LPVOID        lpBPBArray;
    BYTE          bBlockDeviceNumber;
  } MSCDEX_Init;
  typedef MSCDEX_Init _FP *LPMSCDEX_Init;

#define DEVCMD_IOCTL_READ 3
  /* ber das IOCTL_READ-Kommando werden vielfltige Statusinfor-     */
  /* mationen ber Treiber, Laufwerk und eingelegte CD abgefragt.     */
  typedef struct tagMSCDEX_IOCTLIO
  { RequestHeader ReqHdr;
    BYTE          bMediaDescriptor;                        /* immer 0 */
    LPVOID        lpTransferAdress;   /* = Adresse des Control-Blocks */
    WORD          wTransferCount;       /* = Gre des Control-Blocks */
    WORD          wStartingSectorNo;                       /* immer 0 */
    LPVOID        lpVolumeID;                           /* immer NULL */
  } MSCDEX_IOCTLIO;
  typedef MSCDEX_IOCTLIO _FP *LPMSCDEX_IOCTLIO;

  #define IOCTLI_GET_DEVHDR_ADRESS 0
  /*- Liefert Adresse des DeviceHeaders im Speicher ------------------*/
    typedef struct tagMSCDEX_Raddr
    { BYTE           bControlBlockCode;                        /* = 0 */
      LPDeviceHeader lpDeviceHeaderAdress;    /* DeviceHeader-Adresse */
    } MSCDEX_Raddr;
    typedef MSCDEX_Raddr _FP *LPMSCDEX_Raddr;

  #define IOCTLI_GET_HEAD_LOCATION 1
  /*- Ermittelt (Schreib-/)Lesekopf-Position -------------------------*/
    typedef struct tagMSCDEX_LocHead
    { BYTE bControlBlockCode;                                  /* = 1 */
      BYTE bAdressingMode;              /* = HSG (0) oder REDBOOK (1) */
      LONG lLocation;  /* Position des (Schreib-) Lesekopfes (Sektor) */
    } MSCDEX_LocHead;
    typedef MSCDEX_LocHead _FP *LPMSCDEX_LocHead;

  #define IOCTLI_RESERVED1 2
  /*- Reserviertes Kommando ------------------------------------------*/

  #define IOCTLI_ERROR_STATISTICS 3
  /*- Ermittelt Fehlerstatistik (noch nicht spezifiziert). -----------*/
    typedef struct tagMSCDEX_ErrStat
    { BYTE bControlBlockCode;                                  /* = 3 */
      BYTE bErrStat[ 1 ];    /* Gre des Arrays noch nicht definiert */
    } MSCDEX_ErrStat;
    typedef MSCDEX_ErrStat _FP *LPMSCDEX_ErrStat;

  #define IOCTLI_GET_AUDIO_CHANNEL_INFO 4
  /*- Ermittelt, welche CD-Kanle auf welchem CD-Player Ausgang      -*/
  /*- liegen und wie laut diese sind.                                -*/
    typedef struct tagMSCDEX_AudInfo
    { BYTE bControlBlockCode;                                  /* = 4 */
      BYTE bOutput0, bVolume0; /* Welche Audio-Spuren werden an       */
      BYTE bOutput1, bVolume1; /* welchen CD-Ausgang umgeleitet, und  */
      BYTE bOutput2, bVolume2; /* wie laut sind die CD-Ausgnge?      */
      BYTE bOutput3, bVolume3;
    } MSCDEX_AudInfo;
    typedef MSCDEX_AudInfo _FP *LPMSCDEX_AudInfo;

  #define IOCTLI_READ_DRIVE_BYTES 5
  /*- Lese Daten direkt vom Laufwerk (Daten sind Hardwareabhngig)   -*/
    typedef struct tagMSCDEX_DrvBytes
    { BYTE bControlBlockCode;                                  /* = 5 */
      BYTE bNumReadBytes;               /* = Anzahl zu lesender Bytes */
      BYTE bReadBuffer[ 128 ];        /* In 128-Byte Blcken          */
    } MSCDEX_DrvBytes;                /* (vorherige werden verworfen) */
  typedef MSCDEX_DrvBytes _FP *LPMSCDEX_DrvBytes;

  #define IOCTLI_GET_DEV_STAT 6
  /*- Ermittle Laufwerkszustand --------------------------------------*/
    typedef struct tagMSCDEX_DevStat
    { BYTE bControlBlockCode;                                  /* = 6 */
      LONG lDeviceStatus; /* enthlt nach Aufruf den Laufwerks-Status */
    } MSCDEX_DevStat;
    typedef MSCDEX_DevStat _FP *LPMSCDEX_DevStat;

  #define DS_DOOR_OPEN                 0x00000001L
   /*- Lade geffnet -------------------------------------------------*/
  #define DS_DOOR_UNLOCKED             0x00000002L
   /*- Lade nicht veriegelt ------------------------------------------*/
  #define DS_READS_COOKED_AND_RAW      0x00000004L
   /*- Daten knnen Cooked und Raw gelesen werden, sonst nur Cooked --*/
  #define DS_READ_WRITE                0x00000008L
   /*- Lauwerk kann auch schreiben (z.B. WORM), sonst nur lesen ------*/
  #define DS_AUDIO_VIDEO               0x00000010L
   /*- Laufwerk untersttzt Musik- und Video CD's, sonst nur Daten ---*/
  #define DS_ISO_9600_INTERLEAVE       0x00000020L
  /* Bit 6 reserviert */
  #define DS_PREFETCH                  0x00000080L
   /*- vorrausschauendes Lesen mglich -------------------------------*/
  #define DS_MANIPULATE_AUDIO_CHANNEL  0x00000100L
   /*- Audio-Kanle knnen an CD-Ausgnge umgeleitet werden ----------*/
  #define DS_HSG_AND_REDBOOK           0x00000200L
   /*- Adressierungsmodi HSG und REDBOOK, sonst nur HSG --------------*/
  /* BIT 10 reserviert */
  #define DS_NO_DISC_IN_DRIVE          0x00000800L
   /*- Keine Disk im Laufwerk ----------------------------------------*/
  #define DS_SUPPORT_RW_SUB_CHANNELS   0x00001000L
   /*- Sub-Channel-Info kann gelesen werden --------------------------*/

  #define IOCTLI_SECTOR_SIZE 7
  /*- Ermittelt Sektor-Gre in Bytes --------------------------------*/
    typedef struct tagMSCDEX_SectSize
    { BYTE bControlBlockCode;                                  /* = 7 */
      BYTE bReadMode;                              /* COOKED oder RAW */
      WORD wSectorSize;           /* enthlt nach AUfruf Sektor-Gre */
    } MSCDEX_SectSize;            /* COOKED: 2048 oder RAW: 2352 Byte */
    typedef MSCDEX_SectSize _FP *LPMSCDEX_SectSize;

  #define IOCTLI_VOLUME_SIZE 8
  /*- Position des Lead-Out-Tracks (erster nach Ende der CD) ---------*/
    typedef struct tagMSCDEX_VolSize
    { BYTE bControlBlockCode;                                  /* = 8 */
      LONG lVolumeSize;/* nach Aufruf Position des letzten Tracks + 1 */
    } MSCDEX_VolSize;
    typedef MSCDEX_VolSize _FP *LPMSCDEX_VolSize;

  #define IOCTLI_MEDIA_CHANGED 9
  /*- Hat ein CD-Wechsel stattgefunden? ------------------------------*/
    typedef struct tagMSCDEX_MedChng
    { BYTE bControlBlockCode;                                  /* = 9 */
      BYTE bMedChng;  /* nach Aufruf: NOT_CHANGED, DONT_KNOW, CHANGED */
    } MSCDEX_MedChng;
    typedef MSCDEX_MedChng _FP *LPMSCDEX_MedChng;
    #define NOT_CHANGED 1
    #define DONT_KNOW   0
    #define CHANGED     0xFF

  #define IOCTLI_AUDIO_DISK_INFO 10
  /*- Ermittelt CD-Informationen */
    typedef struct tagMSCDEX_DiskInfo
    { BYTE bControlBlockCode;                                 /* = 10 */
      BYTE bLowestTrack;                              /* erster Track */
      BYTE bHighestTrack;                            /* Letzter Track */
      LONG bLeadOutTrack;                   /* Gre der CD (REDBOOK) */
    } MSCDEX_DiskInfo;
    typedef MSCDEX_DiskInfo _FP *LPMSCDEX_DiskInfo;

  #define IOCTLI_AUDIO_TRACK_INFO 11
  /*- Ermittelt Informationen ber einen bestimmten Track ------------*/
    typedef struct tagMSCDEX_TnoInfo
    { BYTE bControlBlockCode;                                 /* = 11 */
      BYTE bTrackNo;         /* = Nummer des zu untersuchenden Tracks */
      LONG lStartingPoint;               /* Adresse im REDBOOK Format */
      BYTE bTrackControlInfo;            /* Info ber Track           */
    } MSCDEX_TnoInfo;
    typedef MSCDEX_TnoInfo _FP *LPMSCDEX_TnoInfo;

    /*- Definitionen fr TrackControlInfo ----------------------------*/
    #define TCI_4AUDIO_CHANNELS         0x80
    #define TCI_DATA_TRACK              0x40
    #define TCI_TRACK_MASK              0xC0
    #define TCI_PRE_EMPHASIS            0x10
    #define TCI_DIGITAL_COPY_PROHIBITED 0x20

  #define IOCTLI_AUDIO_QUERY_CHANNEL 12
  /*- Ermittle aktuelle Track- und Disk-Spielzeit --------------------*/
    typedef struct tagMSCDEX_QInfo
    { BYTE bControlBlockCode;                                 /* = 12 */
      BYTE bCONTROLandADR;  /* TrackControl, Adressierung (s.TnoInfo) */
      BYTE bTrackNo;           /* Nummer des aktuellen Tracks (Lieds) */
      BYTE bIndex;       /* Aktueller Index innerhalb aktuellem Track */
      BYTE bTrackMin;                     /* Aktuelle Track-Spielzeit */
      BYTE bTrackSec;
      BYTE bTrackFrame;
      BYTE bZero;                                          /* immer 0 */
      BYTE bDiskMin;                        /* Gesamtspielzeit der CD */
      BYTE bDiskSec;
      BYTE bDiskFrame;
    } MSCDEX_QInfo;
    typedef MSCDEX_QInfo _FP *LPMSCDEX_QInfo;

  #define IOCTLI_AUDIO_SUB_CHANNEL_INFO 13
  /*- Ermittelt Sub-Channel-Info whrend des Abspielens --------------*/
    typedef struct tagMSCDEX_SubChanInfo
    { BYTE      bControlBlockCode;                            /* = 13 */
      LONG      lStartingSector;                 /* = REDBOOK-Adresse */
      LPVOID    lpTransferAdress;            /* = Adresse des Buffers */
      LONG      lNumberOfSectorsToRead;  /* = Anzahl zu lesender Sub- */
    } MSCDEX_SubChanInfo;                /* Channel-Informationen.    */
    typedef MSCDEX_SubChanInfo _FP *LPMSCDEX_SubChanInfo;

  #define IOCTLI_UPCODE 14
  /*- Ermittle UPC/EAN-Nummer der eingelegten CD. --------------------*/
    typedef struct tagMSCDEX_UPCode
    { BYTE bControlBlockCode;                                 /* = 14 */
      BYTE bCONTROLandADR;                              /* s. TnoInfo */
      BYTE bUPC_EAN[ 7 ];                     /* 13 Stellige BCD-Zahl */
      BYTE bZero;                      /* (letztes Nibble unbebnutzt) */
      BYTE bAFrame;
    } MSCDEX_UPCode;
    typedef MSCDEX_UPCode _FP *LPMSCDEX_UPCode;

  #define IOCTLI_AUDIO_STATUS_INFO 15
  /*- Ermittelt Audio-Status -----------------------------------------*/
    typedef struct tagMSCDEX_AudStat
    { BYTE bControlBlockCode;                                 /* = 15 */
      WORD wAudioStatus;      /* Bit 0 = Audio Pause, 1-15 reserviert */
      LONG lResumeStart; /* Startposition fr DEVCMD_RESUME (REDBOOK) */
      LONG lResumeEnd;     /* Endposition fr DEVCMD_RESUME (REDBOOK) */
    } MSCDEX_AudStat;
    typedef MSCDEX_AudStat _FP *LPMSCDEX_AudStat;

#define DEVCMD_INPUTFLUSH 7
/*- INPUTFLUSH verwendet Standard-RequestHeader ----------------------*/
#define DEVCMD_OUTPUTFLUSH 11
/*- OUTPUTFLUSH verwendet Standard-RequestHeader ---------------------*/

#define DEVCMD_IOCTL_WRITE 12
  /* ber das WRITE-Kommando werden einige Laufwerks-Parameter ver-   */
  /* ndert. Dazu wird der MSCDEX_IOCTLIO-Block verwendet (s.o.)      */
  #define IOCTLO_EJECT_DISK 0
  /*- ffnet CD-Lade -------------------------------------------------*/
    typedef struct tagMSCDEX_Eject
    { BYTE bControlBlockCode;                                  /* = 0 */
    } MSCDEX_Eject;
    typedef MSCDEX_Eject _FP *LPMSCDEX_Eject;

  #define IOCTLO_LOCK_DOOR 1
  /*- Verriegelt Tr -------------------------------------------------*/
    typedef struct tagMSCDEX_LockDoor
    { BYTE bControlBlockCode;                                  /* = 1 */
      BYTE bLock;              /* = 1: Verriegeln, 0: ffnen der Lade */
    } MSCDEX_LockDoor;
    typedef MSCDEX_LockDoor _FP *LPMSCDEX_LockDoor;

  #define IOCTLO_RESET_DRIVE 2
  /*- Zurcksetzen des Laufwerks -------------------------------------*/
    typedef struct tagMSCDEX_Reset
    { BYTE bControlBlockCode;                                  /* = 2 */
    } MSCDEX_Reset;
    typedef MSCDEX_Reset _FP *LPMSCDEX_Reset;

  #define IOCTLO_SET_AUDIO_CHANNEL_INFO 3
  /*- Gegenstck zu IOCTLI_GET_AUDIO_CHANNEL_INFO --------------------*/
  #define IOCTLO_WRITE_DRIVE_BYTES 4
  /*- Gegenstck zu IOCTLI_READ_DRIVE_BYTES --------------------------*/
  #define IOCTLO_CLOSE_TRAY 5
    typedef struct tagMSCDEX_CloseTray
    { BYTE bControlBlockCode;                                  /* = 5 */
    } MSCDEX_CloseTray;
    typedef MSCDEX_CloseTray _FP *LPMSCDEX_CloseTray;

#define DEVCMD_DEVICE_OPEN 13
/*- Device ffnen ----------------------------------------------------*/
#define DEVCMD_DEVICE_CLOSE 14
/*- Device Schlieen -------------------------------------------------*/
#define DEVCMD_READLONG 128
/*- Liest Daten von einer CD -----------------------------------------*/

  /*- Folgende Struktur wird auch beim Schreiben von Daten verwendet -*/
  typedef struct tagMSCDEX_ReadWriteL
  { RequestHeader ReqHdr;
    BYTE          bAdressingMode;         /* = HSG(0) oder REDBOOK(1) */
    LPVOID        lpTransferAdress;          /* = Adresse des Buffers */
    WORD          wNumSec; /* = Anzahl der zu bertragenden  Sektoren */
    LONG          lStartingSector;         /* = (Schreib)Leseposition */
    BYTE          bDataReadWriteMode;      /* = betragungsmodus s.u. */
    BYTE          bInterleaveSize; /* = Anzahl zus.hngender Sektoren */
    BYTE          bInterleaveSkipFactor;         /* = Gre der Lcke */
  } MSCDEX_ReadWriteL;
  typedef MSCDEX_ReadWriteL _FP *LPMSCDEX_ReadWriteL;

  /*- bDataReadWriteMode-Definitionen: -------------------------------*/
  /* Beim Lesen werden nur RAW und COOKED verwendet:                  */
  #define COOKED 0
  #define RAW    1

  /* Beim Schreiben kommen folgende Konstanten zum Tragen:            */
  #define MODE0 0       /* Lschen der Sektoren (berschreiben mit 0) */
  #define MODE1 1   /* Beschreiben der Sektoren mit angegebenen Daten */
  #define MODE2FORM1 2                           /* COOKED Data-Write */
  #define MODE2FORM2 3                              /* RAW Data-Write */

#define DEVCMD_READLONG_PREFETCH 130
/*- bermittelt den nchsten zu lesenden Sektor an Treiber, soda     */
/*- dieser vorrausschauend den Sektor "Cachen" kann.                  */

#define DEVCMD_SEEK 131
/*- Setzt Position des (Schreib)Lesekopfes auf gesamter CD -----------*/
  typedef struct MSCDEX_SeekReq
  { RequestHeader ReqHdr;
    BYTE          bAdressingMode;         /* = HSG(0) oder REDBOOK(1) */
    LPVOID        lpTransferAdress;/* wird ignoriert daher = 0 setzen */
    WORD          wNumSec;         /* wird ignoriert daher = 0 setzen */
    LONG          lStartingSector;         /* = anzufahrende Position */
  } MSCDEX_SeekReq;
   typedef MSCDEX_SeekReq _FP *LPMSCDEX_SeekReq;

#define DEVCMD_PLAY_AUDIO 132
/* Audio-Track abspielen ---------------------------------------------*/
  typedef struct tagMSCDEX_PlayReq
  { RequestHeader ReqHdr;
    BYTE          bAdressingMode;         /* = HSG(0) oder REDBOOK(1) */
    LONG          lStartSector;      /* = erster zu spielender Sektor */
    LONG          lSectorCount;    /* = Anzahl zu spielender Sektoren */
  } MSCDEX_PlayReq;
  typedef MSCDEX_PlayReq _FP *LPMSCDEX_PlayReq;

#define DEVCMD_STOP_AUDIO 133
/*- Audio-Ausgabe stoppen (verwendet Standard RequestHeader) ---------*/
#define DEVCMD_WRITELONG 134
/*- Daten schreiben (s. READ_LONG) -----------------------------------*/
#define DEVCMD_WRITELONG_VERIFY 135
/*- geschriebene Daten mit denen im Speicher vergleichen -------------*/
#define DEVCMD_RESUME_AUDIO 136
/*- Durch einmaliges STOP_AUDIO unterbrochene Musik weiterspielen ----*/

typedef struct HSG_DIR_ENTRY
{
    BYTE  len_dr;                            /* Lnge dieser Struktur */
    BYTE  XAR_len;       /* XAR-Lnge in LBN (logische Block Nummern) */
    ULONG loc_extentI;                  /* Start-LBN's (Intel Format) */
    ULONG loc_extentM;               /* Start-LBN's (Molorola Format) */
    ULONG data_lenI;                     /* Dateilnge (Intel Format) */
    ULONG data_lenM;                  /* Dateilnge (Motorola Format) */
    BYTE  record_time[6];                           /* Datum und Zeit */
    BYTE  file_flags_hsg;                                    /* Flags */
    BYTE  reserved;                                     /* reserviert */
    BYTE  il_size;   /* Interleave: Anzahl zusammenhngender Sektoren */
    BYTE  il_skip;                     /* Interleave: Gre der Lcke */
    WORD  VSSNI;         /* Volume Set Sequence Number (Intel Format) */
    WORD  VSSNM;      /* Volume Set Sequence Number (Motorola Format) */
    WORD  len_fi;                                  /* Lnge des Namen */
/*  BYTE  file_id[ ... ];Filename, variable Lnge (max. 32 Zeichen)   */
/*  BYTE  padding; Pad-Byte (damit System-Daten Word-aligned sind     */
/*  BYTE  sys_data[ ... ];          System Daten (variable Lnge)     */
} HSG_DIR_ENTRY;
typedef HSG_DIR_ENTRY _FP *LPHSG_DIR_ENTRY;

typedef struct ISO_DIR_ENTRY
{
    BYTE  len_dr;                            /* Lnge dieser Struktur */
    BYTE  XAR_len;       /* XAR-Lnge in LBN (logische Block Nummern) */
    ULONG loc_extentI;                    /* Start-LBN (Intel Format) */
    ULONG loc_extentM;                 /* Start-LBN (Motorola Format) */
    ULONG data_lenI;                     /* Dateilnge (Intel Format) */
    ULONG data_lenM;                  /* Dateilnge (Motorola Format) */
    BYTE  record_time[7];                           /* Datum und Zeit */
    BYTE  file_flags_iso;                                    /* Flags */
    BYTE  il_size;   /* Interleave: Anzahl zusammenhngender Sektoren */
    BYTE  il_skip;                     /* Interleave: Gre der Lcke */
    WORD  VSSNI;         /* Volume Set Sequence Number (Intel Format) */
    WORD  VSSNM;      /* Volume Set Sequence Number (Motorola Format) */
    BYTE  len_fi;                                  /* Lnge des Namen */
/*  BYTE  file_id[ ... ];Filename, variable Lnge (max. 32 Zeichen)   */
/*  BYTE  padding; Pad-Byte (damit System-Daten Word-aligned sind     */
/*  BYTE  sys_data[ ... ];          System Daten (variable Lnge)     */
} ISO_DIR_ENTRY;
typedef ISO_DIR_ENTRY _FP *LPISO_DIR_ENTRY;

typedef struct DIR_ENTRY
{
    BYTE  XAR_len;       /* XAR-Lnge in LBN (logische Block Nummern) */
    ULONG loc_extent;                                    /* Start-LBN */
    WORD  lb_size;       /* Gre eines (LBN). Wenn LBN-Gre = 2048, */
                         /* dann entspricht ein LBN einem physischen  */
                         /* Sektor/Frame.                             */
    ULONG data_len;                             /* Dateilnge (BYTES) */
    BYTE  record_time[7];                           /* Datum und Zeit */
    BYTE  file_flags;                                        /* Flags */
    BYTE  il_size;   /* Interleave: Anzahl zusammenhngender Sektoren */
    BYTE  il_skip;                     /* Interleave: Gre der Lcke */
    WORD  VSSN;                         /* Volume Set Sequence Number */
    BYTE  len_fi;                            /* Lnge des Dateinamens */
    BYTE  file_id[38];           /* Dateiname (mit '\0' abgschlossen) */
    WORD  file_version;                               /* File-Version */
    BYTE  len_su;                            /* Lnge der Systemdaten */
    BYTE  su_data[220];                                /* Systemdaten */
} DIR_ENTRY;
typedef DIR_ENTRY _FP *LPDIR_ENTRY;
/*- Prototypen -------------------------------------------------------*/

INT _CallStrategy( INT iCD_Drive_Letter, LPRequestHeader lpReqHdr );
INT cd_IsError( WORD wStatus );
INT cd_GetReqHdrError( LPRequestHeader lpReqHdr );
INT cd_IsReqHdrBusy( LPRequestHeader lpReqHdr );
INT cd_IsReqHdrDone( LPRequestHeader lpReqHdr );
INT cd_GetDeviceHeaderAdress( INT            iCD_Drive_Letter,
                              LPMSCDEX_Raddr lpRA );
INT cd_GetLocationOfHead( INT iCD_Drive_Letter, LPMSCDEX_LocHead lpLH );
INT cd_GetAudioChannelInfo( INT              iCD_Drive_Letter,
                            LPMSCDEX_AudInfo lpAI );
INT cd_ReadDriveBytes( INT iCD_Drive_Letter, LPMSCDEX_DrvBytes lpDB );
INT cd_GetDevStat( INT iCD_Drive_Letter, LPMSCDEX_DevStat lpDS );
VOID cd_PrintDevStat( LPMSCDEX_DevStat lpDS );
INT cd_GetSectorSize( INT               iCD_Drive_Letter,
                      LPMSCDEX_SectSize lpSEC,
                      INT               iReadMode );
INT cd_GetVolumeSize( INT iCD_Drive_Letter, LPMSCDEX_VolSize lpVS );
INT cd_GetMediaChanged( INT iCD_Drive_Letter, LPMSCDEX_MedChng lpMC );
INT cd_GetAudioDiskInfo( INT iCD_Drive_Letter, LPMSCDEX_DiskInfo lpDI );
INT cd_GetAudioTrackInfo( INT              iCD_Drive_Letter,
                          INT              iTrackNo,
                          LPMSCDEX_TnoInfo lpTI );
INT cd_IsDataTrack( LPMSCDEX_TnoInfo lpTI );
LONG cd_GetTrackLen( INT iCD_Drive_Letter, INT iTno );
INT cd_QueryAudioChannel( INT iCD_Drive_Letter, LPMSCDEX_QInfo lpQI );
INT cd_GetAudioSubChannel( INT                  iCD_Drive_Letter,
                           LPMSCDEX_SubChanInfo lpSCI );
INT cd_GetUPCode( INT iCD_Drive_Letter, LPMSCDEX_UPCode lpUPC );
VOID cd_PrintUPCode( LPMSCDEX_UPCode lpUPC );
INT cd_GetAudioStatusInfo(INT iCD_Drive_Letter, LPMSCDEX_AudStat lpAS );
INT cd_Eject( INT iCD_Drive_Letter );
INT cd_LockDoor( INT iCD_Drive_Letter, INT iLock  );
INT cd_ResetDrive( INT iCD_Drive_Letter );
INT cd_SetAudioChannelInfo( INT              iCD_Drive_Letter,
                            LPMSCDEX_AudInfo lpAI );
INT cd_WriteDriveBytes( INT iCD_Drive_Letter, LPMSCDEX_DrvBytes lpDB );
INT cd_CloseTray( INT iCD_Drive_Letter );
INT cd_PlayAudio( INT  iCD_Drive_Letter,
                  INT  iAdressMode,
                  LONG lStart,
                  LONG lLen );
INT cd_StopAudio( INT iCD_Drive_Letter );
INT cd_ResumeAudio(  INT iCD_Drive_Letter );
INT cd_Seek( INT iCD_Drive_Letter, INT iAdressingMode, LONG lSector );
INT cd_ReadLong( INT       iCD_Drive_Letter,
                 INT       iAdressMode,
                 LONG      lStart,
                 INT       iNumSec,
                 LPVOID    lpReadData,
                 INT       iDataMode );
INT cd_ReadLongPrefetch( INT  iCD_Drive_Letter,
                         INT  iAdressMode,
                         LONG lStart );
INT cd_WriteLong( INT       iCD_Drive_Letter,
                  INT       iAdressMode,
                  LONG      lStart,
                  INT       iNumSec,
                  LPVOID    lpWriteData,
                  INT       iDataMode );
INT cd_WriteLongVerify( INT       iCD_Drive_Letter,
                        INT       iAdressMode,
                        LONG      lStart,
                        INT       iNumSec,
                        LPVOID    lpVerifyData,
                        INT       iDataMode );
VOID cd_PrintDiskTracks( INT iCD_Drive_Letter );
VOID cd_FastForward( INT iCD_Drive_Letter );
INT cd_PrintActPlay( INT iCD_Drive_Letter );
VOID cd_PrintSektor( LPVOID lpSektor, INT iSize, INT iCols, INT iRows );
VOID cd_PrintDirEntry( LPDIR_ENTRY lpDirEntry );

#endif


