/**********************************************************************/
/*                   C D R O M . C                                    */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Demo Programm fr MSCDEX-Programmierung          */
/*                   Verwendet CDUTIL                                 */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 08.04.1994                                       */
/*  letztes Update : 18.04.1994                                       */
/*                                                                    */
/* Achtung: Struct-Member Alignment mu auf 1 Byte gesetzt werden, da */
/*          MSCDEX nicht auf logischen Strukturen, sondern auf        */
/*          Keine Optimierungen!                                      */
/*          physischen BYTE-Offsets operiert!                         */
/*          Bentigt : cdutil.c/h, cddev.c/h, args.c/h, types.h       */
/* COMPILER: Borland C++ 3.1, Microsoft Visual C++  1.5               */
/**********************************************************************/
#include <dos.h>
#include <stdio.h>
#include <ctype.h>
#include <conio.h>
#include <stdlib.h>
#include <process.h>

#include "args.h"
#include "cddev.h"
#include "cdutil.h"

/*- Folgende Zeilen innerhalb eines Projektes entfernen: -------------*/
#include "cdutil.c"
#include "cddev.c"
#include "args.c"

/**********************************************************************/
/*--  H A U P T P R O G R A M M                                     --*/
/**********************************************************************/
VOID main( INT argc, PCHAR argv[] )
{ MSCDEX_DevStat  DS;
  MSCDEX_MedChng  MC;
  MSCDEX_DiskInfo DI;
  MSCDEX_TnoInfo  TI;
  MSCDEX_UPCode   UPC;

  INT i, j;
  INT iVersion;
  INT iDrive = 0;
  INT iNumTrack = 5;
  LONG lSektor;
  LPCHAR lpFileName;

  if( argc == 1 )
  {
    printf("Parameter:\n");
    printf("A-Z:            Laufwerksbuchstabe\n");
    printf(" -DEVSTAT       zeigt Device-Status\n");
    printf(" -VTOC          zeigt Volume Table Of Contents\n");
    printf(" -UPC           zeigt Universal Product Code\n");
    printf(" -CONTENTS      zeigt Tracks\n");
    printf(" -SHOW:X        zeigt Sektor Nummer X");
    printf(" [mit -RAW als Raw-Data]\n");
    printf(" -PLAY:X        spielt Audio-Track Nummer X");
    printf(" [mit -WAIT werden Status-Infos angezeigt]\n");
    printf(" -ENTRY:name    zeigt Directory-Entry der");
    printf(" angegebenen Datei\n");
    exit(0);
  }

  iDrive = -1;
  for( i = 1; i < argc; i++ )
    for( j = 0; j < 26; j++ )
      if( ( toupper( argv[ i ][ 0 ] ) == ( char ) ( j + 'A' ) ) &&
          ( argv[ i ][ 1 ] == ':' ) &&
          ( argv[ i ][ 2 ] == '\0' ) ) iDrive = j;

  if( iDrive < 0 )
  {
    printf("Ungltige Laufwerksangabe!\n");
    exit( 0 );
  }

  if( !MSCDEX_Installed() )
  {
    printf("Konnte MSCDEX nicht finden!");
    exit( 0 );
  }

  iVersion = MSCDEX200_GetVersion();
  printf("MSCDEX V%d.%02d entdeckt\n", ( INT )HIBYTE( iVersion ),
                                       ( INT )LOBYTE( iVersion ) );

  if( iVersion > 0x200 )
  if( !MSCDEX200_CDRomDriveCheck( iDrive ) )
  {
    printf("Angegebenes Laufwerk ist kein CD-Rom Laufwerk!");
    exit( 0 );
  }

  /*- Dafr sorgen das evtl. CD-Wechsel erkannt werden ---------------*/
  cd_GetMediaChanged( iDrive, &MC );
  cd_GetDevStat( iDrive, &DS );
  if( GetArg( argc, argv, "-DEVSTAT", _none, NULL, 0 ) )
      cd_PrintDevStat( &DS );
  else
  if( DS.lDeviceStatus & DS_NO_DISC_IN_DRIVE )
  {
    printf("Keine CD im Laufwerk!\n");
    exit( 0 );
  }

  /*- Display Volume Table Of Contents -------------------------------*/
  if( GetArg( argc, argv, "-VTOC", _none, NULL, 0 ) )
  { INT i, iResult;
    BYTE Sektor[ 2048 ];
    i = 0;
    do
    {
      iResult = MSCDEX_ReadVTOC( iDrive, Sektor, i );
      if( iResult ) cd_PrintSektor( Sektor, 2048, 16, 24 );
      i++;
    } while( iResult );
  }

  /*- Universal Product Code anzeigen --------------------------------*/
  if( GetArg( argc, argv, "-UPC", _none, NULL, 0 ) )
  {
    cd_GetUPCode( iDrive, &UPC );
    cd_PrintUPCode( &UPC );
  }

  /*- Inhaltsverzeichnis (Titel) aznzeigen ---------------------------*/
  if( GetArg( argc, argv, "-CONTENTS", _none, NULL, 0 ) )
    cd_PrintDiskTracks( iDrive );


  /*- Sektorinhalt anzeigen ------------------------------------------*/
  lSektor = 0L;
  if( GetArg( argc, argv, "-SHOW:", _long, &lSektor, 1 ) )
  { char cSektor[ 2352 ];                        /* Auch RAW erlauben */
    INT iSize;
    iSize = 2048;
    if( GetArg( argc, argv, "-RAW", _none, NULL, 1 ) ) iSize = 2352;
              /*- Achtung! Audio-CD's werden nicht gelesen Copyight! -*/
    if( !cd_IsError( cd_ReadLong( iDrive, HSG, lSektor, 1, cSektor,
                            ( iSize == 2048 ) ? COOKED : RAW ) ) )
      cd_PrintSektor( cSektor, iSize, 16, 24 );
    else printf("Kann Sektor nicht lesen! (Audio CD?)\n");
  }

  /*- Verzeichniseintrag anzeigen ------------------------------------*/
  if( GetArg( argc, argv, "-ENTRY:", _string, &lpFileName, 1 ) )
  { DIR_ENTRY DirEntry;
    INT iIsHsg;

    if( MSCDEX200_GetDirectoryEntry( iDrive, STRUCT_COPY,
                                      lpFileName, &DirEntry, &iIsHsg ) )
        cd_PrintDirEntry( &DirEntry );
    else printf("Error: %d\n", _doserrno );
  }

  /*- Titel abspielen ------------------------------------------------*/
  iNumTrack = 0;
  if( GetArg( argc, argv, "-PLAY:", _int, &iNumTrack, 1 ) )
  {
    if( !( DS.lDeviceStatus & DS_AUDIO_VIDEO ) )
    {
      printf("Laufwerk untersttzt keine Audio-Wiedergabe!\n");
      exit( 0 );
    }
    if( DS.lDeviceStatus & DS_DOOR_OPEN ) cd_CloseTray( iDrive );
    cd_GetAudioDiskInfo( iDrive, &DI );
    if( ( iNumTrack < DI.bLowestTrack ) ||
        ( iNumTrack > DI.bHighestTrack ) )
    {
      printf("ungltige Liednummer\n");
      exit(0);
    }
    cd_GetAudioTrackInfo( iDrive, iNumTrack, &TI );
    if( cd_IsDataTrack( &TI ) )
    {
      printf("Kein Audio Track!\n");
      exit(0);
    }
    cd_StopAudio( iDrive );
    if( !cd_IsError ( cd_PlayAudio( iDrive, REDBOOK, TI.lStartingPoint,
                               cd_GetTrackLen( iDrive, iNumTrack ) ) ) )
    {
      if( GetArg( argc, argv, "-WAIT", _none, NULL, 0 )  )
      {
          while( cd_PrintActPlay( iDrive ) );
          printf("\n");
      }
      printf("Play - OK\n");
    }
  }
}


