/**********************************************************************/
/*                   C D U T I L . C                                  */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Utils fr MSCDEX-Programmierung                  */
/*                   Helper, Wrapper, Interfaces                      */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 08.04.1994                                       */
/*  letztes Update : 18.04.1994                                       */
/*                                                                    */
/* Achtung: Struct-Member Alignment mu auf 1 Byte gesetzt werden, da */
/*          MSCDEX nicht auf logischen Strukturen, sondern auf        */
/*          physischen BYTE-Offsets operiert!                         */
/*          Bentigt : args.c, args.h, types.h                        */
/**********************************************************************/
#ifndef __CDUTIL_C             /* CDUTIL.C kann auch #Included werden */
#define __CDUTIL_C

#include <dos.h>
#include <process.h>
#include <stdlib.h>

#include "types.h"
#include "cddev.h"
#include "cdutil.h"

/**********************************************************************/
/* Time2Frame : Wandelt Zeit in Anzahl der Frames (75 Frames = 1 sec) */
/**------------------------------------------------------------------**/
/* Eingabe : iMin   - Minute ( 0 - 59+ )                              */
/*           iSec   - Sekunde ( 0 - 59 )                              */
/*           iFrame - Frame ( 0 - 75 )                                */
/* Ausgabe : Anzahl der Frames                                        */
/**********************************************************************/
LONG Time2Frame( INT iMin, INT iSec, INT iFrame )
{
  return iMin * 60L * 75L + iSec * 75L + iFrame;
}

/**********************************************************************/
/* Time2HSG : Wandelt Zeit in HSG-Adresse                             */
/**------------------------------------------------------------------**/
/* Eingabe : iMin   - Minute ( 0 - 59+ )                              */
/*           iSec   - Sekunde ( 0 - 59 )                              */
/*           iFrame - Frame ( 0 - 75 )                                */
/* Ausgabe : Gewandelte HSG-Adresse                                   */
/**********************************************************************/
LONG Time2HSG( INT iMin, INT iSec, INT iFrame )
{
  return Time2Frame( iMin, iSec, iFrame ) - 150L;
}

/**********************************************************************/
/* Time2REDBOOK : Wandelt Zeit in REDBOOK-Adresse                     */
/**------------------------------------------------------------------**/
/* Eingabe : iMin   - Minute ( 0 - 59+ )                              */
/*           iSec   - Sekunde ( 0 - 59 )                              */
/*           iFrame - Frame ( 0 - 75 )                                */
/* Ausgabe : Gewandelte REDBOOK-Adresse                               */
/**********************************************************************/
LONG Time2REDBOOK( INT iMin, INT iSec, INT iFrame )
{
  return ( ( ( LONG ) iMin   << 16 ) |
           ( ( LONG ) iSec   <<  8 ) |
           ( ( LONG ) iFrame <<  0 ) );
}

/**********************************************************************/
/* FrameTime : Splittet Anzahl von Frames (Sektoren) in Minute,       */
/*             Sekunde, Frame auf.                                    */
/**------------------------------------------------------------------**/
/* Eingabe : lFrames  - zu wandelnde Anzahl von Frames (Sektoren)     */
/*           lpiMin   - Adresse einer INT-Variablen zur Aufnahme der  */
/*                      Minuten                                       */
/*           lpiSec   - dito fr Sekunden                             */
/*           lpiFrame - dito fr Frames                               */
/**********************************************************************/
VOID Frame2Time( LONG lFrame, LPINT lpiMin,
                              LPINT lpiSec,
                              LPINT lpiFrame )
{
  *lpiMin = ( INT ) ( lFrame / ( 60L * 75L ) );
  lFrame -= *lpiMin * 60L * 75L;
  *lpiSec = ( INT ) ( lFrame / 75L );
  lFrame -= *lpiSec * 75L;
  *lpiFrame = ( INT ) lFrame;
}

/**********************************************************************/
/* REDBOOK2Time : Splittet REDBOOK-Adresse in Minute, Sekunde, Frame  */
/*                auf.                                                */
/**------------------------------------------------------------------**/
/* Eingabe : lREDBOOK - zu wandelnde REDBOOK-Adresse                  */
/*           lpiMin   - Adresse einer INT-Variablen zur Aufnahme der  */
/*                      Minuten                                       */
/*           lpiSec   - dito fr Sekunden                             */
/*           lpiFrame - dito fr Frames                               */
/**********************************************************************/
VOID REDBOOK2Time( LONG lREDBOOK, LPINT lpiMin,
                                  LPINT lpiSec,
                                  LPINT lpiFrame )
{
  *lpiFrame = ( INT ) ( lREDBOOK >>  0 ) & 0x00FF;
  *lpiSec   = ( INT ) ( lREDBOOK >>  8 ) & 0x00FF;
  *lpiMin   = ( INT ) ( lREDBOOK >> 16 ) & 0x00FF;
}

/**********************************************************************/
/* HSG2Time : Splittet HSG-Adresse in Minute, Sekunde, Frame auf.     */
/**------------------------------------------------------------------**/
/* Eingabe : lHSG     - zu wandelnde HSG-Adresse                      */
/*           lpiMin   - Adresse einer INT-Variablen zur Aufnahme der  */
/*                      Minuten                                       */
/*           lpiSec   - dito fr Sekunden                             */
/*           lpiFrame - dito fr Frames                               */
/**********************************************************************/
VOID HSG2Time( LONG lHSG, LPINT lpiMin,
                          LPINT lpiSec,
                          LPINT lpiFrame )
{
  Frame2Time( lHSG + 150, lpiMin, lpiSec, lpiFrame );
}

/**********************************************************************/
/* REDBOOK2HSG : Wandle REDBOOK-Adresse in HSG-Adresse                */
/**------------------------------------------------------------------**/
/* Eingabe : lREDBOOK - zu wandelnde REDBOOK-Adresse                  */
/* Ausgabe : gewandelte HSG-Adresse                                   */
/**********************************************************************/
LONG REDBOOK2HSG( LONG lREDBOOK )
{ INT iMin, iSec, iFrame;
  REDBOOK2Time( lREDBOOK, &iMin, &iSec, &iFrame );
  return Time2HSG( iMin, iSec, iFrame );
}

/**********************************************************************/
/* HSG2REDBOOK : Wandle HSG-Adresse in REDBOOK-Adresse                */
/**------------------------------------------------------------------**/
/* Eingabe : lHSG - zu wandelnde HSG-Adresse                          */
/* Ausgabe : gewandelte REDBOOK-Adresse                               */
/**********************************************************************/
LONG HSG2REDBOOK( LONG lHSG )
{ INT iMin, iSec, iFrame;
  HSG2Time( lHSG, &iMin, &iSec, &iFrame );
  return Time2REDBOOK( iMin, iSec, iFrame );
}

/*- MSCDEX-Interface Funktionen --------------------------------------*/

/**********************************************************************/
/* MSCDEX_GetNumberOfDriveLetters : Ermittle Anzahl der untersttzen  */
/*                                  CD-Rom Laufwerke.                 */
/**------------------------------------------------------------------**/
/* Eingabe : lpiNumber      - Adresse eines INT, das Anzahl der Lauf- */
/*                            werke aufnehmen soll (Ausgabeparameter) */
/*           lpiStartLetter - ASCII-Code des ersten Laufwerksbuch-    */
/*                            stabens der von einem CD-ROM Laufwerk   */
/*                            verwendet wird.                         */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**------------------------------------------------------------------**/
/* Info :  Diese Funktion kann verwendet werden, um herauszufinden    */
/*         ob berhaupt CD-Rom Laufwerke untersttzt werden bzw. ob   */
/*         MSCDEX installiert wurde. Vor dem Aufruf wird BX auf 0     */
/*         gesetzt. Ist dieses Register nach dem Aurfuf immer noch    */
/*         gleich null, so wird kein CD-Rom Laufwerk untersttzt.     */
/**********************************************************************/
INT MSCDEX_GetNumberOfDriveLetters( LPINT lpiNumber,
                                    LPINT lpiStartLetter )
{ union _REGS regs;                            /* AX = 0x1500, BX = 0 */
  regs.x.ax = MSCDEX_MUXCODE( MSCDEX_GET_NO_OF_DRIVE_LETTERS );
  regs.x.bx = 0;
  _int86( DOS_MULTIPLEX_INT, &regs, &regs );
  if( !regs.x.cflag )                         /* Carry-Flag gesetzt ? */
  {
    *lpiNumber = regs.x.bx;
    *lpiStartLetter = regs.x.cx;
    return TRUE;
  }
  *lpiNumber = 0;
  *lpiStartLetter = 0;
  return FALSE;
}

/**********************************************************************/
/* MSCDEX_Installed : Ist MSCDEX berhaupt installiert ?              */
/**------------------------------------------------------------------**/
/* Info :  Diese Funktion testet, ob die Anzahl der untersttzen      */
/*         Laufwerke von 0 verschieden ist.                           */
/**********************************************************************/
INT MSCDEX_Installed( VOID )
{ INT iNumber, iStartLetter;
  iNumber = 0;
  MSCDEX_GetNumberOfDriveLetters( &iNumber,
                                  &iStartLetter );
  return ( iNumber == 0 ) ? FALSE : TRUE;
}

/**********************************************************************/
/* MSCDEX_GetCDRomDriveDeviceList : Ermittle Informationen ber       */
/*                                  angeschlossene CD-Rom Laufwerke.  */
/**------------------------------------------------------------------**/
/* Eingabe : lpDevElement - Adresse eines DevElement-Arrays, das die  */
/*                          Informationen ALLER angeschlossener Lauf- */
/*                          werke aufnehmen soll.                     */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**------------------------------------------------------------------**/
/* Info :  Ein DevElement enthlt nach dem Aufruf dieser Funktion die */
/*         Adresse des Device-Headers zu der angegebenen SubUnit. Das */
/*         dieser Funktion bergebene Array mu Platz fr alle ange-  */
/*         schlossenen (bzw. untersttzten) CD-Rom Laufwerke ent-     */
/*         halten. Die sicherste Methode besteht darin, Platz fr 26  */
/*         Elemente (Laufwerksbuchstaben von A: bis Z:) zur Verfgung */
/*         zu stellen und auf die ersten Elemente zuzugreifen.        */
/**********************************************************************/
INT MSCDEX_GetCDRomDriveDeviceList( LPDevElement lpDevElement )
{ union _REGS regs;                                    /* AX = 0x1501 */
  struct _SREGS sregs;                 /* ES:BX = Adresse des Buffers */
  regs.x.ax = MSCDEX_MUXCODE( MSCDEX_GET_DRIVE_DEVICE_LIST );
  sregs.es  = FP_SEG( lpDevElement );
  regs.x.bx = FP_OFF( lpDevElement );
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  return ( regs.x.cflag == 0 ) ? TRUE : FALSE;
}

/**********************************************************************/
/* MSCDEX_GetCopyrightFilename : Ermittle Namen des Copyright-Files   */
/*                               einer CD.                            */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung (0=A, 1=B, 2=C etc.) */
/*           lpName           - Adresse eines 38 Zeichen groen       */
/*                              Buffers zur Aufnahme des Namens.      */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**------------------------------------------------------------------**/
/* Info: Obwohl 38 Bytes zur Aufnahme des Filenamens zur Verfgung    */
/*       stehen, werden nur die ersten 11 (8.3) Zeichen verwendet.    */
/**********************************************************************/
INT MSCDEX_GetCopyrightFilename( INT       iCD_Drive_Letter,
                                 LPCHAR    lpName )
{ union _REGS regs;           /* AX = 0x1502, CX = Laufwerksbuchstabe */
  struct _SREGS sregs;                  /* ES:BX = Adresse des Namens */
  regs.x.ax = MSCDEX_MUXCODE( MSCDEX_GET_COPYRIGHT_FILENAME );
  sregs.es  = FP_SEG( lpName );
  regs.x.bx = FP_OFF( lpName );
  regs.x.cx = iCD_Drive_Letter;
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  return ( regs.x.cflag == 0 ) ? TRUE : FALSE;
}

/**********************************************************************/
/* MSCDEX_GetAbstractFilename : Ermittle Namen des Abstract-Files     */
/*                              einer CD.                             */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung (0=A, 1=B, 2=C etc.) */
/*           lpName           - Adresse eines 38 Zeichen groen       */
/*                              Buffers zur Aufnahme des Namens.      */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**------------------------------------------------------------------**/
/* Info: Obwohl 38 Bytes zur Aufnahme des Filenamens zur Verfgung    */
/*       stehen, werden nur die ersten 11 (8.3) Zeichen verwendet.    */
/**********************************************************************/
INT MSCDEX_GetAbstractFilename( INT       iCD_Drive_Letter,
                                LPCHAR    lpName )
{ union _REGS regs;           /* AX = 0x1503, CX = Laufwerksbuchstabe */
  struct _SREGS sregs;                  /* ES:BX = Adresse des Namens */
  regs.x.ax = MSCDEX_MUXCODE( MSCDEX_GET_ABSTRACT_FILENAME );
  sregs.es  = FP_SEG( lpName );
  regs.x.bx = FP_OFF( lpName );
  regs.x.cx = iCD_Drive_Letter;
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  return ( regs.x.cflag == 0 ) ? TRUE : FALSE;
}

/**********************************************************************/
/* MSCDEX_GetBibDocFilename : Ermittle Namen der bibliographischen    */
/*                            Dokumentationsdatei einer CD.           */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung (0=A, 1=B, 2=C etc.) */
/*           lpName           - Adresse eines 38 Zeichen groen       */
/*                              Buffers zur Aufnahme des Namens.      */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**------------------------------------------------------------------**/
/* Info: Obwohl 38 Bytes zur Aufnahme des Filenamens zur Verfgung    */
/*       stehen, werden nur die ersten 11 (8.3) Zeichen verwendet.    */
/**********************************************************************/
INT MSCDEX_GetBibDocFilename( INT       iCD_Drive_Letter,
                              LPCHAR    lpName )
{ union _REGS regs;           /* AX = 0x1504, CX = Laufwerksbuchstabe */
  struct _SREGS sregs;                  /* ES:BX = Adresse des Namens */
  regs.x.ax = MSCDEX_MUXCODE( MSCDEX_GET_BIB_DOC_FILENAME );
  sregs.es  = FP_SEG( lpName );
  regs.x.bx = FP_OFF( lpName );
  regs.x.cx = iCD_Drive_Letter;
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  return ( regs.x.cflag == 0 ) ? TRUE : FALSE;
}

/**********************************************************************/
/* MSCDEX_ReadVTOC : Lese "Volume Table of Contents" (Inhalts-        */
/*                   verzeichnis).                                    */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung (0=A, 1=B, 2=C etc.) */
/*           lpSector         - Adresse eines 2048 Zeichen groen     */
/*                              Buffers zur Aufnahme eines Sektors.   */
/*           iNumSec          - Nummer des zu lesenden TOC-Sektors.   */
/* Ausgabe : <> 0 - Operation erfolgreich                             */
/*                  0x0101 - Gelesener Sektor = standard VTOC-Sektor  */
/*                  0x01FF - Gelesener Sektor = letzter VTOC-Sektor   */
/*                  0x0100 - Gelesener Sektor = anderer VTOC-Sektor   */
/*           0    - Fehler (s. _dosernno )                            */
/**********************************************************************/
INT MSCDEX_ReadVTOC( INT       iCD_Drive_Letter,
                     LPBYTE    lpSector,
                     INT       iNumSec )
{ union _REGS regs;           /* AX = 0x1505, CX = Laufwerksbuchstabe */
  struct _SREGS sregs;                  /* ES:BX = Adresse des Namens */
  regs.x.ax = MSCDEX_MUXCODE( MSCDEX_READ_VTOC );
  sregs.es  = FP_SEG( lpSector );
  regs.x.bx = FP_OFF( lpSector );
  regs.x.cx = iCD_Drive_Letter;
  regs.x.dx = iNumSec;
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  return ( regs.x.cflag == 0 ) ? ( 0x0100 | regs.h.al ) : 0;
}

/**********************************************************************/
/* MSCDEX_DebugOn : Debugging ermglichen.                            */
/*                  (nur fr MSCDEX-Debug-Version)                    */
/**------------------------------------------------------------------**/
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**********************************************************************/
INT MSCDEX_DebugOn( VOID )
{ union _REGS regs;                                    /* AX = 0x1506 */
  regs.x.ax = MSCDEX_MUXCODE( MSCDEX_DEBUG_ON );
  _int86( DOS_MULTIPLEX_INT, &regs, &regs );
  return ( regs.x.cflag == 0 ) ? TRUE : FALSE;
}

/**********************************************************************/
/* MSCDEX_DebugOf : Debugging abschalten.                             */
/*                  (nur fr MSCDEX-Debug-Version)                    */
/**------------------------------------------------------------------**/
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**********************************************************************/
INT MSCDEX_DebugOff( VOID )
{ union _REGS regs;                                    /* AX = 0x1507 */
  regs.x.ax = MSCDEX_MUXCODE( MSCDEX_DEBUG_OFF );
  _int86( DOS_MULTIPLEX_INT, &regs, &regs );
  return ( regs.x.cflag == 0 );
}

/**********************************************************************/
/* MSCDEX_AbsoluteRead : Lese Daten aus Sektor(en)                    */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung (0=A, 1=B, 2=C etc.) */
/*           lpSector         - Adresse der 2048 Zeichen groen       */
/*                              Buffer zur Aufnahme der Sektoren.     */
/*           iSecCnt          - Anzahl der zu lesenden Sektoren.      */
/*           lSecStart        - Nummer des ersten zu lesenden Sektors */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**------------------------------------------------------------------**/
/* Info: Fr jeden zu lesenden Sektor mu ein 2048-Zeichen groer     */
/*       Buffer bereitgestellt werden.                                */
/**********************************************************************/
INT MSCDEX_AbsoluteRead( INT       iCD_Drive_Letter,
                         LPBYTE    lpSector,
                         INT       iSecCnt,
                         LONG      lSecStart )
{ union _REGS regs;         /* AX = 0x1508, CX = Laufwerkstbuchstabe  */
  struct _SREGS sregs;      /* DX = Anzahl zu lesender Sektoren       */
                            /* ES:BX = Adresse des Lesebuffers        */
                            /* SI:DI = Adresse des ersten Sektors     */

  regs.x.ax = MSCDEX_MUXCODE( MSCDEX_ABSOLUTE_DISK_READ );
  sregs.es  = FP_SEG( lpSector );
  regs.x.bx = FP_OFF( lpSector );
  regs.x.cx = iCD_Drive_Letter;
  regs.x.dx = iSecCnt;
  regs.x.si = HIWORD( lSecStart );
  regs.x.di = LOWORD( lSecStart );
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  return ( regs.x.cflag == 0 ) ? TRUE  : FALSE;
}

/**********************************************************************/
/* MSCDEX_AbsoluteWrite: Schreibe Daten in Sektor(en)                 */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung (0=A, 1=B, 2=C etc.) */
/*           lpSector  - Adresse der 2048 Zeichen groen Buffer,die   */
/*                       die Daten enthalten.                         */
/*           iSecCnt   - Anzahl der zu schreibenden Sektoren.         */
/*           lSecStart - Nummer des ersten zu schreibenden Sektors    */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**------------------------------------------------------------------**/
/* Info: Fr jeden Sektor mu ein 2048-Zeichen groer Buffer bereit-  */
/*       gestellt werden.                                             */
/**********************************************************************/
INT MSCDEX_AbsoluteWrite( INT       iCD_Drive_Letter,
                          LPBYTE    lpSector,
                          INT       iSecCnt,
                          LONG      lSecStart )
{ union _REGS regs;         /* AX = 0x1509, CX = Laufwerkstbuchstabe  */
  struct _SREGS sregs;      /* DX = Anzahl zu schreibender Sektoren   */
                            /* ES:BX = Adresse des Schreibbuffers     */
                            /* SI:DI = Adresse des ersten Sektors     */

  regs.x.ax = MSCDEX_MUXCODE( MSCDEX_ABSOLUTE_DISK_WRITE );
  sregs.es  = FP_SEG( lpSector );
  regs.x.bx = FP_OFF( lpSector );
  regs.x.cx = iCD_Drive_Letter;
  regs.x.dx = iSecCnt;
  regs.x.si = HIWORD( lSecStart );
  regs.x.di = LOWORD( lSecStart );
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  return ( regs.x.cflag == 0 ) ? TRUE : FALSE;
}

/**********************************************************************/
/* MSCDEX200_CDRomDriveCheck : Ist Laufwerk ein CD-Rom Laufwerk?      */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung (0=A, 1=B, 2=C etc.) */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**********************************************************************/
INT MSCDEX200_CDRomDriveCheck( INT iCD_Drive_Letter )
{ union _REGS regs;           /* AX = 0x150B, CX = Laufwerksbuchstabe */

  regs.x.ax = MSCDEX_MUXCODE( MSCDEX200_CD_ROM_DRIVE_CHECK );
  regs.x.cx = iCD_Drive_Letter;
  regs.x.bx = 0;
  _int86( DOS_MULTIPLEX_INT, &regs, &regs );
  return ( ( regs.x.bx == 0xADAD ) && ( regs.x.ax != 0 ) ) ?
    TRUE : FALSE;
}

/**********************************************************************/
/* MSCDEX200_GetVersion : Ermittle MSCDEX-Version                     */
/**------------------------------------------------------------------**/
/* Ausgabe : Versionsnummer (Major.Minor)                             */
/**------------------------------------------------------------------**/
/* Info : Diese Funktion existiert erst seit MSCDEX V2.00! Kleinere   */
/*        Versionsnummern werden als 1.00 angegeben!                  */
/**********************************************************************/
INT MSCDEX200_GetVersion( VOID )
{ union _REGS regs;                                    /* AX = 0x150C */
  regs.x.ax = MSCDEX_MUXCODE( MSCDEX200_GET_VERSION );
  regs.x.bx = 0;                               /* BX erst mal lschen */
  _int86( DOS_MULTIPLEX_INT, &regs, &regs );
  if( ( regs.x.bx ) && ( regs.x.cflag == 0 ) ) return regs.x.bx;
  return ( regs.x.cflag == 0 ) ? 0x0100 : -1;    /* V1.00 oder Fehler */
}

/**********************************************************************/
/* MSCDEX200_GetCDromDriveLetters : Ermittle von ALLEN CD-Laufwerken  */
/*                                  verwendete Laufwerksbuchstaben.   */
/**------------------------------------------------------------------**/
/* Eingabe : lpLetters - Adress des Zeichen-Arrays, das Laufwerksbuch-*/
/*                       staben aufnehmen soll.                       */
/*                       (s. GetNumberOfDriveLetters )                */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**********************************************************************/
INT MSCDEX200_GetCDRomDriveLetters( LPCHAR lpLetters )
{ union _REGS regs;                                    /* AX = 0x150D */
  struct _SREGS sregs;                  /* ES:BX = Adresse des Arrays */

  regs.x.ax = MSCDEX_MUXCODE( MSCDEX200_GET_DRIVE_LETTERS );
  sregs.es  = FP_SEG( lpLetters );
  regs.x.bx = FP_OFF( lpLetters );
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  return ( regs.x.cflag == 0 ) ? TRUE : FALSE;
}

/**********************************************************************/
/* MSCDEX200_GetVDPreference : Ermittle Volume Descriptor Preference  */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung (0=A, 1=B, 2=C etc.) */
/*           lpiPreference    - Adresse des INT's, in das Preference  */
/*                              gespeichert werden soll.              */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**********************************************************************/
INT MSCDEX200_GetVDPreference( INT   iCD_Drive_Letter,
                               LPINT lpiPreference )
{ union _REGS regs;                                    /* AX = 0x150D */
  struct _SREGS sregs;                  /* ES:BX = Adresse des Arrays */

  regs.x.ax = MSCDEX_MUXCODE( MSCDEX200_GETSET_VD_PREFERENCE );
  regs.x.cx = iCD_Drive_Letter;
  regs.x.bx = 0x0000;                           /* 0 = Get Preference */
  regs.x.dx = 0;              /* Ergebnisregister vorsorglich lschen */
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  *lpiPreference = regs.x.dx;
  return ( regs.x.cflag == 0 ) ? TRUE : FALSE;
}

/**********************************************************************/
/* MSCDEX200_SetVDPreference : Setze Volume Descriptor Preference     */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung (0=A, 1=B, 2=C etc.) */
/*           iPreference      - Neue Preference                       */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**********************************************************************/
INT MSCDEX200_SetVDPreference( INT iCD_Drive_Letter,
                               INT iPreference )
{ union _REGS regs;                                    /* AX = 0x150D */
  struct _SREGS sregs;                  /* ES:BX = Adresse des Arrays */

  regs.x.ax = MSCDEX_MUXCODE( MSCDEX200_GETSET_VD_PREFERENCE );
  regs.x.cx = iCD_Drive_Letter;
  regs.x.bx = 0x0001;                           /* 1 = Set Preference */
  regs.x.dx = iPreference;
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  return ( regs.x.cflag == 0 ) ? TRUE : FALSE;
}

/**********************************************************************/
/* MSCDEX200_GetDirectoryEntry : Ermittle Datei-Informationen         */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Laufwerkskennung (0=A, 1=B, 2=C etc.) */
/*           iCopyFlag        - 0 - direct copy, 1 - copy to struct   */
/*           lpPathName       - Pfadname der Datei                    */
/*           lpData           - Adresse, an die Daten kopiert werden  */
/*                              sollen                                */
/*           lpiHSG           - Directory Daten HSG(<>0) oder ISO(=0) */
/* Ausgabe : TRUE  - Operation erfolgreich                            */
/*           FALSE - Operation fehlgeschlagen ( Fehler in _doserrno ) */
/**------------------------------------------------------------------**/
/* Info : Der bereitgestellte Buffer sollte mindestens 255 Bytes      */
/*        (copy to struct) oder 280 Bytes (direct copy) Gro sein.    */
/*        Bei direct copy ist der Programmierer fr die Dekodierung   */
/*        der Strukturdaten verantwortlich. Um welchen der beiden     */
/*        Typen (HSG_ENTRY oder ISO_ENTRY) es sich handelt, wird in   */
/*        der Int-Variablen vermerkt auf die lpiHSG verweist.         */
/**********************************************************************/
INT MSCDEX200_GetDirectoryEntry( INT       iCD_Drive_Letter,
                                 INT       iCopyFlag,
                                 LPCHAR    lpPathName,
                                 LPVOID    lpData,
                                 LPINT     lpiHSG )
{ union _REGS regs;    /* AX = 0x150F, CL = CD_Drive, CH = copy flags */
  struct _SREGS sregs;       /* ES:BX = Pfadname, SI:DI = Datenadress */

  regs.x.ax = MSCDEX_MUXCODE( MSCDEX200_GET_DIRECTORY_ENTRY );
  regs.h.cl = ( BYTE )iCD_Drive_Letter;
                                              /* Bit 0 setzen/lschen */
  regs.h.ch = ( BYTE ) ( iCopyFlag ? 0x01 : 0x00 );
  sregs.es  = FP_SEG( lpPathName );
  regs.x.bx = FP_OFF( lpPathName );
  regs.x.si = FP_SEG( lpData );
  regs.x.di = FP_OFF( lpData );
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  *lpiHSG = regs.x.ax;
  return ( regs.x.cflag == 0 ) ? TRUE : FALSE;
}

/**********************************************************************/
/* MSCDEX_ReadWriteReq : Fhre MSCDEX-IOCTLI_READ/WRITE-Request aus   */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter  - Kennung des Laufwerks, an dessen     */
/*                               Treiber ein Request abgeschickt      */
/*                               werden soll.                         */
/*           iCommand          - Nummer des auszufhrenden Kommandos  */
/*           lpControlBlock    - Adresse des Controlblocks, der READ/ */
/*                               WRITE-Request nher bestimmt.        */
/*           iControlBlockSize - Gre des ControlBlocks              */
/* Ausgabe : Device-Status                                            */
/**------------------------------------------------------------------**/
/* Info    : MSCDEX fllt das SubUnit-Feld bei Aufruf eines Treibers, */
/*           der mehrere Laufwerke untersttzt, automatisch aus. Wird */
/*           ein Treiber-Kommando nicht mit Hilfe von MSCDEX ab-      */
/*           gesetzt, mu das SubUnit Feld vom Programmierer gesetzt  */
/*           werden (s. _CallStrategy() )                             */
/**********************************************************************/
INT MSCDEX_ReadWriteReq( INT       iCD_Drive_Letter,
                         INT       iCommand,
                         LPVOID    lpControlBlock,
                         INT       iControlBlockSize )
{ MSCDEX_IOCTLIO IOCTLIO;
  IOCTLIO.ReqHdr.bLength    = sizeof( RequestHeader );
  IOCTLIO.ReqHdr.bSubUnit   = 0;
  IOCTLIO.ReqHdr.bCommand   = ( BYTE )iCommand;

  IOCTLIO.bMediaDescriptor  = 0;
  IOCTLIO.lpTransferAdress  = lpControlBlock;
  IOCTLIO.wTransferCount    = iControlBlockSize;
  IOCTLIO.wStartingSectorNo = 0;
  IOCTLIO.lpVolumeID        = NULL;

  return MSCDEX_SendDeviceRequest( iCD_Drive_Letter,
                                      &IOCTLIO.ReqHdr );
}

/**********************************************************************/
/* MSCDEX210_SendDeviceRequest: Sende Device-Request an Device-       */
/*                              Treiber                               */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Nummer des Laufwerks, an dessen       */
/*                              Device-Treiber ein Device-Request ge- */
/*                              sendet werden soll. (0=A, 1=B, etc.)  */
/*           lpReqHdr         - Adresse des Device-Request-Headers    */
/* Ausgabe : Status des Device nach Operation                         */
/*           (s. RequestHeader.wStatus)                               */
/**------------------------------------------------------------------**/
/* Info: Durch diesen Aufruf ruft MSCDEX die Strategy-Routine des     */
/*       Device-Treibers auf.                                         */
/**********************************************************************/
INT MSCDEX210_SendDeviceRequest( INT             iCD_Drive_Letter,
                                 LPRequestHeader lpReqHdr )
{ union _REGS regs;           /* AX = 0x1510, CX = Laufwerksbuchstabe */
  struct _SREGS sregs;         /* ES:BX = Adresse des Request-Headers */

  regs.x.ax = MSCDEX_MUXCODE( MSCDEX210_SEND_DEVICE_REQUEST );
  regs.x.cx = iCD_Drive_Letter;
  sregs.es  = FP_SEG( lpReqHdr );
  regs.x.bx = FP_OFF( lpReqHdr );
  _int86x( DOS_MULTIPLEX_INT, &regs, &regs, &sregs );
  return lpReqHdr->wStatus;
}

/**********************************************************************/
/* MSCDEX_SendDeviceRequest: Sende Device-Request an Device-          */
/*                           Treiber                                  */
/**------------------------------------------------------------------**/
/* Eingabe : iCD_Drive_Letter - Nummer des Laufwerks, an dessen       */
/*                              Device-Treiber ein Device-Request ge- */
/*                              sendet werden soll. (0=A, 1=B, etc.)  */
/*           lpReqHdr         - Adresse des Device-Request-Headers    */
/* Ausgabe : Status des Device nach Operation                         */
/*           (s. RequestHeader.wStatus)                               */
/**------------------------------------------------------------------**/
/* Info: Dise Funktion ruft die Interrupt- und Strategy-Routinen ent- */
/*       weder ber den MSCDEX-MUX auf, oder ruft _CallStrategy auf.  */
/*       Auf welchem Weg die Device-Treiber funktionen aufgerufen     */
/*       werden ist abhngig von der MSCDEX-Version.                  */
/**********************************************************************/
INT MSCDEX_SendDeviceRequest( INT             iCD_Drive_Letter,
                              LPRequestHeader lpReqHdr )
{ static INT iVersion = 0x0;

  if( iVersion == 0 )   /* Falls Version noch nicht bekannt, ermitteln */
      iVersion = MSCDEX200_GetVersion();

  if( iVersion < 0x210 )   /* Bei Versionen kleiner 2.10 Direktaufruf */
      return _CallStrategy( iCD_Drive_Letter, lpReqHdr );
  return MSCDEX210_SendDeviceRequest( iCD_Drive_Letter, lpReqHdr );
}

#endif

