/**********************************************************************/
/*                   C T T S . C                                      */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Interfacefunktionen fr Zugriff auf Creative-    */
/*                   Treiber "CTTS.DRV" (Text to Speach)              */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 20.03.1994                                       */
/*  letztes Update : 25.03.1994                                       */
/**********************************************************************/
#ifndef __CTTS_C                      /* CTTS.C kann #Included werden */
#define __CTTS_C

/*- Include Dateien einbinden ----------------------------------------*/

#include <dos.h>
#include <stdlib.h>

#include "ctts.h"

/**********************************************************************/
/* ctts_GetDrvVer : Ermittle Treiber-Versionsnummer                   */
/**------------------------------------------------------------------**/
/* Eingabe : lpEntry - Adresse des Treibereinsprungs                  */
/* Ausgabe : Versionsnummer - HiByte / LoByte                         */
/*           0xFFFF - kein Treibereinsprung                           */
/**********************************************************************/
WORD ctts_GetDrvVer( LPVOID lpEntry )
{ WORD cret;
  if( lpEntry )
  { _asm mov BX, CTTS_DRIVERVERSION
    _asm call lpEntry
    _asm mov cret, AX
    return cret;
  }
  return 0xFFFF;
}

/**********************************************************************/
/* ctts_GetEnvSettings : Treiber mit Umgebungsvariable versorgen      */
/**------------------------------------------------------------------**/
/* Eingabe : lpEntry - Adresse des Treibereinsprungs                  */
/* Ausgabe : 0 - alles Ok                                             */
/*           1 - BLASTER-String = NULL                                */
/*           2 - BLASTER-String fehlerhaft                            */
/*           0xFFFF - kein Treibereinsprung                           */
/**********************************************************************/
WORD ctts_GetEnvSettings( LPVOID lpEntry )
{ WORD cax, ces, cdi;
  LPCHAR lpBlaster;
  if( lpEntry )
  {
    lpBlaster = getenv( "BLASTER" );
    ces = FP_SEG( lpBlaster );      /* Adresse des "Blaster"-Strings */
    cdi = FP_OFF( lpBlaster );      /* bergeben                     */
    _asm mov ES, ces
    _asm mov DI, cdi
    _asm mov BX, CTTS_SETSETTINGS
    _asm call lpEntry
    _asm mov cax, AX
    return cax;
  }
  return 0xFFFF;
}

/**********************************************************************/
/* ctts_Init : Treiber initialisieren                                 */
/**------------------------------------------------------------------**/
/* Eingabe : lpEntry - Adresse des Treibereinsprungs                  */
/* Ausgabe : 0 - alles Ok                                             */
/*           <> 0 - fehlerhafte Initialisierung                       */
/*           0xFFFF - kein Treibereinsprung                           */
/**------------------------------------------------------------------**/
/* Info : - 'cttsInit' darf erst nach 'cttsSetEnvSettings' aufgerufen */
/*          werden. cttsInit versucht den SBTALKER-Driver - der       */
/*          vorher gestartet worden sein mu - zu initialisieren.     */
/**********************************************************************/
WORD ctts_Init( LPVOID lpEntry )
{ WORD cret;
  if( lpEntry )
  {
    _asm mov BX, CTTS_INIT
    _asm call lpEntry
    _asm mov cret, AX
    return cret;
  }
  return 0xFFFF;
}

/**********************************************************************/
/* ctts_SetSpeechParam : Sprach-Parameter festlegen                   */
/**------------------------------------------------------------------**/
/* Eingabe : lpEntry - Adresse des Treibereinsprungs                  */
/*           bGender - Geschlecht ( 0 = mnnlich, 1 = weiblich )      */
/*           bTone   - 0 = Bass, 1 = Hhen                            */
/*           bVolume - Lautstrke ( 0 - 9 )                           */
/*           bPitch  - Tonhhe ( 0 - 9 )                              */
/*           bSpeed  - Geschwindigkeit ( 0 - 9 )                      */
/**********************************************************************/
VOID ctts_SetSpeechParam( LPVOID lpEntry,
                          BYTE bGender,
                          BYTE bTone,
                          BYTE bVolume,
                          BYTE bPitch,
                          BYTE bSpeed )
{
  if( lpEntry )
  {
    _asm mov BX, CTTS_SETSPEECHPARAM
    _asm mov AL, bGender
    _asm mov AH, bTone
    _asm mov DL, bVolume
    _asm mov DH, bPitch
    _asm mov CL, bSpeed
    _asm call lpEntry
  }
}

/**********************************************************************/
/* ctts_Terminate : Treiber deinstallieren                            */
/**------------------------------------------------------------------**/
/* Eingabe : lpEntry - Adresse des Treibereinsprungs                  */
/**------------------------------------------------------------------**/
/* Info : - Diese Funktion mu vor 'sb_UnloadDriver' aufgerufen       */
/*          werden                                                    */
/**********************************************************************/
VOID ctts_Terminate( LPVOID lpEntry )
{
  if( lpEntry )
  {
    _asm mov BX, CTTS_TERMINATE
    _asm call lpEntry
  }
}

/**********************************************************************/
/* ctts_Say : Text aufsagen                                           */
/**------------------------------------------------------------------**/
/* Eingabe : lpEntry - Adresse des Treibereinsprungs                  */
/*           lpString - Auszugebender Text (darf keine Umlaute        */
/*                      enthalten )                                   */
/* Ausgabe: 0 - alles OK                                              */
/*          1 - Ausgabestring leer oder NULL                          */
/*          2 - Ausgabestring zu lang                                 */
/*          0xFFFF - kein Treibereinsprung                            */
/**********************************************************************/
WORD ctts_Say( LPVOID lpEntry, LPCHAR lpString)
{ WORD cret, ces, cdi;
  if( lpEntry )
  {
    ces = FP_SEG( lpString );
    cdi = FP_OFF( lpString );
    _asm mov ES, ces
    _asm mov DI, cdi
    _asm mov BX, CTTS_SAY;
    _asm call lpEntry
    _asm mov cret, AX
    return cret;
  }
  return 0xFFFF;
}
#endif

