/**********************************************************************/
/*                   D M A U T I L  . C                               */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Stellt Funktionen zur Programmierung des DMA-    */
/*                   Controllers zur Verfgung.                       */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 20.03.1994                                       */
/*  letztes Update : 25.03.1994                                       */
/**********************************************************************/
#ifndef __DMAUTIL_C           /* DMAUTIL.C kann auch #Included werden */
#define __DMAUTIL_C

/*-- Include-Dateien einbinden ---------------------------------------*/

#include <dos.h>
#include <conio.h>
#include <stdlib.h>

#include "dmautil.h"

/*- Globale Variablen ------------------------------------------------*/
/*- Arrays werden fr Zugriff auf DMA-Controller nach Kanal verwendet */
INT dma_adress   [8] = {0x00,0x02,0x04,0x06,0xC0,0xC4,0xC8,0xCC};
INT dma_count    [8] = {0x01,0x03,0x05,0x07,0xC2,0xC6,0xCA,0xCE};
INT dma_page     [8] = {0x87,0x83,0x81,0x82,0x88,0x8B,0x89,0x8A};

INT dma_status   [2] = {0x08,0xD0};         /* Statusregister [Lesen] */
INT dma_command  [2] = {0x08,0xD0};    /* Befehlsregister [Schreiben] */
INT dma_request  [2] = {0x09,0xD2};           /* DMA-Request auslsen */
INT dma_chmask   [2] = {0x0A,0xD4};       /* Kanle einzeln maskieren */
INT dma_mode     [2] = {0x0B,0xD6};                  /* Transfermodus */
INT dma_flipflop [2] = {0x0C,0xD8};        /* Adress-/Zhler-Flipflop */
INT dma_masterclr[2] = {0x0D,0xDA};            /* Controller resetten */
INT dma_temp     [2] = {0x0D,0xDA};            /* Temporres Register */
INT dma_maskclr  [2] = {0x0E,0xDC};          /* Alle Kanle freigeben */
INT dma_mask     [2] = {0x0F,0xDE};        /* Kanle gesamt maskieren */

/**********************************************************************/
/* dma_Masterclear : zu einem Kanal gehrigen Controller  resetten    */
/**------------------------------------------------------------------**/
/* Eingabe : iChan : Nummer des Kanals (0-7)                          */
/**********************************************************************/
VOID dma_MasterClear( INT iChan )
{ iChan &= 0x0007;
  outp( dma_masterclr[ iChan / 4 ], 0 );
}

/**********************************************************************/
/* dma_SetRequest : bertragung auf angegebenem Kanal auslsen        */
/**------------------------------------------------------------------**/
/* Eingabe : iChan : Nummer des Kanals (0-7)                          */
/**********************************************************************/
VOID dma_SetRequest( INT iChan )
{ iChan &= 0x0007;
  outp( dma_request[ iChan / 4 ], REQUEST_SET | ( iChan & 0x03 ) );
}

/**********************************************************************/
/* dma_ClrRequest : bertragung auf angegebenem Kanal stoppen         */
/**------------------------------------------------------------------**/
/* Eingabe : iChan : Nummer des Kanals (0-7)                          */
/**********************************************************************/
VOID dma_ClrRequest( INT iChan )
{ iChan &= 0x0007;
  outp( dma_request[ iChan / 4 ], REQUEST_CLR | ( iChan & 0x03 ) );
}

/**********************************************************************/
/* dma_SetMask : Angegebenen Kanal maskieren (sperren)                */
/**------------------------------------------------------------------**/
/* Eingabe : iChan : Nummer des Kanals (0-7)                          */
/**********************************************************************/
VOID dma_SetMask( INT iChan )
{ iChan &= 0x0007;
  outp( dma_chmask[ iChan / 4 ], CHANNEL_SET | ( iChan & 0x03 ) );
}

/**********************************************************************/
/* dma_ClrMask : Angegebenen Kanal freigeben                          */
/**------------------------------------------------------------------**/
/* Eingabe : iChan : Nummer des Kanals (0-7)                          */
/**********************************************************************/
VOID dma_ClrMask( INT iChan )
{ iChan &= 0x0007;
  outp( dma_chmask[ iChan / 4 ], CHANNEL_CLR | ( iChan & 0x03 ) );
}

/**********************************************************************/
/* dma_ReadStatus : Status des zum angegebeben Kanal zugehrigen      */
/*                  Controllers lesen                                 */
/**------------------------------------------------------------------**/
/* Eingabe : iChan : Nummer des Kanals (0-7)                          */
/* Ausgabe : Status des Controllers                                   */
/**********************************************************************/
BYTE dma_ReadStatus( INT iChan )
{ iChan &= 0x0007;
  return ( BYTE )inp( dma_status[ iChan / 4 ] );
}

/**********************************************************************/
/* dma_ClrFlipFlop : FlipFlop des zum angegebeben Kanal zugehrigen   */
/*                   Controllers zurcksetzen                         */
/**------------------------------------------------------------------**/
/* Eingabe : iChan : Nummer des Kanals (0-7)                          */
/**------------------------------------------------------------------**/
/* Info : Das FlipFlop dient zur Unterscheidung von LO- und HI-Byte   */
/*        der Transferadresse bzw. des Transferzhlers.               */
/**********************************************************************/
VOID dma_ClrFlipFlop( INT iChan )
{ iChan &= 0x0007;
  outp( dma_flipflop[ iChan / 4 ], 0 );
}

/**********************************************************************/
/* dma_ReadCount : Transferzhler des angegebenen Kanals lesen        */
/**------------------------------------------------------------------**/
/* Eingabe : iChan : Nummer des Kanals (0-7)                          */
/* Ausgabe : Aktueller Transferzhler (0-65535)                       */
/**********************************************************************/
WORD dma_ReadCount( INT iChan )
{ BYTE l, h;
  iChan &= 0x0007;

  dma_ClrFlipFlop( iChan );
  l = ( BYTE )inp( dma_count[ iChan ] );
  h = ( BYTE )inp( dma_count[ iChan ] );
  return MAKEWORD( h, l );
}

/**********************************************************************/
/* dma_SetChannel : DMA-Kanal zum Transfer vorbereiten                */
/**------------------------------------------------------------------**/
/* Eingabe : iChan  : Nummer des Kanals (0-7)                         */
/*           lpMem  : Adresse des Speichers                           */
/*           uSize  : Anzahl zu transferierender Bytes                */
/*           bMode  : bertragungsmodus                               */
/*           bAlign : DMA_BIT8 oder DMA_BIT16 bertragung             */
/**------------------------------------------------------------------**/
/* Info : Um 16-Bit bertragungen zu ermglichen, mu die dem DMA-    */
/*        Kontroller bergebene lineare Adresse mit 2 multipliziert   */
/*        werden. Bei 16-Bit bertragungen knnen bis 128K Daten      */
/*        transferiert werden.                                        */
/**********************************************************************/
VOID dma_SetChannel( INT    iChan, LPVOID lpMem, WORD  uSize,
                     BYTE   bMode )
{ WORD  uAdress;
  BYTE  bPage;

  iChan &= 0x0007;                               /* Max. 8 DMA-Kanle */

  dma_SetMask( iChan );                             /*- Kanal sperren */
                       /* DMA transferiert 1 Byte mehr als angegeben! */
  uSize = uSize ? uSize - 1 : 0;    /* mind. 1 Byte (0==1) bertragen */

  /*- lineare 20-Bit Adresse erzeugen --------------------------------*/
  if( iChan <= 3 )                                        /* 8Bit DMA */
  {                     /* Adresse = untere 16 Bit der 20-Bit Adresse */
    uAdress = ( WORD ) ((((( LONG ) lpMem ) & 0xFFFF0000L ) >> 12L ) +
                         ((( LONG ) lpMem ) & 0xFFFFL ));
                            /* Seite = obere 4 Bit der 20-Bit Adresse */
    bPage = ( BYTE) (((((( LONG ) lpMem ) & 0xFFFF0000L ) >> 12L ) +
                         ((( LONG ) lpMem ) & 0xFFFFL )) >> 16);
  }
  else                                                  /* 16-Bit DMA */
  {                     /* Adresse = untere 16 Bit der 20-Bit Adresse */
    uAdress = ( WORD ) ((((( LONG ) lpMem ) & 0xFFFF0000L ) >> 13L ) +
                        (((( LONG ) lpMem ) & 0xFFFFL ) >> 1L ));
                            /* Seite = obere 4 Bit der 20-Bit Adresse */
    bPage = ( BYTE) (((((( LONG ) lpMem ) & 0xFFFF0000L ) >> 12L ) +
                         ((( LONG ) lpMem ) & 0xFFFFL )) >> 16);
    bPage &= ( BYTE ) 0xFE;
    uSize /= 2;           /* Es werden WORDs und nicht BYTEs gezhlt! */
  }

  outp( dma_mode[ iChan / 4 ], bMode | ( iChan & MODE_CHANNELMSK ) );

  dma_ClrFlipFlop( iChan );   /* Adress-/Zhlerflipflop lschen und...*/
                /* Adresse an DMA-Controller bermitteln (LO/HI-BYTE) */
  outp( dma_adress[ iChan ], LOBYTE( uAdress ) );
  outp( dma_adress[ iChan ], HIBYTE( uAdress ) );
  outp( dma_page[ iChan ], bPage );          /* Speicher-Seite setzen */

  dma_ClrFlipFlop( iChan );  /* Adress-/Zhlerflipflop lschen und ...*/
                 /* Zhler an DMA-Controller bermitteln (LO/HI-BYTE) */
  outp( dma_count[ iChan ], LOBYTE( uSize ) );
  outp( dma_count[ iChan ], HIBYTE( uSize ) );

  dma_ClrMask( iChan );                 /* DMA-Kanal wieder freigeben */
}

/**********************************************************************/
/* dma_Mem2Mem : Speicherblock innerhalb einer 64K Seite verschieben  */
/**------------------------------------------------------------------**/
/* Eingabe : bPage   : Adresse der 64K-Seite                          */
/*           uSource : Quelle                                         */
/*           uDest   : Ziel                                           */
/*           uSize   : Anzahl zu transferierender Bytes               */
/**------------------------------------------------------------------**/
/* Hinweis: Der Speicher-Speicher DMA Transfer ist bei neueren        */
/*          Rechnern (ab 80286) nicht implementiert. Im Protected-    */
/*          oder Virtual-8086-Mode des Prozessors werden DMA0         */
/*          Zugriffe generell abgefangen und fhren zu einem          */
/*          Systemabsturz.                                            */
/**********************************************************************/
VOID dma_Mem2Mem( BYTE bPage, WORD uSource, WORD uDest, WORD  uSize )
{ 
  dma_MasterClear( 0 );    /* ber Kanal 0 Master-Controller resetten */
                       /* DMA transferiert 1 Byte mehr als angegeben! */
  uSize = uSize ? uSize - 1 : 0;    /* mind. 1 Byte (0==1) bertragen */

  outp( dma_adress[ 0 ], LOBYTE( uSource ) );
  outp( dma_adress[ 0 ], HIBYTE( uSource ) );
  outp( dma_page[ 0 ], bPage );              /* Speicher-Seite setzen */

  outp( dma_adress[ 1 ], LOBYTE( uDest ) );
  outp( dma_adress[ 1 ], HIBYTE( uDest ) );
  outp( dma_page[ 1 ], bPage );              /* Speicher-Seite setzen */

                 /* Zhler an DMA-Controller bermitteln (LO/HI-BYTE) */
  outp( dma_count[ 0 ], LOBYTE( uSize ) );
  outp( dma_count[ 0 ], HIBYTE( uSize ) );

      /* dma_ClrFlipFlop( 1 );  Adress-/Zhlerflipflop lschen und ...*/
  outp( dma_count[ 1 ], LOBYTE( uSize ) );
  outp( dma_count[ 1 ], HIBYTE( uSize ) );

    /* Durch setzen von COMMAND_ADH0 wird Adresseregister von Kanal 0 */
    /* nicht Inkrementiert. Der Speicherblock den Kanal2 bearbeitet   */
    /* wird somit mit einem Wert initialisiert.                       */
  outp( dma_command[ 0 ], COMMAND_MEM2MEM |
                          COMMAND_FIXEDPRI |
                          COMMAND_COMPRESS );

   /* Wenn sich die Speicherblcke berlappen mu zw. Inkrementierund */
   /* und Dekrementierung unterschieden werden (MODE_DECREMENT). Bei  */
   /* Dekrementieren mu dem jeweiligen DMA-Kanal als Adresse die     */
   /* Speicheradress + Anzahl zu kopierender Bytes bergeben werden!  */
  outp( dma_mode[ 0 ], MODE_BLOCK | MODE_READ  | 0 );      /* Kanal 0 */
  outp( dma_mode[ 0 ], MODE_BLOCK | MODE_WRITE | 1 );      /* Kanal 1 */

  dma_SetRequest( 0 );
}

/**********************************************************************/
/* dma_Until64kPage : Anzahl der Bytes eines bergebenen Speicher-    */
/*                    bereichs innerhalb dessen 64k Seite ermitteln.  */
/**------------------------------------------------------------------**/
/* Eingabe : lpMem : Adresse des Speichers der untersucht werden soll */
/*           uSize : Gre des Speicherbereichs                       */
/* Ausgabe : Anzahl der Bytes innerhalb des Speichers bis zum         */
/*           erreichen des Speicherendes oder der nchsten 64k-Seite  */
/**********************************************************************/
WORD dma_Until64kPage( LPVOID lpMem, WORD uSize )
{ ULONG lAdr, lPageEnd;
                     /* Ermittle 20Bit lineare Adresse aus far Zeiger */
  lAdr = ( ( LONG )FP_SEG( lpMem ) << 4L ) +
           ( LONG )FP_OFF( lpMem );

  lPageEnd = lAdr + 0xFFFFL;                      /* 1 Seite addieren */
  lPageEnd &= 0xFFFF0000L;          /* Bytes bis Seitenende ermitteln */
  lPageEnd -= lAdr;

  return min( ( WORD ) lPageEnd, uSize );
}

/**********************************************************************/
/* dma_AllocMem : DMA fhigen Speicher allokieren.                    */
/**------------------------------------------------------------------**/
/* Eingabe : uSize : Gre des zu allokierenden Speichers (max. 64k). */
/* Ausgabe : Adresse des allokierten Speichers.                       */
/**------------------------------------------------------------------**/
/* Info : - Diese Funktion allokiert solange Speicher, bis der        */
/*          gefundene Speicher innerhalb einer physischen 64K Grenze  */
/*          liegt. Hinweis: Die Allokationsstrategie ist NICHT        */
/*          optimal (s. HeapWalk).                                    */
/*          Der allokierte Speicher beginnt an einer Segment-Adresse  */
/*          und ist somit Page- bzw. 16Byte-Aligned.                  */
/**********************************************************************/
LPVOID dma_AllocMem( WORD uSize )
{ WORD uSegment = 0xFFFFU;
  WORD uOldSegment = 0xFFFFU;
                             /* Kann noch Speicher allokiert werden ? */
  while( !_dos_allocmem( uSize / 16 + 1, &uSegment ) )
  {
    if( uOldSegment != 0xFFFFU )          /* alten Speicher freigeben */
        dma_Free( MK_FP( uOldSegment, 0 ) );
    uOldSegment = uSegment;                  /* Neuen Speicher merken */
                               /* liegt neuer Speicher in 64K Seite ? */
    if( dma_Until64kPage( MK_FP( uSegment, 0 ), uSize ) == uSize )
        return MK_FP( uSegment, 0 );
  }
  if( uSegment != 0xFFFFU )                 /* Schade, kein Speicher! */
      dma_Free( MK_FP( uOldSegment, 0 ) );
  return NULL;
}

/**********************************************************************/
/* dma_FreeMem : DMA fhigen Speicher freigeben.                      */
/**------------------------------------------------------------------**/
/* Eingabe : uSize : Gre des allokierten Speichers.                 */
/**********************************************************************/
VOID dma_Free( LPVOID lpMem )
{                    /* Segment ermitteln und ber DOS-Call freigeben */
  _dos_freemem( FP_SEG( lpMem ) );
}

#endif

