/**********************************************************************/
/*                           D U M P C . C                            */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : ein Filter, der Zeichen von der Standardeingabe*/
/*                     einliest und sie als Hex- und ASCII-Dump auf   */
/*                     dem Standardausgabe-Gert ausgibt              */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 14.08.1987                                     */
/*    letztes Update : 29.10.1991                                     */
/*--------------------------------------------------------------------*/
/*    SPEICHERMODELL : SMALL                                          */
/**********************************************************************/
                                                              
/*== Include-Dateien einbinden =======================================*/
                                                              
#include <stdio.h>
#include <dos.h>
                                                              
/*== Typedefs ========================================================*/
                                                              
typedef unsigned char BYTE;           /* so bastelt man sich ein BYTE */
                                                              
/*== Konstanten ======================================================*/
                                                              
#define NUL     0                         /* Code des NUL-Zeichens    */
#define BEL     7                         /* Code des Klingelzeichens */
#define BS      8                         /* Code der Backspace-Taste */
#define TAB     9                         /* Code der Tabulator-Taste */
#define LF      10                        /* Code des Linefeed        */
#define CR      13                        /* Code der Return-Taste    */
#define ESC     27                        /* Code der Escape-Taste    */
                                                              
/*== Makros ==========================================================*/
                                                              
#define tohex(c) ( ((c)<10) ? ((c) | 48) : ((c) + 'A' - 10) )
                                                              
/**********************************************************************/
/* GETSTDEIN: liest eine bestimmte Anzahl von Zeichen vom Standard-   */
/*            eingabegert in einen Puffer                            */
/* Eingabe : PUFFER    = Zeiger auf Puffer, der die Zeichen aufnnimt  */
/*           MAXANZAHL = Maximale Anzahl der einzulesenden Zeichen    */
/* Ausgabe : Anzahl der gelesenen Zeichen                             */
/**********************************************************************/
                                                              
int GetStdein( char *Puffer, int MaxAnzahl)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
 struct SREGS Segmente;              /* nimmt die Segmentregister auf */
                                                              
 segread(&Segmente);           /* Inhalt der Segmentregister einlesen */
 Register.h.ah = 0x3F;                         /* Funktionsnummer fr */
 Register.x.bx = 0;        /* das Standardeingabegert trgt Handle 0 */
 Register.x.cx = MaxAnzahl;               /* Anzahl zu lesender Bytes */
 Register.x.dx = (unsigned int) Puffer;   /* Offsetadresse des Puffer */
 intdosx(&Register, &Register, &Segmente);/* DOS-Intr. 21(h) aufrufen */
 return(Register.x.ax);         /* Anzahl gelesener Bytes an Aufrufer */
}
                                                              
/**********************************************************************/
/* STRAP: Zeichen an String anhngen                                  */
/* Eingabe : STRING     = Zeiger auf String, an den angehangen werden */
/*                        soll                                        */
/*           ANHAENGSEL = Zeiger auf den String mit dem Anhngsel     */
/* Ausgabe : Pointer hinter das letzte zugefgte Zeichen              */
/**********************************************************************/
                                                              
char *Strap( char *String, char *Anhaengsel)
{
 while (*Anhaengsel)                 /* wiederholen bis '\0' entdeckt */
  *String++ = *Anhaengsel++;                    /* Zeichen bertragen */
 return(String);          /* Pointer an aufrufende Funktion bergeben */
}
                                                              
/**********************************************************************/
/* DODUMP: liest die Zeichen ein und gibt sie als Dump aus            */
/* Eingabe : keine                                                    */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                              
void DoDump( void )
{
 char NeunByte[9],               /*nimmt die eingelesenen Zeichen auf */
      DumpPuf[80],                   /* nimmt eine Zeile des DUMP auf */
      *NextAscii;       /* zeigt auf nchstes ASCII-Zeichen im Puffer */
 BYTE i,                                           /* Schleifenzhler */
      Anzahl;                        /* Anzahl der eingelesenen Bytes */
                                                              
 DumpPuf[30] = 219;     /* Trennzeichen zwischen Hex und ASCII setzen */
 while((Anzahl = GetStdein(NeunByte, 9)) != 0)        /* noch ein Z.? */
  {                                                             /* Ja */
   for (i = 0; i < 30; DumpPuf[i++] = ' ')/* Puffer mit Spaces fllen */
      ;
   NextAscii = &DumpPuf[31];       /* hier beginnen die ASCII-Zeichen */
   for (i = 0; i < Anzahl; i++)    /* alle eingelesenen Z. bearbeiten */
    {
     DumpPuf[i*3] = tohex((BYTE) NeunByte[i] >> 4);    /* Code in Hex */
     DumpPuf[i*3+1] = tohex((BYTE) NeunByte[i] & 15);  /* umwandeln   */
                                                              
     switch (NeunByte[i])                     /* ASCII-Code auswerten */
      {
       case NUL : NextAscii = Strap(NextAscii, "<NUL>");
                  break;
       case BEL : NextAscii = Strap(NextAscii, "<BEL>");
                  break;
       case BS  : NextAscii = Strap(NextAscii, "<BS>");
                  break;
       case TAB : NextAscii = Strap(NextAscii, "<TAB>");
                  break;
       case LF  : NextAscii = Strap(NextAscii, "<LF>");
                  break;
       case CR  : NextAscii = Strap(NextAscii, "<CR>");
                  break;
       case ESC : NextAscii = Strap(NextAscii, "<ESC>");
                  break;
       case EOF : NextAscii = Strap(NextAscii, "<EOF>");
                  break;
       default  : *NextAscii++ = NeunByte[i];
      }
    }
   *NextAscii = 219;             /* Endezeichen fr ASCII-Darstellung */
   *(NextAscii+1) = '\r';       /* Carriage-Return an Ende des Puffer */
   *(NextAscii+2) = '\0';   /* NUL wird bei Ausgabe in LF umgewandelt */
   puts(DumpPuf);        /* String auf Standardausgabegert schreiben */
  }
}
                                                              
/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/
                                                              
void main( void )
{
 DoDump();                           /* Zeichen einlesen und ausgeben */
}
