/**********************************************************************/
/*                        E D I T . C                                 */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Text-Editor                                      */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 03.05.1994                                       */
/*  letztes Update : 15.05.1994                                       */
/**********************************************************************/
#ifndef __EDIT_C                 /* EDIT.C kann auch #Included werden */
#define __EDIT_C

/*- Include-Dateien einbinden ---------------------------------------*/
#include "edit.h"
#include "win.h"
#include "kbd.h"

/*- Folgende Zeilen innerhalb eines Projektes entfernen: -------------*/
#include "win.c"

/**********************************************************************/
/* edit_GetRows : Zeilenanfnge der darzustellenden Zeilen ermitteln. */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit       - Adresse des Editors                       */
/*           iStartRow    - erste neuzuberechnende Bildschirmzeile    */
/*           iEndRow      - letzte neuzuberechnende Bildschirmzeile   */
/*           iStartBufPos - Begin der ersten Bildschirmzeile im Text  */
/* Ausgabe : Anzahl der neuberechneten Zeilen                         */
/**------------------------------------------------------------------**/
/* Info : Diese Funktion berechnet die Zeilenanfnge nur so weit wie  */
/*        ntig, d.h. ist der neuberechnete Zeilenanfang schon        */
/*        identisch mit dem schon vor dem Aufruf dieser Funktion      */
/*        bekannten Zeilenanfang wird dir Neuberechnung gestoppt,     */
/*        wobei die Annahme gemacht wird, das der rest des Textes un- */
/*        verndert ist.                                              */
/**********************************************************************/
int edit_GetRows( LPEDIT lpEdit,
                  int    iStartRow,
                  int    iEndRow,
                  int    iStartBufPos )
{ int i, j, iUsed, iChanged,iNumChanged;
  LPBYTE lpBuffer;

  iChanged = TRUE;

  lpEdit->iRowStarts[ iStartRow ] = iStartBufPos;

  for( i = iStartRow + 1; i < iEndRow; i++ )
       lpEdit->iRowStarts[ i ] = lpEdit->iUsed + 1;

  i = iStartRow;
  j = iStartBufPos;

  iUsed   = lpEdit->iUsed;
  lpBuffer = &lpEdit->lpBuffer[ j ];

  iNumChanged =0;
  while( ( i <= iEndRow ) && ( j <= iUsed ) && iChanged )
  {
    j++;
    if( *(lpBuffer++) == '\n' )
    {
      i++;
      iChanged = ( lpEdit->iRowStarts[ i ] != j );
      if( iChanged ) iNumChanged++;
      lpEdit->iRowStarts[ i ] = j;
    }
  }
  return iNumChanged;
}

/**********************************************************************/
/* edit_LeftOffset : erstes darzustellendes Zeichen aller Zeilen      */
/*                   bestimmen.                                       */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit  - Adresse des Editors                            */
/*           iRowPos - Adresse eines Integers, das die Position der   */
/*                     Eingabemarke innerhalb der aktuellen Zeile     */
/*                     aufnehmen soll.                                */
/* Ausgabe : TRUE  - 'LeftOffset' hat sich gendert.                  */
/*           FALSE - 'LeftOffset' unverndert.                        */
/**------------------------------------------------------------------**/
/* Info : Im Textmodus gilt der LeftOffset fr alle dargestellten     */
/*        Zeilen des Editors, da hier eine Zeichenorientierung vor-   */
/*        liegt. In graphischen Umgebungen (z.B. Windows) mu der     */
/*        LeftOffset fr jede Zeile berechnet werden, wobei das Ma   */
/*        nicht 'Zeichen', sondern 'cm' oder hnliches ist.           */
/*        (Proportionalschriften beachten!)                           */
/**********************************************************************/
int edit_LeftOffset( LPEDIT lpEdit, LPWORD iRowPos )
{ int iInc;
     /* Bildschirminhalt um 1/3 Breite aber mind. 10 Zeichen scrollen */
  iInc = ((lpEdit->Win.iW)/3) > 10 ? 10 : (lpEdit->Win.iW)/3;
  if(!iInc) iInc = 1;

           /* Position der Eingabemarke innerhalb der aktuellen Zeile */
  *iRowPos = lpEdit->iEditPos - lpEdit->iRowStarts[ lpEdit->iEditRow ];

              /* Mu Inhalt nach links oder rechts verschoben werden? */
  if( ( *iRowPos - lpEdit->iLeftOffset ) >= lpEdit->Win.iW ) /* links */
  {
    lpEdit->iLeftOffset = ((*iRowPos-lpEdit->Win.iW)/iInc)*iInc+iInc;
    return TRUE;
  }

  if( *iRowPos < lpEdit->iLeftOffset )                      /* rechts */
  {
    lpEdit->iLeftOffset = ((*iRowPos-iInc )/iInc)*iInc;
    if( lpEdit->iLeftOffset < 0 ) lpEdit->iLeftOffset = 0;
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_Display : Aktuellen Editorinhalt anzeigen.                    */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit - Adresse des Editors                             */
/*           iStart - erste anzuzeigende Bildschirmzeile              */
/*           iEnd   - letzte anzuzeigende Bildschirmzeile             */
/**********************************************************************/
void edit_Display( LPEDIT lpEdit, int iStart, int iEnd )
{ int i, n, iW, iRowPos, OldFlags;

  if( lpEdit->Win.uFlags & WIN_ACTIVE )
    lpEdit->Win.iAttr |= 0x08;                /* HighColor einblenden */
  else
    lpEdit->Win.iAttr &= ~0x08;               /* Highcolor ausblenden */

  OldFlags = lpEdit->Win.uFlags;                      /* Flags retten */
  lpEdit->Win.uFlags &= ~( WIN_CRLF | WIN_SCROLL | WIN_HASCURSOR );

  if( lpEdit->uFlags & EDIT_DISPLAY )       /* Editor nur fr Anzeige */
  {
    iStart = 0;                               /* alle Zeilen anzeigen */
    iEnd = lpEdit->Win.iH - 1;
    iRowPos = 0;
    OldFlags &= ~WIN_HASCURSOR;
  }
  else                      /* mu Bildschirminhalt gescrollt werden? */
  if( edit_LeftOffset( lpEdit, &iRowPos ) )
  {
    iStart = 0;
    iEnd = lpEdit->Win.iH - 1;
  }

  for( i = iStart; ( i <= iEnd ) && ( i < lpEdit->Win.iH ); i++ )
  {
    win_GotoXY( &lpEdit->Win, 0, i );   /* Bildschirmzeile "anfahren" */

    n = lpEdit->iRowStarts[ i + 1 ];     /* letztes Zeichen der Zeile */

    if( n > lpEdit->iUsed ) n = lpEdit->iUsed;    /* letztes Zeichen? */
            /* iW = Anzahl der auszugebenden Zeichen in der Zeile (i) */
    iW = ( n - lpEdit->iRowStarts[ i ] ) - lpEdit->iLeftOffset;

    if( iW <= 0 ) _win_Print( &lpEdit->Win, "\n", 1 );
    else
    {                       /* Maximal eine Bildschirmbreite ausgeben */
      iW = iW > lpEdit->Win.iW ? lpEdit->Win.iW : iW;
      _win_Print( &lpEdit->Win,                   /* Zeichen ausgeben */
                  &lpEdit->lpBuffer[ lpEdit->iRowStarts[ i ] +
                                     lpEdit->iLeftOffset ], iW );
      if( iW < lpEdit->Win.iW )             /* Rest der Zeile lschen */
      {
        win_GotoXY( &lpEdit->Win, iW, i );
        _win_Print( &lpEdit->Win, "\n", 1 );
      }
    }
  }

  lpEdit->Win.uFlags = OldFlags;
  win_GotoXY( &lpEdit->Win,     /* edit_LeftOffset() brechnet iRowPos */
              iRowPos - lpEdit->iLeftOffset,
              lpEdit->iEditRow );
}

/**********************************************************************/
/* edit_SetBuffer : Eingabebuffer initialisieren                      */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit   - Adresse des Editors                           */
/*           lpBuffer - Adresse des neuen Texted                      */
/*           iBufSize - Anzahl der zeichen im Buffer                  */
/**------------------------------------------------------------------**/
/* Info : Nach Aufruf dieser Funktion befindet sich die Eingabemarke  */
/*        am Anfang des durch diese Funktion angegebenen Textes.      */
/**********************************************************************/
void edit_SetBuffer( LPEDIT lpEdit, LPBYTE lpBuffer, int iBufSize )
{ int i;
                              /* neuer Inhalt grer als Editbuffer ? */
  if( iBufSize > lpEdit->iSize ) iBufSize = lpEdit->iSize;

  for( i = 0; i < iBufSize; i++ )            /* Bufferinhalt kopieren */
    lpEdit->lpBuffer[ i ] = lpBuffer[ i ];

  lpEdit->iUsed = iBufSize;
  lpEdit->iEditPos = 0;
  lpEdit->iEditRow = 0;
  lpEdit->iLeftOffset = 0;

  edit_GetRows( lpEdit, 0, lpEdit->Win.iH, 0 );
  edit_Display( lpEdit, 0, lpEdit->Win.iH - 1 );
}

/**********************************************************************/
/* edit_init : Editor initialisieren                                  */
/**------------------------------------------------------------------**/
/* Eingabe : iX, iY, iW, iH, iA - Fensterposition                     */
/*                                -dimension                          */
/*                                -attribut                           */
/*           lpEdit   - Adresse des Editors                           */
/*           lpBuffer - zu verwendender Textspeicher                  */
/*           iBufUsed - im Textspeicher schon enthaltene Zeichen      */
/*           iBufSize - Gre des Textspeichers                       */
/*           uFlags   - Editorflags : EDIT_DISPLAY - nur Anzeigen     */
/*                                    EDIT_NOCR    - keine Eingabe    */
/*                                                   von 'CR'         */
/**------------------------------------------------------------------**/
/* Info : Der Textspeicher zur Aufnahme der eingegebenen Zeichen mu  */
/*        "von auen" zur Verfgung gestellt werden.                  */
/**********************************************************************/
void edit_Init( int iX, int iY, int iW, int iH, int iA,
                LPEDIT lpEdit, LPBYTE lpBuffer,
                int    iBufUsed,
                int    iBufSize,
                int    uFlags )
{ int i;
  UINT winFlags = WIN_SCROLL | WIN_CRLF;

  if( !( lpEdit->uFlags & EDIT_DISPLAY ) )
    winFlags |= WIN_HASCURSOR;

  win_Init( &lpEdit->Win, iX, iY, iW, iH, iA, winFlags );

  lpEdit->lpBuffer    = lpBuffer;
  lpEdit->iUsed       = iBufUsed;
  lpEdit->iSize       = iBufSize;
  lpEdit->iEditPos    = 0;
  lpEdit->iEditRow    = 0;
  lpEdit->iLeftOffset = 0;
  lpEdit->uFlags      = uFlags;

  edit_GetRows( lpEdit, 0, iH, 0 );            /* Text formatieren... */
  edit_Display( lpEdit, 0, iH - 1 );                /* und darstellen */
}

/**********************************************************************/
/* edit_PreviousRow : Vorgngerzeile der ersten Bildschirmzeile er-   */
/*                    mitteln.                                        */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit - Adresse des Editors                             */
/* Ausgabe : TRUE  - Vorgngerzeile existiert                         */
/*           FALSE - erste Bildschirmzeile = erste Editorzeile        */
/**------------------------------------------------------------------**/
/* Info : Solange eine Zeile durch ein '\n' begrenzt wird, ist das    */
/*        Auffinden der Vorgngerzeile durch Suchen im Text zu        */
/*        ermitteln. Werden WordWrap-Features eingebaut, mu ein      */
/*        Array den Anfang aller Vorgngerzeilen aufnehmen, aus dem   */
/*        der Beginn der Vorgngerzeile einfach ausgelesen werden     */
/*        kann.                                                       */
/**********************************************************************/
int edit_PreviousRow( LPEDIT lpEdit )
{ int iPrevRow, i;

                                    /* Start der ersten Zeile im Text */
  iPrevRow = lpEdit->iRowStarts[ 0 ];

  if( iPrevRow )      /* Beginnt erste Bildschirmzeile am Textanfang? */
  {
    iPrevRow--;            /* nein, Vorgngerzeile mu als existieren */

    for( i = lpEdit->Win.iH + 1; i > 0; i-- ) /* Bildschirmzeilen-    */
             lpEdit->iRowStarts[ i ] =        /* anfnge (intern)     */
             lpEdit->iRowStarts[ i - 1 ];     /* nach unten scrollen. */

    while( ( iPrevRow > 0 ) &&          /* Zeichen vor '\n' Suchen... */
           ( lpEdit->lpBuffer[ iPrevRow - 1 ] != '\n' ) )
             iPrevRow--;
              /* und als neue Begin der ersten Bildschirmzeile setzen */
    lpEdit->iRowStarts[ 0 ] = iPrevRow;
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_NextRow : Nachfolgerzeile der letzen Bildschirmzeile suchen   */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit - Adresse des Editors                             */
/* Ausgabe : TRUE  - Nachfolgerzeile existiert                        */
/*           FALSE - letze Editorzeile wird schon angezeigt           */
/*                   (mu nicht letzte Bildschirmzeile sein!)         */
/**------------------------------------------------------------------**/
/* Info : Nicht verwendete Bildschirmzeilen (z.B. bei kurzen Texten)  */
/*        "beginnen" am letzten Zeichen des Textes (s. iRowStarts)    */
/**********************************************************************/
int edit_NextRow( LPEDIT lpEdit )
{ int i;
                              /* letzte Bildschirmzeile != Textende ? */
  if( lpEdit->iRowStarts[ lpEdit->Win.iH ] <= lpEdit->iUsed )
  {                             /* Zeilen (intern) nach oben scrollen */
    for( i = 0 ; i <= lpEdit->Win.iH + 1; i++ )
         lpEdit->iRowStarts[ i ] = lpEdit->iRowStarts[ i + 1 ];
                                    /* nchsten Zeilenafang berechnen */
    edit_GetRows( lpEdit, lpEdit->Win.iH - 1, lpEdit->Win.iH + 1,
                 lpEdit->iRowStarts[ lpEdit->Win.iH - 1 ] );
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_StartOfText : erste Zeile des Textes anzeigen und Eingabe-    */
/*                    marke an Textanfang begeben                     *7
/**-------------------------------------------------------------------*/
/* Eingabe : lpEdit - Adresse des Editors                             */
/* Ausgabe : TRUE  - Bildschirm mu neu dargestellt werden            */
/*           FALSE - erste Textzeile wird schon angezeigt             */
/**********************************************************************/
int edit_StartOfText( LPEDIT lpEdit )
{
  if( lpEdit->iEditPos != 0 )
  {
    lpEdit->iEditPos = 0;
    lpEdit->iEditRow = 0;
    if( lpEdit->iRowStarts[ 0 ] != 0 )
        edit_GetRows( lpEdit, 0, lpEdit->Win.iH, 0 );
    edit_Display( lpEdit, 0, lpEdit->Win.iH - 1 );
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_EndOfText : Letzte Zeile des Textes anzeigen und Eingabemarke */
/*                  auf letztes Zeichen bewegen.                      */
/**-------------------------------------------------------------------*/
/* Eingabe : lpEdit   - Adresse des Editors                           */
/*           iDisplay - nderungen auch anzeigen                      */
/* Ausgabe : TRUE  - Bildschirm mu neu dargestellt werden            */
/*           FALSE - letzte Textzeile wird schon angezeigt            */
/**********************************************************************/
int edit_EndOfText( LPEDIT lpEdit, int iDisplay )
{ int j;          /* Steht Eingabemarke schon auf dem letzen Zeichen? */
  if( lpEdit->iEditPos != lpEdit->iUsed )
  {
                            /* Wird letzte Textzeile schon angezeigt? */
    for( j = 0; j < lpEdit->Win.iH; j++ )
      if( lpEdit->iRowStarts[ j ] > lpEdit->iUsed )
      {   /* Suche letze Textzeile auf Bildschirm und bewege Eingabe- */
          /* marke und aktuelle Zeile auf diese Bildschirm zeile      */
          lpEdit->iEditRow = j - 1;
          lpEdit->iEditPos = lpEdit->iUsed;
          if( lpEdit->iEditRow < 0 ) lpEdit->iEditRow = 0;
          if( iDisplay ) edit_Display( lpEdit, 0, lpEdit->Win.iH );
          return TRUE;
      }
                    /* letztes Zeichen = Start aller Bildschirmzeilen */
    for( j = 0; j <= lpEdit->Win.iH + 1; j++ )
        lpEdit->iRowStarts[ j ] = lpEdit->iUsed + 1;

                        /* lpEdit->Win.iH-1 Zeilen nach oben Scrollen */
    j = 0;
    lpEdit->iRowStarts[ 0 ] = lpEdit->iUsed;
    while( edit_PreviousRow( lpEdit ) & ( j < lpEdit->Win.iH - 1 ) )
           j++;

    lpEdit->iEditRow = j;
    lpEdit->iEditPos = lpEdit->iUsed;
    if( iDisplay ) edit_Display( lpEdit, 0, lpEdit->Win.iH );
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_SetRowPos : Setze Eingabemarke in aktueller Eingabezeile      */
/*                  (iEditRow) auf angegebene Position                */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit  - Adresse des Editors                            */
/*           iRowPos - Spalte, auf die EIngabemarke in aktueller      */
/*                     Eingabezeile gesetzt werden soll.              */
/**********************************************************************/
void edit_SetRowPos( LPEDIT lpEdit, int iRowPos )
{
                         /* Hat neue Zeile berhaupt iRowPos Zeichen? */
  if( ( lpEdit->iRowStarts[ lpEdit->iEditRow + 1 ] -
        lpEdit->iRowStarts[ lpEdit->iEditRow ] ) > iRowPos )
        lpEdit->iEditPos =
          lpEdit->iRowStarts[ lpEdit->iEditRow ] + iRowPos;
      else         /* Nein, dann auf letztes Zeichen der Zeile setzen */
        lpEdit->iEditPos =
          lpEdit->iRowStarts[ lpEdit->iEditRow + 1 ] - 1;
}

/**********************************************************************/
/* edit_CRSRUp : Eingabemarke eine Zeile nach oben bewegen            */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit     - Adresse des Editors                         */
/*           iUpdatePos - Eingabemarke in neuer Zeile auf alte Spalte */
/*                        setzen                                      */
/* Ausgabe : TRUE  - Eingabemarke wurde bewegt                        */
/*           FALSE - Eingabemarke steht in erster Textzeile           */
/**********************************************************************/
int edit_CRSRUp( LPEDIT lpEdit, int iUpdatePos )
{ int iRowPos, iStart, iEnd;

  iStart = 0;                               /* Neuanzuzeigende Zeilen */
  iEnd = 0;
     /* Position der Eingabemarke innerhalb aktueller Zeile ermitteln */
  iRowPos = lpEdit->iEditPos - lpEdit->iRowStarts[ lpEdit->iEditRow ];
           /* Bei Textanzeige soll Bildschirminhalt gescrollt werden! */
  if( lpEdit->uFlags & EDIT_DISPLAY ) lpEdit->iEditRow = 0;
                                    /* Aktuelle Zeile != Textanfang ? */
  if( lpEdit->iRowStarts[ lpEdit->iEditRow ] > 0 )
  {
    lpEdit->iEditRow--;              /* Aktuelle Zeile dekrementieren */
    if( lpEdit->iEditRow < 0 )            /* Text nach unten scrollen */
    {
      edit_PreviousRow( lpEdit );
      lpEdit->iEditRow = 0;
      iEnd = lpEdit->Win.iH - 1; /* Alle Bildschirmzeilen neuanzeigen */
    }      /* Eingabemarke in neuer Zeile auf gleiche Position setzen */
    if( iUpdatePos ) edit_SetRowPos( lpEdit, iRowPos );
    edit_Display( lpEdit, iStart, iEnd );
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_CRSRDn : Eingabemarke eine Zeile nach unten bewegen           */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit     - Adresse des Editors                         */
/*           iUpdatePos - Eingabemarke in neuer Zeile auf alte Spalte */
/*                        setzen                                      */
/* Ausgabe : TRUE  - Eingabemarke wurde bewegt                        */
/*           FALSE - Eingabemarke steht in letzter Textzeile          */
/**********************************************************************/
int edit_CRSRDn( LPEDIT lpEdit, int iUpdatePos )
{ int iRowPos, iStart, iEnd;

  iStart = 0;                               /* Neuanzuzeigende Zeilen */
  iEnd = 0;
     /* Position der Eingabemarke innerhalb aktueller Zeile ermitteln */
  iRowPos = lpEdit->iEditPos - lpEdit->iRowStarts[ lpEdit->iEditRow ];
           /* Bei Textanzeige soll Bildschirminhalt gescrollt werden! */
  if( lpEdit->uFlags & EDIT_DISPLAY )
    while((lpEdit->iRowStarts[lpEdit->iEditRow+1]<lpEdit->iUsed) &&
          (lpEdit->iEditRow<(lpEdit->Win.iH-1))) lpEdit->iEditRow++;

  if( lpEdit->iRowStarts[ lpEdit->iEditRow + 1 ] < lpEdit->iUsed )
  {
    lpEdit->iEditRow++;
    if( lpEdit->iEditRow == lpEdit->Win.iH )
    {
      iStart = 0;
      iEnd = lpEdit->Win.iH - 1;
      edit_NextRow( lpEdit );
      lpEdit->iEditRow = lpEdit->Win.iH - 1;
    }
    if( iUpdatePos ) edit_SetRowPos( lpEdit, iRowPos );
    edit_Display( lpEdit, iStart, iEnd );
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_CRSRLeft : Eingabemarke ein Zeichen nach links bewegen        */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit     - Adresse des Editors                         */
/* Ausgabe : TRUE  - Eingabemarke wurde bewegt                        */
/*           FALSE - Eingabemarke steht am Textanfang                 */
/**********************************************************************/
int edit_CRSRLeft( LPEDIT lpEdit )
{
                          /* Bei Nur-Anzeige Bildschirminhalt bewegen */
  if( lpEdit->uFlags & EDIT_DISPLAY )
  {
    if( lpEdit->iLeftOffset > 0 ) lpEdit->iLeftOffset--;
    edit_Display( lpEdit, 0, 0 );
    return TRUE;
  }

  if( lpEdit->iEditPos > 0 )
  {
    lpEdit->iEditPos--;             /* Eingabeposition dekrementieren */
    if( lpEdit->iEditPos < lpEdit->iRowStarts[ lpEdit->iEditRow ] )
        edit_CRSRUp( lpEdit, FALSE );    /* Zeile Hoch und Darstellen */
    edit_Display( lpEdit, 0, 0);           /* ndert sich LeftOffset? */
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_CRSRRight : Eingabemarke ein Zeichen nach rechts bewegen      */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit     - Adresse des Editors                         */
/* Ausgabe : TRUE  - Eingabemarke wurde bewegt                        */
/*           FALSE - Eingabemarke steht am Textende                   */
/**********************************************************************/
int edit_CRSRRight( LPEDIT lpEdit )
{
                          /* Bei Nur-Anzeige Bildschirminhalt bewegen */
  if( lpEdit->uFlags & EDIT_DISPLAY )
  {
    lpEdit->iLeftOffset++;
    edit_Display( lpEdit, 0, 0 );
    return TRUE;
  }

  if( lpEdit->iEditPos < lpEdit->iUsed )
  {
    lpEdit->iEditPos++;             /* Eingabeposition inkrementieren */
    if( lpEdit->iEditPos >= lpEdit->iRowStarts[ lpEdit->iEditRow + 1 ] )
        edit_CRSRDn( lpEdit, FALSE );  /* Zeile runter und Darstellen */
    edit_Display( lpEdit, 0, 0);           /* ndert sich LeftOffset? */
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_Home : Eingabemarke auf Zeilenanfang setzen                   */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit     - Adresse des Editors                         */
/*           iDisplay   - != 0  nderung direkt anzeigen              */
/*                        == 0  nderungen nicht darstellen           */
/**********************************************************************/
void edit_Home( LPEDIT lpEdit, int iDisplay )
{                      /* Bei Nur-Anzeige LeftOffset auf Zeilenanfang */
  if( lpEdit->uFlags & EDIT_DISPLAY )
  {
    lpEdit->iLeftOffset = 0;
    if( iDisplay ) edit_Display( lpEdit, 0, 0 );
  }

  if( lpEdit->iEditPos > 0 )  /* Eingabemarke auf Zeilenanfang setzen */
  {
    lpEdit->iEditPos = lpEdit->iRowStarts[ lpEdit->iEditRow ];
    if( iDisplay ) edit_Display( lpEdit, 0, 0 );
  }
}

/**********************************************************************/
/* edit_End : Eingabemarke auf Zeilenende setzen                      */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit     - Adresse des Editors                         */
/**********************************************************************/
void edit_End( LPEDIT lpEdit )
{
  if( lpEdit->iEditPos < lpEdit->iUsed )
  {
    lpEdit->iEditPos = lpEdit->iRowStarts[ lpEdit->iEditRow + 1 ] - 1;
    edit_Display( lpEdit, 0, 0 );
  }
}

/**********************************************************************/
/* edit_CRSRPgDn : Eingabemarke eine Bildschirmzeile nach unten       */
/*                 bewegen                                            */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit     - Adresse des Editors                         */
/**********************************************************************/
int edit_CRSRPgDn( LPEDIT lpEdit )
{ int i, j, iH, iRowPos;

     /* Position der Eingabemarke innerhalb aktueller Zeile ermitteln */
  iRowPos = lpEdit->iEditPos - lpEdit->iRowStarts[ lpEdit->iEditRow ];
  iH = ( lpEdit->Win.iH - 1 );        /* Anzahl zu scrollender Zeilen */
  if( !iH ) iH = 1;                            /* Einzeiler beachten! */

  for( i = 0; i < iH; i++ )                  /* Neue Zeilen ermitteln */
       if( edit_NextRow( lpEdit ) ) j++;

  if( ( !j ) &&                /* Ende des Textes schon dargestellt ? */
      ( lpEdit->iRowStarts[ lpEdit->iEditRow + 1 ] > lpEdit->iUsed ) )
        return FALSE;

  if( j < iH )             /* Eingabezeile soweit wie mglich bewegen */
    while( ( j < lpEdit->Win.iH ) &&
           ( lpEdit->iRowStarts[ j + 1 ] < lpEdit->iUsed ) )
             j++;
  else  j = lpEdit->iEditRow;/* Inhalt wurde um eine Seite gescrollt, */
                             /* Eingabezeile bleibt also auf gleicher */
                             /* Bildschirmzeile.                      */

  lpEdit->iEditRow = j;
  edit_SetRowPos( lpEdit, iRowPos );
  edit_Display( lpEdit, 0, lpEdit->Win.iH - 1 );
  return TRUE;
}

/**********************************************************************/
/* edit_CRSRPgUp : Eingabemarke eine Bildschirmzeile nach oben        */
/*                 bewegen                                            */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit     - Adresse des Editors                         */
/**********************************************************************/
int edit_CRSRPgUp( LPEDIT lpEdit )
{ int i, j, iH, iRowPos;

     /* Position der Eingabemarke innerhalb aktueller Zeile ermitteln */
  iRowPos = lpEdit->iEditPos - lpEdit->iRowStarts[ lpEdit->iEditRow ];
  iH = ( lpEdit->Win.iH - 1 );        /* Anzahl zu scrollender Zeilen */
  if( !iH ) iH = 1;                            /* Einzeiler beachten! */

  for( i = 0; i < iH; i++ )                  /* Neue Zeilen ermitteln */
       if( edit_PreviousRow( lpEdit ) ) j++;

  if(!( j + lpEdit->iEditRow )) return FALSE;      /* Keine Bewegung? */

  if( j < iH ) j = 0;                   /* Neue aktuelle Eingabezeile */
  else  j = lpEdit->iEditRow;

  lpEdit->iEditRow = j;
  edit_SetRowPos( lpEdit, iRowPos );
  edit_Display( lpEdit, 0, lpEdit->Win.iH );
  return TRUE;
}

/**********************************************************************/
/* edit_AdjustRow : Finde zu Eingabeposition passende Bildschirmzeile */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit - Adresse des Editors                             */
/* Ausgabe : TRUE  - Eingabezeile wird auf Bildschirm angezeigt       */
/*           FALSE - Eingabezeile wird nicht angezeigt                */
/**********************************************************************/
int edit_AdjustRow( LPEDIT lpEdit )
{
  int i;
  for( i = 0; i < lpEdit->Win.iH - 1; i++ )  /* Alle Bildschirmzeilen */
  {
    if( ( lpEdit->iEditPos >= lpEdit->iRowStarts[ i ] ) &&
        ( lpEdit->iEditPos < lpEdit->iRowStarts[ i + 1 ] ) )
        {
          lpEdit->iEditRow = i;
          return TRUE;
        }
  }
  if( lpEdit->iEditPos >= lpEdit->iRowStarts[ lpEdit->Win.iH - 1 ] )
      lpEdit->iEditRow = lpEdit->Win.iH;
  else
  if( lpEdit->iEditPos < lpEdit->iRowStarts[ 0 ] )
      lpEdit->iEditRow = -1;
  return FALSE;
}

/**********************************************************************/
/* edit_DeleteChar : Zeichen unter Eingabemarke lschen               */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit - Adresse des Editors                             */
/* Ausgabe : TRUE   - Zeichen wurde gelscht                          */
/*           FALSE  - Eingabemarke steht am Textende, lschen nicht   */
/*                   mglich                                          */
/**********************************************************************/
int edit_DeleteChar( LPEDIT lpEdit )
{ int i, iStart, iEnd;                /* Bei Nur-Anzeige kein Lschen */
  if( lpEdit->uFlags & EDIT_DISPLAY ) return TRUE;

  if( lpEdit->iEditPos < lpEdit->iUsed )
  {     /* Zeichen aus Buffer entfernen, bzw. Buffer "zusammenziehen" */
    for( i = lpEdit->iEditPos; i < lpEdit->iUsed; i++ )
         lpEdit->lpBuffer[ i ] = lpEdit->lpBuffer[ i + 1 ];
    lpEdit->iUsed--;   /* Anzahl der Zeichen im Buffer dekrementieren */
                          /* Zeilen nach aktueller Zeile neu anzeigen */
    for( i = lpEdit->iEditRow + 1; i < lpEdit->Win.iH + 1; i++ )
         lpEdit->iRowStarts[ i ] -= 1;

    iStart = lpEdit->iEditRow;
    iEnd   = lpEdit->iEditRow;
            /* Berechne wieviele Zeilen neu dargestellt werden mssen */
    iEnd += edit_GetRows( lpEdit,
                          lpEdit->iEditRow,
                          lpEdit->Win.iH + 1,
                          lpEdit->iRowStarts[ lpEdit->iEditRow ] ) + 1;
    edit_Display( lpEdit, iStart, iEnd );
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_DeleteArea : Textbereich lschen                              */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit - Adresse des Editors                             */
/*           iStart - erstes zu lschenden Zeichen                    */
/*           iLen   - Anzahl der zu lschenden Zeichen                */
/* Ausgabe : TRUE   - Bereich wurde gelscht                          */
/*           FALSE  - Bereich nicht vollstndig im Text               */
/*                   mglich                                          */
/**********************************************************************/
int edit_DeleteArea( LPEDIT lpEdit, int iStart, int iLen )
{ int i, iEnd;
                        /* zu lschender Bereich vollstndig im Text? */
  if( iStart + iLen <= lpEdit->iUsed )
  {
    iEnd = lpEdit->iUsed - iLen;                    /* neues Textende */
    for( i = iStart; i < iEnd; i++ )       /* Buffer "zusammenziehen" */
         lpEdit->lpBuffer[ i ] = lpEdit->lpBuffer[ i + iLen ];
    lpEdit->iUsed = iEnd;                  /* Neue Buffergre setzen */

                         /* Eingabemarke nach zu lschendem Bereich ? */
    if( lpEdit->iEditPos >= iStart )
        if( lpEdit->iEditPos <= iStart + iLen )
                lpEdit->iEditPos = iStart; /* Eingabemarke im Bereich */
        else    lpEdit->iEditPos -= iLen;      /* Eingabemarke danach */

    if( iStart < lpEdit->iRowStarts[ 0 ] ) /* Anfang der ersten Bild- */
       lpEdit->iRowStarts[0] -= iLen;      /* schirmzeile updaten     */

    iStart = 0;         /* Anzahl neudarzustellender Zeilen berechnen */
    iEnd += edit_GetRows( lpEdit,
                          0,
                          lpEdit->Win.iH + 1,
                          lpEdit->iRowStarts[ 0 ] );
    edit_AdjustRow( lpEdit );      /* aktuelle Eingabemarke ermitteln */
    edit_Display( lpEdit, iStart, iEnd );
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_DeleteFirstRow : Erste Zeile des Textes lschen               */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit   - Adresse des Editors                           */
/*           iDisplay - nderungen anzeigen                           */
/* Ausgabe : TRUE   - Erste Zeile konnte gelscht werden              */
/*           FALSE  - Keine weiteren Zeichen mehr zum Lschen da!     */
/**********************************************************************/
int edit_DelFirstRow( LPEDIT lpEdit, int iDisplay )
{
  int i;
  i = 0;
  while( ( lpEdit->lpBuffer[ i ] != '\n' ) && ( i < lpEdit->iUsed ) )
         i++;
  if( lpEdit->lpBuffer[ i ] == '\n' ) i++;
  if( !edit_DeleteArea( lpEdit, 0, i ) ) return FALSE;
  if( iDisplay ) edit_Display( lpEdit, 0, lpEdit->Win.iH );
  return TRUE;
}

/**********************************************************************/
/* edit_Insert : Text an Einfgemarke einfgen                        */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit   - Adresse des Editors                           */
/*           lpBuffer - Adresse des einzufgenden Textes              */
/*           iBufSize - Anzahl der zu einzufgenden Zeichen           */
/* Ausgabe : TRUE   - Zeichen knnten eingefgt werden                */
/*           FALSE  - Textbuffer zu klein                             */
/**********************************************************************/
int edit_Insert( LPEDIT lpEdit, LPBYTE lpBuffer, int iBufSize )
{ int i, iEnd, iStart;
  if( lpEdit->iUsed + iBufSize < lpEdit->iSize )
  {
    for( i = lpEdit->iUsed + iBufSize; i > lpEdit->iEditPos; i-- )
         lpEdit->lpBuffer[ i ] = lpEdit->lpBuffer[ i - iBufSize ];
                                            /* neuen Text "einpassen" */
    for( i = 0; i < iBufSize; i++ )
         lpEdit->lpBuffer[ lpEdit->iEditPos + i ] = lpBuffer[ i ];

    lpEdit->iUsed    += iBufSize;      /* Editpos und Gre des Text- */
    lpEdit->iEditPos += iBufSize;      /* Buffers anpassen            */
    iStart = lpEdit->iEditRow;
    iEnd   = lpEdit->iEditRow + 1;

                         /* Anfnge der Bildschirmzeilen neuberechnen */
    for( i = lpEdit->iEditRow + 1; i <= lpEdit->Win.iH; i ++ )
         if( lpEdit->iRowStarts[ i ] <= lpEdit->iUsed )
             lpEdit->iRowStarts[ i ] += iBufSize;

    iEnd = edit_GetRows( lpEdit,
                         iStart,
                         lpEdit->Win.iH + 1,
                         lpEdit->iRowStarts[ lpEdit->iEditRow ] ) + 1;
    iEnd += iStart;

    edit_AdjustRow( lpEdit );      /* aktuelle Eingabemarke ermitteln */
                                       /* neue Zeile durch Einfgung? */
    if( lpEdit->iEditRow == lpEdit->Win.iH )
    {
      edit_NextRow( lpEdit );
      iStart = 0;
      iEnd   = lpEdit->Win.iH - 1;
      lpEdit->iEditRow = lpEdit->Win.iH - 1;
    }
    edit_Display( lpEdit, iStart, iEnd );
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* _edit_Append : Text ans Ende des schon vorhandenen Textes anfgen. */
/*                nderungen nicht darstellen                         */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit   - Adresse des Editors                           */
/*           lpBuffer - Adresse des anzuhngenden Textes              */
/*           iBufSize - Anzahl der zu anzuhngenden Zeichen           */
/* Ausgabe : TRUE   - Zeichen knnten eingefgt werden                */
/*           FALSE  - Textbuffer zu klein                             */
/**********************************************************************/
int _edit_Append( LPEDIT lpEdit, LPBYTE lpBuffer, int iBufSize )
{
  if( lpEdit->iUsed + iBufSize < lpEdit->iSize )
  { int i;
                                                     /* Taxt anhngen */
    for( i = 0; i < iBufSize; i++ )
         lpEdit->lpBuffer[ lpEdit->iUsed++ ] = lpBuffer[ i ];
    edit_GetRows( lpEdit, 0,
                  lpEdit->Win.iH + 2,
                  lpEdit->iRowStarts[ 0 ] );
    edit_AdjustRow( lpEdit );
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* edit_Append : Text ans Ende des schon vorhandenen Textes anfgen.  */
/*               Falls Buffer zu klein, erste Zeile lschen!          */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit   - Adresse des Editors                           */
/*           lpBuffer - Adresse des anzuhngenden Textes              */
/*           iBufSize - Anzahl der zu anzuhngenden Zeichen           */
/*                      < 0 : Stringende wird gesucht                 */
/* Ausgabe : TRUE   - Zeichen knnten eingefgt werden                */
/*           FALSE  - Textbuffer zu klein                             */
/**********************************************************************/
int edit_Append( LPEDIT lpEdit, LPBYTE lpBuffer, int iBufSize )
{
  if( iBufSize < 0 )
  {
    iBufSize = 0;
    while( lpBuffer[ iBufSize ] ) iBufSize++;
  }

  while( !_edit_Append( lpEdit, lpBuffer, iBufSize ) )
         if( !edit_DelFirstRow( lpEdit, FALSE ) ) return FALSE;
  edit_EndOfText( lpEdit, FALSE );
  edit_Home( lpEdit, TRUE );
  return TRUE;
}

/**********************************************************************/
/* edit_Leave : Editor verliert Eingabefokus                          */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit - Adresse des Editors                             */
/**********************************************************************/
void edit_Leave( LPEDIT lpEdit )
{
  lpEdit->Win.uFlags &= ~WIN_ACTIVE;           /* ACTIVE-Flag lschen */
  edit_Display( lpEdit, 0, lpEdit->Win.iH );  /* alle Zeilen anzeigen */
}

/**********************************************************************/
/* edit_Enter : Editor erhlt Eingabefokus                            */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit - Adresse des Editors                             */
/**********************************************************************/
void edit_Enter( LPEDIT lpEdit )
{
  lpEdit->Win.uFlags |= WIN_ACTIVE;             /* ACTIVE-Flag setzen */
  edit_Display( lpEdit, 0, lpEdit->Win.iH );  /* alle Zeilen anzeigen */
}

/**********************************************************************/
/* edit_ProcessChar : Tastendrcke bearbeiten                         */
/**------------------------------------------------------------------**/
/* Eingabe : lpEdit - Adresse des Editors                             */
/*           uChar  - Tastencode (auch erweiterter > 256)             */
/**********************************************************************/
void edit_ProcessChar( LPEDIT lpEdit, UINT uChar )
{ int Beep;
  BYTE bIns;

  bIns = LOBYTE( uChar );                    /* einzufgendes Zeichen */
  Beep = FALSE;

  switch( uChar )
  {
    case KBD_UP:       Beep = !edit_CRSRUp( lpEdit, TRUE );    break;
    case KBD_LEFT:     Beep = !edit_CRSRLeft( lpEdit );        break;
    case KBD_DN:       Beep = !edit_CRSRDn( lpEdit, TRUE );    break;
    case KBD_RIGHT:    Beep = !edit_CRSRRight( lpEdit );       break;
    case KBD_DEL:      Beep = !edit_DeleteChar( lpEdit );      break;
    case KBD_HOME:     edit_Home( lpEdit, TRUE );              break;
    case KBD_END:      edit_End( lpEdit );                     break;
    case KBD_PGUP:     Beep = !edit_CRSRPgUp( lpEdit );        break;
    case KBD_PGDN:     Beep = !edit_CRSRPgDn( lpEdit );        break;
    case KBD_CTRLPGDN: Beep = !edit_EndOfText( lpEdit, TRUE ); break;
    case KBD_CTRLPGUP: Beep = !edit_StartOfText( lpEdit );     break;
    case KBD_BACK:
         if( ! ( lpEdit->uFlags & EDIT_DISPLAY ) )
         {
            Beep = !edit_CRSRLeft( lpEdit );
            if( !Beep ) Beep = !edit_DeleteChar( lpEdit );
            if( Beep ) win_Beep();
         }
    break;
    case KBD_LF:
    case KBD_CR:
           if( lpEdit->uFlags & EDIT_NOCR ) break;
           bIns = '\n';
    default:
           if( ( ! ( lpEdit->uFlags & EDIT_DISPLAY ) ) && bIns )
             edit_Insert( lpEdit, &bIns, 1);
  }
  if( Beep ) win_Beep();
}

/**********************************************************************/
/* control_leave : Control und zugehriges Objekt verliert            */
/*                 Eingabefokus                                       */
/**------------------------------------------------------------------**/
/* Eingabe : lpWin - Adresse des Fensters, in dem Control angezeigt   */
/*                   wird                                             */
/*           lpControl - Adresse des Controls                         */
/**********************************************************************/
void control_Leave( LPWINDOW lpWin, LPCONTROL lpControl )
{ UINT OldFlags;
  switch( lpControl->bType )
  {
    case CTL_EDIT:
         edit_Leave( ( LPEDIT )lpControl->lpAdditional );
    break;
  }

  OldFlags = lpWin->uFlags;
  lpWin->uFlags &= ~WIN_ACTIVE;
  lpWin->iAttr &= ~0x08;
  win_PrintAt( lpWin, lpControl->iX, lpControl->iY, lpControl->lpText );
  lpWin->uFlags = OldFlags;
}

/**********************************************************************/
/* control_enter : Control und zugehriges Objekt erhlt Eingabefokus */
/**------------------------------------------------------------------**/
/* Eingabe : lpWin - Adresse des Fensters, in dem Control angezeigt   */
/*                   wird                                             */
/*           lpControl - Adresse des Controls                         */
/**********************************************************************/
void control_Enter( LPWINDOW lpWin, LPCONTROL lpControl )
{ UINT OldFlags;

  OldFlags = lpWin->uFlags;
  lpWin->uFlags |= WIN_ACTIVE;
  lpWin->iAttr |= 0x08;
  win_PrintAt( lpWin, lpControl->iX, lpControl->iY, lpControl->lpText );
  lpWin->uFlags = OldFlags ;

  switch( lpControl->bType )
  {
    case CTL_EDIT:
         edit_Enter( ( LPEDIT )lpControl->lpAdditional );
    break;
  }
}

/**********************************************************************/
/* control_ProcessChar : Control und zugehriges Objekt erhlt        */
/*                       Tastencode                                   */
/**------------------------------------------------------------------**/
/* Eingabe : lpControl - Adresse des Controls                         */
/*           uChar     - zu verarbeitender Tastencode                 */
/**********************************************************************/
void control_ProcessChar( LPCONTROL lpControl, UINT uChar )
{
  switch( lpControl->bType )
  {
    case CTL_EDIT:
         edit_ProcessChar( lpControl->lpAdditional, uChar );
  }
}
#endif

