/**********************************************************************/
/*                          F I X P A R T C . C                       */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : Zeigt die Partitionierung einer Festplatte an. */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 26.04.1989                                     */
/*    letztes Update : 12.01.1992                                     */
/*--------------------------------------------------------------------*/
/*    Speichermodell : SMALL                                          */
/*--------------------------------------------------------------------*/
/*    Aufruf         : FIXPARTC [ Laufwerksnummer ]                   */
/*                       Defaulteinstellung ist Laufwerk 0 ("C")      */
/**********************************************************************/

#include <dos.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

/*== Konstanten ======================================================*/

#define TRUE  ( 1 == 1 )
#define FALSE ( 1 == 0 )

/*== Makros ==========================================================*/

#define HI(x) ( *((BYTE *) (&x)+1) )    /* liefert HI-BYTE eines WORD */
#define LO(x) ( *((BYTE *) &x) )        /* liefert LO-BYTE eines WORD */

/*== Typdeklarationen ================================================*/

typedef unsigned char BYTE;
typedef unsigned int WORD;

typedef struct {             /* beschreibt die Position eines Sektors */
                BYTE Kopf;                       /* Schreib-/LeseKopf */
                WORD SecZyl;            /* Sektor- und Zylindernummer */
               } SECPOS;

typedef struct {                  /* Eintrag in der Partitionstabelle */
                BYTE          Status;            /* Partitions-Status */
                SECPOS        StartSec;              /* erster Sektor */
                BYTE          PartTyp;              /* Partitions-Typ */
                SECPOS        EndSec;               /* letzter Sektor */
                unsigned long SecOfs;      /* Offset des Boot-Sektors */
                unsigned long SecAnz;          /* Anzahl der Sektoren */
               } PARTENTRY;

typedef struct {                 /* beschreibt den Partititionssektor */
                BYTE      BootCode[ 0x1BE ];
                PARTENTRY PartTable[ 4 ];
                WORD      IdCode;                           /* 0xAA55 */
               } PARTSEC;

typedef PARTSEC far *PARSPTR;   /* Zeiger auf Partitionssektor im Sp. */

/**********************************************************************/
/*  ReadPartSec : Liest einen Partitionssektor von der Festplatte in  */
/*                einen Puffer                                        */
/*  Eingabe: - Laufwerk : BIOS-Code des Laufwerks (0x80, 0x81 etc.)   */
/*           - Kopf     : Nummer des Schreib-/LeseKopfs               */
/*           - SekZyl   : Sektor- und Zylindernummer im BIOS-Format   */
/*           - Buf      : Puffer, in den der Sektor geladen wird      */
/*  Ausgabe: TRUE, wenn der Sektor einwandfrei gelesen werden konnte, */
/*           sonst FALSE                                              */
/**********************************************************************/

BYTE ReadPartSec( BYTE Laufwerk, BYTE Kopf, WORD SekZyl, PARSPTR Buf )

{
 union REGS   Regs;              /* Prozessorregs fr Interruptaufruf */
 struct SREGS SRegs;

 Regs.x.ax = 0x0201;                   /* Fktnr. fr "Read", 1 Sektor */
 Regs.h.dl = Laufwerk;                      /* die weiteren Parameter */
 Regs.h.dh = Kopf;                          /* in die verschiedenen   */
 Regs.x.cx = SekZyl;                        /* Register laden         */
 Regs.x.bx = FP_OFF( Buf );
 SRegs.es  = FP_SEG( Buf );

 int86x( 0x13, &Regs, &Regs, &SRegs );  /* Festplatten-Intr. aufrufen */
 return !Regs.x.cflag;
}

/**********************************************************************/
/*  GetSecZyl: ermittelt aus der kombinierten Sektor/Zylinder-Codier- */
/*             ung des BIOS die Sektor- und Zylinder-Nummer           */
/*  Eingabe: SekZyl   : der zu dekodierende Wert                      */
/*           Sektor   : Referenz auf die Sektor-Variable              */
/*           Zylinder : Referenz auf die Zylinder-Variable            */
/*  Ausgabe: keine                                                    */
/**********************************************************************/

void GetSecZyl( WORD SekZyl, int *Sektor, int *Zylinder )

{
 *Sektor   = SekZyl & 63;                  /* Bits 6 und 7 ausblenden */
 *Zylinder = HI(SekZyl) + ( ( (WORD) LO(SekZyl) & 192 ) << 2 );
}

/**********************************************************************/
/*  ShowPartition: zeigt die Partitionierung einer Festplatte auf     */
/*                 dem Bildschirm an                                  */
/*  Eingabe: LW : Nummer des angesprochenen Festplattenlaufwerks      */
/*                (0, 1, 2 etc.)                                      */
/*  Ausgabe: keine                                                    */
/**********************************************************************/

void ShowPartition( BYTE LW )
{
 #define AP ( ParSec.PartTable[ Entry ] )

 BYTE       Kopf,                     /* Kopf der aktuellen Partition */
            Entry;                                 /* Schleifenzhler */
 int        Sektor,                   /* nehmen Sektor- und Zylinder- */
            Zylinder;                 /* Nummer auf                   */
 PARTSEC    ParSec;                  /* der aktuelle Partitionssektor */
 union REGS Regs;                /* Prozessorregs fr Interruptaufruf */

 printf("\n");
 LW |= 0x80;                  /* Laufwerksnummer fr BIOS aufbereiten */
 if ( ReadPartSec( LW, 0, 1, &ParSec ) )    /* Partitionssektor lesen */
  {                                   /* Sektor konnte gelesen werden */
   Regs.h.ah = 8;                 /* Kenndaten des Laufwerks erfragen */
   Regs.h.dl = LW;
   int86( 0x13, &Regs, &Regs );         /* Festplatten-Intr. aufrufen */
   GetSecZyl( Regs.x.cx, &Sektor, &Zylinder );
   printf( ""
           "ͻ\n");
   printf( " Laufwerk %2d:    %2d Kpfe mit jeweils %4d"
           " Zylindern zu je%3d Sektoren   \n",
           LW-0x80, Regs.h.dh+1, Zylinder, Sektor );
   printf( " Partitionstabelle im Partitionssektor     "
           "                              \n");
   printf( ""
           "͹\n");
   printf( "                              Start    "
           "     Ende     Entf.v.       \n");
   printf( "NrBootTyp                Kopf Zyl. Sek."
           "Kopf Zyl. Sek.BootSekAnzahl \n");
   printf( ""
           "͹\n");

   /*-- die Partitionstabelle durchlaufen ----------------------------*/
   for ( Entry=0; Entry < 4; ++Entry )
    {
     printf( " %d", Entry );
     if ( AP.Status == 0x80 )                     /* Partition aktiv? */
      printf(" Ja ");
     else
      printf ("Nein");
     printf("");
     switch( AP.PartTyp )                  /* Partitionstyp auswerten */
      {
       case 0x00 : printf( "nicht belegt       " );
                   break;
       case 0x01 : printf( "DOS, 12-Bit-FAT    " );
                   break;
       case 0x02 :
       case 0x03 : printf( "XENIX              " );
                   break;
       case 0x04 : printf( "DOS, 16-Bit-FAT    " );
                   break;
       case 0x05 : printf( "DOS, extended Part." );
                   break;
       case 0x06 : printf( "DOS 4.0 > 32 MB    " );
                   break;
       case 0xDB : printf( "Concurrent DOS     " );
                   break;
       default   : printf( "unbekannt (%3d)    ",
                           ParSec.PartTable[ Entry ].PartTyp );
      }

     /*-- physikalische und logische Angaben auswerten ---------------*/
     GetSecZyl( AP.StartSec.SecZyl, &Sektor, &Zylinder );
     printf( "%2d %5d  %3d ", AP.StartSec.Kopf, Zylinder, Sektor );
     GetSecZyl( AP.EndSec.SecZyl, &Sektor, &Zylinder );
     printf( "%2d %5d  %3d ", AP.EndSec.Kopf, Zylinder, Sektor );
     printf( "%6lu %6lu \n", AP.SecOfs, AP.SecAnz );
    }
   printf( ""
           "ͼ\n" );
  }
 else
  printf("Fehler beim Zugriff auf den Bootsektor!\n");
}

/***********************************************************************
*                          H A U P T P R O G R A M M                   *
***********************************************************************/

int main( int argc, char *argv[] )
{
 int Laufwerk;

 printf( "\n FIXPARTC - (c)"
         " 1989, 92 by MICHAEL TISCHER \n" );
 Laufwerk = 0;                        /* Default ist erste Festplatte */
 if ( argc == 2 )                      /* anderes Laufwerk angegeben? */
  {                                                             /* Ja */
   Laufwerk = atoi ( argv[1] );
   if ( Laufwerk == 0 && *argv[1] != '0' )
    {
     printf("\nUngltige Laufwerksnummer!");
     return( 1 );                                 /* Programm beenden */
    }
  }
 ShowPartition( (BYTE) Laufwerk );       /* Partitionssektor anzeigen */
 return( 0 );
}
