/**********************************************************************/
/*                           F M U T I L . C                          */
/*--------------------------------------------------------------------*/
/* Aufgabe: Demonstriert den Zugriff auf die FM-Synthese der          */
/*          SoundBlaster-Karte.                                       */
/*--------------------------------------------------------------------*/
/* Autor          : Michael Tischer / Bruno Jennrich                  */
/* entwickelt am  : 06.02.1994                                        */
/* letztes Update : 08.02.1994                                        */
/**********************************************************************/
#ifndef __FMUTIL_C                  /* FMUTIL.C kann #Included werden */
#define __FMUTIL_C
/*--- Include-Dateien einbinden --------------------------------------*/

#include <dos.h>
#include <stdio.h>
#include <conio.h>
#include <process.h>
#include <sys\timeb.h>

#include "types.h"
#include "sbutil.h"
#include "fmutil.h"
#include "dsputil.h"

/*--- Globale Variablen ----------------------------------------------*/

BYTE OPL2Mirror[ 2 ][ 256 ];  /* Spiegelung der Soundblaster-Register */
                                           /* Offsets der Oszilatoren */
BYTE Oszilator[ 18 ] = { 0x00,0x01,0x02,0x03,0x04,0x05,0x08,0x09,0x0a,
                         0x0b,0x0c,0x0d,0x10,0x11,0x12,0x13,0x14,0x15 };
                         /* Nummern der Modulator/Carrier Oszilatoren */
BYTE Modulator[ 9 ] = { 0,1,2,6,7,8,12,13,14 };        /* Modulatoren */
BYTE Carrier  [ 9 ] = { 3,4,5,9,10,11,15,16,17 };/* Carrier (Trger ) */
INT  Channel [ 18 ] = { 0, 1, 2, 0, 1, 2, 3, 4, 5,
                        3, 4, 5, 6, 7, 8, 6, 7, 8 };
SBBASE _FP FMBASE;

/**********************************************************************/
/* fm_Write: Schreibe Wert in Soundblaster-Register und Mirror        */
/*--------------------------------------------------------------------*/
/* Eingabe: iOpl     -  Nummer des OPL2-Chips (0,1) (nur bei OPL3)    */
/*          iReg     -  Nummer des zu beschreibenden Registers        */
/*          iVal     -  Neuer Registerwert                            */
/* Ausgabe: Erfolg der Operation                                      */
/*          TRUE  - Register konnte gesetzt werden                    */
/*          FALSE - Register nicht gesetzt (kein zweiter OPL2!)       */
/**********************************************************************/
INT fm_Write( INT iOpl, INT iReg, BYTE iVal )
{ INT iOplBase;
  if( iOpl && ( FMBASE.uDspVersion < DSP_3XX ) ) return FALSE;
                            /* zu beschreibendes Register bermitteln */
  iOplBase = FMBASE.iDspPort;
  if( iOpl ) iOplBase += 2;
  outp( iOplBase + SB_REGPORT, iReg );
                   /* Warten um "verschlucken" der Karte zu verhinden */
  inp ( iOplBase + SB_REGPORT ); 
  inp ( iOplBase + SB_REGPORT );
  inp ( iOplBase + SB_REGPORT );
  inp ( iOplBase + SB_REGPORT );
  inp ( iOplBase + SB_REGPORT );
  inp ( iOplBase + SB_REGPORT );
            
                        /* zu schreibenden Registerinhalt bermitteln */
  outp( iOplBase + SB_DATAPORT, iVal );
         /* neuen Registerwert fr Lesezugriffe in Speicher schreiben */
  if( iOpl )  OPL2Mirror[ FM_SECNDOPL2 ][ iReg ] = iVal;
  else        OPL2Mirror[ FM_FIRSTOPL2 ][ iReg ] = iVal;
  return TRUE;
}

/**********************************************************************/
/* fm_WriteBit: Einzelnes Bit in Soundblaster-Register setzen/lschen */
/*--------------------------------------------------------------------*/
/* Eingabe: iOpl     -  Nummer des OPL2-Chips (0,1) (nur bei OPL3)    */
/*          iReg     -  Nummer des zu beschreibenden Registers        */
/*          iBit     -  Nummer des zu ndernden Bits (0-7)            */
/*          bSet     -  TRUE:  Bit Setzen                             */
/*                      FALSE: Bit Lschen                            */
/* Ausgabe: Erfolg der Operation                                      */
/*          TRUE  - Register konnte gesetzt werden                    */
/*          FALSE - Register nicht gesetzt (kein zweiter OPL2!)       */
/**********************************************************************/
INT fm_WriteBit( INT iOpl, INT iReg, INT iBit, BYTE bSet )
{
   if( iOpl ) iOpl = 1;                 /* iOpl auf 0 und 1 begrenzen */
   if( bSet ) bSet = 1;                 /* bSet auf 0 und 1 begrenzen */
   return fm_Write( iOpl, iReg,
             ( BYTE ) ( ( OPL2Mirror[ iOpl ][ iReg ] &
         ( ~( 1 << iBit ) ) ) |     /* zu nderndes Bit ausblenden... */
         ( bSet << iBit ) ) );    /* und evtl. einblenden */
}

/**********************************************************************/
/* Reset_Card: Setze alle Kartenregister zurck. Wohldefinierter aus- */
/*             gangszustand!                                          */
/**********************************************************************/
VOID fm_Reset( VOID )
{
  INT i;
  for( i = 0; i <= 255; i++ )
  {
    fm_Write( FM_FIRSTOPL2, ( BYTE ) i, 0);                /* 1. OPL2 */
    fm_Write( FM_SECNDOPL2, ( BYTE ) i, 0);                /* 2. OPL2 */
  }
}

/**********************************************************************/
/* fm_SetBase : Routinen mit initialisierter SBBase-Struktur          */
/*              versorgen                                             */
/**------------------------------------------------------------------**/
/* Eingabe: pSBBASE - Adresse einer initialisierten SBBase-Struktur   */
/*          iReset   - wenn <> 0 - Soundblaster resetten              */
/**********************************************************************/
VOID fm_SetBase( PSBBASE pSBBASE, INT iReset )
{
  dsp_SetBase( pSBBASE, iReset );
  FMBASE = *pSBBASE;
  if( iReset ) fm_Reset();
}

/**********************************************************************/
/* fm_GetAdLib: Ports auf AdLib kompatible Karte durchsuchen          */
/**------------------------------------------------------------------**/
/* Eingabe: pSBBASE - Adresse einer zu initialisierenden SBBASE-      */
/*                    Struktur                                        */
/* Ausgabe: NO_ERROR - AdLib kompatible Karte gefunden                */
/*          sonst    - keine AdLib Karte                              */
/**********************************************************************/
INT fm_GetAdLib( PSBBASE pSBBASE )
{ WORD cx;

  pSBBASE->iDspPort = ADLIB_PORT;           /* AdLib Port 0x388/0x389 */
  pSBBASE->iMixPort = -1;
  pSBBASE->iMixPort = -1;
  pSBBASE->uDspVersion = 0xFFFFU;        /* wird spter initialisiert */
  pSBBASE->iDspIrq  = -1;
  pSBBASE->iDspDmaB = -1;
  pSBBASE->iDspDmaW = -1;

  fm_SetBase( pSBBASE, TRUE ); /* Basisport und Mirror initialisieren */

  fm_Write( FM_FIRSTOPL2, AL_TIMER1, 0xFF );     /* Startwert Timer 1 */
  fm_Write( FM_FIRSTOPL2, AL_TISTATE, AL_TIRESET ); /* Timer resetten */
  fm_Write( FM_FIRSTOPL2, AL_TISTATE, AL_TI1STARTSTOP );  /*  starten */

  cx = 0xFFFF;
  while( !( inp( pSBBASE->iDspPort ) & AL_STAT1OVRFLW ) && cx ) cx--;
  return cx ? NO_ERROR : ERROR;
}


/**********************************************************************/
/* fm_GetChannel : Kanalnummer eines Oszilators ermitteln             */
/*--------------------------------------------------------------------*/
/* Eingabe: o_nr  - Nummer (nicht Offset) des Oszilators              */
/* Ausgabe: Nummer des zugehrigen Kanals                             */
/**********************************************************************/
INT fm_GetChannel( INT o_nr ){ return Channel[ o_nr ]; }

/**********************************************************************/
/* fm_GetModulator : Oszilatornummer eines Kanals ermitteln           */
/*--------------------------------------------------------------------*/
/* Eingabe: ch_nr  - Nummer des Kanals                                */
/* Ausgabe: Oszilator-Nummer des zugehrigen Modulators               */
/**********************************************************************/
INT fm_GetModulator( INT ch_nr ){ return Modulator[ ch_nr ]; }

/**********************************************************************/
/* fm_GetCarrier : Oszilatornummer eines Kanals ermitteln             */
/*--------------------------------------------------------------------*/
/* Eingabe: ch_nr  - Nummer des Kanals                                */
/* Ausgabe: Oszilator-Nummer des zugehrigen Carriers                 */
/**********************************************************************/
INT fm_GetCarrier( INT ch_nr ){ return Carrier[ ch_nr ]; }

/**********************************************************************/
/* fm_SetOszilator: Setze Oszilatorparameter                          */
/*--------------------------------------------------------------------*/
/* Eingabe: iOpl       - Nummer des OPL2-Chips (nur bei OPL3!)        */
/*          iOsz       - Nummer (nicht Offset) des Oszilators         */
/*          bA         - Attack / Anschwellen ( 0 - 15 )              */
/*          bD         - Decay / Abschwellen ( 0 - 15 )               */
/*          bS         - Systain / Halten      ( 0 - 15 )             */
/*          bR         - Release / Ausklingen  ( 0 - 15 )             */
/*          bShortADSR - Hllkurvenverkrzung mit zunehmender Oktave  */
/*          bContADSR  - kontinuierliche Hllkurve an/aus             */
/*          bVibrato   - Vibrato an/aus                               */
/*          bTremolo   - Tremolo an/aus                               */
/*          bMute      - Dmpfung ( 0: laut, 63: leise )              */
/*          bHiMute    - Hochtondmpfung                              */
/*                       0: 0dB pro Oktave                            */
/*                       1: 3dB pro Oktave                            */
/*                       2: 1.5dB pro Oktave                          */
/*                       1: 6dB pro Oktave                            */
/*          bFrqFactor - Multiplikationsparameter fr Frequenz        */
/*          bWave      - Wellenform ( 0 - 3 )                         */
/* Ausgabe: keine                                                     */
/*--------------------------------------------------------------------*/
/* Info: Oszilatornummern != Oszilatoroffsets                         */
/*       Frequenzparameter ist kein linearer Faktor                   */
/**********************************************************************/
VOID fm_SetOszilator( INT iOpl, INT iOsz, BYTE bA, BYTE bD, BYTE bS,
                      BYTE bR, BYTE bShortADSR, BYTE bContADSR,
                      BYTE bVibrato, BYTE bTremolo, BYTE bMute,
                      BYTE bHiMute, BYTE bFrqFactor, BYTE bWave
                    )
{                                          /* Alle Einzel-Bits setzen */
   fm_Write( iOpl, 0x20 + Oszilator[ iOsz ],
     (BYTE)( ( OPL2Mirror[ iOpl ][ 0x20 + Oszilator[ iOsz ] ] & 0xF0 ) |
             bFrqFactor));
   fm_WriteBit( iOpl, 0x20 + Oszilator[ iOsz ], 4, bShortADSR );
   fm_WriteBit( iOpl, 0x20 + Oszilator[ iOsz ], 5, bContADSR );
   fm_WriteBit( iOpl, 0x20 + Oszilator[ iOsz ], 6, bVibrato );
   fm_WriteBit( iOpl, 0x20 + Oszilator[ iOsz ], 7, bTremolo );

                  /* Attack (Hi-Nibble) und Decay (Lo-Nibble) in Byte */
   fm_Write( iOpl, 0x60 + Oszilator[ iOsz ],
             ( BYTE ) ( bD | ( bA << 4 ) ) );
                                                   /* Dmpfung setzen */
   fm_Write( iOpl, 0x40 + Oszilator[ iOsz ],
            ( BYTE ) ( bMute | ( bHiMute << 5 ) ) );
           /* Sustain (Hi-Nibble) und Release (Lo-Nibble) in ein Byte */
   fm_Write( iOpl, 0x80 + Oszilator[ iOsz ],
            ( BYTE ) ( bR | ( bS << 4 ) ) );

   fm_WriteBit( iOpl, 0x01, 5, TRUE );    /* Wellenform nderungen erlauben */
   fm_Write( iOpl, 0xE0 + Oszilator[ iOsz ], bWave ); /* Wellenform schreiben*/
   fm_WriteBit( iOpl, 0x01, 5, FALSE );  /* Wellenform nderungen verbieten */
}

/**********************************************************************/
/* fm_SetModulator : Modulator eines Kanals programmieren             */
/*--------------------------------------------------------------------*/
/* Eingabe: iOpl       - Nummer des OPL2-Chips (nur bei OPL3!)        */
/*          iChn       - Kanalnummer                                  */
/*          bA         - Attack / Anschwellen ( 0 - 15 )              */
/*          bD         - Decay / Abschwellen ( 0 - 15 )               */
/*          bS         - Systain / Halten      ( 0 - 15 )             */
/*          bR         - Release / Ausklingen  ( 0 - 15 )             */
/*          bShortADSR - Hllkurvenverkrzung mit zunehmender Oktave  */
/*          bContADSR  - kontinuierliche Hllkurve an/aus             */
/*          bVibrato   - Vibrato an/aus                               */
/*          bTremolo   - Tremolo an/aus                               */
/*          bMute      - Dmpfung ( 0: laut, 63: leise )              */
/*          bHiMute    - Hochtondmpfung                              */
/*                       0: 0dB pro Oktave                            */
/*                       1: 3dB pro Oktave                            */
/*                       2: 1.5dB pro Oktave                          */
/*                       1: 6dB pro Oktave                            */
/*          bFrqFactor - Multiplikationsparameter fr Frequenz        */
/*          bWave      - Wellenform ( 0 - 3 )                         */
/* Ausgabe: keine                                                     */
/*--------------------------------------------------------------------*/
VOID fm_SetModulator( INT iOpl, INT iChn, BYTE bA, BYTE bD, BYTE bS,
                      BYTE bR, BYTE bShortADSR, BYTE bContADSR,
                      BYTE bVibrato, BYTE bTremolo, BYTE bMute,
                      BYTE bHiMute, BYTE bFrqFactor, BYTE bWave
                    )
{
  fm_SetOszilator( iOpl, Modulator[ iChn ], bA, bD, bS, bR,
                   bShortADSR, bContADSR, bVibrato, bTremolo,
                   bMute, bHiMute, bFrqFactor, bWave );
}

/**********************************************************************/
/* fm_SetCarrier : Carrier / Trger eines Kanals programmieren        */
/*--------------------------------------------------------------------*/
/* Eingabe: iOpl       - Nummer des OPL2-Chips (nur bei OPL3!)        */
/*          iChn       - Kanalnummer                                  */
/*          bA         - Attack / Anschwellen ( 0 - 15 )              */
/*          bD         - Decay / Abschwellen ( 0 - 15 )               */
/*          bS         - Systain / Halten      ( 0 - 15 )             */
/*          bR         - Release / Ausklingen  ( 0 - 15 )             */
/*          bShortADSR - Hllkurvenverkrzung mit zunehmender Oktave  */
/*          bContADSR  - kontinuierliche Hllkurve an/aus             */
/*          bVibrato   - Vibrato an/aus                               */
/*          bTremolo   - Tremolo an/aus                               */
/*          bMute      - Dmpfung ( 0: laut, 63: leise )              */
/*          bHiMute    - Hochtondmpfung                              */
/*                       0: 0dB pro Oktave                            */
/*                       1: 3dB pro Oktave                            */
/*                       2: 1.5dB pro Oktave                          */
/*                       1: 6dB pro Oktave                            */
/*          bFrqFactor - Multiplikationsparameter fr Frequenz        */
/*          bWave      - Wellenform ( 0 - 3 )                         */
/* Ausgabe: keine                                                     */
/*--------------------------------------------------------------------*/
/* Info: Oszilatornummern != Oszilatoroffsets                         */
/*       Frequenzparameter ist kein linearer Faktor                   */
/**********************************************************************/
VOID fm_SetCarrier( INT iOpl, INT iChn, BYTE bA, BYTE bD, BYTE bS,
                    BYTE bR, BYTE bShortADSR, BYTE bContADSR,
                    BYTE bVibrato, BYTE bTremolo, BYTE bMute,
                    BYTE bHiMute, BYTE bFrqFactor, BYTE bWave
                  )
{
  fm_SetOszilator( iOpl, Carrier[ iChn ], bA, bD, bS, bR,
                   bShortADSR, bContADSR, bVibrato, bTremolo,
                   bMute, bHiMute, bFrqFactor, bWave );
}

/**********************************************************************/
/* fm_SetChannel: Setze Kanal                                         */
/*--------------------------------------------------------------------*/
/* Eingabe: iOpl       - Nummer des OPL2-Chips (nur bei OPL3!)        */
/*          iChn       - Nummer (UND Offset) des Kanals               */
/*          bOct       - Oktave (0 - 8)                               */
/*          iFrq       - Frequenz (0 - 1023)                          */
/*          bFM        - FM-Synthese oder addierte Oszilatoren        */
/*          bFeedBack  - Modulator-Rckkopplung                       */
/* Ausgabe: keine                                                     */
/**********************************************************************/
VOID fm_SetChannel( INT iOpl, INT iChn, BYTE bOct, INT iFrq, BYTE bFM,
                    BYTE bFeedBack
                  )
{                                       /* LoByte der Frequenz setzen */
  fm_Write( iOpl, 0xA0 + iChn, ( BYTE ) ( iFrq & 0xFF  ) );
                           /* 3 obere Frequenz-Bits und Oktave setzen */
  fm_Write( iOpl, 0xB0 + iChn,
           ( BYTE ) ( ( ( iFrq >> 8 ) & 0x3 ) | ( bOct << 2 ) ) );
                      /* Oszilator-Verknpfung und Modulator-Feedback */

  if( OPL2Mirror[ 1 ][ 5 ] & 1 )      /* OPL3-Stereomodus */
  {
    fm_Write( iOpl, 0xC0 + iChn,
       ( BYTE )
       ( ( OPL2Mirror[ iOpl ][ 0xC0 ] & 0xF1 ) | ( bFeedBack << 1 ) ) );
    fm_QuadroMode( iOpl, iChn, bFM );
  }
  else
     fm_Write( iOpl, 0xC0 + iChn,
               ( BYTE ) ( ( OPL2Mirror[ iOpl ][ 0xC0 ] & 0xF0 ) |
                        ( ( bFM ? 0 : 1 ) | ( bFeedBack << 1 ) ) ) );
}

/**********************************************************************/
/* fm_SetCard: Soundkarten Parameter setzen                           */
/*--------------------------------------------------------------------*/
/* Eingabe: iOpl     - Nummer des OPL2-Chips (nur bei OPL3!)          */
/*          bVibrato - TRUE: Vibratotiefe = 14 / 100 der Kanalfrequenz*/
/*                    FALSE: Tremolotiefe = 7 / 100 der Kanalfrequenz */
/*          bTremolo - TRUE:  Tremolotiefe = 4.8 dB                   */
/*                    FALSE: Tremolotiefe = 1 dB                      */
/* Ausgabe: keine                                                     */
/*--------------------------------------------------------------------*/
/* Info: Tremolo- und Vibratotiefe nur fr alle Oszilatoren einstell- */
/*       bar                                                          */
/**********************************************************************/
VOID fm_SetCard( INT iOpl, BYTE bVibrato, BYTE bTremolo )
{
   fm_WriteBit( iOpl, 0xBD, 6, bVibrato );
   fm_WriteBit( iOpl, 0xBD, 7, bTremolo );
}

/**********************************************************************/
/* fm_PlayChannel: Kanal an- und abschalten                           */
/*--------------------------------------------------------------------*/
/* Eingabe: iOpl - Nummer des OPL2-Chips (nur bei OPL3!)              */
/*          iChn - Nummer des Kanals                                  */
/*          bOn  - TRUE: Ton anschalten, FALSE: Ton abschalten        */
/* Ausgabe: keine                                                     */
/**********************************************************************/
VOID fm_PlayChannel( INT iOpl, INT iChn, BYTE bOn )
{
  fm_WriteBit( iOpl, 0xB0 + iChn, 5, bOn );
}

/**********************************************************************/
/* Helper-Functions                                                   */
/*--------------------------------------------------------------------*/
/* Schalten Rythmusmodus bzw. Rythmusinstrument an und ab             */
/**********************************************************************/
VOID fm_PlayHiHat     ( INT iOpl, BYTE bOn )
                      { fm_WriteBit( iOpl, 0xBD, 0, bOn ); }
VOID fm_PlayTopCymbal ( INT iOpl, BYTE bOn )
                      { fm_WriteBit( iOpl, 0xBD, 1, bOn ); }
VOID fm_PlayTomTom    ( INT iOpl, BYTE bOn )
                      { fm_WriteBit( iOpl, 0xBD, 2, bOn ); }
VOID fm_PlaySnareDrum ( INT iOpl, BYTE bOn )
                      { fm_WriteBit( iOpl, 0xBD, 3, bOn ); }
VOID fm_PlayBassDrum  ( INT iOpl, BYTE bOn )
                      { fm_WriteBit( iOpl, 0xBD, 4, bOn ); }
VOID fm_PercussionMode( INT iOpl, BYTE bOn )
                      { fm_WriteBit( iOpl, 0xBD, 5, bOn ); }

/**********************************************************************/
/* fm_PollTime : Wartet angebene Anzahl von tausendstel Sekunden      */
/**------------------------------------------------------------------**/
/* Eingabe : lMilli : Anzahl abzuwartender Tausendstel                */
/**********************************************************************/
VOID fm_PollTime( LONG lMilli )
{ struct timeb tb;
  LONG lEnd, lAct;

  ftime( &tb );
  lAct = tb.time * 1000L + tb.millitm;  /* Aktuelle Uhrzeit ermitteln */
  lEnd = lAct + lMilli;                         /* End-Zeit berechnen */
  while( lAct < lEnd )                    /* Aktuelle Zeit < End-Zeit */
  {
    ftime( &tb );          /* Aktuelle Zeit in Tausendstel umrechnen: */
    lAct = tb.time * 1000L + tb.millitm;
  }
}

/*-- OPL3 - 4 Operator-Synthese --------------------------------------*/

/**********************************************************************/
/* fm_QuadroOn : Stereobetrieb der OLPs umschalten                    */
/**------------------------------------------------------------------**/
/* Eingabe : iOn - TRUE  : zweiten OPL2 aktivieren (OPL3-Modus)       */
/*                 FALSE : zweiten OPL2 deaktivieren                  */
/*                 (Kompatibilittsmodus)                             */
/* Ausgabe : TRUE  - zweiter OPL2 vorhanden                           */
/*           FALSE - kein zweiter OPL2                                */
/**------------------------------------------------------------------**/
/* Hinweis: Innerhalb der Funktion fm_Write(Bit) wird festgestellt,   */
/*          ob der zweite OPL2 berhaupt angesprochen wertden kann,   */
/*          bzw. ob die Soundblasterkarte ber einen OPL3 verfgt.    */
/**********************************************************************/
INT fm_QuadroOn( INT iOn )
{                 /* Bit 0 in Register 5' gesetzt: zweiter OPL2 aktiv */
  return fm_WriteBit( FM_SECNDOPL2, 0x05, 0, ( BYTE )iOn );
}

/**********************************************************************/
/* fm_ChannelLR : Kanal auf linken und/ode rechten Ausgang legen      */
/**------------------------------------------------------------------**/
/* Eingabe: iOpl - Nummer des OPL2-Chips (nur bei OPL3!)              */
/*          iChn - Nummer des Kanals                                  */
/*          iL   - != 0 : Kanal auf linken Ausgang legen              */
/*          iR   - != 0 : Kanal auf rechten Ausgang legen             */
/**------------------------------------------------------------------**/
/* Hinweis: Diese Funktion funktioniert nur mit einem OPL3            */
/**********************************************************************/
VOID fm_ChannelLR( INT iOpl, INT iChn, INT iL, INT iR )
{
  fm_WriteBit( iOpl, 0xC0 + iChn, 4, ( BYTE )iL );
  fm_WriteBit( iOpl, 0xC0 + iChn, 5, ( BYTE )iR );
}

/**********************************************************************/
/* fm_QuadroChannel: Anzahl der Operatoren eines Kanals festlegen     */
/**------------------------------------------------------------------**/
/* Eingabe: iOpl - Nummer des OPL2-Chips (nur bei OPL3!)              */
/*          iCH0 != 0 : Kanal 0 des OPL's hat 4 Operatoren            */
/*               == 0 : Kanal 0 des OPL's hat 2 Operatoren            */
/*          iCH1 != 0 : Kanal 1 des OPL's hat 4 Operatoren            */
/*               == 0 : Kanal 1 des OPL's hat 2 Operatoren            */
/*          iCH1 != 0 : Kanal 2 des OPL's hat 4 Operatoren            */
/*               == 0 : Kanal 2 des OPL's hat 2 Operatoren            */
/**********************************************************************/
VOID fm_QuadroChannel( INT iOpl, INT iCH0, INT iCH1, INT iCH2 )
{
  fm_WriteBit( FM_SECNDOPL2, 0x04, iOpl ? 3 + 0 : 0 + 0 , (BYTE)iCH0 );
  fm_WriteBit( FM_SECNDOPL2, 0x04, iOpl ? 3 + 1 : 0 + 1 , (BYTE)iCH1 );
  fm_WriteBit( FM_SECNDOPL2, 0x04, iOpl ? 3 + 2 : 0 + 2 , (BYTE)iCH2 );
}

/**********************************************************************/
/* fm_QuadroMode : Zellenverknpfung fr 4 Operatoren Kanle fest-    */
/*                 legen.                                             */
/**------------------------------------------------------------------**/
/* Eingabe: iOpl  - Nummer des OPL2-Chips (nur bei OPL3!)             */
/*          iChn  - Nummer des Kanals (0-2)                           */
/*          iMode - Verknpfungsmodus (0-3)                           */
/**------------------------------------------------------------------**/
/* Hinweis: Der Modus wird auf die beiden "alten" Verknpfungsbits    */
/*          der beiden 2 Operator-Kanle eines 4-Operator Kanals ver- */
/*          teilt!                                                    */
/**********************************************************************/
VOID fm_QuadroMode( INT iOpl, INT iChn, INT iMode )
{
  if( iChn < 3 )
  {
    fm_WriteBit( iOpl, 0xC0 + iChn, 0, (BYTE)(iMode & 0x01) );
    fm_WriteBit( iOpl, 0xC0 + iChn + 3, 0, (BYTE)(iMode & 0x02) );
  }
}
#endif

