/***********************************************************************
*                              I S E V C . C                           *
**--------------------------------------------------------------------**
*  Aufgabe          : Stellt fest, ob eine EGA- oder VGA-Karte in-     *
*                     stalliert ist.                                   *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    :  6.08.1990                                       *
*  letztes Update am: 14.01.1991                                       *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung       : CL /AS isevc.c                                   *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung       : ber die integrierte Entwicklungsumgebung        *
**--------------------------------------------------------------------**
*  Aufruf           : isevc                                            *
***********************************************************************/

#include <dos.h>                         /* Include-Dateien einbinden */
#include <stdarg.h>
#include <stdio.h>

/*-- Konstanten ------------------------------------------------------*/

#define EGA_MONO  0                             /* EGA an MDA-Monitor */
#define EGA_COLOR 1                             /* EGA an EGA-Monitor */
#define VGA_MONO  2              /* VGA an analogem Monochrom-Monitor */
#define VGA_COLOR 3                             /* VGA an VGA-Monitor */
#define WEDERNOCH 4                      /* keine EGA- oder VGA-Karte */

/*-- Typdeklarationen ------------------------------------------------*/

typedef unsigned char BYTE;

/***********************************************************************
*  IsEgaVga : Stellt fest, ob eine EGA- oder VGA-Karte installiert ist.*
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : eine der Konstanten EGA_MONO, EGA_COLOR etc.              *
***********************************************************************/

BYTE IsEgaVga( void )
{
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */

 Regs.x.ax = 0x1a00;              /* Funktion 1Ah gibt es nur bei VGA */
 int86( 0x10, &Regs, &Regs );
 if ( Regs.h.al == 0x1a )              /* ist die Funktion verfgbar? */
   switch ( Regs.h.bl )                         /* Ja, Code auswerten */
    {
     case  4 : return EGA_COLOR;
     case  5 : return EGA_MONO;
     case  7 : return VGA_MONO;
     case  8 : return VGA_COLOR;
     default : return WEDERNOCH;
    }
 else                                    /* Nein, aber vielleicht EGA */
  {
   Regs.h.ah = 0x12;                    /* Funktion 12h Unterfunktion */
   Regs.h.bl = 0x10;                    /* 10h aufrufen               */
   int86(0x10, &Regs, &Regs );                 /* Video-BIOS aufrufen */
   if ( Regs.h.bl != 0x10 )                                   /* EGA? */
    return Regs.h.bh == 0 ? EGA_COLOR : EGA_MONO;               /* Ja */
   else                                                       /* Nein */
    return WEDERNOCH;
  }
}

/***********************************************************************
*  H A U P T P R O G R A M M                                           *
***********************************************************************/

void main( void )
{
 printf( "ISEVP  -  (c) 1990 by MICHAEL TISCHER\n\n" );
 switch ( IsEgaVga() )
  {
   case WEDERNOCH :
     printf( "Bei der aktiven Videokarte handelt es sich " \
             "weder um eine EGA-, noch um eine VGA-Karte.");
     break;

   case EGA_MONO  :
     printf( "Es ist eine EGA-Karte an einem MDA-Monitor aktiv." );
     break;

   case EGA_COLOR :
     printf( "Es ist eine EGA-Karte an einem EGA- oder " \
             " Multiscan-Monitor aktiv." );
     break;

   case VGA_MONO  :
     printf( "Es ist eine VGA-Karte an einem analogen " \
             "Monochrom-Monitor aktiv.");
     break;

   case VGA_COLOR :
     printf( "Es ist eine VGA-Karte an einem VGA- " \
             " oder Multiscan-Monitor aktiv." );
  }
 printf( "\n\n" );
}

