/**********************************************************************/
/*                           K O N F I G C                            */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : gibt die Konfiguration des PC auf dem Bild-    */
/*                     schirm aus                                     */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 13.08.1987                                     */
/*    letztes Update : 19.10.1991                                     */
/*--------------------------------------------------------------------*/
/*    Speichermodell : SMALL                                          */
/**********************************************************************/
                                                                        
/*== Include-Dateien einbinden =======================================*/

#include <dos.h>                          /* Header-Dateien einbinden */
#include <stdio.h>
                                                                        
/*== Typedefs ========================================================*/
                                                                        
typedef unsigned char BYTE;           /* so bastelt man sich ein BYTE */
                                                                        
/*== Makros ==========================================================*/
                                                                        
#ifdef MK_FP
  #undef MK_FP
#endif

#ifdef peekb
  #undef peekb
#endif

#define MK_FP(seg, ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))
#define peekb(seg, ofs) *((BYTE far *) MK_FP(seg, ofs))
                                                                        
/*== Konstanten ======================================================*/
                                                                        
#define TRUE  ( 0 == 0 )         /* Konstanten, erleichtern das Lesen */
#define FASLE ( 0 == 1 )         /* des Programmtextes                */
                                                                        
/**********************************************************************/
/* CLS: Bildschirm lschen und Cursor in die obere linke Ecke         */
/* Eingabe : keine                                                    */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                                        
void Cls( void )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                                        
 Register.h.ah = 6;                  /* Funktionsnummer fr Scroll-UP */
 Register.h.al = 0;                            /* 0 steht fr lschen */
 Register.h.bh = 7;              /* weie Schrift auf schwarzem Grund */
 Register.x.cx = 0;                     /* obere linke Bildschirmecke */
 Register.h.dh = 24;                       /* Koordinaten der unteren */
 Register.h.dl = 79;                       /* rechten Bildschirmecke  */
 int86(0x10, &Register, &Register);  /* BIOS-Video-Interrupt aufrufen */
                                                                        
 Register.h.ah = 2;      /* Funktionsnummer fr Cursorposition setzen */
 Register.h.bh = 0;                              /* Bildschirmseite 0 */
 Register.x.dx = 0;    /* Koordinate der oberen linken Bildschirmecke */
 int86(0x10, &Register, &Register);  /* BIOS-Video-Interrupt aufrufen */
}
                                                                        
/**********************************************************************/
/* PRINTKONFIG: Ausgabe der Konfiguration eines PC                    */
/* Eingabe : keine                                                    */
/* Ausgabe : keine                                                    */
/* Info    : die Konfiguration wird in Abhngigkeit des PC-Typs aus-  */
/*           gegeben                                                  */
/**********************************************************************/
                                                                        
void PrintKonfig( void )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
 BYTE AT;                                           /* AT oder hher? */
                                                                        
 Cls();                                         /* Bildschirm lschen */
 AT = (peekb(0xF000, 0xFFFE) == 0xFC);
 printf("KONFIGC  -  (c) 1987, 91 by Michael Tischer\n\n");
 printf("Konfiguration Ihres PC\n");
 printf("----------------------------------------------------------\n");
 printf("PC-Typ                  : ");

 switch( peekb(0xF000, 0xFFFE) )           /* PC-Typ erneut ermitteln */
  {
   case 0xFF : printf("PC\n");            /* 0xFF ist ein normaler PC */
               break;
   case 0xFE : printf("XT\n");                     /* 0xFE ist ein XT */
               break;
   default   : printf("AT oder hher\n");          /* 0xFC ist ein AT */
               break;
  }
 printf("RAM-Speicher            : ");
 int86(0x12, &Register, &Register);                /* RAM-Gre holen */
 printf("%d KB\n",Register.x.ax);                     /* und ausgeben */
 if ( AT )                                      /* ist der PC ein AT? */
  {                                                             /* JA */
   Register.h.ah = 0x88;    /* Funktionsnr. fr zustz. RAM ermitteln */
   int86(0x15, &Register, &Register);              /* RAM-Gre holen */
   printf("zustzlicher RAM        : %d KB ber 1MB\n", Register.x.ax);
  }
 int86(0x11, &Register, &Register);  /* BIOS-Konfigurations-Interrupt */
 printf("Videomodus nach Start   : ");
 switch(Register.x.ax & 48)
  {
   case  0 : printf("undefiniert\n");
             break;
   case 16 : printf("40*25 Zeichen Color-Karte\n");
             break;
   case 32 : printf("80*25 Zeichen Color-Karte\n");
             break;
   case 48 : printf("80*25 Zeichen Mono-Karte\n");
             break;
  }
 printf("Diskettenlaufwerke      : %d\n", (Register.x.ax >> 6 & 3) + 1);
 printf("Serielle Schnittstellen : %d\n", Register.x.ax >> 9 & 0x03);
 printf("Parallele Schnittstellen: %d\n\n", Register.x.ax >> 14);
}
                                                                        
/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/
                                                                        
void main()
{
 PrintKonfig();                         /* die Konfiguration ausgeben */
}
                                                                        
