/***********************************************************************
*                              L A U F T C . C                         *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Programmierung von Smooth-      *
*                     Scrolling in Verbindung mit EGA- und VGA-        *
*                     Karten                                           *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    : 26.08.1990                                       *
*  letztes Update am: 14.01.1991                                       *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung       : CL /AS lauftc.c lauftca;                         *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung       : ber Projekt-Date mit folgendem Inhalt           *
*                       lauftc.c                                       *
*                       lauftca.obj                                    *
**--------------------------------------------------------------------**
*  Aufruf           : lauftc                                           *
***********************************************************************/

#include <dos.h>                         /* Include-Dateien einbinden */
#include <stdarg.h>
#include <string.h>
#include <stdio.h>

#ifdef __TURBOC__                        /* Kompilierung mit Turbo C? */
  #define CLI()         disable()
  #define STI()         enable()
  #define outpw( p, w ) outport( p, w )
  #ifndef inp
    #define outp( p, b )  outportb( p, b )
    #define inp( p )      inportb( p )
  #endif
#else                                /* Nein, mit QuickC 2.0 oder MSC */
  #include <conio.h>
  #define MK_FP(seg,ofs) ((void far *)\
                         (((unsigned long)(seg) << 16) | (ofs)))
  #define CLI()          _disable()
  #define STI()          _enable()
#endif

#define FAST       2        /* Konstanten fr SPEED in ShowLaufText() */
#define MEDIUM     1
#define SLOW       0

#define FARBE      0x5E                        /* Gelb auf Lila Grund */
#define FARBE1     0x5F                        /* Wei auf Lila Grund */
#define BREITE     8                  /* Breite der Zeichen in Pixeln */
#define HOEHE      14              /* Hhe der Zeichen in Scan-Zeilen */
#define SPALTEN    216              /* Spalten pro Zeile im Video-RAM */
#define BANDSIZE   10800                        /* Gre eines Bandes */
#define BAENDER    3                             /* Anzahl der Bnder */
#define MAXLEN     61                  /* maximale Anzahl von Zeichen */
#define STARTZ     5     /* Startzeile der Zeichen auf dem Bildschirm */

#define CrtAttr    0x3C0          /* Crt-Attribut-Controller-Register */
#define CrtStatus  0x3da                                /* Statusport */
#define CrtAdr     0x3d4                        /* Monitor Adressport */

#define TRUE       ( 0 == 0 )
#define FALSE      ( 0 == 1 )

#define EGA       0                                   /* Karten-Typen */
#define VGA       1
#define WEDERNOCH 2

typedef unsigned char BYTE;               /* wir basteln uns ein BYTE */
typedef unsigned int WORD;                               /* dito WORD */
typedef BYTE BOOL;

typedef WORD VRAM[BAENDER][25][SPALTEN];      /* Aufbau des Video-RAM */
typedef VRAM far *VPTR;                   /* FAR-Zeiger auf Video-RAM */

typedef BYTE ZDEF[256][14];                 /* Aufbau des Zeichensatz */
typedef ZDEF far *ZPTR;                     /* Zeiger auf Zeichensatz */

/*-- externe Funktionen ----------------------------------------------*/

extern void far * getfontptr( void );           /* Assembler-Funktion */

/*-- globale Variablen -----------------------------------------------*/

VPTR vp;                                       /* Zeiger in Video-RAM */

/***********************************************************************
*  SetOrigin : Legt den sichtbaren Teil des Video-RAM durch die Pro-   *
*              grammierung des Video-Controllers fest.                 *
**--------------------------------------------------------------------**
*  Eingabe :   Band    = Nummer des anzuzeigenden Bands (1-5)          *
*              Spalte, = Nummer der Spalte und Zeile, die in der ober- *
*              Zeile     linken Bildschirmecke angezeigt werden sollen *
*              PixX,   = Pixelloffsets                                 *
*              PixY                                                    *
*  Ausgabe :   keine                                                   *
***********************************************************************/

void SetOrigin( BYTE band, BYTE spalte, BYTE zeile,
                BYTE pixx, BYTE pixy)
{
 int offset;                        /* Offset fr Start des Video-RAM */

 offset = ( BANDSIZE >> 1) * band + zeile * SPALTEN + spalte;

 /*- auf ernmeuten vertikalen Rcklauf und dessen Ende warten --------*/

 while ( !(( inp(CrtStatus) & 8 ) == 8 ))
  ;
 while ( !(( inp(CrtStatus) & 8 ) == 0 ))
  ;

 /* Offset fr Start d. Video-RAM in Register 0x0C und 0x0D schreiben */
 /* wird allerdings erst nach dem nchsten vertikalen Rcklauf glitg */

 CLI();                                             /* Interrupts aus */
 outpw( CrtAdr, ( offset & 0xFF00 ) + 0x0c );
 outpw( CrtAdr, ( (BYTE) offset << 8 ) + 0x0d );
 STI();                                       /* Interrupts wieder an */

 /*- auf Anfang des vertikalen Rcklaufs warten, damit der neue Pixel-*/
 /*- offset gesetzt werden kann und gleichzeitig die neue Start-     -*/
 /*- adresse gltig wird                                             -*/

 while ( !(( inp( CrtStatus ) & 8 ) == 8 ))
  ;

 /*- Pixeloffsets in das Register 0x08 bzw. in das Register 0x13 des -*/
 /*- Attribut-Controllers schreiben                                  -*/

 CLI();                                             /* Interrupts aus */
 outpw( CrtAdr, ( pixy << 8 ) + 0x08 );
 outp( CrtAttr, 0x13 | 0x20 );
 outp( CrtAttr, pixx );
 STI();                                       /* Interrupts wieder an */
}

/***********************************************************************
*  PrintChar : Baut ein Zeichen auerhalb des sichtbaren Bereichs des  *
*              Video-RAMs auf.                                         *
**--------------------------------------------------------------------**
*  Eingabe :   Zeichen = das aufzubauende Zeichen                      *
*              Band    = Nummer des Bands (0-4)                        *
*              Spalte  = Spalte im Video-RAM, an der das Zeichen be-   *
*                        ginnen soll                                   *
*  Info    :   Das aufgebaute Zeichen kann durch Aufruf von Smooth-    *
*              Left in den sichtbaren Teil des Bildschirms geholt wer- *
*              den.                                                    *
*              Das Punktmuster des Zeichens wird dem 14*8 Punkte Zei-  *
*              chesatz im EGA-/VGA-ROM entnommen.                      *
***********************************************************************/

void PrintChar( char zeichen, BYTE band, BYTE spalte )
{
 char ch;                            /* nimmt jeweils einen Punkt auf */
 BYTE i, k,                                        /* Schleifenzhler */
      Maske;                      /* Bitmaske fr Aufbau des Zeichens */
 static ZPTR fptr = (ZPTR) 0;        /* Zeiger auf Zeichensatz im ROM */

 if ( fptr == (ZPTR) 0 )   /* Zeiger auf Zeichensatz schon ermittelt? */
  fptr = getfontptr();  /*Nein, mit Hilfe der Assembler-Funktion laden*/

 /*- das Zeichen Zeile fr Zeile durchlaufen und aufbauen ------------*/

 for ( i = 0; i < HOEHE; ++i )
  {
   Maske = (*fptr)[zeichen][i];     /* Bitmuster fr eine Zeile holen */
   for ( k = 0; k < BREITE; ++k )   /* die einzelnen Bits durchlaufen */
    {
     ch = ( Maske & 128 ) ? 219 : 32;
     (*vp)[band][STARTZ+i][spalte*BREITE+k] = (BYTE) ch+( FARBE << 8 );
     Maske <<= 1;                          /* nchstes Bit bearbeiten */
    }
  }
}

/***********************************************************************
*  IsEgaVga : Stellt fest, ob eine EGA- oder VGA-Karte installiert ist.*
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : EGA, VGA oder WEDERNOCH                                   *
***********************************************************************/

BYTE IsEgaVga( void )
{
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */

 Regs.x.ax = 0x1a00;              /* Funktion 1Ah gibt es nur bei VGA */
 int86( 0x10, &Regs, &Regs );
 if ( Regs.h.al == 0x1a )              /* ist die Funktion verfgbar? */
  return VGA;
 else
  {
   Regs.h.ah = 0x12;                    /* Funktion 12h Unterfunktion */
   Regs.h.bl = 0x10;                    /* 10h aufrufen               */
   int86(0x10, &Regs, &Regs );                 /* Video-BIOS aufrufen */
   return ( Regs.h.bl != 0x10 ) ? EGA : WEDERNOCH;
  }
}

/***********************************************************************
*  ShowLaufText : Bringt einen Text als Lauftext auf den Bildschirm    *
**--------------------------------------------------------------------**
*  Eingabe :  ltext = der Lauftext als String                          *
*             speed = Geschwindigkeit (SLOW, MEDIUM oder SLOW)         *
*             vk    = Videokarte (EGA oder VGA)                        *
***********************************************************************/

void ShowLaufText( char * ltext, BYTE speed, BYTE vk )
{
 int        band,                    /* das aktuell durchlaufene Band */
            spalte,                     /* aktuell bearbeitete Spalte */
            index,               /* Index in den aufzubauenden String */
            len,                    /* Lnge des auszugebenden Textes */
            i, k,                                  /* Schleifenzhler */
            pixx;                        /* horizontaler Panning-Wert */
 WORD far  *wptr;             /* Zeiger zum Durchlaufen des Video-RAM */
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */

 static BYTE steptable[2][3][10] =
        {
          {                                      /* EGA-Schrittweiten */
            {   0,   1,   2,   3,   4,   5,   6,   7, 255, 255 },
            {   0,   2,   4,   6, 255, 255, 255, 255, 255, 255 },
            {   0,   4, 255, 255, 255, 255, 255, 255, 255, 255 }
          },
          {                                      /* VGA-Schrittweiten */
            {   8,   0,   1,   2,   3,   4,   5,   6,   7, 255 },
            {   8,   2,   5, 255, 255, 255, 255, 255, 255, 255 },
            {   8,   3, 255, 255, 255, 255, 255, 255, 255, 255 }
          }
        };

 vp = MK_FP( 0xB800, 0x0000 );         /* Zeiger auf Video-RAM setzen */

 /*- gesamten Video-RAM mit Leerzeichen fllen -----------------------*/

 for ( index = 0; index < BAENDER; ++index )
  for ( i = 0; i < 25; ++i )
   for ( k = 0; k < SPALTEN; ++k )
    (*vp)[index][ i ][ k ] = ( FARBE << 8 ) + 32;

 /*- horizontale Fhrungsschienen malen ------------------------------*/

 for ( k = 0; k < BAENDER; ++k )
  for ( i = 0; i < SPALTEN; ++i )
   {
    (*vp)[ k ][ STARTZ-2 ][ i ] = (BYTE) '' + ( FARBE1 << 8 );
    (*vp)[ k ][ STARTZ + HOEHE + 2][ i ] = (BYTE) '' + ( FARBE1 << 8 );
   }

 /*- blinkenden Cursor aus dem Bildschirm entfernen ------------------*/

 Regs.h.ah = 0x02;               /* Funktionsnummer fr "Set Cursor"  */
 Regs.h.bh = 0;                                    /* Bildschirmseite */
 Regs.x.dx = 0;                                         /* Koordinate */
 int86( 0x10, &Regs, &Regs );

 /*- Farbe fr den Bildschirmrahmen festlegen ------------------------*/

 Regs.h.ah = 0x10;    /* Funktionsnummer fr "Rahmenfarbe festlegen"  */
 Regs.h.al = 0x01;                            /* Unterfunktionsnummer */
 Regs.h.bh = FARBE >> 4;                               /* Rahmenfarbe */
 int86( 0x10, &Regs, &Regs );

 /*- Anzahl der Spalten pro Zeile im Video-RAM auf SPALTEN festlegen -*/

 outpw( CrtAdr, ( ( SPALTEN >> 1 ) << 8 ) + 0x13 );

 /*-- den Lauftext komplett im Video-RAM aufbauen --------------------*/

 if ( ( len = strlen( ltext ) ) > MAXLEN )         /* String zu lang? */
  *(ltext + ( len = MAXLEN )) = '\0';                   /* Ja, krzen */

 for ( spalte = band = index = 0; index < len; )
  {
   PrintChar( *(ltext+index++), band, spalte++ ); /* Zeichen aufbauen */
   if ( spalte >= SPALTEN / BREITE )                  /* Bandwechsel? */
    {                                                           /* Ja */
     spalte = 0;                       /* wieder in Spalte 1 beginnen */
     ++band;                                   /* in das nchste Band */
     index -= 80 / BREITE;                /* Zeichen eine Page zurck */
    }
  }

  /*-- den Lauftext schrittweise von rechts nach links ber den ------*/
  /*-- Bildschirm laufen lassen                                 ------*/

  for ( spalte = band = 0 , i = (len - ( 80 / BREITE )) * BREITE;
        i > 0;
        --i )
   {
    for ( k = 0; ( pixx = steptable[vk][speed][k]) != 255 ; ++k )
      SetOrigin( band, spalte, 0, pixx, 0 );

    if ( ++spalte == SPALTEN - 80 )                   /* Bandwechsel? */
     {                                                          /* Ja */
      spalte = 0;                      /* wieder in Spalte 0 beginnen */
      ++band;                                  /* Band inkrementieren */
     }
   }

 /*- wieder 80 Spalten pro Zeile im Video-RAM einstellen -------------*/

 outpw( CrtAdr, ( 40 << 8 ) + 0x13 );

 SetOrigin( 0, 0, 0, 8, 0 );             /* Einstellungen auf Default */

 /*- Cursor wieder auf dem Bildschirm erscheinen lassen --------------*/

 Regs.h.ah = 0x02;               /* Funktionsnummer fr "Set Cursor"  */
 Regs.h.bh = 0;                                    /* Bildschirmseite */
 Regs.x.dx = 0;                                         /* Koordinate */
 int86( 0x10, &Regs, &Regs );

 /*- Farbe fr den Bildschirmrahmen zurcksetzen ---------------------*/

 Regs.h.ah = 0x10;    /* Funktionsnummer fr "Rahmenfarbe festlegen"  */
 Regs.h.al = 0x01;                            /* Unterfunktionsnummer */
 Regs.h.bh = 0;                              /* Rahmen wieder schwarz */
 int86( 0x10, &Regs, &Regs );

 /*-- Bildschirm lschen ---------------------------------------------*/

 for ( wptr = (WORD far *) vp, i = 80*25; i-- ; )
  *wptr++ = 0x0720;
}

/*--------------------------------------------------------------------*/
/*-- H A U P T P R O G R A M M                                      --*/
/*--------------------------------------------------------------------*/

void main( void )
{
 BYTE vk;                         /* Art der installierten Videokarte */

 if ( ( vk = IsEgaVga() ) == WEDERNOCH )
  printf( "LAUFTC  -  (c) 1990 by MICHAEL TISCHER\n" \
          "Achtung! Keine EGA- oder VGA-Karte installiert.\n" );
 else
  ShowLaufText( "+++ PCIntern  (c) 1987-1991 by Data Becker +++ " \
                "          ", FAST, vk );
}
