/**********************************************************************/
/*                               L E D C                              */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : Setzt die verschiedenen Bits im BIOS-Tastatur- */
/*                     Flag und lt dadurch die LEDs auf der AT-     */
/*                     Tastatur aufleuchten                           */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 22.08.1988                                     */
/*    letztes Update :  3.01.1992                                     */
/*--------------------------------------------------------------------*/
/*    Speichermodell : SMALL                                          */
/**********************************************************************/
                                                                        
/*== Include-Dateien einbinden =======================================*/
                                                                        
#include <stdio.h>
#include <dos.h>
#include <bios.h>
                                                                        
/*== Makros ==========================================================*/
                                                                        
#ifndef MK_FP                    /* wurde MK_FP noch nicht definiert? */
  #define MK_FP(seg, ofs) \
                 ((void far *) ((unsigned long) (seg)<<16|(ofs)))
#endif
                                                                        
/*-- BIOS_KBF erstellt einen Pointer auf das BIOS-Tastatur-Flag ------*/

#define BIOS_KBF ((unsigned far *) MK_FP(0x40, 0x17))

#define TICKS(ms) ((ms*10+549) / 550 )   /*Millsek. in Ticks umrechnen*/

/*== Konstanten ======================================================*/
                                                                        
#define SCRL  16                                   /* Scroll-Lock-Bit */
#define NUML  32                                      /* Num-Lock-Bit */
#define CAPL  64                                     /* Caps-Lock-Bit */
#define INS  128                                        /* Insert-Bit */
                                                                        
#ifdef __TURBOC__                         /* Definitionen fr TURBO C */
   #define GetBiosTime(x)    ( x = biostime( 0, NULL ) )
#else                        /* Definitionen fr Microsoft C-Compiler */
   #define GetBiosTime(x)    ( _bios_timeofday( _TIME_GETCLOCK, &x) )
#endif

/**********************************************************************/
/*  Delay:  Hlt die Programmausfhrung ungeachtet der Geschwin-      */
/*          digkeit des Systems fr ein bestimmtes Zeitintervall an   */
/*  Eingabe  : PAUSE = Lnge des Zeitintervalls in Ticks              */
/*  Ausgabee : keine                                                  */
/*  Info     : ein Tick entspricht 1/18,2 Sekunden                    */
/**********************************************************************/

void delay( unsigned int pause )
{
 long aktzeit,                                       /* aktuelle Zeit */
      zielzeit;                                /* zu erreichende Zeit */

 if ( pause )                                       /* Pause nicht 0? */
  {                                                           /* Nein */
   GetBiosTime( zielzeit );
   zielzeit += (long) pause;                    /* Zielzeit errechnen */

   do                            /* Wartschleife, aktuelle Zeit holen */
    GetBiosTime( aktzeit );
   while ( aktzeit < zielzeit );                    /* Zeit erreicht? */
  }                                           /* Ja, Funktion beenden */
}

/***********************************************************************
*  Funktion         : S E T _ F L A G                                  *
**--------------------------------------------------------------------**
*  Aufgabe          : Setzt einzelne Bits bzw. Flags im BIOS-Tastatur- *
*                     Flag                                             *
*  Eingabe-Parameter: FLAG = die zu setzenden Bits bzw. Flags.         *
*  Return-Wert      : keiner                                           *
***********************************************************************/
                                                                        
void set_flag( unsigned flag )
{
 union REGS regs;                  /* nimmt die Prozessorregister auf */
                                                                        
 *BIOS_KBF |= flag;  /* die angegebenen Bits im BIOS-Keyb.Flag setzen */
 regs.h.ah = 1;                   /* Funktionsnr.: Zeichen vorhanden? */
 int86(0x16, &regs, &regs);       /* BIOS-Tastatur-Interrupt aufrufem */
}
                                                                        
/***********************************************************************
*  Funktion         : C L R _ F L A G                                  *
**--------------------------------------------------------------------**
*  Aufgabe          : Lscht einzelne Bits bzw. Flags im BIOS-Tastatur *
*                     Flag                                             *
*  Eingabe-Parameter: FLAG = die zu lschenden Bits bzw. Flags.        *
*  Return-Wert      : keiner                                           *
***********************************************************************/
                                                                        
void clr_flag( unsigned flag )
{
 union REGS regs;                  /* nimmt die Prozessorregister auf */
                                                                        
 *BIOS_KBF &= ~flag;             /* Bits im BIOS-Keyb.Flag ausblenden */
 regs.h.ah = 1;                   /* Funktionsnr.: Zeichen vorhanden? */
 int86(0x16, &regs, &regs);       /* BIOS-Tastatur-Interrupt aufrufem */
}
                                                                        
/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/
                                                                        
void main()
{
 unsigned i;                                       /* Schleifenzhler */
                                                                        
 printf( "LEDC  -  (c) 1988, 92 by Michael Tischer\n\n");
 printf( "Achten Sie auf die LEDs Ihrer Tastatur!\n");
                                                                        
 for (i=0; i<10; ++i)              /* die Schleife 10 mal durchlaufen */
  {
   set_flag( CAPL );                               /* CAPS anschalten */
   delay( TICKS(100) );                   /* 100 Millisekunden warten */
   clr_flag( CAPL );                       /* CAPS wieder ausschalten */
   set_flag( NUML);                                 /* NUM anschalten */
   delay( TICKS(100) );                   /* 100 Millisekunden warten */
   clr_flag( NUML );                        /* NUM wieder ausschalten */
   set_flag( SCRL);                         /* SCROLL-LOCK anschalten */
   delay( TICKS(100) );                   /* 100 Millisekunden warten */
   clr_flag( SCRL );                /* SCROLL-LOCK wieder ausschalten */
  }
                                                                        
 for (i=0; i<10; ++i)              /* die Schleife 10 mal durchlaufen */
  {
   set_flag(CAPL | SCRL | NUML);                /* alle drei Flags an */
   delay( TICKS(500) );                  /* 200 Milli-Sekunden warten */
   clr_flag(CAPL | SCRL | NUML);             /* alle Flags wieder aus */
   delay( TICKS(500) );                  /* 200 Milli-Sekunden warten */
  }
}
