/**********************************************************************/
/*                                M F 2 C                             */
/**------------------------------------------------------------------**/
/*    Demonstriert die Abfrage der zustzlichen Tasten einer MF-II-   */
/*    Tastatur.                                                       */
/**------------------------------------------------------------------**/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  :  1.01.1992                                     */
/*    letztes Update :  1.01.1992                                     */
/**********************************************************************/

/*== Include-Dateien einbinden =======================================*/
                                                                        
#include <stdio.h>
#include <dos.h>

/*== Typedefs ========================================================*/
                                                                        
typedef unsigned char BYTE;               /* wir basteln uns ein BYTE */
typedef unsigned int WORD;
                                                                        
/*== Konstanten ======================================================*/
                                                                        
#define TRUE  ( 0 == 0 )         /* Konstanten, erleichtern das Lesen */
#define FALSE ( 0 == 1 )         /* des Programmtextes                */

/*== Bildschirmroutinen fr Microsoft C ==============================*/

#ifndef __TURBOC__                                    /* Microsoft C? */

  /********************************************************************/
  /* Gotoxy        : setzt den Cursor                                 */
  /* Eingabe       : Koordinaten des Cursors                          */
  /* Ausgabe       : keine                                            */
  /********************************************************************/

  void gotoxy( int x, int y )
  {
   union REGS regs;                           /* zum Interrupt-Aufruf */

   regs.h.ah = 0x02;          /* Funktionsnummer fr Interrupt-Aufruf */
   regs.h.bh = 0;                                            /* Farbe */
   regs.h.dh = y - 1;
   regs.h.dl = x - 1;
   int86( 0x10, &regs, &regs );                    /* InterruptAufruf */
  }

  /********************************************************************/
  /* clrscr        : lscht den Bildschirm                            */
  /* Eingabe       : keine                                            */
  /* Ausgabe       : keine                                            */
  /********************************************************************/

  void clrscr( void )
  {
   union REGS regs;                           /* zum Interrupt-Aufruf */

   regs.h.ah = 0x07;          /* Funktionsnummer fr Interrupt-Aufruf */
   regs.h.al = 0x00;
   regs.h.ch = 0;
   regs.h.cl = 0;
   regs.h.dh = 24;
   regs.h.dl = 79;
   int86( 0x10, &regs, &regs );                    /* InterruptAufruf */
   gotoxy( 1, 1 );                                   /* Cursor setzen */
  }

#endif

/**********************************************************************/
/* HexByte : Verwandelt ein Byte in einen zweiziffrigen Hex-String    */
/* Eingabe : WERT = das umzuwandelnde Byte                            */
/* Ausgabe : der zweiziffrige Hex-String                              */
/**********************************************************************/

char *HexByte( BYTE wert )
{
 char HexDigits[16] = "0123456789ABCDEF";
 static char dummy[3] = "00";

 dummy[0] = HexDigits[ wert >> 4 ];             /* die beiden Nibbles */
 dummy[1] = HexDigits[ wert & 0x0F ];           /* nach Hex wandeln   */
 return dummy;
}

/**********************************************************************/
/* TestMF: Prft, ob die erweiterten BIOS-Funktionen zur Abfrage der  */
/*         MF-II-Tastatur zur Verfgung stehen                        */
/* Eingabe : keiner                                                   */
/* Ausgabe : TRUE, wenn die Funktionen zur Verfgung stehen, sonst    */
/*           FALSE                                                    */
/**********************************************************************/

int TestMF( void )
{
 union REGS regs;                             /* zum Interrupt-Aufruf */

 regs.x.ax = 0x1200;    /* erweiterte Status-Fkt fr MF-II-Tastaturen */
 int86( 0x16, &regs, &regs );
 return ( regs.x.ax != 0x1200 );  /*AX=0x1200 : Funktion nicht prsent*/
}

/**********************************************************************/
/* GetMFKey : Fragt eine Taste mit Hilfe der erweiterten Tastatur-    */
/*            funktion 0x10 ab.                                       */
/* Eingabe : keiner                                                   */
/* Ausgabe : der zurckgelieferte Tastencode                          */
/**********************************************************************/

WORD GetMFKey( void )
{
 union REGS regs;                             /* zum Interrupt-Aufruf */

 regs.h.ah = 0x10;   /* erweiterte Lese-Funktion fr MF-II-Tastaturen */
 int86( 0x16, &regs, &regs );
 return regs.x.ax;                        /* Tastencode zurckliefern */
}

/**********************************************************************/
/*                     H A U P T P R O G R A M M                      */
/**********************************************************************/

void main( void )
{
 WORD taste;

 clrscr();
 printf( "MF2C  -  (c) 1992 by Michael Tischer\n\n" );
 if ( TestMF() )
  {
   printf( "Die BIOS-Erweiterungen fr MF-II-Tastaturen wurden " \
           "entdeckt! \n\nBettigen Sie nun beliebige " \
           "Tasten und Tastenkombinationen, deren Codes Sie \n" \
           "in Erfahrung bringen mchten.\n\n" \
            "Die Bettigung von <Esc> beendet das Programm.\n\n" );

   do                                              /* Eingabeschleife */
    {
     taste = GetMFKey();                               /* Taste holen */
     printf( "Scan : %s ", HexByte((BYTE) (taste >> 8)) );
     printf( "ASCII: %s", HexByte((BYTE) (taste & 255)) );
     if ( ((taste & 255) == 0xe0) && ((taste & 65280 ) != 0 ) )
      printf( " <---- MF-II-Taste" );
     printf( "\n" );
    }
   while ( taste != 0x011b );               /* wiederholen bis ESCAPE */
   printf( "\n\n" );
  }
 else
  printf( "Keine BIOS-Erweiterungen fr MF-II-Tastaturen prsent!");
}
