/***********************************************************************
*                          M I K A D O C . C                           *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert den Einsatz des 512-Zeichen-Modus   *
*                     bei EGA- und VGA-Karten am Beispiel einer Rou-   *
*                     tine zur Darstellung eines Grafikfensters inner- *
*                     halb des Textmodus                               *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    :  2.04.1990                                       *
*  letztes Update am: 14.01.1991                                       *
***********************************************************************/

/*-- Konstanten definieren und Include-Dateien einbinden -------------*/

#include <dos.h>                         /* Include-Dateien einbinden */
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

#ifdef __TURBOC__                        /* Kompilierung mit Turbo C? */
  #define CLI()         disable()
  #define STI()         enable()
  #define outpw( p, w ) outport( p, w )
  #ifndef inp
    #define outp( p, b )  outportb( p, b )
    #define inp( p )      inportb( p )
  #endif
#else                                /* Nein, mit QuickC 2.0 oder MSC */
  #include <conio.h>
  #define random(x)      (rand() % ( x + 1 ))
  #define MK_FP(seg,ofs) ((void far *)\
                         (((unsigned long)(seg) << 16) | (ofs)))
  #define CLI()          _disable()
  #define STI()          _enable()
#endif

#define EGAVGA_SEQUENCER 0x3C4          /* Sequencer Adre-/Datenport */
#define EGAVGA_MONCTR    0x3D4                  /* Monitor-Controller */
#define EGAVGA_GRAPHCTR  0x3CE  /*Graphics-Controller Adre-/Datenport*/

#define CHAR_BREITE      8
#define CHAR_BYTES       32
#define MIKADOS          5  /* Anzahl der gleichz. sichtbaren Mikados */

#define TRUE  ( 0 == 0 )
#define FALSE ( 0 == 1 )

#define BLACK         0x00                           /* Farbattribute */
#define BLUE          0x01
#define GREEN         0x02
#define CYAN          0x03
#define RED           0x04
#define MAGENTA       0x05
#define BROWN         0x06
#define LIGHTGREY     0x07
#define GREY          0x01
#define LIGHTBLUE     0x09
#define LIGHTGREEN    0x0A
#define LIGHTCYAN     0x0B
#define LIGHTRED      0x0C
#define LIGHTMAGENTA  0x0D
#define YELLOW        0x0E
#define WHITE         0x0F

/*-- Typdeklarationen ------------------------------------------------*/

typedef unsigned char BYTE;
typedef BYTE BOOL;

typedef BYTE PALARY[16];                /* ein Satz Paletten-Register */

/*-- globale Variablen -----------------------------------------------*/

BYTE far *vioptr = (BYTE far *) 0xb8000000,       /* Zeiger auf V-RAM */
     far *fontptr;                      /* Zeiger auf den Grafik-Font */

BYTE CharHoehe,
     lenx;                    /* Breite des Grafikfensters in Zeichen */
int  xmax,                /* Max-Pixel-Koordinaten des Grafikfensters */
     ymax;

/***********************************************************************
*  IsEgaVga : Stellt fest, ob eine EGA- oder VGA-Karte installiert ist.*
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : EGA, VGA oder WEDERNOCH                                   *
***********************************************************************/

BYTE IsEgaVga( void )
{
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */

 Regs.x.ax = 0x1a00;              /* Funktion 1Ah gibt es nur bei VGA */
 int86( 0x10, &Regs, &Regs );
 if ( Regs.h.al == 0x1a )              /* ist die Funktion verfgbar? */
  {                                                    /* Ja, ist VGA */
   CharHoehe = 16;                            /* Zeichenhoehe bei VGA */
   return 1;
  }
 else
  {
   CharHoehe = 14;                            /* Zeichenhoehe bei EGA */
   Regs.h.ah = 0x12;                    /* Funktion 12h Unterfunktion */
   Regs.h.bl = 0x10;                    /* 10h aufrufen               */
   int86(0x10, &Regs, &Regs );                 /* Video-BIOS aufrufen */
   return Regs.h.bl != 0x10;
  }
}

/***********************************************************************
*  SetCursor : legt die Position des blinkenden Bildschirmcursors fest *
**--------------------------------------------------------------------**
*  Eingabe-Parameter: SPALTE = die neue Cursor-Spalte (0-79)           *
*                     ZEILE  = die neue Cursor-Zeile (0-24)            *
*  Return-Wert      : keiner                                           *
***********************************************************************/

void SetCursor( BYTE spalte, BYTE zeile )
{
 union REGS regs;               /* Prozessorregs. fr Interruptaufruf */

 regs.h.ah = 2;                     /* Funktionsnummer fr Set Cursor */
 regs.h.bh = 0;                /* auf die Bildschirmseite 0 zugreifen */
 regs.h.dh = zeile;                                   /* Zeile merken */
 regs.h.dl = spalte;                                 /* Spalte merken */
 int86(0x10, &regs, &regs);              /* BIOS-Video-Intr. aufrufen */
}

/***********************************************************************
*  PrintfAt : gibt einen formatierten String an einer beliebigen Bild- *
*             schirmposition aus                                       *
**--------------------------------------------------------------------**
*  Eingabe-Parameter: SPALTE = die Ausgabeposition                     *
*                     ZEILE                                            *
*                     FARBE  = das Attribut fr die Zeichen            *
*                     STRING = Pointer auf den String                  *
*  Return-Wert      : keiner                                           *
*  Info             : Diese Funktion darf nur aufgerufen werden, nach- *
*                     dem zuvor die Existenz einer EGA- oder VGA-Karte *
*                     festgestellt wurde                               *
***********************************************************************/

void PrintfAt( BYTE spalte, BYTE zeile, BYTE farbe, char * string, ... )
{
 va_list parameter;              /* Parameter-Liste fr VA_... Macros */
 char ausgabe[255],                 /* Puffer fr formatierten String */
      *ausptr;
 BYTE far *vptr;                           /* Zeiger in den Video-RAM */

 va_start( parameter, string );                /* Parameter umwandeln */
 vsprintf( ausgabe, string, parameter );               /* formatieren */

 vptr = (BYTE far *) MK_FP( 0xB800, spalte*2+zeile*160 );

 for ( ausptr = ausgabe; *ausptr ; )            /* String durchlaufen */
  {
   *vptr++ = *(ausptr++);           /* Zeichen in Video-Ram schreiben */
   *vptr++ = farbe;                /* Attribut in Video-Ram schreiben */
  }
}

/***********************************************************************
*  ClrScr: lscht den Bildschirm                                       *
**--------------------------------------------------------------------**
*  Eingabe-Parameter: FARBE  = das Attribut fr die Zeichen            *
*  Return-Wert      : keiner                                           *
***********************************************************************/

void ClrScr( BYTE farbe )
{
 BYTE far *vptr;                           /* Zeiger in den Video-RAM */
 int  count = 2000;               /* Anzahl der zu lschenden Zeichen */

 vptr = (BYTE far *) MK_FP( 0xB800, 0 ); /*Zeiger auf Video-RAM setzen*/

 for ( ; count--; )                      /* den Video-RAM durchlaufen */
  {
   *vptr++ = ' ';                   /* Zeichen in Video-Ram schreiben */
   *vptr++ = farbe;                /* Attribut in Video-Ram schreiben */
  }
}

/***********************************************************************
*  SetCharWidth: Legt die Breite der Zeichen bei VGA-Karten auf 8 oder *
*                9 Punkte fest.                                        *
**--------------------------------------------------------------------**
*  Eingabe : BREITE = Breite der Zeichen (8 oder 9)                    *
***********************************************************************/

void SetCharWidth( BYTE breite )
{
 union REGS Regs;            /* Prozessorregister zum Interruptaufruf */
 unsigned char x;              /* nimmt Wert fr Misc-Output-Reg. auf */

 Regs.x.bx = ( breite == 8 ) ? 0x0001 : 0x0800;

 x = inp( 0x3CC ) & (255-12);    /* horizontale Auflsung von 720 auf */
 if ( breite == 9 )              /* 640 Punkte umschalten und umge-   */
  x |= 4;                        /* kehrt                             */
 outp( 0x3C2, x);

 CLI();  /* Sequencer von acht auf neun Punkte schalten und umgekehrt */
 outpw( EGAVGA_SEQUENCER, 0x0100 );
 outpw( EGAVGA_SEQUENCER, 0x01 + ( Regs.h.bl << 8 ) );
 outpw( EGAVGA_SEQUENCER, 0x0300 );
 STI();

 Regs.x.ax = 0x1000;               /* Bildschirm horizontal justieren */
 Regs.h.bl = 0x13;
 int86( 0x10, &Regs, &Regs );
}

/***********************************************************************
*  SelectMaps : Whlt die Zeichenstze aus, zwischen denen ber das    *
*               Bit 3 im Attribut-Byte der Zeichen ausgewhlt wird.    *
**--------------------------------------------------------------------**
*  Eingabe : MAP0 = Nummer des ersten Zeichensatzes  (Bit 3 = 0 )      *
*            MAP1 = Nummer des zweiten Zeichensatzes (Bit 3 = 1 )      *
*  Info    : - Bei einer EGA-Karte knnen die Zeichenstze 0 bis 3,    *
*              bei einer VGA-Karte die Zeichenstze 0 bis 7 gewhlt    *
*              werden                                                  *
***********************************************************************/

void SelectMaps( BYTE map0, BYTE map1)
{
 union REGS Regs;            /* Prozessorregister zum Interruptaufruf */

 Regs.x.ax = 0x1103;   /* Character-Map-Select-Register programmieren */
 Regs.h.bl = ( map0 & 3 ) + ( ( map0 & 4 ) << 2 ) +
             ( ( map1 & 3 ) << 2 ) + ( ( map1 & 4 ) << 3 );
 int86( 0x10, &Regs, &Regs );                        /* BIOS aufrufen */
}

/***********************************************************************
*  GetFontAccess: Erlaubt den direkten Zugriff auf die zweite Memory-  *
*                 map in der die Zeichenstze gelagert werden, ber    *
*                 die Adresse A000:0000                                *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Info    : - Nach dem Aufruf dieser Prozedur ist kein Zugriff auf    *
*              den Video-RAM bei B800:0000 mehr mglich!               *
***********************************************************************/

void GetFontAccess( void )
{
 static unsigned SeqRegs[4] = { 0x0100, 0x0402, 0x0704, 0x0300 },
                 GCRegs[3]  = { 0x0204, 0x0005, 0x0006 };
 BYTE i;                                           /* Schleifenzhler */

 CLI();                                     /* keine Interrupts jetzt */

 for ( i=0; i<4; ++i )            /* die Register des Sequencer laden */
  outpw( EGAVGA_SEQUENCER, SeqRegs[ i ] );

 for ( i=0; i<3; ++i ) /* die Register des Graphics-Controllers laden */
  outpw( EGAVGA_GRAPHCTR, GCRegs[ i ] );

 STI();                                 /* Interrupts wieder zulassen */
}

/***********************************************************************
*  ReleaseFontAccess: Gibt den Zugriff auf den Video-RAM bei B800:000  *
*                     wieder frei, versperrt damit aber den Zugang zu  *
*                     den Zeichenstzen in der Memory-Page #2.         *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************/

void ReleaseFontAccess( void )
{
 static unsigned SeqRegs[4] = { 0x0100, 0x0302, 0x0304, 0x0300 },
                 GCRegs[3]  = { 0x0004, 0x1005, 0x0E06 };
 BYTE i;                                           /* Schleifenzhler */

 CLI();                                     /* keine Interrupts jetzt */

 for ( i=0; i<4; ++i )            /* die Register des Sequencer laden */
  outpw( EGAVGA_SEQUENCER, SeqRegs[ i ] );

 for ( i=0; i<3; ++i ) /* die Register des Graphics-Controllers laden */
  outpw( EGAVGA_GRAPHCTR, GCRegs[ i ] );

 STI();                                 /* Interrupts wieder zulassen */
}

/***********************************************************************
*  ClearGraphArea: Lscht den Grafikbereich, indem die Zeichenmuster   *
*                  der darin enthaltenen Zeichen auf 0 gesetzt werden. *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************/

void ClearGraphArea( void )
{
 int zeichen,                          /* die zu durchlauf}en Zeichen */
     zeile;                /* Zeile innerhalb des jeweiligen Zeichens */

 for ( zeichen = 0; zeichen < 256; ++zeichen ) /* Zeichen durchlaufen */
  for ( zeile = 0; zeile < CharHoehe; ++ zeile )    /* Zeilen durchl. */
   *(fontptr+zeichen*CHAR_BYTES+zeile) = 0;       /* und auf 0 setzen */
}

/***********************************************************************
*  InitGraphArea: Bereitet einen Bildschirmbereich auf die Ausgabe von *
*                 grafischen Darstellungen vor.                        *
**--------------------------------------------------------------------**
*  Eingabe : X     = Startspalte des Bereichs (1-80)                   *
*            Y     = Startzeile des Bereichs (1-25)                    *
*            XLEN  = Breite des Bereichs in Zeichen                    *
*            YLEN  = Tiefe des Bereichs in Zeichen                     *
*            MAP   = Nummer des Grafik-Zeichensatzes                   *
*            FARBE = Farbe des Grafikbereichs (0 bis 7 oder 0xFF)      *
*  Info    : - Wird als Farbe der Wert 0xFF angegeben, wird ein fort-  *
*              laufender Farbcode erzeugt, der fr den 'Mikado'-Effekt *
*              antwortlich ist                                         *
***********************************************************************/

void InitGraphArea( BYTE x, BYTE y, BYTE xlen, BYTE ylen, BYTE map,
                    BYTE farbe )
{
 unsigned offset;                          /* Offset in den Video-RAM */
 int      spalte, zeile;                        /* Schleifenvariablen */
 BYTE     zcode;                           /* fortlauf}er Zeichencode */

 if ( xlen * ylen > 256 )                         /* Bereich zu gro? */
  printf( "Fehler: Der Grafikbereich darf maximal 256 Zeichen " \
          "umfassen!\n" );
 else
  {
   if ( CharHoehe == 16 )                                     /* VGA? */
    SetCharWidth( 8 );                 /* Ja, Zeichenbreite umstellen */
   SelectMaps( 0, map );                    /* Zeichenstze auswhlen */
   xmax = xlen*CHAR_BREITE;        /* Max-Pixel-Koordinaten berechnen */
   ymax = ylen*CharHoehe;
   lenx = xlen;
   fontptr = MK_FP( 0xA000, map * 0x4000 );  /* Zeiger auf Grafik-Map */
   GetFontAccess();           /* Zugriff auf Zeichenstze ermglichen */
   ClearGraphArea();                           /* Zeichensatz lschen */
   ReleaseFontAccess();      /* Zugriff auf Video-RAM wieder erlauben */

   /*-- den Grafikbereich mit fortlaufenden Zeichencodes fllen ------*/

   zcode = 0;
   for ( zeile = ylen-1; zeile >= 0; --zeile )      /* Zeilen durchl. */
    for ( spalte = 0; spalte < xlen; ++spalte )    /* Spalten durchl. */
     {                             /* Zeichencode und Attribut setzen */
      offset = ((zeile+y)*80+spalte+x) << 1;   /* Offset in Video-RAM */
      *(vioptr+offset) = zcode;                  /* Zeichen schreiben */
      *(vioptr+offset+1) = ( farbe == 0xff ) ? ( zcode % 6 ) + 1 + 8
                                             : farbe | 0x08;
      ++zcode;                                /* nchster Zeichencode */
    }
  }
}

/***********************************************************************
*  CloseGraphArea: Beendet die Arbeit mit dem Grafikbereich.           *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************/

void CloseGraphArea( void )
{
 ReleaseFontAccess();              /* Zugriff auf Video-RAM freigeben */
 SelectMaps( 0, 0 );                  /* immer Zeichensatz 0 anzeigen */
 if ( CharHoehe == 16 )                                       /* VGA? */
  SetCharWidth( 9 );                   /* Ja, Zeichenbreite umstellen */
}

/***********************************************************************
*  SetPixel: Setzt oder lscht einen Pixel innerhalb des Grafikfensters*
**--------------------------------------------------------------------**
*  Eingabe : X,Y   = Pixelkoordinate (0-...)                           *
*            ON    = TRUE fr setzen und FALSE fr lschen             *
***********************************************************************/

void SetPixel( int x, int y, BOOL on )
{
 BYTE charnum,             /* Zeichencode fr Zeichen ber Koordinate */
      linenr,                         /* Zeile innerhalb des Zeichens */
      far *bptr;

 if ( ( x < xmax ) && ( y < ymax ) )              /* Koordinate o.k.? */
  {                          /* Ja, Zeichennummer und Zeile berechnen */
   charnum = ((x / CHAR_BREITE) + (y / CharHoehe * lenx));
   linenr  = CharHoehe - ( y % CharHoehe ) - 1;
   bptr = fontptr + charnum * CHAR_BYTES + linenr;
   if ( on )                          /* Zeichen setzen oder lschen? */
    *bptr = *bptr | ( 1 << (CHAR_BREITE - 1 - ( x % CHAR_BREITE ) ) );
   else
    *bptr = *bptr & !( 1 << (CHAR_BREITE - 1 - ( x % CHAR_BREITE ) ) );
  }
}

/***********************************************************************
*  Line: Zieht eine Linie innerhalb des Grafikfensters nach dem Bresen-*
*        ham-Algorithmus                                               *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endkoordinate                                    *
*            ON     = TRUE fr Punkte setzen und FALSE fr P. lschen  *
***********************************************************************/

/*-- Unterfunktion, vertauscht zwei Integer-Variablen ----------------*/

void SwapInt( int *i1, int *i2 )
{
 int dummy;

 dummy = *i2;
 *i2   = *i1;
 *i1   = dummy;
}

/*-- Hauptprozedur ---------------------------------------------------*/

void Line( int x1, int y1, int x2, int y2, BOOL on )
{
 int d, dx, dy,
     aincr, bincr,
     xincr, yincr,
     x, y;

 if ( abs(x2-x1) < abs(y2-y1) )       /* ber X- oder Y-Achse laufen? */
  {                                               /* ber die Y-Achse */
   if ( y1 > y2 )                                    /* y1 grer y2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   xincr = ( x2 > x1 ) ?  1 : -1;       /* Spalteninkrement festlegen */

   dy = y2 - y1;
   dx = abs( x2-x1 );
   d  = 2 * dx - dy;
   aincr = 2 * (dx - dy);
   bincr = 2 * dx;
   x = x1;
   y = y1;

   SetPixel( x, y, on );                       /* ersten Punkt setzen */
   for (y=y1+1; y<= y2; ++y )         /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       x += xincr;
       d += aincr;
      }
     else
      d += bincr;
     SetPixel( x, y, on );
    }
  }
 else                                             /* ber die X-Achse */
  {
   if ( x1 > x2 )                                    /* x1 grer x2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   yincr = ( y2 > y1 ) ? 1 : -1;         /* Zeileninkrement festlegen */

   dx = x2 - x1;
   dy = abs( y2-y1 );
   d  = 2 * dy - dx;
   aincr = 2 * (dy - dx);
   bincr = 2 * dy;
   x = x1;
   y = y1;

   SetPixel( x, y, on );                       /* ersten Punkt setzen */
   for (x=x1+1; x<=x2; ++x )          /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       y += yincr;
       d += aincr;
      }
     else
      d += bincr;
     SetPixel( x, y, on );
    }
  }
}

/***********************************************************************
*  SetPalCol: definiert eine Farbe aus der 16-teiligen-Farbpalette     *
*             oder die Farbe des Bildschirmrahmens (Overscan-Color)    *
**--------------------------------------------------------------------**
*  Eingabe : RegNr = Nummer des Paletten-Registers (0 bis 15) oder 16  *
*                    fr die Farbe des Bildschirmrahmens               *
*            Col   = Farbwert im Bereich 0 bis 15                      *
***********************************************************************/

void SetPalCol( BYTE RegNr, BYTE Col )
{
 union REGS Regs;            /* Prozessorregister zum Interruptaufruf */

 Regs.x.ax = 0x1000;         /* Video-Funktion 10h, Unterfunktion 00h */
 Regs.h.bh = Col;                                         /* Farbwert */
 Regs.h.bl = RegNr;        /* Registernummer des Attribut-Controllers */
 int86( 0x10, &Regs, &Regs );        /* BIOS-Video-Interrupt aufrufen */
}

/***********************************************************************
*  SetPalAry: installiert eine neue Farbpalette mit 16 Farben, ohne    *
*             allerdings die Farbe des Bildschirmrahmens zu verndern. *
**--------------------------------------------------------------------**
*  Eingabe : NewColPtr = Zeiger auf Paletten-Array vom Typ PALARY      *
***********************************************************************/

void SetPalAry( BYTE *NewColPtr )
{
 BYTE i;                                           /* Schleifenzhler */

 for (i = 0; i < 16; ++i )    /* die 16 Eintrge im Array durchlaufen */
  SetPalCol( i, NewColPtr[i] );          /* jeweils eine Farbe setzen */
}

/***********************************************************************
*  GetPalCol: ermittelt den Inhalt eines Paletten-Registers            *
**--------------------------------------------------------------------**
*  Eingabe : RegNr = Nummer des Paletten-Registers (0 bis 15) oder 16  *
*                    fr die Farbe des Bildschirmrahmens               *
*  Ausgabe : der Farbwert                                              *
*  Info    : Bei der EGA-Karte ist ein Auslesen der Paletten-Register  *
*            nicht mglich, deshalb wird hier von der normalen Ein-    *
*            stellung der Paletten-Register ausgegangen und der jeweils*
*            entsprechende Wert zurckgeliefert.                       *
***********************************************************************/

BYTE GetPalCol( BYTE RegNr )
{
 union REGS Regs;            /* Prozessorregister zum Interruptaufruf */

 if ( CharHoehe == 14 )                                 /* EGA-Karte? */
  return RegNr;      /*Ja, kein Auslesen der Paletten-Register mglich*/
 else                                                    /* Nein, VGA */
  {
   Regs.x.ax = 0x1007;       /* Video-Funktion 10h, Unterfunktion 07h */
   Regs.h.bl = RegNr;      /* Registernummer des Attribut-Controllers */
   int86( 0x10, &Regs, &Regs );      /* BIOS-Video-Interrupt aufrufen */
   return Regs.h.bh;       /* hier steht der Inhalt des Pal-Registers */
  }
}

/***********************************************************************
*  GetPalAry: ermittelt die Inhalt der 16-Paletten-Register und trgt  *
*             sie in ein Array des Aufrufers ein.                      *
**--------------------------------------------------------------------**
*  Eingabe : ColAryPtr = Zeiger auf Paletten-Array vom Typ PALARY, in  *
*                        das die ermittelten Farben eingatragen werden *
***********************************************************************/

void GetPalAry( BYTE *ColAryPtr )
{
 BYTE i;                                           /* Schleifenzhler */

 for (i = 0; i < 16; ++i )    /* die 16 Eintrge im Array durchlaufen */
  ColAryPtr[i] = GetPalCol( i );      /* jeweils eine Farbe ermitteln */
}

/***********************************************************************
*  Mikado: Demonstriert die Arbeit mit den verschiedenen Routinen aus  *
*          diesem Programm                                             *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************/

void Mikado( void )
{
 typedef struct {                /* nimmt Koordinaten einer Linie auf */
                 int x1, y1,
                     x2, y2;
                } LINIE;

 static PALARY NewCols =
        {    /*--------- Farben fr die normalen Textzeichen ---------*/
          BLACK,                               /* vormals ... schwarz */
          BLUE,                                /*                blau */
          GREEN,                               /*                grn */
          RED,                                 /*              kobalt */
          CYAN,                                /*                 rot */
          MAGENTA,                             /*             violett */
          YELLOW,                              /*               braun */
          WHITE,                               /*            hellgrau */
          /*------------- hier {nen die Grafik-Farben ----------------*/
          LIGHTBLUE,                           /* vormals  dunkelgrau */
          LIGHTGREEN,                          /*            hellblau */
          LIGHTRED,                            /*            hellgrn */
          LIGHTCYAN,                           /*       helles kobalt */
          LIGHTMAGENTA,                        /*             hellrot */
          BLUE,                                /*         hellviolett */
          YELLOW,                              /*                gelb */
          WHITE };                             /*               weiss */

 int    i, j,                                      /* Schleifenzhler */
        first,                   /* Array-Index des jngesten Mikados */
        last;                     /* Array-Index des ltesten Mikados */
 BOOL   clear;                                     /* Mikados lschen */
 LINIE  lar[MIKADOS];                                 /* Mikado-Array */
 PALARY OldCols;                        /* nimmt die alten Farben auf */

 GetPalAry( OldCols );       /* bisherige Farbeinstellungen ermitteln */
 SetPalAry( NewCols );               /* neue Farbpalette installieren */
 /*TextColor( 7 );
 TextBackGround( 1 );
 GotoXY(1,1); */                 /* und mit Zeichen aus dem Standard- */
 ClrScr( 0x07 );                                /* Bildschirm lschen */
 for (i=0; i<25; ++i )           /* Zeichensatz fllen                */
  for (j=0; j<80; ++j )
   PrintfAt( j, i, 0x07, "%c", 32 + (((int) i*80+j) % 224) );

 /*-- Grafikbereich initialisieren und Mikados fallen lassen ---------*/

 PrintfAt( 27,6, 0x70, "       M I K A D O       " );
 SetCursor(27,6);
 InitGraphArea( 27, 7, 25, 10, 1, 0xFF );
 GetFontAccess();                  /* Zugriff auf Zeichensatz sichern */

 clear = FALSE;                         /* noch keine Mikados lschen */
 first = 0;                      /* mit erster Array-Position starten */
 last = 0;
 do
  {                                                /* Mikado-Schleife */
   if (first == MIKADOS )                             /* Wrap-Around? */
    first = 0;
   lar[first].x1 = random( xmax-1 );               /* Mikado erzeugen */
   lar[first].x2 = random( xmax-1 );
   lar[first].y1 = random( ymax-1 );
   lar[first].y2 = random( ymax-1 );
   Line( lar[first].x1, lar[first].y1,                /* und zeichnen */
         lar[first].x2, lar[first].y2, TRUE );
   if ( ++first == MIKADOS )                        /* schon lschen? */
    clear = TRUE;
   if ( clear )                                     /* jetzt lschen? */
    {                                                           /* Ja */
     Line( lar[last].x1, lar[last].y1,
           lar[last].x2, lar[last].y2, FALSE );
     if ( ++last == MIKADOS )
      last = 0;
    }
  }
 while (!kbhit());                 /* wiederholen, bis Taste gedrckt */
 getch();                      /* Taste aus Tastatur-Puffer entfernen */

 /*-- Programm beenden -----------------------------------------------*/

 CloseGraphArea();
 SetPalAry( OldCols );        /* alte Farbpalette wieder restaurieren */
 SetCursor(0,24);
 printf( "\nJetzt ist wieder der normale Zeichensaz aktiv.\n" );
}

/*--------------------------------------------------------------------*/
/*-- H A U P T P R O G R A M M ---------------------------------------*/
/*--------------------------------------------------------------------*/

void main()
{
 if ( IsEgaVga() )       /* ist eine EGA- oder VGA-Karte installiert? */
  Mikado();                                   /* Ja, Demo durchfhren */
 else                   /* Nein, Programm kann nicht gestartet werden */
  printf( "Achtung! Keine EGA- oder VGA-Karte installiert." );
}








