/**********************************************************************/
/*                   M I X U T I L  . C                               */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Stellt Funktionen zur Programmierung des Sound-  */
/*                   blaster Mixers zur Verfgung.                    */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 20.03.1994                                       */
/*  letztes Update : 25.03.1994                                       */
/*                                                                    */
/*  Info           : mix_???  - Gelten fr alle Mixer-Versionen       */
/*                   mix3_??? - ... fr Mixer CT1345 ( ab DSP V3.00 ) */
/*                   mix4_??? - ... fr Mixer CT1745 ( ab DSP V4.00 ) */
/* COMPILER: Borland C++ 3.1                                          */
/**********************************************************************/
#ifndef __MIXUTIL_C                     /* Kann auch #Included werden */
#define __MIXUTIL_H

/*-- Include-Dateien einbinden ---------------------------------------*/

#include <dos.h>
#include <stdlib.h>
#include <conio.h>

#include "sbutil.h"
#include "mixutil.h"

/*-- Globale Modul-Variablen -----------------------------------------*/

SBBASE _FP MIXBASE;                        /* Port Adresse des Mixers */

BYTE bMix3DACStereo,     /* Diese Variablen werden zum Retten einiger */
     bMix3DACFilter,     /* Mixer-Werte verwendet.                    */
     bMix3ADCFilter;

BYTE bMix4SourceL,
     bMix4SourceR;

/* Die folgenden Tabellen dienen zur Laustrke Einstellung.           */
/* Fr jede Mgliche Quelle werden Register und einzustellende        */
/* Laustrke (links, rechts, beide) vermerkt.                         */
BYTE bMix3VolTable[ MAX_SRC ][ 2 ] = { { MIX3_CDVOL, CH_BOTH },
                                       { MIX3_LINEVOL, CH_BOTH },
                                       { MIX3_VOICEVOL, CH_BOTH },
                                       { MIX3_MASTERVOL, CH_BOTH },
                                       { MIX3_MIDIVOL, CH_BOTH },
                                       { MIX3_MICVOL, CH_MAX },
                                       { 0, 0 }, /* Speaker */

                                       { MIX3_CDVOL, CH_LEFT },
                                       { MIX3_LINEVOL, CH_LEFT },
                                       { MIX3_VOICEVOL, CH_LEFT },
                                       { MIX3_MASTERVOL, CH_LEFT },
                                       { MIX3_MIDIVOL, CH_LEFT },

                                       { MIX3_CDVOL, CH_RIGHT },
                                       { MIX3_LINEVOL, CH_RIGHT },
                                       { MIX3_VOICEVOL, CH_RIGHT },
                                       { MIX3_MASTERVOL, CH_RIGHT },
                                       { MIX3_MIDIVOL, CH_RIGHT } };

BYTE bMix4VolTable[ MAX_SRC ][ 2 ] = { { MIX4_CDVOL_L, CH_BOTH },
                                       { MIX4_LINEVOL_L, CH_BOTH },
                                       { MIX4_VOICEVOL_L, CH_BOTH },
                                       { MIX4_MASTERVOL_L, CH_BOTH },
                                       { MIX4_MIDIVOL_L, CH_BOTH },
                                       { MIX4_MICVOL, CH_LEFT },
                                       { MIX4_PCSPEAKERVOL, CH_LEFT },

                                       { MIX4_CDVOL_L, CH_LEFT },
                                       { MIX4_LINEVOL_L, CH_LEFT },
                                       { MIX4_VOICEVOL_L, CH_LEFT },
                                       { MIX4_MASTERVOL_L, CH_LEFT },
                                       { MIX4_MIDIVOL_L, CH_LEFT },

                                       { MIX4_CDVOL_L, CH_RIGHT },
                                       { MIX4_LINEVOL_L, CH_RIGHT },
                                       { MIX4_VOICEVOL_L, CH_RIGHT },
                                       { MIX4_MASTERVOL_L, CH_RIGHT },
                                       { MIX4_MIDIVOL_L, CH_RIGHT } };

/* Der CT1745-Mixer erlaubt das An- und Abschalten mehrerer           */
/* Aufnahme- und Wiedergabequellen. Das folgende Array vermerkt fr   */
/* jede mgliche Quelle die entsprechenden Schalterbits.              */
BYTE bMix4SourceBits[ MAX_SRC ] = {
/* CD */                               MIX4_CD_L | MIX4_CD_R,
/* LINE */                             MIX4_LINE_L | MIX4_LINE_R,
/* VOICE ( nicht schaltbar ) */        0,
/* MASTER ( nicht schaltbar ) */       0,
/* MIDI */                             MIX4_MIDI_L | MIX4_MIDI_R,
/* MIC */                              MIX4_MIC,
/* PCSPEAKER ( nicht schaltbar ) */    0,

/* CD_L */                             MIX4_CD_L,
/* LINE_L */                           MIX4_LINE_L,
/* VOICE_L */                          0,
/* MASTER_L */                         0,
/* MIDI_L */                           MIX4_MIDI_L,

/* CD_R */                             MIX4_CD_R,
/* LINE_R */                           MIX4_LINE_R,
/* VOICE_R */                          0,
/* MASTER_R */                         0,
/* MIDI_R */                           MIX4_MIDI_R };

/**********************************************************************/
/* mix_SetBase : Setzt Port-Adresse                                   */
/**------------------------------------------------------------------**/
/* Eingabe : pSBBASE - Adresse einer initialisierten SBBase-Struktur  */
/*           iReset   - <> 0 : Mixer soll zurckgesetzt werden        */
/**------------------------------------------------------------------**/
/* Info    : - Bevor eine der folgenden Funktionen aufgerufen werden  */
/*             darf, mu mit dieser Funktion die Mixer-Adresse        */
/*             angegeben worden sein!                                 */
/**********************************************************************/
WORD mix_SetBase( PSBBASE pSBBASE, WORD iReset )
{
  if( pSBBASE )
    if( pSBBASE->iMixPort != -1 )
    {
      MIXBASE = *pSBBASE;
      if( iReset ) mix_Reset();
      return NO_ERROR;
    }
  return ERROR;
}

/**********************************************************************/
/* mix_Write : Byte an Mixer-Register bermitteln                     */
/**------------------------------------------------------------------**/
/* Eingabe : iReg  - Nummer des zu ndernden Registers                */
/*           iData - Wert, der ins Register geschrieben werden soll   */
/**********************************************************************/
VOID mix_Write( WORD iReg, WORD iData )
{                               /* Zu beschreibendes Register angeben */
   outp( MIXBASE.iMixPort + MIX_REGISTERPORT, ( BYTE ) iReg );
                                    /* Neuen Registerwert bermitteln */
   outp( MIXBASE.iMixPort + MIX_DATAPORT,     ( BYTE ) iData );
}

/**********************************************************************/
/* mix_Read : Byte aus Mixer-Register lesen                           */
/**------------------------------------------------------------------**/
/* Eingabe : iReg  - Nummer des zu lesenden Registers                 */
/* Ausgabe : Wert des Mixer-Registers                                 */
/**********************************************************************/
WORD mix_Read( WORD iReg )
{                                      /* Zu lesendes Register angeben */
   outp( MIXBASE.iMixPort + MIX_REGISTERPORT, ( BYTE )iReg );
                              /* Registerwert an AUfrufer bermitteln */
   return inp( MIXBASE.iMixPort + MIX_DATAPORT );
}

/**********************************************************************/
/* mix_Reset : Mixer zurcksetzen                                     */
/**********************************************************************/
VOID mix_Reset( VOID )
{
  mix_Write( MIX_RESET, 0 );
}

/**********************************************************************/
/* mix3_SetADCFilter : ADC-Filter an- oder abschalten                 */
/**------------------------------------------------------------------**/
/* Eingabe : iState - == 0: Filter abschalten                         */
/*                    <> 0: Filter anschalten                         */
/**------------------------------------------------------------------**/
/* Info    : - Das Bit, welches den Filter kontrolliert ist ein       */
/*             AUS-Schalter. Bei gesetztem Bit ist der Filter daher   */
/*             ausgeschaltet!                                         */
/*             ADC = Analog/Digital Converter (Aufnahme)              */
/**********************************************************************/
VOID mix3_SetADCFilter( WORD iState )
{  BYTE bIn;

   bIn = ( BYTE )mix_Read( MIX3_ADCSTATE );  /* Alten Reg.-Wert lesen */
                           /* Filter-Bit setzen / lschen (LOW-Aktiv) */
   mix_Write( MIX3_ADCSTATE,
              iState ? ( bIn & ~MIX3_ADCFILTEROFF ) :
                       ( bIn |  MIX3_ADCFILTEROFF ) );
}

/**********************************************************************/
/* mix3_GetADCFilter : Zustand des ADC-Filters ermitteln              */
/**------------------------------------------------------------------**/
/* Ausgabe : == 0 - Filter abgeschaltet                               */
/*           <> 0 - Filter angeschaltet                               */
/**********************************************************************/
WORD mix3_GetADCFilter( VOID )
{                                /* Achtung! Filter Bit ist LOW-Aktiv */
  return ( ( mix_Read( MIX3_ADCSTATE ) & MIX3_ADCFILTEROFF ) == 0 );
}

/**********************************************************************/
/* mix3_SetDACFilter : DAC-Filter an- oder abschalten                 */
/**------------------------------------------------------------------**/
/* Eingabe : iState - == 0 : Filter abschalten                        */
/*                    <> 0 : Filter anschalten                        */
/**------------------------------------------------------------------**/
/* Info :      DAC : Digital/Analog Converter (Wiedergabe)            */
/**********************************************************************/
VOID mix3_SetDACFilter( WORD iState )
{  BYTE bIn;


   bIn = ( BYTE )mix_Read( MIX3_DACSTATE );  /* Alten Reg.-Wert lesen */
              /* Filter-Bit setzen/lschen (Filter-Bit ist LOW-Aktiv) */
   mix_Write( MIX3_DACSTATE,
              iState ? ( bIn & ~MIX3_DACFILTEROFF ) :
                       ( bIn |  MIX3_DACFILTEROFF ) );
}

/**********************************************************************/
/* mix3_GetDACFilter : Zustand des DAC-Filters ermitteln              */
/**------------------------------------------------------------------**/
/* Ausgabe : == 0 - Filter abgeschaltet                               */
/*           <> 0 - Filter angeschaltet                               */
/**********************************************************************/
WORD mix3_GetDACFilter( VOID )
{                                /* Achtung! Filter-Bit ist LOW-Aktiv */
  return ( ( mix_Read( MIX3_DACSTATE ) & MIX3_DACFILTEROFF ) == 0 );
}

/**********************************************************************/
/* mix3_SetDACStereo : Stereo-Wiedergabe (DAC) an- oder abschalten    */
/**------------------------------------------------------------------**/
/* Eingabe : iState - == 0: Stereo-Wiedergabe abschalten => Mono      */
/*                    <> 0: Stereo-Wiedergabe anschalten              */
/**********************************************************************/
VOID mix3_SetDACStereo( WORD iState )
{  BYTE bIn;

   bIn = ( BYTE )mix_Read( MIX3_DACSTATE );  /* Alten Reg.-Wert lesen */
   mix_Write( MIX3_DACSTATE,
              iState ? ( bIn |  MIX3_STEREOON ) :
                       ( bIn & ~MIX3_STEREOON ) );
}

/**********************************************************************/
/* mix3_GetDACStereo : Zustand der Stereo-Wiedergabe ermitteln        */
/**------------------------------------------------------------------**/
/* Ausgabe : == 0 - Stereo-Wiedergabe abgeschaltet                    */
/*           <> 0 - Stereo-Wiedergabe angeschaltet                    */
/**********************************************************************/
WORD mix3_GetDACStereo( VOID )
{
  return ( ( mix_Read( MIX3_DACSTATE ) & MIX3_STEREOON ) != 0 );
}

/**********************************************************************/
/* mix3_SetADDACLowPass : LowPass Filter fr Aufnahme setzen          */
/**------------------------------------------------------------------**/
/* Eingabe : iState : == 0: 3.2 kHz LowPass Filter                    */
/*                    <> 0: 8.8 kHz LowPass Filter                    */
/**------------------------------------------------------------------**/
/* Info : - Der LowPass Filter wird nur dann verwendet, wenn der ADC  */
/*          oder DAC Filter angeschaltet werden. Ansonsten wird der   */
/*          LowPass Filter bei Aufnahme und WIedergabe ignoriert.     */
/**********************************************************************/
VOID mix3_SetADDACLowPass( WORD iState )
{  BYTE bIn;

   bIn = ( BYTE )mix_Read( MIX3_ADCSTATE );  /* Alten Reg.-Wert lesen */
   mix_Write( MIX3_ADCSTATE,
              iState ? ( bIn |  MIX3_LOWPASS88 ) :
                       ( bIn & ~MIX3_LOWPASS88 ) );
}

/**********************************************************************/
/* mix3_GetADDACLowPass : Ermittle verwendeten LowPass Filter         */
/**------------------------------------------------------------------**/
/* Ausgabe : == 0: 3.2 kHz LowPass Filter wird verwendet              */
/*           <> 0: 8.8 kHz LowPass Filter wird verwendet              */
/**********************************************************************/
WORD mix3_GetADDACLowPass( VOID )
{
  return ( ( mix_Read( MIX3_ADCSTATE ) & MIX3_LOWPASS88 ) != 0 );
}

/**********************************************************************/
/* mix3_PrepareForStereo : Mixer fr Stereo-Aufnahme/Wiedergabe       */
/*                         vorbereiten                                */
/**------------------------------------------------------------------**/
/* Eingabe : iMode : == DAC : Ausgabe                                 */
/*                   <> DAC : Aufnahme                                */
/**------------------------------------------------------------------**/
/* Info : - Bei Stereo-Aufnahme/Wiedergabe mssen alle Filter         */
/*          abgeschaltet sein. Fr eine Ordnungsgeme ausgabe mu    */
/*          aber der Stereo-Schalter angeschaltet werden.             */
/**********************************************************************/
VOID mix3_PrepareForStereo( WORD iMode )
{                                       /* Alte Mixer Zustnde merken */
  bMix3DACStereo = ( BYTE )mix3_GetDACStereo();
  bMix3DACFilter = ( BYTE )mix3_GetDACFilter();
  bMix3ADCFilter = ( BYTE )mix3_GetADCFilter();

                           /* Filter und Stereo-Wiedergabe abschalten */
  mix3_SetDACStereo( iMode );
  mix3_SetDACFilter( FALSE );
  mix3_SetADCFilter( FALSE );
}

/**********************************************************************/
/* mix3_RestoreFromStereo : Mixer in alten Zustand zurckversetzen    */
/**------------------------------------------------------------------**/
/* Info : - Der Zustand des Mixers mu vorher durch die Funktion      */
/*          'mix3_PrepareForStereo' in den entsprechenden globalen    */
/*          Variablen gerettet worden sein.                           */
/**********************************************************************/
VOID mix3_RestoreFromStereo( VOID )
{                    /* Filter und Stereo-Wiedergabe wiederherstellen */
  mix3_SetDACStereo( bMix3DACStereo );
  mix3_SetDACFilter( bMix3DACFilter );
  mix3_SetADCFilter( bMix3ADCFilter );
}

/**********************************************************************/
/* mix3_SetVolume : Ausgabe-Laustrke festlegen                       */
/**------------------------------------------------------------------**/
/* Eingabe : iSource - Quelle ( Mikrophon, DSP, CD, LINE, MIDI ) und  */
/*                     Gesamtlautstrke ( Master )                    */
/*           iLeft   - Lautstrke fr linken Kanal                    */
/*           iRight  - Lautstrke fr rechten Kanal                   */
/**------------------------------------------------------------------**/
/* Info : - Die Gesamtlautstrke mu grer 0 sein, damit die anderen */
/*          Quellen gehrt werden knnen. Die Laustrke wird fr den  */
/*          linken und rechten Kanal getrennt angegeben. Da ein       */
/*          Mikrophon immmer Monophon ist, wird dessen Laustrke      */
/*          durch das Maximum von linkem und rechten Kanal bestimmt.  */
/*          Die Angabe der Lautstrken erfolgt durch Werte zwischen   */
/*          0 und 255, die in die fr die Karte entsprechenden Werte  */
/*          umgerechnet werden.                                       */
/**********************************************************************/
VOID mix3_SetVolume( WORD iSource, WORD iVolL, WORD iVolR )
{ BYTE bIn, bVolR, bVolL;
                    /* Werte von 0 - 255 auf Werte von 0 - 7 abbilden */
  bVolL = ( BYTE )( iVolL >> 5 );
  bVolR = ( BYTE )( iVolR >> 5 );
  switch( iSource )
  {
    case MIC:                     /* PC-Speaker wird nicht verwendet! */
    case VOICE:   case VOICE_L:     case VOICE_R:
    case LINE:    case LINE_L:      case LINE_R:
    case CD:      case CD_L:        case CD_R:
    case MIDI:    case MIDI_L:      case MIDI_R:
    case MASTER:  case MASTER_L:    case MASTER_R:
                                    /* Alten Lautstrke-Wert lesen... */
      bIn = ( BYTE )mix_Read( bMix3VolTable[ iSource ][ PORT ] );
                                                  /* ...und ndern... */
      switch( bMix3VolTable[ iSource ][ CHANNEL ] )
      {
        case CH_LEFT:                      /* Nur Linken Kanal ndern */
          bIn = ( BYTE )( ( bIn & 0x0F ) | ( bVolL << 5 ) );
        break;
        case CH_RIGHT:                    /* Nur rechten Kanal ndern */
          bIn = ( BYTE )( ( bIn & 0xF0 ) | ( bVolR << 1 ) );
        break;
        case CH_BOTH:                      /* Links und Rechts ndern */
          bIn = ( BYTE )( ( bVolL << 5 ) | ( bVolR << 1 ) );
        break;
        case CH_MAX:           /* Maximum von Links und Rechts setzen */
          bIn = ( BYTE ) (max( bVolL << 1, bVolR << 1 ) );
        break;
      }
                                           /* ... und zurckschreiben */
      mix_Write( bMix3VolTable[ iSource ][ PORT ], bIn );
  }
}

/**********************************************************************/
/* mix3_GetVolume : Ausgabe-Laustrke ermitteln                       */
/**------------------------------------------------------------------**/
/* Eingabe : iSource - Quelle ( Mikrophon, DSP, CD, LINE, MIDI ) und  */
/*                     Gesamtlautstrke ( Master )                    */
/* Ausgabe : Laustrke der Quelle zwischen 0 und 255, bei Stereo als  */
/*           LO- und HIBYTE                                           */
/**********************************************************************/
WORD mix3_GetVolume( WORD iSource )
{ BYTE bIn, bLeft, bRight;

  switch( iSource )
  {
    case MIC:                     /* PC-Speaker wird nicht verwendet! */
    case VOICE:   case VOICE_L:     case VOICE_R:
    case LINE:    case LINE_L:      case LINE_R:
    case CD:      case CD_L:        case CD_R:
    case MIDI:    case MIDI_L:      case MIDI_R:
    case MASTER:  case MASTER_L:    case MASTER_R:
                                       /* Alten Lautstrke-Wert lesen */
      bIn = ( BYTE )mix_Read( bMix3VolTable[ iSource ][ PORT ] );
      bLeft  = ( BYTE )( ( ( bIn & 0xF0 ) >> 5 ) << 5 );
      bRight = ( BYTE )( ( ( bIn & 0x0F ) >> 1 ) << 5 );

      switch( bMix3VolTable[ iSource ][ CHANNEL ] )
      {
        case CH_LEFT:  return bLeft;
        case CH_MAX:
        case CH_RIGHT: return bRight;
        case CH_BOTH:  return MAKEWORD( bLeft, bRight );
      }
  }
  return 0xFFFF;
}

/**********************************************************************/
/* mix3_SetADCSource : Aufnahme-Quelle setzen                         */
/**------------------------------------------------------------------**/
/* Eingabe : iSource - Quelle fr Aufnahme                            */
/*                     Erlaubt: MIC, CD, LINE                         */
/**------------------------------------------------------------------**/
/* Info : Der A/D-Wandler bentigt eine Quelle. Diese wird hier       */
/*        angegeben.                                                  */
/**********************************************************************/
VOID mix3_SetADCSource( WORD iSource )
{  BYTE bIn;

   bIn = ( BYTE )mix_Read( MIX3_ADCSTATE );
                 /* Immer alte Quelle lschen bevor neue gesetzt wird */
   switch( iSource )
   {
      case MIC:  bIn &= ~MIX3_SRCMSK; bIn |= MIX3_MICSRC;  break;
      case CD:   bIn &= ~MIX3_SRCMSK; bIn |= MIX3_CDSRC;   break;
      case LINE: bIn &= ~MIX3_SRCMSK; bIn |= MIX3_LINESRC; break;
   }
   mix_Write( MIX3_ADCSTATE, bIn );
}

/**********************************************************************/
/* mix3_GetADCSource : Aktuelle Aufnahme-Quelle ermitteln             */
/**------------------------------------------------------------------**/
/* Eingabe : iSource - Quelle fr Aufnahme                            */
/*                     Erlaubt: MIC, CD, LINE                         */
/**------------------------------------------------------------------**/
/* Info : Der A/D-Wandler bentigt eine Quelle. Diese wird hier       */
/*        angegeben.                                                  */
/**********************************************************************/
WORD mix3_GetADCSource( VOID )
{ BYTE bIn;

  bIn = ( BYTE )( mix_Read( MIX3_ADCSTATE ) & MIX3_SRCMSK );

  switch( bIn )
  {
    case MIX3_MICSRC_:
    case MIX3_MICSRC:  return MIC;
    case MIX3_CDSRC:   return CD;
    case MIX3_LINESRC: return LINE;
    default:           return 0xFFFF;
  }
}

/**********************************************************************/
/* mix4_PrepareForMonoADC : Mixer fr Mono-Aufnahme vorbereiten       */
/**------------------------------------------------------------------**/
/* Info : - Dieser Befehl setzt Aufnahme-Quellen des rechten A/D-     */
/*          Kanals auch im linken Kanal. Im Mono-Aufnahme-Betrieb     */
/*          benutzt der DSP4.xx nur den linken Kanal.                 */
/**********************************************************************/
VOID mix4_PrepareForMonoADC( VOID )
{
  bMix4SourceL = ( BYTE )mix_Read( MIX4_ADCSOURCE_L );
  bMix4SourceR = ( BYTE )mix_Read( MIX4_ADCSOURCE_R );
  mix_Write( MIX4_ADCSOURCE_L, bMix4SourceL | bMix4SourceR );
}

/**********************************************************************/
/* mix4_RestoreFromMonoADC : Mixer-Zustand wiederherstellen.          */
/**------------------------------------------------------------------**/
/* Info : Diese Funktion darf nur aufgerufen werden, falls vorher die */
/*        Funktion 'mix4_PrepareForMonoADC' aufgerufen wurde.         */
/**********************************************************************/
VOID mix4_RestoreFromMonoADC( VOID )
{
  mix_Write( MIX4_ADCSOURCE_L, bMix4SourceL );
  mix_Write( MIX4_ADCSOURCE_L, bMix4SourceR );
}

/**********************************************************************/
/* mix4_SetVolume : Ausgabe-Laustrke einer Mixer-Quelle setzen       */
/**------------------------------------------------------------------**/
/* Eingabe : iSource - Quelle ( Mikrophon, DSP, CD, LINE, MIDI,       */
/*                     PCSPEAKER ) und Gesamtlautstrke ( Master )    */
/*           iLeft   - Lautstrke fr linken Kanal                    */
/*           iRight  - Lautstrke fr rechten Kanal                   */
/**------------------------------------------------------------------**/
/* Info : - Die Gesamtlautstrke mu grer 0 sein, damit die anderen */
/*          Quellen gehrt werden knnen. Die Lautstrke wird fr den */
/*          linken und rechten Kanal getrennt angegeben. Da ein       */
/*          Mikrophon immmer Monophon ist, wird dessen Lautstrke     */
/*          durch das Maximum von linkem und rechten Kanal bestimmt.  */
/*          Die Angabe der Lautstrken erfolgt durch Werten zwischen  */
/*          0 und 255, die in die fr die Karte entsprechenden Werte  */
/*          umgerechnet werden.                                       */
/*                                                                    */
/*          Diese Funktion nutzt aus, das die Register fr linken und */
/*          rechten Kanal einer Quelle an aufeinanderfolgenden Port-  */
/*          adressen liegen.                                          */
/**********************************************************************/
VOID mix4_SetVolume( WORD iSource, WORD iVolL, WORD iVolR )
{
  iVolL &= ~0x07;
  iVolR &= ~0x07;

  switch( iSource )
  {
    case MIC:
    case PCSPEAKER:
    case VOICE:   case VOICE_L:     case VOICE_R:
    case LINE:    case LINE_L:      case LINE_R:
    case CD:      case CD_L:        case CD_R:
    case MIDI:    case MIDI_L:      case MIDI_R:
    case MASTER:  case MASTER_L:    case MASTER_R:
    switch( bMix4VolTable[ iSource ][ CHANNEL ] )
    {
      case CH_LEFT:
        mix_Write( bMix4VolTable[ iSource ][ PORT ], iVolL );
      break;
      case CH_RIGHT:
        mix_Write( bMix4VolTable[ iSource ][ PORT ] + 1, iVolR );
      break;
      case CH_BOTH:
        mix_Write( bMix4VolTable[ iSource ][ PORT ], iVolL );
        mix_Write( bMix4VolTable[ iSource ][ PORT ] + 1, iVolR );
      break;

      case CH_MAX:
        mix_Write( bMix4VolTable[ iSource ][ PORT ],
                   max( iVolL, iVolR ) );
      break;
    }
  }
}

/**********************************************************************/
/* mix4_GetVolume : Ausgabe-Laustrke ermitteln                       */
/**------------------------------------------------------------------**/
/* Eingabe : iSource : Quelle ( Mikrophon, DSP, CD, LINE, MIDI,       */
/*                     PCSPEAKER ) und Gesamtlautstrke ( Master )    */
/* Ausgabe : Lautstrke der Quelle zwischen 0 und 255, bei Stereo als */
/*           LO- und HIBYTE                                           */
/**********************************************************************/
WORD mix4_GetVolume( WORD iSource )
{ BYTE bLeft, bRight;

  switch( iSource )
  {
    case MIC:
    case PCSPEAKER:
    case VOICE:   case VOICE_L:     case VOICE_R:
    case LINE:    case LINE_L:      case LINE_R:
    case CD:      case CD_L:        case CD_R:
    case MIDI:    case MIDI_L:      case MIDI_R:
    case MASTER:  case MASTER_L:    case MASTER_R:
                                       /* Alten Lautstrke-Wert lesen */
      bLeft = ( BYTE )mix_Read( bMix4VolTable[ iSource ][ PORT ] );
      bLeft &= ~0x07;
      bRight = ( BYTE )mix_Read( bMix4VolTable[ iSource ][ PORT ] + 1 );
      bRight &= ~0x07;

      switch( bMix4VolTable[ iSource ][ CHANNEL ] )
      {
        case CH_LEFT:  return bLeft;
        case CH_MAX:
        case CH_RIGHT: return bRight;
        case CH_BOTH:  return MAKEWORD( bLeft, bRight );
      }
  }
  return 0xFFFF;
}

/**********************************************************************/
/* mix4_SetADCSourceL : Aufnahme-Quelle fr linken ADC setzen         */
/**------------------------------------------------------------------**/
/* Eingabe : iSource - Aufnahme-Quelle                                */
/*           iState  - Quelle an oder abschalten                      */
/*                     == 0: abschalten                               */
/*                     <> 0: anschalten                               */
/**********************************************************************/
VOID mix4_SetADCSourceL( WORD iSource, WORD iState )
{ BYTE bInL;

  bInL = ( BYTE )mix_Read( MIX4_ADCSOURCE_L );
  switch( iSource )
  {
    case MIDI: case MIDI_L: case MIDI_R:
    case CD:   case CD_L:   case CD_R:
    case LINE: case LINE_L: case LINE_R:
    case MIC:
    if( iState )                               /* Eingabekanal setzen */
        bInL |= bMix4SourceBits[ iSource ];
    else                                      /* Eingabekanal lschen */
        bInL &= ~bMix4SourceBits[ iSource ];
  }
  mix_Write( MIX4_ADCSOURCE_L, bInL );
}

/**********************************************************************/
/* mix4_SetADCSourceR : Aufnahme-Quelle fr rechten ADC setzen        */
/**------------------------------------------------------------------**/
/* Eingabe : iSource - Aufnahme-Quelle                                */
/*           iState  - Quelle an oder abschalten                      */
/*                     == 0: abschalten                               */
/*                     <> 0: anschalten                               */
/**********************************************************************/
VOID mix4_SetADCSourceR( WORD iSource, WORD iState )
{ BYTE bInR;

  bInR = ( BYTE )mix_Read( MIX4_ADCSOURCE_R );
  switch( iSource )
  {
    case MIDI: case MIDI_L: case MIDI_R:
    case CD:   case CD_L:   case CD_R:
    case LINE: case LINE_L: case LINE_R:
    case MIC:
    if( iState )                               /* Eingabekanal setzen */
        bInR |= bMix4SourceBits[ iSource ];
    else                                      /* Eingabekanal lschen */
        bInR &= ~bMix4SourceBits[ iSource ];
  }
  mix_Write( MIX4_ADCSOURCE_R, bInR );
}

/**********************************************************************/
/* mix4_GetADCSourceL : Aufnahme-Quelle fr linken ADC ermitteln      */
/**------------------------------------------------------------------**/
/* Eingabe : iSource - Aufnahme-Quelle                                */
/* Ausgabe : ID, fr welche ADC-Kanle die angegebene Aufnahme-       */
/*           Quelle gesetzt ist.                                      */
/**********************************************************************/
WORD mix4_GetADCSourceL( WORD iSource )
{ BYTE bInL;

  bInL = ( BYTE )mix_Read( MIX4_ADCSOURCE_L );

  switch( iSource )
  {
    case MIDI:
    case CD:
    case LINE:
    case MIC:
      if( ( bInL & bMix4SourceBits[ iSource ] ) ==
                 bMix4SourceBits[ iSource ] ) return CH_BOTH;

    break;
    case MIDI_L:
    case CD_L:
    case LINE_L:
      if( bInL & bMix4SourceBits[ iSource ] ) return CH_LEFT;
    break;
    case MIDI_R:
    case CD_R:
    case LINE_R:
      if( bInL & bMix4SourceBits[ iSource ] ) return CH_RIGHT;
    break;
  }
  return CH_NONE;
}

/**********************************************************************/
/* mix4_GetADCSourceR : Aufnahme-Quelle fr rechten ADC ermitteln     */
/**------------------------------------------------------------------**/
/* Eingabe : iSource : Aufnahme-Quelle                                */
/* Ausgabe : ID, auf welchen ADC-Kanlen die angegebene Aufnahme      */
/*           Quelle gesetzt ist.                                      */
/*           Mgliche Ausgaben:  CH_BOTH, CH_LEFT, CH_RIGHT, CH_NONE  */
/**********************************************************************/
WORD mix4_GetADCSourceR( WORD iSource )
{ BYTE bInR;

  bInR = ( BYTE )mix_Read( MIX4_ADCSOURCE_R );

  switch( iSource )
  {
    case MIDI:
    case CD:
    case LINE:
    case MIC:
      if( ( bInR & bMix4SourceBits[ iSource ] ) ==
                   bMix4SourceBits[ iSource ] ) return CH_BOTH;

    break;
    case MIDI_L:
    case CD_L:
    case LINE_L:
      if( bInR & bMix4SourceBits[ iSource ] ) return CH_LEFT;
    break;
    case MIDI_R:
    case CD_R:
    case LINE_R:
      if( bInR & bMix4SourceBits[ iSource ] ) return CH_RIGHT;
    break;
  }
  return CH_NONE;
}

/**********************************************************************/
/* mix4_SetOUTSource : Ausgabe(Wiedergabe)-Quelle setzen              */
/**------------------------------------------------------------------**/
/* Eingabe : iSource - Quelle                                         */
/*           iState  - Quelle an oder abschalten                      */
/**********************************************************************/
VOID mix4_SetOUTSource( WORD iSource, WORD iState )
{
  BYTE bIn;
  bIn = ( BYTE )mix_Read( MIX4_OUTSOURCE );
  switch( iSource )
  {
    case CD:   case CD_L:   case CD_R:
    case LINE: case LINE_L: case LINE_R:
    case MIC:
      if( iState ) bIn |=  bMix4SourceBits[ iSource ];   /* Setzen */
      else         bIn &= ~bMix4SourceBits[ iSource ];  /* Lschen */
  }
  mix_Write( MIX4_OUTSOURCE, bIn );
}

/**********************************************************************/
/* mix4_GetOUTSource : Ausgabe-Quelle ermitteln                       */
/**------------------------------------------------------------------**/
/* Eingabe : iSource - Aufnahme-Quelle                                */
/* Ausgabe : ID, auf welchen DAC-Kanlen die angegebene Aufnahme      */
/*           Quelle gesetzt ist.                                      */
/*           Mgliche Ausgaben:  CH_BOTH, CH_LEFT, CH_RIGHT, CH_NONE  */
/**********************************************************************/
WORD mix4_GetOUTSource( WORD iSource )
{ BYTE bIn;

  bIn = ( BYTE )mix_Read( MIX4_OUTSOURCE );

  switch( iSource )
  {
    case CD:
    case LINE:
    case MIC:
      if( ( bIn & bMix4SourceBits[ iSource ] ) )
        return CH_BOTH;
    break;
    case CD_L:
    case LINE_L:
      if( ( bIn & bMix4SourceBits[ iSource ] ) )
        return CH_LEFT;
    break;
    case CD_R:
    case LINE_R:
      if( ( bIn & bMix4SourceBits[ iSource ] ) )
        return CH_RIGHT;
    break;
  }
  return CH_NONE;
}

/**********************************************************************/
/* mix4_SetADCGain : Aufnahme Vorverstrker setzen                    */
/**------------------------------------------------------------------**/
/* Eingabe : iGainL, iGainR - 0 :  0 dB                               */
/*                            1 :  6 dB                               */
/*                            2 : 12 dB                               */
/*                            3 : 18 dB                               */
/**********************************************************************/
VOID mix4_SetADCGain( WORD iGainL, WORD iGainR )
{ BYTE bIn, bGainL, bGainR;

  bGainL = ( BYTE )( iGainL &= 0x03 );
  bGainL <<= 6;

  bGainR = ( BYTE )( iGainR &= 0x03 );
  bGainR <<= 6;

  bIn = ( BYTE )( ( mix_Read( MIX4_ADCGAIN_L ) & 0x3F ) | bGainL);
  mix_Write( MIX4_ADCGAIN_L, bIn );

  bIn = ( BYTE )( ( mix_Read( MIX4_ADCGAIN_R ) & 0x3F ) | bGainR );
  mix_Write( MIX4_ADCGAIN_R, bIn );
}


/**********************************************************************/
/* mix4_GetADCGain : Werte des Aufnahme Vorverstrker ermitteln       */
/**------------------------------------------------------------------**/
/* Eingabe : iChannel - CH_LEFT  : Gain fr linken Kanal              */
/*                      CH_RIGHT : Gain fr rechten Kanal             */
/*                      andere   : Gain fr beide Kanle              */
/* Ausgabe : Aktuelle Vorwerstrkerwerte                              */
/*           Bei Ermittlung des Wertes beider Kanle, wird im Lo-Byte */
/*           der Wert fr den linken , im Hi-Byte der Wert fr den    */
/*           rechten Kanal geliefert. Sonst wird der erfragte Wert    */
/*           immer im Lo-Byte angegeben!                              */
/**********************************************************************/
WORD mix4_GetADCGain( WORD iChannel )
{ BYTE bGainL, bGainR;

  bGainL = ( BYTE )mix_Read( MIX4_ADCGAIN_L );
  bGainL &= 0xC0;
  bGainL >>= 6;

  bGainR = ( BYTE )mix_Read( MIX4_ADCGAIN_R );
  bGainR &= 0xC0;
  bGainR >>= 6;

  switch (iChannel )
  {
    case CH_LEFT:  return bGainL;
    case CH_RIGHT: return bGainR;
    default:  return MAKEWORD( bGainL, bGainR );
  }
}

/**********************************************************************/
/* mix4_SetOUTGain : Ausgabe Vorverstrker setzen                     */
/**------------------------------------------------------------------**/
/* Eingabe : iGainL, iGainR - 0 :  0 dB                               */
/*                            1 :  6 dB                               */
/*                            2 : 12 dB                               */
/*                            3 : 18 dB                               */
/**********************************************************************/
VOID mix4_SetOUTGain( WORD iGainL, WORD iGainR )
{ BYTE bIn, bGainL, bGainR;

  bGainL = ( BYTE )( ( iGainL & 0x03 ) << 6 );
  bGainR = ( BYTE )( ( iGainR & 0x03 ) << 6 );

  bIn = ( BYTE )( ( mix_Read( MIX4_OUTGAIN_L ) & 0x3F ) | bGainL );
  mix_Write( MIX4_OUTGAIN_L, bIn );

  bIn = ( BYTE )( ( mix_Read( MIX4_OUTGAIN_R ) & 0x3F ) | bGainR );
  mix_Write( MIX4_OUTGAIN_R, bIn );
}

/**********************************************************************/
/* mix4_GetOUTGain : Ausgabe Vorverstrker holen                      */
/**------------------------------------------------------------------**/
/* Eingabe : iChannel - CH_LEFT  : Gain fr linken Kanal              */
/*                      CH_RIGHT : Gain fr rechten Kanal             */
/*                      andere   : Gain fr beide Kanle              */
/* Ausgabe : Aktuelle Vorwerstrkerwerte                              */
/*           Bei Ermittlung des Wertes beider Kanle, wird im Lo-Byte */
/*           der Wert fr den linken , im Hi-Byte der Wert fr den    */
/*           rechten Kanal geliefert. Sonst wird der erfragte Wert    */
/*           immer im Lo-Byte angegeben!                              */
/**********************************************************************/
WORD mix4_GetOUTGain( WORD iChannel )
{ BYTE bGainL, bGainR;

  bGainL = ( BYTE )mix_Read( MIX4_OUTGAIN_L );
  bGainL &= 0xC0;
  bGainL >>= 6;
  bGainR = ( BYTE )mix_Read( MIX4_OUTGAIN_R );
  bGainR &= 0xC0;
  bGainR >>= 6;

  switch (iChannel )
  {
    case CH_LEFT:  return bGainL;
    case CH_RIGHT: return bGainR;
    case CH_BOTH:  return MAKEWORD( bGainL, bGainR );
  }
  return 0;
}

/**********************************************************************/
/* mix4_SetAGC : Automatic Gain Control setzen / lschen              */
/**------------------------------------------------------------------**/
/* Eingabe : iState - == 0 : AGC lschen (0 dB )                      */
/*                    <> 0 : AGC setzen ( 20dB )                      */
/**------------------------------------------------------------------**/
/* Info : - AGC wird fr Aufnahmen mit einem Mikro verwendet und      */
/*          erlaubt das Vorschalten eines Vorverstrkers, der das     */
/*          Mikrophon-Signal um 20dB verstrkt.                       */
/**********************************************************************/
VOID mix4_SetAGC( WORD iState )
{
   BYTE bIn;
   bIn = ( BYTE )mix_Read( MIX4_AGC );
   mix_Write( MIX4_AGC, iState ?
                        ( bIn |  MIX4_AGCON ) :
                        ( bIn & ~MIX4_AGCON ) );
}

/**********************************************************************/
/* mix4_GetAGC : Automatic Gain Control holen                         */
/**------------------------------------------------------------------**/
/* Ausgabe: TRUE  : AGC gesetzt                                       */
/*          FALSE : AGC gelscht                                      */
/**********************************************************************/
WORD mix4_GetAGC( VOID )
{
   return ( ( mix_Read( MIX4_AGC ) & MIX4_AGCON ) != 0 );
}

/**********************************************************************/
/* mix4_SetTrebble : Hhen setzen                                     */
/*--------------------------------------------------------------------*/
/* Eingabe : iTrebbleL, iTrebbleR : Hhe fr linken und rechten       */
/*                                  Kanal setzen. Werte von 0 bis 15  */
/*                                  entsprechen -14 dB bis 14 dB in   */
/*                                  Schritten zu 2 dB.                */
/*--------------------------------------------------------------------*/
/* Info : Die Soundblaster16 Karte verfgt ber einen Vorverstrker   */
/*        dessen Klangmerkmale gendert werden knnen.                */
/**********************************************************************/
VOID mix4_SetTrebble( WORD iTrebbleL, WORD iTrebbleR )
{ 
  mix_Write( MIX4_TREBLE_L, ( BYTE )(iTrebbleL << 4) );
  mix_Write( MIX4_TREBLE_R, ( BYTE )(iTrebbleL << 4) );
}

/**********************************************************************/
/* mix4_GetTrebble : Aktuelle Hhenstellung holen                     */
/**------------------------------------------------------------------**/
/* Eingabe : iChannel : CH_LEFT  : Gain fr linken Kanal              */
/*                      CH_RIGHT : Gain fr rechten Kanal             */
/*                      CH_BOTH  : Gain fr beide Kanle              */
/* Ausgabe : Aktuelle Vorwerstrkerwerte                              */
/*           Bei Ermittlung des Wertes beider Kanle, wird im Lo-Byte */
/*           der Wert fr den linken , im Hi-Byte der Wert fr den    */
/*           rechten Kanal geliefert. Sonst wird der erfragte Wert    */
/*           immer im Lo-Byte angegeben!                              */
/**********************************************************************/
WORD mix4_GetTrebble( WORD iChannel )
{ BYTE bTrebbleL, bTrebbleR;

  bTrebbleL = ( BYTE )mix_Read( MIX4_TREBLE_L ) >> 4;
  bTrebbleR = ( BYTE )mix_Read( MIX4_TREBLE_R ) >> 4;

  switch (iChannel )
  {
    case CH_LEFT:  return bTrebbleL;
    case CH_RIGHT: return bTrebbleR;
    case CH_BOTH:  return MAKEWORD( bTrebbleL, bTrebbleR );
  }
  return 0;
}

/**********************************************************************/
/* mix4_SetBass : Bsse setzen                                        */
/*--------------------------------------------------------------------*/
/* Eingabe : iBassL, iBassR : Bsse fr linken und rechten Ausgabe-   */
/*                            kanal setzen. Werte von 0 bis 15        */
/*                            entsprechen -14 dB bis 14 dB in         */
/*                            Schritten zu 2 dB.                      */
/*--------------------------------------------------------------------*/
/* Info - Die Soundblaster16 Karte verfgt ber einen Vorverstrker   */
/*        dessen Klangmerkmale gendert werden knnen.                */
/**********************************************************************/
VOID mix4_SetBass( WORD iBassL, WORD iBassR )
{ 
  mix_Write( MIX4_BASS_L, ( BYTE )(iBassL << 4) );
  mix_Write( MIX4_BASS_R, ( BYTE )(iBassR << 4) );
}

/**********************************************************************/
/* mix4_GetBass : Aktuelle Hhenstellung holen                        */
/**------------------------------------------------------------------**/
/* Eingabe : iChannel : CH_LEFT  : Gain fr linken Kanal              */
/*                      CH_RIGHT : Gain fr rechten Kanal             */
/*                      CH_BOTH  : Gain fr beide Kanle              */
/* Ausgabe : Aktuelle Vorwerstrkerwerte                              */
/*           Bei Ermittlung des Wertes beider Kanle, wird im Lo-Byte */
/*           der Wert fr den linken , im Hi-Byte der Wert fr den    */
/*           rechten Kanal geliefert. Sonst wird der erfragte Wert    */
/*           immer im Lo-Byte angegeben!                              */
/**********************************************************************/
WORD mix4_GetBass( WORD iChannel )
{ BYTE bBassL, bBassR;

  bBassL = ( BYTE )mix_Read( MIX4_BASS_L )>>4;
  bBassR = ( BYTE )mix_Read( MIX4_BASS_R )>>4;
  switch (iChannel )
  {
    case CH_LEFT:  return bBassL;
    case CH_RIGHT: return bBassR;
    case CH_BOTH:  return MAKEWORD( bBassL, bBassR );
  }
  return 0;
}
#endif



