/**********************************************************************/
/*                             N O K E Y C                            */
/*--------------------------------------------------------------------*/
/*    Zeigt, wie man den Tastaturpuffer lscht, um den Anwender bei   */
/*    wichtigen Abfragen (z.B. "Datei lschen?") Schutz vor vor-      */
/*    eiligen Eingaben zu bieten.                                     */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  :  1.01.1992                                     */
/*    letztes Update :  1.01.1992                                     */
/**********************************************************************/

#include <stdio.h>
#include <dos.h>
#include <bios.h>

/*== Makros ==========================================================*/

#ifndef MK_FP       /* wenn es MK_FP noch nicht gibt, dann definieren */
  #define MK_FP(seg,ofs) \
    ((void far *) (((unsigned long)(seg) << 16) | (unsigned)(ofs)))
#endif

#ifdef __TURBOC__                         /* Definitionen fr TURBO C */

   #define GetKbKey()        ( bioskey( 0 ) )
   #define GetKbReady()      ( bioskey( 1 ) != 0 )
   #define GetBiosTime(x)    ( x = biostime( 0, NULL ) )
   #define CLI()             ( disable() )
   #define STI()             ( enable() )

#else                        /* Definitionen fr Microsoft C-Compiler */

   #define GetKbKey()        ( _bios_keybrd( _KEYBRD_READ ) )
   #define GetKbReady()      ( _bios_keybrd( _KEYBRD_READY ) != 0 )
   #define GetBiosTime(x)    ( _bios_timeofday( _TIME_GETCLOCK, &x) )
   #define CLI()             ( _disable() )
   #define STI()             ( _enable() )

#endif

/*== Bildschirmroutinen fr Microsoft C ==============================*/

#ifndef __TURBOC__                                    /* Microsoft C? */

  /********************************************************************/
  /* Gotoxy        : setzt den Cursor                                 */
  /* Eingabe       : Koordinaten des Cursors                          */
  /* Ausgabe       : keine                                            */
  /********************************************************************/

  void gotoxy( int x, int y )
  {
   union REGS regs;                  /* Register fr Interrupt-Aufruf */

   regs.h.ah = 0x02;          /* Funktionsnummer fr Interrupt-Aufruf */
   regs.h.bh = 0;                                            /* Farbe */
   regs.h.dh = y - 1;
   regs.h.dl = x - 1;
   int86( 0x10, &regs, &regs );                    /* InterruptAufruf */
  }

  /********************************************************************/
  /* clrscr        : lscht den Bildschirm                            */
  /* Eingabe       : keine                                            */
  /* Ausgabe       : keine                                            */
  /********************************************************************/

  void clrscr( void )
  {
   union REGS regs;                  /* Register fr Interrupt-Aufruf */

   regs.h.ah = 0x07;          /* Funktionsnummer fr Interrupt-Aufruf */
   regs.h.al = 0x00;
   regs.h.ch = 0;
   regs.h.cl = 0;
   regs.h.dh = 24;
   regs.h.dl = 79;
   int86( 0x10, &regs, &regs );                    /* InterruptAufruf */
   gotoxy( 1, 1 );                                   /* Cursor setzen */
  }

#endif

/**********************************************************************/
/*  Delay:  Hlt die Programmausfhrung ungeachtet der Geschwin-      */
/*          digkeit des Systems fr ein bestimmtes Zeitintervall an   */
/*  Eingabe  : PAUSE = Lnge des Zeitintervalls in Ticks              */
/*  Ausgabee : keine                                                  */
/*  Info     : ein Tick entspricht 1/18,2 Sekunden                    */
/**********************************************************************/

void delay( unsigned int pause )
{
 long aktzeit,                                       /* aktuelle Zeit */
      zielzeit;                                /* zu erreichende Zeit */

 if ( pause )                                       /* Pause nicht 0? */
  {                                                           /* Nein */
   GetBiosTime( zielzeit );
   zielzeit += (long) pause;                    /* Zielzeit errechnen */

   do                            /* Wartschleife, aktuelle Zeit holen */
    GetBiosTime( aktzeit );
   while ( aktzeit <= zielzeit );                   /* Zeit erreicht? */
  }                                           /* Ja, Funktion beenden */
}

/**********************************************************************/
/* ClearKbBuffer : lscht den Inhalt des Tastaturpuffers              */
/* Eingabe : keine                                                    */
/* Ausgabe : keine                                                    */
/**********************************************************************/

void ClearKbBuffer( void )
{
 CLI();                            /* keine Hardware-Interrupts jetzt */
 *(int far *) MK_FP(0x40,0x1a) =       /* kein Zeichen mehr im Puffer */
 *(int far *) MK_FP(0x40,0x1C);
 STI( );                             /* Hardware-Interrupts wieder an */
}

/**********************************************************************/
/*                     H A U P T P R O G R A M M                      */
/**********************************************************************/

void main( void )
{
 int           i,                                  /* Schleifenzhler */
               ccount;            /* Anzahl Zeichen im Tastaturpuffer */
 unsigned char ch;                                /* nimmt Tasten auf */

 clrscr();
 printf( "NOKEYC  -  (c) 1992 by Michael Tischer\n\n" );
 printf( "Bei 0 werden alle Zeichen im Tastaurpuffer gelscht.\n\n" );

 for ( i = 10; i; --i )              /* dem Anwender Zeit zur Eingabe */
  {
   printf( "%5d", i );
   delay( 13 );                          /* dreiviertel Sekunde Pause */
  }

/* ClearKbBuffer();  */                      /* Tastaturpuffer leeren */

 /*-- Zeichen ausgeben, die sich noch im Tastaturpuffer befinden -----*/

 ccount = 0;                            /* noch kein Zeichen entdeckt */
 printf( "\n\nZeichen im Tastaturpuffer :\n" );

 while GetKbReady()            /* noch ein Zeichen im Tastaturpuffer? */
  {                                       /* Ja, auslesen und ausgben */
   ch = GetKbKey();
   printf( "   %3d   ", (int) ch );     /* zunchst nur Code ausgeben */
   if ( (int) ch > 32 )                        /* kein Sonderzeichen? */
    printf ( "(%c)", ch );             /* Nein, dann auch als Zeichen */
   printf("\n");
   ++ccount;                         /* mehr als ein Zeichen gefunden */
  }

 if ( ccount == 0 )                        /* keine Zeichen gefunden? */
  printf( "(Keine)\n" );                                      /* Nein */
 printf( "\n" );
}
