/**********************************************************************/
/*                          P L I N K C . C                           */
/*--------------------------------------------------------------------*/
/*    Aufgabe       : bertragen von Dateien ber die parallele       */
/*                    Schnittstelle                                   */
/*--------------------------------------------------------------------*/
/*    Autor         : Michael Tischer                                 */
/*    entwickelt am : 27.09.1991                                      */
/*    letztes Update: 28.11.1991                                      */
/*--------------------------------------------------------------------*/
/*    Speichermodell: SMALL                                           */
/**********************************************************************/

/*== Include-Dateien einbinden =======================================*/

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <setjmp.h>
#include <string.h>
#ifdef __TURBOC__                                /* Turbo C Compiler? */
  #include <dir.h>                  /* Directory-Funktionen einbinden */
  #include <ctype.h>                                 /* fr toupper() */
#endif

/*== Typdefinitionen =================================================*/

typedef unsigned char BYTE;                     /* definiere ein Byte */
typedef unsigned int WORD;                /* wir basteln uns ein WORD */
typedef struct {                 /* Header zum bertragen von Blcken */
                BYTE         Token;
                unsigned int Len;
               } BHEADER;

/*== Funktionen aus dem Assembler-Modul ==============================*/

extern void IntrInstall( int far  * escape_flag,
                         WORD far * timeout_count );
extern void IntrRemove( void );
extern void EscapeDirekt( int ausloesen );

/*== Konstanten ======================================================*/

#define ONESEK        18                              /* eine Sekunde */
#define TENSEK        182                            /* zehn Sekunden */
#define TO_DEFAULT    TENSEK                 /* Default-Wert Time-Out */

#define TRUE          ( 0 == 0 )
#define FALSE         ( 0 == 1 )
#define MAXBLOCK      4096        /* Blockgre 4 KB ( w.g. Cluster ) */

/*-- Konstanten fr bertragungsprotokoll ----------------------------*/

#define ACK           0x00                             /* Acknowledge */
#define NAK           0xFF                         /* Non-Acknowledge */
#define MAX_TRY       5              /* Anzahl erlaubter Fehlversuche */

/*-- Tokens fr die Kommunikation zwischen Sender und Empfnger ------*/

#define TOK_DATSTART  0                     /* jetzt kommt eine Datei */
#define TOK_DATNEXT   1            /* nchsten Datenblock einer Datei */
#define TOK_DATEND    2        /* bertragung der Datei abgeschlossen */
#define TOK_ENDE      3                           /* Programm beenden */
#define TOK_ESCAPE    4         /* Escape auf Remote-Rechner gedrckt */

/*-- Codes fr die LongJump-Aufrufe ----------------------------------*/

#define LJ_OKSENDER   1          /* alle Dateien erfolgreich gesendet */
#define LJ_OKEMPF     2         /* alle Dateien erfolgreich empfangen */
#define LJ_TIMEOUT    3        /* Time-out: Gegenber antwortet nicht */
#define LJ_ESCAPE     4   /* auf der lokalen Maschine wurde ESC gedr. */
#define LJ_REMESCAPE  5 /* auf der Remote-Maschine wurde ESC gedrckt */
#define LJ_DATA       6                      /* Kommunikations-Fehler */
#define LJ_NOLINK     7                           /* keine Verbindung */
#define LJ_NOPAR      8                        /* keine Schnittstelle */
#define LJ_PARA       9                 /* ungltige Aufruf-Parameter */

/*== Makros ==========================================================*/

/*-- Die unteren drei Bits des Eingaberegisters sind nicht belegt ----*/
/*-- und knnen je nach Computer 1 oder 0 sein, deshalb werden    ----*/
/*-- sie von GetB() ausgeblendet.                                 ----*/

#ifdef __TURBOC__                                /* Turbo C Compiler? */
  #define GetB()                         ( inportb( EinPort ) & 0xF8 )
  #define PutB( Was )                    outportb( AusPort, Was )
  #define DIRSTRUCT                      struct ffblk
  #define FINDFIRST( path, buf, attr )   findfirst( path, buf, attr )
  #define FINDNEXT( buf )                findnext( buf )
  #define DATEINAME                      ff_name
#else                                            /* Nein, Microsoft C */
  #define GetB()                         ( inp( EinPort ) & 0xF8 )
  #define PutB( Was )                    outp( AusPort, Was )
  #define DIRSTRUCT                      struct         find_t
  #define FINDFIRST( path, buf, attr )   _dos_findfirst(path, attr, buf)
  #define FINDNEXT( buf )                _dos_findnext( buf )
  #define DATEINAME                      name
#endif

#ifdef MK_FP                          /* Makro MK_FP schon definiert? */
  #undef MK_FP                              /* Ja, dann Makro lschen */
#endif

#define MK_FP(seg,ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))

/*== Globale Variablen ===============================================*/

int     EinPort;                          /* Adresse des Eingabeports */
int     AusPort;                          /* Adresse des Ausgabeports */
int     Escape = 0;                           /* kein ESCAPE gedrckt */
WORD    Timeout = TO_DEFAULT;               /* gewhlter Timeout-Wert */
WORD    TO_Count;                               /* Zhler fr Timeout */
jmp_buf RueckSprung;               /* Rckspungadresse fr Beendigung */
BYTE    *BlockBuf;                /* Puffer zur Aufnahme eines Blocks */
FILE    *Datei = NULL;         /* Dateivariable fr bearbeitete Datei */

/**********************************************************************/
/* GetPortAdr: Initialisiert die Portadressen einer parallelen        */
/*             Schnittstelle in den globalen Variablen EINPORT und    */
/*             AUSPORT                                                */
/* Eingabe       : NUMMER = Nummer der parallelen Schnittstelle (1-4) */
/* Ausgabe       : TRUE, wenn Schnittstelle gltig                    */
/* Globale Var.  : EinPort/W, AusPort/W                               */
/* Info          : Die Basisadressen der bis zu 4 parallelen Schnitt- */
/*                 stellen stehen in den vier Speicherworten          */
/*                 beginnend bei 0040:0008                            */
/**********************************************************************/

int GetPortAdr( int Nummer )
{                  /* Portadresse aus BIOS-Variablen-Segment auslesen */
 AusPort = *( WORD far * ) MK_FP( 0x0040, 6 + Nummer * 2 );
 if ( AusPort != 0 )                      /* Schnittstelle vorhanden? */
  {                                                             /* Ja */
   EinPort = AusPort + 1;              /* Adresse fr Eingaberegister */
   return TRUE;                                   /* Ende ohne Fehler */
  }
 else
  return FALSE;              /* Fehler: Schnittstelle nicht vorhanden */
}

/**********************************************************************/
/* Port_Init     : Initialisiert die zur bertragung notwendigen      */
/*                 Register                                           */
/* Eingabe       : SENDER = TRUE, wenn Sender, FALSE, wenn Empfnger  */
/* Ausgabe       : TRUE, wenn Register erfolgreich initialisiert      */
/* Globale Var.  : EinPort/R, AusPort/R                               */
/* Info          : Die Asymmetrie: sende 00010000, warte auf 00000000 */
/*                 ist durch die Invertierung des Signals notwendig.  */
/*                 Normalerweise enthalten das Ein- u. das            */
/*                 Ausgaberegister die gewnschten Werte, die Initia- */
/*                 lisierung ist aber beim Neustart nach einer abge-  */
/*                 brochenen bertragung notwendig.                   */
/**********************************************************************/

int Port_Init( int Sender )
{
 EscapeDirekt( TRUE );                 /* bei Escape Timeout auslsen */
 if ( Sender )                               /* Gert ist der Sender? */
 {
  TO_Count = Timeout * 5;               /* Time-Out-Zhler einstellen */
  PutB( 0x10 );                                   /* Sende: 00010000b */
  while ( ( GetB() != 0x00 ) && TO_Count )     /* Warte auf 00000000b */
   ;
  }
 else                                          /* Gert ist Empfnger */
  {
   TO_Count = Timeout * 5;              /* Time-Out-Zhler einstellen */
   while ( ( GetB() != 0x00 ) && TO_Count )    /* Warte auf 00000000b */
    ;
   PutB( 0x10 );                                  /* Sende: 00010000b */
  }
 EscapeDirekt( FALSE );           /* bei Escape kein Timeout auslsen */
 return ( TO_Count != 0 );           /* Initialisierung abgeschlossen */
}

/**********************************************************************/
/* SendeByte    : Sendet ein Byte in 2 Teilen an den Fremdrechner und */
/*                berprft das Ergebnis                              */
/* Eingabe      : WERT = zu sendendes Byte                            */
/* Ausgabe      : bertragung erfolgreich? ( 0 = Fehler, -1 = ok )    */
/* Globale Var. : Timeout/R, Einport/R, Ausport/R (in Makros)         */
/**********************************************************************/

int SendeByte( BYTE Wert )
{
 BYTE Rueck;                                      /* Empfangenes Byte */

 /*-- Unteres Nibble Senden  -----------------------------------------*/

 TO_Count = Timeout;                 /* Timeout-Zhler initialisieren */
 PutB( Wert & 0x0F );                         /* Senden, BUSY lschen */
 while ( ( ( GetB() & 128 ) == 0 ) && TO_Count ) /* Wart. auf Meldung */
  ;
 if ( TO_Count == 0 )                              /* Timeout-Fehler? */
  longjmp( RueckSprung, LJ_TIMEOUT );        /* bertragung abbrechen */

 Rueck = ( GetB() >> 3 ) & 0x0F;                   /* Bits 3-6 in 0-3 */

 /*-- Oberes Nibble Senden -------------------------------------------*/

 TO_Count = Timeout;                 /* Timeout-Zhler initialisieren */
 PutB( ( Wert >> 4 ) | 0x10 );                 /* Senden, BUSY setzen */
 while ( ( ( GetB() & 128 ) != 0 ) && TO_Count ) /* Wart. auf Meldung */
  ;

 if ( TO_Count == 0 )                              /* Timeout-Fehler? */
  longjmp( RueckSprung, LJ_TIMEOUT );        /* bertragung abbrechen */

 Rueck = Rueck | ( ( GetB() << 1 ) & 0xF0 );       /* Bits 3-6 in 4-7 */
 return ( Wert == Rueck );               /* Byte korrekt bermittelt? */
}

/**********************************************************************/
/* EmpfangeByte : Empfngt ein Byte in 2 Teilen vom Fremdrechner und  */
/*                sendet empfangene Teile zu Prfzwecken zurck       */
/* Eingabe      : keine                                               */
/* Ausgabe      : empfangenes Byte                                    */
/* Globale Var. : Timeout/R, Einport/R, Ausport/R (in Makros)         */
/**********************************************************************/

BYTE EmpfangeByte( void )
{
 BYTE LoNib, HiNib;                             /* Empfangene Nibbles */

 /*-- Unteres Nibble Empfangen und Zurcksenden ----------------------*/

 TO_Count = Timeout;                 /* Timeout-Zhler initialisieren */
 while ( ( ( GetB() & 128 ) == 0 ) && TO_Count ) /* Warten bis BUSY 1 */
  ;

 if ( TO_Count == 0 )                              /* Timeout-Fehler? */
  longjmp( RueckSprung, LJ_TIMEOUT );        /* bertragung abbrechen */

 LoNib = ( GetB() >> 3 ) & 0x0F;                   /* Bits 3-6 in 0-3 */
 PutB( LoNib );                                       /* Zurcksenden */

 /*-- Oberes Nibble Empfangen und Zurcksenden -----------------------*/

 TO_Count = Timeout;                 /* Timeout-Zhler initialisieren */
 while ( ( ( GetB() & 128 ) != 0 ) && TO_Count ) /* Warten bis BUSY 0 */
  ;

 if ( TO_Count == 0 )                              /* Timeout-Fehler? */
  longjmp( RueckSprung, LJ_TIMEOUT );        /* bertragung abbrechen */

 HiNib = ( GetB() << 1 ) & 0xF0;                   /* Bits 3-6 in 4-7 */
 PutB( ( HiNib >> 4 ) | 0x10 );          /* Zurcksenden, Busy setzen */

 return( LoNib | HiNib );                         /* Empfangenes Byte */
}

/**********************************************************************/
/* SendeBlock: Sendet einen Datenblock                                */
/* Eingabe   : TOKEN  = Befehl fr Empfnger                          */
/*             ANZAHL = Anzahl der zu bertragenden Bytes             */
/*             DATEN  = Zeiger auf den Puffer mit den Daten           */
/* Ausgabe   : keine, denn bei einem Fehler wird sofort ber          */
/*             LongJmp zur Fehlerbehandlungsroutine gesprungen        */
/**********************************************************************/

void SendeBlock( BYTE Token, int Anzahl, void *Daten )
{
 BHEADER header;          /* Header zur Aufnahme von Token und Anzahl */
 BYTE    *bptr,               /* Zeiger auf aktuell zu sendendes Byte */
         EmpfEscape; /* wurde auf dem Remote-Rechner ESCAPE gedrckt? */
 int     ok,                                           /* Fehler-Flag */
         i,                                        /* Schleifenzhler */
         try;                     /* verbleibende Anzahl der Versuche */

 if ( Escape )           /* wurde auf diesem Rechner Escape gedrckt? */
  {
   Token = TOK_ESCAPE;                     /* Ja, Escape-Token senden */
   Anzahl = 0;
  }

 /*-- zunchst den Header senden -------------------------------------*/

 header.Token = Token;                             /* Header aufbauen */
 header.Len = Anzahl;

 for ( try = MAX_TRY; try; --try )    /* maximal MAX_TRY Fehlversuche */
  {
   ok = TRUE;      /* zunchst von ferhlerfreier bertragung ausgehen */
   for ( bptr = (BYTE *) &header, i = sizeof( header); i; --i )
     ok = ok & SendeByte( *bptr++ );               /* ein Byte senden */

   ok = ok & SendeByte( (BYTE) (ok ? ACK : NAK) );  /*Besttig. senden*/
   if ( ok )                             /* alles richtig bertragen? */
     break;                    /* Ja, kein weiterer Versuch notwendig */
  }

 if ( try == 0 )              /* konnte der Header bertragen werden? */
  longjmp( RueckSprung, LJ_DATA );     /* Nein, bertragung abbrechen */

 if ( Token == TOK_ESCAPE )         /* wurde ESCAPE-Meldung gesendet? */
  longjmp( RueckSprung, LJ_ESCAPE );     /* Ja, bertragung abbrechen */

 /*-- den eigentlichen Datenblock senden -----------------------------*/

 if ( Anzahl )                                   /* Lnge ungleich 0? */
  {
   for ( try = MAX_TRY; try; -- try ) /* maximal MAX_TRY Fehlversuche */
    {
     ok = TRUE;              /* von fehlerfreier bertragung ausgehen */
     for ( bptr = (BYTE *) Daten, i = Anzahl; i; --i )
       ok = ok & SendeByte( *bptr++ ); /* Byte send. und Status abfr. */

     ok = ok & SendeByte( (BYTE) (ok ? ACK : NAK) ); /*Besttig.senden*/
     if ( ok )                           /* alles richtig bertragen? */
       break;                  /* Ja, kein weiterer Versuch notwendig */
    }
   if ( try == 0 )        /* konnte der Datenblock bertragen werden? */
    longjmp( RueckSprung, LJ_DATA );   /* Nein, bertragung abbrechen */
  }

 /*-- ESCAPE-Byte vom Empfnger abfragen -----------------------------*/

 for ( try = MAX_TRY; try; -- try )           /* Fehlversuche erlaubt */
  {
   EmpfEscape = EmpfangeByte();             /* Remote-Escape abfragen */
   if ( EmpfEscape == (BYTE) TRUE  ||  EmpfEscape == (BYTE) FALSE )
    break;                                 /* Escape-Status empfangen */
  }
 if ( try == 0 )                /* wurde der Escape-Status empfangen? */
  longjmp( RueckSprung, LJ_DATA );     /* Nein, bertragung abbrechen */

 if ( EmpfEscape )                  /* Escape auf dem Remote-Rechner? */
  longjmp( RueckSprung, LJ_REMESCAPE );  /* Ja, bertragung abbrechen */
}

/**********************************************************************/
/* EmpfangeBlock: Empfngt einen Datenblock                           */
/* Eingabe   : TOKEN  = Zeiger auf Variable zur Aufnahme des Tokens   */
/*             LEN    = Zeiger auf Variable zur Aufnahme der Lnge    */
/*             DATEN  = Zeiger auf Puffer zur Aufnahme der Daten      */
/* Ausgabe   : keine, denn bei einem Fehler wird sofort ber          */
/*             LongJmp zur Fehlerbehandlungsroutine gesprungen        */
/* Info      : Der bergebene Puffer mu fr MAXBLOCK Bytes Platz     */
/*             bieten, da die Blocklnge in der Regel nicht anti-     */
/*             zipiert werden kann                                    */
/**********************************************************************/

void EmpfangeBlock( BYTE *Token, int *Len, void *Daten )
{
 BHEADER header;              /* zur Aufnahme des empfangenen Headers */
 BYTE    *bptr;                       /* Laufzeiger in Empfangspuffer */
 int     ok,                                           /* Fehler-Flag */
         i,                                        /* Schleifenzhler */
         try,                     /* verbleibende Anzahl der Versuche */
         EscapeStatus;   /* zum Speichern des aktuellen Escape-Status */

 /*-- Zunchst den Header empfangen ----------------------------------*/

 for ( try = MAX_TRY; try; -- try )   /* maximal MAX_TRY Fehlversuche */
  {
   for ( bptr = (BYTE *) &header, i = sizeof(header); i; --i )
     *bptr++ = EmpfangeByte( );

   if ( EmpfangeByte() == ACK );     /* alle Bytes richtig empfangen? */
     break;                    /* Ja, kein erneuter Versuch notwendig */
  }

 if ( try == 0 )               /* wurde der Header richtig empfangen? */
  longjmp( RueckSprung, LJ_DATA );     /* Nein, bertragung abbrechen */

 if ( ( *Token = header.Token ) == TOK_ESCAPE )     /* Sender ESCAPE? */
  longjmp( RueckSprung, LJ_REMESCAPE );  /* Ja, bertragung abbrechen */

 /*-- der Header war o.k., jetzt den eigentlichen Datenblock empf. ---*/

 if ( ( *Len = header.Len ) != 0 )                /* kein Datenblock? */
  {                                                           /* doch */
   for ( try = MAX_TRY; try; -- try ) /* maximal MAX_TRY Fehlversuche */
    {
     for ( bptr = (BYTE *) Daten, i = header.Len; i; --i )
       *bptr++ = EmpfangeByte( );

     if ( EmpfangeByte() == ACK );   /* alle Bytes richtig empfangen? */
      break;                   /* Ja, kein erneuter Versuch notwendig */
    }

  if ( try == 0 )               /* wurde der Block richtig empfangen? */
   longjmp( RueckSprung, LJ_DATA );    /* Nein, bertragung abbrechen */
  }

 /*-- aktuellen Escape-Status an den Remote-Rechner senden -----------*/

 EscapeStatus = Escape;                              /* Status merken */
 for ( try = MAX_TRY; try; -- try )      /* Fehlversuche sind erlaubt */
  {
   if ( SendeByte( (BYTE) (EscapeStatus != 0) ) )      /* angekommen? */
    break;                     /* Ja, kein weiterer Versuch notwendig */
  }

 if ( try == 0 )            /* konnte der ESC-Status gesendet werden? */
   longjmp( RueckSprung, LJ_DATA );    /* Nein, bertragung abbrechen */

 if ( EscapeStatus )      /* wurde auf diesem Rechner Escape gedrckt */
  longjmp( RueckSprung, LJ_ESCAPE );     /* Ja, bertragung abbrechen */
}

/**********************************************************************/
/* SendeDatei: Sendet eine Datei                                      */
/* Eingabe   : NAME = Zeiger auf den Puffer mit dem Dateinamen        */
/* Ausgabe   : keine                                                  */
/**********************************************************************/

void SendeDatei( char *Name )
{
 int           Status;                                 /* Sendestatus */
 WORD          Gelesen;                  /* Anzahl eingelesener Bytes */
 unsigned long Groesse;                    /* Anzahl gesendeter Bytes */

 printf( "%-13s", Name );
 Datei = fopen( Name, "rb" );                       /* Datei erffnen */
 SendeBlock( TOK_DATSTART, strlen(Name)+1, Name );  /* Datein. senden */

 /*-- Dateiinhalt bertragen -----------------------------------------*/

 Groesse = 0;
 do
  {
   Gelesen = fread( BlockBuf, 1, MAXBLOCK, Datei ); /* Block einlesen */
   if ( Gelesen > 0 )                    /* Ende noch nicht erreicht? */
    {                                                         /* Nein */
     SendeBlock( TOK_DATNEXT, Gelesen, BlockBuf );    /* Block Senden */
     Groesse += Gelesen;
     printf( "\r%-13s (%ld)", Name, Groesse );
    }
  }
 while (  Gelesen > 0 );
 printf( "\n" );

 SendeBlock( TOK_DATEND, 0, NULL );        /* bertragung abschlieen */

 fclose( Datei );                                 /* Datei schliessen */
 Datei = NULL;                               /* Datei ist geschlossen */
}

/**********************************************************************/
/* EmpfangeDatei: Empfngt eine Datei                                 */
/* Eingabe      : keine                                               */
/* Ausgabe      : Das letzte empfangene Token                         */
/**********************************************************************/

int EmpfangeDatei( void )
{
 int           Status;                                 /* Sendestatus */
 WORD          ZuSchreiben;               /* Gre des letzten Blocks */
 unsigned long Groesse;
 BYTE          Token;                            /* Empfangenes Token */
 int           Len;                          /* Empfangene Blocklnge */
 char          Name[13];                                 /* Dateiname */

 EmpfangeBlock( &Token, &Len, BlockBuf );
 if ( Token == TOK_DATSTART )
  {
   strcpy( Name, BlockBuf );
   Datei = fopen( Name, "wb" );          /* Datei erffnen (erzeugen) */
   printf( "%-13s", Name );

   /*-- Dateiinhalt empfangen ----------------------------------------*/

   Groesse = 0;
   do
    {
     EmpfangeBlock( &Token, &Len, BlockBuf );      /* Block empfangen */
     if ( Token == TOK_DATNEXT )              /* nchster Datenblock? */
      {                                                         /* Ja */
       fwrite( BlockBuf, 1, Len, Datei );          /* Block schreiben */
       Groesse += Len;
       printf( "\r%-13s (%ld)", Name, Groesse );
      }
    }
   while ( Token == TOK_DATNEXT );
   fclose( Datei );                        /* Datei wieder schliessen */
   Datei = NULL;                             /* Datei ist geschlossen */
   printf( "\n" );
  }
 return Token;                           /* Fehlerstatus zurckmelden */
}

/**********************************************************************/
/*                  H A U P T P R O G R A M M                         */
/**********************************************************************/

void main( int argc, char *argv[] )
{
 DIRSTRUCT SRec;                      /* Struktur fr Directory-Suche */
 BYTE      Sender;           /* bertragungsmodus (Senden, Empfangen) */
 int       sjStatus,                             /* Code von longjmup */
           Nummer,                        /* Nummer der Schnittstelle */
           i,                                      /* Schleifenzhler */
           Gefunden;                           /* Fr Directory-Suche */

 static char *Meldung[ 9 ] =
  { "ENDE: Alle Dateien erfolgreich gesendet.",
    "ENDE: Alle Dateien erfolgreich empfangen.",
    "FEHLER: Time-Out, Remote-System antwortet nicht!",
    "ENDE: Abbruch durch Escape.",
    "ENDE: Abbruch auf der Remote-Maschine durch Escape.",
    "FEHLER: Schnittstelle oder Kabel defekt!",
    "FEHLER: Kein Kontakt zu Remote-Rechner!",
    "FEHLER: Angegebene Schnittstelle existiert nicht!",
    "FEHLER: Unbekannte oder ungltige Parameter!" };

 printf( "\n\nDatenbertragung ber die parallele Schnittstelle   " );
 printf( "(c) 1991 by Michael Tischer\n" );
 printf( "====================================================" );
 printf( "===========================\n\n" );

 if ( strcmp( argv[ 1 ], "?" ) == 0 )         /* nur Syntax ausgeben? */
  {                                                             /* Ja */
   printf( "Aufruf: plinkc [/Pn] [/Tnn] [Dateiname]\n" );
   exit ( 0 );
  }

 sjStatus = setjmp( RueckSprung );   /* Rcksprungadr. zur Beendigung */
 if ( sjStatus )                               /* Longjmp aufgerufen? */
  {                                                             /* Ja */
   IntrRemove( );                   /* Interrupt-Handler deaktivieren */
   if ( Datei )                      /* war noch eine Datei geffnet? */
     fclose( Datei );
   free( BlockBuf );       /* Belegten Speicherplatz wieder freigeben */
   printf( "\n\n%s\n", Meldung[ sjStatus - 1 ] );
   exit( 0 );
  }

 BlockBuf = malloc( MAXBLOCK );           /* Puffer fr Daten anlegen */
 IntrInstall( &Escape, &TO_Count );         /* Interrupt-Handler init */

 /*-- Defaultwerte setzen und Kommmandozeile auswerten ---------------*/

 Sender = FALSE;                             /* Default ist Empfnger */
 Nummer = 1;                                      /* Default ist LPT1 */

 for ( i = 1; i < argc; i++ )
  {
   if ( argv[i][0] == '/' )                              /* Parameter */
    {
     switch ( toupper( argv[i][1] ) )
      {
       case 'T' : Timeout = (atol( &argv[i][2] ) * TENSEK) / 10;
                  if ( Timeout == 0 )
                   longjmp( RueckSprung, LJ_PARA );       /* ungltig */
                  break;
       case 'P' : Nummer = argv[i][2] - 48;          /* Schnittstelle */
                  if ( Nummer == 0   ||   Nummer > 4 )
                   longjmp( RueckSprung, LJ_PARA );       /* ungltig */
                  break;
       default  : longjmp( RueckSprung, LJ_PARA );     /* unbek. Para */
                  break;
      }
     argv[i][0] = '\0';                           /* Argument lschen */
    }
   else                         /* kein Parameter, mu Dateiname sein */
    Sender = TRUE;                                          /* Sender */
  }

 /*-- bertragung starten --------------------------------------------*/

 if ( GetPortAdr(Nummer) == FALSE )    /* existiert die Schnittstelle */
  longjmp( RueckSprung, LJ_NOPAR );                   /* Nein, Fehler */

 if ( Port_Init( Sender ) == FALSE )           /* Verbindung aufbauen */
  longjmp( RueckSprung, LJ_NOLINK );         /* Fehler, nicht mglich */

 if ( Sender )                                             /* Sender? */
  {
   printf( "Senden auf LPT%d:\n\n", Nummer );

   /*-- Alle Dateien bertragen --------------------------------------*/

   for ( i = 1; i < argc; i++ )          /* Kommandozeile durchlaufen */
    {
     if ( argv[i][0] != '\0' )                          /* Dateiname? */
      {                                                         /* Ja */
       Gefunden = FINDFIRST( argv[i], &SRec, 0);
       while ( !Gefunden )
        {
         if ( SRec.DATEINAME[0] != '.' )
          SendeDatei( SRec.DATEINAME );             /*Datei bertragen*/
         Gefunden = FINDNEXT( &SRec );
        }
      }
    }
   SendeBlock( TOK_ENDE, 0 , NULL );         /* alle Dateien gesendet */
   longjmp( RueckSprung, LJ_OKSENDER );
  }
 else                                              /* Nein, Empfnger */
  {
   printf( "Empfangen auf LPT%d:\n\n", Nummer );
   while ( EmpfangeDatei() != TOK_ENDE )         /* Dateien empfangen */
    ;                                            /* bis ENDE-Token    */
   longjmp( RueckSprung, LJ_OKEMPF );
  }
}
