/**********************************************************************/
/*                         R E C L O C K C . C                        */
/*--------------------------------------------------------------------*/
/*    Aufgabe       : Demonstrioert das Record-Locking mit Hilfe der  */
/*                    Funktionen aus dem NETFILEC-Modul               */
/*--------------------------------------------------------------------*/
/*    Speichermodell: SMALL                                           */
/*--------------------------------------------------------------------*/
/*    Autor         : Michael Tischer                                 */
/*    entwickelt am : 10.02.1992                                      */
/*    letztes Update: 13.02.1991                                      */
/**********************************************************************/

#include "netfilec.c"               /* die Netzwerkroutinen einbinden */
#include <stdio.h>

/*== Konstanten ======================================================*/

#define DATEINAME "reclockc.dat"           /* Dateiname der Testdatei */
#define SAETZE    10                         /* Anzahl der Datenstze */

/*== Typdefinitionen =================================================*/

typedef char Test[ 160 ];                        /* Datentyp fr Test */
typedef char TestString[ 161 ];     /* Datentyp fr Bildschirmausgabe */

#ifndef __TURBOC__                                    /* Microsoft C? */

  #define clrscr() clearwindow( 1, 1, 80, 25 )

  /********************************************************************/
  /* Gotoxy        : setzt den Cursor                                 */
  /* Eingabe       : Koordinaten des Cursors                          */
  /* Ausgabe       : keine                                            */
  /********************************************************************/

  void gotoxy( int x, int y )

  {
   regs.h.ah = 0x02;          /* Funktionsnummer fr Interrupt-Aufruf */
   regs.h.bh = 0;                                            /* Farbe */
   regs.h.dh = y - 1;
   regs.h.dl = x - 1;
   int86( 0x10, &regs, &regs );                    /* InterruptAufruf */
  }
#endif

/**********************************************************************/
/* clearwindow   : lscht einen Bildschirmbereich                     */
/* Eingabe       : s.u.                                               */
/* Ausgabe       : keine                                              */
/**********************************************************************/

void clearwindow( int x1, int y1, int x2, int y2 )
{
 regs.h.ah = 0x07;            /* Funktionsnummer fr Interrupt-Aufruf */
 regs.h.al = 0x00;
 regs.h.bh = 0x07;
 regs.h.ch = y1 - 1;
 regs.h.cl = x1 - 1;
 regs.h.dh = y2 - 1;
 regs.h.dl = x2 - 1;
 int86( 0x10, &regs, &regs );                      /* InterruptAufruf */
 gotoxy( x1, y1 );                                   /* Cursor setzen */
}

/**********************************************************************/
/* OeffneDatei : Vorhandene Netzwerkdatei erffnen. Anderenfalls      */
/*               erzeugen einer neuen Testdatei und fllen der Datei  */
/*               mit Testdatenstzten                                 */
/* Eingabe     : s.u.                                                 */
/* Ausgabe     : Datei                                                */
/**********************************************************************/

int OeffneDatei( NFILE *Datei )                      /* Netzwerkdatei */
{
 int  i;                                           /* Schleifenzhler */
 Test TestSatz;           /* notwendig fr das Erzeugen der Testdaten */

 /*-- Datei fr Ein- u. Ausgabe im Deny-None-Modus erffnen ----------*/

 NetReset( DATEINAME, FM_RW | SM_NO, sizeof( Test ), Datei );
 if ( NetError == NE_FileNotFound )         /* Datei nicht vorhanden? */
 {
  /*-- Datei erzeugen und mit Testdatenstzen fllen -----------------*/

  NetRewrite( DATEINAME, FM_RW | SM_NO, sizeof( Test ), Datei );
  if ( NetError == NE_OK )                   /* Erzeugen ohne Fehler? */
  {
   if ( NetLock( Datei, 0L, (long) SAETZE ) ) /* alle Records sperren */
   {
    NetSeek( Datei, 0L );          /* Zeiger auf den Anfang der Datei */
    for ( i = 0; i < SAETZE; i++ )
    {
     memset( TestSatz, 'A' + i, 160 );
     NetWrite( Datei, TestSatz );              /* Testdaten schreiben */
    }
    return NetUnLock( Datei, 0L, (long) SAETZE );
   }
   else
    return FALSE;                              /* Fehler beim Locking */
  }
  else
   return FALSE;                    /* Fehler beim Erzeugen der Datei */
 }
 else
  return ( NetError == 0 );                    /* ffnen ohne Fehler? */
}

/**********************************************************************/
/* NetzEdit    : Demonstration der Netzwerkfunktionen                 */
/* Eingabe     : s.u                                                  */
/* Ausgabe     : Datei                                                */
/**********************************************************************/

void NetzEdit( NFILE *TestFile )                     /* Netzwerkdatei */
{
 unsigned long AktRecord;                    /* Aktuelle Recordnummer */
 TestString    AktSatz;                        /* Aktueller Datensatz */
 int           Aktion;                           /* gewnschte Aktion */
 int           Status;                             /* Record gelockt? */
 char          Zeichen[ 10 ];
 char          SDummy[ 50 ];                        /* Netzwerkstatus */
 int           Gesperrt[ SAETZE ];
 int           i;                                  /* Schleifenzhler */

 /*-- Men ausgeben --------------------------------------------------*/

 printf( "Mgliche Funktionen\n" );
 printf( "  1: Dateizeiger positionieren\n" );
 printf( "  2: Record sperren\n" );
 printf( "  3: Record lesen\n" );
 printf( "  4: Datensatz ndern\n" );
 printf( "  5: Record schreiben\n" );
 printf( "  6: Sperre aufheben\n" );
 printf( "  7: Ende\n" );

 /*-- Status der Datenstze initialisieren und ausgeben --------------*/

 gotoxy( 58, 4 );
 printf( "Status der Datenstze:" );
 for ( i = 0; i < SAETZE; ++i )
  {
   Gesperrt[i] = FALSE;
   gotoxy( 64, i+5 );
   printf( "%2d   frei", i );
  }

 AktRecord = 0;                                /* Aktueller Datensatz */
 Status = FALSE;                              /* Record nicht gelockt */
 memset( AktSatz, 32, 160 );             /* Leeren Datensatz erzeugen */

 do
 {
  /*-- Informationen ausgeben ----------------------------------------*/

  gotoxy( 1, 16 );              /* Position des Dateizeigers ausgeben */
  printf( "Aktueller Record: %4li\n",  AktRecord );
  printf( "Status          :    %s\n",
           Gesperrt[AktRecord] ? "gesperrt" : "frei    " );
  NetErrorMsg( NetError, SDummy );
  printf( "Netzwerkstatus  : %4i = %s", NetError, SDummy );
  gotoxy( 1, 21 );                          /* Testdatensatz ausgeben */
  printf( "Aktueller Datensatz:\n" );
  AktSatz[ 160 ] = 0;
  printf( "%s", AktSatz );

  NetSeek( TestFile, AktRecord );        /* Dateizeiger positionieren */
  gotoxy( 1, 13 );
  printf( "Auswahl:                            " );
  gotoxy( 10, 13 );
  scanf( "%i", &Aktion );
  switch( Aktion )
  {
   case 1 : gotoxy( 1, 13 );
            printf( "Neue Datensatznummer: " );
            do
             {
              gotoxy( 22, 13 );
              printf( "                      " );
              gotoxy( 22, 13 );
              scanf( "%li", &AktRecord );
             }
            while (!( AktRecord >= 0  &&  AktRecord < SAETZE ));
            break;

   case 2 : Status = NetLock( TestFile, AktRecord, 1L );
            if ( Status )
             {
              Gesperrt[ AktRecord ] = TRUE;
              gotoxy( 64, (int) AktRecord +5 );
              printf( "%2d   gesperrt", AktRecord );
             }
            break;

   case 3 : NetRead( TestFile, AktSatz );          /* Datensatz lesen */
            break;

   case 4 : gotoxy( 1, 13 );
            printf( "Neues Zeichen: " );
            scanf( "%s", Zeichen );
            memset( AktSatz, Zeichen[ 0 ], 160 );
            break;

   case 5 : NetWrite( TestFile, AktSatz );     /* Datensatz schreiben */
            break;

   case 6 : Status = NetUnLock( TestFile, AktRecord, 1L );
            if ( Status )
             {
              Gesperrt[ AktRecord ] = FALSE;
              gotoxy( 64, (int) AktRecord+5 );
              printf( "%2d   frei    ", AktRecord);
             }
            break;
  }
   
 }
 while ( Aktion != 7 );
}

/**********************************************************************/
/*                     H A U P T P R O G R A M M                      */
/**********************************************************************/

void main( )

{
 NFILE Datei;                                            /* Testdatei */

 clrscr();
 printf( "RECLOCKC Demonstration des Record-Locking mit DOS " \
         "  (c) 1992 by Michael Tischer\n" );
 printf( "====================================================" \
         "===========================\n\n" );

 if ( ShareInst() )                            /* Schare installiert? */
 {
  if ( OeffneDatei( &Datei ) )        /* Datei erffnet oder erzeugt? */
  {
   NetzEdit( &Datei );        /* Demonstration der Netzwerkfunktionen */
   NetClose( &Datei );                            /* Datei schliessen */
   clrscr( );
  }
  else
   printf( "\nFehler %i beim ffnen der Netzwerkdatei", NetError );
 }
 else
  printf( "\nTest nicht mglich, SHARE mu installiert sein" );
}
