/***********************************************************************
*                           S 3 2 2 0 C . C                            *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Arbeit mit Sprites im VGA-      *
*                     320*200-Grafikmodus bei 256 Farben und vier      *
*                     Bildschirmseiten.                                *
*                     Das Programm arbeitet dabei mit den Assembler-   *
*                     Routinen aus den Moduln S3220CA.ASM und          *
*                     V3220CA.ASM zusammen.                            *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    :  9.09.1990                                       *
*  letztes Update am: 14.01.1991                                       *
**--------------------------------------------------------------------**
*  Speichermodell   : SMALL                                            *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung       : CL /AS s3220c.c v3220ca s3220ca                  *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung       : ber eine Projekt-Datei mit folgendem Inhalt:    *
*                      s3220c.c                                        *
*                      v3220ca.obj                                     *
*                      s3220ca.obj                                     *
**--------------------------------------------------------------------**
*  Aufruf           : s3220c                                           *
***********************************************************************/

#include <dos.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <conio.h>

/*-- Compiler-abhngige Deklarationen --------------------------------*/

#ifdef __TURBOC__
  #include <alloc.h>
#else
  #include <malloc.h>
  #define random(x) ( rand() % (x+1) )             /* Zufallsfunktion */
#endif

/*-- Typdeklarationen ------------------------------------------------*/

typedef unsigned char BYTE;

typedef struct {                    /* Erscheinungsbild eines Sprites */
                BYTE breite,                         /* Breite gesamt */
                     hoehe,                    /* Hhe in Punktzeilen */
                     seite,                  /* abgelegt in Seite ... */
                     *bmskp,                  /* Zeiger auf Bit-Maske */
                     msklen;          /* Lnge jeweils eines Eintrags */
                int  zeile;       /* Punktzeile, ab der das Sprite in */
               } SPLOOK;          /* seiner Seite abgelegt wurde      */

typedef struct {                           /* Sprite-Beschreiber (ID) */
                BYTE hgseite;                     /* Hintergrundseite */
                int  x[2], y[2],      /* Koordinaten in Seite 0 und 1 */
                     hgx, hgy;              /* Puffer fr Hintergrund */
                SPLOOK * splookp;      /* Zeiger auf Erscheinungsbild */
               } SPID;

/*-- externe Referenzen auf die Assembler-Routinen -------------------*/

extern void init320200( void );
extern void setpix( int x, int y, unsigned char farbe);
extern BYTE getpix( int x, int y );
extern void setpage( BYTE page );
extern void showpage( BYTE page );
extern void far * getfontptr( void );
extern void waitvsync( void );
extern void blockmove( BYTE vonpage, int vonx, int vony,
                       BYTE nachpage, int nachx, int nachy,
                       BYTE breite, BYTE hoehe, BYTE *bmskp );

/*-- Konstanten ------------------------------------------------------*/

#define NOBITMASK (BYTE *) 0

#define MAXX 319                        /* Maximale X- und Y-Ordinate */
#define MAXY 199

#define OUT_LEFT   1      /* fr Kollisionsmarkierung in SpriteMove() */
#define OUT_TOP    2
#define OUT_RIGHT  4
#define OUT_BOTTOM 8
#define OUT_NO     0                             /* nirgendwo drauen */

/***********************************************************************
*  IsVga: Stellt fest, ob eine VGA-Karte installiert ist.              *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : 0, wenn keine VGA angetroffen wurde, sonst ungleich 0     *
***********************************************************************/

BYTE IsVga( void )
{
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */

 Regs.x.ax = 0x1a00;              /* Funktion 1Ah gibt es nur bei VGA */
 int86( 0x10, &Regs, &Regs );
 return ( Regs.h.al == 0x1a );         /* ist die Funktion verfgbar? */
}

/***********************************************************************
*  Line: Zieht eine Linie nach dem Bresenham-Algorithmus               *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endkoordinate                                    *
*            FARBE  = Farbe der Linie                                  *
***********************************************************************/

/*-- Unterfunktion, vertauscht zwei Integer-Variablen ----------------*/

void SwapInt( int *i1, int *i2 )
{
 int dummy;

 dummy = *i2;
 *i2   = *i1;
 *i1   = dummy;
}

/*-- Hauptteil der Funktion ------------------------------------------*/

void Line( int x1, int y1, int x2, int y2, BYTE farbe )
{
 int d, dx, dy,
     aincr, bincr,
     xincr, yincr,
     x, y;

 if ( abs(x2-x1) < abs(y2-y1) )       /* ber X- oder Y-Achse laufen? */
  {                                               /* ber die Y-Achse */
   if ( y1 > y2 )                                    /* y1 grer y2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   xincr = ( x2 > x1 ) ?  1 : -1;       /* Spalteninkrement festlegen */

   dy = y2 - y1;
   dx = abs( x2-x1 );
   d  = 2 * dx - dy;
   aincr = 2 * (dx - dy);
   bincr = 2 * dx;
   x = x1;
   y = y1;

   setpix( x, y, farbe );                      /* ersten Punkt setzen */
   for (y=y1+1; y<= y2; ++y )         /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       x += xincr;
       d += aincr;
      }
     else
      d += bincr;
     setpix(x, y, farbe);
    }
  }
 else                                             /* ber die X-Achse */
  {
   if ( x1 > x2 )                                    /* x1 grer x2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   yincr = ( y2 > y1 ) ? 1 : -1;         /* Zeileninkrement festlegen */

   dx = x2 - x1;
   dy = abs( y2-y1 );
   d  = 2 * dy - dx;
   aincr = 2 * (dy - dx);
   bincr = 2 * dy;
   x = x1;
   y = y1;

   setpix(x, y, farbe);                        /* ersten Punkt setzen */
   for (x=x1+1; x<=x2; ++x )          /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       y += yincr;
       d += aincr;
      }
     else
      d += bincr;
     setpix(x, y, farbe);
    }
  }
}

/***********************************************************************
*  PrintChar : Baut ein Zeichen auerhalb des sichtbaren Bereichs des  *
*              Video-RAMs auf.                                         *
**--------------------------------------------------------------------**
*  Eingabe :   zeichen = das aufzubauende Zeichen                      *
*              x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************/

void PrintChar( char zeichen, int x, int y, BYTE vg, BYTE hg )
{
 typedef BYTE ZDEF[256][8];                 /* Aufbau des Zeichensatz */
 typedef ZDEF far *ZPTR;                    /* Zeiger auf Zeichensatz */

 BYTE         i, k,                                /* Schleifenzhler */
              Maske;              /* Bitmaske fr Aufbau des Zeichens */

 static ZPTR fptr = (ZPTR) 0;        /* Zeiger auf Zeichensatz im ROM */

 if ( fptr == (ZPTR) 0 )   /* Zeiger auf Zeichensatz schon ermittelt? */
  fptr = getfontptr();  /*Nein, mit Hilfe der Assembler-Funktion laden*/

 /*- das Zeichen Punkt fr Punkt aufbauen ----------------------------*/

 if ( hg == 255 )                    /* Zeichen transparent zeichnen? */
  for ( i = 0; i < 8; ++i )      /* Ja, nur Vordergrund-Punkte setzen */
   {
    Maske = (*fptr)[zeichen][i];    /* Bitmuster fr eine Zeile holen */
    for ( k = 0; k < 8; ++k, Maske <<= 1 ) /* die Spalten durchlaufen */
     if ( Maske & 128 )                              /* Punkt setzen? */
      setpix( x+k, y+i, vg );                                   /* Ja */
   }
 else                                     /* Nein, jeden Punkt setzen */
  for ( i = 0; i < 8; ++i )                     /* Zeilen durchlaufen */
  {
   Maske = (*fptr)[zeichen][i];     /* Bitmuster fr eine Zeile holen */
   for ( k = 0; k < 8; ++k, Maske <<= 1 )  /* die Spalten durchlaufen */
    setpix( x+k, y+i, ( Maske & 128 ) ? vg : hg );
  }
}


/***********************************************************************
*  GrafPrintf: Gibt einen formatierten String im Grafikbildschirm aus  *
*  Eingabe : X, Y   = Startkoordinate (0 - ...)                        *
*            VG     = Vordergrundfarbe                                 *
*            HG     = Hintergrundfarbe (255 = transparent)             *
*            STRING = String mit Formatinformationen                   *
*            ...    = Ausdrcke wie bei Printf                         *
***********************************************************************/

void GrafPrintf( int x, int y, BYTE vg, BYTE hg, char * string, ... )
{
 va_list parameter;              /* Parameter-Liste fr VA_... Macros */
 char ausgabe[255],                 /* Puffer fr formatierten String */
      *cp;

 va_start( parameter, string );                /* Parameter umwandeln */
 vsprintf( ausgabe, string, parameter );               /* formatieren */
 for ( cp = ausgabe; *cp; ++cp, x+= 8 )    /* formatierten String     */
   PrintChar( *cp, x, y, vg, hg );         /* ber PrintChar ausgeben */
}

/***********************************************************************
*  CreateSprite: Erzeugt ein Sprite auf der Basis einer bereits kompi- *
*                lierten Punktmusters                                  *
**--------------------------------------------------------------------**
*  Eingabe : SPLOOKP = Zeiger auf die von CompileSpriteI() erzeugte    *
*                      Datenstruktur                                   *
*            HGSEITE = Bildschirmseite, in der der Sprite-Hintergrund  *
*                      aufbewahrt werden soll                          *
*            HGX,   =  Koordniate in der HGSEITE, ab der der Hinter-   *
*            HGY       grund des Sprites abgelegt werden soll          *
*  Ausgabe : Zeiger auf die erstellte Sprite-Struktur                  *
*  Info    : Fr die Ablege des Sprite-Hintergrundes werden zwei ne-   *
*            beneinanderliegende Flchen von der Gre des jeweiligen  *
*            Sprites bentigt.                                         *
***********************************************************************/

SPID *CreateSprite( SPLOOK *splookp, BYTE hgseite, int hgx, int hgy )
{
 SPID *spidp;                  /* Zeiger auf erzeugte Sprite-Struktur */

 spidp = (SPID *) malloc( sizeof(SPID) );   /* Sprite-Struktur allok. */
 spidp->splookp = splookp;                    /* Daten in die Sprite- */
 spidp->hgseite = hgseite;                    /* Struktur eintragen   */
 spidp->hgx = hgx;
 spidp->hgy = hgy;

 return spidp;            /* Zeiger auf Sprite-Struktur zurckliefern */
}

/***********************************************************************
*  CompileSprite: Erzeugt das Punkt- und Bitmusters eines Sprites,     *
*                 auf der Basis einer zur Laufzeit angegebenen Sprite- *
*                 definition                                           *
**--------------------------------------------------------------------**
*  Eingabe : BUFP    = Zeiger auf Array mit Zeigern auf Strings, die   *
*                      das Zeichenmuster des Sprites enthalten         *
*            HOEHE   = Hoehe des Sprites und damit Anzahl der Strings  *
*            SEITE   = Grafik-Seite zum Aufbau des Sprites             *
*            Y       = Punktzeile, in der das Sprite aufgebaut wird    *
*            FB      = ASCII-Zeichen, das fr die kleinste Farbe steht *
*            VGFARBE = erste Farbcode fr FB                           *
*  Info    : Die Sprites werden in der angegebenen Punktzeile ab dem   *
*            linken Zeilenrand angelegt                                *
***********************************************************************/

SPLOOK *CompileSprite( char **bufp, BYTE hoehe, BYTE seite, BYTE y,
                       char fb, BYTE vgfarbe )
{
 BYTE   breite,                                 /* Breite der Strings */
        c,                    /* nimmt Zeichen aus C-Sprite-Array auf */
        i, k, l,                                /* Schleifenvariablen */
        pixc,             /* Punktzhler fr Erstellung der Bit-Maske */
        pixm,                                           /* Punktmaske */
        *lspb;                         /* Laufzeiger in Sprite-Puffer */
 int    abstand,        /* Abstand von Sprite-Anfang zu Sprite-Anfang */
        lx, ly;                                    /* Laufkoordinaten */
 SPLOOK *splookp;              /* Zeiger auf erzeugte Sprite-Struktur */

 /*-- Sprite-Look-Struktur erzeugen und mit Daten fllen -------------*/

 splookp = (SPLOOK *) malloc( sizeof(SPLOOK) );
 breite = strlen( *bufp );
 splookp->breite = abstand = ( ( breite + 3 + 3 ) / 4 ) * 4;
 splookp->bmskp = (BYTE *) malloc( (abstand*hoehe+7)/8*4 );
 splookp->hoehe = hoehe;
 splookp->zeile = y;
 splookp->seite = seite;
 splookp->msklen = (abstand*hoehe+7)/8;

 /*-- Sprite-Hintergrund in seiner Home-Seite mit Codes fr den ------*/
 /*-- transparenten Zeichenhintergrund fllen                   ------*/

 setpage( seite );                    /* Seite zum Zeichnen festlegen */
 for ( ly = y + hoehe-1, lx = 4 * abstand - 1; ly >= (int) y; --ly)
   Line( 0, ly, lx, ly, 255 );

 /*-- Sprite in seiner Home-Seite viermal nebeneinander aufbauen -----*/

 for ( l = 0, lx = 0; l < 4; ++l, lx+=abstand+1 )
  for ( i = 0; i < hoehe; ++i )
   for ( k = 0; k < breite; ++k )
     setpix( lx+k, y+i, ( c = *(*(bufp+i)+k) ) == ' ' ? 255
                                                      : vgfarbe+(c-fb));

 /*-- die vier erzeugten Sprites durchlaufen und dabei Bit-Maske fr -*/
 /*-- das Kopieren der Sprites in die Bit-Planes erzeugen            -*/

 for ( l = pixm = pixc = 0, lx = 0; l < 4; ++l, lx+=abstand )
  {
   lspb = splookp->bmskp + splookp->msklen * l;
   for ( i = 0; i < hoehe; ++i )
    for ( k = 0; k < abstand; ++k )
     {
      pixm >>= 1;       /* Pixelmaske um ein Bit nach rechts schieben */
      if ( getpix( lx+k, y+i ) != 255 )         /* Hintergrund-Punkt? */
       pixm |= 128;                  /* Nein, Bit fr Mit-Mask setzen */
      if ( ++pixc == 8 )             /* schon acht Punkte bearbeitet? */
       {                     /* Ja Bit-Maske im Sprite-Puffer ablegen */
        *lspb++ = pixm;
        pixc = pixm = 0;       /* Pixelzhler und -maske wieder auf 0 */
       }
     }
    if ( pixc )                /* letztes Nibbl noch nicht im Puffer? */
     {                                            /* Nein, jetzt aber */
      *lspb = pixm >> 4;      /* Hi-Nibble in das Lo-Nibble und absp. */
      pixc = pixm = 0;         /* Pixelzhler und -maske wieder auf 0 */
     }
   }
 return splookp;        /* Zeiger auf den Sprite-Puffer zurckliefern */
}

/***********************************************************************
*  PrintSprite : Sprite in einer bestimmten Seite ausgeben             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in der das Sprite gezeichnet werden soll   *
*                    (0 oder 1)                                        *
***********************************************************************/

void PrintSprite( register SPID *spidp, BYTE seite )
{
 BYTE breite;                            /* Breite des Sprites gesamt */
 int  x;                  /* X-Koordinate des Sprites in seiner Seite */
 SPLOOK *splookp;      /* Zeiger auf das Erscheinungsbild des Sprites */

 breite = (splookp = spidp->splookp)->breite;
 x = spidp->x[seite];
 blockmove( splookp->seite, breite * (x % 4), splookp->zeile, seite,
            x & ~3, spidp->y[seite], breite, splookp->hoehe,
            splookp->bmskp + (x % 4) * splookp->msklen );
}

/***********************************************************************
*  GetSpriteBg: Hintergrund eines Sprites holen und an der dafr vor-  *
*               gesehenen Position ablegen                             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, aus der der Hintergrund geholt werden soll *
*                    (0 oder 1)                                        *
***********************************************************************/

void GetSpriteBg( register SPID *spidp, BYTE seite )
{
 SPLOOK *splookp;      /* Zeiger auf das Erscheinungsbild des Sprites */

 splookp = spidp->splookp;
 blockmove( seite, spidp->x[seite], spidp->y[seite],
            spidp->hgseite, spidp->hgx + ( splookp->breite * seite ),
            spidp->hgy, splookp->breite, splookp->hoehe, NOBITMASK );
}

/***********************************************************************
*  RestoreSpriteBg: Zuvor gesicherten Hintergrund eines Sprites wieder *
*                   in die ursprngliche Grafikseite kopieren          *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in die der Hinntergrund zurckkopiert      *
*                    werden soll (0 oder 1)                            *
***********************************************************************/

void RestoreSpriteBg( register SPID *spidp, BYTE seite )
{
 SPLOOK *splookp;      /* Zeiger auf das Erscheinungsbild des Sprites */

 splookp = spidp->splookp;
 blockmove( spidp->hgseite, spidp->hgx + ( splookp->breite * seite ),
            spidp->hgy, seite, spidp->x[seite], spidp->y[seite],
            splookp->breite, splookp->hoehe, NOBITMASK );
}

/***********************************************************************
*  MoveSprite: Sprite innerhlicherten Hintergrund eines Sprites wieder *
*                   in die ursprngliche Grafikseite kopieren          *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP  = Zeiger auf die Sprite-Struktur                   *
*            SEITE  = Seite, in die der Hintergrund zurckkopiert      *
*                     werden soll (0 oder 1)                           *
*            DELTAX = Bewegungszhler in X- und Y-Richtung             *
*            DELTAY                                                    *
*  Ausgabe : Kollisionsmarkierung, siehe Konstanten OUT_...            *
***********************************************************************/

BYTE MoveSprite( SPID *spidp, BYTE seite, int deltax, int deltay )
{
 int  neux, neuy;                           /* neue Sprite-Koordinate */
 BYTE out;                             /* Kollision mit Rand anzeigen */

 /*-- X-Koordinate verschieben und Kollision mit Rand testen ---------*/

 if ( ( neux = spidp->x[seite] + deltax ) < 0 )
  {
   neux = 0 - deltax - spidp->x[seite];
   out = OUT_LEFT;
  }
 else
  if ( neux > 319 - spidp->splookp->breite )
   {
    neux = 640-neux-2*(spidp->splookp->breite);
    out = OUT_RIGHT;
   }
  else
    out = OUT_NO;

 /*-- Y-Koordinate verschieben und Kollision mit Rand testen ---------*/

 if ( ( neuy = spidp->y[seite] + deltay ) < 0 )       /* oberer Rand? */
  {                                    /* Ja, deltay mu negativ sein */
   neuy = 0 - deltay - spidp->y[seite];
   out |= OUT_TOP;
  }
 else
  if ( neuy + spidp->splookp->hoehe > 199+1  )       /* unterer Rand? */
   {                                   /* Ja, deltay mu positiv sein */
    neuy = 400-neuy-2*(spidp->splookp->hoehe);
    out |= OUT_BOTTOM;
   }

 /*-- neue Position nur setzen, wenn gegenber der alten verndert ---*/

 if ( neux != spidp->x[seite]  ||  neuy != spidp->y[seite] )
  {                                              /* ist neue Position */
   RestoreSpriteBg( spidp, seite );       /* Hintergrund zurcksetzen */
   spidp->x[seite] = neux;                 /* neue Koordinaten merken */
   spidp->y[seite] = neuy;
   GetSpriteBg( spidp, seite );            /* neuen Hintergrund holen */
   PrintSprite( spidp, seite );  /* Sprite in angegeb. Seite zeichnen */
  }
 return out;
}

/***********************************************************************
*  SetSprite: Sprite an eine bestimmte Position versetzen              *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            x0, y0 = Sprite-Koordinate fr die Seite 0                *
*            x1, y1 = Sprite-Koordinate fr die Seite 1                *
*  Info    : Der Aufruf dieser Funktion sollte dem ersten Aufruf von   *
*            MoveSprite() vorausgehen                                  *
***********************************************************************/

void SetSprite( SPID *spidp, int x0, int y0, int x1, int y1 )
{
 spidp->x[0] = x0;           /* Koordinaten in Sprite-Struktur merken */
 spidp->x[1] = x1;
 spidp->y[0] = y0;
 spidp->y[1] = y1;

 GetSpriteBg( spidp, 0 );              /* Sprite-Hintergrund in Seite */
 GetSpriteBg( spidp, 1 );              /* 0 und 1 holen               */
 PrintSprite( spidp, 0 );                        /* Sprite in Seite 0 */
 PrintSprite( spidp, 1 );                        /* und 1 zeichnen    */
}

/***********************************************************************
*  RemoveSpriote: Entfernt ein Sprite von seiner augenblicklichen      *
*                 Position und macht es dadurch unsichtbar             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*  Info    : Nach dem Aufruf dieser Funktion mu erst wieder die Funk- *
*            tion SetSprite() auferufen werden, bevor das Sprite ber  *
*            MoveSprite() bewegt werden kann                           *
***********************************************************************/

void RemoveSprite( SPID *spidp )
{
 RestoreSpriteBg( spidp, 0 );          /* Sprite-Hintergrund in Seite */
 RestoreSpriteBg( spidp, 1 );          /* 0 und 1 wieder zurcksetzen */
}

/***********************************************************************
*  Demo: Demonstriert den Einsatz der verschiedenen Funktionen aus     *
*        diesem Modul                                                  *
***********************************************************************/

void Demo( void )
{
 static char *GleiterRauf[20] =
                { "               AA               ",
                  "              AAAA              ",
                  "              AAAA              ",
                  "               AA               ",
                  "             GGBBGG             ",
                  "            GBBCCBBG            ",
                  "           GBBBCCBBBG           ",
                  "          GBBBBBBBBBBG          ",
                  "          GBBBBBBBBBBG          ",
                  " G       GBBBBBBBBBBBBG       G ",
                  "GCG     GGDBBBBBBBBBBDGG     GCG",
                  "GCG   GGBBBDBBB  BBBDBBBGG   GCG",
                  "GCBGGGBBBBBDBB    BBDBBBBBGGGBCG",
                  "GCBBBBBBBBBBDB    BDBBBBBBBBBBCG",
                  "BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB",
                  "GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ",
                  "  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ",
                  "    GGBBDDDDDGGGGGDDDDDDBBG     ",
                  "      GDDDDGGG    GGGDDDDG      ",
                  "       DDDD          DDDD       "  };


 static char *GleiterRunter[20] =
                {
                  "       DDDD          DDDD       ",
                  "      GDDDDGGG    GGGDDDDG      ",
                  "    GGBBDDDDDGGGGGDDDDDDBBG     ",
                  "  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ",
                  "GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ",
                  "BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB",
                  "GCBBBBBBBBBBDB    BDBBBBBBBBBBCG",
                  "GCBGGGBBBBBDBB    BBDBBBBBGGGBCG",
                  "GCG   GGBBBDBBB  BBBDBBBGG   GCG",
                  "GCG     GGDBBBBBBBBBBDGG     GCG",
                  " G       GBBBBBBBBBBBBG       G ",
                  "          GBBBBBBBBBBG          ",
                  "          GBBBBBBBBBBG          ",
                  "           GBBBCCBBBG           ",
                  "            GBBCCBBG            ",
                  "             GGBBGG             ",
                  "               AA               ",
                  "              AAAA              ",
                  "              AAAA              ",
                  "               AA               " };

 #define SPRANZ 6                               /* Anzahl der Sprites */
 #define BREITE 38         /* Breite der Copyright-Meldung in Zeichen */
 #define HOEHE  6                                   /* Hhe in Zeilen */
 #define SX     (MAXX-(BREITE*8)) / 2              /* Startordinate X */
 #define SY     (MAXY-(HOEHE*8)) / 2               /* Startordinate Y */

 struct {                       /* beschreibt die verwalteten Sprites */
          SPID *spidp;                        /* Zeiger auf Sprite-Id */
          int  deltax[2],          /* X-Verschiebung fr Page 0 und 1 */
               deltay[2];          /* Y-Verschiebung fr Page 0 und 1 */
        } sprites[ SPRANZ ];
 BYTE   page,                            /* aktuell bearbeitete Seite */
        out;                   /* nimmt Flags fr Seitenkollision auf */
 int    x, y, i,                                   /* Schleifenzhler */
        dx, dy;                                     /* Bewegungswerte */
 char   lc;
 SPLOOK *gleiterupp, *gleiterdnp;           /* Zeiger auf die Sprites */

 srand( *(long far *) 0x0040006c );     /* Zufallsgen. initialisieren */

 /*-- die ersten beiden Grafikseiten mit Buchstaben fllen -----------*/

 for ( page = 0; page < 2; ++ page )
  {
   setpage( page );
   for ( lc = 0, y = 0; y < 200-8; y += 12 )
     for ( x = 0; x < 320-8; x += 8 )
       GrafPrintf( x, y, lc % 255, 255, "%c", lc++ & 127 );

   /*-- Copyright-Meldung ausgeben -----------------------------------*/

   Line( SX-1, SY-1, SX+BREITE*8, SY-1, 15 );
   Line( SX+BREITE*8, SY-1, SX+BREITE*8, SY+HOEHE*8,15 );
   Line( SX+BREITE*8, SY+HOEHE*8, SX-1, SY+HOEHE*8, 15 );
   Line( SX-1, SY+HOEHE*8, SX-1, SY-1, 15 );
   GrafPrintf( SX, SY,    15, 4,
               "                                      "  );
   GrafPrintf( SX, SY+8,  15, 4,
               " S3220C.C (c) 1990 by Michael Tischer " );
   GrafPrintf( SX, SY+16, 15, 4,
               "                                      "  );
   GrafPrintf( SX, SY+24, 15, 4,
               "  Sprite-Demo fuer den 320*200 Punkte "  );
   GrafPrintf( SX, SY+32, 15, 4,
               "  Grafikmodus von VGA-Karten          "  );
   GrafPrintf( SX, SY+40, 15, 4,
               "                                      "  );
  }

 /*-- Vorlagen fr die verschiedenen Sprites aufbauen ----------------*/

 gleiterupp = CompileSprite( GleiterRauf,   20, 2, 0, 'A', 1 );
 gleiterdnp = CompileSprite( GleiterRunter, 20, 2, 40, 'A', 1 );

 /*-- die verschiedenen Sprites erzeugen -----------------------------*/

 for ( i = 0; i < SPRANZ ; ++ i)
  {
   sprites[ i ].spidp = CreateSprite( gleiterupp, 3, ( i % 3 ) * 100,
                                      (i / 3) * 30 );
   do                            /* Bewegungswerte der Sprites whlen */
    {
     dx = 0;
     dy = random(8) - 4;
    }
   while ( dx==0  &&  dy==0 );

   sprites[ i ].deltax[0] = sprites[ i ].deltax[1] = dx * 2;
   sprites[ i ].deltay[0] = sprites[ i ].deltay[1] = dy * 2;

   x = ( 320 / SPRANZ * i ) + (320 / SPRANZ - 40) / 2 ;
   y = random( 200 - 40 );
   SetSprite( sprites[ i ].spidp, x, y, x - dx, y - dy );
  }

 /*-- die Sprites bewegen und an den Seitenrndern reflektieren ------*/

 page = 1;                                      /* mit Page 1 starten */
 while ( !kbhit() )          /* ein Tastenderuck beendet die Schleife */
  {
   showpage( 1 - page );                 /* die andere Seite anzeigen */

   for ( i = 0; i < SPRANZ; ++ i)          /* die Sprites durchlaufen */
    {                /* Sprite bewegen und Kontakt mit Seite abfragen */
     out = MoveSprite( sprites[i].spidp, page, sprites[i].deltax[page],
                       sprites[i].deltay[page] );
     if ( out & OUT_TOP  ||  out & OUT_BOTTOM )  /*Kontakt oben/unten?*/
      {  /* Ja, Bewegungsrichtung umdrehen und neues Erscheinungsbild */
       sprites[i].deltay[page] = 0 - sprites[i].deltay[page];
       sprites[i].spidp->splookp = ( out & OUT_TOP ) ? gleiterdnp
                                                     : gleiterupp;
      }
     if ( out & OUT_LEFT  ||  out & OUT_RIGHT )
      sprites[i].deltax[page] = 0 - sprites[i].deltax[page];
    }
   page = (page+1) & 1;       /* von 1 auf 0 und umgekehrt umschalten */
  }
}

/*--------------------------------------------------------------------*/
/*-- H A U P T P R O G R A M M                                      --*/
/*--------------------------------------------------------------------*/

void main( void )
{
 union REGS regs;

 if ( IsVga() )                             /* VGA-Karte installiert? */
  {                                                  /* Ja, los gehts */
   init320200();                        /* Grafikmodus initialisieren */
   Demo();
   getch();                                       /* auf Taste warten */
   regs.x.ax = 0x0003;          /* wieder in Textmodus zurckschalten */
   int86( 0x10, &regs, &regs );
  }
 else
  printf( "S3220C.C - (c) 1990 by MICHAEL TISCHER\n\nAchtung! Fr "\
          "die Ausfhrung dieses Programms wird eine VGA-Karte "\
          "bentigt.\n\n" );
}

