/***********************************************************************
*                          S 6 4 3 5 C . C                             *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Arbeit mit Sprites in den       *
*                     640*350 Punkte EGA- und VGA-Grafikmodi mit 16    *
*                     Farben bei Nutzung von zwei Bildschirmseiten.    *
*                     Das Programm greift dabei auf die Assembler-     *
*                     Routinen aus den Moduln S6435CA.ASM und          *
*                     V16COLCA.ASM zurck.                             *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    :  5.12.1990                                       *
*  letztes Update am: 14.01.1991                                       *
**--------------------------------------------------------------------**
*  Speichermodell   : SMALL                                            *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung       : CL /AS s6435c.c s6435ca v16colca                 *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung       : ber eine Projekt-Datei mit folgendem Inhalt:    *
*                      s6435c.c                                        *
*                      v16colca.obj                                    *
*                      s6435ca.obj                                     *
**--------------------------------------------------------------------**
*  Aufruf           : s6435c                                           *
***********************************************************************/

#include <dos.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <conio.h>

/*-- Compiler-abhngige Deklarationen --------------------------------*/

#ifdef __TURBOC__
  #include <alloc.h>
#else
  #include <malloc.h>
  #define random(x) ( rand() % (x+1) )             /* Zufallsfunktion */
#endif

/*-- Typdeklarationen ------------------------------------------------*/

typedef unsigned char BYTE;
typedef BYTE BOOL;

typedef struct {        /* Pixel-Puffer fr GetVideo() und PutVideo() */
                BYTE breitebyte,       /* Breite des Bereichs in Byte */
                     hoehe;                      /* Anzahl der Zeilen */
                int  pixblen;              /* Lnge des Pixel-Puffers */
                void *pixbptr;             /* Zeiger auf Pixel-Puffer */
               } PIXBUF;
typedef PIXBUF *PIXPTR;              /* Zeiger auf einen Pixel-Puffer */

typedef struct {                    /* Erscheinungsbild eines Sprites */
                BYTE   breite,                       /* Breite gesamt */
                       hoehe;                  /* Hhe in Punktzeilen */
                void   *bmskp[8];     /* Zeiger auf Bit-Maske fr AND */
                PIXPTR pixmp[8];       /* Zeiger auf Pixel-Definition */
               } SPLOOK;

typedef struct {                           /* Sprite-Beschreiber (ID) */
                SPLOOK *splookp;       /* Zeiger auf Erscheinungsbild */
                int    x[2], y[2];    /* Koordinaten in Seite 0 und 1 */
                PIXPTR hgptr[2];     /* Zeiger auf Hintergrund-Puffer */
               } SPID;

typedef struct {                           /* beschreibt ein Bit-Feld */
                BYTE *feldptr,      /* Zeiger auf Puffer mit Bit-Feld */
                     *aktptr, /* Zeiger auf aktuell bearbeitetes Byte */
                     aktbit, /* aktuell bearbeitetes Bit in akt. Byte */
                     aktbyte;                  /* aktueller Byte-Wert */
               } BITFELD;
typedef BITFELD *BFPTR;                    /* Zeiger auf ein Bit-Feld */

/*-- externe Referenzen auf die Assembler-Routinen -------------------*/

extern void init640350( void );
extern void setpix( int x, int y, unsigned char farbe);
extern BYTE getpix( int x, int y );
extern void setpage( int page );
extern void showpage( int page );
extern void far * getfontptr( void );

extern void copybuf2video( BYTE *bufptr, BYTE page,
                           int nachx, int nachy, BYTE breite,
                           BYTE hoehe );
extern void copyvideo2buf( BYTE *bufptr, BYTE page,
                           int vonx, int vony, BYTE breite,
                           BYTE hoehe );
extern void mergeandcopybuf2video( void * spribufptr, void * hgbufptr,
                                   void * andbufptr, BYTE page,
                                   int  nachx, int  nachy,
                                   BYTE breite, BYTE hoehe );

/*-- Konstanten ------------------------------------------------------*/

#define TRUE  ( 0 == 0 )
#define FALSE ( 0 == 1 )

#define MAXX 639                        /* Maximale X- und Y-Ordinate */
#define MAXY 349

#define OUT_LEFT   1      /* fr Kollisionsmarkierung in SpriteMove() */
#define OUT_TOP    2
#define OUT_RIGHT  4
#define OUT_BOTTOM 8
#define OUT_NO     0                             /* nirgendwo drauen */

#define EGA       0                                   /* Karten-Typen */
#define VGA       1
#define WEDERNOCH 2

#define ALLOKBUF ((PIXPTR) 0)        /* GetVideo(): Puffer allokieren */

/***********************************************************************
*  IsEgaVga : Stellt fest, ob eine EGA- oder VGA-Karte installiert ist.*
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : EGA, VGA oder WEDERNOCH                                   *
***********************************************************************/

BYTE IsEgaVga( void )
{
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */

 Regs.x.ax = 0x1a00;              /* Funktion 1Ah gibt es nur bei VGA */
 int86( 0x10, &Regs, &Regs );
 if ( Regs.h.al == 0x1a )              /* ist die Funktion verfgbar? */
  return VGA;
 else
  {
   Regs.h.ah = 0x12;                    /* Funktion 12h Unterfunktion */
   Regs.h.bl = 0x10;                    /* 10h aufrufen               */
   int86(0x10, &Regs, &Regs );                 /* Video-BIOS aufrufen */
   return (BYTE) (( Regs.h.bl != 0x10 ) ? EGA : WEDERNOCH);
  }
}

/***********************************************************************
*  Line: Zieht eine Linie nach dem Bresenham-Algorithmus               *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endkoordinate                                    *
*            FARBE  = Farbe der Linie                                  *
***********************************************************************/

/*-- Unterfunktion, vertauscht zwei Integer-Variablen ----------------*/

void SwapInt( int *i1, int *i2 )
{
 int dummy;

 dummy = *i2;
 *i2   = *i1;
 *i1   = dummy;
}

/*-- Hauptteil der Funktion ------------------------------------------*/

void Line( int x1, int y1, int x2, int y2, BYTE farbe )
{
 int d, dx, dy,
     aincr, bincr,
     xincr, yincr,
     x, y;

 if ( abs(x2-x1) < abs(y2-y1) )       /* ber X- oder Y-Achse laufen? */
  {                                               /* ber die Y-Achse */
   if ( y1 > y2 )                                    /* y1 grer y2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   xincr = ( x2 > x1 ) ?  1 : -1;       /* Spalteninkrement festlegen */

   dy = y2 - y1;
   dx = abs( x2-x1 );
   d  = 2 * dx - dy;
   aincr = 2 * (dx - dy);
   bincr = 2 * dx;
   x = x1;
   y = y1;

   setpix( x, y, farbe );                      /* ersten Punkt setzen */
   for (y=y1+1; y<= y2; ++y )         /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       x += xincr;
       d += aincr;
      }
     else
      d += bincr;
     setpix(x, y, farbe);
    }
  }
 else                                             /* ber die X-Achse */
  {
   if ( x1 > x2 )                                    /* x1 grer x2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   yincr = ( y2 > y1 ) ? 1 : -1;         /* Zeileninkrement festlegen */

   dx = x2 - x1;
   dy = abs( y2-y1 );
   d  = 2 * dy - dx;
   aincr = 2 * (dy - dx);
   bincr = 2 * dy;
   x = x1;
   y = y1;

   setpix(x, y, farbe);                        /* ersten Punkt setzen */
   for (x=x1+1; x<=x2; ++x )          /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       y += yincr;
       d += aincr;
      }
     else
      d += bincr;
     setpix(x, y, farbe);
    }
  }
}

/***********************************************************************
*  PrintChar : Baut ein Zeichen auerhalb des sichtbaren Bereichs des  *
*              Video-RAMs auf.                                         *
**--------------------------------------------------------------------**
*  Eingabe :   zeichen = das aufzubauende Zeichen                      *
*              x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************/

void PrintChar( char zeichen, int x, int y, BYTE vg, BYTE hg )
{
 typedef BYTE ZDEF[256][8];                 /* Aufbau des Zeichensatz */
 typedef ZDEF far *ZPTR;                    /* Zeiger auf Zeichensatz */

 BYTE         i, k,                                /* Schleifenzhler */
              Maske;              /* Bitmaske fr Aufbau des Zeichens */

 static ZPTR fptr = (ZPTR) 0;        /* Zeiger auf Zeichensatz im ROM */

 if ( fptr == (ZPTR) 0 )   /* Zeiger auf Zeichensatz schon ermittelt? */
  fptr = getfontptr();  /*Nein, mit Hilfe der Assembler-Funktion laden*/

 /*- das Zeichen Punkt fr Punkt aufbauen ----------------------------*/

 if ( hg == 255 )                    /* Zeichen transparent zeichnen? */
  for ( i = 0; i < 8; ++i )      /* Ja, nur Vordergrund-Punkte setzen */
   {
    Maske = (*fptr)[zeichen][i];    /* Bitmuster fr eine Zeile holen */
    for ( k = 0; k < 8; ++k, Maske <<= 1 ) /* die Spalten durchlaufen */
     if ( Maske & 128 )                              /* Punkt setzen? */
      setpix( x+k, y+i, vg );                                   /* Ja */
   }
 else                                     /* Nein, jeden Punkt setzen */
  for ( i = 0; i < 8; ++i )                     /* Zeilen durchlaufen */
  {
   Maske = (*fptr)[zeichen][i];     /* Bitmuster fr eine Zeile holen */
   for ( k = 0; k < 8; ++k, Maske <<= 1 )  /* die Spalten durchlaufen */
    setpix( x+k, y+i, ( Maske & 128 ) ? vg : hg );
  }
}

/***********************************************************************
*  GrafPrintf: Gibt einen formatierten String im Grafikbildschirm aus  *
*  Eingabe : X, Y   = Startkoordinate (0 - ...)                        *
*            VG     = Vordergrundfarbe                                 *
*            HG     = Hintergrundfarbe (255 = transparent)             *
*            STRING = String mit Formatinformationen                   *
*            ...    = Ausdrcke wie bei Printf                         *
***********************************************************************/

void GrafPrintf( int x, int y, BYTE vg, BYTE hg, char * string, ... )
{
 va_list parameter;              /* Parameter-Liste fr VA_... Macros */
 char ausgabe[255],                 /* Puffer fr formatierten String */
      *cp;

 va_start( parameter, string );                /* Parameter umwandeln */
 vsprintf( ausgabe, string, parameter );               /* formatieren */
 for ( cp = ausgabe; *cp; ++cp, x+= 8 )    /* formatierten String     */
   PrintChar( *cp, x, y, vg, hg );         /* ber PrintChar ausgeben */
}

/***********************************************************************
*  GetVideo: Holt den Inhalt eines rechteckigen Bereichs aus dem       *
*            Video-RAM in einen Puffer                                 *
**--------------------------------------------------------------------**
*  Eingabe : PAGE   = Bildschirmseite (0 oder 1)                       *
*            X1, Y1 = Startkoordinate                                  *
*            BREITE = Breite des rechteckigen Bereichs in Punkten      *
*            HOEHE  = Hhe des rechteckigen Bereichs in Punkten        *
*            BUFPTR = Zeiger auf Pixel-Puffer, in dem die Informa-     *
*                     tionen abgelegt werden sollen                    *
*  Ausgabe : Zeiger auf den angelegten Pixel-Puffer mit dem Inhalt     *
*            des angegebenen Bereichs                                  *
*  Info    : Wird fr den Parameter BUFPTR der Wert ALLOKBUF bergeben,*
*            wird ein neuer Pixel-Puffer ber den Heap allokiert und   *
*            zurckgeliefert. Dieser Puffer kann bei einem neuer-      *
*            lichen Aufruf wieder angegeben werden, sofern der bis-    *
*            herige Inhalt nicht mehr bentigt wird und die Gre des  *
*            rechteckigen Bereichs gegenber dem vorhergehenden Aufruf *
*            unverndert bleibt.                                       *
*            Der angegebene Bereich mu an einer durch acht teilbaren  *
*            X-Ordinate beginnen und sich ber ein vielfaches von acht *
*            Punkten erstrecken.                                       *
***********************************************************************/

PIXPTR GetVideo( BYTE page, int x1, int y1, BYTE breite, BYTE hoehe,
                 PIXPTR bufptr )
{
 if ( bufptr == ALLOKBUF )      /* kein Puffer beim Aufruf bergeben? */
  {                                                 /* Nein, erzeugen */
   bufptr = malloc( sizeof( PIXBUF ) );        /* Pixelpuffer anlegen */
   bufptr->pixbptr = malloc( (breite*hoehe) / 2 );   /* Pix-P. allok. */
   bufptr->hoehe = hoehe;               /* Hhe des Puffers in Zeilen */
   bufptr->breitebyte = breite / 8;    /* Breite einer Zeile in Bytes */
   bufptr->pixblen = (breite*hoehe) / 2;   /* Gesamtlngelnge des P. */
  }

 copyvideo2buf( bufptr->pixbptr, page, x1, y1, breite / 8, hoehe );
 return bufptr;        /* Zeiger auf Puffer an Aufrufer zurckliefern */
}

/***********************************************************************
*  PutVideo: Schreibt den Inhalt eines zuvor ber GetVideo gesicherten *
*            rechteckigen Bildschirmbereichs wieder in den Video-RAM   *
**--------------------------------------------------------------------**
*  Eingabe : BUFPTR = Beim vorherigen Aufruf von Getvideo() zurck-    *
*                     gelieferter Zeiger auf den Pixel-Puffer          *
*            PAGE   = Bildschirmseite (0 oder 1)                       *
*            X1, Y1 = Startkoordinate                                  *
*  Info    : Der Pixel-Puffer wird durch diese Prozedur nicht gelscht,*
*            dafr mu die Prozedur FreiPixBuf aufgerufen werden.      *
*            Die angegebene X-Ordinate mu ein Vielfaches von acht     *
*            sein!                                                     *
***********************************************************************/

void PutVideo( PIXPTR bufptr, BYTE page, int x1, int y1 )
{
 copybuf2video( bufptr->pixbptr, page, x1, y1,
                bufptr->breitebyte, bufptr->hoehe );
}

/***********************************************************************
*  FreePixBuf: Lscht einen Pixel-Puffer, der bei einem vorherigen     *
*              Aufruf von GetVideo ber den Heap allokiert wurde       *
**--------------------------------------------------------------------**
*  Eingabe : BUFPTR = Beim vorherigen Aufruf von Getvideo() zurck-    *
*                     gelieferter Zeiger auf den Pixel-Puffer          *
***********************************************************************/

void FreePixBuf( PIXPTR bufptr )
{
 free( bufptr->pixbptr );
 free( bufptr );
}

/***********************************************************************
*  CreateSprite: Erzeugt ein Sprite auf der Basis einer bereits kompi- *
*                lierten Punktmusters                                  *
**--------------------------------------------------------------------**
*  Eingabe : SPLOOKP = Zeiger auf die von CompileSpriteI() erzeugte    *
*                      Datenstruktur                                   *
*  Ausgabe : Zeiger auf die erstellte Sprite-Struktur                  *
*  Info    : Fr die Ablege des Sprite-Hintergrundes werden zwei ne-   *
*            beneinanderliegende Flchen von der Gre des jeweiligen  *
*            Sprites bentigt.                                         *
***********************************************************************/

SPID *CreateSprite( SPLOOK *splookp )
{
 SPID *spidp;                  /* Zeiger auf erzeugte Sprite-Struktur */

 spidp = (SPID *) malloc( sizeof(SPID) );   /* Sprite-Struktur allok. */
 spidp->splookp = splookp;                    /* Daten in die Sprite- */

 /*- zwei Hintergrund-Puffer erzeugen, indem ein entsprechend groer -*/
 /*- Bereich aus dem Video-RAM ber GetVideo gesichert wird          -*/

 spidp->hgptr[0] = GetVideo( 0, 0, 0, splookp->breite,
                             splookp->hoehe, ALLOKBUF );
 spidp->hgptr[1] = GetVideo( 0, 0, 0, splookp->breite,
                             splookp->hoehe, ALLOKBUF );
 return spidp;            /* Zeiger auf Sprite-Struktur zurckliefern */
}

/***********************************************************************
*  BfInit: Leitet die Arbeit mit einem Bit-Feld ein und erzeugt das    *
*          Bit-Feld                                                    *
**--------------------------------------------------------------------**
*  Eingabe : ANZBIT = Anzahl der Bits, die in dem Bit-Feld Platz fin-  *
*                     den sollen                                       *
*  Ausgabe : Zeiger auf Bit-Feld-Beschreiber                           *
***********************************************************************/

BFPTR BfInit( int AnzBit )
{
 BFPTR bfptr;                     /* Zeiger auf erzeugten Beschreiber */

 bfptr = malloc( sizeof( BITFELD ) );         /* Beschreiber erzeugen */

 /*-- das eigentliche Bit-Feld erzeugen und initialisieren -----------*/

 bfptr->feldptr = bfptr->aktptr = malloc( ( AnzBit + 7 ) / 8 );
 bfptr->aktbit = bfptr->aktbyte = 0;

 return bfptr;                /* Zeiger auf Beschreiber zurckliefern */
}

/***********************************************************************
*  BfAppendBit: Hngt ein Bit an ein Bit-Feld an                       *
**--------------------------------------------------------------------**
*  Eingabe : BFID = Zeiger auf den Bit-Feld-Beschreiber, der bei Auf-  *
*                   ruf von BfInit() zurckgeliefert wurde             *
*            BIT  = Wert des anzuhngenden Bits (0 oder 1)             *
*  Ausgabe : keine                                                     *
***********************************************************************/

void BfAppendBit( BFPTR bfid, BYTE bit )
{
 bfid->aktbyte |= bit;               /* Bit in Bitposition 0 einbl}en */
 if ( bfid->aktbit == 7 )                 /* ist das Byte jetzt voll? */
  {                                                             /* Ja */
   *(bfid->aktptr++) = bfid->aktbyte;       /* Byte im Puffer ablegen */
   bfid->aktbyte = bfid->aktbit = 0;            /* Bit-Maske wieder 0 */
  }
 else                                 /* das Byte ist noch nicht voll */
  {
   ++bfid->aktbit;                         /* ein Bit mehr bearbeitet */
   bfid->aktbyte <<= 1;                          /* Bit-Maske shiften */
  }
}

/***********************************************************************
*  BfEnd : Beendet die Arbeit mit einem Bit-Feld und lscht dessen Be- *
*          schreiber, ohne allerdings das eigentliche Bit-Feld zu      *
*          lschen.                                                    *
**--------------------------------------------------------------------**
*  Eingabe : BFID = Zeiger auf den Bit-Feld-Beschreiber, der bei Auf-  *
*                   ruf von BfInit() zurckgeliefert wurde             *
*  Ausgabe : Zeiger auf das eigentliche Bit-Feld, dessen Puffer ber   *
*            FREE() wieder freigegeben werden kann.                    *
***********************************************************************/

void *BfEnd( BFPTR bfid )
{
 void *retptr;                 /* Zeiger auf das eigentliche Bit-Feld */

 if ( bfid->aktbit )                 /* letztes Byte noch nicht voll? */
  *bfid->aktptr = bfid->aktbyte << (7 - bfid->aktbit );  /*NO, abschl.*/

 retptr = bfid->feldptr;                /* Zeiger auf Bit-Feld merken */
 free( bfid );                                 /* Beschreiber lschen */

 return retptr;                  /* Zeiger auf Bit-Feld zurckliefern */
}

/***********************************************************************
*  CompileSprite: Erzeugt das Punkt- und Bitmusters eines Sprites,     *
*                 auf der Basis einer zur Laufzeit angegebenen Sprite- *
*                 definition                                           *
**--------------------------------------------------------------------**
*  Eingabe : BUFP    = Zeiger auf Array mit Zeigern auf Strings, die   *
*                      das Zeichenmuster des Sprites enthalten         *
*            HOEHE   = Hoehe des Sprites und damit Anzahl der Strings  *
*  Info    : Im bergebenen Sprite-Muster steht ein Leerzeichen fr    *
*            einen Hintergrund-Punkt, das A fr den Farbcode 0, B fr  *
*            1, C fr 2 usw.                                           *
***********************************************************************/

SPLOOK *CompileSprite( char **bufp, BYTE hoehe )
{
 BYTE   sbreite,                                /* Breite der Strings */
        gbreite,                                /* Breite der Sprites */
        c,                    /* nimmt Zeichen aus C-Sprite-Array auf */
        i, k, l, y;                             /* Schleifenvariablen */
 SPLOOK *splookp;              /* Zeiger auf erzeugte Sprite-Struktur */
 PIXPTR tpix;                /* nimmt Sprite-Hintergrund temporr auf */
 BFPTR  bfptr;              /* Zeiger auf Beschreiber eines Bit-Felds */

 /*-- Sprite-Look-Struktur erzeugen und mit Daten fllen -------------*/

 splookp = (SPLOOK *) malloc( sizeof(SPLOOK) );
 sbreite = (BYTE) strlen( *bufp );  /* Lnge der Strings, Logo-Breite */
 gbreite = ( ( sbreite + 7 + 7 ) / 8 ) * 8;           /* Gesamtbreite */
 splookp->breite = gbreite;
 splookp->hoehe = hoehe;

 setpage( 1 );                         /* Sprites in Seite 1 aufbauen */
 showpage( 0 );                              /* aber Seite 0 anzeigen */

 tpix = GetVideo( 1, 0, 0, gbreite, hoehe, ALLOKBUF ); /* Hgr. merken */

 /*-- das Sprite acht mal aufbauen und kodieren ----------------------*/

 for (l = 0; l < 8; ++l )
  {              /* zunchst Hintergrund mit schwarzen Punkten fllen */
   for ( y = 0; y < hoehe; ++y )
    Line( 0, y, gbreite-1, y, 0 );

   bfptr = BfInit( gbreite*hoehe );  /*Speicher fr AND-Puffer anlegen*/

   for ( i = 0; i < hoehe ; ++i )               /* Zeilen durchlaufen */
    {
     for ( y = l; y; --y )       /* AND-Bits fr linken Rand erzeugen */
       BfAppendBit( bfptr, 1 );

     for ( k = 0; k < sbreite; ++k )       /* die Spalten durchlaufen */
      {
       if ( ( c = *(*(bufp+i)+k) ) == 32 )      /* Hintergrund-Punkt? */
        {                                    /* Ja, Farbcode 0 setzen */
         setpix( k+l, i, 0 );
         BfAppendBit( bfptr, 1 );         /* Hintergrund-Punkt bleibt */
         }
        else                   /* Nein, Farbcode wie angegeben setzen */
         {
          setpix( k+l, i, c-64 );
          BfAppendBit( bfptr, 0 );    /* Hintergrund-Punkt ausblenden */
         }
      }

     for ( y = gbreite-sbreite-l; y ; --y ) /* AND-Bits fr den rech- */
      BfAppendBit( bfptr, 1 );              /* ten Rand anhngen      */
    }
   splookp->bmskp[ l ] = BfEnd( bfptr );

   /*-- Pixel-Muster des Sprites aus Video-RAM holen -----------------*/
   splookp->pixmp[ l ] = GetVideo( 1, 0, 0, gbreite, hoehe, ALLOKBUF );
  }                             /* nchstes der acht Sprites aufbauen */

 PutVideo( tpix, 1, 0, 0 );        /* Sprite-Hintergrund in Seite 1   */
 FreePixBuf( tpix );               /* restaurieren und Puffer lschen */

 return splookp;            /* Zeiger auf Sprite-Puffer zurckliefern */
}

/***********************************************************************
*  PrintSprite : Sprite in einer bestimmten Seite ausgeben             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in der das Sprite gezeichnet werden soll   *
*                    (0 oder 1)                                        *
***********************************************************************/

void PrintSprite( register SPID *spidp, BYTE seite )
{
 int x;                                     /* X-Ordinate des Sprites */

 x = spidp->x[seite];
 mergeandcopybuf2video( spidp->splookp->pixmp[x % 8]->pixbptr,
                        spidp->hgptr[seite]->pixbptr,
                        spidp->splookp->bmskp[x % 8],
                        seite,
                        x & (~7),
                        spidp->y[seite],
                        spidp->splookp->breite / 8,
                        spidp->splookp->hoehe );
}

/***********************************************************************
*  GetSpriteBg: Hintergrund eines Sprites holen und an der dafr vor-  *
*               gesehenen Position ablegen                             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, aus der der Hintergrund geholt werden soll *
*                    (0 oder 1)                                        *
***********************************************************************/

void GetSpriteBg( register SPID *spidp, BYTE seite )
{
 GetVideo( seite, spidp->x[seite] & (~7),  spidp->y[seite],
           spidp->splookp->breite, spidp->splookp->hoehe,
           spidp->hgptr[seite] );
}

/***********************************************************************
*  RestoreSpriteBg: Zuvor gesicherten Hintergrund eines Sprites wieder *
*                   in die ursprngliche Grafikseite kopieren          *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in die der Hinntergrund zurckkopiert      *
*                    werden soll (0 oder 1)                            *
***********************************************************************/

void RestoreSpriteBg( register SPID *spidp, BYTE seite )
{
 PutVideo( spidp->hgptr[seite], seite,
           spidp->x[seite] & (~7), spidp->y[seite] );
}

/***********************************************************************
*  MoveSprite: Sprite innerhlicherten Hintergrund eines Sprites wieder *
*                   in die ursprngliche Grafikseite kopieren          *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP  = Zeiger auf die Sprite-Struktur                   *
*            SEITE  = Seite, in die der Hintergrund zurckkopiert      *
*                     werden soll (0 oder 1)                           *
*            DELTAX = Bewegungszhler in X- und Y-Richtung             *
*            DELTAY                                                    *
*  Ausgabe : Kollisionsmarkierung, siehe Konstanten OUT_...            *
***********************************************************************/

BYTE MoveSprite( SPID *spidp, BYTE seite, int deltax, int deltay )
{
 int  neux, neuy;                           /* neue Sprite-Koordinate */
 BYTE out;                             /* Kollision mit Rand anzeigen */

 /*-- X-Koordinate verschieben und Kollision mit Rand testen ---------*/

 if ( ( neux = spidp->x[seite] + deltax ) < 0 )
  {
   neux = 0 - deltax - spidp->x[seite];
   out = OUT_LEFT;
  }
 else
  if ( neux > MAXX - spidp->splookp->breite )
   {
    neux = (2*(MAXX+1))-neux-2*(spidp->splookp->breite);
    out = OUT_RIGHT;
   }
  else
    out = OUT_NO;

 /*-- Y-Koordinate verschieben und Kollision mit Rand testen ---------*/

 if ( ( neuy = spidp->y[seite] + deltay ) < 0 )       /* oberer Rand? */
  {                                    /* Ja, deltay mu negativ sein */
   neuy = 0 - deltay - spidp->y[seite];
   out |= OUT_TOP;
  }
 else
  if ( neuy + spidp->splookp->hoehe > MAXY+1  )      /* unterer Rand? */
   {                                   /* Ja, deltay mu positiv sein */
    neuy = (2*(MAXY+1))-neuy-2*(spidp->splookp->hoehe);
    out |= OUT_BOTTOM;
   }

 /*-- neue Position nur setzen, wenn gegenber der alten verndert ---*/

 if ( neux != spidp->x[seite]  ||  neuy != spidp->y[seite] )
  {                                              /* ist neue Position */
   RestoreSpriteBg( spidp, seite );       /* Hintergrund zurcksetzen */
   spidp->x[seite] = neux;                 /* neue Koordinaten merken */
   spidp->y[seite] = neuy;
   GetSpriteBg( spidp, seite );            /* neuen Hintergrund holen */
   PrintSprite( spidp, seite );  /* Sprite in angegeb. Seite zeichnen */
  }
 return out;
}

/***********************************************************************
*  SetSprite: Sprite an eine bestimmte Position versetzen              *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            x0, y0 = Sprite-Koordinate fr die Seite 0                *
*            x1, y1 = Sprite-Koordinate fr die Seite 1                *
*  Info    : Der Aufruf dieser Funktion sollte dem ersten Aufruf von   *
*            MoveSprite() vorausgehen                                  *
***********************************************************************/

void SetSprite( SPID *spidp, int x0, int y0, int x1, int y1 )
{
 spidp->x[0] = x0;           /* Koordinaten in Sprite-Struktur merken */
 spidp->x[1] = x1;
 spidp->y[0] = y0;
 spidp->y[1] = y1;

 GetSpriteBg( spidp, 0 );              /* Sprite-Hintergrund in Seite */
 GetSpriteBg( spidp, 1 );              /* 0 und 1 holen               */
 PrintSprite( spidp, 0 );                        /* Sprite in Seite 0 */
 PrintSprite( spidp, 1 );                        /* und 1 zeichnen    */
}

/***********************************************************************
*  RemoveSprite: Entfernt ein Sprite von seiner augenblicklichen       *
*                Position und macht es dadurch unsichtbar              *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*  Info    : Nach dem Aufruf dieser Funktion mu erst wieder die Funk- *
*            tion SetSprite() auferufen werden, bevor das Sprite ber  *
*            MoveSprite() bewegt werden kann                           *
***********************************************************************/

void RemoveSprite( SPID *spidp )
{
 RestoreSpriteBg( spidp, 0 );          /* Sprite-Hintergrund in Seite */
 RestoreSpriteBg( spidp, 1 );          /* 0 und 1 wieder zurcksetzen */
}

/***********************************************************************
*  Demo: Demonstriert den Einsatz der verschiedenen Funktionen aus     *
*        diesem Modul                                                  *
***********************************************************************/

void Demo( void )
{
 static char *GleiterRauf[20] =
                { "               AA               ",
                  "              AAAA              ",
                  "              AAAA              ",
                  "               AA               ",
                  "             GGBBGG             ",
                  "            GBBCCBBG            ",
                  "           GBBBCCBBBG           ",
                  "          GBBBBBBBBBBG          ",
                  "          GBBBBBBBBBBG          ",
                  " G       GBBBBBBBBBBBBG       G ",
                  "GCG     GGDBBBBBBBBBBDGG     GCG",
                  "GCG   GGBBBDBBB  BBBDBBBGG   GCG",
                  "GCBGGGBBBBBDBB    BBDBBBBBGGGBCG",
                  "GCBBBBBBBBBBDB    BDBBBBBBBBBBCG",
                  "BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB",
                  "GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ",
                  "  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ",
                  "    GGBBDDDDDGGGGGDDDDDDBBG     ",
                  "      GDDDDGGG    GGGDDDDG      ",
                  "       DDDD          DDDD       "  };


 static char *GleiterRunter[20] =
                {
                  "       DDDD          DDDD       ",
                  "      GDDDDGGG    GGGDDDDG      ",
                  "    GGBBDDDDDGGGGGDDDDDDBBG     ",
                  "  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ",
                  "GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ",
                  "BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB",
                  "GCBBBBBBBBBBDB    BDBBBBBBBBBBCG",
                  "GCBGGGBBBBBDBB    BBDBBBBBGGGBCG",
                  "GCG   GGBBBDBBB  BBBDBBBGG   GCG",
                  "GCG     GGDBBBBBBBBBBDGG     GCG",
                  " G       GBBBBBBBBBBBBG       G ",
                  "          GBBBBBBBBBBG          ",
                  "          GBBBBBBBBBBG          ",
                  "           GBBBCCBBBG           ",
                  "            GBBCCBBG            ",
                  "             GGBBGG             ",
                  "               AA               ",
                  "              AAAA              ",
                  "              AAAA              ",
                  "               AA               " };

 #define SPRANZ 6                               /* Anzahl der Sprites */
 #define BREITE 42         /* Breite der Copyright-Meldung in Zeichen */
 #define HOEHE  6                                   /* Hhe in Zeilen */
 #define SX     (MAXX-(BREITE*8)) / 2              /* Startordinate X */
 #define SY     (MAXY-(HOEHE*8)) / 2               /* Startordinate Y */

 struct {                       /* beschreibt die verwalteten Sprites */
          SPID *spidp;                        /* Zeiger auf Sprite-Id */
          int  deltax[2],          /* X-Verschiebung fr Page 0 und 1 */
               deltay[2];          /* Y-Verschiebung fr Page 0 und 1 */
        } sprites[ SPRANZ ];
 BYTE   page,                            /* aktuell bearbeitete Seite */
        out;                   /* nimmt Flags fr Seitenkollision auf */
 int    x, y, i,                                   /* Schleifenzhler */
        dx, dy;                                     /* Bewegungswerte */
 char   lc;
 SPLOOK *gleiterupp, *gleiterdnp;           /* Zeiger auf die Sprites */

 srand( *(int far *) 0x0040006cl );     /* Zufallsgen. initialisieren */

 /*-- Vorlagen fr die verschiedenen Sprites aufbauen ----------------*/

 gleiterupp = CompileSprite( GleiterRauf,   20 );
 gleiterdnp = CompileSprite( GleiterRunter, 20 );

 /*-- die ersten beiden Grafikseiten mit Buchstaben fllen -----------*/

 for ( page = 0; page < 2; ++ page )
  {
   setpage( page );
   showpage( page );
   for ( lc = 0, y = 0; y < (MAXY+1)-8; y += 12 )
     for ( x = 0; x < (MAXX+1)-8; x += 8 )
       GrafPrintf( x, y, lc & 15, 255, "%c", lc++ & 127 );

   /*-- Copyright-Meldung ausgeben -----------------------------------*/

   Line( SX-1, SY-1, SX+BREITE*8, SY-1, 15 );
   Line( SX+BREITE*8, SY-1, SX+BREITE*8, SY+HOEHE*8,15 );
   Line( SX+BREITE*8, SY+HOEHE*8, SX-1, SY+HOEHE*8, 15 );
   Line( SX-1, SY+HOEHE*8, SX-1, SY-1, 15 );
   GrafPrintf( SX, SY,    15, 4,
               "                                          "  );
   GrafPrintf( SX, SY+8,  15, 4,
               " S6435C.C  -  (c) 1990 by Michael Tischer " );
   GrafPrintf( SX, SY+16, 15, 4,
               "                                          "  );
   GrafPrintf( SX, SY+24, 15, 4,
               "    Sprite-Demo fuer den 640*350 Punkte   "  );
   GrafPrintf( SX, SY+32, 15, 4,
               "    Grafikmodus von EGA- und VGA-Karten   "  );
   GrafPrintf( SX, SY+40, 15, 4,
               "                                          "  );
  }

 /*-- die verschiedenen Sprites erzeugen -----------------------------*/

 for ( i = 0; i < SPRANZ ; ++ i)
  {
   sprites[ i ].spidp = CreateSprite( gleiterupp );
   do                            /* Bewegungswerte der Sprites whlen */
    {
     dx = 0;
     dy = random(8) - 4;
    }
   while ( dx==0  &&  dy==0 );

   sprites[ i ].deltax[0] = sprites[ i ].deltax[1] = dx * 2;
   sprites[ i ].deltay[0] = sprites[ i ].deltay[1] = dy * 2;

   x = ( MAXX / SPRANZ * i ) + (MAXX / SPRANZ - 40) / 2 ;
   y = random( (MAXY+1) - 40 );
   SetSprite( sprites[ i ].spidp, x, y, x - dx, y - dy );
  }

 /*-- die Sprites bewegen und an den Seitenrndern reflektieren ------*/

 page = 1;                                      /* mit Page 1 starten */
 while ( !kbhit() )          /* ein Tastenderuck beendet die Schleife */
  {
   showpage( (BYTE) (1 - page) );        /* die andere Seite anzeigen */

   for ( i = 0; i < SPRANZ; ++ i)          /* die Sprites durchlaufen */
    {                /* Sprite bewegen und Kontakt mit Seite abfragen */
     out = MoveSprite( sprites[i].spidp, page, sprites[i].deltax[page],
                       sprites[i].deltay[page] );
     if ( out & OUT_TOP  ||  out & OUT_BOTTOM )  /*Kontakt oben/unten?*/
      {  /* Ja, Bewegungsrichtung umdrehen und neues Erscheinungsbild */
       sprites[i].deltay[page] = 0 - sprites[i].deltay[page];
       sprites[i].spidp->splookp = ( out & OUT_TOP ) ? gleiterdnp
                                                     : gleiterupp;
      }
     if ( out & OUT_LEFT  ||  out & OUT_RIGHT )
      sprites[i].deltax[page] = 0 - sprites[i].deltax[page];
    }
   page = (page+1) & 1;       /* von 1 auf 0 und umgekehrt umschalten */
  }
}

/*--------------------------------------------------------------------*/
/*-- H A U P T P R O G R A M M                                      --*/
/*--------------------------------------------------------------------*/

void main( void )
{
 union REGS regs;

 if ( IsEgaVga() != WEDERNOCH )   /* EGA- oder VGA-Karte installiert? */
  {                                                  /* Ja, los gehts */
   init640350();                        /* Grafikmodus initialisieren */
   Demo();
   getch();                                       /* auf Taste warten */
   regs.x.ax = 0x0003;          /* wieder in Textmodus zurckschalten */
   int86( 0x10, &regs, &regs );
  }
 else
  printf( "S6435C.C - (c) 1990 by MICHAEL TISCHER\n\nAchtung! Fr "\
          "die Ausfhrung dieses Programms wird eine EGA- oder "\
          "VGA-Karte bentigt.\n\n" );
}

