/**********************************************************************/
/*                   S B U T I L . C                                  */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Stellt Funktionen zur Initialisierung der SBBASE */
/*                   Struktur bereit.                                 */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 20.03.1994                                       */
/*  letztes Update : 25.03.1994                                       */
/**********************************************************************/
#ifndef __SBUTIL_C                      /* Kann auch #Included werden */
#define __SBUTIL_C

/*-- Include-Dateien einbinden ---------------------------------------*/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <io.h>
#include <fcntl.h>

#include "types.h"
#include "sbutil.h"
#include "args.h"

/**********************************************************************/
/* sb_GetEnviron : Initialisiere SBBASE-Struktur aufgrund bergebenem */
/*                 Environment-String (BLASTER-Environment-Variable)  */
/**------------------------------------------------------------------**/
/* Eingabe : pSBBASE - Adresse der zu initialisierenden Struktur      */
/*           pEnv    - Adresse des Environment-Strings                */
/* Ausgabe : ==  0 : fehlerfreie Initialisierung                      */
/*           == -1 : fehlerbehaftete Initialisierung                  */
/* Info : - Diese Struktur mu den einzelnen Modulen bergeben werden */
/*          (???_SetBase()-Funktionen), die daraufhin weitere         */
/*          Initialisierungen der Struktur (z.B. DSP-Versionsnummer)  */
/*          durchfhren.                                              */
/**********************************************************************/
INT sb_GetEnviron( PSBBASE pSBBASE, PCHAR pEnv )
{
   if( pEnv )
   {
      pSBBASE->iDspPort = htoi( strichr( pEnv, "A" ), -1 );
      pSBBASE->iMixPort = htoi( strichr( pEnv, "M" ), -1 );
      if( pSBBASE->iMixPort == -1 )
        pSBBASE->iMixPort = pSBBASE->iDspPort;
      pSBBASE->iMpuPort = htoi( strichr( pEnv, "P" ), -1 );
      pSBBASE->uDspVersion = 0xFFFFU;   /* wird spter initialisiert */
      pSBBASE->iDspIrq  = htoi( strichr( pEnv, "I" ), -1 );
      pSBBASE->iDspDmaB = htoi( strichr( pEnv, "D" ), -1 );
      pSBBASE->iDspDmaW = htoi( strichr( pEnv, "H" ), -1 );
      return NO_ERROR;
   }
   return ERROR;
}

/**********************************************************************/
/* sb_Print : SBBASE-Struktur anzeigen                                */
/**------------------------------------------------------------------**/
/* Eingabe : pSBBASE - Adresse der anzuzeigenden Struktur             */
/**********************************************************************/
VOID sb_Print( PSBBASE pSBBASE )
{
  printf("DSP-Port: 0x%X \n", pSBBASE->iDspPort );
  printf("MIX-Port: 0x%X \n", pSBBASE->iMixPort);
  printf("MPU-Port: 0x%X \n", pSBBASE->iMpuPort );
  if( pSBBASE->uDspVersion != 0xFFFFU )
  {
    printf("DSP-Version: %d.%02d \n", HIBYTE( pSBBASE->uDspVersion ),
                                      LOBYTE( pSBBASE->uDspVersion ) );
    printf("Karte : %s\n", pSBBASE->pDspName );
  }
  else  printf("DSP nicht Initialisiert!\n");
  printf("DSP-IRQ: %d \n", pSBBASE->iDspIrq );
  printf("DSP- 8 Bit DMA: %d \n", pSBBASE->iDspDmaB );
  printf("DSP-16 Bit DMA: %d \n", pSBBASE->iDspDmaW );
}

/**********************************************************************/
/* sb_LoadDriver : Soundblaster-Treiber laden                         */
/**------------------------------------------------------------------**/
/* Eingabe : lpName - Name des zu ladenden Drivers (Filename)         */
/* Ausgabe : Adresse des Treibereinsprungs                            */
/*           == NULL - Treiber konnte nicht geladen werden            */
/**********************************************************************/
LPVOID sb_LoadDriver( PCHAR pName )
{ INT iHandle = -1;
  WORD uSegment = 0;

  if (_dos_open( pName, O_RDWR, &iHandle) == 0)
  { WORD   uDummy;
    LONG   lFileSize;
    LONG   lNumSeg;
    LPBYTE lpMem;
                                       /* Ge des Treibers ermitteln */
    lFileSize = filelength( iHandle );
    lNumSeg = lFileSize / 16L + 1;            /* Gre in Paragraphen */
    if( lNumSeg < 0xFFFFU )
                       /* Speicher mu auf Paragraphengrenze beginnen */
      if( _dos_allocmem( ( WORD ) ( lNumSeg ), &uSegment ) == 0 )
      {
        lpMem = MK_FP( uSegment, 0 );
        while( lFileSize )
          if( _dos_read( iHandle, lpMem,
                         ( lFileSize > 65535L ) ? 65535U :
                         ( WORD )lFileSize, &uDummy ) == 0 )
          { WORD iInc;
            iInc = ( lFileSize > 65535L ) ? 65535U : ( WORD )lFileSize;
            lpMem += iInc;
            lFileSize -= iInc;
          }
          else                  /* Nach Lesefehler Speicher freigeben */
          {
            _dos_freemem( uSegment );
            uSegment = 0;
            lFileSize = 0;
          }
      }
      else uSegment = 0;
    _dos_close( iHandle );
  }
  return ( LPVOID ) MK_FP( uSegment, 0 );
}

/**********************************************************************/
/* sb_UnloadDriver : Soundblaster-Treiber aus speicher entfernen      */
/**------------------------------------------------------------------**/
/* Eingabe : lpEntry : Adresse des Speicherblocks, den der Treiber    */
/*                     belegt.                                        */
/**********************************************************************/
VOID sb_UnloadDriver( LPVOID lpEntry )
{
  if( lpEntry ) _dos_freemem( FP_SEG( lpEntry ) );
}
#endif

