/**********************************************************************/
/*                     S E R U T I L . C                              */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Funktionen fr Direktzugriff auf serielle        */
/*                   Schnittstelle                                    */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 08.04.1994                                       */
/*  letztes Update : 18.04.1994                                       */
/**********************************************************************/
#ifndef __SERUTIL_C                     /* Kann auch #Included werden */
#define __SERUTIL_C

#include <conio.h>
#include "serutil.h"
#include "irqutil.h"
#include "win.h"

/**********************************************************************/
/* ser_UARTType : Typ des UART-Chips ermitteln                        */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort     - Basisport der zu testenden Schnittstelle  */
/* Ausgabe : 0 (NOSER)    - Kein UART-Chip gefunden                   */
/*           1 (INS8250)  - INS8250 oder INS8250-B Chip               */
/*           2 (NS16450)  - INS8250A, INS82C50A, NS16450, NS16C450    */
/*           3 (NS16550A) - NS16550A Chip                             */
/*           4 (NS16C552) - NS16C552 Chip                             */
/**********************************************************************/
INT ser_UARTType( INT iSerPort )
{
  /*- Grundfunktionalitt prfen ------------------------------------ */

  outp( iSerPort + SER_LINE_CONTROL, 0xAA ); /* Divisor-Latch gesetzt */
  if( inp( iSerPort + SER_LINE_CONTROL ) != 0xAA ) return NOSER;

  outp( iSerPort + SER_DIVISOR_MSB, 0x55 );    /* Divisor beschreiben */
  if( inp( iSerPort + SER_DIVISOR_MSB ) != 0x55 ) return NOSER;

  outp( iSerPort + SER_LINE_CONTROL, 0x55 ); /* Divisor-Latch lschen */
  if( inp( iSerPort + SER_LINE_CONTROL ) != 0x55 ) return NOSER;

  outp( iSerPort + SER_IRQ_ENABLE, 0x55 );
  if( inp( iSerPort + SER_IRQ_ENABLE ) != 0x05 ) return NOSER;

  outp( iSerPort + SER_FIFO, 0 );             /* FIFO und IRQ lschen */
  outp( iSerPort + SER_IRQ_ENABLE, 0 );
  if( inp( iSerPort + SER_IRQ_ID ) != 1 ) return NOSER;

  outp( iSerPort + SER_MODEM_CONTROL, 0xF5 );
  if( inp( iSerPort + SER_MODEM_CONTROL ) != 0x15 ) return NOSER;

  outp( iSerPort + SER_MODEM_CONTROL, SER_MCR_LOOP );      /* Looping */
  inp( iSerPort + SER_MODEM_STATUS );
  if( ( inp( iSerPort + SER_MODEM_STATUS ) & 0xF0 ) != 0 ) return NOSER;

  outp( iSerPort + SER_MODEM_CONTROL, 0x1F );
  if( ( inp( iSerPort + SER_MODEM_STATUS ) & 0xF0 ) != 0xF0 )
      return NOSER;

  outp( iSerPort + SER_MODEM_CONTROL, SER_MCR_DTR | SER_MCR_RTS );

  outp( iSerPort + SER_SCRATCH, 0x55 ); /* Scratch-Register vorhanden?*/
  if( inp( iSerPort + SER_SCRATCH ) != 0x55 ) return INS8250;
  outp( iSerPort + SER_SCRATCH, 0 );

  outp( iSerPort + SER_FIFO, 0xCF );              /* FIFO vorhanden ? */
  if( ( inp( iSerPort + SER_IRQ_ID ) & 0xC0 ) != 0xC0 ) return NS16450;
  outp( iSerPort + SER_FIFO, 0 );
                            /* Alternate-Function Register vorhanden? */
  outp( iSerPort + SER_LINE_CONTROL, SER_LCR_SETDIVISOR );
  outp( iSerPort + SER_2FUNCTION, 0x07 );
  if( inp( iSerPort + SER_2FUNCTION ) != 0x07 )
  {
    outp( iSerPort + SER_LINE_CONTROL, 0 );
    return NS16550A;
  }
  outp( iSerPort + SER_LINE_CONTROL, 0 );               /* Rcksetzen */
  outp( iSerPort + SER_2FUNCTION, 0 );
  return NS16C552;
}

/**********************************************************************/
/* ser_Init : serielle Schnittstelle initialisieren                   */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort - Basisport der Schnittstelle, die              */
/*                      initialisiert werden soll.                    */
/*           lBaud    - Baudrate ( von 1 - 115200 )                   */
/*           bParams  - Bitmaske der restlichen Parameter             */
/*                      (s. SER_LCR_...-Bits)                         */
/* Ausgabe : TRUE  - Schnittstelle konnte initialisiert werden        */
/*           FALSE - keine Schnittstelle gefunden                     */
/**********************************************************************/
INT ser_Init( INT  iSerPort, LONG lBaudRate, BYTE bParams )
{ WORD uDivisor;
  if( ser_UARTType( iSerPort ) != NOSER )
  {                                     /* Baudratendivisor berechnen */
    uDivisor = ( WORD )( SER_MAXBAUD / lBaudRate );
    outp( iSerPort + SER_LINE_CONTROL, /* Divisor-Zugriff ermglichen */
                inp( SER_LINE_CONTROL ) | SER_LCR_SETDIVISOR );
                                           /* Baudratendivisor setzen */
    outp( iSerPort + SER_DIVISOR_LSB, LOBYTE( uDivisor ) );
    outp( iSerPort + SER_DIVISOR_MSB, HIBYTE( uDivisor ) );
                                        /* Divisor-Zugriff verhindern */
    outp( iSerPort + SER_LINE_CONTROL,
                inp( SER_LINE_CONTROL ) & ~SER_LCR_SETDIVISOR );

  /* Andere Parameter erst nach Rcksetzen des Baudraten Latches      */
  /* setzen, da durch diese Operation alle Schnittstellen Parameter   */
  /* gelscht werden!                                                 */
           /* bertragungsparameter auer Baudrate setzen */
    outp( iSerPort + SER_LINE_CONTROL, bParams );
        /* Ein Byte lesen, um evtl. vorhandenen Fehler zurckzusetzen */
    inp( iSerPort + SER_TXBUFFER );
    return TRUE;
  }
  return FALSE;
}

/**********************************************************************/
/* ser_FIFOLevel : Setze Gre des FIFO-Buffers                       */
/**------------------------------------------------------------------**/
/* Eingabe : 0 - FIFO Buffergre = 0, Disable und Reset (1Byte)      */
/*           SER_FIFO_TRIGGER4/8/14 - Gre = 4, 8 oder 14 Byte       */
/**********************************************************************/
VOID ser_FIFOLevel( INT iSerPort, BYTE bLevel )
{
  if( bLevel ) outp( iSerPort + SER_FIFO, bLevel | SER_FIFO_ENABLE );
  else         outp( iSerPort + SER_FIFO, SER_FIFO_RESETRECEIVE |
                                          SER_FIFO_RESETTRANSMIT );
}

/**********************************************************************/
/* ser_IsDataAvaiable : Stehen Daten zum Lesen bereit ?               */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort - Basisport der zu berprfenden Schnittstelle. */
/* Ausgabe : == 0 : Kein Byte zum Lesen vorhanden                     */
/*           != 0 : Byte steht bereit                                 */
/**------------------------------------------------------------------**/
/* Info : Ein Byte wird Bit fr Bit versendet, und ist somit erst     */
/*        dann wieder ein vollstndiges Byte, wenn die empfangende    */
/*        Schnittstelle die einzelnen Bits zusammengesetzt hat. Dies  */
/*        wird von dieser Funktion geprft.                           */
/**********************************************************************/
INT ser_IsDataAvaiable( INT iSerPort )
{
  return inp( iSerPort + SER_LINE_STATUS ) & SER_LSR_DATARECEIVED;
}

/**********************************************************************/
/* ser_IsWritingPossible : Kann Schnittstelle nchstes Byte senden ?  */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort - Basisport der zu berprfenden Schnittstelle. */
/* Ausgabe : == 0 : Byte darf nicht gesendet werden.                  */
/*           != 0 : Schnittstelle zum Senden bereit.                  */
/**------------------------------------------------------------------**/
/* Info : Eine serielle Schnittstelle sollte in folgenden Fllen      */
/*        nicht zum Senden eines Bytes herangezogen werden:           */
/*        1. Ein empfangenes Byte wurde noch nicht von Schnittstelle  */
/*           "abgeholt".                                              */
/*        2. Ein alter Sendeauftrag ist noch nicht abgeschlossen.     */
/**********************************************************************/
INT ser_IsWritingPossible( INT iSerPort )
{
  return ( inp( iSerPort + SER_LINE_STATUS ) & SER_LSR_TSREMPTY );
}

/**********************************************************************/
/* ser_IsModemStatusSet : Zustand der Eingangs-Leitungen berprfen   */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort    - Basisport der Schnittstelle.               */
/*           bTestStatus - Bitmuster der zu testenden Leitungen       */
/*                         (CTS, DSR, RI, CD)                         */
/**********************************************************************/
INT ser_IsModemStatusSet( INT iSerPort, BYTE bTestStatus )
{
  return ( ( BYTE )inp( iSerPort + SER_MODEM_STATUS ) & bTestStatus )
          == bTestStatus;
}

/**********************************************************************/
/* ser_SetModemControl : Signalleitungen fr Kommunikation mit Modem  */
/*                       etc. setzen.                                 */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort    - Basisport der Schnittstelle.               */
/*           bNewControl - Neuer Zustand der DTR, RTS etc. Leitungen  */
/**********************************************************************/
VOID ser_SetModemControl( INT iSerPort, BYTE bNewControl )
{
  outp( iSerPort + SER_MODEM_CONTROL, bNewControl );
}

/**********************************************************************/
/* ser_WriteByte : Ein Byte senden                                    */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort - Basisport der Schnittstelle, ber die ein     */
/*                      Byte versendet werden soll.                   */
/*           bData    - zu versendendes Byte                          */
/*           uTimeOut - Anzahl der Schleifendurchlufe nach denen bei */
/*                      Misserfolg des Sendens ein TimeOut-Fehler     */
/*                      gemeldet wird. (Ist iTimeOut = 0 wird bis in  */
/*                      "alle Ewigkeit" gewartet.)                    */
/*           bSigMask - Bitmaske der zu testenden Signalleitungen     */
/*                      (RTS, CTS, CD, RI)                            */
/*           bSigVals - Zustand der Signalleitungen nach dem aus-     */
/*                      maskieren mit obiger Maske.                   */
/* Ausgabe : == 0 - Byte wurde gesendet                               */
/*           != 0 - Fehler                                            */
/**********************************************************************/
INT ser_WriteByte( INT iSerPort, BYTE bData, UINT uTimeOut,
                   BYTE bSigMask, BYTE bSigVals )
{ if( uTimeOut )                                  /* TimeOut-Schleife */
  {
    while( !ser_IsWritingPossible( iSerPort ) && uTimeOut ) uTimeOut--;
    if( !uTimeOut ) return SER_ERRTIMEOUT;
  }
  else while( !ser_IsWritingPossible( iSerPort ) );        /* Warten! */

                                            /* Signalleitungen testen */
  if( ( ( BYTE ) inp( iSerPort + SER_MODEM_STATUS ) & bSigMask ) ==
                 bSigVals )
  {                     /* Byte zum Senden an Schnittstelle bergeben */
    outp( iSerPort + SER_TXBUFFER, bData );
                                 /* Schnittstellen-Fehler zurckgeben */
    return inp( iSerPort + SER_LINE_STATUS ) & SER_LSR_ERRORMSK;
  }
  else return SER_ERRSIGNALS;
}

/**********************************************************************/
/* ser_ReadByte : Byte empfangen                                      */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort - Basisport der Schnittstelle, ber die ein     */
/*                      Byte empfangen werden soll.                   */
/*           pData    - Adresse des Bytes, das das empfangene Byte     */
/*                      aufnehmen soll.                               */
/*           uTimeOut - Anzahl der Schleifendurchlufe nach denen bei */
/*                      Misserfolg des Empfangens ein TimeOut-Fehler  */
/*                      gemeldet wird. (Ist iTimeOut = 0 wird bis in  */
/*                      "alle Ewigkeit" gewartet.)                    */
/*           bSigMask - Bitmaske der zu testenden Signalleitungen     */
/*                      (RTS, CTS, CD, RI)                            */
/*           bSigVals - Zustand der Signalleitungen nach dem aus-     */
/*                      maskieren mit obiger Maske.                   */
/* Ausgabe : == 0 - Byte wurde gesendet                               */
/*           != 0 - Fehler                                            */
/**********************************************************************/
INT ser_ReadByte( INT iSerPort, PBYTE pData, UINT uTimeOut,
                  BYTE bSigMask, BYTE bSigVals )
{ if( uTimeOut )                                  /* TimeOut-Schleife */
  {
    while( !ser_IsDataAvaiable( iSerPort ) && uTimeOut ) uTimeOut--;
    if( !uTimeOut ) return SER_ERRTIMEOUT;
  }
  else while( !ser_IsDataAvaiable( iSerPort ) );             /* Warten! */

                                            /* Signalleitungen testen */
  if( ( ( BYTE ) inp( iSerPort + SER_MODEM_STATUS ) & bSigMask ) ==
          bSigVals )
  {                       /* empfangenes Byte von Schnittstelle lesen */
    *pData = ( BYTE )inp( iSerPort + SER_RXBUFFER );
    return inp( iSerPort + SER_LINE_STATUS ) & SER_LSR_ERRORMSK;
  }
  else return SER_ERRSIGNALS;
}

/**********************************************************************/
/* ser_WritePaket : Daten-Paket senden                                */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort - Basisport der Schnittstelle, ber die Daten   */
/*                      versendet werden soll.                        */
/*           pData    - Adresse der zu versendenden Daten             */
/*           iLen     - >= 0 : Anzahl der zu sendenden Bytes.         */
/*                       < 0 : Gre des Buffer = strlen( pData )     */
/*           uTimeOut - Anzahl der Schleifendurchlufe nach denen bei */
/*                      Misserfolg des Sendens ein TimeOut-Fehler     */
/*                      gemeldet wird. (Ist iTimeOut = 0 wird bis in  */
/*                      "alle Ewigkeit" gewartet.)                    */
/*           bSigMask - Bitmaske der zu testenden Signalleitungen     */
/*                      (RTS, CTS, CD, RI)                            */
/*           bSigVals - Zustand der Signalleitungen nach dem aus-     */
/*                      maskieren mit obiger Maske.                   */
/* Ausgabe : == 0 - Byte wurde gesendet                               */
/*           != 0 - Fehler                                            */
/**********************************************************************/
INT ser_WritePaket( INT iSerPort, PBYTE pData, INT iLen,
                    UINT uTimeOut, BYTE bSigMask, BYTE bSigVals )
{ INT i, e;
  if( iLen < 0 ) { iLen = 0; while( pData[ iLen ] ) iLen++; }
  for( i = 0; i < iLen; i++ )
    if( ( e = ser_WriteByte( iSerPort,
                             pData[ i ],
                             uTimeOut,
                             bSigMask,
                             bSigVals ) ) != 0 ) return e;
  return SER_SUCCESS;
}

/**********************************************************************/
/* ser_ReadPaket : Daten-Paket empfangen                              */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort - Basisport der Schnittstelle, ber die Daten   */
/*                      empfangen werden soll.                        */
/*           pData    - Adresse der zu sendenden Daten                */
/*           iLen     - Gre des Empfangsbuffers                     */
/*           uTimeOut - Anzahl der Schleifendurchlufe nach denen bei */
/*                      Misserfolg des Sendens ein TimeOut-Fehler     */
/*                      gemeldet wird. (Ist iTimeOut = 0 wird bis in  */
/*                      "alle Ewigkeit" gewartet.)                    */
/*           bSigMask - Bitmaske der zu testenden Signalleitungen     */
/*                      (RTS, CTS, CD, RI)                            */
/*           bSigVals - Zustand der Signalleitungen nach dem aus-     */
/*                      maskieren mit obiger Maske.                   */
/* Ausgabe : == 0 - Byte wurde gesendet                               */
/*           != 0 - Fehler                                            */
/**********************************************************************/
INT ser_ReadPaket( INT iSerPort, PBYTE pData, INT iLen,
                   UINT uTimeOut, BYTE bSigMask, BYTE bSigVals )
{ INT i, e;
  for( i = 0; i < iLen; i++ )
    if( ( e = ser_ReadByte( iSerPort,
                           &pData[ i ],
                            uTimeOut,
                            bSigMask,
                            bSigVals ) ) != 0 ) return e;
  return SER_SUCCESS;
}

/**********************************************************************/
/* ser_CLRIRQ : Meldung serieller Interrupts an IRQ-Controller        */
/*              unterbinden.                                          */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort - Basisport der Schnittstelle, die keine IRQ's  */
/*                      mehr an den IRQ-Controller weiterleiten darf. */
/**********************************************************************/
VOID ser_CLRIRQ( INT iSerPort )
{
  outp( iSerPort + SER_MODEM_CONTROL,
        inp( iSerPort + SER_MODEM_CONTROL ) & ~SER_MCR_IRQENABLED );
}

/**********************************************************************/
/* ser_SETIRQ : Meldung serieller Interrupts an IRQ-Controller        */
/*              erlauben.                                             */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort - Basisport der Schnittstelle, der IRQ's an den */
/*                      IRQ-Controller weiterleiten mu.              */
/**********************************************************************/
VOID ser_SETIRQ( INT iSerPort )
{
  outp( iSerPort + SER_MODEM_CONTROL,
        inp( iSerPort + SER_MODEM_CONTROL ) | SER_MCR_IRQENABLED );
}

/**********************************************************************/
/* ser_SetIRQHandler : Interrupt-Handler setzen                       */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort  - Basisport der Seriellenschnittstelle, fr    */
/*                       die ein Interrupt-Handler eingerichtet       */
/*                       werden soll.                                 */
/*           iSerIRQ   - von der Schnittstelle belgte IRQ(!)-Leitung. */
/*           lpHandler - Adresse des Interrupt-Handlers.              */
/*           bEnablers - Zustnde, durch die ein IRQ ausgelst werden */
/*                       soll (s. SER_IER_...-Bits)                   */
/* Ausgabe : Adresse des alten IRQ-Handlers                           */
/**********************************************************************/
VOID ( _interrupt _FP *ser_SetIRQHandler( INT iSerPort,
                                           INT iSerIRQ,
                        VOID ( _interrupt _FP *lpHandler ) (),
                                          BYTE bEnablers ) ) ()
{                                              /* IRQ-Auslser setzen */
  outp( iSerPort + SER_IRQ_ENABLE, bEnablers );
  ser_SETIRQ( iSerPort );           /* IRQ's an IRQ-Controller melden */
                          /* Handler setzen (IRQ wird dort "enabled") */
  return irq_SetHandler( iSerIRQ, lpHandler );
}

/**********************************************************************/
/* ser_RestoreIRQHandler : Alten IRQ-Handler restaurieren             */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort  - Basisport der Seriellenschnittstelle, deren  */
/*                       alter Interrupt-Handler wieder eingerichtet  */
/*                       werden soll.                                 */
/*           iSerIRQ   - von der Schnittstelle belgte IRQ(!)-Leitung. */
/*           lpHandler - Adresse des alten Interrupt-Handlers.        */
/**********************************************************************/
VOID ser_RestoreIRQHandler( INT iSerPort,
                            INT iSerIRQ,
         VOID ( _interrupt _FP *lpHandler ) () )
{
  ser_CLRIRQ( iSerPort );/* IRQ's nicht mehr an IRQ-Controller melden */
                         /* Handler setzen und alle "Enabler" lschen */
  ser_SetIRQHandler( iSerPort, iSerIRQ, lpHandler, 0 );
  irq_Disable( iSerIRQ );     /* IRQ auch controllerseitig "disablen" */
}

/**********************************************************************/
/* ser_PrintError : Fehlermeldung ausgeben                            */
/**------------------------------------------------------------------**/
/* Eingabe : pWin - Adresse eines Windows, in das Ausgabe erfolgen    */
/*                  soll, oder NULL fr Ausgabe auf Bildschirm.       */
/*           e - Fehler                                               */
/**********************************************************************/
VOID ser_PrintError( PWINDOW pWin, INT e )
{
  switch( e )
  {
    case SER_LSR_DATARECEIVED:
         win_printf( pWin, "Alte Daten!\n" );
    break;
    case SER_ERRTIMEOUT:
         win_printf( pWin, "TimeOut-Error! ");
    break;
    case SER_ERRSIGNALS:
         win_printf( pWin, "Signalleitungen!");
    break;
    default:
      if( e & SER_LSR_OVERRUNERROR )
          win_printf( pWin, "Overrun Error!\n" );
      if( e & SER_LSR_PARITYERROR )
          win_printf( pWin, "Parity Error!\n" );
      if( e & SER_LSR_FRAMINGERROR )
          win_printf( pWin, "Framing Error!\n" );
      if( e & SER_LSR_BREAKDETECT )
          win_printf( pWin, "Break detect!\n" );
  }
}

/**********************************************************************/
/* ser_PrintModemStatus : Zustand der Signalleitungen anzeigen        */
/**------------------------------------------------------------------**/
/* Eingabe : pWin - Adresse eines Windows, in das Ausgabe erfolgen    */
/*                  soll, oder NULL fr Ausgabe auf Bildschirm.       */
/*           iSerPort - Basisport der Schnittstelle, deren Leitungs-  */
/*                      Zustnde ausgegeben werden sollen.            */
/**********************************************************************/
VOID ser_PrintModemStatus( PWINDOW pWin, INT iSerPort )
{ BYTE b;
  b = ( BYTE )inp( iSerPort + SER_MODEM_STATUS );
  win_printf(pWin,"%s", b & SER_MSR_DCTS ? "DCTS :" : " CTS :" );
  win_printf(pWin,"%s\n", b & SER_MSR_CTS ? "[X]" : "[ ]" );
  win_printf(pWin,"%s", b & SER_MSR_DDSR ? "DDSR :" : " DSR :" );
  win_printf(pWin,"%s\n", b & SER_MSR_DSR ? "[X]" : "[ ]" );
  win_printf(pWin,"%s", b & SER_MSR_DRI ? "DRI  :" : " RI  :" );
  win_printf(pWin,"%s\n", b & SER_MSR_RI ? "[X]" : "[ ]" );
  win_printf(pWin,"%s", b & SER_MSR_DCD ? "DCD  :" : " CD  :" );
  win_printf(pWin, "%s\n", b & SER_MSR_CD ? "[X]" : "[ ]" );
}

/**********************************************************************/
/* ser_PrintLineStatus : Zustand der Schnittstelle anzeigen           */
/**------------------------------------------------------------------**/
/* Eingabe : pWin - Adresse eines Windows, in das Ausgabe erfolgen    */
/*                  soll, oder NULL fr Ausgabe auf Bildschirm.       */
/*           iSerPort - Basisport der Schnittstelle, deren Interne-   */
/*                      Zustnde ausgegeben werden sollen.            */
/**********************************************************************/
VOID ser_PrintLineStatus( PWINDOW pWin, INT iSerPort )
{ BYTE b;
  b = ( BYTE )inp( iSerPort + SER_MODEM_STATUS );
  win_printf(pWin, "Data received %s\n",
                   b & SER_LSR_DATARECEIVED ? "[X]" : "[ ]" );
  win_printf(pWin, "Overrun Error %s\n",
                  b & SER_LSR_OVERRUNERROR ? "[X]" : "[ ]" );
  win_printf(pWin, "Parity Error  %s\n",
                  b & SER_LSR_PARITYERROR  ? "[X]" : "[ ]" );
  win_printf(pWin, "Framing Error %s\n",
                  b & SER_LSR_FRAMINGERROR ? "[X]" : "[ ]" );
  win_printf(pWin, "Break detect  %s\n",
                  b & SER_LSR_BREAKDETECT ? "[X]" : "[ ]" );
  win_printf(pWin, "THR Empty     %s\n",
                  b & SER_LSR_THREMPTY ? "[X]" : "[ ]" );
  win_printf(pWin, "TSR Empty     %s\n",
                  b & SER_LSR_TSREMPTY ? "[X]" : "[ ]" );
}

/**********************************************************************/
/* ser_GetBaud : Aktuelle Baudrate einer Schnittstelle ermitteln      */
/**------------------------------------------------------------------**/
/* Eingabe : iSerPort - Basisadresse der Schnittstelle, deren Baud-   */
/*                      rate ermittelt werden soll.                   */
/* Ausgabe : Baudrate                                                 */
/**********************************************************************/
LONG ser_GetBaud( INT iSerPort )
{ UINT uDivisor;
  BYTE bSettings;

  _disable();
  bSettings = ( BYTE ) inp( iSerPort + SER_LINE_CONTROL );
  outp( iSerPort + SER_LINE_CONTROL, bSettings | SER_LCR_SETDIVISOR );
                                            /* Baudratendivisor lesen */
  uDivisor = MAKEWORD( inp( iSerPort + SER_DIVISOR_MSB ),
                       inp( iSerPort + SER_DIVISOR_LSB ) );

  outp( iSerPort + SER_LINE_CONTROL, bSettings );
  _enable();
  if( uDivisor ) return SER_MAXBAUD / uDivisor;
  return 0L;
}

/**********************************************************************/
/* ser_PrintCardSettings : bertragungsparameter der Schnittstelle    */
/*                         anzeigen                                   */
/**------------------------------------------------------------------**/
/* Eingabe : pWin - Adresse eines Windows, in das Ausgabe erfolgen    */
/*                  soll, oder NULL fr Ausgabe auf Bildschirm.       */
/*           iSerPort - Basisport der Schnittstelle, deren ber-      */
/*                      tragungsparameter ausgegeben werden sollen.   */
/**********************************************************************/
VOID ser_PrintCardSettings( PWINDOW pWin, INT iSerPort )
{
  LONG lBaudRate;
  BYTE bSettings;
  INT WordLen;

  switch( ser_UARTType( iSerPort ) )
  {
    case NOSER:
         win_printf(pWin, " Keine Schnittstelle entdeckt!\n");
         return;
    case INS8250:
         win_printf(pWin, " INS8250 UART-Chip\n");
         break;
    case NS16450:
         win_printf(pWin, " NS16450 UART-Chip\n");
         break;
    case NS16550A:
         win_printf(pWin, " NS16550A UART-Chip\n");
         break;
    case NS16C552:
         win_printf(pWin, " NS16C552 UART-Chip\n");
         break;
  }
  lBaudRate = ser_GetBaud( iSerPort );
  win_printf(pWin, " Baudrate %ld\n", lBaudRate );

  bSettings = ( BYTE ) inp( iSerPort + SER_LINE_CONTROL );
  WordLen = 5 + ( bSettings & SER_LCR_WORDLEN );
  win_printf(pWin, " Datenbits : %d\n", WordLen );
  win_printf(pWin, " StopBits  : %s\n",
                    bSettings & SER_LCR_2STOPBITS ?
                    ( WordLen == 5 ? "1.5" : "2" ) : "1");
  win_printf(pWin, " Paritt   : ");
  switch( bSettings & SER_LCR_PARITYMSK )
  {
    case SER_LCR_ODDPARITY:  win_printf(pWin, "un");
    case SER_LCR_EVENPARITY: win_printf(pWin, "gerade"); break;
    case SER_LCR_PARITYSET:  win_printf(pWin, "Immer gesetzt"); break;
    case SER_LCR_PARITYCLR:  win_printf(pWin, "Immer gelscht"); break;
    default: win_printf(pWin, "keine");
  }                                  /* PARITYON = 0 => keine Paritt */

  win_printf(pWin, "\n");
  if( bSettings & SER_LCR_SENDBREAK )
      win_printf(pWin, "Sende Break-Signal\n");
}
#endif

