/**********************************************************************/
/*                               T S R C                              */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : C-Modul, das mit Hilfe eines Assembler-Moduls  */
/*                     in ein TSR-Programm verwandelt wird            */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 15.08.1988                                     */
/*    letztes Update : 19.11.1991                                     */
/*--------------------------------------------------------------------*/
/*    Speichermodell : SMALL                                          */
/**********************************************************************/

/*== Include-Dateien einbinden =======================================*/

#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include <string.h>
#include <bios.h>

/*== Typedefs ========================================================*/
                                                                        
typedef unsigned char BYTE;               /* wir basteln uns ein Byte */
typedef unsigned int WORD;
typedef BYTE BOOL;                           /* wie BOOLEAN in Pascal */
typedef union vel far * VP;     /* VP ist ein FAR-Pointer in den VRAM */
                                                                        
typedef void (*OAFP)(void);    /* Zeiger auf Funktion  ohne Argumente */
typedef void (*SHKFP)( WORD KeyMask, BYTE ScCode );   /* TsrSetHotkey */

/*== Makros ==========================================================*/
                                                                        
#ifndef MK_FP                    /* wurde MK_FP noch nicht definiert? */
#define MK_FP(seg, ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))
#endif
#define VOFS(x,y) ( 80 * ( y ) + ( x ) )
#define VPOS(x,y) (VP) ( vptr + VOFS( x, y ) )
                                                                        
/*== Strukturen und Unions ===========================================*/
                                                                        
struct velb {       /* beschreibt eine Bildschirmposition als 2 Bytes */
             BYTE zeichen,                          /* der ASCII-Code */
                  attribut;                /* das zugehrige Attribut */
            };
                                                                        
struct velw {        /* beschreibt eine Bildschirmposition als 1 Word */
             WORD inhalt;     /* nimmt ASCII-Zeichen und Attribut auf */
            };

union vel {                     /* beschreibt eine Bildschirmposition */
           struct velb h;
           struct velw x;
          };

/*== Einbindung der Funktionen aus dem Assembler-Modul ===============*/

extern void TsrInit( BOOL tc, void (*fkt)(void), unsigned heap );
extern BOOL TsrIsInst( BYTE i2F_fktnr );
extern void TsrUnInst( void );
extern OAFP TsrSetPtr( void far *fkt );
extern BOOL TsrCanUnInst( void );
extern void TsrCall( void );
extern void far TsrSetHotkey( WORD keymask, BYTE sccode );

/*== Konstanten und Makros ===========================================*/
                                                                        
#ifdef __TURBOC__                     /* wird mit TURBO-C compiliert? */
  #include <alloc.h>
  #define TC TRUE                                               /* Ja */
  #define KeyAvail() ( bioskey(1) != 0 )
  #define GetKey() bioskey(0)
#else                           /* es wird mit Microsoft C gearbeitet */
  #include <malloc.h>
  #define TC FALSE
  #define KeyAvail() ( _bios_keybrd( _KEYBRD_READY ) != 0 )
  #define GetKey() _bios_keybrd( _KEYBRD_READ )
#endif

/*-- Scan-Codes der verschiedenen Tasten -----------------------------*/

#define SC_ESC             0x01
#define SC_1               0x02
#define SC_2               0x03
#define SC_3               0x04
#define SC_4               0x05
#define SC_5               0x06
#define SC_6               0x07
#define SC_7               0x08
#define SC_8               0x09
#define SC_9               0x0A
#define SC_0               0x0B
#define SC_SCHARFES_S      0x0C
#define SC_APOSTROPH       0x0D
#define SC_BACKSPACE       0x0E
#define SC_TAB             0x0F
#define SC_Q               0x10
#define SC_W               0x11
#define SC_E               0x12
#define SC_R               0x13
#define SC_T               0x14
#define SC_Z               0x15
#define SC_U               0x16
#define SC_I               0x17
#define SC_O               0x18
#define SC_P               0x19
#define SC_UE              0x1A
#define SC_PLUS            0x1B
#define SC_RETURN          0x1C
#define SC_CONTROL         0x1D
#define SC_A               0x1E
#define SC_S               0x1F
#define SC_D               0x20
#define SC_F               0x21
#define SC_G               0x22
#define SC_H               0x23
#define SC_J               0x24
#define SC_K               0x25
#define SC_L               0x26
#define SC_OE              0x27
#define SC_AE              0x28
#define SC_GROESSER        0x29
#define SC_SHIFT_LINKS     0x2A
#define SC_FIS             0x2B
#define SC_Y               0x2C
#define SC_X               0x2D
#define SC_C               0x2E
#define SC_V               0x2F
#define SC_B               0x30
#define SC_N               0x31
#define SC_M               0x32
#define SC_KOMMA           0x33
#define SC_PUNKT           0x34
#define SC_STRICH          0x35
#define SC_SHIFT_RECHTS    0x36
#define SC_PRINT_SCREEN    0x37
#define SC_ALT             0x38
#define SC_SPACE           0x39
#define SC_CAPS            0x3A
#define SC_F1              0x3B
#define SC_F2              0x3C
#define SC_F3              0x3D
#define SC_F4              0x3E
#define SC_F5              0x3F
#define SC_F6              0x40
#define SC_F7              0x41
#define SC_F8              0x42
#define SC_F9              0x43
#define SC_F10             0x44
#define SC_NUM_LOCK        0x45
#define SC_SCROLL_LOCK     0x46
#define SC_CURSOR_HOME     0x47
#define SC_CURSOR_UP       0x48
#define SC_CURSOR_PG_UP    0x49
#define SC_NUM_MINUS       0x4A
#define SC_CURSOR_LEFT     0x4B
#define SC_NUM_5           0x4C
#define SC_CURSOR_RIGHT    0x4D
#define SC_NUM_PLUS        0x4E
#define SC_CURSOR_END      0x4F
#define SC_CURSOR_DOWN     0x50
#define SC_CURSOR_PG_DOWN  0x51
#define SC_INSERT          0x52
#define SC_DELETE          0x53
#define SC_SYS_REQUEST     0x54
#define SC_F11             0x57
#define SC_F12             0x58
#define SC_NOKEY           0x80            /* keine zustzliche Taste */

/*-- Codes der Umschalttasten fr den Aufbau der Hotkey-Maske --------*/
                                                                        
#define RSHIFT       1           /* rechte SHIFT-Taste niedergedrckt */
#define LSHIFT       2            /* linke SHIFT-Taste niedergedrckt */
#define CTRL         4                   /* CTRL-Taste niedergedrckt */
#define ALT          8                    /* ALT-Taste niedergedrckt */
#define SYSREQ    1024             /* SYS-REQ-Taste (nur AT-Tastatur) */
#define BREAK     4096                  /* Break-Taste niedergedrckt */
#define NUM       8192                     /* Num-Lock niedergedrckt */
#define CAPS     16384                    /* Caps-Lock niedergedrckt */
#define INSERT   32768                 /* INSERT-Taste niedergedrckt */
                                                                        
#define I2F_CODE   0xC4                     /* Funktionsnummer INT 2F */
#define I2F_FKT_0  0xAA                /* Code fr INT 2F, Funktion 0 */
#define I2F_FKT_1  0xBB                /* Code fr INT 2F, Funktion 1 */

#define NOF      0x07                                /* normale Farbe */
#define INV      0x70                                /* inverse Farbe */
#define HNOF     0x0f                          /* helle normale Farbe */
#define HINV     0xf0                          /* helle inverse Farbe */
                                                                        
#define HEAP_FREI 1024           /* 1 KByte auf dem Heap Platz lassen */
                                                                        
#define TRUE  ( 0 == 0 )            /* Konstanten zur Arbeit mit BOOL */
#define FALSE ( 0 == 1 )

/*== globale Variablen ===============================================*/
                                                                        
VP vptr;                /* Pointer auf das erste Zeichen im Video-RAM */
unsigned atimes = 0;        /* Anzahl Aktivierungen des TSR-Programms */
union vel * scrbuf;        /* Pointer auf Puffer mit Bildschirminhalt */
char * leerzeile;                       /* Pointer auf eine Leerzeile */

/***********************************************************************
*  Funktion         : D I S P _ I N I T                                *
**--------------------------------------------------------------------**
*  Aufgabe          : Ermittelt die Basisadresse des Video-RAM.        *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : keiner                                           *
***********************************************************************/
                                                                        
void disp_init(void)
 {
  union REGS regs;           /* Prozessorregs fr den Interruptaufruf */
                                                                        
  regs.h.ah = 15;           /* Funktionsnummer: Video-Modus ermitteln */
  int86(0x10, &regs, &regs);     /* den BIOS-Video-Interrupt aufrufen */
                                                                        
  /*-- je nach Video-Modus Basisadresse des Video-RAM berechnen ------*/
                                                                        
  vptr = (VP) MK_FP((regs.h.al == 7) ? 0xb000 : 0xb800, 0);
 }
                                                                        
/***********************************************************************
*  Funktion         : D I S P _ P R I N T                              *
**--------------------------------------------------------------------**
*  Aufgabe          : Gibt einen String auf dem Bildschirm aus.        *
*  Eingabe-Parameter: - SPALTE   = Die Ausgabespalte.                  *
*                     - ZEILE    = Die Ausgabezeile.                   *
*                     - FARBE    = Attribut fr die Zeichen.           *
*                     - STRING   = Pointer auf String                  *
*  Return-Wert      : keiner                                           *
***********************************************************************/
                                                                        
void disp_print(BYTE spalte, BYTE zeile, BYTE farbe, char * string)
 {
  register VP lptr;        /* Laufzeiger zum Zugriff in den Video-RAM */
                                                                        
  lptr = VPOS(spalte, zeile);          /* Pointer in Video-RAM setzen */
  for ( ; *string ; ++lptr)             /* den String bis durchlaufen */
   {
    lptr->h.zeichen = *(string++);  /* Zeichen in VIDEO-RAM schreiben */
    lptr->h.attribut = farbe;      /* Attribut fr das Zeichen setzen */
   }
 }
                                                                        
/***********************************************************************
*  Funktion         : S A V E _ S C R E N                              *
**--------------------------------------------------------------------**
*  Aufgabe          : Sichert den Bildschirminhalt in einem Puffer.    *
*  Eingabe-Parameter: - SPTR     = Pointer auf den Puffer, in dem der  *
*                                  Bildschirm gespeichert wird.        *
*  Return-Wert      : keiner                                           *
*  Info             : Es wird davon ausgegangen, da der Puffer gro   *
*                     genug ist, um den Bildschirminhalt aufzunehmen.  *
***********************************************************************/
                                                                        
void save_screen( union vel * sptr )
 {
  register VP lptr;        /* Laufzeiger zum Zugriff in den Video-RAM */
  unsigned i;                                      /* Schleifenzhler */
  lptr = VPOS(0, 0);                   /* Pointer in Video-RAM setzen */
                                                                        
  for (i=0; i<2000; i++) /* die 2000 Bildschirmpositionen durchlaufen */
   (sptr++)->x.inhalt = (lptr++)->x.inhalt; /* Zeichen & Attr. merken */
 }
                                                                        
/***********************************************************************
*  Funktion         : R E S T O R E _ S C R E E N                      *
**--------------------------------------------------------------------**
*  Aufgabe          : Kopiert den Inhalt eines Puffers in den Video-   *
*                     RAM.                                             *
*  Eingabe-Parameter: - SPTR     = Pointer auf den Puffer, in dem sich *
*                                  der Bildschirminhalt befindet.      *
*  Return-Wert      : keiner                                           *
***********************************************************************/
                                                                        
void restore_screen( union vel * sptr )
 {
  register VP lptr;        /* Laufzeiger zum Zugriff in den Video-RAM */
  unsigned i;                                      /* Schleifenzhler */
  lptr = VPOS(0, 0);                   /* Pointer in Video-RAM setzen */
                                                                        
  for (i=0; i<2000; i++) /* die 2000 Bildschirmpositionen durchlaufen */
   (lptr++)->x.inhalt = (sptr++)->x.inhalt; /* Z. & Attr. zurckholen */
 }
                                                                        
/***********************************************************************
*  Funktion         : E N D F K T                                      *
**--------------------------------------------------------------------**
*  Aufgabe          : Wird bei der Reinstallation des TSR-Programms    *
*                     aufgerufen.                                      *
*  Eingabe-Parameter: keiner                                           *
*  Return-Wert      : keiner                                           *
*  Info             : Dieser Prozedur mu FAR sein, damit sie in der   *
*                     bereits installierten Kopie des TSR-Programms    *
*                     aufgerufen werden kann.                          *
***********************************************************************/
                                                                        
void far endfkt( void )
{
 /*-- die allokierten Puffer wieder freigeben ------------------------*/
                                                                        
 free( leerzeile );        /* den allokierten Puffer wieder freigeben */
 free( (void *) scrbuf );      /* allokierten Puffer wieder freigeben */
                                                                        
 printf("Das TSR-Programm wurde %u mal aktiviert.\n", atimes);
}
                                                                        
/***********************************************************************
*  Funktion         : T S R                                            *
**--------------------------------------------------------------------**
*  Aufgabe          : Wird bei Bettigung des Hotkey durch das Assem-  *
*                     blermodul aufgerufen.                            *
*  Eingabe-Parameter: keiner                                           *
*  Return-Wert      : keiner                                           *
***********************************************************************/
                                                                        
void tsr( void )
{
 BYTE i;                                           /* Schleifenzhler */
                                                                        
 ++atimes;                 /* Anzahl der Aktivierungen inkrementieren */

 while ( KeyAvail() )                   /* Tastaturpuffer leer machen */
  GetKey();

 disp_init();                      /* Adresse des Video-RAM ermitteln */
 save_screen( scrbuf );      /* den aktuellen Bildschirminhalt merken */
 for (i=0; i<25; i++)          /* die 25 Bildschirmzeilen durchlaufen */
  disp_print(0, i, INV, leerzeile);              /* die Zeile lschen */
 disp_print(22, 11, INV, "TSRC  -  (c) 1988, 91 by MICHAEL TISCHER");
 disp_print(28, 13, INV, "Bitte eine Taste bettigen ...");
 GetKey();                                   /* auf eine Taste warten */
 restore_screen( scrbuf );     /* den alten Bildschirm zurckkopieren */
}
                                                                        
/**********************************************************************/
/* GetHeapEnd: Ermittelt das aktuelle Ende des Heps in Abhngigkeit   */
/*             des Compilers                                          */
/* Eingabe : keine                                                    */
/* Ausgabe : Zeiger auf das erste Byte hinter dem Ende des belegten   */
/*           Heaps                                                    */
/**********************************************************************/

void far *GetHeapEnd( void )
{
 #ifdef __TURBOC__                                        /* TurboC ? */
   return (void far *) sbrk(0);
 #else                                                   /* Nein, MSC */
   struct _heapinfo hi; /* Stuktur mit Informatioen ber Heap-Eintrag */
   unsigned heapstatus;                     /* Status von _heapwalk() */
   void far *letzter;            /* Zeiger auf letzten belegten Block */

   hi._pentry = NULL;                       /* am Heap-Start anfangen */

   /*-- Heap bis zum letzten Block durchlaufen -----------------------*/

   while( (heapstatus = _heapwalk( &hi )) != _HEAPEND )
    if ( hi._useflag == _USEDENTRY )                 /* Block belegt? */
     letzter = (void far *) ((BYTE far *) hi._pentry + hi._size + 1);

   return letzter;
 #endif
}

/**********************************************************************/
/* ParamGetHotKey: Sucht unter den Kommanodzeilenparametern nach Hot- */
/*                 key-Befehlen (/T) und wertet diese aus             */
/* Eingabe : ARGC,   = Kommandozeilen-Parameter, wie bei main()       */
/*           ARGV                                                     */
/*           KEYMASK = Zeiger auf Variable, die die Key-Maske auf-    */
/*                     nehmen soll                                    */
/*           SCCODE  = Zeiger auf Variable, die den Scan-Code auf-    */
/*                     nehmen soll                                    */
/* Ausgabe : TRUE, wenn die Hotkeys bekannt sind, sonst FALSE         */
/* Info    : - Parameter, die nicht mit /T beginnen, lt die Funktion*/
/*             unangetastet, damit sie von einer anderen Routine aus- */
/*             gewertet werden knnen                                 */
/*           - Werden keine /T-Parameter entdeckt, wird in der je-    */
/*             weiligen Variable 0 bzw. SC_NOKEY zurckgeliefert.     */
/**********************************************************************/
                                                                        
BOOL ParamGetHotKey(int argc, char *argv[], WORD *KeyMask, BYTE *ScCode)

{
 struct UmschaltT {
                   char Name[7];
                   WORD Wert;
                  };

 static struct UmschaltT UTasten[9] =
                 { { "LSHIFT", LSHIFT },
                   { "RSHIFT" ,RSHIFT },
                   { "CTRL"   ,CTRL   },
                   { "ALT"    ,ALT    },
                   { "SYSREQ" ,SYSREQ },
                   { "BREAK"  ,BREAK  },
                   { "NUM"    ,NUM    },
                   { "CAPS"   ,CAPS   },
                   { "INSERT" ,INSERT }
                  };

 int i , j,                                        /* Schleifenzhler */
     code;                            /* zur Umwandlung des Scan-Code */
     char arg[80];                    /* zur Aufnahme eines Arguments */

 *KeyMask = 0;
 *ScCode = SC_NOKEY;

 for ( i = 1; i < argc; ++i )        /* die Kommandozeile durchlaufen */
  {
   strcpy( arg, argv[i] );                          /* Argument holen */
   strupr( arg );
   if ( arg[0] == '/'  &&  arg[1] == 'T' )
    {                                       /* es ist ein /T-Argument */
     code = atoi( &arg[2] );               /* Code nach binr wandeln */
     if ( code )               /* konnte die Zahl ungweandelt werden? */
      {                                                         /* Ja */
       if ( code < 128 )                            /* gltiger Code? */
        *ScCode = code;                                 /* Ja, merken */
       else
         return FALSE;                             /* ungltiger Code */
      }
     else            /* keine Zahl, mu Name einer Umschalttaste sein */
      {
       for ( j = 0; j < 9; ++ j ) /* Umschalttasten-Array durchsuchen */
        if (!strcmp( UTasten[j].Name, &arg[2] ))  /*String vergleichen*/
          break;                 /* sind gleich, FOR-Schleife beenden */

       if ( j < 9 )                                /* Namen gefunden? */
        *KeyMask = *KeyMask | UTasten[j].Wert;     /* Ja, Flag einbl. */
       else
        return FALSE;                  /* Nein, weder Zahl noch Taste */
      }
    }
  }
 return TRUE;                  /* wenn die Funktion bis hierin kommt, */
}                              /* waren die Parameter o.k.            */

/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/
                                                                        
void main( int argc, char *argv[] )
{
 WORD KeyMask;              /* nimmt Bit-Maske fr Umschalttasten auf */
 BYTE ScCode;                      /* nimmt Scan-Code des Hotkeys auf */

 printf("TSRC  -  (c) 1988, 91 by MICHAEL TISCHER\n");
 if (!ParamGetHotKey( argc, argv, &KeyMask, &ScCode ))
  {                         /* Fehler in den Kommandozeilenparametern */
   printf( "Ungltiger Kommandozeilen-Parameter!\n" );
   exit(1);
  }

 /*-- Kommandozeilen-Parameter waren o.k. ----------------------------*/

 if ( !TsrIsInst( I2F_CODE ))        /* Programm bereits installiert? */
  {                                                           /* Nein */
   atimes = 0;             /* das Programm wurde noch nicht aktiviert */
   printf( "Das TSR-Programm wurde installiert.\n" );
   if ( KeyMask == 0  &&  ScCode == SC_NOKEY )    /* keine Parameter? */
    {                                      /* Nein, Vorgabe ist ALT-H */
     TsrSetHotkey( ALT, SC_H );
     printf( "Start: <ALT> + H\n" );
    }
   else                          /* Hotkey des Anwenders installieren */
    TsrSetHotkey( KeyMask, ScCode );

   /*-- Puffer fr Bildschirmverwaltung allokieren -------------------*/

   scrbuf = (union vel *) malloc(80 * 25 * sizeof(union vel));
   leerzeile = (char *) malloc( 80 + 1 );        /* Puffer allokieren */
   *(leerzeile + 80 ) = '\0';      /* den Puffer mit NUL abschliessen */
   memset(leerzeile, ' ', 80);        /* den Puffer mit Spaces fllen */

   TsrInit( TC, tsr, HEAP_FREI );        /* das Programm installieren */
  }
 else                             /* Programm war bereits installiert */
  {                                                             /* JA */
   if ( KeyMask == 0  &&  ScCode == SC_NOKEY )    /* keine Parameter? */
    {                               /* Nein, Reinstallation versuchen */
     if ( TsrCanUnInst() )
      {
       (*(OAFP) TsrSetPtr(endfkt))();
       TsrUnInst();
       printf( "Das Programm wurde erfolgreich reinstalliert.\n" );
      }
     else
      printf( "Das Programm kann nicht reinstalliert werden.\n" );
    }
   else                                    /* neuen Hotkey bertragen */
    {
     printf( "Neuer Hotkey installiert\n" );
     (*(SHKFP) TsrSetPtr(TsrSetHotkey))( KeyMask, ScCode );
    }
  }
}

