/***********************************************************************
*                              V D A C C . C                           *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Programmierung der DAC-Farb-    *
*                     Register in den 256 Farben Grfikmodi der VGA-    *
*                     Karte. Das Programm greift dabei auf die Assem-  *
*                     bler-Routinen aus dem Modul V3240CA.ASM zurck.  *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    :  2.01.1991                                       *
*  letztes Update am: 14.01.1991                                       *
**--------------------------------------------------------------------**
*  Speichermodell   : SMALL                                            *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung       : CL /AS vdacc.c v3240ca                           *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung       : ber eine Projekt-Datei mit folgendem Inhalt:    *
*                       vdacc.c                                        *
*                       v3240ca.obj                                    *
**--------------------------------------------------------------------**
*  Aufruf           : vdacc                                            *
**--------------------------------------------------------------------**
*  Info             : Warnmeldung "Structure passed by value ..." in   *
*                     Turbo C ist kein Fehler!                         *
***********************************************************************/

#include <dos.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

/*-- Typdeklarationen ------------------------------------------------*/

typedef unsigned char BYTE;               /* wir basteln uns ein BYTE */

typedef union {                        /* beschreibt ein DAC-Register */
               struct { BYTE Rot, Gruen, Blau; } b;
               BYTE RGB[3];
              } DACREG;
typedef DACREG DACARRAY[256];                /* komplette DAC-Tabelle */

/*-- externe Referenzen auf die Assembler-Routinen -------------------*/

extern void init320400( void );
extern void setpix( int x, int y, unsigned char farbe);
extern BYTE getpix( int x, int y );
extern void setpage( BYTE page );
extern void showpage( BYTE page );
extern void far * getfontptr( void );

/*-- Konstanten ------------------------------------------------------*/

#define MAXX 319                        /* Maximale X- und Y-Ordinate */
#define MAXY 399

#define BREITE  10              /* Breite eines Farbblocks in Punkten */
#define HOEHE   20                /* Hhe eines Farbblocks in Punkten */
#define ABSTAND 2                     /* Abstand zwischen den Blcken */
#define GBREITE ( 16 * BREITE + ( 15 * ABSTAND ) )    /* Gesamtbreite */
#define GHOEHE  ( 16 * HOEHE + ( 15 * ABSTAND ) )       /* Gesamthhe */
#define STARTX  ( MAXX - GBREITE ) / 2       /* obere linke Blockecke */
#define STARTY  ( MAXY - GHOEHE ) / 2

/***********************************************************************
*  IsVga: Stellt fest, ob eine VGA-Karte installiert ist.              *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : 0, wenn keine VGA angetroffen wurde, sonst ungleich 0     *
***********************************************************************/

BYTE IsVga( void )
{
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */

 Regs.x.ax = 0x1a00;              /* Funktion 1Ah gibt es nur bei VGA */
 int86( 0x10, &Regs, &Regs );
 return (BYTE) ( Regs.h.al == 0x1a );  /* ist die Funktion verfgbar? */
}

/***********************************************************************
*  PrintChar : Baut ein Zeichen auerhalb des sichtbaren Bereichs des  *
*              Video-RAMs auf.                                         *
**--------------------------------------------------------------------**
*  Eingabe :   zeichen = das aufzubauende Zeichen                      *
*              x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************/

void PrintChar( char zeichen, int x, int y, BYTE vg, BYTE hg )
{
 typedef BYTE ZDEF[256][8];                 /* Aufbau des Zeichensatz */
 typedef ZDEF far *ZPTR;                    /* Zeiger auf Zeichensatz */

 BYTE         i, k,                                /* Schleifenzhler */
              Maske;              /* Bitmaske fr Aufbau des Zeichens */

 static ZPTR fptr = (ZPTR) 0;        /* Zeiger auf Zeichensatz im ROM */

 if ( fptr == (ZPTR) 0 )   /* Zeiger auf Zeichensatz schon ermittelt? */
  fptr = getfontptr();  /*Nein, mit Hilfe der Assembler-Funktion laden*/

 /*- das Zeichen Punkt fr Punkt aufbauen ----------------------------*/

 if ( hg == 255 )                    /* Zeichen transparent zeichnen? */
  for ( i = 0; i < 8; ++i )      /* Ja, nur Vordergrund-Punkte setzen */
   {
    Maske = (*fptr)[zeichen][i];    /* Bitmuster fr eine Zeile holen */
    for ( k = 0; k < 8; ++k, Maske <<= 1 ) /* die Spalten durchlaufen */
     if ( Maske & 128 )                              /* Punkt setzen? */
      setpix( x+k, y+i, vg );                                   /* Ja */
   }
 else                                     /* Nein, jeden Punkt setzen */
  for ( i = 0; i < 8; ++i )                     /* Zeilen durchlaufen */
  {
   Maske = (*fptr)[zeichen][i];     /* Bitmuster fr eine Zeile holen */
   for ( k = 0; k < 8; ++k, Maske <<= 1 )  /* die Spalten durchlaufen */
    setpix( x+k, y+i, ( Maske & 128 ) ? vg : hg );
  }
}

/***********************************************************************
*  Line: Zieht eine Linie nach dem Bresenham-Algorithmus               *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endkoordinate                                    *
*            FARBE  = Farbe der Linie                                  *
***********************************************************************/

/*-- Unterfunktion, vertauscht zwei Integer-Variablen ----------------*/

void SwapInt( int *i1, int *i2 )
{
 int dummy;

 dummy = *i2;
 *i2   = *i1;
 *i1   = dummy;
}

/*-- Hauptteil der Funktion ------------------------------------------*/

void Line( int x1, int y1, int x2, int y2, BYTE farbe )
{
 int d, dx, dy,
     aincr, bincr,
     xincr, yincr,
     x, y;

 if ( abs(x2-x1) < abs(y2-y1) )       /* ber X- oder Y-Achse laufen? */
  {                                               /* ber die Y-Achse */
   if ( y1 > y2 )                                    /* y1 grer y2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   xincr = ( x2 > x1 ) ?  1 : -1;       /* Spalteninkrement festlegen */

   dy = y2 - y1;
   dx = abs( x2-x1 );
   d  = 2 * dx - dy;
   aincr = 2 * (dx - dy);
   bincr = 2 * dx;
   x = x1;
   y = y1;

   setpix( x, y, farbe );                      /* ersten Punkt setzen */
   for (y=y1+1; y<= y2; ++y )         /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       x += xincr;
       d += aincr;
      }
     else
      d += bincr;
     setpix(x, y, farbe);
    }
  }
 else                                             /* ber die X-Achse */
  {
   if ( x1 > x2 )                                    /* x1 grer x2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   yincr = ( y2 > y1 ) ? 1 : -1;         /* Zeileninkrement festlegen */

   dx = x2 - x1;
   dy = abs( y2-y1 );
   d  = 2 * dy - dx;
   aincr = 2 * (dy - dx);
   bincr = 2 * dy;
   x = x1;
   y = y1;

   setpix(x, y, farbe);                        /* ersten Punkt setzen */
   for (x=x1+1; x<=x2; ++x )          /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       y += yincr;
       d += aincr;
      }
     else
      d += bincr;
     setpix(x, y, farbe);
    }
  }
}

/***********************************************************************
*  GrafPrintf: Gibt einen formatierten String im Grafikbildschirm aus  *
*  Eingabe : X, Y   = Startkoordinate (0 - ...)                        *
*            VG     = Vordergrundfarbe                                 *
*            HG     = Hintergrundfarbe (255 = transparent)             *
*            STRING = String mit Formatinformationen                   *
*            ...    = Ausdrcke wie bei Printf                         *
***********************************************************************/

void GrafPrintf( int x, int y, BYTE vg, BYTE hg, char * string, ... )
{
 va_list parameter;              /* Parameter-Liste fr VA_... Macros */
 char ausgabe[255],                 /* Puffer fr formatierten String */
      *cp;

 va_start( parameter, string );                /* Parameter umwandeln */
 vsprintf( ausgabe, string, parameter );               /* formatieren */
 for ( cp = ausgabe; *cp; ++cp, x+= 8 )    /* formatierten String     */
   PrintChar( *cp, x, y, vg, hg );         /* ber PrintChar ausgeben */
}

/***********************************************************************
*  GetDac: Liefert den Inhalt einer bestimmten Anzahl von DAC-Registern*
**--------------------------------------------------------------------**
*  Eingabe : FIRST = Nummer des ersten DAC-Registers (0-255)           *
*            ANZ   = Anzahl der DAC-Register                           *
*            BUFP  = Zeiger auf den Puffer, in den die Inhalte der DAC-*
*                    Register geladen werden sollen. Dabei mu es sich *
*                    um eine Variable vom Typ DACREG oder ein Array    *
*                    dieses Typs handeln.                              *
*  Info    : Der bergebene Puffer mu fr jedes der zu lesenden DAC-  *
*            Register drei Bytes bereithalten, in denen der Rot-, Grn-*
*            und Blauanteil der jeweiligen Farbe festgehalten wird     *
***********************************************************************/

void GetDac( int First, int Anz, void far *BufP )
{
 union REGS Regs;           /* Prozessorregister fr Interrupt-Aufruf */
 struct SREGS SRegs;                               /* Segmentregister */

 Regs.x.ax = 0x1017;              /* Funktions- und Unterfunktionsnr. */
 Regs.x.bx = First;                /* Nummer des ersten DAC-Registers */
 Regs.x.cx = Anz;                    /* Anzahl der zu lad}en Register */
 Regs.x.dx = FP_OFF( BufP );
 SRegs.es  = FP_SEG( BufP );           /* Zeiger auf den Puffer laden */
 int86x( 0x10, &Regs, &Regs, &SRegs ); /*BIOS-Video-Interrupt aufrufen*/
}

/***********************************************************************
*  SetDac: Ldt eine bestimmte Anzahl von DAC-Registern                *
**--------------------------------------------------------------------**
*  Eingabe : FIRST = Nummer des ersten DAC-Registers (0-255)           *
*            ANZ   = Anzahl der DAC-Register                           *
*            BUFP  = Zeiger auf den Puffer, aus dem die verschiedenen  *
*                    DAC-Register geladen werden sollen. Dabei mu es  *
*                    sich um eine Variable vom Typ DACREG oder ein     *
*                    Array dieses Typs handeln.                        *
*  Info    : Siehe GetDac()                                            *
***********************************************************************/

void SetDac( int First, int Anz, void far *BufP )
{
 union REGS Regs;           /* Prozessorregister fr Interrupt-Aufruf */
 struct SREGS SRegs;                               /* Segmentregister */

 Regs.x.ax = 0x1012;              /* Funktions- und Unterfunktionsnr. */
 Regs.x.bx = First;                /* Nummer des ersten DAC-Registers */
 Regs.x.cx = Anz;                    /* Anzahl der zu lad}en Register */
 Regs.x.dx = FP_OFF( BufP );
 SRegs.es  = FP_SEG( BufP );           /* Zeiger auf den Puffer laden */
 int86x( 0x10, &Regs, &Regs, &SRegs ); /*BIOS-Video-Interrupt aufrufen*/
}

/***********************************************************************
*  PrintDac: Gibt den Inhalt eines DAC-Registers auf dem Bildschirm    *
*            aus und stellt die Farbe im DAC-Register 255 ein          *
**--------------------------------------------------------------------**
*  Eingabe : DREG  = Das DAC-Register                                  *
*            NR    = Die Nummer dieses Registers                       *
*            FARBE = Ausgabefarbe                                      *
***********************************************************************/

void PrintDac( DACREG DReg, BYTE Nr, BYTE Farbe )
{
 SetDac( 255, 1, &DReg );                /* Farbe in DAC-Register 255 */
 GrafPrintf( 60, MAXY-10, Farbe, 0, "DAC:%3d  R:%2d  G:%2d  B:%2d",
             Nr, DReg.b.Rot, DReg.b.Gruen, DReg.b.Blau);
}

/***********************************************************************
*  Umrahmen: Zieht einen Rahmen um eines der Farbfelder                *
**--------------------------------------------------------------------**
*  Eingabe : X     = X-Ordinate des Farbfeldes (0-15)                  *
*            Y     = Y-Ordinate des Farbfeldes (0-15)                  *
*            FARBE = Farbe des Rahmens                                 *
*  Info    : Die Linienstrke betrgt unabhngig des Abstands der      *
*            Farbfelder einen Punkt.                                   *
***********************************************************************/

void Umrahmen( int x, int y, BYTE Farbe )
{
 int sx, sy,                          /* obere linke Ecke des Rahmens */
     ex, ey;                        /* untere rechte Ecke des Rahmens */

 /*-- Eck-Ordinaten des Rahmens berechnen ----------------------------*/

 ex = ( sx = STARTX + x * (BREITE + ABSTAND) - 1 ) + BREITE + 1;
 ey = ( sy = STARTY + y * (HOEHE + ABSTAND) - 1 ) + HOEHE + 1;

 Line( sx, sy, ex, sy, Farbe );                      /* Rahmen ziehen */
 Line( ex, sy, ex, ey, Farbe );
 Line( ex, ey, sx, ey, Farbe );
 Line( sx, ey, sx, sy, Farbe );
}

/***********************************************************************
*  ChangeDacReg: Verndert den Inhalt eines DAC-Registers im Speicher  *
*                und in der DAC-Tabelle der Video-Karte und gibt ihn   *
*                anschlieend auf dem Bildschirm aus                   *
**--------------------------------------------------------------------**
*  Eingabe : DREGP = Zeiger auf zu vernderndes DAC-Register           *
*            NR    = Nummer des DAC-Registers                          *
*            KOMP  = Nummer der zu verndernden Komponente (0-2)       *
*                     0 = Rot, 1 == Gruen, 2 == Blau                   *
*            INKR  = Inkrement fr diese Komponente                    *
***********************************************************************/

void ChangeDacReg( DACREG *DRegP, BYTE Nr, BYTE Komp, BYTE Inkr )
{
 if ( ( DRegP->RGB[ Komp ] += Inkr) > 63 ) /*Komponente inkrementieren*/
  DRegP->RGB[ Komp ] = 0;               /* Grer 63: auf Null setzen */
 SetDac( Nr, 1, DRegP );                        /* DAC-Register laden */
 PrintDac( *DRegP, Nr, 15 );                 /* neuen Inhalt ausgeben */
}

/***********************************************************************
*  Demo: Demonstriert die Programmierung der DAC-Register und das      *
*        Farbmodell der VGA-Karte                                      *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************/

void Demo( void )
{
 int      x,  y,
          ix, jx,
          iy, jy,
          k,  f;                                   /* Schleifenzhler */
 char     ch;                                                /* Taste */
 DACARRAY dacbuf;                 /* Array mit kompletter DAC-Tabelle */
 DACREG   DReg;                  /* das jeweils aktuelle DAC-Register */

 /*-- Bildschirm aufbauen --------------------------------------------*/

 setpage( 0 );                                  /* Seite 0 bearbeiten */
 showpage( 0 );                                   /* Seite 0 anzeigen */
 GetDac( 0, 256, dacbuf );             /* komplette DAC-Tabelle laden */

 GrafPrintf( 10, 0, 255, 0,
             "VDACC  -  (c) 1991 by MICHAEL TISCHER" );

 /*-- den Block aus 16*16 Farbfeldern aufbauen -----------------------*/

 iy = STARTY;                    /* Ausgangspunkt auf dem Bildschirm  */
 jy = STARTY + HOEHE - 1;
 f  = 0;
 for ( y = 0; y < 16; ++y )         /* die 16 Blockzeilen durchlaufen */
  {
   ix = STARTX;
   jx = STARTX + BREITE - 1;
   for ( x = 0; x < 16; ++x )     /* die 16 Blockspaolten durchlaufen */
    {
     for ( k = iy; k <= jy; ++k ) /* Block aus einzelnen Linien formen*/
      Line( ix, k, jx, k, (BYTE) f );
     ix += BREITE + ABSTAND;                 /* nchster Block rechts */
     jx += BREITE + ABSTAND;
     ++f;                  /* Farbe fr nchsten Block inkrementieren */
     }
   iy += HOEHE + ABSTAND;                  /* Ausgabepos. fr nchste */
   jy += HOEHE + ABSTAND;                  /* Blockzeile              */
  }

 /*-- Eingaben des Anwenders abfragen und darauf reagieren -----------*/

 ix = 0;         /* in der oberen linken Ecke mit der Farbe 0 starten */
 iy = 0;
 jx = 0;
 jy = 0;
 k = 0;
 GetDac( 0, 1, &DReg );                              /* Farbe 0 holen */
 Umrahmen( 0, 0, 15 );                           /* Farbfeld umrahmen */
 PrintDac( DReg, 0, 15 );                      /* und Inhalt ausgeben */
 do
  {
   ch = (char) getch();                             /* Taste abfragen */
   if ( ch )                               /* keine erweiterte Taste? */
    switch ( ch )                                  /* Nein, auswerten */
     {
      case 'r' : ChangeDacReg( &DReg, (BYTE) k, 0, +1 ); /* r = Rot + */
                 break;
      case 'g' : ChangeDacReg( &DReg, (BYTE) k, 1, +1 ); /* g = Grn +*/
                 break;
      case 'b' : ChangeDacReg( &DReg, (BYTE) k, 2, +1 ); /* b = Blau +*/
                 break;
      case 'R' : ChangeDacReg( &DReg, (BYTE) k, 0, -1 ); /* R = Rot - */
                 break;
      case 'G' : ChangeDacReg( &DReg, (BYTE) k, 1, -1 ); /* G = Grn -*/
                 break;
      case 'B' : ChangeDacReg( &DReg, (BYTE) k, 2, -1 ); /* B = Blau -*/
                 break;
      case ' ' : {          /* Space = ursprnglichen Wert einstellen */
                  DReg = dacbuf[ k ];
                  ChangeDacReg( &DReg, (BYTE) k, 1, 0 );
                  break;
                 }
     }
   else                                   /* erweiterter Tastaturcode */
    switch ( getch() )
     {
      case 72 : if ( iy == 0 )                         /* Cursor oben */
                 jy = 15;
                else
                 jy = iy - 1;
                break;

      case 80 : if ( iy == 15 )                       /* Cursor unten */
                 jy = 0;
                else
                 jy = iy + 1;
                break;

      case 75 : if ( ix == 0  )                       /* Cursor links */
                 jx = 15;
                else
                 jx = ix - 1;
                break;

      case 77 : if ( ix == 15 )                      /* Cursor rechts */
                 jx = 0;
                else
                 jx = ix + 1;
     }

   if ( ix != jx  ||  iy != jy )             /* neue Cursor-Position? */
    {                                                           /* Ja */
     Umrahmen( ix, iy, 0 );        /* Rahmen an alter Pos. ausblenden */
     Umrahmen( jx, jy, 15 );            /* Rahmen um neue Pos. ziehen */
     ix = jx;                                /* neues Farbfeld merken */
     iy = jy;
     k = iy*16+ix;                /* Nummer des neuen Felds berechnen */
     GetDac( k, 1, &DReg );                     /* DAC-Register laden */
     PrintDac( DReg, (BYTE) k, 15 );            /* und ausgeben       */
    }
  }
 while ( ch != 13 );              /* wiederholen, bis RETURN bettigt */

 SetDac( 0, 256, dacbuf );         /* DAC-Tabelle wieder restaurieren */
}

/*--------------------------------------------------------------------*/
/*-- H A U P T P R O G R A M M                                      --*/
/*--------------------------------------------------------------------*/

void main( void )
{
 union REGS regs;

 if ( IsVga() )                             /* VGA-Karte installiert? */
  {                                                  /* Ja, los gehts */
   init320400();                        /* Grafikmodus initialisieren */
   Demo();
   regs.x.ax = 0x0003;          /* wieder in Textmodus zurckschalten */
   int86( 0x10, &regs, &regs );
  }
 else
  printf( "VDACC  -  (c) 1991 by MICHAEL TISCHER\n\nAchtung! Fr "\
          "die Ausfhrung dieses Programms wird eine VGA-Karte "\
          "bentigt.\n\n" );
}
