/**********************************************************************/
/*                            V I D E O C                             */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : stellt einige Funktionen zur Verfgung, die    */
/*                     von der Library des MICROSOFT und des TURBO    */
/*                     C-Compilers noch nicht zur Verfgung gestellt  */
/*                     werden                                         */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 13.08.1987                                     */
/*    letztes Update : 13.01.1991                                     */
/*--------------------------------------------------------------------*/
/*    (MICROSOFT C)                                                   */
/*    Erstellung     : CL /AS VIDEOC.C                                */
/*    Aufruf         : VIDEOC                                         */
/*--------------------------------------------------------------------*/
/*    (BORLAND TURBO C)                                               */
/*    Erstellung     : ber den Befehl RUN in der Befehlszeile        */
/*                     (ohne Project-Datei)                           */
/**********************************************************************/

/*== Include-Dateien einbinden =======================================*/

#include <dos.h>                          /* Header-Dateien einbinden */
#include <io.h>
#include <stdio.h>

/*== Konstanten ======================================================*/

#define NORMAL        0x07        /* Definition der Zeichen-Attribute */
#define HELL          0x0F        /* in bezug auf eine monochrome     */
#define INVERS        0x70        /* Bildschirmkarte                  */
#define UNTERSTRICHEN 0x01
#define BLINKEN       0x80

/**********************************************************************/
/* GETVIDEOMODE: aktuellen Videmodus sowie Parameter auslesen         */
/* Eingabe : VIDEOMODUS = Zeiger auf Variable, in der der aktuelle    */
/*                        Video-Modus zurckgeliefert wird            */
/*           ANZAHL     = Zeiger auf Var. fr Anzahl der Textspalten  */
/*           SEITE      = Zeiger auf Var. fr aktuelle Bildschirm-    */
/*                        seite                                       */
/* Ausgabe : keine                                                    */
/**********************************************************************/

void GetVideoMode( int *VideoModus, int *Anzahl, int *Seite)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */

 Register.h.ah = 15;                               /* Funktionsnummer */
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
 *VideoModus = Register.h.al;                /* Nummer des Videomodus */
 *Anzahl = Register.h.ah;                 /* Anzahl Zeichen pro Zeile */
 *Seite = Register.h.bh;              /* Nummer der aktuellen B.Seite */
}

/**********************************************************************/
/* SETCURSORTYPE: definiert das Erscheinungsbild des blinkenen Bild-  */
/*                schirmcursors                                       */
/* Eingabe : ANFANG = Punktzeile fr Cursorstart                      */
/* Eingabe : ENDE   = Punktzeile fr Cursorende                       */
/* Ausgabe : keine                                                    */
/* Info    : bei einer monochromen Bildschirmkarte knnen die Para-   */
/*           meter im Bereich zwischen 0 und 13, bei einer Color-     */
/*           Bildschirmkarte zwischen 0 und 7 liegen                  */
/**********************************************************************/

void SetCursorType(int Anfang, int Ende)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */

 Register.h.ah = 1;                                /* Funktionsnummer */
 Register.h.ch = Anfang;                   /* Anfangszeile des Cursor */
 Register.h.cl = Ende;                         /* Endzeile des Cursor */
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
}
                                                                        
/**********************************************************************/
/* SETCURSORPOS: definiert die Position des Cursors in der angegebenen*/
/*               Bildschirmseite                                      */
/* Eingabe : SEITE   = Bildschirmseite                                */
/*           SPALTE  = Textspalte                                     */
/*           ZEILE   = Textzeile                                      */
/* Ausgabe : keine                                                    */
/* Info    : die Position des blinkenden Bildschirmcursors ndert     */
/*           sich durch den Aufruf dieser Funktion nur dann, wenn die */
/*           angegebene Bildschirmseite die aktuelle Bildschirmseite  */
/*           ist                                                      */
/**********************************************************************/

void SetCursorPos( int Seite, int Spalte, int Zeile)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */

 Register.h.ah = 2;                                /* Funktionsnummer */
 Register.h.bh = Seite;                            /* Bildschirmseite */
 Register.h.dh = Zeile;                            /* Bildschirmzeile */
 Register.h.dl = Spalte;                          /* Bildschirmspalte */
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
}

/**********************************************************************/
/* GETCURSORPOS: holt die Position des Cursor in einer bestimmten     */
/*               Bildschirmseite sowie seine Start- und Endzeile      */
/* Eingabe : SEITE   = Bildschirmseite                                */
/*           SPALTE  = Zeiger auf Var. fr akt. Textspalte            */
/*           ZEILE   = Zeiger auf Var. fr akt. Textzeile             */
/*           ANFANG  = Zeiger auf Var. fr erste Punktzeile des C.    */
/*           ENDE    = Zeiger auf Var. fr letzte Punktzeile des C.   */
/* Ausgabe : keine                                                    */
/**********************************************************************/

void GetCursorPos( int Seite, int *Spalte,
                   int *Zeile, int *Anfang, int *Ende)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */

 Register.h.ah = 3;                                /* Funktionsnummer */
 Register.h.bh = Seite;                            /* Bildschirmseite */
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
 *Spalte = Register.h.dl;                  /* Ergebnisse der Funktion */
 *Zeile  = Register.h.dh;                  /* aus den Registern lesen */
 *Anfang = Register.h.ch;                  /* und den entsprechenden  */
 *Ende   = Register.h.cl;                  /* Variablen zuweisen      */
}

/**********************************************************************/
/* SETDISPLAYPAGE: setzt die auf dem Bildschirm darzustellende Bild-  */
/*                 schirmseite                                        */
/* Eingabe : SEITE = Bildschirmseite                                  */
/* Ausgabe : keine                                                    */
/**********************************************************************/

void SetDisplayPage( int Seite )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */

 Register.h.ah = 5;                                /* Funktionsnummer */
 Register.h.al = Seite;                            /* Bildschirmseite */
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
}

/**********************************************************************/
/* SCROLLUP: schiebt einen Bildschirmbereich um eine oder mehrere     */
/*           Zeilen nach oben oder lscht ihn                         */
/* Eingabe:  ANZAHL   = Anzahl der Zeilen um die gescrollt werden soll*/
/*           FARBE    = Farbe bzw. Attribut fr die Leerzeilen        */
/*           SPALTEOL = Spalte in der oberen linken Ecke des Bereichs */
/*           ZEILEOL  = Zeile in der oberen linken Ecke des Bereichs  */
/*           SPALTEUR = Spalte in der unteren rechten Ecke des Ber.   */
/*           ZEILEUR  = Zeile in der unteren rechten Ecke des Berichs */
/* Ausgabe : keine                                                    */
/* Info    : wird als Anzahl 0 bergeben, so wird der Bildschirm-     */
/*           bereich mit Leerzeichen gefllt                          */
/**********************************************************************/

void ScrollUp( int Anzahl, int Farbe, int SpalteOL,
               int ZeileOL, int SpalteUR, int ZeileUR )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */

 Register.h.ah = 6;                                /* Funktionsnummer */
 Register.h.al = Anzahl;                             /* Anzahl Zeilen */
 Register.h.bh = Farbe;                     /* Farbe der Leerzeile(n) */
 Register.h.ch = ZeileOL;               /* Koordinaten des zu scroll- */
 Register.h.cl = SpalteOL;              /* enden oder zu lschenden   */
 Register.h.dh = ZeileUR;               /* Bildschirmfensters setzen  */
 Register.h.dl = SpalteUR;
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
}

/**********************************************************************/
/* SCROLLDOWN: verschiebt einen Bildschirmbereich um eine oder meh-   */
/*             rere Zeilen nach unten oder lscht ihn                 */
/* Eingabe:  ANZAHL   = Anzahl der Zeilen um die gescrollt werden soll*/
/*           FARBE    = Farbe bzw. Attribut fr die Leerzeilen        */
/*           SPALTEOL = Spalte in der oberen linken Ecke des Bereichs */
/*           ZEILEOL  = Zeile in der oberen linken Ecke des Bereichs  */
/*           SPALTEUR = Spalte in der unteren rechten Ecke des Ber.   */
/*           ZEILEUR  = Zeile in der unteren rechten Ecke des Berichs */
/* Ausgabe : keine                                                    */
/* Info    : wird als Anzahl 0 bergeben, so wird der Bildschirm-     */
/*           bereich mit Leerzeichen gefllt                          */
/**********************************************************************/

void ScrollDown( int Anzahl, int Farbe, int SpalteOL,
                 int ZeileOL, int SpalteUR, int ZeileUR )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */

 Register.h.ah = 7;                                /* Funktionsnummer */
 Register.h.al = Anzahl;                             /* Anzahl Zeilen */
 Register.h.bh = Farbe;                     /* Farbe der Leerzeile(n) */
 Register.h.ch = ZeileOL;               /* Koordinaten des zu scroll- */
 Register.h.cl = SpalteOL;              /* enden oder zu lschenden   */
 Register.h.dh = ZeileUR;               /* Bildschirmfensters setzen  */
 Register.h.dl = SpalteUR;
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
}

/**********************************************************************/
/* GETCHAR: liest aus einer bestimmten Bildschirmposition das dort    */
/*          dargestellte Zeichen und sein Attribut-Byte aus           */
/* Eingabe:  SEITE  = Bildschirmseite aus der das Zeichen gelesen wird*/
/*           SPALTE = Bildschirmspalte des Zeichens                   */
/*           ZEILE  = Bildschirmzeile des Zeichens                    */
/*           ZEICHEN= Zeiger auf Var. fr das Zeichen                 */
/*           FARBE  = Zeiger auf Var. fr Attribut-Byte (Farbe)       */
/* Ausgabe : keine                                                    */
/**********************************************************************/

void GetChar( int Seite, int Spalte, int Zeile,
              char *Zeichen, int *Farbe)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
 int Dummy;               /* fr Variablen, die nicht bentigt werden */
 int AktSeite;                        /* die aktuelle Bildschirmseite */
 int AktZeile;                        /* die aktuelle Bildschirmzeile */
 int AktSpalte;                      /* die aktuelle Bildschirmspalte */

 GetVideoMode(&Dummy, &Dummy, &AktSeite);  /* aktuelle Bs.seite holen */
 GetCursorPos( AktSeite, &AktSpalte, &AktZeile,   /* aktuelle Cursor- */
              &Dummy, &Dummy);                    /* position holen   */
 SetCursorPos(Seite, Spalte, Zeile);                 /* Cursor setzen */
 Register.h.ah = 8;                                /* Funktionsnummer */
 Register.h.bh = Seite;                            /* Bildschirmseite */
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
 *Zeichen = Register.h.al;            /* Ergebnisse aus den Registern */
 *Farbe = Register.h.ah;              /* auslesen und zuweisen        */
 SetCursorPos(AktSeite, AktSpalte, AktZeile);  /* Cursor an alte Pos. */
}

/**********************************************************************/
/* WRITECHAR: schreibt ein Zeichen mit einem vorgegebenen Attribut an */
/*            die aktuelle Cursorposition in der angegebenen Seite    */
/* Eingabe :  SEITE  = in dieser Bildschirmseite erscheint das Zeichen*/
/*            ZEICHEN= das auszugebende Zeichen                       */
/*            FARBE  = sein Attribut bzw. Farbe                       */
/* Ausgabe : keine                                                    */
/**********************************************************************/

void WriteChar( int Seite, char Zeichen, int Farbe)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */

 Register.h.ah = 9;                                /* Funktionsnummer */
 Register.h.al = Zeichen;                 /* das auszugebende Zeichen */
 Register.h.bh = Seite;                            /* Bildschirmseite */
 Register.h.bl = Farbe;            /* Farbe des auszugebenden Zeichen */
 Register.x.cx = 1;                /* das Zeichen nur einmal ausgeben */
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
}

/**********************************************************************/
/* WRITETEXT: schreibt eine Zeichenkette mit konstanter Farbe ab      */
/*            einer vorgegebenen Position innerhalb einer Bs.seite    */
/* Eingabe :  SEITE  = in dieser Bildschirmseite erscheint das Zeichen*/
/*            SPALTE = Bildschirmspalte des Zeichens                  */
/*            ZEILE  = Bildschirmzeile des Zeichens                   */
/*            FARBE  = sein Attribut bzw. Farbe                       */
/*            TEXT   = Zeiger auf den auszugebenden String            */
/* Ausgabe : keine                                                    */
/* Info    : Text ist ein Pointer auf einen Zeichen-Vektor, der den   */
/*           den auszugebenden Text enthlt und mit einem '\0'        */
/*           Zeichen abgeschlossen wird                               */
/**********************************************************************/

void WriteText( int Seite, int Spalte, int Zeile, int Farbe, char *Text)
{
 union REGS InRegister,     /* Register-Variable zum Interrupt-Aufruf */
            OutRegister;

 SetCursorPos(Seite, Spalte, Zeile);                 /* Cursor setzen */

 InRegister.h.ah = 14;                             /* Funktionsnummer */
 InRegister.h.bh = Seite;                          /* Bildschirmseite */
 while (*Text)                      /* Text bis '\0' Zeichen ausgeben */
  {
   WriteChar(Seite, ' ', Farbe);        /* Farbe fr Zeichen vorgeben */
   InRegister.h.al = *Text++;             /* das auszugebende Zeichen */
   int86(0x10, &InRegister, &OutRegister);      /* Interrupt aufrufen */
  }
}

/**********************************************************************/
/* CLEARSCREEN: lscht den 80*25 Zeichen Textbildschirm und setzt den */
/*              Cursor in die obere linke Bildschirmecke              */
/* Eingabe : keine                                                    */
/* Ausgabe : keine                                                    */
/**********************************************************************/

void ClearScreen( void )
{
 int AktSeite;                            /* aktuelle Bildschirmseite */
 int Dummy;                                         /* Dummy-Variable */

 ScrollUp(0, NORMAL, 0, 0, 79, 24);      /* ganzen Bildschirm lschen */
 GetVideoMode(&Dummy, &Dummy, &AktSeite);       /* akt. B.Seite holen */
 SetCursorPos(AktSeite, 0, 0);                       /* Cursor setzen */
}

/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/

void main()

{
 int i, j, k, l;                                /* Schleifenvariablen */
 char Pfeile[3];      /* nimmt Anzahl der Pfeile als ASCII-String auf */

 ClearScreen();                                 /* Bildschirm lschen */
 for (i = 1; i < 25; i++)                  /* alle Zeilen durchlaufen */
  for (j = 0; j < 80; j++)                /* alle Spalten durchlaufen */
   {
    SetCursorPos(0, j, i);                    /* Cursor positionieren */
    WriteChar(0, i*80+j&255, NORMAL);            /* Zeichen schreiben */
   }
 ScrollDown(0, NORMAL, 5, 8, 19, 22);            /* Fenster 1 lschen */
 WriteText(0, 5, 8, INVERS, "   Fenster 1   ");
 ScrollDown(0, NORMAL, 60, 2, 74, 16);           /* Fenster 2 lschen */
 WriteText(0, 60, 2, INVERS, "   Fenster 2   ");
 WriteText(0, 30, 12, INVERS | BLINKEN, " >>> PC INTERN <<< ");
 WriteText(0, 0, 0, INVERS, "                      es werden noch    ");
 WriteText(0, 40, 0, INVERS,"Pfeile gezeichnet                       ");
 for (i = 49; i >= 0 ; i--)                     /* 50 Pfeile zeichnen */
  {
   sprintf(Pfeile, "%2d", i);     /* Anzahl Pfeile nach ASCII wandeln */
   WriteText(0, 37, 0, INVERS, Pfeile);               /* und ausgeben */
   for (j = 1; j < 16; j+= 2)    /* jeder Pfeil besteht aus 16 Zeilen */
    {
     for (k = 0; k < j; k++)       /* eine Zeile des Pfeils erstellen */
      {
       SetCursorPos(0, 12-(j>>1)+k, 9);            /* Pfeil Fenster 1 */
       WriteChar(0, '*', HELL);
       SetCursorPos(0, 67-(j>>1)+k, 16);           /* Pfeil Fenster 2 */
       WriteChar(0, '*', HELL);
      }
     ScrollDown(1, NORMAL, 5, 9, 19, 22); /* Fenster 1 runterscrollen */
     ScrollUp(1, NORMAL, 60, 3, 74, 16);    /* Fenster 2 raufscrollen */
     for (l = 0; l < 4000 ; l++)                     /* Warteschleife */
      ;
    }
   }
 ClearScreen();                                 /* Bildschirm lschen */
}
