/**********************************************************************/
/*                       W I N . C                                    */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Text-Modus Fenster verwalten                     */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 03.05.1994                                       */
/*  letztes Update : 15.05.1994                                       */
/* COMPILER: Borland C++ 3.1                                          */
/**********************************************************************/
#ifndef __WIN_C                  /* WIN.C kann auch #Included werden! */
#define __WIN_C

#include <dos.h>
#include <stdarg.h>
#include <stdio.h>
#include <conio.h>
#include <malloc.h>

#include "win.h"
#include "kbd.h"

/*- Globale Variablen ------------------------------------------------*/
static LPBYTE _lpVIOSMEM = NULL;   /* Adresse des Bildschirmspeichers */
static WORD   _wVIOSMODE = 0;                 /* Aktueller Videomodus */
static INT    _Columns   = 0;      /* Anzahl der dargestellten Zeilen */

/**********************************************************************/
/* win_Init : Initialisiert WINDOW-Strutkur                           */
/**------------------------------------------------------------------**/
/* Eingabe : pWin   - Adresse der zu intialisierenden WINDOW-Struktur */
/*           iX, iY - Position des Fensters relativ zu bergeordnetem */
/*                    Fenster bzw. Bildschirm. Ursprung : (0,0)       */
/*           iW, iH   - Breite und Hhe des Fensters                  */
/*           iLA, iHA - Textattribute (passiv, aktiv)                 */
/*           uFlags - Flags                                           */
/**********************************************************************/
VOID _FP win_Init( PWINDOW pWin,
                   INT     iX,
                   INT     iY,
                   INT     iW,
                   INT     iH,
                   BYTE    iLA,
                   BYTE    iHA,
                   UINT    uFlags )
{
  pWin->iX      = iX;                             /* Parameter setzen */
  pWin->iY      = iY;
  pWin->iW      = iW;
  pWin->iH      = iH;
  pWin->iLoAttr = iLA;
  pWin->iHiAttr = iHA; 
  
  pWin->uFlags = uFlags;

  pWin->iCX = 0;            /* Cursorposition = Ursprung des Fensters */
  pWin->iCY = 0;
  win_LoVideo( pWin );
}

/**********************************************************************/
/* win_LoVideo : Ausgabe auf niedrige Intensitt umschalten           */
/**------------------------------------------------------------------**/
/* Eingabe : pWin - Adresse des Fenster, in dem folgende Ausgaben mit */
/*           niedriger Intensitt stattfinden sollen.                 */
/**********************************************************************/
VOID _FP win_LoVideo( PWINDOW pWin )
{
  pWin ->iAttr = pWin->iLoAttr;
}

/**********************************************************************/
/* win_HiVideo : Ausgabe auf hohe Intensitt umschalten               */
/**------------------------------------------------------------------**/
/* Eingabe : pWin - Adresse des Fenster, in dem folgende Ausgaben mit */
/*           hoher Intensitt stattfinden sollen.                     */
/**********************************************************************/
VOID _FP win_HiVideo( PWINDOW pWin )
{
  pWin ->iAttr = pWin->iHiAttr;
}

/**********************************************************************/
/* win_GetVIOSMEM : Ermittelt Startadresse des Bildschrimspeichers    */
/*                  und aktuellen Videomodus.                         */
/**------------------------------------------------------------------**/
/* Info : Die Adresse des Bildschirmspeichers wird in der globalen    */
/*        Variablen '_lpVIOSMEM' gespeichert. Der Videomodus steht in */
/*        '_iVIOSMODE'.                                               */
/**********************************************************************/
VOID _FP win_GetVIOSMEM( VOID )
{  static union _REGS regs;

   if( _lpVIOSMEM ) return;              /* Adresse schon ermittelt ? */

   regs.h.ah = VIOSETPAGE;
   regs.h.al = VIOPAGE;
   _int86( VIOINT, &regs, &regs );

   regs.h.ah = VIOGETMODE;
   regs.h.al = VIOPAGE;
   _int86( VIOINT, &regs, &regs );
   _Columns = regs.h.ah;                        /* Anzahl der Spalten */
   _wVIOSMODE = regs.h.al;
   if( _wVIOSMODE == 0x07 ) _lpVIOSMEM = MK_FP( 0xB000, 0 );
   else                     _lpVIOSMEM = MK_FP( 0xB800, 0 );
}

/**********************************************************************/
/* win_GetMode : Ermittelt aktuellen Bildschirmmodus.                 */
/**------------------------------------------------------------------**/
/* Ausgabe : VM_COLOR (1) - Farbbildschirm                            */
/*           VM_MONO  (2) - Monochrombildschirm                       */
/*           sonst.       - unbekannt!                                */
/**********************************************************************/
WORD _FP win_GetMode( VOID )
{
  win_GetVIOSMEM();
  return _wVIOSMODE;
}

/**********************************************************************/
/* win_GotoXY : Setze Ausgabeposition innerhalb eines Fensters        */
/**------------------------------------------------------------------**/
/* Eingabe : pWin  - Adresse des Fensters, indem neue Ausgabe-        */
/*                    position gesetzt werden soll.                   */
/*           iX, iY - Neue Position                                   */
/**------------------------------------------------------------------**/
/* Info : Ist das Flags "WIN_HASCURSOR" gesetzt, wird durch diese     */
/*        Funktion auch der blinkende Bildschirmcursor gesetzt.       */
/**********************************************************************/
VOID _FP win_GotoXY( PWINDOW pWin,
                     INT     iX,
                     INT     iY )
{ static union _REGS regs;

  if( iX < 0 ) iX = 0;        /* Gltigkeit der neuen Position prfen */
  if( iY < 0 ) iY = 0;
  if( iX > pWin->iW ) iX = pWin->iW - 1;
  if( iY > pWin->iH ) iY = pWin->iH - 1;

  pWin->iCX = iX;
  pWin->iCY = iY;

  if( pWin->uFlags & WIN_ACTIVE )                  /* Fenster aktiv ? */
  {
    if( pWin->uFlags & WIN_HASCURSOR ) /* Besitzt Fenster den Cursor? */
    {
      regs.h.dh = ( BYTE ) ( pWin->iY + pWin->iCY );
      regs.h.dl = ( BYTE ) ( pWin->iX + pWin->iCX );
    }
    else
    {
      regs.h.dh = 0xFF;                                      /* Nein! */
      regs.h.dl = 0xFF;
    }

    regs.h.ah = VIOSETCURSOR;
    regs.h.bh = VIOPAGE;
    _int86( VIOINT, &regs, &regs );
  }
}

/**********************************************************************/
/* win_Clr : Fensterinhalt lschen                                    */
/**------------------------------------------------------------------**/
/* Eingabe : pWin - Adresse des zu lschenden Fensters                */
/**********************************************************************/
VOID _FP win_Clr( PWINDOW pWin )
{ static INT iX, iY;
  static LPWORD lpRow;

  win_GetVIOSMEM();
  for( iY = 0; iY < pWin->iH; iY++ )
  { static WORD iAdr;
    iAdr = (pWin->iX+(iY+pWin->iY)*_Columns)*2;
    lpRow = ( LPWORD )&_lpVIOSMEM[ iAdr ];
    iAdr = MAKEWORD( pWin->iAttr, ' ' );
    for( iX = 0; iX < pWin->iW; iX++ )  *( lpRow++ ) = iAdr;
  }
  pWin->iCX = 0;
  pWin->iCY = 0;
}

/**********************************************************************/
/* win_Save : Fensterinhalt speichern                                 */
/**------------------------------------------------------------------**/
/* Eingabe : pWin - Adresse des zu speichernden Fensters              */
/* Ausgabe : Adresse des Speichers, der Fensterinformationen samt     */
/*           Inhalt aufgenommen hat, oder NULL falls kein freier      */
/*           Speicher mehr verfgbar.                                 */
/**********************************************************************/
PBYTE _FP win_Save( PWINDOW pWin )
{ PBYTE   pMem;
  LPWORD  lpRows;
  PWINDOW pWinData;

  win_GetVIOSMEM();
  pMem = malloc( ( pWin->iW * pWin->iH * 2 ) + sizeof( WINDOW ) );
  if( pMem )
  { static INT iY, iX;
    pWinData = ( PWINDOW )pMem;
    *pWinData = *pWin;
    lpRows = ( PWORD ) &pWinData[ 1 ];
    for( iY = 0; iY < pWin->iH; iY++ )
    { static WORD   iAdr;
      static LPWORD lpR;
      iAdr = (pWin->iX+(iY+pWin->iY)*_Columns)*2;
      lpR = ( LPWORD )&_lpVIOSMEM[ iAdr ];
      for( iX = 0; iX < pWin->iW; iX++ )  *(lpRows++) = *(lpR++);
    }
  }
  return pMem;
}

/**********************************************************************/
/* win_Restore : Fensterinhalt wiederherstellen                       */
/**------------------------------------------------------------------**/
/* Eingabe : pMem - Adresse des vorher durch win_Save ermittelten     */
/*                   Speichers, der sowohl Fenster-Informationen      */
/*                   (Position, Dimension, Cursor), als auch Inhalt   */
/*                   enthlt.                                         */
/*           bFree - Soll Speicher nach Wiederherstellung frei-       */
/*                   gegeben werden? (bFree != 0 )                    */
/**********************************************************************/
VOID _FP win_Restore( PBYTE pMem,
                      BYTE  bFree )
{ LPWORD  lpRows;
  PWINDOW pWinData;

  win_GetVIOSMEM();
  if( pMem )
  { static INT iX, iY;
    pWinData = ( PWINDOW )pMem;
    lpRows = ( PWORD ) &pWinData[ 1 ];
    for( iY = 0; iY < pWinData->iH; iY++ )
    { static WORD   iAdr;
      static LPWORD lpR;
      iAdr = (pWinData->iX+(iY+pWinData->iY)*_Columns)*2;
      lpR = ( LPWORD ) &_lpVIOSMEM[ iAdr ];
      for( iX = 0; iX < pWinData->iW; iX++ )  *(lpR++) = *(lpRows++);
    }
    win_GotoXY( pWinData, pWinData->iCX, pWinData->iCY );

    if( bFree ) free( pMem );
  }
}

/**********************************************************************/
/* win_GetActScreenSettings : Aktuelle Bildschirmeinstellungen er-    */
/*                            mitteln                                 */
/**------------------------------------------------------------------**/
/* Eingabe : pWin - Adresse der WINDOW-Struktur, die Bildschirm-      */
/*                  einstellungen aufnehmen soll                      */
/**------------------------------------------------------------------**/
/* Info : Aktueller Videomodus und aktuelle Videoseite werden nicht   */
/*        in WINDOW-Struktur gespeichert.                             */
/**********************************************************************/
VOID _FP win_GetScreenSettings( PWINDOW pWin )
{ static union _REGS regs;

  win_GetVIOSMEM();

  win_Init( pWin, 0, 0, _Columns, 25,
      0x00, 0x00, WIN_HASCURSOR | WIN_ACTIVE );

  regs.h.ah = VIOGETCURSOR;
  regs.h.bh = VIOPAGE;
  _int86( VIOINT, &regs, &regs );

  pWin->iCX = regs.h.dl;
  pWin->iCY = regs.h.dh;
}

/**********************************************************************/
/* win_ScrollUp : Fensterinhalt nach oben scrollen                    */
/**------------------------------------------------------------------**/
/* Eingabe : pWin    - Addresse des Fensters                          */
/*           NumCols - Anzahl der nach oben zu scrollenden Zeilen     */
/**------------------------------------------------------------------**/
/* Info : Der durch das Scrollen frei werdende Bereich wird durch     */
/*        Leerzeichen im aktuellen Textattribut besetzt.              */
/**********************************************************************/
VOID _FP win_ScrollUp( PWINDOW pWin,
                       INT     NumCols )
{ static INT iX, iY;
  static LPWORD lpRow, lpRowDest;
  static WINDOW Clear;

  win_GetVIOSMEM();

  for( iY = NumCols; iY < pWin->iH; iY++ )
  { static WORD iDAdr, iAdr;
    iDAdr = (pWin->iX+((iY-NumCols)+pWin->iY)*_Columns)*2;
    iAdr = (pWin->iX+(iY+pWin->iY)*_Columns)*2 ;
    lpRowDest = ( LPWORD ) &_lpVIOSMEM[ iDAdr ];
    lpRow     = ( LPWORD ) &_lpVIOSMEM[ iAdr];
    for( iX = 0; iX < pWin->iW; iX++ ) *( lpRowDest++ ) = *( lpRow++ );
  }
  win_Init( &Clear, pWin->iX, pWin->iY + ( pWin->iH - NumCols ),
            pWin->iW, NumCols, pWin->iLoAttr, pWin->iHiAttr,
            WIN_CRLF | WIN_SCROLL );
  win_Clr( &Clear );
}

/**********************************************************************/
/* _win_PrINT : Textausgabe-Engine                                    */
/**------------------------------------------------------------------**/
/* Eingabe : pWin  - Adresse des Fensters, in das Text ausgegeben     */
/*                   werden soll.                                     */
/*           pText - Adresse des auszugebenden Textes                 */
/*           iCnt   - ANzahl der auszugebenden Zeichen.               */
/*                    (-1 = ganzer Text)                              */
/**********************************************************************/
VOID _FP _win_Print( PWINDOW pWin,
                     PCHAR   pText,
                     INT     iCnt )
{ INT i;
  static LPBYTE lpRow;
  static WORD   iAdr;
  i = 0;

  win_GetVIOSMEM();

  iAdr = (pWin->iX+pWin->iCX+(pWin->iCY+pWin->iY)*_Columns)*2;
  lpRow = &_lpVIOSMEM[ iAdr ];

  while( ( pText[ i ] ) && iCnt )
  { static CHAR c;
    c = pText[ i ];
    switch( c )
    { default:                           /* einzelne Zeichen ausgeben */
        *( lpRow++ ) = c;
        *( lpRow++ ) = pWin->iAttr;

        pWin->iCX++;
        if( pWin->iCX < pWin->iW )  break;     /* Zeilende erreicht ? */
      case '\r':                                    /* CR = CR + LF ? */
        if( !(pWin->uFlags & WIN_CRLF) && ( pText[ i ]== '\r') )
        {
          pWin->iCX = 0;         /* An den Anfang der aktuellen Zeile */
          iAdr = (pWin->iX+pWin->iCX+(pWin->iCY+pWin->iY)
          *_Columns )*2;
          lpRow = &_lpVIOSMEM[ iAdr ];
          break;
        }
      case '\n':                                                /* LF */
        while( pWin->iCX < pWin->iW )       /* Rest der Zeile lschen */
        {
          *( lpRow++ ) = ' ';
          *( lpRow++ ) = pWin->iAttr;
          pWin->iCX++;
        }
        pWin->iCY++;                                 /* Nchste Zeile */
        if( pWin->iCY >= pWin->iH )         /* Letzte Zeile erreicht? */
        {
          pWin->iCY = pWin->iH - 1;
          if( pWin->uFlags & WIN_SCROLL )          /* Inhalt scrollen */
          win_ScrollUp( pWin, 1 );
        }
        pWin->iCX = 0;
        iAdr = (pWin->iX+pWin->iCX+(pWin->iCY+pWin->iY)
               *_Columns )*2;
        lpRow = &_lpVIOSMEM[ iAdr ];
        break;
    }
    i++;
    if( iCnt > 0 ) iCnt--;    /* Zhler dekrementieren, falls positiv */
  }
  win_GotoXY( pWin, pWin->iCX, pWin->iCY );
}

/**********************************************************************/
/* win_Print : C-String in Fenster ausgeben                           */
/**------------------------------------------------------------------**/
/* Eingabe : pWin  - Fenster, in das String ausgegeben werden soll    */
/*           pText - auszugebender String.                            */
/**-------------------------------------------------------------------*/
/* Info : Diese Funktion verwendet _win_Print zur Ausgabe.            */
/**********************************************************************/
VOID _FP win_Print( PWINDOW pWin,
                    PCHAR   pText )
{
  _win_Print( pWin, pText, -1 );
}

/**********************************************************************/
/* win_PrintAT : Text an beliebiger Stelle im Fenster ausgeben        */
/**-------------------------------------------------------------------*/
/* Eingabe : pWin   - Fenster, in das String ausgegeben werden soll   */
/*           iX, iY - Ausgabeposition                                 */
/*           pText  - auszugebender String.                           */
/**********************************************************************/
VOID _FP win_PrintAt( PWINDOW pWin,
                      INT     iX,
                      INT     iY,
                      PCHAR   pText )
{
  win_GotoXY( pWin, iX, iY );
  _win_Print( pWin, pText, -1 );
}

/**********************************************************************/
/* _wprintf : Textausgabe wie printf in Fenster oder auf Bildschirm   */
/**-------------------------------------------------------------------*/
/* Eingabe : pWin     - Adresse des Fensters, in das die Ausgabe er-  */
/*                      folgen soll (optional)                        */
/*           pFormat  - Adresse des Formatstrings                     */
/*           ...      - Optionale Parameter                           */
/**------------------------------------------------------------------**/
/* Info : Diese Funktion verwendet vprintf bzw. vsprintf zur Ausgabe  */
/*        des Formatstrings auf den Bildschirm bzw. in einen Buffer,  */
/*        der anschliessend in ein Fenster ausgegeben wird.           */
/**********************************************************************/
VOID _FP win_printf( PWINDOW pWin,
                     PCHAR   pFormat, ... )
{ static va_list argptr;
  va_start( argptr, pFormat );
  if( pWin )
  { static CHAR Buffer[ 1024 ];                  /* max. 1024 Zeichen */
    vsprintf( Buffer, pFormat, argptr );
    win_Print( pWin, Buffer );
  }
  else vprintf( pFormat, argptr );
  va_end( argptr );
}

/**********************************************************************/
/* win_Beep : Warnton erzeugen                                        */
/**********************************************************************/
VOID _FP win_Beep( VOID )
{ WORD uFrq = 400;
  LONG l;
                   /* Doop! */
  outp( 0x43, 0xB6 );
  outp( 0x42, LOBYTE( 1193180L / uFrq ) );
  outp( 0x42, HIBYTE( 1193180L / uFrq ) );
  outp( 0x61, inp( 0x61 ) | 0x03 );
  for( l = 0; l < 10000L; l++ );
  outp( 0x61, inp( 0x61 ) & ~0x03 );
}

/**********************************************************************/
/* win_OBJECTInitINT - Object mit INT-Typinformationen verbinden      */
/**------------------------------------------------------------------**/
/* Eingabe: pObject    - Typfreies Object                             */
/*          pData      - Typinformationen (PINTDATA)                  */
/*          x, y, w, h - Position, Dimension des Objects              */
/*          pText      - Auszugebender Text                           */
/*          iMin, iMax - Grenzwerte des Integers                      */
/*          pValue     - Adresse der Variablen, die INT aufnimmt      */
/**********************************************************************/
VOID _FP win_OBJECTInitINT( POBJECT  pObject,
                            PINTDATA pData,
                            INT      x,
                            INT      y,
                            INT      w,
                            INT      h,
                            PCHAR    pText,
                            INT      iMin,
                            INT      iMax,
                            PINT     pValue )
{
  pObject->X = x;
  pObject->Y = y;                        
  pObject->W = w; /* werden (noch) nicht verwendet */
  pObject->H = h;
  pObject->iType = OT_INT;
  pObject->pData = ( PVOID ) pData;  
  pData->iMin = iMin;
  pData->iMax = iMax;  
  pData->pText = pText;
  pData->pValue = pValue;
}

/**********************************************************************/
/* win_OBJECTInitBOOL - Object mit INT-Typinformationen verbinden     */
/**------------------------------------------------------------------**/
/* Eingabe: pObject    - Typfreies Object                             */
/*          pData      - Typinformationen (PBOOLDATA)                 */
/*          x, y, w, h - Position, Dimension des Objects              */
/*          pText      - Auszugebender Text                           */
/*          iDisplay   - Ausgabeart ( YES/NO, TRUE/FALSE, ON/OFF )    */
/*          pValue     - Adresse der Variablen, die INT aufnimmt      */
/**********************************************************************/
VOID _FP win_OBJECTInitBOOL( POBJECT   pObject,
                             PBOOLDATA pData,
                             INT       x,
                             INT       y,
                             INT       w,
                             INT       h,
                             PCHAR     pText,
                             INT       iDisplay,
                             PINT      pValue )
{
  pObject->X = x;
  pObject->Y = y;                        
  pObject->W = w; /* werden (noch) nicht verwendet */
  pObject->H = h;
  pObject->iType = OT_BOOL;
  pObject->pData = ( PVOID ) pData;  
  pData->pText = pText;
  pData->iDisplay = iDisplay;   
  pData->pValue = pValue;
}

/**********************************************************************/
/* win_OBJECTPrint - Object darstellen                                */
/**------------------------------------------------------------------**/
/* Eingabe: pWin       - Ausgabefenster                               */
/*          pObject    - auszugebendes Object                         */
/**------------------------------------------------------------------**/
/* Info: In dieser Funktion werden die Objet-Typen ermittelt, um die  */
/*       Objekte ihrer entsprechenden Art nach ausgeben zu knnen.    */
/**********************************************************************/
VOID _FP win_OBJECTPrint( PWINDOW pWin,
                          POBJECT pObject )
{ static CHAR pBuffer[ 256 ];
  switch( pObject->iType )
  {
    case OT_INT:
    { PINTDATA pIntData;
              /* Object dereferenzieren */
      pIntData = ( PINTDATA ) pObject->pData;
      sprintf( pBuffer, "%s%5d", pIntData->pText, *pIntData->pValue );
      win_PrintAt( pWin, pObject->X, pObject->Y, pBuffer );
    }
    break;
    case OT_BOOL:
    { PBOOLDATA pBoolData;
              /* Object dereferenzieren */
      pBoolData = ( PBOOLDATA ) pObject->pData;
      switch( pBoolData->iDisplay )
      {
        case DT_TRUEFALSE:
          sprintf( pBuffer, "%s%s",
          pBoolData->pText,
          *pBoolData->pValue ? "  WAHR":"FALSCH" );
        break;
        case DT_ONOFF:
          sprintf( pBuffer, "%s%s",
          pBoolData->pText,
          *pBoolData->pValue ? " AN":"AUS" );
        break;
        case DT_YESNO:
          sprintf( pBuffer, "%s%s",
          pBoolData->pText,
          *pBoolData->pValue ? "  JA":"NEIN" );
        break;
      }
      win_PrintAt( pWin, pObject->X, pObject->Y, pBuffer );
    }
  }
}

/**********************************************************************/
/* win_OBJECTQueryValue - Objectwert ermitteln                        */
/**------------------------------------------------------------------**/
/* Eingabe: pObject    - Object dessen Wert ermittelt werden soll     */
/* Ausgabe: Wert, dessen Typ vom Objekttyp abhngt, und somit evtl.   */
/*          gewandelt werden mu.                                     */
/**********************************************************************/
LONG _FP win_OBJECTQueryValue( POBJECT pObject )
{
  switch( pObject->iType )
  {
    case OT_INT:
    { PINTDATA pIntData;
              /* Object dereferenzieren */
      pIntData = ( PINTDATA ) pObject->pData;
      return *pIntData->pValue;
    } 

    case OT_BOOL:
    { PBOOLDATA pBoolData;
              /* Object dereferenzieren */
      pBoolData = ( PBOOLDATA ) pObject->pData; 
      return *pBoolData->pValue;
    }
  }
  return 0L;
}

/**********************************************************************/
/* win_OBJECTPrintArray - Objectliste ausgebe                         */
/**------------------------------------------------------------------**/
/* Eingabe: pWin    - Ausgabefenster                                  */
/*          pOArray - Objektliste ( Array )                           */
/*          iNum    - Anzahl der Objekte                              */
/*          iAct    - Nummer des aktuellen Objekts                    */
/**********************************************************************/
VOID _FP win_OBJECTPrintArray( PWINDOW pWin,
                               OBJECT  pOArray[],
                               INT     iNum,
                               INT     iAct )
{ INT i;
  
  for( i = 0; i < iNum; i++ )                     /* Alles darstellen */
  {
    if( i == iAct ) win_HiVideo( pWin );
    else            win_LoVideo( pWin );
    win_OBJECTPrint( pWin, &pOArray[ i ] );
  }
}

/**********************************************************************/
/* win_OBJECTProcessKey - Tastatureingaben verwalten/bearbeiten       */
/**------------------------------------------------------------------**/
/* Eingabe: pWin    - Ausgabefenster                                  */
/*          pObject - Objekt, das Tastatureingaben bearbeiten soll    */
/*          iKey    - Tastaturcode                                    */
/**********************************************************************/
VOID _FP win_OBJECTProcessKey( PWINDOW pWin,
                               POBJECT pObject,
                               INT     iKey )
{
  switch( pObject->iType )
  {
    case OT_INT:
    { PINTDATA pIntData;
              /* Object dereferenzieren */
      pIntData = ( PINTDATA ) pObject->pData;
      switch( iKey )
      {
        case '+': (*pIntData->pValue)++; break;
        case '-': (*pIntData->pValue)--; break;
      }
      if( (*pIntData->pValue) < pIntData->iMin )
        *pIntData->pValue = pIntData->iMax;
      if( (*pIntData->pValue) > pIntData->iMax )
        *pIntData->pValue = pIntData->iMin;

      win_HiVideo( pWin );
      win_OBJECTPrint( pWin, pObject );
    }
    break;
    case OT_BOOL:
    { PBOOLDATA pBoolData;
              /* Object dereferenzieren */
      pBoolData = ( PBOOLDATA ) pObject->pData;
      switch( iKey )
      {
        case '+': case '-':
         if(*pBoolData->pValue) *pBoolData->pValue = 0;
         else                   *pBoolData->pValue = 0xFFFF;
      }
      win_HiVideo( pWin );
      win_OBJECTPrint( pWin, pObject );
    }
    break;
  }
}

/**********************************************************************/
/* win_OBJECTWantsKey - Kann Objekt Tastatureingaben verwenden?       */
/**------------------------------------------------------------------**/
/* Eingabe: pObject - Objekt, das Tastatureingaben bearbeiten soll    */
/*          iKey    - Tastaturcode                                    */
/* Ausgabe: == 0 - Tastencode nicht verwertbar                        */
/*          <> 0 - Tastencode von Objekt verwendebar                  */
/**********************************************************************/
INT _FP win_OBJECTWantsKey( POBJECT pObject, INT iKey )
{
  switch( pObject->iType )
  {
    case OT_INT:
    case OT_BOOL:
    {
      switch( iKey )
      {
        case '+':
        case '-': return TRUE;
      }
    }
    break;
  }
  return FALSE;
}

/**********************************************************************/
/* win_OBJECTProcessArray - Objektliste (Dialog) bearbeiten           */
/**------------------------------------------------------------------**/
/* Eingabe: pWin    - Ausgabefenster                                  */
/*          pOArray - Objektliste                                     */
/*          iNum    - Anzahl der Objekte                              */
/*          pFunc   - Nachrichtenfunktion                             */
/**------------------------------------------------------------------**/
/* Info: Mit TAB und <SHIFT>+TAB wird der Eingabe-FOCUS verschoben.   */
/*       Die Benutzerfunktion pFunc wird aufgerufen, um dem Benutzer  */
/*       die Mglichkeit zu geben, direkt auf neue Zustnde der       */
/*       Objekte reagieren zu knnen.                                 */
/**********************************************************************/
VOID _FP win_OBJECTProcessArray( PWINDOW pWin, 
                                 OBJECT pOArray[],
                                 INT iNum,
                                 VOID ( *lpFunc ) ( INT  iNum,
                                                    INT  msg,
                                                    INT  iParam,
                                                    LONG lParam )
                               )
{ INT iAct;
  INT iLoop;
  
  iAct = 0;
  win_OBJECTPrintArray( pWin, pOArray, iNum, iAct );

  iLoop = TRUE;
  while( iLoop )
  { WORD c;
    LONG oldVal;
    c = _getch();
    
    if( !c ) c = MAKEWORD( _getch(), 0 );
    if( win_OBJECTWantsKey( &pOArray[ iAct ], c ) )
    {
      if( lpFunc ) lpFunc( iAct, MSG_PRECHANGE, 0, 0L );
      oldVal = win_OBJECTQueryValue( &pOArray[ iAct ] );
      win_OBJECTProcessKey( pWin, &pOArray[ iAct ], c );
      if( lpFunc ) lpFunc( iAct, MSG_CHANGED, 0, oldVal );
    }
    else
    {
      win_LoVideo( pWin );
      win_OBJECTPrint( pWin, &pOArray[ iAct ] );         
      if( lpFunc ) lpFunc( iAct, MSG_LOSTFOCUS, 0, 0L );
      switch( c ) 
      { 
        case KBD_ESC: iLoop = FALSE; break;
        case KBD_UP: iAct--; if( iAct < 0 ) iAct = iNum - 1; break;
        case KBD_DN: iAct++; if( iAct >= iNum ) iAct = 0; break;
      }
      win_HiVideo( pWin );
      win_OBJECTPrint( pWin, &pOArray[ iAct ] );
      if( lpFunc ) lpFunc( iAct, MSG_GOTFOCUS, 0, 0L );         
    } 
    if( lpFunc ) lpFunc( iAct, MSG_KEY, c, 0L );
  }  
}
#endif
