/***********************************************************************
*                              X M S C . C                             *
**--------------------------------------------------------------------**
*  Aufgabe        : Demonstriert den Zugriff auf extended Memory und   *
*                   High-Memory-Area mit Hilfe der XMS-Funktionen, wie *
*                   sie z.B. durch den Gertetreiber HIMEM.SYS imple-  *
*                   mentiert werden.                                   *
**--------------------------------------------------------------------**
*  Autor          : MICHAEL TISCHER                                    *
*  entwickelt am  : 27.07.1990                                         *
*  letztes Update : 29.07.1990                                         *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung     : CL /AS /Zp xmsc.c xmsca                            *
*  Aufruf         : xmsc                                               *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung     : ber eine Projekt-Datei mit folgendem Inhalt:      *
*                     xmsc.c                                           *
*                     xmsca.obj                                        *
***********************************************************************/

/*-- Include-Dateien einbinden ---------------------------------------*/

#include <dos.h>                              /* fr Interruptaufrufe */

#ifdef __TURBOC__
  #include <alloc.h>
#else
  #include <malloc.h>
#endif

/*-- Konstanten ------------------------------------------------------*/

#define ERR_NOERR          0x00                        /* kein Fehler */
#define ERR_NOTIMPLEMENTED 0x80  /* angegebene Funktion nicht bekannt */
#define ERR_VDISKFOUND     0x81             /* VDISK-RAMDISK entdeckt */
#define ERR_A20            0x82       /* Fehler auf Adressleitung A20 */
#define ERR_GENERAL        0x8E          /* allgemeiner Treiberfehler */
#define ERR_UNRECOVERABLE  0x8F            /* nicht-behebbarer Fehler */
#define ERR_HMANOTEXIST    0x90                /* keine HMA vorhanden */
#define ERR_HMAINUSE       0x91               /* HMA bereits vergeben */
#define ERR_HMAMINSIZE     0x92              /* zu wenig Platz in HMA */
#define ERR_HMANOTALLOCED  0x93                /* HMA nicht allokiert */
#define ERR_A20STILLON     0x94          /* Adressleitung A20 noch an */
#define ERR_OUTOMEMORY     0xA0     /* kein extended Memory mehr frei */
#define ERR_OUTOHANDLES    0xA1            /* alle XMS-Handles belegt */
#define ERR_INVALIDHANDLE  0xA2                  /* ungltiges Handle */
#define ERR_SHINVALID      0xA3             /* Source-Handle ungltig */
#define ERR_SOINVALID      0xA4             /* Source-Offset ungltig */
#define ERR_DHINVALID      0xA5        /* Destination-Handle ungltig */
#define ERR_DOINVALID      0xA6        /* Destination-Offset ungltig */
#define ERR_LENINVALID     0xA7 /* ungltige Lnge fr Move- Funktion */
#define ERR_OVERLAP        0xA8              /* verbotene berlappung */
#define ERR_PARITY         0xA9                    /* Partits-Fehler */
#define ERR_EMBUNLOCKED    0xAA             /* UMB ist nicht gesperrt */
#define ERR_EMBLOCKED      0xAB              /* UMB ist noch gesperrt */
#define ERR_LOCKOVERFLOW   0xAC      /* berlauf des UMB-Sperrzhlers */
#define ERR_LOCKFAIL       0xAD     /* UMB kann nicht gesperrt werden */
#define ERR_UMBSIZETOOBIG  0xB0            /* kleinerer UMB verfgbar */
#define ERR_NOUMBS         0xB1            /* kein UMB mehr verfgbar */
#define ERR_INVALIDUMB     0xB2    /* UMB-Segmentadresse ist ungltig */

#define TRUE  ( 0 == 0 )
#define FALSE ( 0 == 1 )

/*-- Makros ----------------------------------------------------------*/

#ifndef MK_FP
  #define MK_FP(seg,ofs) \
         ((void far *) (((unsigned long)(seg) << 16) | (unsigned)(ofs)))
#endif

#define Hi(x) (*((BYTE *) &x+1))                /* Hi-Byte eines ints */
#define Lo(x) (*((BYTE *) &x))                  /* Lo-Byte eines ints */

/*-- Typdeklarationen ------------------------------------------------*/

typedef unsigned char BYTE;
typedef BYTE BOOL;
typedef unsigned WORD;

typedef struct                        /* Informationen fr XMS-Aufruf */
         {
          WORD AX,              /* nur die Register AX, BX, DX und SI */
               BX,              /* werden je nach aufgerufener Funk-  */
               DX,              /* tion bentigt, dazu noch eine Seg- */
               SI,              /* mentadresse                        */
               Segment;
         } XMSRegs;

typedef struct                  /* eine extended-Memory-Move-Struktur */
         {
          long LenB;                       /* Anzahl zu versch. Bytes */
          int  SHandle;                              /* Source-Handle */
          long SOffset;                              /* Source-Offset */
          int  DHandle;                         /* Destination-Handle */
          long DOffset;                         /* Destination-Offset */
         } EMMS;

/*-- extern Deklarationen --------------------------------------------*/

extern void XMSCall( BYTE FktNr, XMSRegs *Xr );

/*-- globale Variablen -----------------------------------------------*/

void far * XMSPtr;    /* Zeiger auf den Extended-Memory-Manager (XMM) */
BYTE XMSErr;                      /* Fehlercode der letzten Operation */

/***********************************************************************
* XMSInit : Initialisiert die Routinen zum Aufruf der XMS-Funktionen   *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Ausgabe : TRUE, wenn ein XMS-Treiber entdeckt wurde, sonst FALSE     *
* Info    : - Der Aufruf dieser Funktion mu dem Aufruf aller anderen  *
*             Prozeduren und Funktionen aus diesem Programm voraus-    *
*             gehen.                                                   *
***********************************************************************/

BOOL XMSInit( void )
{
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */
 struct SREGS SRegs;                               /* Segmentregister */
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 Regs.x.ax = 0x4300;    /* Verfgbarkeit des XMS-Managers feststellen */
 int86( 0x2F, &Regs, &Regs );              /* DOS-Dispatcher aufrufen */

 if ( Regs.h.al == 0x80 )                 /* XMS-Manager angetroffen? */
  {                                                             /* Ja */
   Regs.x.ax = 0x4310;           /* Einsprungspunkt des XMM ermitteln */
   int86x( 0x2F, &Regs, &Regs, &SRegs );
   XMSPtr = MK_FP( SRegs.es, Regs.x.bx );  /*Adresse in glob. Var. sp.*/
   XMSErr = ERR_NOERR;                /* noch kein Fehler aufgetreten */
   return TRUE;           /* Handler angetroffen, Modul initialisiert */
  }
 else                                 /* kein XMS-Handler installiert */
  return FALSE;
}

/***********************************************************************
* XMSQueryVer: liefert die XMS-Versionsnummer und andere Statusinfor-  *
*              mationen                                                *
**--------------------------------------------------------------------**
* Eingabe : VerNr = nimmt die Versionsnummer nach dem Funktionsaufruf  *
*                   auf (Format: 235 == 2.35)                          *
*           RevNr = nimmt die Revisionsnummer nach dem Funktionsaufruf *
*                   auf                                                *
* Ausgabe : TRUE, wenn eine HMA zur Verfgung steht, sonst FALSE       *
***********************************************************************/

BOOL XMSQueryVer( int * VerNr, int * RevNr)
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 XMSCall( 0, &Xr );                       /* XMS-Funktion #0 aufrufen */
 *VerNr = Hi(Xr.AX)*100 + ( Lo(Xr.AX) >> 4 ) * 10 +
           ( Lo(Xr.AX) & 15 );
 *RevNr = Hi(Xr.BX)*100 + ( Lo(Xr.BX) >> 4 ) * 10 +
          ( Lo(Xr.BX) & 15 );
 return ( Xr.DX == 1 );
}

/***********************************************************************
* XMSGetHMA : Liefert dem Aufrufer das Zugriffsrecht auf die HMA.      *
**--------------------------------------------------------------------**
* Eingabe : LenB = Anzahl der zu allokierenden Bytes                   *
* Info    : TSR-Programme sollten immer nur genau die tatschlich be-  *
*           ntigte Speichergre anfordern, Applikationen hingegen    *
*           den Wert 0xFFFF angeben.                                   *
* Ausgabe : TRUE, wenn die HMA zur Verfgung gestellt werden konnte,   *
*           sonst FALSE;                                               *
***********************************************************************/

BOOL XMSGetHMA( WORD LenB )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 Xr.DX = LenB;                      /* Lnge im Register DX bergeben */
 XMSCall( 1, &Xr );                       /* XMS-Funktion #1 aufrufen */
 return XMSErr == ERR_NOERR;
}

/***********************************************************************
* XMSReleaseHMA : Gibt die HMA wieder frei und ermglicht dadurch ihre *
*                 Weitergabe an andere Programme.                      *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Info    : - Vor der Beendigung eines Programms sollte diese Prozedur *
*             aufgerufen werden, sofern die HMA zuvor ber einen Aufruf*
*             von XMSGetHMA allokiert wurde, weil sie sonst nicht an   *
*             nachfolgend aufgerufene Programme vergeben werden kann.  *
*           - Die in der HAM gespeicherten Daten gehen durch den Auf-  *
*             ruf dieser Prozedur verloren.                            *
***********************************************************************/

void XMSReleaseHMA( void )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 XMSCall( 2, &Xr );                       /* XMS-Funktion #2 aufrufen */
}

/***********************************************************************
* XMSA20OnGlobal: Schaltet die Adressleitung A20 frei, soda ein di-   *
*                 rekter Zugriff auf die HMA mglich wird.             *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Info    : - Die Freischaltung der Adressleitung A20 ist bei vielen   *
*             Rechnern ein relativ zeitaufwendiger Vorgagn. Rufen Sie  *
*             dieser Prozedur deshalb nicht unntig oft auf.           *
***********************************************************************/

void XMSA20OnGlobal( void )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 XMSCall( 3, &Xr );                       /* XMS-Funktion #3 aufrufen */
}

/***********************************************************************
* XMSA20OffGlobal: Als Gegenstck zu der Prozedur XMSA20OnGlobal       *
*                  schaltet diese Prozedur die Adressleitung A20 wie-  *
*                  der ab, so da kein direkter Zugriff auf die HMA    *
*                  mehr mglich ist.                                   *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Info    : - Diese Prozedur sollte vor der Beendigung eines Programms *
*             immer aufgerufen werden, falls zuvor die Leitung A20     *
*             mittels eines Aufrufs von XMSA20OnGlobal freigeschaltet  *
*             wurde.                                                   *
***********************************************************************/

void XMSA20OffGlobal( void )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 XMSCall( 4, &Xr );                       /* XMS-Funktion #4 aufrufen */
}

/***********************************************************************
* XMSA20OnLocal: Siehe XMSA20OnGlobal                                  *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Info    : - Von der gleichartigen globalen Prozedur unterscheidet    *
*             sich diese lokale Prozedur dadurch, da die Freischal-   *
*             tung nur dann erfolgt, wenn dies nicht bereits durch ei- *
*             nen vorhergehenden Aufruf erfolgt ist.                   *
***********************************************************************/

void XMSA20OnLocal( void )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 XMSCall( 5, &Xr );                       /* XMS-Funktion #5 aufrufen */
}

/***********************************************************************
* XMSA20OffLocal : Siehe XMSA29OffGlobal                               *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Info    : - Von der gleichartigen globalen Prozedur unterscheidet    *
*             sich diese lokale Prozedur dadurch, da die Abschaltung  *
*             nur dann erfolgt, wenn dies nicht bereits durch einen    *
*             vorhergehenden Aufruf erfolgt ist.                       *
***********************************************************************/

void XMSA20OffLocal( void )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 XMSCall( 6, &Xr );                       /* XMS-Funktion #6 aufrufen */
}

/***********************************************************************
* XMSIsA20On : Liefert den Status der Adressleitung A20                *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Ausgabe : TRUE, wenn die Leitungs A20 freigeschaltet ist, sonst      *
*           FALSE.                                                     *
***********************************************************************/

BOOL XMSIsA20On( void )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 XMSCall( 7, &Xr );                       /* XMS-Funktion #7 aufrufen */
 return ( Xr.AX == 1 );              /* AX == 1 ---> Leitung ist frei */
}

/***********************************************************************
* XMSQueryFree : Liefert die Gre des freien extended Memory und die  *
*                des grten freien Blocks                             *
**--------------------------------------------------------------------**
* Eingabe : GesFrei: Nimmt die Gesamtgre des freien EM auf.          *
*           MaxBl  : Nimmt die Gre des grten freien Blocks auf.    *
* Info    : - Beide Angabe in KB                                       *
*           - Die Gre der HMA wird nicht mitgezhlt, auch wenn sie   *
*             noch nicht an ein Programm vergeben wurde.               *
***********************************************************************/

void XMSQueryFree( int * GesFrei, int * MaxBl )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 XMSCall( 8, &Xr );                       /* XMS-Funktion #8 aufrufen */
 *GesFrei = Xr.AX;                         /* Gesamtgre steht in AX */
 *MaxBl   = Xr.DX;                     /* freier Speicher steht in DX */
}

/***********************************************************************
* XMSGetMem : Allokiert einen extended-Memory-Block (EMB)              *
**--------------------------------------------------------------------**
* Eingabe : LenKB : Gre des angeforderten Blocks in KB               *
* Ausgabe : Handle zum weiteren Zugriff auf dne Block oder 0, wenn     *
*           kein Block allokiert werden konnte. Dann steht auch in der *
*           globalen Variable XMSErr ein entsprechender Fehlercode.    *
***********************************************************************/

int XMSGetMem( int LenKb )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 Xr.DX = LenKb;                /* Lnge wird im DX-Register bergeben */
 XMSCall( 9, &Xr );                       /* XMS-Funktion #9 aufrufen */
 return Xr.DX;                                /* Handle zurckliefern */
}

/***********************************************************************
* XMSFreeMem : Gibt einen zuvor allokierten extended-Memory-Block      *
*              (EMB) wieder frei.                                      *
**--------------------------------------------------------------------**
* Eingabe : Handle : Das Handle zum Zugriff auf den Block, das beim    *
*                    Aufruf von XMSGetMem zurckgeliefert wurde.       *
* Info    : - Der Inhalt des EMB geht durch diesen Aufruf unwider-     *
*             bringlich verloren und auch das Handle wird ungltig.    *
*           - Vor der Beendigung eines Programms sollten alle allo-    *
*             kierten Bereich mit Hilfe dieser Prozedur wieder frei-   *
*             gegeben werden, damit sie an nachfolgend aufgerufene     *
*             Programm wieder vergeben werden knnen.                  *
***********************************************************************/

void XMSFreeMem( int Handle )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 Xr.DX = Handle;              /* Handle wird im DX-Register bergeben */
 XMSCall( 10, &Xr );                     /* XMS-Funktion #10 aufrufen */
}

/***********************************************************************
* XMSCopy : Kopiert Speicherbereiche zwischen dem extended Memory und  *
*           dem konventionellen Speicher bzw. innerhalb der beiden     *
*           Speichergruppen.                                           *
**--------------------------------------------------------------------**
* Eingabe : VonHandle  : Handle des Speicherbereichs, der verschoben   *
*                        werden soll.                                  *
*           VonOffset  : Offset in diesen Block ab dem verschoben wird.*
*           NachHandle : Handle des Speicherbereichs, der das Ziel der *
*                        Verschiebung darstellt.                       *
*           NachOffset : Offset in den Ziel-Block.                     *
*           LenW       : Anzahl der zu verschiebenden Worte            *
* Info    : - Um normalen Speicher in die Operation einzubeziehen,     *
*             mu als Handle der Wert 0 und fr den Offset die Seg-    *
*             ment und Offsetadresse in der gewohnte Form (Offset vor  *
*             Segment) angegeben werden.                               *
***********************************************************************/

void XMSCopy( int VonHandle, long VonOffset, int NachHandle,
              long NachOffset, int LenW )

{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */
 EMMS Mi;                                           /* nimmt EEMS auf */
 void far * MiPtr;

 Mi.LenB = 2 * LenW;                 /* zunchst den EMMS aufbereiten */
 Mi.SHandle = VonHandle;
 Mi.SOffset = VonOffset;
 Mi.DHandle = NachHandle;
 Mi.DOffset = NachOffset;

 MiPtr = &Mi;                          /* Far-Zeiger auf die Struktur */
 Xr.SI = FP_OFF( MiPtr );                   /* Offsetadresse des EMMS */
 Xr.Segment = FP_SEG( MiPtr );             /* Segmentadresse des EMMS */
 XMSCall( 11, &Xr );                     /* XMS-Funktion #11 aufrufen */
}

/***********************************************************************
* XMSLock : Sperrt einen extended-Memory-Block gegen sein Verschiebung *
*           durch den XMM und liefert gleichzeitig seine absolute      *
*           Adresse zurck.                                            *
**--------------------------------------------------------------------**
* Eingabe : Handle : Handle des Speicherbereichs, das bei einem vor-   *
*                    hergehenden Aufruf von XMSGetMem zurckgeliefert  *
*                    wurde.                                            *
* Ausgabe : Die lineare Adresse des zugehrigen Speicherblocks.        *
***********************************************************************/

long XMSLock( int Handle )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 Xr.DX = Handle;                                    /* Handle des EMB */
 XMSCall( 12, &Xr );                     /* XMS-Funktion #12 aufrufen */
 return ((long) Xr.DX << 16) + Xr.BX;        /* 32-Bit-Adr. berechnen */
}

/***********************************************************************
* XMSUnlock : Gibt einen gesperrten extended-Memory-Block wieder fr   *
*             die Verschiebung frei.                                   *
**--------------------------------------------------------------------**
* Eingabe : Handle : Handle des Speicherbereichs, das bei einem vor-   *
*                    hergehenden Aufruf von XMSGetMem zurckgeliefert  *
*                    wurde.                                            *
***********************************************************************/

void XMSUnlock( int Handle )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 Xr.DX = Handle;                                    /* Handle des EMB */
 XMSCall( 13, &Xr );                     /* XMS-Funktion #13 aufrufen */
}

/***********************************************************************
* XMSQueryInfo : Liefert verschiedene Informationen ber einen bereits *
*                allokierten extended-Memory-Block                     *
**--------------------------------------------------------------------**
* Eingabe : Handle : Handle des Speicherbereichs                       *
*           Lock   : Variable, in die der Lock-Zhler eingetragen wird *
*           LenKB  : Variable, in die die Lnge des Blocks in KB ein-  *
*                    getragen wird                                     *
*           FreeH  : Variable, ind die die Anzahl der noch freien      *
*                    Handles eingetragen wird                          *
* Info    : Die Startadresse eines Blocks knnen Sie mit Hilfe dieser  *
*           Prozedur nicht ermitteln, verwenden Sie dafr die Funktion *
*           XMSLock.                                                   *
***********************************************************************/

void XMSQueryInfo( int Handle, int * Lock, int * LenKB, int * FreeH )

{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 Xr.DX = Handle;                                    /* Handle des EMB */
 XMSCall( 14, &Xr );                     /* XMS-Funktion #14 aufrufen */
 *Lock  = Hi( Xr.BX );                          /* Register auswerten */
 *FreeH = Lo( Xr.BX );
 *LenKB = Xr.DX;
}

/***********************************************************************
* XMSRealloc : Vergert oder verkleinert einen zuvor ber XMSGetMem   *
*              allokierten extended-Memory-Block                       *
**--------------------------------------------------------------------**
* Eingabe : Handle   : Handle des Speicherbereichs                     *
*           NeuLenKB : Neue Lnge des Speicherbereichs in KB           *
* Ausgabe : TRUE, wenn der Block in seiner Gre verndert werden      *
*           konnte, sonst FALSE                                        *
* Info    : Der angegebene Block darf nicht gesperrt sein!             *
***********************************************************************/

BOOL XMSRealloc( int Handle, int NeuLenKB)
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 Xr.DX = Handle;                                    /* Handle des EMB */
 Xr.BX = NeuLenKB;               /* die neue Lnge in das BX-Register */
 XMSCall( 15, &Xr );                     /* XMS-Funktion #15 aufrufen */
 return ( XMSErr == ERR_NOERR );
}

/***********************************************************************
* XMSGetUMB : Allokiert einen upper-Memory-Block (UMB).                *
**--------------------------------------------------------------------**
* Eingabe : LenPara : Gre des zu allokierenden Bereichs in Para-     *
*                     graphen (jeweils 16 Byte)                        *
*           Seg     : Variable, die die Segmentadresse des allo-       *
*                     kierten UMB im Erfolgsfall aufnimmt              *
*           MaxPara : Variable, die die Lnge des grten verfgbaren  *
*                     UMB im Fehlerfall angibt                         *
* Ausgabe : TRUE, wenn ein UMB allokierte werden konnte, sonst FALSE   *
* Info    : Achtung! Diese Funktion wird nicht bei allen XMS-Treibern  *
*                    untersttzt und ist extrem Hardware-abhngig.     *
***********************************************************************/

BOOL XMSGetUMB( int LenPara, WORD * Seg, WORD * MaxPara )
{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 Xr.DX = LenPara;                         /* gewnschte Lnge nach DX */
 XMSCall( 16, &Xr );                     /* XMS-Funktion #16 aufrufen */
 *Seg = Xr.BX;                        /* Segmentadresse zurckliefern */
 *MaxPara = Xr.DX;                           /* Lnge des grten UMB */
 return ( XMSErr == ERR_NOERR );
}

/***********************************************************************
* XMSFreeUMB : Gibt einen zuvor ber XMSGetUMB allokierten UMB wieder  *
*              frei.                                                   *
**--------------------------------------------------------------------**
* Eingabe : Seg : Segmentadresse des freizugebenden UMB                *
* Info    : Achtung! Diese Funktion wird nicht bei allen XMS-Treibern  *
*                    untersttzt und ist extrem Hardware-abhngig.     *
***********************************************************************/

void XMSFreeUMB( WORD Seg )

{
 XMSRegs Xr;                               /* Register fr XMS-Aufruf */

 Xr.DX = Seg;                       /* Segmentadresse des UMB nach DX */
 XMSCall( 17, &Xr );                     /* XMS-Funktion #17 aufrufen */
}

/*--------------------------------------------------------------------*/
/*-- Test- und Demoprozeduren                                       --*/
/*--------------------------------------------------------------------*/

/***********************************************************************
* HMATest : Testet die Verfgbarkeit der HMA und demonstriert den Um-  *
*           gang mit ihr.                                              *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
***********************************************************************/

void HMATest( void)

{
 BOOL A20;                   /* aktuelle Status der Adressleitung A20 */
 BYTE far * hmap;                               /* Zeiger auf die HMA */
 WORD i,                                           /* Schleifenzhler */
      err;                       /* Anzahl der Fehler bei HMA-Zugriff */

 printf( "HMA-Test  -  Bettigen Sie bitte eine Taste, um den Test " \
         "zu starten..." );
 getch();
 printf ("\n\n" );

 /*-- HMA allokieren und jede Speicherstelle testen ------------------*/

 if ( XMSGetHMA(0xFFFF) )                  /* HMA in Besitz gebracht? */
  {                                                             /* Ja */
   if ( ( A20 = XMSIsA20On() ) == FALSE ) /* Leitungsstatus ermitteln */
    XMSA20OnGlobal();                           /* jetzt freischalten */

   hmap = MK_FP( 0xFFFF, 0x0010 );                  /* Zeiger auf HMA */
   err = 0;                           /* bis jetzt noch keinen Fehler */
   for ( i = 1; i < 65520; ++i, ++hmap )
    {                           /* jede Speicherstelle einzeln testen */
     printf( "\rSpeicherstelle: %u", i );
     *hmap = i % 256;                   /* Speicherstelle beschreiben */
     if ( *hmap != i % 256 )                   /* und wieder auslesen */
      {                                                    /* Fehler! */
       printf( " FEHLER!\n" );
       ++err;
      }
    }

   XMSReleaseHMA();                       /* die HMA wieder freigeben */
   if ( A20 == FALSE )                   /* war die Leitung A20 frei? */
    XMSA20OffGlobal();               /* Nein, jetzt wieder abschalten */

   printf( "\n" );
   if ( err == 0 )                    /* Resultat des Tests auswerten */
    printf( "HMA ok, keine Speicherstelle fehlerhaft.\n" );
   else
    printf( "ACHTUNG! %d fehlerhafte Speicherstellen in der HMA " \
            "entdeckt!\n", err );
  }
 else
  printf( "ACHTUNG! Kein Zugriff auf die HMA mglich.\n" );
}

/***********************************************************************
* EMBTest : Testet den extended Memory und demonstriert den Aufruf     *
*           verschiedener XMS-Funktionen                               *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
***********************************************************************/

void EMBTest( void )

{
 long Adr;
 BYTE * barp;                               /* Zeiger auf 1 KB-Puffer */
 int  i, j,                                        /* Schleifenzhler */
      err,                       /* Anzahl der Fehler bei HMA-Zugriff */
      Handle,                           /* Handle zum Zugriff auf EMB */
      GesFrei,           /* Gre des gesamten freien extended-Memory */
      MaxBl;                                  /* grter freier Block */

 printf( "EMB-Test  -  Bettigen Sie bitte eine Taste, um den Test " \
         "zu starten..." );
 getch();
 printf( "\n" );

 XMSQueryFree( &GesFrei, &MaxBl );   /* Gre des ext. Mem. ermitteln */
 printf( "Gesamtgre des freien extended Memory (inkl. HMA): %d KB\n",
         GesFrei );
 printf( "                        davon grter freier Block: %d KB\n",
         MaxBl );

 GesFrei -= 64;                   /* tatschliche Gre ohne HMA ber. */
 if ( MaxBl >= GesFrei )                    /* kann der Wert stimmen? */
  MaxBl -= 64;                                                /* Nein */

 if ( MaxBl > 0 )                     /* noch gengend Speicher frei? */
  {                                                             /* Ja */
   Handle = XMSGetMem( MaxBl );
   printf( "%d KB allokiert.\n", MaxBl );
   printf( "Handle       = %d\n", Handle );
   Adr = XMSLock( Handle );                      /* Adresse ermitteln */
   XMSUnlock( Handle );                       /* Lock wieder aufheben */
   printf( "Startadresse = %ld (%d KB)\n", Adr, Adr >> 10 );

   barp = malloc( 1024 );              /* Puffer auf Heap allokierten */
   err = 0;                           /* bis jetzt noch keinen Fehler */

   /*-- den allokierten EMB KB fr KB durchlaufen und testen ---------*/

   for  ( i = 0; i < MaxBl; ++i )
    {
     printf( "\rKB-Test: %d", i+1 );
     memset( barp, i % 255, 1024 );
     XMSCopy( 0, (long) ((void far *) barp),
              Handle, (long) i*1024, 512 );
     memset( barp, 255, 1024 );
     XMSCopy( Handle, (long) i*1024, 0,
              (long) ((void far *) barp), 512 );

     /*-- zurckkopierten Puffer mit erwartetem Resultat vergl. ------*/

     for ( j = 0; j < 1024; ++j )
      if ( *(barp+j) != i % 255 )
       {                                                   /* Fehler! */
        printf( " FEHLER!\n" );
        ++err;
        break;
       }
    }

   printf( "\n" );
   if ( err == 0 )                    /* Resultat des Tests auswerten */
    printf( "EMB ok, keiner der getesten 1 KB-Blcke war " \
            "fehlerhaft.\n");
   else
    printf( "ACHTUNG! %d fehlerhafte 1 KB-Blcke im EMB entdeckt\n",
            err );

   free( barp );                           /* Puffer wieder freigeben */
   XMSFreeMem( Handle );                      /* EMB wieder freigeben */
  }
}

/***********************************************************************
*                        H A U P T P R O G R A M M                     *
***********************************************************************/

void main( void )
{
 int VerNr,                                         /* Versionsnummer */
     RevNr,                                       /* Revisions-Nummer */
     i;                                            /* Schleifenzhler */

 for ( i = 1; i < 25; ++i )                     /* Bildschirm lschen */
  printf ( "\n" );

 printf("XMSC  -  XMS-Demoprogramm von MICHAEL TISCHER\n\n" );
 if ( XMSInit() )
  {
   if ( XMSQueryVer( &VerNr, &RevNr ) )
    printf( "Zugriff auf HMA mglich.\n" );
   else
    printf( "Kein Zugriff auf HMA.\n" );
   printf( "XMS-Versionsnummer: %d.%d\n", VerNr / 100, VerNr % 100 );
   printf( "Revisionsnummer   : %d.%d\n\n", RevNr / 100, RevNr % 100 );
   HMATest();                                           /* HMA testen */
   printf( "\n" );
   EMBTest();                               /* extended Memory testen */
  }
 else
  printf( "Kein XMS-Treiber installiert!\n" );
}

