{********************************************************************}
{                          C T T S . P A S                           }
{*------------------------------------------------------------------*}
{ Aufgabe        : Interfacefunktionen fr Zugriff auf Creative-     }
{                  Treiber "CTTS.DRV" (Text to Speach)               }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 20.03.1994                                        }
{ letztes Update :  8.10.1994                                        }
{********************************************************************}

Unit CTTS;
Interface

Const
 CTTS_DRIVERVERSION  = 0;
 CTTS_SETSETTINGS    = 1;
 CTTS_INI            = 2;
 CTTS_TERMINAT       = 3;
 CTTS_SETSPEECHPARA  = 4;
 CTTS_SA             = 5;

Function ctts_GetDrvVer( lpEntry : Longint ) : Word;

Function ctts_GetEnvSettings( lpEntry : Longint ) : Word;

Function ctts_Init( lpEntry : Longint ) : Word;

Procedure ctts_SetSpeechParam( lpEntry : Longint;
                               bGender,
                               bTone,
                               bVolume,
                               bPitch,
                               bSpeed : Byte );

Procedure ctts_Terminate( lpEntry : Longint );

Function ctts_Say( lpEntry : Longint; SpeechStr : String) : Word;

Implementation
Uses DOS;

{********************************************************************}
{ ctts_GetDrvVer : Ermittle Treiber-Versionsnummer                   }
{*------------------------------------------------------------------*}
{ Eingabe : lpEntry - Adresse des Treibereinsprungs                  }
{ Ausgabe : Versionsnummer - HiByte / LoByte                         }
{           $FFFF - kein Treibereinsprung                            }
{********************************************************************}
Function ctts_GetDrvVer( lpEntry : Longint ) : Word;

var cret : Word;

Begin
  if lpEntry <> 0 then Begin
   asm
     mov BX, CTTS_DRIVERVERSION
     call lpEntry
     mov cret, AX
   End;
   ctts_GetDrvVer := cret;
  End else ctts_GetDrvVer := $ffff;
End;

{********************************************************************}
{ ctts_GetEnvSettings : Treiber mit Umgebungsvariable versorgen      }
{*------------------------------------------------------------------*}
{ Eingabe : lpEntry - Adresse des Treibereinsprungs                  }
{ Ausgabe : 0 - alles Ok                                             }
{           1 - BLASTER-String = NULL                                }
{           2 - BLASTER-String fehlerhaft                            }
{           $FFFF - kein Treibereinsprung                            }
{********************************************************************}
Function ctts_GetEnvSettings( lpEntry : Longint ) : Word;

var cax       : Word;
    lpBlaster : String;

Begin
  if lpEntry <> 0 then
  Begin
    lpBlaster := GetEnv( 'BLASTER' );
    asm
      les di, dword ptr lpblaster     { Adresse des Blaster-Strings }
      mov BX, CTTS_SETSETTINGS
      call lpEntry
      mov cax, AX
    end;
    ctts_GetEnvSettings := cax;
  End
  else ctts_GetEnvSettings := $FFFF;
End;

{********************************************************************}
{ ctts_Init : Treiber initialisieren                                 }
{*------------------------------------------------------------------*}
{ Eingabe : lpEntry - Adresse des Treibereinsprungs                  }
{ Ausgabe : 0     - alles Ok                                         }
{           <> 0  - fehlerhafte Initialisierung                      }
{           $FFFF - kein Treibereinsprung                            }
{*------------------------------------------------------------------*}
{ Info : - cttsInit darf erst nach cttsSetEnvSettings aufgerufen     }
{          werden. cttsInit versucht den SBTALKER-Driver - der       }
{          vorher gestartet worden sein mu - zu initialisieren.     }
{********************************************************************}
Function ctts_Init( lpEntry : Longint ) : Word;

var cret : Word;

Begin
  if lpEntry <> 0 then
  Begin
    asm
      mov BX, CTTS_INI
      call lpEntry
      mov cret, AX
    end;
    Ctts_init := cret;
  End
  else ctts_Init := $FFFF;
End;

{********************************************************************}
{ ctts_SetSpeechParam : Sprach-Parameter festlegen                   }
{*------------------------------------------------------------------*}
{ Eingabe : lpEntry - Adresse des Treibereinsprungs                  }
{           bGender - Geschlecht ( 0 = mnnlich, 1 = weiblich )      }
{           bTone   - 0 = Bass, 1 = Hhen                            }
{           bVolume - Lautstrke ( 0 - 9 )                           }
{           bPitch  - Tonhhe ( 0 - 9 )                              }
{           bSpeed  - Geschwindigkeit ( 0 - 9 )                      }
{********************************************************************}
Procedure ctts_SetSpeechParam( lpEntry : Longint;
                               bGender,
                               bTone,
                               bVolume,
                               bPitch,
                               bSpeed : Byte );
Begin
  if lpEntry <> 0 then
    asm
      mov BX, CTTS_SETSPEECHPARA
      mov AL, bGender
      mov AH, bTone
      mov DL, bVolume
      mov DH, bPitch
      mov CL, bSpeed
      call lpEntry
    end;
End;

{********************************************************************}
{ ctts_Terminate : Treiber deinstallieren                            }
{*------------------------------------------------------------------*}
{ Eingabe : lpEntry - Adresse des Treibereinsprungs                  }
{*------------------------------------------------------------------*}
{ Info : - Diese Funktion mu vor 'sb_UnloadDriver' aufgerufen       }
{          werden                                                    }
{********************************************************************}
Procedure ctts_Terminate( lpEntry : Longint );

Begin
  if lpEntry <> 0 then
    asm
      mov BX, CTTS_TERMINAT
      call lpEntry
    end;
End;

{********************************************************************}
{ ctts_Say : Text aufsagen                                           }
{*------------------------------------------------------------------*}
{ Eingabe : lpEntry   - Adresse des Treibereinsprungs                }
{           SpeechStr - Auszugebender Text (darf keine Umlaute       }
{                       enthalten )                                  }
{ Ausgabe: 0     - alles OK                                          }
{          1     - Ausgabestring leer oder NULL                      }
{          2     - Ausgabestring zu lang                             }
{          $FFFF - kein Treibereinsprung                             }
{********************************************************************}
Function ctts_Say( lpEntry : Longint; SpeechStr : String) : Word;

var cret      : Word;
    SpeechPtr : longint;

Begin
  SpeechStr := SpeechStr + #0;
  SpeechPtr := longint(@SpeechStr)+1;
  if lpEntry <> 0 then
  Begin
    asm
      les di, SpeechPtr
      mov BX, CTTS_SA
      call lpEntry
      mov cret, AX
    end;
    ctts_say := cret;
  End;
  ctts_say := $FFFF;
End;

End.

