{**********************************************************************}
{*                         D D P T P . P A S                          *}
{*--------------------------------------------------------------------*}
{*    Aufgabe       : ermglicht die selektive Vernderung einzelner  *}
{*                    Werte in der Disk-Drive-Parameter-Tabele        *}
{*--------------------------------------------------------------------*}
{*    Autor         : Michael Tischer                                 *}
{*    entwickelt am : 22.08.1991                                      *}
{*    letztes Update: 27.01.1992                                      *}
{**********************************************************************}

program DISKOPT;

Uses Crt, Dos;                   { die Crt- und die DOS-Unit einbinden }

type DDPT_T   = array[ 0..10 ] of byte;        { Struktur fr die DDPT }
     DDPT_PTR = ^DDPT_T;                         { Zeiger auf die DDPT }

var  DDPT : DDPT_PTR;                            { Zeiger auf die DDPT }

{**********************************************************************}
{* byte_hex : Wandelt ein Byte in eine HEX-Zahl                       *}
{* Eingabe  : Zu wandelnde Zahl                                       *}
{* Ausgabe  : Zahl als HEX-String                                     *}
{**********************************************************************}

function byte_hex( zahl : byte ) : string;

{-- Wandel einer Ziffer 0 - 15 in Hex 0 - F ---------------------------}

function h_ziffer( ziffer : byte ) : char;

begin
  if ( ziffer >= 10 ) then                   { Ziffer >= 10 also A - F }
    h_ziffer := chr( 55 + ziffer )
  else                                       { Nein Ziffer wie dezimal }
    h_ziffer := chr( 48 + ziffer );
end;

begin
  byte_hex := '$' + h_ziffer( zahl div 16 ) + h_ziffer( zahl mod 16 );
end;

{**********************************************************************}
{* hex_byte : Wandel einen HexString in ein Byte                      *}
{* Eingabe  : Zu wandelnder HEX-String                                *}
{* Ausgabe  : Zahl                                                    *}
{**********************************************************************}

function hex_byte( hex : string ) : byte;

{-- Wandel einer Hex-Ziffer 0 - F in  0 - 15  -------------------------}

function d_ziffer( ziffer : char ) : byte;

begin
  if ( ziffer >= 'A' ) and ( Ziffer <= 'F' ) then
    d_ziffer := ord( ziffer ) - 55
  else                                       { Nein Ziffer wie dezimal }
    d_ziffer := ord( ziffer ) - 48;
end;

begin
  if ( hex[ 1 ] = '$' ) then
    delete( hex, 1, 1 );
  if length( hex ) = 1 then
    hex := '0' + hex;
  hex_byte := d_ziffer( hex[ 1 ] ) * 16 + d_ziffer( hex[ 2 ] );
end;

{**********************************************************************}
{* RAM_DDPT : Test ob DDPT im RAM oder im ROM                         *}
{* Eingabe  : keine                                                   *}
{* Ausgabe  : TRUE wenn DDPT im RAM                                   *}
{* Info     : die Funktion schreibt einen Wert in die DDPT, liest     *}
{*            diesen wieder aus und stellt so fest, ob der Wert       *}
{*            verndert werden konnte, sich die DDPT als im RAM       *}
{*            befindet                                                *}
{**********************************************************************}

function RAM_DDPT : boolean;

var buffer : byte;              { Speicher fr aktuellen Wert der DDPT }

begin
  buffer := DDPT^[ 0 ];                      { Wert der DDPT speichern }
  DDPT^[ 0 ] := not buffer;                         { Wert invertieren }
  RAM_DDPT := ( DDPT^[ 0 ] = not buffer );     { Schreibtest auswerten }
  DDPT^[ 0 ] := buffer                       { alten Wert restaurieren }
end;

{**********************************************************************}
{* Werteanzeigen: Werte der DDPT ausgeben                             *}
{* Eingabe      : keine                                               *}
{* Ausgabe      : keine                                               *}
{**********************************************************************}

procedure Werteanzeigen;

begin
  writeln( 'Steprate                (SR): ',
               byte_hex( DDPT^[ 0 ] shr 4 ) );
  writeln( #13#10'Head-Unload-Time        (HU): ',
               byte_hex( DDPT^[ 0 ] and $F ) );
  writeln( 'Head-Load-Time          (HL): ',
               byte_hex( DDPT^[ 1 ] shr 1 ) );
  writeln( 'Head-Settle-Time        (HS): ',
               byte_hex( DDPT^[ 9 ] ) );
  writeln( #13#10'Nachlaufzeit des Motors (MN): ',
               byte_hex( DDPT^[ 2 ] ) );
  writeln( 'Anlaufzeit des Motors   (MA): ',
               byte_hex( DDPT^[ 10 ] ) );
end;

{**********************************************************************}
{* NeueWerte: Neue Werte der DDPT setzen                              *}
{* Eingabe  : keine                                                   *}
{* Ausgabe  : keine                                                   *}
{**********************************************************************}

procedure NeueWerte;

var i,j       : byte;                                { Schleifenzhler }
    Art       : string[ 2 ];     { Parameter der verndert werden soll }
    Wert      : byte;                        { Neuer zu setzender Wert }
    HilfsWert : byte;                        { Hilfswert zum Speichern }
    Befehl    : string[ 6 ];                 { Kommandozeilenparameter }

begin
  {-- Schleife: alle Parameter durchlaufen ----------------------------}

  for i := 1 to Paramcount do
    begin
      Befehl := paramstr( i );                       { Parameter holen }
      for j := 1 to length( befehl ) do        { Befehl in Groschrift }
        befehl[ j ] := upcase( Befehl[ j ] );
      Art := copy( befehl, 1, 2 );    { Wert der verndert werden soll }
      delete( Befehl, 1, 3 );                   { Neuen Wert ermitteln }
      Wert := hex_byte( befehl );
      if ( Art = 'SR' ) then                               { Steprate? }
        begin
          Wert := Wert shl 4;                   { Wert in obere Nibble }
          HilfsWert := DDPT^[ 0 ] and $0F;            { Unteres Nibble }
          DDPT^[ 0 ] := Wert or HilfsWert;            { Wert speichern }
        end
      else if ( Art = 'HU' ) then                  { Head-Unload-Time? }
        begin
          Wert := Wert and $0F;           { Wert nur im unteren Nibble }
          HilfsWert := DDPT^[ 0 ] and $F0;             { Oberes Nibble }
          DDPT^[ 0 ] := Wert or HilfsWert;            { Wert speichern }
        end
      else if ( Art = 'HL' ) then                    { Head-Load-Time? }
        DDPT^[ 1 ] := Wert shl 1           { Wert in Bit 1-7 speichern }
      else if ( Art = 'HS' ) then                  { Head-Settle-Time? }
        DDPT^[ 9 ] := Wert                            { Wert speichern }
      else if ( Art = 'MN' ) then               { Motor-Nachlauf-Zeit? }
        DDPT^[ 2 ] := Wert                            { Wert speichern }
      else if ( Art = 'MA' ) then                 { Motor-Anlauf-Zeit? }
        DDPT^[ 10 ] := Wert;                          { Wert speichern }
    end;
end;

{**********************************************************************}
{*                         HAUPTPROGRAMM                              *}
{**********************************************************************}

begin
  ClrScr;                                         { Bildschirm lschen }
  writeln( 'DDPTP  -  (c) 1992 by Michael Tischer');
  writeln( 'Erlaubt die Vernderung der aktuellen DDPT' );

  GetIntVec( $1E, pointer( DDPT ) );       { Zeiger auf die DDPT holen }

  if ( RAM_DDPT ) then                   { DDPT im RAM, also nderbar? }
    begin
      if ( Paramcount > 0 ) then        { Sollen Werte gesetzt werden? }
        begin
          NeueWerte;                           { Ja, neue Werte setzen }
          writeln( #13#10#10'Neuer Inhalt der DDPT:');
          WerteAnzeigen;                { Neue Werte der DDPT ausgeben }
          exit;
        end;
    end
  else
    writeln( 'Eine nderung der Disk-Drive-Paramter-Table ' +
              'ist nicht mglich, weil sie im ROM liegt!' );

  writeln( #13#10'Inhalt der DDPT:');
  WerteAnzeigen;                        { Alte Werte der DDPT ausgeben }
end.
