{**********************************************************************}
{*                             D I R P 2                              *}
{*--------------------------------------------------------------------*}
{*    Aufgabe        : Zeigt alle Dateien in einem beliebigen Ver-    *}
{*                     zeichnis inklusive Unterverzeichnissen und     *}
{*                     Volume-Namen auf dem Bildschirm an.            *}
{*                     Die Abfrage erfolgt dabei ber die beiden      *}
{*                     Pascal-Funktionen FindFirst und FindNext aus   *}
{*                     der DOS-Unit.                                  *}
{*                     Siehe in disem Zusammenhang auch das Programm  *}
{*                     DIRP1.PAS.                                     *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  :  8.07.1988                                     *}
{*    letztes Update : 15.10.1991                                     *}
{**********************************************************************}
                                                                        
program DIRP2;
                                                                        
Uses Crt, Dos;                   { die CRT- und die DOS-Unit einbinden }
                                                                        
{-- Typdeklarationen --------------------------------------------------}

type MonVec    = array[1..12] of string[3];    { Array mit Monatsnamen }
                                                                        

{-- Konstanten --------------------------------------------------------}

const EINTR = 14;        { soviele Eintrge sind gleichzeitig sichtbar }
      Monate : MonVec = ( 'Jan', 'Feb', 'Mr', 'Apr', 'Mai', 'Jun',
                          'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez');

{**********************************************************************}
{* PRINTDATA: Informationen ber einen Eintrag ausgeben               *}
{* Eingabe : DIRBUF = Datenstrktur mit Dateiinformationen             *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure PrintData( DirBuf : SearchRec );
                                                                        
var Zaehler : byte;
    Date,               { zum Aufsplitten des TIME-Feldes in SearchRec }
    Time    : word;

type longrec = record        { zum Aufsplitten eines LONG in zwei Word }
                 LoWord,
                 HiWord  : word
               end;

begin
  writeln;      { dadurch wird das Fenster um eine Zeile hochgescrollt }

  write( DirBuf.Name );   { der Name wurde schon nach Pascal gewandelt }

  GotoXY(13, EINTR);
  write('', DirBuf.Size:7);

  Date := longrec(DirBuf.Time).HiWord;  { Datum und Zeit aus SearchRec }
  Time := longrec(DirBuf.Time).LoWord;

  GotoXY(21, EINTR);
  write('', Date and 31:2,'. ');                      { Tag ermitteln }
  write (Monate[Date shr 5 and 15]);                  { Monat ausgeben }
  write(Date shr 9 + 1980:5);                         { Jahr ermitteln }
  GotoXY(34, EINTR);
  write('', Time shr 11:2, ' Uhr');                { Stunde ermitteln }
  write(Time shr 5 and 63:3);                       { Minute ermitteln }

  GotoXY(44, EINTR);
  write('');                    { Trennstrich zum vorhergehenden Feld }
  Zaehler := 1;                               { die Attribute ausgeben }
  while ( Zaehler < 32 ) do
    begin
      if (DirBuf.Attr and Zaehler) <> 0 then write('X')
                                             else write(' ');
      Zaehler := Zaehler shl 1;
    end;
  write('');                         { rechte Wand des Fensterrahmens }
end;
                                                                        
{**********************************************************************}
{* BildschirmAufbau: bereitet den Bildschirm fr die Ausgabe des      *}
{*                   Directories vor                                  *}
{* Eingabe : keine                                                    *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure BildschirmAufbau;
                                                                        
var Zaehler : integer;                               { Schleifenzhler }
                                                                        
begin
 ClrScr;                                          { Bildschirm lschen }
 Window(14,(20-EINTR) shr 1+1,64,(20-EINTR) shr 1 +5+EINTR);
 GotoXY(1,1);            { Cursor in die obere linke Ecke des Fensters }

 write('ͻ');
 write(' Dateiname   Gre    Datum      Zeit   RHSVD');
 write('Ķ');

 for Zaehler := 1 to EINTR do
  write('                                             ');
 write('ͼ');

 Window(15,(20-EINTR) shr 1+4,66,(20-EINTR) shr 1 +3+EINTR);
 GotoXY(1, EINTR);        { Cursor in die obere linke Ecke des Fenster }
end;
                                                                        
{**********************************************************************}
{* Dir: kontrolliert das Einlesen und die Ausgabe des Directories     *}
{* Eingabe : PFAD     = Such-Pfad mit Dateimaske                      *}
{*           ATTRIBUT = Such-Attribut                                 *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure Dir( Pfad : string; Attr : byte );

var AnzEintraege,              { Gesamt-Anzahl der gefundenen Eintrge }
    AnzBild      : integer;              { Anzahl der Eintrge in Bild }
    Taste        : char;            { dient zum Warten auf Tastendruck }
    DirBuf       : SearchRec;    { nimmt jeweils einen Dir-Eintrag auf }

begin
  clrscr;                                         { Bildschirm lschen }
  BildschirmAufbau;        { Bildschirm fr Directory-Ausgabe aufbauen }

  AnzBild := -1;              { noch kein Eintrag im Fenster angezeigt }
  AnzEintraege := 0;                    { noch keinen Eintrag gefunden }
  FindFirst( Pfad, Attr, DirBuf );             { ersten Eintrag suchen }
  if DOSError = 0 then                          { eine Datei gefunden? }
    repeat          { Ja, ausgeben und weitersuchen, bis alle gelistet }
      AnzEintraege := succ(AnzEintraege);  { ein Eintrag mehr gefunden }
      AnzBild := succ(AnzBild);          { ein Eintrag mehr im Fenster }
      if AnzBild = EINTR then                  { ist das Fenster voll? }
        begin                                                     { JA }
          Window(14, (20-EINTR) shr 1 + 5 + EINTR,
                66, (20-EINTR) shr 1 + 6+ EINTR );
          GotoXY(1, 1);     { Cursor in die unterste Zeile des Fenster }
          TextBackground( LightGray );            { weier Hintergrund }
          TextColor( Black );                       { schwarze Schrift }
          write('                Bitte Taste drcken                ');
          Taste := ReadKey;                           { Taste einlesen }
          GotoXY(1, 1);   { Cursor in die obere linke Ecke des Fenster }
          TextBackground( Black );             { schwarzer Hintergrund }
          TextColor( LightGray );                      { weie Schrift }
          write('                                                   ');
          Window(15,(20-EINTR) shr 1+4,65,(20-EINTR) shr 1 +3+EINTR);
          GotoXY(1, EINTR);     { Cursor an seine alte Position zurck }
          AnzBild := 0;             { mit zhlen wieder bei 0 anfangen }
        end;
      PrintData( DirBuf );                { Daten des Eintrag ausgeben }
      FindNext( DirBuf );                       { nchste Datei suchen }
    until DOSError <> 0;               { bis keine Datei mehr gefunden }

  Window(14,(20-EINTR) shr 1 +5+EINTR,65,(20-EINTR) shr 1 +6+EINTR);
  GotoXY(1, 1);           { Cursor in die obere linke Ecke des Fenster }
  TextBackground( LightGray );                    { weier Hintergrund }
  TextColor( Black );                               { schwarze Schrift }
  write('                                                   ');

  GotoXY(2, 1);
  case AnzEintraege of
    0 : write('keine Datei gefunden');
    1 : write('eine Datei gefunden');
    else write(AnzEintraege,' Dateien gefunden')
  end;

 Window(1, 1, 80, 25);   { wieder ganzen Bildschirm als Fenster setzen }
end;
                                                                        
{**********************************************************************}
{**                           HAUPTPROGRAMM                          **}
{**********************************************************************}
                                                                        
begin
  case ParamCount of                    { je nach Anzahl der Argumente }
    0  : Dir( '*.*', AnyFile );      { alle Dateien im aktuellen Verz. }
    1  : Dir( ParamStr(1), AnyFile );        { angegebene Dat anzeigen }
    else writeln('Ungltige Anzahl Parameter');
  end;
end.
