{********************************************************************}
{                             D S P . P A S                          }
{*------------------------------------------------------------------*}
{ Aufgabe        : Demo-Programm zur Veranschaulichung der DSP-      }
{                  Programmierung                                    }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 20.03.1994                                        }
{ letztes Update :  8.10.1994                                        }
{********************************************************************}

{$m 16000,16000,16000}

{$A-}                           { kein Word-Alignment von Strukturen }
{$X+}                                  { Funtionsergebnisse optional }

Uses ARGS,DSPUTIL,SBUTIL,MIXUTIL,DMAUTIL,DOS;

Const
 DSP_NAME = 'DSP';
 BUFSIZE  = 8192;
 ERROR    = $ffff;
 NO_ERROR = 0;

var
  SBB            : SBBASE;
  lFrequency     : Longint;
  iADC,iST       : Boolean;
  iDuration,
  iSource,ior    : Integer;
  AktDatei       : Integer; { Dateizugriff ber DOS }
  lpRecordSource : string;
  DateiNamen     : NArgStrings;
  AnzDateien     : integer;
  lpBuffer       : pointer;
  uMemSize, uFrq : Word;
  dspHeader      : DSPRECPLAY;

{********************************************************************}
{                 H A U P T P R O G R A M M                          }
{********************************************************************}
Begin
  if sb_GetEnviron( SBB, GetEnv( 'BLASTER' ) ) <> NO_ERROR  then
  Begin
    Writeln( 'BLASTER-Umgebungsvariable nicht vorhanden' );
    Halt( 0 );
  End;

  if dsp_SetBase( SBB, TRUE ) = NO_ERROR then
  Begin
    AnzDateien := GetNArg( '-', DateiNamen );

    if AnzDateien = 0 then
    Begin
      Writeln( 'Aufruf: ',DSP_NAME ,' [-bX][-dX][-fX][-r][-s][-w] Dateiname' );
      Writeln;
      Writeln( '-bX   : DMA Buffer Gre ' );
      Writeln( '-dX   : Lnge der Aufnahme in Sekunden' );
      Writeln( '-fX   : Frequenz in Hz' );
      Writeln( '-r    : Aufnahme (sonst Wiedergabe)' );
      Writeln( '-s    : Stereo (sonst Mono)' );
      Writeln( '-w    : 16-Bit Transfer (Word) ' );
      Writeln( '-iMIC/CD/LINE : Aufnahmequelle (Input)' );
      Halt( 0 );
    End;

    {-- es wurden Dateinamen in der Kommandozeile eingegeben --------}

    iADC := GetArg( '-r', _none, NIL, 0 ) <> 0;  { Aufnahme (Record) }

    dspHeader.iStereo := FALSE;                    { Defaults setzen }
    dspHeader.uFrequency := 10000;
    dspHeader.iBits := 8;

    ior := dsp_FileOpen( Dateinamen[0], AktDatei );
    if not iADC then
      if ior=0 then
        dsp_ReadHeader( AktDatei, dspHeader )
      else
        Begin
          Writeln( 'Datei ', Dateinamen[0],' nicht gefunden!' );
          Halt( 0 );
        End;

    if GetArg( '-s', _none, NIL, 0 ) <> 0 then
        dspHeader.iStereo := TRUE;

    lFrequency := dspHeader.uFrequency;                 { Frequenz }
    if GetArg( '-f', _long, @lFrequency, 1 ) <> 0 then
      dspHeader.uFrequency := Word ( lFrequency );

    if SBB.uDspVersion >= DSP_4XX then
      if GetArg( '-w', _none, Nil, 0 ) <> 0 then
        dspHeader.iBits := 16;

    if iADC then dsp_WriteHeader( AktDatei, dspHeader );

    iDuration := 10;                        { Spieldauer bei Aufnahme }
    GetArg( '-d', _int, @iDuration, 1 );

    uMemSize := BUFSIZE;   { fr DMA zur Verfgung stehender Speicher }
    GetArg( '-b', _int, @uMemSize, 1 );

    lpRecordSource := '';
    GetArg( '-i', _string, @lpRecordSource, 1 );
    iSource := -1;
    if( Pos( Up( lpRecordSource ), 'MIC' ) = 1 ) then iSource := MIC
    else
    if( Pos( Up( lpRecordSource ), 'CD' ) = 1 ) then iSource := CD
    else
    if( Pos( Up( lpRecordSource ), 'LINE' ) = 1 ) then iSource := LINE;

    sb_Print( SBB );
  End
  else
  Begin
    Writeln( 'Keine DSP-Ansteuerung mglich!' );
    Halt( 0 );
  End;

  if ( ( dspHeader.iStereo ) and not dsp_CanStereo ) then
    Writeln(' WARNUNG! Stereomodus wird NICHT untersttzt.' );
  if dspHeader.iBits > dsp_MaxBits then
    Writeln(' WARNUNG! 16-Bit Modus wird NICHT untersttzt!' );

  uFrq := dspHeader.uFrequency;
  iSt  := dspHeader.iStereo;
  dsp_AdjustFrq( uFrq, iADC, iSt );
  if uFrq < dspHeader.uFrequency then
    Writeln( 'WARNUNG! Frequenz zu hoch! Wird angepasst!' );

  uMemSize := uMemSize and not $0003;
                                  { uMemSize ist nun durch 4 Teilbar }
  lpBuffer := dma_AllocMem( uMemSize );    { DMA-Speicher allokieren }
  if lpBuffer <> NIL then
    Begin
      dsp_DoRecPlay( AktDatei, iADC, iSource, dspHeader, iDuration,
                     lpBuffer, uMemSize );
      dma_Free( lpBuffer );                       { Speicher freigeben }
    End
  else
    Writeln( 'Kein freier Speicher mehr!' );
  dsp_FileClose( AktDatei );                        { Datei schlieen }
End.
