{**********************************************************************}
{*                      F I X P A R T P . P A S                       *}
{*--------------------------------------------------------------------*}
{*    Aufgabe        : Zeigt die Partitionierung einer Festplatte an. *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  : 26.04.1989                                     *}
{*    letztes Update : 12.01.1992                                     *}
{*--------------------------------------------------------------------*}
{*    Aufruf         : FIXPARTP [ Laufwerksnummer ]                   *}
{*                       Defaulteinstellung ist Laufwerk 0 ("C")      *}
{**********************************************************************}

uses Dos;                                         { DOS-Unit einbinden }

{== Typdeklarationen ==================================================}

type  SecPos    = record       { beschreibt die Position eines Sektors }
                    Kopf : byte;                   { Schreib-/Lesekopf }
                    SecZyl : word;        { Sektor- und Zylindernummer }
                  end;

      PartEntry = record            { Eintrag in der Partitionstabelle }
                    Status   : byte;               { Partitions-Status }
                    StartSec : SecPos;                 { erster Sektor }
                    PartTyp  : byte;                  { Partitions-Typ }
                    EndSec   : SecPos;                { letzter Sektor }
                    SecOfs   : longint;      { Offset des Boot-Sektors }
                    SecAnz   : longint;          { Anzahl der Sektoren }
                  end;

      PartSec   = record           { beschreibt den Partititionssektor }
                    BootCode  : array [0..$1BD] of byte;
                    PartTable : array [1..4] of PartEntry;
                    IdCode    : word;                          { $AA55 }
                  end;

{**********************************************************************}
{*  ReadPartSec : Liest einen Partitionssektor von der Festplatte in  *}
{*                einen Puffer                                        *}
{**------------------------------------------------------------------**}
{*  Eingabe: - Laufwerk : BIOS-Code des Laufwerks ($80, $81 etc.)     *}
{*           - Kopf     : Nummer des Schreib-/Lesekopfs               *}
{*           - SekZyl   : Sektor- und Zylindernummer im BIOS-Format   *}
{*           - Buf      : Puffer, in den der Sektor geladen wird      *}
{**********************************************************************}

function ReadPartSec( Laufwerk, Kopf : byte;
                      SekZyl         : word;
                      var Buf        : PartSec ) : boolean;

var Regs : Registers;              { Prozessorregs fr Interruptaufruf }

begin
  Regs.AX := $0201;                      { Fktnr. fr "Read", 1 Sektor }
  Regs.DL := Laufwerk;                        { die weiteren Parameter }
  Regs.DH := Kopf;                            { in die verschiedenen   }
  Regs.CX := SekZyl;                          { Register laden         }
  Regs.ES := seg( Buf );
  Regs.BX := ofs( Buf );
  Intr( $13, Regs);                       { Festplatten-Intr. aufrufen }
  ReadPartSec := ( Regs.Flags and 1 ) = 0;  {Carry-Flag zeigt Fehler an}
end;

{**********************************************************************}
{*  GetSecZyl: ermittelt aus der kombinierten Sektor/Zylinder-Codier- *}
{*             ung des BIOS die Sektor- und Zylinder-Nummer           *}
{**------------------------------------------------------------------**}
{*  Eingabe: SekZyl   : der zu dekodierende Wert                      *}
{*           Sektor   : Referenz auf die Sektor-Variable              *}
{*           Zylinder : Referenz auf die Zylinder-Variable            *}
{**********************************************************************}

procedure GetSecZyl( SekZyl : word; var Sektor, Zylinder : integer );

begin
  Sektor   := SekZyl and 63;                 { Bits 6 und 7 ausblenden }
  Zylinder := hi( SekZyl ) + ( lo( SekZyl) and 192 ) shl 2;
end;

{**********************************************************************}
{*  ShowPartition: zeigt die Partitionierung einer Festplatte auf     *}
{*                 dem Bildschirm an                                  *}
{**------------------------------------------------------------------**}
{*  Eingabe: LW : Nummer des angesprochenen Festplattenlaufwerks      *}
{*                (0, 1, 2 etc.)                                      *}
{**********************************************************************}

procedure ShowPartition( LW : byte );

var Kopf     : byte;                    { Kopf der aktuellen Partition }
    SecZyl   : byte;     { Sektor und Zylinder der aktuellen Partition }
    ParSec   : PartSec;                { der aktuelle Partitionssektor }
    Entry    : byte;                                 { Schleifenzhler }
    Sektor,                             { nehmen Sektor- und Zylinder- }
    Zylinder : integer;                 { Nummer auf                   }
    Regs     : Registers;          { Prozessorregs fr Interruptaufruf }

begin
  writeln;
  LW := LW + $80;               { Laufwerksnummer fr BIOS aufbereiten }
  if ReadPartSec( LW, 0, 1, ParSec ) then     { Partitionssektor lesen }
    begin                               { Sektor konnte gelesen werden }
      Regs.AH := 8;                 { Kenndaten des Laufwerks erfragen }
      Regs.DL := LW;
      Intr( $13, Regs);                   { Festplatten-Intr. aufrufen }
      GetSecZyl( Regs.CX, Sektor, Zylinder );
      writeln(''+
              'ͻ');
      writeln(' Laufwerk ', LW-$80, ':    ', Regs.DH+1:2,
              ' Kpfe mit jeweils', Zylinder:5, ' Zylindern zu je',
              Sektor:3, ' Sektoren    ');
      writeln(' Partitionstabelle im Partitionssektor     '+
              '                              ');
      writeln(''+
              '͹');
      writeln('                              Start    '+
              '     Ende     Entf.v.       ');
      writeln('NrBootTyp                Kopf Zyl. Sek.'+
              'Kopf Zyl. Sek.BootSekAnzahl ');
      writeln(''+
              '͹');
      for Entry:=1 to 4 do         { die Tabellen-Eintrge durchlaufen }
        with ParSec.PartTable[ Entry ] do
          begin
          write(' ', Entry, '');
          if Status = $80 then write (' Ja ')
                               else write ('Nein');
          write('');
          case PartTyp of                    { Partitionstyp auswerten }
            $00        : write('nicht belegt       ');
            $01        : write('DOS, 12-Bit-FAT    ');
            $02 or $03 : write('XENIX              ');
            $04        : write('DOS, 16-Bit-FAT    ');
            $05        : write('DOS, extended Part.');
            $06        : write('DOS 4.0 > 32 MB    ');
            $DB        : write('Concurrent DOS     ');
            else         write('unbekannt (',PartTyp:3,')    ');
          end;
          GetSecZyl( StartSec.SecZyl, Sektor, Zylinder );
          write('', StartSec.Kopf:2,' ',Zylinder:5,'  ',Sektor:3 );
          GetSecZyl( EndSec.SecZyl, Sektor, Zylinder );
          write(' ', EndSec.Kopf:2,' ',Zylinder:5,'  ',Sektor:3 );
          writeln(' ', SecOfs:7,'', SecAnz:7,'');
        end;
      writeln(''+
              'ͼ'#13#10);
    end
  else
    writeln('Fehler beim Zugriff auf den Bootsektor!');
end;

{***********************************************************************
*                          H A U P T P R O G R A M M                   *
***********************************************************************}

var Laufwerk,                              { Variablen zur Umrechnung  }
    Fehler    : integer;                   { des bergebenen Arguments }

begin
  writeln( #13#10' FIXPARTP - (c)',
           ' 1989, 92 by MICHAEL TISCHER ' );
  Laufwerk := 0;                        { Default ist erste Festplatte }
  if ParamCount = 1 then                 { anderes Laufwerk angegeben? }
    begin                                                         { Ja }
      val( ParamStr(1), Laufwerk, Fehler );       { Ascii nach Dezimal }
      if Fehler <> 0 then                     { Fehler bei Umwandlung? }
        begin                                                     { Ja }
          writeln(#13#10'Ungltige Laufwerksnummer!');
          exit;                                     { Programm beenden }
        end;
    end;
  ShowPartition( Laufwerk );               { Partitionssektor anzeigen }
end.
