{********************************************************************}
{                          F M . P A S                               }
{--------------------------------------------------------------------}
{ Aufgabe        : Demonstriert den Zugriff auf die FM-Synthese der  }
{                  SoundBlaster-Karte.                               }
{--------------------------------------------------------------------}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 06.02.1994                                        }
{ letztes Update : 08.10.1994                                        }
{********************************************************************}

{$define DSP_VERSIONONLY}           { Nur Versionskontrolle benutzen }
Uses crt,WIN,SBUTIL,FMUTIL,DSPUTIL,ARGS,DOS;

Type
TASTE = Record
  cKey  : char;                                 { Taste auf Keyboard }
  iNote : Integer;                               { zu spielende Note }
  bOct  : Byte;
End;

CHDATA = Record
  iOctave,
  iFrequency : Integer;
  iFM        : Boolean;
  iFeedBack  : Integer;
End;

OSZIDATA = Record
  iAttack,
  iDecay,
  iSustain,
  iRelease   : Integer;
  iShortADSR,
  iContADSR,
  iVibrato,
  iTremolo   : Boolean;
  iMute,
  iHiMute,
  iFRQ,
  iWave      :Integer
End;

Const
 QWERTZ : Array[0..13] of TASTE= ( ( cKey:'Q'; iNote:_c; bOct:0 ),
                                   ( cKey:'W'; iNote:_d; bOct:0 ),
                                   ( cKey:'E'; iNote:_e; bOct:0 ),
                                   ( cKey:'R'; iNote:_f; bOct:0 ),
                                   ( cKey:'T'; iNote:_g; bOct:0 ),
                                   ( cKey:'Z'; iNote:_a; bOct:0 ),
                                   ( cKey:'U'; iNote:_h; bOct:0 ),

                                   ( cKey:'A'; iNote:_c; bOct:1 ),
                                   ( cKey:'S'; iNote:_d; bOct:1 ),
                                   ( cKey:'D'; iNote:_e; bOct:1 ),
                                   ( cKey:'F'; iNote:_f; bOct:1 ),
                                   ( cKey:'G'; iNote:_g; bOct:1 ),
                                   ( cKey:'H'; iNote:_a; bOct:1 ),
                                   ( cKey:'J'; iNote:_h; bOct:1 ) );

var
 SBB                : SBBASE;
 screenbuf          : ^Byte;
 wind, tast, screen : WINDOW ;
 Ints               : Array[0..14] of INTDATA ;
 Bools              : Array[0..5] of BOOLDATA;
 Objects            : Array[0..19] of OBJ;

 OPLOBJ,                       { 'Konstanten' um auf Eingaben direkt }
 MODEOBJ,                      { reagieren zu knnen.                }
 MODEINT,
 MODEBOOL,
 MODEY,
 OPL3_4OPOBJ,
 CHANNELOBJ,
 OSZILATOROBJ,
 FRQOBJ,
 FRQINT,
 OCTAVEOBJ,
 OCTAVEINT,
 MAXOBJ         : Integer;

 OD     : OSZIDATA;
 ALLOSZ : Array[0..1,0..17] of OSZIDATA;

 CH     : CHDATA;
 ALLCH  : Array[0..1,0..8] of CHDATA;

 iOpl,                                         { Aktueller Oszilator }
 iChannel,                                         { Aktueller Kanal }
 iOszilator : Integer;
 iOPL3,                                   { liegt ein OPL3-Chip vor? }
 iOPL3_4OP  : Boolean;                              { 4 Operatoren ? }

 i, j, y, o, int, bool : Integer;

{********************************************************************}
{ Func : Eigene Benachrichtigungsfunktion                            }
{*------------------------------------------------------------------*}
{ Eingabe : iAct   - OBJECT auf das sich Nachricht bezieht           }
{           iMsg   - Nachrichtentyp                                  }
{           iParam - Integer-Parameter                               }
{           lParam - Long-Parameter                                  }
{********************************************************************}
{$f+}
Procedure Func( iAct, iMsg, iParam : Integer; lParam : Longint );

var i,
    old : Integer;

Begin
  if iMsg = MSG_CHANGED then
    Begin
      fm_SetOszilator( iOpl,                         { Modulator }
                       iOszilator,
                       OD.iAttack,
                       OD.iDecay,
                       OD.iSustain,
                       OD.iRelease,
                       Byte ( OD.iShortADSR ),
                       Byte ( OD.iContADSR ),
                       Byte ( OD.iTremolo ),
                       Byte ( OD.iVibrato ),
                       OD.iMute,
                       OD.iHiMute,
                       OD.iFRQ,
                       OD.iWave );

      if iAct = OPLOBJ then                             { OPL wechsel }
        Begin
          ALLCH[lParam, iChannel] := CH;        { aktuelle Daten retten }
          ALLOSZ[lParam ,iOszilator] := OD;

          CH := ALLCH[iOpl, iChannel];               { neue Daten laden }
          iOszilator := fm_GetModulator( iChannel );
          OD := ALLOSZ[iOpl, fm_GetModulator( iOszilator )];

          win_ObjectPrintArray( wind, @Objects, MAXOBJ, iAct );
        End;

      if iAct = OPL3_4OPOBJ then              { Anzahl der Operatoren }
        Begin
          if iOPL3_4OP then
            Begin
              CH.iFM := FALSE;
              win_ObjectInitINT( Objects[MODEOBJ],
                                 Ints[MODEINT], 0, MODEY, 100, 1,
                                 'Zellverknpfung     :', 0, 3, @CH.iFM );
              fm_QuadroChannel( FM_FIRSTOPL2, TRUE, TRUE, TRUE );
              fm_QuadroChannel( FM_SECNDOPL2, TRUE, TRUE, TRUE );
            End
          else
            Begin
              CH.iFM := TRUE;
              win_ObjectInitBOOL( Objects[MODEOBJ],
                                  Bools[MODEBOOL], 0, MODEY, 100, 1,
                                  'FM-Synthese         : ',
                                  DT_YESNO, @CH.iFM );
              fm_QuadroChannel( FM_FIRSTOPL2, FALSE, FALSE, FALSE );
              fm_QuadroChannel( FM_SECNDOPL2, FALSE, FALSE, FALSE );
            End;
          win_ObjectPrintArray( wind, @Objects, MAXOBJ, iAct );
        End;

      if iAct = OSZILATOROBJ then                    { Oszilatorwechsel }
        Begin
          ALLOSZ[iOpl ,lParam] := OD;
          OD := ALLOSZ[iOpl , iOszilator];
          CH := ALLCH[iOpl , fm_GetChannel( iOszilator )];
          iChannel := fm_GetChannel( iOszilator );

          win_ObjectPrintArray( wind, @Objects, MAXOBJ, iAct );
        End;

      if iAct = CHANNELOBJ then                         { Kanalwechsel }
        Begin
          ALLCH[iOpl ,lParam] := CH;
          CH := ALLCH[iOpl , iChannel];
          iOszilator := fm_GetModulator( iChannel );
          OD := ALLOSZ[iOpl , fm_GetModulator( iOszilator )];

          win_ObjectPrintArray( wind, @Objects, MAXOBJ, iAct );
        End;

      fm_SetOszilator( iOpl, iOszilator,
                       OD.iAttack, OD.iDecay,
                       OD.iSustain, OD.iRelease,
                       Byte ( OD.iShortADSR ), Byte ( OD.iContADSR ),
                       Byte ( OD.iTremolo ), Byte ( OD.iVibrato ),
                       OD.iMute, OD.iHiMute,
                       OD.iFRQ, OD.iWave );
                   { aktuellen Zustand von Kanal/Oszilator speichern }
      ALLOSZ[iOpl ,iOszilator] := OD;
      ALLCH[iOpl ,iChannel] := CH;
    End;

  if iMsg = MSG_KEY then                               { Tastendruck }
    Begin
      fm_SetChannel( iOpl, iChannel,             { Kanalnummer }
                     Byte ( ALLCH[iOpl ,iChannel].iOctave ),
                     Byte ( ALLCH[iOpl,iChannel].iFrequency ),
                     Byte ( ALLCH[iOpl,iChannel].iFM ),
                     ALLCH[iOpl,iChannel].iFeedBack );

      case iParam of
        KBD_F1:
          begin
            fm_PlayChannel( iOpl, 6, FALSE  );
            fm_PlayBassDrum( iOpl, FALSE );
            fm_PlayBassDrum( iOpl, TRUE );
          end;

        KBD_F2:
          begin
            fm_PlayChannel( iOpl, 7, FALSE  );
            fm_PlayHiHat( iOpl, FALSE );
            fm_PlayHiHat( iOpl, TRUE );
          end;

      KBD_F3:
         begin
           fm_PlayChannel( iOpl, 8, FALSE  );
           fm_PlayTomTom( iOpl, FALSE );
           fm_PlayTomTom( iOpl, TRUE );
         end;

      KBD_F4:
         begin
           fm_PlayChannel( iOpl, 7, FALSE  );
           fm_PlaySnareDrum( iOpl, FALSE );
           fm_PlaySnareDrum( iOpl, TRUE );
         end;

      KBD_F5:
         begin
           fm_PlayChannel( iOpl, 8, FALSE  );
           fm_PlayTopCymbal( iOpl, FALSE );
           fm_PlayTopCymbal( iOpl, TRUE );
         end;

      else
        Begin
          fm_PlayChannel( iOpl, iChannel, FALSE );
          for i := 0 to sizeof( QWERTZ ) div sizeof( TASTE ) - 1 do
            if UpCase( char( iParam ) ) = QWERTZ[i].cKey then
              Begin
                fm_SetChannel( iOpl, iChannel,             { Kanalnummer }
                               ( QWERTZ[i].bOct + Byte ( ALLCH[iOpl ,iChannel].iOctave ) ),
                               QWERTZ[i].iNote, Byte ( ALLCH[iOpl,iChannel].iFM ),
                               ALLCH[iOpl,iChannel].iFeedBack );
                fm_PlayChannel( iOpl, iChannel, TRUE );

                {-- Display an neue Werte anpassen! --------------------------}
                win_LoVideo( wind );
                if iAct = OCTAVEOBJ then win_HiVideo( wind );
                old := Ints[OCTAVEINT].pValue^;
                Ints[OCTAVEINT].pValue^ := QWERTZ[i].bOct + Byte ( CH.iOctave );
                win_ObjectPrint( wind, Objects[OCTAVEOBJ] );
                Ints[OCTAVEINT].pValue^ := old;
                win_LoVideo( wind );

                if iAct = FRQOBJ then win_HiVideo( wind );
                Ints[FRQINT].pValue^ := QWERTZ[i].iNote;
                win_ObjectPrint( wind, Objects[FRQOBJ] );
              End;
        End;
    End;
  End;
End;
{$f-}

{--------------------------------------------------------------------}
{-- H A U P T P R O G R A M M                                      --}
{--------------------------------------------------------------------}

Begin
  if sb_GetEnviron( SBB, GetEnv( 'BLASTER' ) ) = NO_ERROR then
    fm_SetBase( SBB, TRUE )
  else
    if fm_GetAdLib( SBB ) <> NO_ERROR then
      Begin
        Writeln( 'BLASTER-Umgebungsvariable nicht vorhanden ');
        Writeln( 'und keine AdLib kompatible Soundkarte gefunden! ');
        Halt( 0 );
      End;


  iOPL3 := fm_QuadroOn( TRUE );
  iOPL3_4OP := FALSE;                    {-- 4 Operatoren mglich ? --}

  for j := 0 to 1 do
    Begin
      for i := 0 to 8 do
        Begin
          ALLCH[j,i].iOctave := 0;
          ALLCH[j,i].iFrequency := 0;
          ALLCH[j,i].iFM := TRUE;
          ALLCH[j,i].iFeedBack := 0;
          fm_SetChannel( j, i, ALLCH[j,i].iOctave,
                         ALLCH[j,i].iFrequency,
                         Byte ( ALLCH[j,i].iFM ),
                         ALLCH[j,i].iFeedBack );
                                     { Kanle den Ausgngen zuweisen }
          if iOPL3 then fm_ChannelLR( j, i, Boolean ( j ), not Boolean( j ) );
        End;
      for i := 0 to 17 do
        Begin
          ALLOSZ[j,i].iAttack := 10;
          ALLOSZ[j,i].iDecay := 3;
          ALLOSZ[j,i].iSustain := 15;
          ALLOSZ[j,i].iRelease := 6;
          ALLOSZ[j,i].iShortADSR := TRUE;
          ALLOSZ[j,i].iContADSR := FALSE;
          ALLOSZ[j,i].iTremolo := TRUE;
          ALLOSZ[j,i].iVibrato := TRUE;
          ALLOSZ[j,i].iMute := 0;
          ALLOSZ[j,i].iHiMute := 0;
          ALLOSZ[j,i].iFRQ := 7;
          ALLOSZ[j,i].iWave := 1;

          fm_SetOszilator( j, i,
                           ALLOSZ[j,i].iAttack,
                           ALLOSZ[j,i].iDecay,
                           ALLOSZ[j,i].iSustain,
                           ALLOSZ[j,i].iRelease,
                           Byte ( ALLOSZ[j,i].iShortADSR ),
                           Byte ( ALLOSZ[j,i].iContADSR ),
                           Byte ( ALLOSZ[j,i].iTremolo ),
                           Byte ( ALLOSZ[j,i].iVibrato ),
                           ALLOSZ[j,i].iMute,
                           ALLOSZ[j,i].iHiMute,
                           ALLOSZ[j,i].iFRQ,
                           ALLOSZ[j,i].iWave );
        End;
    End;

  iOpl := 0;
  iChannel := 0;
  iOszilator := 0;

  OD := ALLOSZ[iOpl , 0];
  CH := ALLCH[iOpl , 0];

  o := 0;
  y := 0;
  int := 0;
  bool := 0;

  OPLOBJ := -1;
  if iOPL3 then
    Begin
      OPLOBJ := o;
      win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                         'OPL-Nr              :', 0, 1, @iOpl );
      Inc( o );Inc( int );Inc( y );

      OPL3_4OPOBJ := o;
      win_ObjectInitBOOL( Objects[o], Bools[bool], 0, y, 100, 1,
                          '4-Operatoren        : ', DT_YESNO, @iOPL3_4OP );
      Inc( o );Inc( bool );Inc( y, 2 );
    End;

  CHANNELOBJ := o;
  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                     'Channel        (0-8):', 0, 8, @iChannel );
  Inc( o );Inc( int );Inc( y );

  OCTAVEOBJ := o;
  OCTAVEINT := int;
  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                     'Oktave         (0-8):', 0, 8, @CH.iOctave );
  Inc( o );Inc( int );Inc( y );

  FRQOBJ := o;
  FRQINT := int;
  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                     'Frequenz    (0-1023):', 0, 1023, @CH.iFrequency );
  Inc( o );Inc( int );Inc( y );

  MODEOBJ := o;
  MODEY := y;
  MODEINT := int;
  MODEBOOL := bool;

  if iOPL3_4OP then
    win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                       'Zellverknpfung     :', 0, 3, @CH.iFM )
  else
    win_ObjectInitBOOL( Objects[o], Bools[bool], 0, y, 100, 1,
                        'FM-Synthese         : ', DT_YESNO, @CH.iFM );
  Inc( o );Inc( y );Inc( bool );Inc( int );

  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                    'Feedback      (0- 3):', 0, 3, @CH.iFeedBack );
  Inc( o );Inc( int );Inc( y,2 );
  
  OSZILATOROBJ := o;
  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                     'Oszilator     (0-17):', 0, 17, @iOszilator );
  Inc( o );Inc( int );Inc( y );

  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                     'Attack        (0-15):', 0, 15, @OD.iAttack );
  Inc( o );Inc( int );Inc( y );

  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                     'Decay         (0-15):', 0, 15,@OD.iDecay );
  Inc( o );Inc( int );Inc( y );

  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                     'Sustain       (0-15):', 0, 15, @OD.iSustain );
  Inc( o );Inc( int );Inc( y );

  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                     'Release       (0-15):', 0, 15, @OD.iRelease );
  Inc( o );Inc( int );Inc( y );

  win_ObjectInitBOOL( Objects[o], Bools[bool], 0, y, 100, 1,
                      'Short-ADSR          :  ', DT_ONOFF, @OD.iShortADSR );
  Inc( o );Inc( bool );Inc( y );

  win_ObjectInitBOOL( Objects[o], Bools[bool], 0, y, 100, 1,
                      'Cont.-ADSR          :  ', DT_ONOFF, @OD.iContADSR );
  Inc( o );Inc( bool );Inc( y );

  win_ObjectInitBOOL( Objects[o], Bools[bool], 0, y, 100, 1,
                      'Vibrato             :  ', DT_ONOFF, @OD.iVibrato );
  Inc( o );Inc( bool );Inc( y );

  win_ObjectInitBOOL( Objects[o], Bools[bool], 0, y, 100, 1,
                      'Tremolo             :  ', DT_ONOFF, @OD.iTremolo );
  Inc( o );Inc( bool );Inc( y );

  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                     'Mute          (0-63):', 0, 63, @OD.iMute );
  Inc( o );Inc( int );Inc( y );

  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                     'Hi-Mute       (0- 3):', 0, 3, @OD.iHiMute );
  Inc( o );Inc( int );Inc( y );

  win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                     '* Frq         (0-15):', 0, 15, @OD.iFRQ );
  Inc( o );Inc( int );Inc( y );
  if iOPL3 then
    win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                       'Wave          (0- 7):', 0, 7, @OD.iWave )
  else
    win_ObjectInitINT( Objects[o], Ints[int], 0, y, 100, 1,
                       'Wave          (0- 3):', 0, 3, @OD.iWave );
  Inc( o );Inc( int );Inc( y );

  MAXOBJ := o;
  win_GetScreenSettings( screen );
  screenbuf := win_Save( screen );
  win_Clr( screen );

  win_Init( wind, 1, 1, 30, 22, $1F, $71, WIN_ACTIVE );
  win_Clr( wind );

  win_Init( tast, 32, 1, 44, 22, $1F, $17, WIN_ACTIVE );
  win_Clr( tast );

  win_print( tast, '                   '+#10 );
  win_print( tast, '                   '+#10 );
  win_print( tast, '                   '+#10 );
  win_print( tast, '                   '+#10 );
  win_print( tast, '    q w e r t z u   a s d e f g h'+#10 );
  win_print( tast, '      1. Oktave       2. Oktave'+#10  );
  win_print( tast, #10 );
  win_print( tast, 'F1 - BassDrum   F2 - HiHat  F3 - TomTom'+#10 );
  win_print( tast, 'F4 - SnareDrum  F5 - TopCymbal'+#10#10 );
  win_print( tast, 'Navigation: '+#10);
  win_print( tast, '* <CURSOR OBEN> & <CURSOR UNTEN>'+#10);
  win_print( tast, '  Auswahl der Einstellung'+#10#10 );
  win_print( tast, '* <+> & <->'+#10);
  win_print( tast, '  anderer Wert'+#10#10 );
  win_print( tast, '<ESC> fr Ende'+#10#10 );
  win_print( tast, 'Micro-Piano (c)'+#10+' Michael Tischer &'+#10 );
  win_print( tast, ' Bruno Jennrich');
                                        { Percussionmodus aktivieren }
  fm_PercussionMode( 0, TRUE );
  fm_PercussionMode( 1, TRUE );

  win_ObjectProcessArray( wind, @Objects, MAXOBJ, Func );

  fm_Reset;                                       { komplette Stille }
  win_Restore( screenbuf, TRUE );
End.

