{**********************************************************************}
{                              J O Y S T P                             }
{----------------------------------------------------------------------}
{    Aufgabe        : Demonstriert die Abfrage des Joysticks  mit      }
{                     Hilfe des BIOS                                   }
{----------------------------------------------------------------------}
{    Autor          : MICHAEL TISCHER                                  }
{    entwickelt am  : 25.02.1991                                       }
{    letztes Update : 25.02.1991                                       }
{**********************************************************************}

program JOYSTP;

uses DOS, CRT;

{== Typdeklarationen ==================================================}

type JSPOS = record          { beschreibt die Position eines Joysticks }
                x,
                y : integer;
               end;

{== Konstanten ========================================================}

const CRLF = #13#10;

{== globale Variablen =================================================}

var jsp        : array [1..2] of JSPOS;   { aktuelle Joystick-Position }
    maxx, maxy,                           { maximale Joystick-Position }
    minx, miny,                           { minimale Joystick-Position }
    x, y,                               { aktuelle Bildschirm-Position }
    xold, yold : integer;                 { letzte Bildschirm-Position }
    aktstick   : byte;                         { ausgewhlter Joystick }
    j1but,                             { Button 1 auf Joystick 1 und 2 }
    j2but      : array[1..2] of byte;  { Button 2 auf Joystick 1 und 2 }
    xfaktor,                             { Umrechnungsfaktoren X und Y }
    yfaktor    : real;

{***********************************************************************
*  GetJoyButton:  Liefert die Stellung der Joystick-Buttons            *
**--------------------------------------------------------------------**
*  Eingabe-Parameter: J1B1 = Variable fr Button 1 / Joystick 1        *
*                     J1B2 = Variable fr Button 2 / Joystick 1        *
*                     J2B1 = Variable fr Button 1 / Joystick 2        *
*                     J2B2 = Variable fr Button 2 / Joystick 2        *
*  Info             : die verschiedenen Variablen nehmen den Wert 1    *
*                     (TRUE) auf, wenn der korrespondierende Joystick- *
*                     Button niedergedrckt ist, sonst 0 (FALSE).      *
**********************************************************************}

procedure GetJoyButton( var j1b1, j1b2, j2b1, j2b2 : byte );

var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }

begin
  Regs.ah := $84;                                       { Funktion 84h }
  Regs.dx := 0;                                    { Unterfunktion 00h }
  intr( $15, Regs );                          { Interrupt 15h aufrufen }
  j1b1 := (( Regs.al and  16 ) shr 4) xor 1;     { Bit 4 von AL = J1B1 }
  j1b2 := (( Regs.al and  32 ) shr 5) xor 1;     { Bit 5 von AL = J1B2 }
  j2b1 := (( regs.al and  64 ) shr 6) xor 1;     { Bit 6 von AL = J2B1 }
  j2b2 := (( regs.al and 128 ) shr 7) xor 1;     { Bit 7 von AL = J2B2 }
end;

{***********************************************************************
*  GetJoyPos : Liefert die Position der beiden Joysticks               *
**--------------------------------------------------------------------**
*  Eingabe-Parameter: JS1 = Joystick-Struktur fr ersten Joystick      *
*                     JS2 = Joystick-Struktur fr zweiten Joystick     *
***********************************************************************}

procedure GetJoyPos( var Js1, Js2 : JSPOS );

var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }

begin
  Regs.ah := $84;                                       { Funktion 84h }
  Regs.dx := 1;                                    { Unterfunktion 01h }
  intr( $15, Regs );                          { Interrupt 15h aufrufen }
  Js1.x := Regs.ax;                            { X-Position Joystick 1 }
  Js1.y := Regs.bx;                            { Y-Position Joystick 1 }
  Js2.x := Regs.cx;                            { X-Position Joystick 2 }
  Js2.y := Regs.dx;                            { Y-Position Joystick 2 }
end;

{**********************************************************************}
  {*                           HAUPTPROGRAMM                          *}
{**********************************************************************}

begin
 {-- zunchst maximale Joystick-Stellung ermitteln --------------------}

 ClrScr;
 writeln( 'Bitte bewegen Sie den Joystick in die obere rechte ' +
         'Schalter-' + CRLF + 'stellung und bettigen Sie einen ' +
         'der beiden Buttons' );

 repeat                   { auf Bettigung einer Joystick-Taste warten }
  GetJoyButton( j1but[1], j2but[1], j1but[2], j2but[2] );
 until ( ( j1but[1] or j2but[1] or j1but[2] or j2but[2] ) <> 0 );

 if ( j1but[1] or j2but[1] ) <> 0 then            { Joystick auswhlen }
   aktstick := 1
 else
   aktstick := 2;

 GetJoyPos( jsp[1], jsp[2] );                      { Position abfragen }
 maxx := jsp[aktstick].x;                          { und merken        }
 miny := jsp[aktstick].y;

 repeat                         { warten, bis Taste wieder losgelassen }
  GetJoyButton( j1but[1], j2but[1], j1but[2], j2but[2] );
 until ( ( j1but[aktstick] or j2but[aktstick] ) = 0 );

 {-- jetzt minimale Position ermitteln --------------------------------}

 writeln( CRLF + CRLF + 'Bitte bewegen Sie den Joystick jetzt in'+
          ' die untere linke Schalter-' + CRLF + 'stellung und ' +
          'bettigen Sie einen der beiden Buttons' );

 repeat                     { wieder auf Bettigung einer Taste warten }
  GetJoyButton( j1but[1], j2but[1], j1but[2], j2but[2] );
 until ( ( j1but[aktstick] or j2but[aktstick] ) <> 0 );

 GetJoyPos( jsp[1], jsp[2] );                      { Position abfragen }
 minx := jsp[aktstick].x;                          { und merken        }
 maxy := jsp[aktstick].y;

 xfaktor := 80.0 / ( maxx - minx + 1 );     { Umrechnungsfaktoren fr  }
 yfaktor := 23.0 / ( maxy - miny + 1 );     { X- und Y-Achse berechnen }

 {-- Joystick abfragen und Joystick-Position anzeigen, bis beide ------}
 {-- Tasten bettigt werden                                      ------}

 ClrScr;
 GotoXY( 45, 1 );
 write( 'JOYSTP - (c) 1991 by MICHAEL TISCHER' );
 GotoXY( 1, 25 );
 write( 'Bettigen Sie beide Joystick-Tasten, um das Programm ' +
        'zu beenden' );

 xold := 0;                                   { alte Position vorgeben }
 yold := 0;

 repeat
   GetJoyPos( jsp[1], jsp[2] );                    { Position abfragen }

   {-- neue X-Position des Joysticks berechnen ------------------------}

   x := round(xfaktor * ( jsp[aktstick].x - minx + 1 ));
   if ( x < 0 ) then
     x := 0;
   if ( x > 79 ) then
     x := 79;

   {-- neue Y-Position des Joysticks berechnen ------------------------}

   y := round(yfaktor * ( jsp[aktstick].y - miny + 1 ));
   if ( y < 0 ) then
     y := 0;
   if ( y > 22 ) then
     y := 22;

   {-- bei vernderter Position neue Position anzeigen ----------------}

   if ( x <> xold ) or ( y <> yold ) then
     begin
       GotoXY( xold+1, yold+2 );
       write( ' ' );
       GotoXY( x+1, y+2 );
       write( 'X' );
       xold := x;
       yold := y;
     end;


   GotoXY( 1, 1 );
   write( '(', jsp[aktstick].x:3, '/', jsp[aktstick].y:3, ')' );
   GetJoyButton( j1but[1], j2but[1], j1but[2], j2but[2] );
 until ( j1but[aktstick] = 1 ) and ( j2but[aktstick] = 1 );
 ClrScr;
 GotoXY( 1, 1 );
 writeln( 'Das war''s\n' );
end.
