{**********************************************************************}
{*                                L E D P                             *}
{*--------------------------------------------------------------------*}
{*    Aufgabe        : setzt die verschiedenen Bits im Tastatur-      *}
{*                     Status-Byte des BIOS und lt dadurch die      *}
{*                     LEDs auf der AT-Tastatur aufleuchten           *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  : 16.08.1988                                     *}
{*    letztes Update :  3.01.1992                                     *}
{**********************************************************************}
                                                                        
program LEDP;
                                                                        
uses CRT,                                     { die CRT-Unit einbinden }
     DOS;                                     { die DOS-Unit einbinden }
                                                                        
const SCRL =  16;                                    { Scroll-Lock-Bit }
      NUML =  32;                                       { Num-Lock-Bit }
      CAPL =  64;                                      { Caps-Lock-Bit }
      INS  = 128;                                         { Insert-Bit }
                                                                        
{**********************************************************************}
{* SETFLAG: setzt eines der Flags im BIOS-Tastatur-Status-Byte        *}
{* Eingabe : das zu setzende Flag (siehe Konstanten)                  *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure SetFlag(Flag : byte);
                                                                        
var BiosTSByte : byte absolute $0040:$0017;{ BIOS-Tastatur-Status-Byte }
    Regs       : Registers;   { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
  BiosTSByte := BiosTSByte or Flag; { das entsprechende Bit einblenden }
  Regs.AH := 1;                     { Funktionsnr.: Zeichen vorhanden? }
  intr($16, Regs);                  { BIOS-Tastatur-Interrupt aufrufem }
end;

{**********************************************************************}
{* CLRFLAG: lscht eines der Flags im BIOS-Tastatur-Status-Byte       *}
{* Eingabe : das zu lschende Flag (siehe Konstanten)                 *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure ClrFlag(Flag : byte);
                                                                        
var BiosTSByte : byte absolute $0040:$0017;{ BIOS-Tastatur-Status-Byte }
    Regs       : Registers;   { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
  BiosTSByte := BiosTSByte and ( not Flag );          { Bit ausblenden }
  Regs.AH := 1;                     { Funktionsnr.: Zeichen vorhanden? }
  intr($16, Regs);                  { BIOS-Tastatur-Interrupt aufrufem }
end;
                                                                        
{**********************************************************************}
{**                           HAUPTPROGRAMM                          **}
{**********************************************************************}
                                                                        
var zaehler : integer;
                                                                        
begin
  writeln('LEDP  -  (c) 1988, 92 by Michael Tischer');
  writeln(#13,#10, 'Achten Sie auf die LEDs Ihrer Tastatur!');
                                                                        
  for zaehler:=1 to 10 do            { die Schleife 10 mal durchlaufen }
    begin
      SetFlag( CAPL);                                { CAPS anschalten }
      Delay( 100 );                         { 100 Millisekunden warten }
      ClrFlag( CAPL );                       { CAPS wieder ausschalten }
      SetFlag( NUML);                                 { NUM anschalten }
      Delay( 100 );                         { 100 Millisekunden warten }
      ClrFlag( NUML );                        { NUM wieder ausschalten }
      SetFlag( SCRL);                         { SCROLL-LOCK anschalten }
      Delay( 100 );                         { 100 Millisekunden warten }
      ClrFlag( SCRL );                { SCROLL-LOCK wieder ausschalten }
    end;
                                                                        
  for zaehler:=1 to 10 do            { die Schleife 10 mal durchlaufen }
    begin
      SetFlag(CAPL or SCRL or NUML);              { alle drei Flags an }
      Delay( 500 );                        { 200 Milli-Sekunden warten }
      ClrFlag(CAPL or SCRL or NUML);           { alle Flags wieder aus }
      Delay( 500 );                        { 200 Milli-Sekunden warten }
    end;
end.
