{**********************************************************************}
{                        M E M D E M O P . P A S                       }
{----------------------------------------------------------------------}
{    Aufgabe       : Demonstration der DOS-Speicherverwaltung          }
{----------------------------------------------------------------------}
{    Autor         : Michael Tischer                                   }
{    entwickelt am : 08.10.1991                                        }
{    letztes Update: 20.10.1991                                        }
{**********************************************************************}
{$M 8096, 0, 10240 }

program MEMDEMOP;

uses crt,                                     { die CRT-Unit einbinden }
     dos;                                     { die DOS-Unit einbinden }

{== Konstanten ========================================================}

const {-- Funktionsnummern fr Interrupt $21 --------------------------}

      GET_MEM      = $48;                   { RAM-Speicher reservieren }
      FREE_MEM     = $49;                     { RAM-Speicher freigeben }
      CHANGE_MEM   = $4A;        { Gre eines Speicherbereichs ndern }
      GET_STRATEGY = $5800;     { Konzept der Speicherverteilung lesen }
      SET_STRATEGY = $5801;    { Konzept der Speicherverteilung setzen }
      GET_UMB      = $5802;                     { UMB-Einbindung lesen }
      SET_UMB      = $5803;                    { UMB-Einbindung setzen }

      {-- Suchstrategien fr SetzeStrategie ---------------------------}

      SUCHE_UNTEN  = $00;       { Verwende ersten freien Speicherblock }
      SUCHE_BESTEN = $01;              { Verwende besten Speicherblock }
      SUCHE_OBEN   = $02;      { Verwende letzten freien Speicherblock }
      ZUERST_UMB   = $80;   { Zuerst im UMB suchen (in Kombination mit }
                            { SUCHE_.... zu verwenden)                 }

      {-- Konstanten fr SetzeUMB -------------------------------------}

      UMB_AUS = $00;                       { UMB nicht bercksichtigen }
      UMB_EIN = $01;                { Speichervergabe auch aus dem UMB }

      {-- Konstanten fr Demo -----------------------------------------}

      TEST_GR        = 10240-1;   { 10239 Paragraphen fr Testumgebung }
      TEST_GR_UMB    = 2560-1;  { 2559 Pragraphen UMB fr Testumgebung }
      TEST_GR_KB     = 160;             { Testumgebung ist 160 KB gro }
      TEST_GR_UMB_KB = 40;           { Testumgebung UMB ist 40 KB gro }
      BLOCKANZ       = 26;   { Anzahl der Adressen fr Ergebnisanzeige }

      {-- Tastencodes fr Bedienereingabe -----------------------------}

      ESC  = #27;                              { Abbruch mit ESC-Taste }
      F1   = #59;                                 { Funktionstaste F1  }
      F2   = #60;                                 { Funktionstaste F2  }
      F3   = #61;                                 { Funktionstaste F3  }
      F8   = #66;                                 { Funktionstaste F8  }
      F9   = #67;                                 { Funktionstaste F9  }
      F10  = #68;                                 { Funktionstaste F10 }

{== Typdeklarationen ==================================================}

type  BlockTyp = record       { Verwaltung eines allok. Speicherblocks }
                   Adresse,                           { Segmentadresse }
                   Groesse : word;    { Gre des Speicherblocks in KB }
                 end;

{== Typisierte Konstanten =============================================}

const JANein     : array[ false..true ] of string =
                   ( 'Nein', 'Ja  ' );
      SText      : array[ SUCHE_UNTEN..SUCHE_OBEN ] of string =
                   ( 'ersten freien Speicherblock verwenden ',
                     'besten freien Speicherblock verwenden ',
                     'letzten freien speicherblock verwenden' );
      FarbFeld   : array[ 0..1 ] of byte = ( $07, $70 );
      TastenText : array[ 0..3 ] of string =
                   ( ' [F1] Speicher allokieren',
                     ' [F2] Speicher freigeben',
                     ' [F3] Gre ndern',
                     ' [ESC] Programmende' );

{== Globale Variablen =================================================}

var Regs       : Registers;            { Register fr Interrupt-Aufruf }
    AdressFeld : array[ 0..1000 ] of word;         { Feld fr Speicher }
    AnzAdress  : word;                               { Anzahl Adressen }
    KonvSeg    : word; { Adresse Testblock im konventionellen Speicher }
    UMBSeg     : word;                  { Adresse Testblock in den UMB }
    BlockFeld  : array[ 0..BLOCKANZ - 1 ] of BlockTyp;      { Speicher }

{**********************************************************************}
{ DOS_GetMem : Reserviert Speicher                                     }
{ Eingabe    : Gewnschte Speichergre in Paragraphen                 }
{ Ausgabe    : Segmentadresse des allokierten Speicherblocks,          }
{              Anzahl der allokierten Paragraphen oder Anzahl der      }
{              maximal verfgbaren Paragraphen                         }
{**********************************************************************}

procedure DOS_GetMem(     Gr  : word;
                      var Adr : word;
                      var Erg : word );

begin
  with Regs do
    begin
      ah := GET_MEM;                                 { Funktionsnummer }
      bx := Gr;             { Anzahl der zu reservierenden Paragraphen }
      msdos( Regs );                                { Interrupt-Aufruf }
      if ( flags and fcarry = 0 ) then           { Aufruf erfolgreich? }
        begin
          Adr := regs.ax;          { Ja, Adresse und Gre zurckgeben }
          Erg := Gr;
        end
      else                                              { Nein, Fehler }
        begin
          Adr := 0;                         { Kein Speicher reserviert }
          Erg := bx;                        { maximal verfgbare Gre }
        end;
    end;
end;

{**********************************************************************}
{ DOS_FreeMem: Gibt reservierten Speicher frei                         }
{ Eingabe    : Segmentadresse des Speichers                            }
{ Ausgabe    : keine                                                   }
{**********************************************************************}

procedure DOS_FreeMem( Adr : word );

begin
  with Regs do
    begin
      ah := FREE_MEM;                                { Funktionsnummer }
      es := Adr;                { Adresse des freizugebenden Speichers }
      msdos( Regs );                                { Interrupt-Aufruf }
    end;
end;

{**********************************************************************}
{ DOS_ChangeMem : ndert reservierte Speichergre                     }
{ Eingabe       : Alte Segementadresse, neue gewnschte Speichergre  }
{                 in Paragraphen                                       }
{ Ausgabe       : Segmentadresse des allokierten Speicherblocks        }
{                 Anzahl der allokierten Paragraphen oder Anzahl der   }
{                 maximal verfgbaren Paragraphen                      }
{**********************************************************************}

procedure DOS_ChangeMem(     Gr  : word;
                         var Adr : word;
                         var Erg : word );
begin
  with Regs do
    begin
      ah := CHANGE_MEM;                              { Funktionsnummer }
      bx := Gr;             { Anzahl der zu reservierenden Paragraphen }
      es := Adr;      { Segmentadresse des zu ndernden Speicherblocks }
      msdos( Regs );                                { Interrupt-Aufruf }
      if ( flags and fcarry = 0 ) then           { Aufruf erfolgreich? }
        Erg := Gr                                 { neue Speichergre }
      else                                              { Nein, Fehler }
        Erg := bx;                  { maximal verfgbare Speichergre }
    end;
end;

{**********************************************************************}
{ LeseStrategie : Liest das Konzept der DOS-Speicherverwaltung         }
{ Eingabe       : keine                                                }
{ Ausgabe       : Konzept der Speicherverwaltung                       }
{**********************************************************************}

function LeseStrategie : integer;

begin
  with Regs do
    begin
      ax := GET_STRATEGY;                     { Funktionsnummer setzen }
      msdos( Regs );                                { Interrupt-Aufruf }
      LeseStrategie := ax;                          { Konzept ausgeben }
    end;
end;

{**********************************************************************}
{ LeseUMB       : Liest den Status der UMB-Einbindung                  }
{ Eingabe       : keine                                                }
{ Ausgabe       : UMB wird bei der Speicherwaltung bercksichtigt      }
{ Info          : Erst ab MS-DOS Version 5.0 verfgbar                 }
{**********************************************************************}

function LeseUMB : integer;

begin
  with Regs do
    begin
      ax := GET_UMB;                          { Funktionsnummer setzen }
      msdos( Regs );                                { Interrupt-Aufruf }
      LeseUMB := al;                                 { Status ausgeben }
    end;
end;

{**********************************************************************}
{ SetzeStrategie: Setzt das Konzept der DOS-Speicherverwaltung         }
{ Eingabe       : Neues Konzept                                        }
{ Ausgabe       : keine                                                }
{**********************************************************************}

procedure SetzeStrategie( Strategie : integer );

begin
  with Regs do
    begin
      ax := SET_STRATEGY;                     { Funktionsnummer setzen }
      bx := Strategie;
      msdos( Regs );                                { Interrupt-Aufruf }
    end;
end;

{**********************************************************************}
{ SetzeUMB      : Setzt den Status der UMB-Einbindung                  }
{ Eingabe       : Neuer Status der UMB-Einbindung                      }
{ Ausgabe       : keine                                                }
{ Info          : Erst ab MS-DOS Version 5.0 verfgbar                 }
{**********************************************************************}

procedure SetzeUMB( UMB : integer );

begin
  with Regs do
    begin
      ax := SET_UMB;                          { Funktionsnummer setzen }
      bx := UMB;
      msdos( Regs );                                { Interrupt-Aufruf }
    end;
end;

{**********************************************************************}
{ AllokiereSpeicher: Erzeugt die Testumgebung fr den Speichertest     }
{ Eingabe          : keine                                             }
{ Ausgabe          : keine                                             }
{**********************************************************************}

procedure AllokiereSpeicher;

var SegAdr     : word;      { Segmentadresse des allokierten Speichers }
    Versuch    : word;                      { gewnschte Speichergre }
    Groesse    : word;        { Gre des allokierten Speicherbereichs }

begin
  {-- 1. Testblock allokieren -----------------------------------------}

  SetzeUMB( UMB_AUS );         { Testblock im konventionellen Speicher }
  DOS_GetMem( TEST_GR, KonvSeg, Groesse );           { Testblock holen }
  if ( KonvSeg = 0 ) then                                    { Fehler? }
   exit;                   { Kein Testblock gefunden, Prozedur beenden }


  SetzeUMB( UMB_EIN );                          { Testblock in der UMB }
  SetzeStrategie( SUCHE_OBEN or ZUERST_UMB );
  DOS_GetMem( TEST_GR_UMB, UMBSeg, Groesse );
  if ( UMBSeg <> 0 ) and ( UMBSeg < $A000 ) then { Kein UMB vorhanden? }
    begin
      DOS_FreeMem( UMBSeg );                      { Speicher freigeben }
      UMBSeg := 0;                                       { Keine UMB's }
    end;

  {-- 2. restlichen konventionellen und UMB-Speicher in 1KB Blcken ---}
  {--    allokieren                                                 ---}

  Versuch := 63;         { Versuche zuerst 15 Paragaphen zu allokieren }
  AnzAdress := 0;  { Anzahl der allokierten 1 KB Blcke zunchst auf 0 }
  repeat
    DOS_GetMem( Versuch, SegAdr, Groesse );       { Speicher anfordern }
    if ( SegAdr <> 0 ) then                       { Speicher erhalten? }
      begin
        AdressFeld[ AnzAdress ] := SegAdr;            { Adresse merken }
        inc( AnzAdress );
      end;
  until ( SegAdr = 0 );                  { Gesamter Speicher allokiert }

  {-- 3. Testbcke wieder freigeben -----------------------------------}

  if ( KonvSeg > 0 ) then          { konnte Speicher allokiert werden? }
    begin                                                         { Ja }
      DOS_FreeMem( KonvSeg );
      dec( KonvSeg );                     { MCB ist auch frei geworden }
    end;
  if ( UMBSeg > 0 ) then                { konnte UMB allokiert werden? }
    begin                                                         { Ja }
      DOS_FreeMem( UMBSeg );
      dec( UMBSeg );                      { MCB ist auch frei geworden }
    end;
end;

{**********************************************************************}
{ SpeicherFreigeben: den bei ErmitteleSpeicher rerservierten Speicher  }
{                    wieder freigeben                                  }
{ Eingabe          : keine                                             }
{ Ausgabe          : keine                                             }
{ Globale Varibalen: AdressFeld/R                                      }
{**********************************************************************}

procedure SpeicherFreigeben;

var i : word;                                        { Schleifenzhler }

begin
  if ( AnzAdress > 0 ) then                { wurde Speicher allokiert? }
    for i := 0 to AnzAdress - 1 do     { Ja, Block fr Block freigeben }
      DOS_FreeMem( AdressFeld[ i ] );
end;

{**********************************************************************}
{ ErgebnisAnzeigen : zeigt die Speicherbelegung an                     }
{ Eingabe          : MITRAHMEN = TRUE, wenn auch der Rahmen ausgegeben }
{                                werden soll                           }
{ Ausgabe          : keine                                             }
{**********************************************************************}

procedure ErgebnisAnzeigen( MitRahmen : boolean );

var SFeld      : array[ 0..TEST_GR_KB - 1 ] of char;
    SFeld_UMB  : array[ 0..TEST_GR_UMB_KB - 1 ] of char;
    i,j        : word;                               { Schleifenzhler }
    Position   : word;                                 { Hilfsvariable }
    Sdummy     : string;
    AltZeichen : char;     { Speicher fr letztes ausgegebenes Zeichen }
    AktFarbe   : byte;                         { Aktuelle Ausgabefarbe }

begin
  fillchar( SFeld[ 0 ], TEST_GR_KB, #32 );    { Anzeigefelder initial. }
  fillchar( SFeld_UMB[ 0 ], TEST_GR_UMB_KB, #32 );

  {-- Speichertabelle fllen ------------------------------------------}

  for i := 0 to BLOCKANZ - 1 do
    begin
      if ( BlockFeld[ i ].Adresse > $A000 ) then                { UMB? }
        begin
          Position := ( BlockFeld[ i ].Adresse - UMBSeg ) div 64;
          for j := 0 to BlockFeld[ i ].Groesse div 64 do
            SFeld_UMB[ Position + j ] := chr( i + 65 );
        end
      else if ( BlockFeld[ i ].Adresse > 0 ) then
        begin
          Position := ( BlockFeld[ i ].Adresse - KonvSeg ) div 64;
          for j := 0 to BlockFeld[ i ].Groesse div 64 do
            SFeld[ Position + j ] := chr( i + 65 );
        end;
    end;

  {-- Rahmen der Speichertabelle zeichnen -----------------------------}

  if ( MitRahmen ) then
    begin
      writeln( 'Konventioneller Speicher:' );
      writeln( '                   1         2         3         4 ' );
      writeln( '          1        0         0         0         0 ' );
      writeln( 'ͻ' );
      for i := 0 to 3 do
        begin
          str( i * 40 : 3, Sdummy );
          writeln( ' ' + Sdummy +
                   ' KB                                         ',
                   TastenText[ i ] );
        end;
      writeln( 'ͼ' );
      if ( UMBSeg > 0 ) then
       begin
        writeln(#13#10'UMB:' );
        writeln('                   1         2         3         4 ');
        writeln('          1        0         0         0         0 ');
        writeln('ͻ');
        writeln('   0 KB                                         ');
        writeln('ͼ');
       end
      else
       writeln( #13#10, 'Keine UMB verfgbar' );
    end;

  {-- Tabelle fr konventionellen Speicher ausgeben -------------------}

  AltZeichen := #0;                     { letztes ausgegebenes Zeichen }
  AktFarbe := 0;                                 { letzte Ausgabefarbe }
  for i := 0 to 3 do
    for j := 0 to 39 do
      begin
        if ( AltZeichen <> SFeld[ i * 40 + j ] ) then   { Farbwechsel? }
          begin
            AktFarbe := ( AktFarbe + 1 ) mod 2;     { Neue Farbenummer }
            textcolor( FarbFeld[ AktFarbe ] shr 4 );
            textbackground( FarbFeld[ AktFarbe ] and $0F );
            AltZeichen := SFeld[ i * 40 + j ];     { Vergleichszeichen }
          end;
        gotoxy( j + 11, i + 11  );                  { Zeichen ausgeben }
        write( SFeld[ i * 40 + j ] );
      end;

  {-- Tabelle fr UMB ausgeben ----------------------------------------}

  if ( UMBSeg > 0 ) then
    begin
      for j := 0 to 39 do
        begin
          if ( AltZeichen <> SFeld_UMB[ j ] ) then           { Farbw.? }
            begin
              AktFarbe := ( AktFarbe + 1 ) mod 2;    { Neue Farbnummer }
              textcolor( FarbFeld[ AktFarbe ] shr 4 );
              textbackground( FarbFeld[ AktFarbe ] and $0F );
              AltZeichen := SFeld_UMB[  j ];       { Vergleichszeichen }
            end;
          gotoxy( j + 11, 21  );
          write( SFeld_UMB[ j ] );
          end;
    end;
  textcolor( $07 );
  textbackground( $00 );
end;

{**********************************************************************}
{ Demo          : Demonstration der Speicherverwaltung                 }
{ Eingabe       : UMB einbeziehen, UMB zuerst durchsuchen,             }
{                 Konzept der Speicherverteilung                       }
{ Ausgabe       : keine                                                }
{ Info          : UMB-Funktionen erst ab MS-DOS Version 5.0 verfgbar  }
{**********************************************************************}

procedure Demo( Mit_UMB    : boolean;
                UMB_zuerst : boolean;
                Strategie  : integer );


var i       : integer;                               { Schleifenzhler }
    Taste   : char;                                     { Eingabetaste }
    Zeiger  : char;      { Zeiger (A-Z) fr den Reservierung gewnscht }
    Versuch : word;                           { Gre der Reservierung }
    Groesse : word;
    sdummy  : string;

begin
  {-- Adressfeld initialisieren ---------------------------------------}

  for i := 0 to BLOCKANZ - 1 do            { noch kein Block allokiert }
    with BlockFeld[ i ] do
      begin
        Adresse := 0;             { Segment-Adresse des Speicherblocks }
        Groesse := 0;                       { Gre des Speicherblocks }
      end;

  gotoxy( 1, 7 );
  ErgebnisAnzeigen( TRUE );                 { Speichertabelle anzeigen }

  {-- Demonstrationsschleife ------------------------------------------}

  repeat
    {-- Gewhltes Konzept der Speicherverwaltung setzen ---------------}

    if ( Mit_UMB ) then                               { UMB verwenden? }
      SetzeUMB( UMB_EIN )
    else
      SetzeUMB( UMB_AUS );

    if ( UMB_zuerst ) then
      SetzeStrategie( Strategie or ZUERST_UMB )
    else
      SetzeStrategie( Strategie );

    {-- Aktuelles Konzept anzeigen ------------------------------------}

    gotoxy( 1, 3 );
    writeln( 'Konzept der Speicherverwaltung: ', SText[ Strategie ],
             '  [F8] ' );
    writeln( 'Zuerst in den UMB suchen      : ', JANein[ UMB_zuerst ],
             '                                    [F9]' );
    writeln( 'UMB verwenden                 : ', JaNein[ Mit_UMB ],
             '                                    [F10]' );
    writeln( '--------------------------------------------------',
             '----------------------------' );

    {-- Eingabe und Verarbeitung --------------------------------------}

    repeat until keypressed;                         { Warte auf Taste }
    Taste := readkey;                                    { Taste lesen }
    if ( ( Taste = #0 ) and ( keypressed ) ) then     { Funktionstaste }
     Taste := readkey;                           { 2. Tastencode holen }

    case Taste of
      F1 :                                  { Speicherblock allokieren }
        begin
          i := -1;             { noch keinen gltigen Block eingegeben }
          repeat
            gotoxy( 1, 23 );
            write( 'Welchen Block reservieren [ A-Z ]: ' );
            readln( Zeiger );
            Zeiger := upcase( Zeiger );
            if ( Zeiger >= 'A' ) and ( Zeiger <= 'Z' ) then
              if ( BlockFeld[ ord( Zeiger ) - 65 ].Adresse = 0 ) then
                i := ord( Zeiger ) - 65;
          until ( i <> - 1 );
          write( 'Wie viele KB sollen reserviert werden  : ' );
          readln( Versuch );
          Versuch := Versuch * 64 - 1;      { Umrechnen in Paragraphen }
          DOS_GetMem( Versuch, BlockFeld[ i ].Adresse,
                      BlockFeld[ i ].Groesse );
          if ( BlockFeld[ i ].Groesse <> Versuch ) then      { Fehler? }
            begin
              str( ( BlockFeld[ i ].Groesse + 1 ) div 64, sdummy );
              write( 'Es sind nur noch ' + sdummy + 'KB frei ' );
              repeat until keypressed;
              while keypressed do
              Taste := readkey;
              Taste := #0;
            end;
          gotoxy( 1, 23 );
          writeln( '                                             ' );
          writeln( '                                             ' );
          write( '                                             ' );
          gotoxy( 1, 7 );
          ErgebnisAnzeigen( FALSE );        { Speichertabelle anzeigen }
        end;

      F2 :                                   { Speicherblock freigeben }
        begin
          i := -1;             { noch keinen gltigen Block eingegeben }
          repeat
            gotoxy( 1, 23 );
            write( 'Welchen Block freigeben [ A-Z ]: ' );
            readln( Zeiger );
            Zeiger := upcase( Zeiger );
            if ( Zeiger >= 'A' ) and ( Zeiger <= 'Z' ) then
              if ( BlockFeld[ ord( Zeiger ) - 65 ].Adresse <> 0 ) then
                i := ord( Zeiger ) - 65;
          until ( i <> - 1 );
          DOS_FreeMem( BlockFeld[ i ].Adresse );
          BlockFeld[ i ].Adresse := 0;
          BlockFeld[ i ].Groesse := 0;
          gotoxy( 1, 23 );
          writeln( '                                             ' );
          gotoxy( 1, 7 );
          ErgebnisAnzeigen( FALSE );        { Speichertabelle anzeigen }
        end;

      F3 :                              { Gre eines Blocks verndern }
        begin
          i := -1;             { noch keinen gltigen Block eingegeben }
          repeat
            gotoxy( 1, 23 );
            write( 'Gre von welchem Block ndern [ A-Z ]: ' );
            readln( Zeiger );
            Zeiger := upcase( Zeiger );
            if ( Zeiger >= 'A' ) and ( Zeiger <= 'Z' ) then
              if ( BlockFeld[ ord( Zeiger ) - 65 ].Adresse <> 0 ) then
                i := ord( Zeiger ) - 65;
          until ( i <> - 1 );
          write( 'Wie viele KB sollen reserviert werden  : ' );
          readln( Versuch );
          Versuch := Versuch * 64 - 1;      { Umrechnen in Paragraphen }
          DOS_ChangeMem( Versuch, BlockFeld[ i ].Adresse, Groesse );
          if ( Groesse <> Versuch ) then                     { Fehler? }
            begin
              str( ( Groesse + 1 ) div 64, sdummy );
              write( 'Es sind nur noch ' + sdummy + 'KB frei ' );
              repeat until keypressed;
              while keypressed do
                Taste := readkey;
              Taste := #0;
            end
          else
            BlockFeld[ i ].Groesse := Groesse;     { neue Gre setzen }
          gotoxy( 1, 23 );
          writeln( '                                             ' );
          writeln( '                                             ' );
          write( '                                             ' );
          gotoxy( 1, 7 );
          ErgebnisAnzeigen( FALSE );        { Speichertabelle anzeigen }
        end;

      F8 :
        Strategie := ( Strategie + 1 ) mod 3;     { Strategie wechseln }

      F9 :                                   { Umschaltung: UMB zuerst }
        UMB_zuerst := not UMB_zuerst;

      F10:                              { Umschaltung: UBM einbeziehen }
        mit_UMB := not mit_UMB;
    end;
  until ( Taste = ESC );
end;

{**********************************************************************}
{                     H A U P T P R O G R A M M                        }
{**********************************************************************}

var StartStrategie  : integer;  { Speicherverteilung bei Programmstart }
    StartUMB        : integer;      { UMB-Einbindung bei Programmstart }
    Akt_UMB_ein     : boolean;               { UMB Verwenden (ja/nein) }
    Akt_UMB_zuerst  : boolean;                  { zuerst im UMB suchen }
    Akt_Strategie   : integer;                { aktuelle Suchstrategie }

begin
  clrscr;
  writeln( 'Demonstrationsprogramm zur DOS-Speicherverwaltung ',
           ' (C) 1991 by Michael Tischer' );
  writeln( '==================================================',
           '============================'#13#10 );
  writeln( '                        Speicherbelegung wird ermittelt ' );

  {-- DOS-Werte beim Programmstart sichern ----------------------------}

  StartStrategie := LeseStrategie;    { Strategie der Speicherbelegung }
  StartUMB := LeseUMB;                               { UMB eingebunden }
  AllokiereSpeicher;                           { Testumgebung schaffen }
  SetzeStrategie( Startstrategie );      { alte Strategie restaurieren }
  SetzeUMB( StartUMB );

  if ( KonvSeg = 0 ) then        { konventionellen Speicher allokiert? }
    begin                   { Nein, Programm mit Fehlermeldung beenden }
      clrscr;
      writeln('MEMDEMOP: Zu wenig Speicher!');
      exit;                                         { Programm beenden }
    end;

  {-- Startwerte fr die Speicherverwaltung ---------------------------}

  Akt_UMB_ein := ( StartUMB = UMB_EIN );
  Akt_UMB_zuerst := ( ( StartStrategie and ZUERST_UMB ) = ZUERST_UMB );
  Akt_Strategie := StartStrategie and ( $FF xor ZUERST_UMB );

  {-- Demonstration der Speicherverteilung ----------------------------}

  clrscr;
  writeln( 'Demonstrationsprogramm zur DOS-Speicherverwaltung ',
           ' (C) 1991 by Michael Tischer' );
  writeln( '==================================================',
           '============================' );
  Demo( AKT_UMB_ein, AKT_UMB_zuerst, Akt_Strategie );

  {-- Alte DOS-Werte restaurieren -------------------------------------}

  SpeicherFreigeben;             { den reservierten Speicher freigeben }
  SetzeStrategie( StartStrategie );
  SetzeUMB( StartUMB );
  clrscr;
end.
