{**********************************************************************}
{*                                M F 2 P                             *}
{*--------------------------------------------------------------------*}
{*    Demonstriert die Abfrage der zustzlichen Tasten einer MF-II-   *}
{*    Tastatur.                                                       *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  :  1.01.1992                                     *}
{*    letztes Update :  1.01.1992                                     *}
{**********************************************************************}
                                                                        
program MF2P;

uses Dos, Crt;

const ZW = #13#10;                                     { Zeilenwechsel }

{**********************************************************************}
{* HexByte : Verwandelt ein Byte in einen zweiziffrigen Hex-String    *}
{* Eingabe : WERT = das umzuwandelnde Byte                            *}
{* Ausgabe : der zweiziffrige Hex-String                              *}
{**********************************************************************}

function HexByte( wert : byte ) : string;

const HexDigits : array [0..15] of char = '0123456789ABCDEF';

var dummy : string[2];                          { nimmt den String auf }

begin
  dummy[0] := chr(2);            { der String umfat grunds. 2 Zeichen }
  dummy[1] := HexDigits[ wert shr 4 ];            { die beiden Nibbles }
  dummy[2] := HexDigits[ wert and $0F ];          { nach Hex wandeln   }
  HexByte := dummy;
end;

{**********************************************************************}
{* TestMF: Prft, ob die erweiterten BIOS-Funktionen zur Abfrage      *}
{*            der MF-II-Tastatur zur Verfgung stehen                 *}
{* Eingabe : keiner                                                   *}
{* Ausgabe : TRUE, wenn die Funktionen zur Verfgung stehen, sonst    *}
{*           FALSE                                                    *}
{**********************************************************************}

function TestMF : boolean;

var Regs : Registers;                           { zum Interrupt-Aufruf }

begin
  Regs.AX := $1200;  { erweiterte Status-Funktion fr MF-II-Tastaturen }
  intr( $16, Regs );
  TestMF := ( Regs.AX <> $1200 );  { AX=$1200 : Funktion nicht prsent }
end;

{**********************************************************************}
{* GetMFKey : Fragt eine Taste mit Hilfe der erweiterten Tastatur-    *}
{*            funktion $10 ab.                                        *}
{* Eingabe : keiner                                                   *}
{* Ausgabe : der zurckgelieferte Tastencode                          *}
{**********************************************************************}

function GetMFKey : word;

var Regs : Registers;                           { zum Interrupt-Aufruf }

begin
  Regs.AH := $10;      { erweiterte Lese-Funktion fr MF-II-Tastaturen }
  intr( $16, Regs );
  GetMFKey := Regs.AX;                      { Tastencode zurckliefern }
end;

{**********************************************************************}
{*                     H A U P T P R O G R A M M                      *}
{**********************************************************************}

var taste : word;

begin
  clrscr;
  writeln( 'MF2P  -  (c) 1992 by Michael Tischer' + ZW );
  if ( TestMF ) then
    begin
      writeln( 'Die BIOS-Erweiterungen fr MF-II-Tastaturen wurden '+
               'entdeckt! ' + ZW + ZW + 'Bettigen Sie nun beliebige ' +
               'Tasten und Tastenkombinationen, deren Codes Sie '+ ZW +
               'in Erfahrung bringen mchten.' + ZW + ZW +
               'Die Bettigung von <Esc> beendet das Programm.' + ZW );

      repeat                                          { Eigabeschleife }
        taste := GetMFKey;                               { Taste holen }
        write( 'Scan : ', HexByte(hi(taste)), '  ',
               'ASCII: ', HexByte(lo(taste)) );
        if ( (lo(taste) = $E0) and (hi(taste) <> 0 ) ) then
          write( '  <---- MF-II-Taste' );
        writeln;
      until ( taste = $011b );                { wiederholen bis ESCAPE }
      writeln( ZW );
    end
  else
    writeln( 'Keine BIOS-Erweiterungen fr MF-II-Tastaturen prsent!');
end.
