{********************************************************************}
{                       M I X U T I L  . P A S                       }
{*------------------------------------------------------------------*}
{ Aufgabe        : Stellt Funktionen zur Programmierung des Sound-   }
{                  blaster Mixers zur Verfgung.                     }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 20.03.1994                                        }
{ letztes Update : 10.10.1994                                        }
{*------------------------------------------------------------------*}
{ Info           : mix_???  - Gelten fr alle Mixer-Versionen        }
{                  mix3_??? - ... fr Mixer CT1345 ( ab DSP V3.00 )  }
{                  mix4_??? - ... fr Mixer CT1745 ( ab DSP V4.00 )  }
{********************************************************************}
Unit MIXUTIL;

Interface

Uses SBUTIL;

Const
 ERROR    = $ffff;
 NO_ERROR = 0;
                                                 { Mixer-Portoffsets }
 MIX_REGISTERPORT =  $04;                   { Register einstellen... }
 MIX_DATAPORT     =  $05;                    { Daten lesen/schreiben }

                                        { CT1335-Mixer (im DSP 2.xx) }
 MIX_RSET         =  $00;                       { Mixer zurcksetzen }
 MIX2_MASTERVOL   =  $02;               { Gesamt-Volume (BITS 3,2,1) }
 MIX2_MIDIVOL     =  $06;             { MIDI(FM) Volume (BITS 3,2,1) }
 MIX2_CDVOL       =  $08;                   { CD Volume (BITS 3,2,1) }
 MIX2_VOICEVOL    =  $0A;             { VOICE(DSP) Volume (BITS 2,1) }

                                       { CT1345 Mixer (im DSP 3.xx ) }
 MIX3_VOICEVOL    =   $04;          { VOICE-Vol. (Bits 7,6,5  3,2,1) }
 MIX3_MICVOL      =   $0A;             { Mikrophon Volume (BITS 2,1) }

 MIX3_ADCSTATE    =   $0C;                  { DSP-Eingang und Filter }
                                                    { Eingangsfilter }
 MIX3_ADCFILTEROFF=   $20;               { Bit 5: Low-Pass Filter an }
 MIX3_LOWPASS88   =   $08;          { Bit 3: 8.8 kHz Low-Pass Filter }
                                                    { Eingangsquelle }
 MIX3_MICSRC      =   $00;            { BIT 2,1 = 00: Mikrophon      }
 MIX3_MICSRC_     =   $04;            { BIT 2,1 = 10: auch Mikrophon }
 MIX3_CDSRC       =   $02;            { BIT 2,1 = 01: CD             }
 MIX3_LINESRC     =   $06;            { BIT 2,1 = 11: LINE IN        }
 MIX3_SRCMSK      =   $06;            { Bitmaske der Source-Bits     }


 MIX3_DACSTATE    = $0E;          { Ausgabefilter und Stereoschalter }
 MIX3_DACFILTEROFF= $20;          { Bit 5: Low-Pass Filter an (s.o.) }
 MIX3_STEREOON    = $02;                      { Bit 1: Stereoausgabe }

 MIX3_MASTERVOL  =  $22;         { Gesamtvol. (BITS 7,6,5 und 3,2,1) }
 MIX3_MIDIVOL    =  $26;          { MIDI-Vol. (BITS 7,6,5 und 3,2,1) }
 MIX3_CDVOL      =  $28;             { CD-Vol (BITS 7,6,5 und 3,2,1) }
 MIX3_LINEVOL    =  $2E;          { LINE-Vol. (BITS 7,6,5 und 3,2,1) }

                              { CT1745 Mixer (im DSP 4.xx) sowie ASP }
                                                      { Lautstrken: }
 MIX4_MASTERVOL_L = $30;             { Gesamt links (BITS 7,6,5,4,3) }
 MIX4_MASTERVOL_R = $31;            { Gesamt rechts (BITS 7,6,5,4,3) }
 MIX4_VOICEVOL_L  = $32;         { VOICE(DSP) links (BITS 7,6,5,4,3) }
 MIX4_VOICEVOL_R  = $33;        { VOICE(DSP) rechts (BITS 7,6,5,4,3) }
 MIX4_MIDIVOL_L   = $34;               { MIDI links (BITS 7,6,5,4,3) }
 MIX4_MIDIVOL_R   = $35;              { MIDI rechts (BITS 7,6,5,4,3) }
 MIX4_CDVOL_L     = $36;                 { CD links (BITS 7,6,5,4,3) }
 MIX4_CDVOL_R     = $37;                { CD rechts (BITS 7,6,5,4,3) }
 MIX4_LINEVOL_L   = $38;               { LINE links (BITS 7,6,5,4,3) }
 MIX4_LINEVOL_R   = $39;              { LINE rechts (BITS 7,6,5,4,3) }
 MIX4_MICVOL      = $3A;               { Mikrophon (BITS 7,6,5,4,3 ) }
 MIX4_PCSPEAKERVOL= $3B;               { PC-Lautsprecher (BITS 7,6 ) }

 MIX4_OUTSOURCE   = $3C;                            { Ausgabequellen }
 MIX4_ADCSOURCE_L = $3D;                        { Sample-Quellen L/R }
 MIX4_ADCSOURCE_R = $3E;

  MIX4_MIDI_L     = $40;                    { Aktive Aufnahme-Quellen}
  MIX4_MIDI_R     = $20 ;
  MIX4_LINE_L     = $10;
  MIX4_LINE_R     = $08;
  MIX4_CD_L       = $04;
  MIX4_CD_R       = $02;
  MIX4_MIC        = $01;

    { Vorverstrker fr Ausgabe(OUT) und Samplen(ADC) (Bits 7 und 6 ) }
 MIX4_ADCGAIN_L  = $3F;
 MIX4_ADCGAIN_R  = $40;
 MIX4_OUTGAIN_L  = $41;
 MIX4_OUTGAIN_R  = $42;

 MIX4_AGC   = $43;                    { Mikrofonvorverstrker (20dB) }
 MIX4_AGCON = $01;                         { Mikrofonvoverstrker an }

        { Hhen und Bsse des Vorverstrkers (jeweils BITS 7,6,5,4 ) }
 MIX4_TREBLE_L  =     $44;
 MIX4_TREBLE_R  =     $45;
 MIX4_BASS_L    =     $46;
 MIX4_BASS_R    =     $47;

              { Welche Interrupts und DMA-Leitungen werden benutzt ? }
 MIX4_IRQ   = $80;
 MIX4_IRQ2  = $01;                   { 4 mgliche Interruptleitungen }
 MIX4_IRQ5  = $02;
 MIX4_IRQ7  = $04;
 MIX4_IRQ10 = $08;

 MIX4_DMA  = $81;              {- Welche DMA Leitung wird benutzt ? }
 MIX4_DMA0 = $01;
 MIX4_DMA1 = $02;
{MIX4_DMA2 = $04                                  Diskettenlaufwerk }
 MIX4_DMA3 = $08;
{MIX4_DMA4 = $10                                       Kaskadierung }
 MIX4_DMA5 = $20;
 MIX4_DMA6 = $40;
 MIX4_DMA7 = $80;

 MIX4_IRQSOURCE = $82;
 MIX4_IRQ8DMA   = $01;             { Interupt von 8-Bit DMA und Midi }
 MIX4_IRQ16DMA  = $02;                     { Interupt von 16-Bit DMA }
 MIX4_IRQMPU    = $04;                            { Interupt von MPU }

{ MIXUTIL-spezifische Konstanten                                     }
{ Fr Zugriff auf Laustrken-Arrays werden Konstanten benutzt,       }
{ die angeben, ob sich eine Portadresse auf den rechten, linken,     }
{ oder beide Kanle bezieht.                                         }
 SBPORT  = 0;         { Fr Array-Zugriff: 1. Element = Port         }
 CHANNEL = 1;                           {  2. Element = Zugriffscode }

 L = 0;                                                      { links }
 R = 1;                                                     { rechts }

 CH_LEFT  = 1;                            { Kennung fr linken Kanal }
 CH_RIGHT = 2;                                       { rechter Kanal }
 CH_BOTH  = 3;                                        { beide Kanle }
 CH_NONE  = 0;                                          { kein Kanal }
 CH_MAX   = 6;            { Maximum von rechtem und linkem verwenden }

              { Durchnumerierung der zur Verfgung stehenden Quellen }
 CD          = 0;
 LINE        = 1;
 VOICE       = 2;
 MASTER      = 3;
 MIDI        = 4;
 MIC         = 5;
 PCSPEAKER   = 6;
 NUM_SOURCES = 7;

 CD_L        = 7;
 LINE_L      = 8;
 VOICE_L     = 9;
 MASTER_L    = 10;
 MIDI_L      = 11;
 CD_R        = 12;
 LINE_R      = 13;
 VOICE_R     = 14;
 MASTER_R    = 15;
 MIDI_R      = 16;

 MAX_SRC     = 17;

 DAC = TRUE;                           { fr 'mix3_PrepareForStereo' }
 ADC = FALSE;

{-- Globale Modul-Variablen -----------------------------------------}
var
 MIXBASE : SBBASE;                         { Port Adresse des Mixers }

 bMix3DACStereo,         { Diese Variablen werden zum Retten einiger }
 bMix3DACFilter,         { Mixer-Werte verwendet.                    }
 bMix3ADCFilter : Byte;

 bMix4SourceL,
 bMix4SourceR   : Byte;

{ Die folgenden Tabellen dienen zur Einstellungder Laustrke .       }
{ Fr jede mgliche Quelle werden Register und einzustellende        }
{ Laustrke (links, rechts, beide) vermerkt.                         }

Const
 bMix3VolTable : Array[0..MAX_SRC - 1,0..1] of Byte =
                                     ( ( MIX3_CDVOL, CH_BOTH ),
                                       ( MIX3_LINEVOL, CH_BOTH ),
                                       ( MIX3_VOICEVOL, CH_BOTH ),
                                       ( MIX3_MASTERVOL, CH_BOTH ),
                                       ( MIX3_MIDIVOL, CH_BOTH ),
                                       ( MIX3_MICVOL, CH_MAX ),
                                       ( 0, 0 ), { Speaker }

                                       ( MIX3_CDVOL, CH_LEFT ),
                                       ( MIX3_LINEVOL, CH_LEFT ),
                                       ( MIX3_VOICEVOL, CH_LEFT ),
                                       ( MIX3_MASTERVOL, CH_LEFT ),
                                       ( MIX3_MIDIVOL, CH_LEFT ),

                                       ( MIX3_CDVOL, CH_RIGHT ),
                                       ( MIX3_LINEVOL, CH_RIGHT ),
                                       ( MIX3_VOICEVOL, CH_RIGHT ),
                                       ( MIX3_MASTERVOL, CH_RIGHT ),
                                       ( MIX3_MIDIVOL, CH_RIGHT ) );

 bMix4VolTable : array[0..MAX_SRC - 1,0..1] of Byte =
                                     ( ( MIX4_CDVOL_L, CH_BOTH ),
                                       ( MIX4_LINEVOL_L, CH_BOTH ),
                                       ( MIX4_VOICEVOL_L, CH_BOTH ),
                                       ( MIX4_MASTERVOL_L, CH_BOTH ),
                                       ( MIX4_MIDIVOL_L, CH_BOTH ),
                                       ( MIX4_MICVOL, CH_LEFT ),
                                       ( MIX4_PCSPEAKERVOL, CH_LEFT ),

                                       ( MIX4_CDVOL_L, CH_LEFT ),
                                       ( MIX4_LINEVOL_L, CH_LEFT ),
                                       ( MIX4_VOICEVOL_L, CH_LEFT ),
                                       ( MIX4_MASTERVOL_L, CH_LEFT ),
                                       ( MIX4_MIDIVOL_L, CH_LEFT ),

                                       ( MIX4_CDVOL_L, CH_RIGHT ),
                                       ( MIX4_LINEVOL_L, CH_RIGHT ),
                                       ( MIX4_VOICEVOL_L, CH_RIGHT ),
                                       ( MIX4_MASTERVOL_L, CH_RIGHT ),
                                       ( MIX4_MIDIVOL_L, CH_RIGHT ) );

{ Der CT1745-Mixer erlaubt das An- und Abschalten mehrerer           }
{ Aufnahme- und Wiedergabequellen. Das folgende Array vermerkt fr   }
{ jede mgliche Quelle die entsprechenden Schalterbits.              }

bMix4SourceBits : Array[0..MAX_SRC - 1] of Byte =
{ CD }                             ( MIX4_CD_L or MIX4_CD_R,
{ LINE }                             MIX4_LINE_L or MIX4_LINE_R,
{ VOICE ( nicht schaltbar ) }        0,
{ MASTER ( nicht schaltbar ) }       0,
{ MIDI }                             MIX4_MIDI_L or MIX4_MIDI_R,
{ MIC }                              MIX4_MIC,
{ PCSPEAKER ( nicht schaltbar ) }    0,

{ CD_L }                             MIX4_CD_L,
{ LINE_L }                           MIX4_LINE_L,
{ VOICE_L }                          0,
{ MASTER_L }                         0,
{ MIDI_L }                           MIX4_MIDI_L,

{ CD_R }                             MIX4_CD_R,
{ LINE_R }                           MIX4_LINE_R,
{ VOICE_R }                          0,
{ MASTER_R }                         0,
{ MIDI_R }                           MIX4_MIDI_R );

{-- Prototypen ---------------}

Procedure mix_Reset;

Function mix_SetBase( var sbbase : SBBASE; iReset:Boolean ) : Word;

Procedure mix_Write( iReg, iData : Word);

Function mix_Read( iReg : Word ) : Word;

Procedure mix3_SetADCFilter( iState : Boolean );

Function mix3_GetADCFilter : Boolean;

Procedure mix3_SetDACFilter( iState : Boolean );

Function mix3_GetDACFilter : Boolean;

Procedure mix3_SetDACStereo( iState : Boolean );

Function mix3_GetDACStereo : Boolean;

Procedure mix3_SetADDACLowPass( iState : Boolean );

Function mix3_GetADDACLowPass : Boolean;

Procedure mix3_PrepareForStereo( iMode : Boolean );

Procedure mix3_RestoreFromStereo;

Procedure mix3_SetVolume( iSource, iVolL, iVolR : Word );

Function mix3_GetVolume( iSource : Word ) : Word;

Procedure mix3_SetADCSource( iSource : Word );

Function mix3_GetADCSource : Word;

Procedure mix4_PrepareForMonoADC;

Procedure mix4_RestoreFromMonoADC;

Procedure mix4_SetVolume( iSource, iVolL, iVolR : Word );

Function mix4_GetVolume( iSource : Word ) : Word;

Procedure mix4_SetADCSourceL( iSource : Word; iState : Boolean );

Procedure mix4_SetADCSourceR( iSource : Word; iState : Boolean );

Function mix4_GetADCSourceL( iSource : Word ) : Word;

Function mix4_GetADCSourceR( iSource : Word ) : Word;

Procedure mix4_SetOUTSource( iSource : Word; iState : Boolean );

Function mix4_GetOUTSource( iSource : Word ) : Word;

Procedure mix4_SetADCGain( iGainL, iGainR  : Word);

Function mix4_GetADCGain( iChannel : Word ) : Word;

Procedure mix4_SetOUTGain( iGainL, iGainR  : Word);

Function mix4_GetOUTGain( iChannel : Word ) : Word;

Procedure mix4_SetAGC( iState : Word );

Function mix4_GetAGC : Boolean;

Procedure mix4_SetTrebble( iTrebbleL, iTrebbleR : Word);

Function mix4_GetTrebble( iChannel : Word ) : Word;

Procedure mix4_SetBass( iBassL, iBassR : Word );

Function mix4_GetBass( iChannel : Word ) : Word;

Implementation

{********************************************************************}
{ mix_Reset : Mixer zurcksetzen                                     }
{********************************************************************}
Procedure mix_Reset;

Begin
  mix_Write( MIX_RSET, 0 );
End;

{********************************************************************}
{ mix_SetBase : Setzt Port-Adresse                                   }
{*------------------------------------------------------------------*}
{ Eingabe : SBBASE - einr initialisierte SBBase-Struktur             }
{           iReset   - <> 0 : Mixer soll zurckgesetzt werden        }
{*------------------------------------------------------------------*}
{ Info    : - Bevor eine der folgenden Funktionen aufgerufen werden  }
{             darf, mu mit Hilfe dieser Funktion eine SBBASE-       }
{             Struktur mit Informationen ber die installierte       }
{             Soundkarte eingestellt worden sein!                    }
{********************************************************************}
Function mix_SetBase( var sbbase : SBBASE; iReset : Boolean ) : Word;

Begin
  if sbbase.iMixPort <> -1 then
    Begin
      MIXBASE := sbbase; { Daten in TPU-Struktur merken }
      if iReset then mix_Reset;
      mix_SetBase := NO_ERROR;
    End
  else
    mix_SetBase := ERROR;
End;

{********************************************************************}
{ mix_Write : Byte an Mixer-Register bermitteln                     }
{*------------------------------------------------------------------*}
{ Eingabe : iReg  - Nummer des zu ndernden Registers                }
{           iData - Wert, der ins Register geschrieben werden soll   }
{********************************************************************}
Procedure mix_Write( iReg, iData : Word);
Begin
   port[MIXBASE.iMixPort +      { Zu beschreibendes Register angeben }
        MIX_REGISTERPORT] := Byte ( iReg ) ;
                                    
   port[MIXBASE.iMixPort +          { Neuen Registerwert bermitteln }
       MIX_DATAPORT] := Byte ( iData );
End;

{********************************************************************}
{ mix_Read : Byte aus Mixer-Register lesen                           }
{*------------------------------------------------------------------*}
{ Eingabe : iReg  - Nummer des zu lesenden Registers                 }
{ Ausgabe : Wert des Mixer-Registers                                 }
{********************************************************************}
Function mix_Read( iReg : Word ) : Word;

Begin                                 
   port[MIXBASE.iMixPort +            { Zu lesendes Register angeben }
        MIX_REGISTERPORT] := Byte ( iReg );

   mix_Read := port[MIXBASE.iMixPort + { Registerwert holen }
               MIX_DATAPORT];
End;

{********************************************************************}
{ mix3_SetADCFilter : ADC-Filter an- oder abschalten                 }
{*------------------------------------------------------------------*}
{ Eingabe : iState    = FALSE: Filter abschalten                     }
{                     = TRUE: Filter anschalten                      }
{*------------------------------------------------------------------*}
{ Info    : - Das Bit, welches den Filter kontrolliert ist ein       }
{             AUS-Schalter. Bei gesetztem Bit ist der Filter daher   }
{             ausgeschaltet!                                         }
{             ADC = Analog/Digital Converter (Aufnahme)              }
{********************************************************************}
Procedure mix3_SetADCFilter( iState : Boolean );

var bIn : Byte;

Begin
   bIn := Byte ( mix_Read( MIX3_ADCSTATE ) );{ akt. Reg.-Wert lesen }

   {-- Filter-Bit setzen / lschen (LOW-Aktiv) -------------}
   if iState then
     mix_Write( MIX3_ADCSTATE, bIn and not MIX3_ADCFILTEROFF )
   else
     mix_Write( MIX3_ADCSTATE, bIn or MIX3_ADCFILTEROFF );
End;

{********************************************************************}
{ mix3_GetADCFilter : Zustand des ADC-Filters ermitteln              }
{*------------------------------------------------------------------*}
{ Ausgabe := FALSE - Filter abgeschaltet                             }
{            TRUE  - Filter angeschaltet                             }
{********************************************************************}
Function mix3_GetADCFilter : Boolean;

Begin                            { Achtung! Filter Bit ist LOW-Aktiv }
  mix3_GetADCFilter := ( ( mix_Read( MIX3_ADCSTATE ) and
                           MIX3_ADCFILTEROFF ) = 0 );
End;

{********************************************************************}
{ mix3_SetDACFilter : DAC-Filter an- oder abschalten                 }
{*------------------------------------------------------------------*}
{ Eingabe : iState - FALSE : Filter abschalten                       }
{                    TRUE  : Filter anschalten                       }
{*------------------------------------------------------------------*}
{ Info :      DAC : Digital/Analog Converter (Wiedergabe)            }
{********************************************************************}
Procedure mix3_SetDACFilter( iState : Boolean );

var Bin : Byte;

Begin
   bIn := Byte ( mix_Read( MIX3_DACSTATE ) );{ akt. Reg.-Wert lesen }

   {-- Filter-Bit setzen/lschen (Filter-Bit ist LOW-Aktiv) -------}
   if iState then
     mix_Write( MIX3_DACSTATE, bIn and not MIX3_DACFILTEROFF )
   else
     mix_Write( MIX3_DACSTATE, bIn or MIX3_DACFILTEROFF );
End;

{********************************************************************}
{ mix3_GetDACFilter : Zustand des DAC-Filters ermitteln              }
{*------------------------------------------------------------------*}
{ Ausgabe := FALSE - Filter abgeschaltet                             }
{            TRUE  - Filter angeschaltet                             }
{********************************************************************}
Function mix3_GetDACFilter : Boolean;

Begin                                { Achtung! Filter-Bit ist LOW-Aktiv }
  mix3_GetDACFilter := ( ( mix_Read( MIX3_DACSTATE ) and
                           MIX3_DACFILTEROFF ) = 0 );
End;

{********************************************************************}
{ mix3_SetDACStereo : Stereo-Wiedergabe (DAC) an- oder abschalten    }
{*------------------------------------------------------------------*}
{ Eingabe : iState - FALSE: Stereo-Wiedergabe abschalten => Mono     }
{                    TRUE : Stereo-Wiedergabe anschalten             }
{********************************************************************}
Procedure mix3_SetDACStereo( iState : Boolean );

var bIn : Byte;

Begin
   bIn := Byte ( mix_Read( MIX3_DACSTATE ) );{ akt. Reg.-Wert lesen }
   if iState then
     mix_Write( MIX3_DACSTATE,bIn or MIX3_STEREOON )
   else
     mix_Write( MIX3_DACSTATE,bIn and not MIX3_STEREOON );
End;

{********************************************************************}
{ mix3_GetDACStereo : Zustand der Stereo-Wiedergabe ermitteln        }
{*------------------------------------------------------------------*}
{ Ausgabe := FALSE - Stereo-Wiedergabe abgeschaltet                  }
{            TRUE  - Stereo-Wiedergabe angeschaltet                  }
{********************************************************************}
Function mix3_GetDACStereo : Boolean;

Begin
  mix3_GetDACStereo := ( ( mix_Read( MIX3_DACSTATE ) and
                           MIX3_STEREOON ) <> 0 );
End;

{********************************************************************}
{ mix3_SetADDACLowPass : LowPass Filter fr Aufnahme setzen          }
{*------------------------------------------------------------------*}
{ Eingabe : iState = FALSE: 3.2 kHz LowPass Filter                   }
{                    TRUE:  8.8 kHz LowPass Filter                   }
{*------------------------------------------------------------------*}
{ Info : - Der LowPass Filter wird nur verwendet, wenn der ADC       }
{          oder DAC Filter angeschaltet werden. Ansonsten wird der   }
{          LowPass Filter bei Aufnahme und WIedergabe ignoriert.     }
{********************************************************************}
Procedure mix3_SetADDACLowPass( iState : Boolean );

var Bin : Byte;

Begin
   bIn := Byte ( mix_Read( MIX3_ADCSTATE ) );{ akt. Reg.-Wert lesen }
   if iState then
     mix_Write( MIX3_ADCSTATE, bIn or MIX3_LOWPASS88 )
   else
     mix_Write( MIX3_ADCSTATE, bIn and not MIX3_LOWPASS88 );
End;

{********************************************************************}
{ mix3_GetADDACLowPass : Ermittle verwendeten LowPass Filter         }
{*------------------------------------------------------------------*}
{ Ausgabe := FALSE: 3.2 kHz LowPass Filter wird verwendet            }
{            TRUE:  8.8 kHz LowPass Filter wird verwendet            }
{********************************************************************}
Function mix3_GetADDACLowPass : Boolean;

Begin
  mix3_GetADDACLowPass := ( ( mix_Read( MIX3_ADCSTATE ) and
                              MIX3_LOWPASS88 ) <> 0 );
End;

{********************************************************************}
{ mix3_PrepareForStereo : Mixer fr Stereo-Aufnahme/Wiedergabe       }
{                         vorbereiten                                }
{*------------------------------------------------------------------*}
{ Eingabe : iMode = FALSE : Ausgabe                                  }
{                 = TRUE  : Aufnahme                                 }
{*------------------------------------------------------------------*}
{ Info : - Bei Stereo-Aufnahme/Wiedergabe mssen alle Filter         }
{          abgeschaltet sein. Fr eine ordnungsgeme Ausgabe mu    }
{          aber der Stereo-Schalter angeschaltet werden.             }
{********************************************************************}
Procedure mix3_PrepareForStereo( iMode : Boolean );

Begin
  {-- aktuelle Mixer-Zustnde merken --------------------------------}
  bMix3DACStereo := Byte (mix3_GetDACStereo);
  bMix3DACFilter := Byte (mix3_GetDACFilter);
  bMix3ADCFilter := Byte (mix3_GetADCFilter);

  {-- Filter und Stereo-Wiedergabe abschalten -----------------------}
  mix3_SetDACStereo( iMode );
  mix3_SetDACFilter( FALSE );
  mix3_SetADCFilter( FALSE );
End;

{********************************************************************}
{ mix3_RestoreFromStereo : Mixer in alten Zustand zurckversetzen    }
{*------------------------------------------------------------------*}
{ Info : - Der Zustand des Mixers mu vorher durch die Funktion      }
{          'mix3_PrepareForStereo' in den entsprechenden globalen    }
{          Variablen gerettet worden sein.                           }
{********************************************************************}
Procedure mix3_RestoreFromStereo;

Begin                    
  {-- Filter und Stereo-Wiedergabe wiederherstellen -----------------}
  mix3_SetDACStereo( Boolean (bMix3DACStereo) );
  mix3_SetDACFilter( Boolean (bMix3DACFilter) );
  mix3_SetADCFilter( Boolean (bMix3ADCFilter) );
End;

{********************************************************************}
{ mix3_SetVolume : Ausgabe-Laustrke festlegen                       }
{*------------------------------------------------------------------*}
{ Eingabe : iSource - Quelle ( Mikrophon, DSP, CD, LINE, MIDI ) und  }
{                     Gesamtlautstrke ( Master )                    }
{           iLeft   - Lautstrke fr linken Kanal                    }
{           iRight  - Lautstrke fr rechten Kanal                   }
{*------------------------------------------------------------------*}
{ Info : - Die Gesamtlautstrke mu grer 0 sein, damit die anderen }
{          Quellen gehrt werden knnen. Die Laustrke wird fr den  }
{          linken und rechten Kanal getrennt angegeben. Da ein       }
{          Mikrophon immmer Monophon ist, wird dessen Laustrke      }
{          durch das Maximum von linkem und rechten Kanal bestimmt.  }
{          Die Angabe der Lautstrken erfolgt durch Werte zwischen   }
{          0 und 255, die auf den fr die jeweilige Karte gltigen   }
{          Wertebereich abgebildet werden.                           }
{********************************************************************}
Procedure mix3_SetVolume( iSource, iVolL, iVolR : Word );

var bIn, bVolR, bVolL : Byte;

Begin
  {-- Werte von 0 - 255 auf Werte von 0 - 7 abbilden ---------------}
  bVolL := Byte ( iVolL shr 5 );
  bVolR := Byte ( iVolR shr 5 );

  case iSource of
    MIC,VOICE,VOICE_L,VOICE_R,LINE,LINE_L,LINE_R,
    CD,CD_L,CD_R,MIDI,MIDI_L,MIDI_R,MASTER,MASTER_L,MASTER_R:

      Begin { aktuellen Lautstrke-Wert lesen }
        bIn := Byte ( mix_Read( bMix3VolTable[iSource,SBPORT] ) );

        {-- neuen Lautstrke-Wert einstellen ---}
        case bMix3VolTable[iSource, CHANNEL] of
          CH_LEFT:                           { Nur Linken Kanal ndern }
            bIn := Byte ( ( bIn and $0F ) or ( bVolL shl 5 ) );

          CH_RIGHT:                         { Nur rechten Kanal ndern }
            bIn := Byte ( ( bIn and $F0 ) or ( bVolR shl 1 ) );

          CH_BOTH:                           { Links und Rechts ndern }
            bIn := Byte ( ( bVolL shl 5 ) or ( bVolR shl 1 ) );

          CH_MAX:                { Maximum von Links und Rechts setzen }
            Begin
              if (bVolL shl 1) > (bVolR shl 1) then
                bIn := Byte ( bVolL shl 1 )
              else
                bIn := Byte ( bVolR shl 1 );
            End;
        End;
                                           { ... und zurckschreiben }
        mix_Write( bMix3VolTable[iSource , SBPORT], bIn );
      End;
  End;
End;

{********************************************************************}
{ mix3_GetVolume : Ausgabe-Laustrke ermitteln                       }
{*------------------------------------------------------------------*}
{ Eingabe : iSource - Quelle ( Mikrophon, DSP, CD, LINE, MIDI ) und  }
{                     Gesamtlautstrke ( Master )                    }
{ Ausgabe : Laustrke der Quelle zwischen 0 und 255, bei Stereo als  }
{           LO- und HIByte                                           }
{********************************************************************}
Function mix3_GetVolume( iSource : Word ) : Word;

var  bIn, bLeft, bRight : Byte;

Begin

  case iSource of
    MIC,VOICE,VOICE_L,VOICE_R,LINE,LINE_L,LINE_R,
    CD,CD_L,CD_R,MIDI,MIDI_L,MIDI_R,MASTER,MASTER_L,MASTER_R:
      Begin  { aktuellen Lautstrke-Wert lesen }
        bIn := Byte (mix_Read( bMix3VolTable[iSource,SBPORT] ) );
        bLeft  := Byte ( ( ( bIn and $F0 ) shr 5 ) shl 5 );
        bRight := Byte ( ( ( bIn and $0F ) shr 1 ) shl 5 );

      case bMix3VolTable[iSource , CHANNEL]  of
        CH_LEFT:  mix3_GetVolume := bLeft;
        CH_MAX,
        CH_RIGHT: mix3_GetVolume := bRight;
        CH_BOTH:  mix3_GetVolume := bLeft * 256 + bRight;
      End;
     End
    else mix3_GetVolume:= $FFFF;
  End;
End;

{********************************************************************}
{ mix3_SetADCSource : Aufnahme-Quelle setzen                         }
{*------------------------------------------------------------------*}
{ Eingabe : iSource - Quelle fr Aufnahme                            }
{                     Erlaubt: MIC, CD, LINE                         }
{*------------------------------------------------------------------*}
{ Info : Der A/D-Wandler bentigt eine Quelle. Diese wird hier       }
{        angegeben.                                                  }
{********************************************************************}
Procedure mix3_SetADCSource( iSource : Word );

var bIn : Byte;

Begin
  bIn := Byte (mix_Read( MIX3_ADCSTATE ) );
                 { immer alte Quelle lschen bevor neue gesetzt wird }
  case iSource of
    MIC:
      Begin
        bIn := bin and not MIX3_SRCMSK;
        bIn := bin or MIX3_MICSRC;
      End;

    CD:
      Begin
        bIn := bIn and not MIX3_SRCMSK;
        bIn := bIn or MIX3_CDSRC;
      End;

    LINE:
      Begin
        bIn := bIn and not MIX3_SRCMSK;
        bIn := bIn or MIX3_LINESRC;
      End;
  End;
  mix_Write( MIX3_ADCSTATE, bIn );
End;

{********************************************************************}
{ mix3_GetADCSource : Aktuelle Aufnahme-Quelle ermitteln             }
{*------------------------------------------------------------------*}
{ Eingabe : iSource - Quelle fr Aufnahme                            }
{                     Erlaubt: MIC, CD, LINE                         }
{*------------------------------------------------------------------*}
{ Info : Der A/D-Wandler bentigt eine Quelle. Diese wird hier       }
{        angegeben.                                                  }
{********************************************************************}
Function mix3_GetADCSource : Word;

var bIn : Byte;

Begin
  bIn := Byte ( mix_Read( MIX3_ADCSTATE ) and MIX3_SRCMSK );

  case bIn of
    MIX3_MICSRC_,
    MIX3_MICSRC:  mix3_GetADCSource := MIC;
    MIX3_CDSRC:   mix3_GetADCSource := CD;
    MIX3_LINESRC: mix3_GetADCSource := LINE;
    else          mix3_GetADCSource := $FFFF;
  End;
End;

{********************************************************************}
{ mix4_PrepareForMonoADC : Mixer fr Mono-Aufnahme vorbereiten       }
{*------------------------------------------------------------------*}
{ Info : - Dieser Befehl setzt Aufnahme-Quellen des rechten A/D-     }
{          Kanals auch im linken Kanal. Im Mono-Aufnahme-Betrieb     }
{          benutzt der DSP4.xx nur den linken Kanal.                 }
{********************************************************************}
Procedure mix4_PrepareForMonoADC;

Begin
  bMix4SourceL := Byte ( mix_Read( MIX4_ADCSOURCE_L ) );
  bMix4SourceR := Byte ( mix_Read( MIX4_ADCSOURCE_R ) );
  mix_Write( MIX4_ADCSOURCE_L, bMix4SourceL or bMix4SourceR );
End;

{********************************************************************}
{ mix4_RestoreFromMonoADC : Mixer-Zustand wiederherstellen.          }
{*------------------------------------------------------------------*}
{ Info : Diese Funktion darf nur aufgerufen werden, nachdem zuvor    }
{        die Funktion 'mix4_PrepareForMonoADC' aufgerufen wurde.     }
{********************************************************************}
Procedure mix4_RestoreFromMonoADC;

Begin
  mix_Write( MIX4_ADCSOURCE_L, bMix4SourceL );
  mix_Write( MIX4_ADCSOURCE_L, bMix4SourceR );
End;

{********************************************************************}
{ mix4_SetVolume : Ausgabe-Laustrke einer Mixer-Quelle setzen       }
{*------------------------------------------------------------------*}
{ Eingabe : iSource - Quelle ( Mikrophon, DSP, CD, LINE, MIDI,       }
{                     PCSPEAKER ) und Gesamtlautstrke ( Master )    }
{           iLeft   - Lautstrke fr linken Kanal                    }
{           iRight  - Lautstrke fr rechten Kanal                   }
{*------------------------------------------------------------------*}
{ Info : - Die Gesamtlautstrke mu grer 0 sein, damit die anderen }
{          Quellen gehrt werden knnen. Die Lautstrke wird fr den }
{          linken und rechten Kanal getrennt angegeben. Da ein       }
{          Mikrophon immmer Monophon ist, wird dessen Lautstrke     }
{          durch das Maximum von linkem und rechten Kanal bestimmt.  }
{          Die Angabe der Lautstrken erfolgt durch Werte zwischen   }
{          0 und 255, die in auf den fr die jeweilige Karte gl-    }
{          tigen Wertebereich abgebildet werden.                     }
{                                                                    }
{          Diese Funktion nutzt aus, da die Register fr linken und }
{          rechten Kanal einer Quelle an aufeinanderfolgenden Port-  }
{          adressen liegen.                                          }
{********************************************************************}
Procedure mix4_SetVolume( iSource, iVolL, iVolR  : Word);

Begin
  iVolL := iVolL and not $07;
  iVolR := iVolR and not $07;

  case iSource of
    MIC, PCSPEAKER, VOICE,VOICE_L,VOICE_R,LINE,LINE_L,LINE_R,
    CD,CD_L,CD_R,MIDI,MIDI_L,MIDI_R,MASTER,MASTER_L,MASTER_R:

    case bMix4VolTable[iSource, CHANNEL] of
      CH_LEFT:
        mix_Write( bMix4VolTable[iSource, SBPORT], iVolL );

      CH_RIGHT:
        mix_Write( bMix4VolTable[iSource, SBPORT] + 1, iVolR );

      CH_BOTH:
        Begin
          mix_Write( bMix4VolTable[iSource, SBPORT], iVolL );
          mix_Write( bMix4VolTable[iSource, SBPORT] + 1, iVolR );
        End;

      CH_MAX:
        Begin
          if iVolL > iVolR then
            mix_Write( bMix4VolTable[iSource,SBPORT], iVolL)
          else
            mix_Write( bMix4VolTable[iSource,SBPORT], iVolR)
      End;
    End;
  End;
End;

{********************************************************************}
{ mix4_GetVolume : Ausgabe-Laustrke ermitteln                       }
{*------------------------------------------------------------------*}
{ Eingabe : iSource : Quelle ( Mikrophon, DSP, CD, LINE, MIDI,       }
{                     PCSPEAKER ) und Gesamtlautstrke ( Master )    }
{ Ausgabe : Lautstrke der Quelle zwischen 0 und 255, bei Stereo als }
{           LO- und HIByte                                           }
{********************************************************************}
Function mix4_GetVolume( iSource : Word ) : Word;

var bLeft, bRight : Byte;

Begin
  case iSource of
    MIC,PCSPEAKER,VOICE,VOICE_L,VOICE_R,LINE,LINE_L,LINE_R,
    CD,CD_L,CD_R,MIDI,MIDI_L,MIDI_R,MASTER,MASTER_L,MASTER_R:

    {-- aktuellen autstrke-Wert lesen --------------}
    Begin
      bLeft := Byte ( mix_Read( bMix4VolTable[iSource, SBPORT] ) );
      bLeft := bLeft and not $07;
      bRight := Byte ( mix_Read( bMix4VolTable[iSource , SBPORT] + 1 ) );
      bRight := bRight and not $07;

      case bMix4VolTable[iSource , CHANNEL] of
        CH_LEFT:
          mix4_GetVolume := bLeft;

        CH_MAX:
          if( bLeft > bRight ) then
               mix4_GetVolume := bLeft
          else mix4_GetVolume := bRight;

        CH_RIGHT:
          mix4_GetVolume := bRight;

        CH_BOTH:
          mix4_GetVolume := bLeft * 256 + bRight;
      End;
    end;
    else mix4_GetVolume := $FFFF;
  End;
End;

{********************************************************************}
{ mix4_SetADCSourceL : Aufnahme-Quelle fr linken ADC setzen         }
{*------------------------------------------------------------------*}
{ Eingabe : iSource - Aufnahme-Quelle                                }
{           iState  - Quelle an oder abschalten                      }
{                     == 0: abschalten                               }
{                     <> 0: anschalten                               }
{********************************************************************}
Procedure mix4_SetADCSourceL( iSource : Word; iState  : Boolean );

var bInL : Byte;

Begin
  bInL := Byte ( mix_Read( MIX4_ADCSOURCE_L ) );

  case iSource of
    MIDI,MIDI_L,MIDI_R,CD,CD_L,CD_R,
    LINE,LINE_L,LINE_R,MIC:
      if iState then                             { Eingabekanal setzen }
        bInL := bInL or bMix4SourceBits[iSource]
      else                                      { Eingabekanal lschen }
        bInL := bInL and not bMix4SourceBits[iSource];
    End;
  mix_Write( MIX4_ADCSOURCE_L, bInL );
End;

{********************************************************************}
{ mix4_SetADCSourceR : Aufnahme-Quelle fr rechten ADC setzen        }
{*------------------------------------------------------------------*}
{ Eingabe : iSource - Aufnahme-Quelle                                }
{           iState  - Quelle an oder abschalten                      }
{                     = FASLE: abschalten                            }
{                     = TRUE: anschalten                             }
{********************************************************************}
Procedure mix4_SetADCSourceR( iSource : Word; iState : Boolean );

var bInR : Byte;

Begin
  bInR := Byte (mix_Read( MIX4_ADCSOURCE_R ) );
  case iSource of
    MIDI,MIDI_L,MIDI_R,CD,CD_L,CD_R,
    LINE,LINE_L,LINE_R,MIC:
      if iState then                             { Eingabekanal setzen }
        bInR := bInR or bMix4SourceBits[iSource]
      else                                      { Eingabekanal lschen }
        bInR := bInR and not bMix4SourceBits[iSource];
  End;
  mix_Write( MIX4_ADCSOURCE_R, bInR );
End;

{********************************************************************}
{ mix4_GetADCSourceL : Aufnahme-Quelle fr linken ADC ermitteln      }
{*------------------------------------------------------------------*}
{ Eingabe : iSource - Aufnahme-Quelle                                }
{ Ausgabe : ID, fr welche ADC-Kanle die angegebene Aufnahme-       }
{           Quelle gesetzt ist.                                      }
{********************************************************************}
Function mix4_GetADCSourceL(iSource : Word) : Word;

var bInL : Byte;

Begin
  bInL := Byte ( mix_Read( MIX4_ADCSOURCE_L ) );
  case iSource of
    MIDI,CD,LINE,MIC:
      if ( bInL and bMix4SourceBits[iSource] ) = bMix4SourceBits[iSource]
        then mix4_GetADCSourceL := CH_BOTH
        else Mix4_GetADCSourceL := CH_None;

    MIDI_L,CD_L,LINE_L:
      if ( bInL and bMix4SourceBits[iSource] <> 0 )
        then mix4_GetADCSourceL := CH_LEFT
        else Mix4_GetADCSourceL := CH_None;

    MIDI_R,CD_R,LINE_R:
      if( bInL and bMix4SourceBits[iSource] <> 0 )
        then mix4_GetADCSourceL := CH_RIGHT
        else Mix4_GetADCSourceL := CH_None;

    else { case else }
      Mix4_GetADCSourceL := CH_None;
  End;
End;

{********************************************************************}
{ mix4_GetADCSourceR : Aufnahme-Quelle fr rechten ADC ermitteln     }
{*------------------------------------------------------------------*}
{ Eingabe : iSource : Aufnahme-Quelle                                }
{ Ausgabe : ID, auf welchen ADC-Kanlen die angegebene Aufnahme      }
{           Quelle gesetzt ist.                                      }
{           Mgliche Ausgaben:  CH_BOTH, CH_LEFT, CH_RIGHT, CH_NONE  }
{********************************************************************}
Function mix4_GetADCSourceR( iSource : Word ) : Word;

var bInR : Byte;

Begin
  bInR := Byte ( mix_Read( MIX4_ADCSOURCE_R ) );

  case iSource of
    MIDI,CD,LINE,MIC:
      if ( bInR and bMix4SourceBits[iSource] ) = bMix4SourceBits[iSource]
        then mix4_GetADCSourceR := CH_BOTH
        else mix4_GetADCSourceR := CH_NONE;

    MIDI_L,CD_L,LINE_L:
      if ( bInR and bMix4SourceBits[iSource] ) <> 0
        then mix4_GetADCSourceR := CH_LEFT
        else mix4_GetADCSourceR := CH_NONE;

    MIDI_R,CD_R,LINE_R:
      if( bInR and bMix4SourceBits[iSource] ) <> 0
        then mix4_GetADCSourceR := CH_RIGHT
        else mix4_GetADCSourceR := CH_NONE;

    else { case else }
      mix4_GetADCSourceR := CH_NONE;
  End;
End;

{********************************************************************}
{ mix4_SetOUTSource : Ausgabe(Wiedergabe)-Quelle setzen              }
{*------------------------------------------------------------------*}
{ Eingabe : iSource - Quelle                                         }
{           iState  - Quelle an oder abschalten                      }
{********************************************************************}
Procedure mix4_SetOUTSource( iSource : Word; iState : Boolean );

var bIn : Byte;

Begin
  bIn := Byte ( mix_Read( MIX4_OUTSOURCE ) );
  case iSource of
    CD,CD_L,CD_R,LINE,LINE_L,LINE_R,MIC:
      if iState then
        bIn := bIn or bMix4SourceBits[iSource] { Setzen }
      else                                               { Lschen }
        bIn := bIn and not bMix4SourceBits[iSource];
  End;
  mix_Write( MIX4_OUTSOURCE, bIn );
End;

{********************************************************************}
{ mix4_GetOUTSource : Ausgabe-Quelle ermitteln                       }
{*------------------------------------------------------------------*}
{ Eingabe : iSource - Aufnahme-Quelle                                }
{ Ausgabe : ID, auf welchen DAC-Kanlen die angegebene Aufnahme      }
{           Quelle gesetzt ist.                                      }
{           Mgliche Ausgaben:  CH_BOTH, CH_LEFT, CH_RIGHT, CH_NONE  }
{********************************************************************}
Function mix4_GetOUTSource( iSource : Word ) : Word;

var bIn : Byte;

Begin
  bIn := Byte ( mix_Read( MIX4_OUTSOURCE ) );

  case iSource of
    CD,LINE,MIC:
      if ( bIn and bMix4SourceBits[iSource] ) <> 0
        then mix4_GetOUTSource := CH_BOTH
        else mix4_GetOUTSource := CH_NONE;

    CD_L,LINE_L:
      if ( bIn and bMix4SourceBits[iSource] ) <> 0
        then mix4_GetOUTSource := CH_LEFT
        else mix4_GetOUTSource := CH_NONE;

    CD_R,LINE_R:
      if ( bIn and bMix4SourceBits[iSource] ) <> 0
        then mix4_GetOUTSource := CH_RIGHT
        else mix4_GetOUTSource := CH_NONE;

    else { case else }
      mix4_GetOUTSource := CH_NONE;
  End;
End;

{********************************************************************}
{ mix4_SetADCGain : Aufnahme Vorverstrker setzen                    }
{*------------------------------------------------------------------*}
{ Eingabe : iGainL, iGainR - 0 :  0 dB                               }
{                            1 :  6 dB                               }
{                            2 : 12 dB                               }
{                            3 : 18 dB                               }
{********************************************************************}
Procedure mix4_SetADCGain( iGainL, iGainR :Word);

var bIn, bGainL, bGainR : Byte;

Begin
  bGainL := Byte ( iGainL and $03 );
  bGainL := bGainl shl 6;
  bGainR := Byte ( iGainR and $03 );
  bGainR := bGainR shl 6;

  bIn := Byte ( ( mix_Read( MIX4_ADCGAIN_L ) and $3F ) or bGainL);
  mix_Write( MIX4_ADCGAIN_L, bIn );

  bIn := Byte ( ( mix_Read( MIX4_ADCGAIN_R ) and $3F ) or bGainR );
  mix_Write( MIX4_ADCGAIN_R, bIn );
End;

{********************************************************************}
{ mix4_GetADCGain : Werte des Aufnahmevorverstrkers ermitteln       }
{*------------------------------------------------------------------*}
{ Eingabe : iChannel - CH_LEFT  : Gain fr linken Kanal              }
{                      CH_RIGHT : Gain fr rechten Kanal             }
{                      andere   : Gain fr beide Kanle              }
{ Ausgabe : Aktuelle Vorwerstrkerwerte                              }
{           Bei Ermittlung des Wertes beider Kanle wird im Lo-Byte  }
{           der Wert fr den linken, im Hi-Byte der Wert fr den     }
{           rechten Kanal geliefert. Sonst wird der erfragte Wert    }
{           immer im Lo-Byte zurckgegeben!                          }
{********************************************************************}
Function mix4_GetADCGain( iChannel : Word ) : Word;

var bGainL, bGainR : Byte;

Begin
  bGainL := Byte ( mix_Read( MIX4_ADCGAIN_L ) );
  bGainL := bGainL and $C0;
  bGainL := bGainL shr 6;

  bGainR := Byte ( mix_Read( MIX4_ADCGAIN_R ) );
  bGainR := bGainR and $C0;
  bGainR := bGainR shr 6;

  case iChannel of
    CH_LEFT:
      mix4_GetADCGain := bGainL;

    CH_RIGHT:
      mix4_GetADCGain := bGainR;

    else { case else }
      mix4_GetADCGain := bGainL * 256 + bGainR;
  End;
End;

{********************************************************************}
{ mix4_SetOUTGain : Ausgabe Vorverstrker setzen                     }
{*------------------------------------------------------------------*}
{ Eingabe : iGainL, iGainR - 0 :  0 dB                               }
{                            1 :  6 dB                               }
{                            2 : 12 dB                               }
{                            3 : 18 dB                               }
{********************************************************************}
Procedure mix4_SetOUTGain( iGainL, iGainR : Word);

var bIn, bGainL, bGainR : Byte;

Begin
  bGainL := Byte ( ( iGainL and $03 ) shl 6 );
  bGainR := Byte ( ( iGainR and $03 ) shl 6 );

  bIn := Byte ( ( mix_Read( MIX4_OUTGAIN_L ) and $3F ) or bGainL );
  mix_Write( MIX4_OUTGAIN_L, bIn );

  bIn := Byte ( ( mix_Read( MIX4_OUTGAIN_R ) and $3F ) or bGainR );
  mix_Write( MIX4_OUTGAIN_R, bIn );
End;

{********************************************************************}
{ mix4_GetOUTGain : Ausgabe Vorverstrker holen                      }
{*------------------------------------------------------------------*}
{ Eingabe : iChannel - CH_LEFT  : Gain fr linken Kanal              }
{                      CH_RIGHT : Gain fr rechten Kanal             }
{                      andere   : Gain fr beide Kanle              }
{ Ausgabe : Aktuelle Vorwerstrkerwerte                              }
{           Bei Ermittlung des Wertes beider Kanle, wird im Lo-Byte }
{           der Wert fr den linken , im Hi-Byte der Wert fr den    }
{           rechten Kanal geliefert. Sonst wird der erfragte Wert    }
{           immer im Lo-Byte angegeben!                              }
{********************************************************************}
Function mix4_GetOUTGain( iChannel:Word ) : Word;

var bGainL, bGainR : Byte;
Begin
  bGainL := Byte ( mix_Read( MIX4_OUTGAIN_L ) );
  bGainL := bGainL and $C0;
  bGainL := bGainL shr 6;
  bGainR := Byte ( mix_Read( MIX4_OUTGAIN_R ) );
  bGainR := bGainR and $C0;
  bGainR := bGainR shr 6;

  case iChannel of
    CH_LEFT:
      mix4_GetOUTGain := bGainL;

    CH_RIGHT:
      mix4_GetOUTGain := bGainR;

    CH_BOTH:
      mix4_GetOUTGain := bGainL * 256 + bGainR ;

    else { case else }
      mix4_GetOUTGain := 0;
  End;
End;

{********************************************************************}
{ mix4_SetAGC : Automatic Gain Control aktivieren / deaktivieren     }
{*------------------------------------------------------------------*}
{ Eingabe : iState - FALSE: AGC deaktivieren (0 dB )                 }
{                    TRUE:  AGC aktivieren ( 20dB )                  }
{*------------------------------------------------------------------*}
{ Info : - AGC wird fr Aufnahmen mit einem Mikro verwendet und      }
{          erlaubt das Zuschalten eines Vorverstrkers, der das      }
{          Mikrophon-Signal um 20dB verstrkt.                       }
{********************************************************************}
Procedure mix4_SetAGC( iState : Word );

var bIn : Byte;

Begin
   bIn := Byte ( mix_Read( MIX4_AGC ) );
   if iState = 0 then
     mix_Write( MIX4_AGC, bIn and not MIX4_AGCON )
   else
     mix_Write( MIX4_AGC, bIn or MIX4_AGCON );
End;

{********************************************************************}
{ mix4_GetAGC : Automatic Gain Control holen                         }
{*------------------------------------------------------------------*}
{ Ausgabe: TRUE  : AGC aktiviert                                     }
{          FALSE : AGC deaktiviert                                   }
{********************************************************************}
Function mix4_GetAGC : Boolean;

Begin
   mix4_GetAGC := ( ( mix_Read( MIX4_AGC ) and MIX4_AGCON ) <> 0 );
End;

{********************************************************************}
{ mix4_SetTrebble : Hhen setzen                                     }
{--------------------------------------------------------------------}
{ Eingabe : iTrebbleL, iTrebbleR : Hhe fr linken und rechten       }
{                                  Kanal setzen. Werte von 0 bis 15  }
{                                  entsprechen -14 dB bis 14 dB in   }
{                                  Schritten zu 2 dB.                }
{--------------------------------------------------------------------}
{ Info : Die Soundblaster16 Karte verfgt ber einen Vorverstrker   }
{        dessen Klangmerkmale gendert werden knnen.                }
{********************************************************************}
Procedure mix4_SetTrebble( iTrebbleL, iTrebbleR : Word);

Begin
  mix_Write( MIX4_TREBLE_L, Byte( iTrebbleL shl 4 ) );
  mix_Write( MIX4_TREBLE_R, Byte( iTrebbleR shl 4 ) );
End;

{********************************************************************}
{ mix4_GetTrebble : Aktuelle Hheneinstellung holen                  }
{*------------------------------------------------------------------*}
{ Eingabe : iChannel : CH_LEFT  : fr linken Kanal                   }
{                      CH_RIGHT : fr rechten Kanal                  }
{                      CH_BOTH  : fr beide Kanle                   }
{ Ausgabe : Aktuelle Vorverstrkerwerte                              }
{           Bei Ermittlung des Wertes beider Kanle wird im Lo-Byte  }
{           der Wert fr den linken, im Hi-Byte der Wert fr den     }
{           rechten Kanal geliefert. Sonst wird der erfragte Wert    }
{           immer im Lo-Byte zurckgegeben.                          }
{********************************************************************}
Function mix4_GetTrebble( iChannel : Word ) : Word;

var bTrebbleL, bTrebbleR : Byte;

Begin
  bTrebbleL := mix_Read( MIX4_TREBLE_L ) shr 4;
  bTrebbleR := mix_Read( MIX4_TREBLE_R ) shr 4;

  case iChannel of
    CH_LEFT:  mix4_GetTrebble := bTrebbleL;
    CH_RIGHT: mix4_GetTrebble := bTrebbleR;
    CH_BOTH:  mix4_GetTrebble := bTrebbleL * 256 + bTrebbleR ;
    else      mix4_GetTrebble := 0
  End;
End;

{********************************************************************}
{ mix4_SetBass : Bsse setzen                                        }
{--------------------------------------------------------------------}
{ Eingabe : iBassL, iBassR : Bsse fr linken und rechten Ausgabe-   }
{                            kanal setzen. Werte von 0 bis 15        }
{                            entsprechen -14 dB bis 14 dB in         }
{                            Schritten zu 2 dB.                      }
{--------------------------------------------------------------------}
{ Info - Die Soundblaster16 Karte verfgt ber einen Vorverstrker   }
{        dessen Klangmerkmale gendert werden knnen.                }
{********************************************************************}
Procedure mix4_SetBass( iBassL, iBassR : Word );

Begin
  mix_Write( MIX4_BASS_L, iBassL shl 4 );
  mix_Write( MIX4_BASS_R, iBassR shl 4 );
End;

{********************************************************************}
{ mix4_GetBass : Aktuelle Basseinstellung  holen                     }
{*------------------------------------------------------------------*}
{ Eingabe : iChannel : CH_LEFT  : fr linken Kanal                   }
{                      CH_RIGHT : fr rechten Kanal                  }
{                      CH_BOTH  : fr beide Kanle                   }
{ Ausgabe : Aktuelle Vorwerstrkerwerte                              }
{           Bei Ermittlung des Wertes beider Kanle, wird im Lo-Byte }
{           der Wert fr den linken , im Hi-Byte der Wert fr den    }
{           rechten Kanal geliefert. Sonst wird der erfragte Wert    }
{           immer im Lo-Byte angegeben!                              }
{********************************************************************}
Function mix4_GetBass( iChannel : Word ) : Word;

var bBassL, bBassR : Byte;

Begin
  bBassL := Byte( mix_Read( MIX4_BASS_L ) shr 4 );
  bBassR := Byte( mix_Read( MIX4_BASS_R ) shr 4);

  case iChannel of
    CH_LEFT:  mix4_GetBass := bBassL;
    CH_RIGHT: mix4_GetBass := bBassR;
    CH_BOTH:  mix4_GetBass := bBassL * 256 + bBassR ;
    else      mix4_GetBass := 0;
  End;
End;

End.