{**********************************************************************}
{*                         M O U S E P . P A S                        *}
{*--------------------------------------------------------------------*}
{*    Aufgabe        : Stellt verschiedene Funktionen zur Arbeit mit  *}
{*                     der Maus zur Verfgung                         *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  : 21.04.1989                                     *}
{*    letztes Update :  3.12.1991                                     *}
{**********************************************************************}

uses Dos;                                         { DOS-Unit einbinden }

{$L mousepa}                           { das Assembler-Modul einbinden }

{== Deklaration externer Funktionen ===================================}

{$F+}                                           { die Funktion ist FAR }
procedure AssHand ; external ;           { der Assembler-Event-Handler }
{$F-}                            { FAR-Funktionen nicht mehr erzwingen }

{== Konstanten ========================================================}

const

{-- Event-Codes -------------------------------------------------------}

      EV_MOU_MOVE     =   1;                             { Maus bewegt }
      EV_LEFT_PRESS   =   2;            { linker Mausknopf niedergedr. }
      EV_LEFT_REL     =   4;            { linker Mausknopf losgelassen }
      EV_RIGHT_PRESS  =   8;           { rechter Mausknopf niedergedr. }
      EV_RIGHT_REL    =  16;           { rechter Mausknopf losgelassen }
      EV_MOU_ALL      =  31;                        { alle Maus-Events }

      LBITS           =   6;          { EV_LEFT_PRESS or EV_LEFT_REL   }
      RBITS           =  24;          { EV_RIGHT_PRESS or EV_RIGHT_REL }

      KEIN_BEREICH    = 255;         { Maus-Cursor nicht in Bereich xy }

      PtrSameChar     = $00ff;            { gleiches Zeichen           }
      PtrSameCol      = $00ff;            { gleiche Farbe              }
      PtrInvCol       = $7777;            { Farbe invertiert           }
      PtrSameColB     = $807f;            { gleiche Farbe blinkend     }
      PtrInvColB      = $F777;            { invertierte Farbe blinkend }

      UND             = 0;      { Event-Verknpfungen fr MouEventWait }
      ODER            = 1;

      ZV              = #13#10;                       { Zeilenvorschub }

{== Typdeklarationen ==================================================}

type  FKTPTR   = longint;                 { Adresse einer FAR-Funktion }
      PTRVIEW  = longint;                      { Maske fr Maus-Cursor }
      BEREICH  = record                {  beschreibt einen Mausbereich }
                   x1,                 { Koordinaten der oberen linken }
                   y1,                 { und unteren rechten Ecke des  }
                   x2,                 { spezifizierten Bereichs       }
                   y2      : byte;
                   PtrMask : PTRVIEW;          { Maske fr Maus-Cursor }
                 end;
      BERARRAY = array [0..100] of BEREICH;
      BERPTR   = ^BERARRAY;
      PTRREC   = record                    { dient dem Zugriff auf die }
                   Ofs : word;             { Bestandteile eines belie- }
                   Seg : word;             { bigen Pointers            }
                 end;
      PTRVREC  = record                   { dient dem Zugriff auf die  }
                   ScreenMask : word;     { Bestandteile eines PTRVIEW }
                   CursorMask : word;
                 end;
      BERBUF   = array [0..10000] of byte;            { Bereichspuffer }
      BBPTR    = ^BERBUF;            { Zeiger auf einen Bereichspuffer }

{== globale Variablen =================================================}

var  AnzBereiche,                                { Anzahl der Bereiche }
     TLine,                                       { Anzahl Text-Zeilen }
     TCol     : byte;                            { Anzahl Text-Spalten }
     MouAvail : boolean;               { ist TRUE, wenn Maus verfgbar }
     OldPtr,                 { altes Erscheinungsbild des Maus-Cursors }
     StdPtr   : PTRVIEW;          { Maske fr den Standard-Maus-Cursor }
     BufPtr   : BBPTR;      { Zeiger auf Puffer fr Bereichs-Erkennung }
     AktBerPtr: BERPTR;             { Pointer auf akt. Bereichs-Vektor }
     BLen     : integer;            { Lnge des Bereichspuffer in Byte }
     ExitOld  : pointer;           { Zeiger auf die alte Exit-Prozedur }

{-- Variablen, die mit jedem Aufruf des Maus-Handlers geladen werden --}

     MouBer,                                  { aktueller Maus-Bereich }
     MouCol,                            { Mausspalte (Text-Bildschirm) }
     MouRow   : byte;                   { Mauszeile  (Text-Bildschirm) }
     MouEvent : integer;                                 { Event-Maske }

{-- Variablen, die nur bei Eintritt eines erwarteten Events durch den -}
{-- Maus-Handler geladen werden                                       -}

     EvBer,                   { Bereich, in dem sich die Maus befindet }
     EvCol,                                               { Mausspalte }
     EvRow : byte;                                         { Mauszeile }

{**********************************************************************}
{*  MouPtrMask: fgt die Cursor- und Screen-Mask aus einer Bitmaske   *}
{*              fr das Zeichen und die Farbe zusammen                *}
{**------------------------------------------------------------------**}
{*  Eingabe: Zeichen = Bitmaske fr die Cursor- und Screen-Mask in    *}
{*                     Bezug auf das Zeichen                          *}
{*           Farbe   = Bitmaske fr die Cursor- und Screen-Mask in    *}
{*                     Bezug auf die Farbe des Zeichens               *}
{*  Ausgabe: die Cursor- und die Screen-Mask als ein Wert vom Typ     *}
{*           PtrView                                                  *}
{*  Info:    fr Zeichen und Farbe knnen die Konstanten PtrSameChar, *}
{*           PtrSameCol, PtrSameColB, PtrInvCol und PtrInvColB sowie  *}
{*           die Ergebinsse der Funktionen PtrDifChar und PtrDifCol   *}
{*           bergeben werden                                         *}
{**********************************************************************}

function MouPtrMask( Zeichen, Farbe  : word ) : PTRVIEW;

var Mask : PTRVIEW;         { die zu erstellen Cursor- und Screen-Mask }

begin
  PTRVREC( Mask ).ScreenMask := ( ( Farbe and $ff ) shl 8 ) +
                                ( Zeichen and $ff );
  PTRVREC( Mask ).CursorMask := ( Farbe and $ff00 ) + ( Zeichen shr 8 );
  MouPtrMask := Mask;                { Maske an Aufrufer zurckliefern }
end;

{**********************************************************************}
{*  PtrDifChar: erstellt den Zeichenteil der Cursor- und Screen-Mask  *}
{*              in Bezug auf das Zeichen                              *}
{**------------------------------------------------------------------**}
{*  Eingabe: ASCII-Code des Zeichens, das den Maus-Cursor bilden soll *}
{*  Ausgabe: Cursor- und Screen-Mask fr dieses Zeichen               *}
{*  Info:    das Funktionsergebnis sollte mit Hilfe der Funktion      *}
{*           MouPtrMask weiterverarbeitet werden                      *}
{**********************************************************************}

function PtrDifChar( Zeichen : byte ) : word;

begin
  PtrDifChar := Zeichen shl 8;
end;

{**********************************************************************}
{*  PtrDifCol: erstellt den Zeichenteil der Cursor- und Screen-Mask   *}
{*             in Bezug auf die Farbe des Maus-Cursors                *}
{**------------------------------------------------------------------**}
{*  Eingabe: Farbe des Zeichens, das den Maus-Cursor bilden soll      *}
{*  Ausgabe: Cursor- und Screen-Mask fr diese Farbe                  *}
{*  Info:    das Funktionsergebnis sollte mit Hilfe der Funktion      *}
{*           MouPtrMask weiterverarbeitet werden                      *}
{**********************************************************************}

function PtrDifCol( Farbe : byte ) : word;

begin
  PtrDifCol := Farbe shl 8;
end;

{**********************************************************************}
{*  MouDefinePtr: bergibt dem Maustreiber die Cursor- und Screen-    *}
{*                Mask, die fortan das Erscheinungsbild des Maus-     *}
{*                Cursors bestimmen                                   *}
{**------------------------------------------------------------------**}
{*  Eingabe: Mask = die Cursor- und Screen-Mask als ein Paramter vom  *}
{*                  Typ PTRVIEW                                       *}
{*  Info:    - der Parameter Mask sollte mit Hilfe der Funktion       *}
{*             MouPtrMask erstellt worden sein                        *}
{*           - die hherwertigen 16-Bit von Mask reprsentieren die   *}
{*             Screen-Mask, die niederwertigen 16 Bit die Cursor-Mask *}
{**********************************************************************}

procedure MouDefinePtr( Mask : PTRVIEW );

var Regs : Registers;              { Prozessorregs fr Interruptaufruf }

begin
  if OldPtr <> Mask then       { Vernderung gegenber letztem Aufruf? }
    begin                                                         { Ja }
      Regs.AX := $000a;           { Fktnr. fr "Set text pointer type" }
      Regs.BX := 0;                       { Software-Cursor einstellen }
      Regs.CX := PTRVREC( Mask ).ScreenMask;   { Lo-Word ist AND-Maske }
      Regs.DX := PTRVREC( Mask ).CursorMask;   { Hi-Word ist XOR-Maske }
      Intr( $33, Regs);                        { Maus-Treiber aufrufen }
      OldPtr := Mask;                          { neue Bit-Maske merken }
    end;
end;

{**********************************************************************}
{*  MouEventHandler: Wird vom Maustreiber ber die Assembler-Routine  *}
{*                   AssHand aufgerufen, sobald ein Ereignis in Ver-  *}
{*                   bindung mit der Maus eintritt                    *}
{**------------------------------------------------------------------**}
{*  Eingabe: EvFlags  = die Event-Mask                                *}
{*           ButState = aktueller Status der Mausknpfe               *}
{*           X, Y     = die aktuelle Position des Maus-Cursors in Be- *}
{*                      zug auf den Textbildschirm                    *}
{**********************************************************************}

procedure MouEventHandler( EvFlags, ButState, x, y : integer );

var NeuBer : byte;                         { Nummer des neuen Bereichs }

begin

  MouEvent := MouEvent and not(1);                  { Bit 0 ausBLenden }
  MouEvent := MouEvent or ( EvFlags and 1 );         { Bit 0  kopieren }

  if ( EvFlags and LBITS ) <> 0 then { linker MK. losg. oder niederg.? }
    begin                                                         { Ja }
      MouEvent := MouEvent and not( LBITS ); { bisherigen Status ausbl.}
      MouEvent := MouEvent or ( EvFlags and LBITS );    { Status einb. }
    end;

  if ( EvFlags and RBITS ) <> 0 then { rechter MK. losg. oder niederg.?}
    begin                                                         { Ja }
      MouEvent := MouEvent and not( RBITS ); { bisherigen Status ausbl.}
      MouEvent := MouEvent or ( EvFlags and RBITS );   { Status einbl. }
    end;

  MouCol := x;                       { Spalte in Textspalten umrechnen }
  MouRow := y;                         { Zeile in Textzeilen umrechnen }

  {-- Bereich ermitteln, in dem sich die Maus befindet und fest-   ----}
  {-- stellen, ob sich der Bereich seit dem letzten letzten Aufruf ----}
  {-- des Handlers verndert hat. In diesem Fall mu das Erschei-  ----}
  {-- nungsbild des Maus-Cursors neu definiert werden.             ----}

  NeuBer := BufPtr^[ MouRow * TCol + MouCol ];         { Bereich holen }
  if NeuBer <> MouBer then                            { neuer Bereich? }
    begin                                                         { Ja }
      if Neuber = KEIN_BEREICH then        { auerhalb eines Bereichs? }
        MouDefinePtr( StdPtr )              { Ja, Standard-Maus-Cursor }
      else                                     { Nein, Bereich bekannt }
        MouDefinePtr( AktBerPtr^[ NeuBer ].PtrMask );
    end;
  MouBer := NeuBer;           { Bereichsnummer in globaler Var. merken }
end;

{**********************************************************************}
{*  MouIBufFill: speichert den Bereichscode fr einen Mausbereich     *}
{*               innerhalb des modulinternen Bereichsspeichers        *}
{**------------------------------------------------------------------**}
{*  Eingabe: x1, y1 = obere linke Ecke des Mausbereichs               *}
{*           x2, y2 = untere rechte Ecke des Mausbereichs             *}
{*           Code   = der Bereichscode                                *}
{**********************************************************************}

procedure MouIBufFill( x1, y1, x2, y2, Code : byte );

var Index   : integer;                   { zeigt auf bearbeitetes Feld }
    Spalte,                                          { Schleifenzhler }
    Zeile   : byte;

begin
  for Zeile:=y1 to y2 do            { die einzelnen Zeilen durchlaufen }
    begin
      Index := Zeile  * TCol + x1;            { erster Index der Zeile }
      for Spalte:=x1 to x2 do   { die Spalten in der Zeile durchlaufen }
        begin
          BufPtr^[ Index ] := Code;                   { Code speichern }
          inc( Index );            { Index auf das nchste Feld setzen }
        end;
    end;
end;

{**********************************************************************}
{*  MouDefBereich: erlaubt die Registrierung verschiedener Bildschirm-*}
{*                 bereiche, die als unterschiedliche Mausbereiche    *}
{*                 erkannt und mit einem speziellen Maus-Cursor ver-  *}
{*                 sehen werden knnen                                *}
{**------------------------------------------------------------------**}
{*  Eingabe: Anzahl = Anzahl der Bildschirmbereiche                   *}
{*           BPtr   = Zeiger auf das Array, in dem die einzelnen      *}
{*                    Bereiche als jeweils eine Struktur vom Typ      *}
{*                    BEREICH beschrieben werden                      *}
{*  Info:    - den freibleibenden Bildschirmbereichen wird der Code   *}
{*             KEIN_BEREICH zugeordnet                                *}
{*           - beim Eintritt des Maus-Cursors in einen der inner-     *}
{*             erhalb des Arrays aufgefhrten Mausbereiche schaltet   *}
{*             der Event-Handler sofort auf das Erscheinungsbild      *}
{*             des Maus-Cursors in diesem Bereich um                  *}
{**********************************************************************}

procedure MouDefBereich( Anzahl : byte; BPtr : BERPTR );

var AktBer,                            { Nummer des aktuellen Bereichs }
    Bereich : byte;                                  { Schleifenzhler }

begin
  AktBerPtr := BPtr;                      { Pointer auf Vektor und An- }
  AnzBereiche := Anzahl;                  { zahl der Bereiche merken   }
  FillChar( BufPtr^, BLen, KEIN_BEREICH );  {alle Elemente=KEIN_BEREICH}
  for Bereich:=0 to anzahl-1 do   { die einzelnen Bereiche durchlaufen }
   with BPtr^[ Bereich ] do
     MouIBufFill( x1, y1, x2, y2, Bereich );

  {-- Mauscursor neu definieren ---------------------------------------}
  AktBer := BufPtr^[ MouRow * TCol + MouCol ];         { Bereich holen }
  if Aktber = KEIN_BEREICH then            { auerhalb eines Bereichs? }
    MouDefinePtr( StdPtr )                  { Ja, Standard-Maus-Cursor }
  else                                         { Nein, Bereich bekannt }
    MouDefinePtr( BPtr^[ AktBer ].PtrMask );
end;

{**********************************************************************}
{*  MouEventWait: wartet auf den Eintritt eines bestimmten Ereig-     *}
{*                nisses in Verbindung mit der Maus                   *}
{**------------------------------------------------------------------**}
{*  Eingabe: TYP        = Art der Verknpfung zwischen den verschie-  *}
{*                        denen Ereignissen                           *}
{*           WAIT_EVENT = Bitmaske, die die zu erwartenden Ereignisse *}
{*                        spezifiziert                                *}
{*  Ausgabe: Bitmaske des eingetretenen Ereignisses                   *}
{*  Info:    - WAIT_EVENT kann durch ODER Verknpfung der verschie-   *}
{*             denen Konstanten wie EV_MOU_MOVE, EV_LEFT_PRESS etc.   *}
{*             erstellt werden                                        *}
{*           - Als Verknpfungstyp knnen die Konstanten UND und ODER *}
{*             angegeben werden. Wird UND gewhlt, kehrt die Funktion *}
{*             erst dann zum Aufrufer zurck, wenn alle spezifizier-  *}
{*             ten Ereignisse eingetreten sind. ODER begngt sich     *}
{*             hingegen mit dem Eintritt mindestens eines der ver-    *}
{*             schiedenen Ereignisse.                                 *}
{**********************************************************************}

function MouEventWait( Typ : BYTE; WaitEvent : integer ) : integer;

var AktEvent : integer;
    Zeile,
    Spalte   : byte;
    Ende     : boolean;

begin
   Spalte := MouCol;                    { aktuelle Mausposition merken }
   Zeile := MouRow;
   Ende := false;

  repeat
    {-- warten bis eines der Ereignisse eintritt ----------------------}

    if Typ = UND then                               { UND-Verknpfung? }
      repeat                    { Ja, alle Ereignisse mssen eintreten }
        AktEvent := MouEvent;               { aktuelles Ereignis holen }
      until AktEvent = WaitEvent
    else                                            { ODER-Verknpfung }
      repeat                   { wenigstens ein Ereignis mu eintreten }
        AktEvent := MouEvent;               { aktuelles Ereignis holen }
      until ( AktEvent and WaitEvent ) <> 0;

    AktEvent := AktEvent and WaitEvent;  { nur Event-Bits stehenlassen }

    {-- wenn auf die Bewegung der Maus gewartet wird, wird das Ereig-- }
    {-- nis nur akzeptiert, wenn der Maus-Cursor in eine anderer Zei-- }
    {-- le und/oder Spalte des Textbildschirms bewegt wurde          - }

    if ( ( (WaitEvent and EV_MOU_MOVE) <> 0 )  and
          ( Spalte = MouCol ) and  ( Zeile = MouRow ) ) then
      begin             { Maus bewegt, aber gleiche Bildschirmposition }
        AktEvent := AktEvent and not( EV_MOU_MOVE ); { Move-Bit ausbl. }
        Ende := ( AktEvent <> 0);              { bleiben Events brig? }
      end
    else                                        { Ereignis eingetreten }
      Ende := TRUE;
  until Ende;

  EvCol := MouCol;                        { aktuelle Mausposition und  }
  EvRow := MouRow;                        { -bereich in globalen Vari- }
  EvBer := MouBer;                        { aBLen festhalten           }

  MouEventWait := AktEvent;
end;

{**********************************************************************}
{*  MouISetEventHandler: installiert einen Event-Handler, der beim    *}
{*                       Eintritt bestimmter Ereignisse in Verbindung *}
{*                       mit der Maus vom Maustreiber aufgerufen wird.*}
{**------------------------------------------------------------------**}
{*  Eingabe: EVENT = Bitmaske, die die Ereignisse beschreibt, bei     *}
{*                   deren Eintritt der Event-Handler aufgerufen      *}
{*                   werden soll                                      *}
{*           FPTR  = Zeiger auf den Event-Handler vom Typ FKTPTR      *}
{*  Info:    - EVENT kann durch ODER Verknpfung der verschiedenen    *}
{*             Konstanten wie EV_MOU_MOVE, EV_LEFT_PRESS etc. er-     *}
{*             stellt werden                                          *}
{*           - Der Event-Handler mu eine FAR-Prozedur sein und darf  *}
{*             keines der ihm bergebenen Prozessorregister verndern *}
{**********************************************************************}

procedure MouISetEventHandler( Event : integer; FPtr : FKTPTR );

var Regs : Registers;              { Prozessorregs fr Interruptaufruf }

begin
  Regs.AX := $000C;                   { Fktnr. fr "Set Mouse Handler" }
  Regs.CX := event;                                { Event-Maske laden }
  Regs.DX := PTRREC( FPtr ).Ofs;          { Offsetadresse des Handlers }
  Regs.ES  := PTRREC( FPtr ).Seg;        { Segmentadresse des Handlers }
  Intr( $33,  Regs );                          { Maus-Treiber aufrufen }
end;

{**********************************************************************}
{*  MouIGetX: ermittelt die Textspalte, in der sich der Maus-Cursor   *}
{*            befindet                                                *}
{**------------------------------------------------------------------**}
{*  Ausgabe: Spalte des Maus-Cursors in Bezug auf den Textbildschirm  *}
{**********************************************************************}

function MouIGetX : byte;

var Regs : Registers;              { Prozessorregs fr Interruptaufruf }

begin
  Regs.AX := $0003;                 { Fktnr.: fr "Get mouse position" }
  Intr( $33,  Regs );                          { Maus-Treiber aufrufen }
  MouIGetX := Regs.CX shr 3;           { Spalte umrechnen und zurckl. }
end;

{**********************************************************************}
{*  MouIGetY: ermittelt die Textzeile, in der sich der Maus-Cursor    *}
{*            befindet                                                *}
{**------------------------------------------------------------------**}
{*  Ausgabe: Zeile des Maus-Cursors in Bezug auf den Textbildschirm   *}
{**********************************************************************}

function MouIGetY : byte;

var Regs : Registers;              { Prozessorregs fr Interruptaufruf }

begin
  Regs.AX := $0003;                 { Fktnr.: fr "Get mouse position" }
  Intr( $33, Regs );                           { Maus-Treiber aufrufen }
  MouIGetY := Regs.DX shr 3;            { Zeile umrechnen und zurckl. }
end;

{**********************************************************************}
{*  MouShowMouse: bringt den Maus-Cursor auf den Bildschirm           *}
{**------------------------------------------------------------------**}
{*  Info: die Aufrufe von MouShowMouse und MouHidemOuse mssen aus-   *}
{*        balanciert sein, damit sie Wirkung zeigen                   *}
{**********************************************************************}

procedure MouShowMouse;

var Regs : Registers;              { Prozessorregs fr Interruptaufruf }

begin
  Regs.AX := $0001;                         { Fktnr.: fr "Show Mouse" }
  Intr( $33, Regs );                           { Maus-Treiber aufrufen }
end;

{**********************************************************************}
{*  MouHideMouse: entfernt den Maus-Cursor vom Bildschirm             *}
{**------------------------------------------------------------------**}
{*  Info: die Aufrufe von MouShowMouse und MouHidemOuse mssen aus-   *}
{*        balanciert sein, damit sie Wirkung zeigen                   *}
{**********************************************************************}

procedure MouHideMouse;

var Regs : Registers;              { Prozessorregs fr Interruptaufruf }

begin
  Regs.AX := $0002;                          { Fktnr. fr "Hide Mouse" }
  Intr( $33, Regs);                            { Maus-Treiber aufrufen }
end;

{**********************************************************************}
{*  MouSetMoveArea: legt den Bewegungsbereich fr den Maus-Cursor     *}
{*                  fest                                              *}
{**------------------------------------------------------------------**}
{*  Eingabe:  x1, y1 = Koordinate der oberen linken Bereichsecke      *}
{*            x2, y2 = Koordinate der unteren rechten Bereichsecke    *}
{*  Info:    - Die Ordinaten beziehen sich auf den Textbildschirm     *}
{*             und nicht auf den virtuellen Grafikbildschirm des      *}
{*             Maustreibers                                           *}
{**********************************************************************}

procedure MouSetMoveArea( x1, y1, x2, y2 : byte );

var Regs : Registers;              { Prozessorregs fr Interruptaufruf }

begin
  Regs.AX := $0008;                 { Fktnr. fr "Set vertical Limits" }
  Regs.CX := integer( y1 ) shl 3;           { Umrechnung in virtuellen }
  Regs.DX := integer( y2 ) shl 3;           { Mausbildschirm           }
  Intr( $33, Regs );                           { Maus-Treiber aufrufen }
  Regs.AX := $0007;               { Fktnr. fr "Set horizontal Limits" }
  Regs.CX := integer( x1 ) shl 3;           { Umrechnung in virtuellen }
  Regs.DX := integer( x2 ) shl 3;           { Mausbildschirm           }
  Intr( $33, Regs );                           { Maus-Treiber aufrufen }
end;

{**********************************************************************}
{*  MouSetSpeed: legt des Verhltnis zwischen der Mausbewegung und    *}
{*               der daraus resultierenden Bewegung des Maus-Cursors  *}
{*               fest                                                 *}
{**------------------------------------------------------------------**}
{*  Eingabe: XSpeed = Geschwindigkeit in X-Richtung                   *}
{*           YSpeed = Geschwindigkeit in Y-Richtung                   *}
{*  Info:    - Beide Parameter beziehen sich auf die Einheit          *}
{*               Mickey / 8 Pixel                                     *}
{**********************************************************************}

procedure MouSetSpeed( XSpeed, YSpeed : integer );

var Regs : Registers;              { Prozessorregs fr Interruptaufruf }

begin
  Regs.AX := $000f;              { Fktnr. "Set mickeys to pixel ratio" }
  Regs.CX := XSpeed;
  Regs.DX := YSpeed;
  Intr( $33, Regs);                            { Maus-Treiber aufrufen }
end;

{**********************************************************************}
{*  MouMovePtr: bewegt den Maus-Cursor an eine bestimmte Bildschirm-  *}
{*              Position                                              *}
{**------------------------------------------------------------------**}
{*  Eingabe: COL = die neue Bildschirmspalte des Maus-Cursors         *}
{*           ROW = die neue Bildschirmzeile des Maus-Cursors          *}
{*  Info:    - Die Ordinaten beziehen sich auf den Textbildschirm     *}
{*             und nicht auf den virtuellen Grafikbildschirm des      *}
{*             Maustreibers                                           *}
{**********************************************************************}

procedure MouMovePtr( Col, Row : byte );

var Regs   : Registers;            { Prozessorregs fr Interruptaufruf }
    NeuBer : byte;              { Bereich, in den die Maus bewegt wird }

begin
  Regs.AX := $0004;              { Fktnr. "Set mouse pointer position" }
  MouCol := col;                              { Koordinate in globalen }
  MouRow := row;                              { Variablen speichern    }
  Regs.CX := integer( col ) shl 3;      { Koordinaten umrechnen und in }
  Regs.DX := integer( row ) shl 3;      { globalen Variablen speichern }
  Intr( $33, Regs );                           { Maus-Treiber aufrufen }

  NeuBer := BufPtr^[ Row * TCol + Col ];               { Bereich holen }
  if NeuBer <> MouBer then                            { neuer Bereich? }
    begin                                                         { Ja }
      if Neuber = KEIN_BEREICH then        { auerhalb eines Bereichs? }
        MouDefinePtr( StdPtr )              { Ja, Standard-Maus-Cursor }
      else                                     { Nein, Bereich bekannt }
        MouDefinePtr( AktBerPtr^[ NeuBer ].PtrMask );
    end;
  MouBer := NeuBer;           { Bereichsnummer in globaler Var. merken }
end;

{**********************************************************************}
{*  MouSetDefaultPtr: Definiert das Erscheinungsbild des Maus-Cursors *}
{*                    fr Bildschirmbereiche, die nicht explizit als  *}
{*                    Mausbereiche gekennzeichnet wurden              *}
{**------------------------------------------------------------------**}
{*  Eingabe: Standard = Cursor- und Screen-Mask fr den Maus-Cursor   *}
{*  Info:    - Der bergebene Parameter sollte mit Hilfe der Funktion *}
{*             MouPtrMask erstellt worden sein                        *}
{**********************************************************************}

procedure MouSetDefaultPtr( Standard : PTRVIEW );

begin
  StdPtr := Standard;              { Bit-Maske in globaler Var. merken }

  {-- befindet sich die Maus im Augenblick in keinem Bereich, wird ----}
  {-- direkt auf das neue Erscheinungsbild umgeschaltet            ----}

  if MouBer = KEIN_BEREICH then                   { in keinem Bereich? }
   MouDefinePtr( Standard );                                    { Nein }
end;

{**********************************************************************}
{*  MouEnd: wird zur Beendigung der Arbeit mit den Funktionen und     *}
{*          Prozeduren aus dem Maus-Modul aufgerufen                  *}
{**------------------------------------------------------------------**}
{*  Info:   - Die Prozedur mu vom Anwendungsprogramm nicht explizit  *}
{*            aufgerufen werden, da die Funktion MouInit sie als      *}
{*            Exit-Prozedur definiert                                 *}
{**********************************************************************}

{$F+}           { mu FAR sein, damit Aufruf als Exit-Prozedur mglich }

procedure  MouEnd;

var Regs : Registers;              { Prozessorregs fr Interruptaufruf }

begin
  MouHideMouse;                 { Maus-Cursor vom Bildschirm entfernen }
  Regs.AX := 0;                              { Reset des Maus-Treibers }
  Intr( $33, Regs);                            { Maus-Treiber aufrufen }

  FreeMem( BufPtr, BLen );     { allokierten Speicher wieder freigeben }

  ExitProc := ExitOld;        { wieder alte Exit-Prozedur installieren }
end;

{$F-}                                      { keine FAR-Prozeduren mehr }

{**********************************************************************}
{*  MouInit: leitet die Arbeit mit den verschiedenen Funktionen und   *}
{*           Prozeduren des Maus-Moduls ein und initialisiert die     *}
{*           verschiedenen Variablen                                  *}
{**------------------------------------------------------------------**}
{*  Eingabe: Spalten = Anzahl der Bildschrimspalten                   *}
{*           Zeilen  = Anzahl der Bildschirmzeilen                    *}
{*  Ausgabe: TRUE, wenn ein Maustreiber installiert ist, sonst FALSE  *}
{*  Info:    - Diese Funktion mu von einem Anwendungsprogrmm als     *}
{*             erste der verschiedenen Prozeduren und Funktionen      *}
{*             dieses Moduls aufgerufen werden                        *}
{**********************************************************************}

function MouInit( Spalten, Zeilen : byte ) : boolean;

var Regs : Registers;              { Prozessorregs fr Interruptaufruf }

begin
  TLine := Zeilen;                     { Anzahl Zeilen- und Spalten in }
  TCol  := Spalten;                    { globalen Variablen speichern  }

  ExitOld := ExitProc;              { Adresse der Exit-Prozedur merken }
  ExitProc := @MouEnd;           { MouEnd als Exit-Prozedur definieren }

  {-- Puffer fr Maus-Bereiche allokieren und fllen ------------------}

  BLen := TLine * TCol;             { Anzahl der Zeichen im Bildschirm }
  GetMem( BufPtr, BLen );        { internen Bereichs-Puffer allokieren }
  MouIBufFill( 0, 0, TCol-1, TLine-1, KEIN_BEREICH );

  Regs.AX := 0;                         { Mouse-Treiber initialisieren }
  Intr( $33, Regs );                           { Maus-Treiber aufrufen }
  MouInit := ( Regs.AX <> 0 );             { Maus-Treiber installiert? }

  MouSetMoveArea( 0, 0, TCol-1, TLine-1 );    { Bewegunsbereich setzen }

  MouCol   := MouIGetX;                     { aktuelle Mausposition in }
  MouRow   := MouIGetY;                     { globale Variablen laden  }
  MouBer   := KEIN_BEREICH;            { Maus-Cursor in keinem Bereich }
  MouEvent := EV_LEFT_REL or EV_RIGHT_REL;   { kein Mausknopf gedrckt }
  StdPtr   := MouPtrMask( PTRSAMECHAR, PTRINVCOL ); { Std.-Maus-Cursor }
  OldPtr   := PTRVIEW( 0 );

 {-- den Assembler-Event-Handler "AssHand" installieren ---------------}
 MouISetEventHandler( EV_MOU_ALL, FKTPTR(@AssHand) );

end;

{***********************************************************************
*                          H A U P T P R O G R A M M                   *
***********************************************************************}

const Bereiche : array[0..4] of BEREICH =           { die Mausbereiche }
  (
   ( x1:  0; y1:  0; x2: 79; y2:  0 ),            { obere Zeile        }
   ( x1:  0; y1:  1; x2:  0; y2: 23 ),            { linke Spalte       }
   ( x1:  0; y1: 24; x2: 78; y2: 24 ),            { untere Zeile       }
   ( x1: 79; y1:  1; x2: 79; y2: 23 ),            { rechte Spalte      }
   ( x1: 79; y1: 24; x2: 79; y2: 24 )             { untere rechte Ecke }
  );

var Dummy : integer;             { nimmt Ergebnis von MouEventWait auf }

begin
  {-- Maus-Cursor fr die verschiedenen Mausbereiche setzen -----------}
  Bereiche[ 0 ].PtrMask := MouPtrMask( PtrDifChar($18), PtrInvCol);
  Bereiche[ 1 ].PtrMask := MouPtrMask( PtrDifChar($1b), PtrInvCol);
  Bereiche[ 2 ].PtrMask := MouPtrMask( PtrDifChar($19), PtrInvCol);
  Bereiche[ 3 ].PtrMask := MouPtrMask( PtrDifChar($1a), PtrInvCol);
  Bereiche[ 4 ].PtrMask := MouPtrMask( PtrDifChar($58), PtrDifCol($40));

  writeln(#13#10,'MOUSEP - (c) 1989, 91 by MICHAEL TISCHER'#13#10);
  if MouInit( 80, 25 ) then               { Mouse-Modul initialisieren }
   begin              { alles o.k., es ist ein Maustreiber installiert }
    writeln('Wenn Sie den  Maus-Cursor ber den  Bildschirm und vor',ZV,
            'allem an dessen Rand entlang bewegen, werden Sie fest-',ZV,
            'feststellen, da  sich das Erscheinungsbild  des Maus-',ZV,
            'Cursors abhngig von seiner Position verndert.',ZV,ZV,
            'Um diese Demo zu beenden, bewegen Sie  den Maus-Cursor',ZV,
            'bitte in die untere rechte  Bildschirmecke und betti-',ZV,
            'gen dann gleichzeitig den linken und den rechten Maus-',ZV,
            'knopf.' );

    MouSetDefaultPtr( MouPtrMask( PtrDifChar( $DB ), PtrDifCol( 3 ) ) );
    MouDefBereich( 5, @Bereiche );                     { Bereiche def. }
    MouShowMouse;             { Mauscursor auf dem Bildschirm anzeigen }

    {-- warten, bis der linke und der rechte Mausknopf gleichzeitig ---}
    {-- niedergedrckt werden und sich der Maus-Cursor dabei in dem ---}
    {-- Bereich 4 befindet                                          ---}

    repeat                                            { Abrageschleife }
     Dummy := MouEventWait( UND, EV_LEFT_PRESS or EV_RIGHT_PRESS );
    until EvBer = 4;
   end
  else                  { keine Maus oder kein Maustreiber installiert }
    writeln('Es ist kein Maustreiber installiert!');
end.

