{**********************************************************************}
{                          P L I N K P . P A S                         }
{----------------------------------------------------------------------}
{    Aufgabe       : bertragen von Dateien ber die parallele         }
{                    Schnittstelle                                     }
{----------------------------------------------------------------------}
{    Autor         : Michael Tischer                                   }
{    entwickelt am : 27.09.1991                                        }
{    letztes Update: 28.11.1991                                        }
{**********************************************************************}
{$M 65520, 0, 655360 }

{== Include-Dateien einbinden =========================================}

uses dos, crt;                   { die DOS- und die CRT-Unit einbinden }

{== Konstanten ========================================================}

const ONESEK       =  18;                               { eine Sekunde }
      TENSEK       =  182;                             { zehn Sekunden }
      TO_DEFAULT   =  TENSEK;                  { Default-Wert Time-Out }

      MAXBLOCK     =  4096;         { Blockgre 4 KB ( w.g. Cluster ) }

{-- Konstanten fr bertragungsprotokoll ------------------------------}

      ACK          = $00;                                { Acknowledge }
      NAK          = $FF;                            { Non-Acknowledge }
      MAX_TRY      = 5;                { Anzahl erlaubter Fehlversuche }

{-- Tokens fr die Kommunikation zwischen Sender und Empfnger --------}

      TOK_DATSTART = 0;                       { jetzt kommt eine Datei }
      TOK_DATNEXT  = 1;              { nchsten Datenblock einer Datei }
      TOK_DATEND   = 2;          { bertragung der Datei abgeschlossen }
      TOK_ENDE     = 3;                             { Programm beenden }
      TOK_ESCAPE   = 4;           { Escape auf Remote-Rechner gedrckt }

{-- Codes fr die LongJump-Aufrufe ------------------------------------}

      LJ_OKSENDER  = 1;            { alle Dateien erfolgreich gesendet }
      LJ_OKEMPF    = 2;           { alle Dateien erfolgreich empfangen }
      LJ_TIMEOUT   = 3;          { Time-out: Gegenber antwortet nicht }
      LJ_ESCAPE    = 4;  { auf der lokalen Maschine wurde ESC gedrckt }
      LJ_REMESCAPE = 5;   { auf der Remote-Maschine wurde ESC gedrckt }
      LJ_DATA      = 6;                        { Kommunikations-Fehler }
      LJ_NOLINK    = 7;                             { keine Verbindung }
      LJ_NOPAR     = 8;                          { keine Schnittstelle }
      LJ_PARA      = 9;                   { ungltige Aufruf-Parameter }

{== Typdefinitionen ===================================================}

type BHEADER = record              { Header zum bertragen von Blcken }
                 case boolean of
                   true  : ( Token : byte;
                             Len   : word );
                   false : ( Feld  : array[ 0..2 ] of byte );
               end;

     JMPBUF  = record     { hier werden die Informationen gespeichert, }
                 BP,      { die zum Rcksprung bentigt werden         }
                 SP,
                 CS,      { Reihenfolge darf nicht verndert werden!   }
                 IP  : word;
               end;

     DBlock = array[ 1..MAXBLOCK ] of byte;               { Datenblock }

{== Globale Variablen =================================================}

var EinPort     : word;                     { Adresse des Eingabeports }
    AusPort     : word;
    Escape      : boolean;                      { kein ESCAPE gedrckt }
    Timeout     : word;                       { gewhlter Timeout-Wert }
    TO_Count    : word;                           { Zhler fr Timeout }
    RueckSprung : JMPBUF;            { Rckspungadresse fr Beendigung }
    BlockBuf    : DBLOCK;           { Puffer zur Aufnahme eines Blocks }
    Datei       : file;          { Dateivariable fr bearbeitete Datei }

{== Deklaration der Assembler-Funktionen ==============================}

{$L plinkpa.obj }                          { Assembler-Modul einbinden }

function  getb : byte; external;
procedure putb( Wert : byte ); external;
procedure intr_install(escape_flag, timeout_count : pointer); external;
procedure intr_remove; external;
procedure EscapeDirekt( Ausloesen : boolean ); external;

{**********************************************************************}
{ SetJmp : legt die Programmstelle fest, zu der ein folgender Aufruf   }
{          von LongJmp zurckkehren soll                               }
{----------------------------------------------------------------------}
{ Eingabe : JB : eine Datenstruktur vom Type JumpBuf, die die zum Rck-}
{                sprung bentigten Informationen aufnimmt              }
{ Ausgabe : NOJMP bei der Rckkehr aus dieser Funktion; jeder andere   }
{           Wert, wenn die Rckkehr durch einen Aufruf der Prozedur    }
{           LongJmp erfolgt.                                           }
{**********************************************************************}

{$F+}                   { Setjmp und Longjmp mssen FAR kodiert werden }

function SetJmp( var JB : JMPBUF ) : integer;

type WordP  = ^word;                             { Zeiger auf ein Word }

begin
  JB.BP := WordP( ptr( SSeg, Sptr+2) )^;
  JB.IP := WordP( ptr( SSeg, Sptr+4) )^;
  JB.CS := WordP( ptr( SSeg, Sptr+6) )^;

  { SP mu auf die Position zeigen, ab der LongJmp spter den neuen    }
  { Inhalt des BP-Registers und die Rcksprungadresse auf den Stack    }
  { legen wird                                                         }

  JB.SP := Sptr + 12 - 6 - 6;

  SetJmp := -1;           { anzeigen, da dies kein LongJmp-Aufruf ist }
end;

{**********************************************************************}
{ LongJmp : ein prozedurbergreifendes GOTO, das die Programmausfhrung}
{           mit der Programmzeile fortsetzt, in der SetJmp aufgerufen  }
{           wurde.                                                     }
{----------------------------------------------------------------------}
{ Eingabe : JB      : Jump-Buffer, in dem zuvor mit Hilfe von SetJmp   }
{                     die Adresse der Programmzeile eingetragen wurde, }
{                     zu der die Programmausfhrung verzweigen soll    }
{           RetCode : Funktionsergebnis, das beim angesprungenen       }
{                     SetJmp-Aufruf zurckgeliefert werden soll        }
{**********************************************************************}

procedure LongJmp( JB : JMPBUF; RetCode : integer );

type WordP  = ^word;                             { Zeiger auf ein Word }

begin
  WordP( ptr( SSeg, JB.SP   ) )^ := JB.BP;
  WordP( ptr( SSeg, JB.SP+2 ) )^ := JB.IP;
  WordP( ptr( SSeg, JB.Sp+4 ) )^ := JB.CS;

  {-- den bergebenen Return-Code in das AX-Register laden und damit --}
  {-- Funktionsergebnis von SetJmp vortuschen                       --}

  inline( $8b / $46 / $06 );                           { mov ax,[bp+6] }

  inline( $8b / $ae / $fa / $ff );                     { mov bp,[bp-6] }

  { mov   sp,bp         ;diese Befehle erzeugt der Compiler            }
  { pop   bp            ;automatisch und trgt damit zur               }
  { ret   6             ;Restauration des Stacks bei                   }
end;

{$F-}

{**********************************************************************}
{ GetPortAdr: Initialisiert die Portadressen einer parallelen          }
{             Schnittstelle in den globalen Variablen EINPORT und      }
{             AUSPORT                                                  }
{ Eingabe       : NUMMER = Nummer der parallelen Schnittstelle (1-4)   }
{ Ausgabe       : TRUE, wenn Schnittstelle gltig                      }
{ Globale Var.  : EinPort/W, AusPort/W                                 }
{ Info          : Die Basisadressen der bis zu 4 parallelen Schnitt-   }
{                 stellen stehen in den vier Speicherworten            }
{                 beginnend bei 0040:0008                              }
{**********************************************************************}

function GetPortAdr( Nummer : integer ) : boolean;

begin
  {-- Portadresse aus BIOS-Variablen-Segment auslesen -----------------}
  AusPort := MemW[ $0040: 6 + Nummer * 2 ];
  if ( AusPort <> 0 ) then                  { Schnittstelle vorhanden? }
    begin                                                         { Ja }
      EinPort := AusPort + 1;            { Adresse fr Eingaberegister }
      GetPortAdr := TRUE;                           { Ende ohne Fehler }
    end
  else
  GetPortAdr := FALSE;         { Fehler: Schnittstelle nicht vorhanden }
end;

{**********************************************************************}
{ Port_Init     : Initialisiert die zur bertragung notwendigen        }
{                 Register                                             }
{ Eingabe       : SENDER = TRUE, wenn Sender, FALSE, wenn Empfnger    }
{ Ausgabe       : TRUE, wenn Register erfolgreich initialisiert        }
{ Globale Var.  : EinPort/R, AusPort/R                                 }
{ Info          : Die Asymmetrie: sende 00010000, warte auf 00000000   }
{                 ist durch die Invertierung des Signals notwendig.    }
{                 Normalerweise enthalten das Ein- u. das              }
{                 Ausgaberegister die gewnschten Werte, die Initia-   }
{                 lisierung ist aber beim Neustart nach einer abge-    }
{                 brochenen bertragung notwendig.                     }
{**********************************************************************}

function Port_Init( Sender : boolean ) : boolean;

begin
  EscapeDirekt( TRUE );                  { bei Escape Timeout auslsen }
  if ( Sender ) then                           { Gert ist der Sender? }
    begin
      TO_Count := Timeout * 5;            { Time-Out-Zhler einstellen }
      PutB( $10 );                                  { Sende: 00010000b }
      while ( ( GetB <> $00 ) and ( TO_Count > 0 ) ) do  { Warte auf 0 }
        ;
    end
  else                                           { Gert ist Empfnger }
    begin
      TO_Count := Timeout * 5;            { Time-Out-Zhler einstellen }
      while ( ( GetB <> $00 ) and ( TO_Count > 0 ) ) do  { Warte auf 0 }
        ;
      PutB( $10 );                                  { Sende: 00010000b }
    end;
  EscapeDirekt( FALSE );            { bei Escape kein Timeout auslsen }
  Port_Init := ( TO_Count > 0 );       { Initialisierung abgeschlossen }
end;

{**********************************************************************}
{ SendeByte    : Sendet ein Byte in 2 Teilen an den Fremdrechner und   }
{                berprft das Ergebnis                                }
{ Eingabe      : WERT = zu sendendes Byte                              }
{ Ausgabe      : bertragung erfolgreich? ( 0 = Fehler, -1 = ok )      }
{ Globale Var. : Timeout/R, Einport/R, Ausport/R (in Makros)           }
{**********************************************************************}

function SendeByte( Wert : byte ) : boolean;

var Rueck : byte;                                   { Empfangenes Byte }

begin
  {-- Unteres Nibble Senden  ------------------------------------------}

  TO_Count := Timeout;                 { Timeout-Zhler initialisieren }
  PutB( Wert and $0F );                         { Senden, BUSY lschen }
  while ( ( ( GetB and 128 ) = 0 ) and ( TO_Count > 0 ) ) do
    ;
  if ( TO_Count = 0 ) then                           { Timeout-Fehler? }
    longjmp( RueckSprung, LJ_TIMEOUT );        { bertragung abbrechen }

  Rueck := ( GetB shr 3 ) and $0F;                   { Bits 3-6 in 0-3 }

  {-- Oberes Nibble Senden --------------------------------------------}

  TO_Count := Timeout;                 { Timeout-Zhler initialisieren }
  PutB( ( Wert shr 4 ) or $10 );                 { Senden, BUSY setzen }
  while ( ( ( GetB and 128 ) <> 0 ) and ( TO_Count > 0 ) ) do

  if ( TO_Count = 0 ) then                           { Timeout-Fehler? }
    longjmp( RueckSprung, LJ_TIMEOUT );        { bertragung abbrechen }

  Rueck := Rueck or ( ( GetB shl 1 ) and $F0 );      { Bits 3-6 in 4-7 }
  SendeByte :=  ( Wert = Rueck );          { Byte korrekt bermittelt? }
end;

{**********************************************************************}
{ EmpfangeByte : Empfngt ein Byte in 2 Teilen vom Fremdrechner und    }
{                sendet empfangene Teile zu Prfzwecken zurck         }
{ Eingabe      : keine                                                 }
{ Ausgabe      : empfangenes Byte                                      }
{ Globale Var. : Timeout/R, Einport/R, Ausport/R (in Makros)           }
{**********************************************************************}

function EmpfangeByte : byte;

var LoNib,
    HiNib : byte;                                 { Empfangene Nibbles }

begin
  {-- Unteres Nibble Empfangen und Zurcksenden -----------------------}

  TO_Count := Timeout;                 { Timeout-Zhler initialisieren }
  while ( ( ( GetB and 128 ) = 0 ) and ( TO_Count > 0 ) ) do
    ;

  if ( TO_Count = 0 ) then                           { Timeout-Fehler? }
    longjmp( RueckSprung, LJ_TIMEOUT );        { bertragung abbrechen }

  LoNib := ( GetB shr 3 ) and $0F;                   { Bits 3-6 in 0-3 }
  PutB( LoNib );                                        { Zurcksenden }

  {-- Oberes Nibble Empfangen und Zurcksenden ------------------------}

  TO_Count := Timeout;                 { Timeout-Zhler initialisieren }
  while ( ( ( GetB and 128 ) <> 0 ) and ( TO_Count > 0 ) ) do
    ;

  if ( TO_Count = 0 ) then                           { Timeout-Fehler? }
    longjmp( RueckSprung, LJ_TIMEOUT );        { bertragung abbrechen }

  HiNib := ( GetB shl 1 ) and $F0;                   { Bits 3-6 in 4-7 }
  PutB( ( HiNib shr 4 ) or $10 );          { Zurcksenden, Busy setzen }

  EmpfangeByte := ( LoNib or HiNib );               { Empfangenes Byte }
end;

{**********************************************************************}
{ SendeBlock: Sendet einen Datenblock                                  }
{ Eingabe   : TOKEN  = Befehl fr Empfnger                            }
{             ANZAHL = Anzahl der zu bertragenden Bytes               }
{             DPTR   = Zeiger auf den Puffer mit den Daten             }
{ Ausgabe   : keine, denn bei einem Fehler wird sofort ber            }
{             LongJmp zur Fehlerbehandlungsroutine gesprungen          }
{**********************************************************************}

procedure SendeBlock( Token  : byte;
                      Anzahl : word;
                      Dptr   : pointer );

var header     : BHEADER;   { Header zur Aufnahme von Token und Anzahl }
    EmpfEscape : byte; { wurde auf dem Remote-Rechner ESCAPE gedrckt? }
    ok         : boolean;                                { Fehler-Flag }
    i          : word;                               { Schleifenzhler }
    try        : word;              { verbleibende Anzahl der Versuche }
    Daten      : ^DBlock;                      { Zeiger auf Datenblock }

begin
  if ( Escape ) then       { wurde auf diesem Rechner Escape gedrckt? }
    begin
      Token := TOK_ESCAPE;                   { Ja, Escape-Token senden }
      Anzahl := 0;
    end;

  {-- zunchst den Header senden --------------------------------------}

  header.Token := Token;                             { Header aufbauen }
  header.Len := Anzahl;

  try := MAX_TRY;
  repeat                                { maximal MAX_TRY Fehlversuche }
    ok := TRUE;      { zunchst von ferhlerfreier bertragung ausgehen }
    for i := 0 to 2 do
      ok := ok and SendeByte( Header.Feld[ i ] );    { ein Byte senden }
    if ( ok ) then
      ok := ok and SendeByte( ACK )               { Besttigung senden }
    else
      ok := ok and SendeByte( NAK );              { Besttigung senden }
    if ( not ok ) then                                       { Fehler? }
      dec( try );                     { Ja, weiterer Versuch notwendig }
  until ( ( ok ) or ( try =  0 ) );

  if ( try = 0 ) then           { konnte der Header bertragen werden? }
    longjmp( RueckSprung, LJ_DATA );     { Nein, bertragung abbrechen }

  if ( Token = TOK_ESCAPE ) then      { wurde ESCAPE-Meldung gesendet? }
    longjmp( RueckSprung, LJ_ESCAPE );     { Ja, bertragung abbrechen }

  {-- den eigentlichen Datenblock senden ------------------------------}

  if ( Anzahl > 0 ) then                           { Lnge ungleich 0? }
    begin
      Daten := DPTR;
      try := MAX_TRY;
      repeat
        ok := TRUE;            { von fehlerfreier bertragung ausgehen }
        for i := 1 to Anzahl do
          ok := ok and SendeByte( Daten^[ i ] );
        if ( ok ) then
          ok := ok and SendeByte( ACK )           { Besttigung senden }
        else
          ok := ok and SendeByte( NAK );          { Besttigung senden }
        if ( not ok ) then                                   { Fehler? }
          dec( try );                 { Ja, weiterer Versuch notwendig }
      until ( ( ok ) or ( try =  0 ) );
      if ( try = 0 ) then   { konnte der Datenblock bertragen werden? }
        longjmp( RueckSprung, LJ_DATA ); { Nein, bertragung abbrechen }
    end;

  {-- ESCAPE-Byte vom Empfnger abfragen ------------------------------}

  try := MAX_TRY;
  repeat
    EmpfEscape := EmpfangeByte;               { Remote-Escape abfragen }
    dec( try );
  until ( ( EmpfEscape = byte( true ) ) or
          ( EmpfEscape = byte( false ) ) );

  if ( try = 0 ) then             { wurde der Escape-Status empfangen? }
    longjmp( RueckSprung, LJ_DATA );     { Nein, bertragung abbrechen }

  if ( EmpfEscape = byte( true ) ) then { Esc. auf dem Remote-Rechner? }
    longjmp( RueckSprung, LJ_REMESCAPE );  { Ja, bertragung abbrechen }
end;

{**********************************************************************}
{ EmpfangeBlock: Empfngt einen Datenblock                             }
{ Eingabe   : TOKEN  = Zeiger auf Variable zur Aufnahme des Tokens     }
{             LEN    = Zeiger auf Variable zur Aufnahme der Lnge      }
{             DATEN  = Zeiger auf Puffer zur Aufnahme der Daten        }
{ Ausgabe   : keine, denn bei einem Fehler wird sofort ber            }
{             LongJmp zur Fehlerbehandlungsroutine gesprungen          }
{ Info      : Der bergebene Puffer mu fr MAXBLOCK Bytes Platz       }
{             bieten, da die Blocklnge in der Regel nicht anti-       }
{             zipiert werden kann                                      }
{**********************************************************************}

procedure EmpfangeBlock( var Token : byte;
                         var Len   : word;
                             Dptr  : pointer );

var header       : BHEADER; { Header zur Aufnahme von Token und Anzahl }
    ok           : boolean;                              { Fehler-Flag }
    i            : word;                             { Schleifenzhler }
    try          : word;            { verbleibende Anzahl der Versuche }
    EscapeStatus : boolean;
    ByteBuffer   : byte;
    Daten        : ^DBlock;                    { Zeiger auf Datenblock }

begin
  {-- Zunchst den Header empfangen -----------------------------------}

  try := MAX_TRY;
  repeat
    for i:= 0 to 2 do
      Header.Feld[ i ] := EmpfangeByte;

    ByteBuffer := EmpfangeByte;
    if ( ByteBuffer <> ACK ) then      { alle Bytes richtig empfangen? }
      dec( try );                { Ja, kein erneuter Versuch notwendig }
  until ( ( try = 0 ) or ( ByteBuffer = ACK ) );

  if ( try = 0 ) then            { wurde der Header richtig empfangen? }
    longjmp( RueckSprung, LJ_DATA );     { Nein, bertragung abbrechen }

  Token := Header.Token;
  Len := Header.Len;
  if ( Token = TOK_ESCAPE ) then                      { Sender ESCAPE? }
    longjmp( RueckSprung, LJ_REMESCAPE );  { Ja, bertragung abbrechen }

  {-- der Header war o.k., jetzt den eigentlichen Datenblock empf. ----}

  if ( Len > 0 ) then                               { kein Datenblock? }
    begin                                                       { doch }
      Daten := Dptr;
      try := MAX_TRY;

      repeat                      { Datenblock Byte fr Byte empfangen }
        for i := 1 to len do
         Daten^[ i ] := EmpfangeByte;

         ByteBuffer := EmpfangeByte;
         if ( ByteBuffer <> ACK ) then { alle Bytes richtig empfangen? }
           dec( try );           { Ja, kein erneuter Versuch notwendig }
      until ( ( try = 0 ) or ( ByteBuffer = ACK ) );

      if ( try = 0 ) then         { wurde der Block richtig empfangen? }
        longjmp( RueckSprung, LJ_DATA ); { Nein, bertragung abbrechen }
    end;

 {-- aktuellen Escape-Status an den Remote-Rechner senden -------------}

  EscapeStatus := Escape;                              { Status merken }

  try := MAX_TRY;
  repeat
    dec( try );
  until ( SendeByte( byte( EscapeStatus ) ) or ( try = 0 ) );

  if ( try = 0 ) then         { konnte der ESC-Status gesendet werden? }
    longjmp( RueckSprung, LJ_DATA );     { Nein, bertragung abbrechen }

  if ( EscapeStatus ) then  { wurde auf diesem Rechner Escape gedrckt }
    longjmp( RueckSprung, LJ_ESCAPE );     { Ja, bertragung abbrechen }
end;

{**********************************************************************}
{ SendeDatei: Sendet eine Datei                                        }
{ Eingabe   : NAME = Dateiname                                         }
{ Ausgabe   : keine                                                    }
{**********************************************************************}

procedure SendeDatei( Name : string );

var Status  : word;                                      { Sendestatus }
    Gelesen : word;                        { Anzahl eingelesener Bytes }
    Groesse : longint;                       { Anzahl gesendeter Bytes }

begin
  write( copy( Name + '                   ', 1, 13 ) );
  assign( Datei, Name );
  reset( Datei, 1 );
  SendeBlock( TOK_DATSTART, length( Name ) + 1, @Name ); { Namen send. }

  {-- Dateiinhalt bertragen ------------------------------------------}

  Groesse := 0;
  repeat
    blockread( Datei, BlockBuf, MAXBLOCK, Gelesen );  { Block einlesen }
    if ( Gelesen > 0 ) then                { Ende noch nicht erreicht? }
      begin {                                                   { Nein }
        SendeBlock( TOK_DATNEXT, Gelesen, @BlockBuf );  { Block Senden }
        inc( Groesse, Gelesen );
        write( #13, copy( Name + '             ', 1, 13 ),
               '(', Groesse, ')' );
      end;
  until (  Gelesen = 0 );
  writeln;

  SendeBlock( TOK_DATEND, 0, NIL );          { bertragung abschlieen }

  close( Datei );                                   { Datei schliessen }
end;

{**********************************************************************}
{ EmpfangeDatei: Empfngt eine Datei                                   }
{ Eingabe      : keine                                                 }
{ Ausgabe      : Das letzte empfangene Token                           }
{**********************************************************************}

function EmpfangeDatei : word;

var Status      : word;                                  { Sendestatus }
    ZuSchreiben : word;                     { Gre des letzten Blocks }
    Groesse     : longint;
    Token       : byte;                            { Empfangenes Token }
    Len         : word;                        { Empfangene Blocklnge }
    i           : word;                              { Schleifenzhler }
    Name        : string[ 13 ];                            { Dateiname }

begin
  EmpfangeBlock( Token, Len, @BlockBuf );
  if ( Token = TOK_DATSTART ) then
    begin
      for i := 0 to BlockBuf[ 1 ] do
        Name[ i ] := chr( BlockBuf[ i + 1 ] );
      assign( Datei, Name );
      rewrite( Datei, 1 );
      write( copy( Name + '                   ', 1, 13 ) );

      {-- Dateiinhalt empfangen ---------------------------------------}

      Groesse := 0;
      repeat
        EmpfangeBlock( Token, Len, @BlockBuf );      { Block empfangen }
        if ( Token = TOK_DATNEXT ) then         { nchster Datenblock? }
          begin                                                   { Ja }
            blockwrite( Datei, BlockBuf, Len );      { Block schreiben }
            inc( Groesse, Len );
            write( #13, copy( Name + '             ', 1, 13 ),
                   '(', Groesse, ')' );
          end;
      until ( TOKEN <> TOK_DATNEXT );
      close( Datei );                        { Datei wieder schliessen }
      writeln;
    end;
  EmpfangeDatei := Token;                  { Fehlerstatus zurckmelden }
end;

{**********************************************************************}
{                  H A U P T P R O G R A M M                           }
{**********************************************************************}

const Meldung : array[ 0..8 ] of string =
              ( 'ENDE: Alle Dateien erfolgreich gesendet.',
                'ENDE: Alle Dateien erfolgreich empfangen.',
                'FEHLER: Time-Out, Remote-System antwortet nicht!',
                'ENDE: Abbruch durch Escape.',
                'ENDE: Abbruch auf der Remote-Maschine durch Escape.',
                'FEHLER: Schnittstelle oder Kabel defekt!',
                'FEHLER: Kein Kontakt zu Remote-Rechner!',
                'FEHLER: Angegebene Schnittstelle existiert nicht!',
                'FEHLER: Unbekannte oder ungltige Parameter!' );

var SRec     : SearchRec;               { Struktur fr Directory-Suche }
    Sender   : boolean;        { bertragungsmodus (Senden, Empfangen) }
    sjStatus : integer;                            { Code von longjump }
    Nummer,                                 { Nummer der Schnittstelle }
    i,                                               { Schleifenzhler }
    Gefunden : byte;                             { Fr Directory-Suche }
    dummy    : integer;
    argv     : array[ 1..10 ] of string;                   { Parameter }


begin
  write( #13#10'Datenbertragung ber die parallele Schnittstelle' );
  writeln( '   (c) 1991 by Michael Tischer' );
  write( '=================================================' );
  writeln( '==============================' );

  Escape := false;
  Timeout := TO_DEFAULT;

  if ( paramstr( 1 ) = '?' ) then               { nur Syntax ausgeben? }
    begin                                                         { Ja }
      writeln( 'Aufruf: plinkc [/Pn] [/Tnn] [Dateiname]' );
      halt( 0 );
    end;

  sjStatus := setjmp( RueckSprung );   { Rcksprungadr. zur Beendigung }
  if ( sjStatus > 0 ) then                       { Longjmp aufgerufen? }
    begin                                                         { Ja }
      Intr_remove;                    { Interrupt-Handler deaktivieren }
      writeln( #13#10#13#10, Meldung[ sjStatus - 1 ] );
      halt( 0 );
    end;

  Intr_Install( @Escape, @TO_Count );         { Interrupt-Handler init }

 {-- Defaultwerte setzen und Kommmandozeile auswerten -----------------}

 Sender := FALSE;                              { Default ist Empfnger }
 Nummer := 1;                                       { Default ist LPT1 }

 for i := 1 to paramcount do
   begin
     argv[ i ] := paramstr( i );                    { Parameter merken }
     if ( argv[ i, 1 ] = '/' ) then                        { Parameter }
       begin
         case ( upcase( argv[ i, 2 ] ) ) of
           'T' : begin
                   delete( argv[ i ], 1, 2 );
                   val( argv[ i ], Timeout, dummy );
                   Timeout := ( Timeout * TENSEK ) div 10;
                   if ( Timeout = 0 ) then
                     longjmp( RueckSprung, LJ_PARA );       { ungltig }
                 end;
           'P' : begin
                   Nummer := ord( argv[ i, 3 ] ) - 48; { Schnittstelle }
                   if ( ( Nummer = 0 ) or ( Nummer > 4 ) ) then
                     longjmp( RueckSprung, LJ_PARA );       { ungltig }
                 end;
           else longjmp( RueckSprung, LJ_PARA );         { unbek. Para }
         end;
         argv[ i ] := '';                           { Argument lschen }
       end
     else                         { kein Parameter, mu Dateiname sein }
       Sender := TRUE;                                        { Sender }
   end;

 {-- bertragung starten ----------------------------------------------}

 if ( not GetPortAdr( Nummer ) ) then    { existiert die Schnittstelle }
   longjmp( RueckSprung, LJ_NOPAR );                    { Nein, Fehler }

 if ( not Port_Init( Sender ) ) then             { Verbindung aufbauen }
   longjmp( RueckSprung, LJ_NOLINK );          { Fehler, nicht mglich }

 if ( Sender ) then                                          { Sender? }
   begin
     writeln( 'Senden auf LPT', Nummer, #13#10 );

     {-- Alle Dateien bertragen --------------------------------------}

     for i := 1 to paramcount do           { Kommandozeile durchlaufen }
       begin
         if ( argv[ i ] <> '' ) then                      { Dateiname? }
           begin                                                  { Ja }
             findfirst( argv[ i ], AnyFile, SRec);
             while ( DosError = 0 ) do              { solange gefunden }
               begin
                 if ( SRec.Attr <> Directory ) then
                   SendeDatei( SRec.Name );           {Datei bertragen}
                 findnext( SRec );
               end;
           end;
       end;
     SendeBlock( TOK_ENDE, 0 , NIL );          { alle Dateien gesendet }
     longjmp( RueckSprung, LJ_OKSENDER );
   end
 else                                                { Nein, Empfnger }
   begin
     writeln( 'Empfangen auf LPT', Nummer, #13#10 );
     while ( EmpfangeDatei <> TOK_ENDE ) do        { Dateien empfangen }
       ;                                           { bis ENDE-Token    }
     longjmp( RueckSprung, LJ_OKEMPF );
   end;
end.
