;**********************************************************************;
;*                         P L I N K P A . A S M                      *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : Stellt die Assemblerschnittstelle zum Pascal-  *;
;*                     Programm PLINKP dar und enthlt neben den      *;
;*                     Interrupt-Handler auch Routinen zum schnellen  *;
;*                     Zugriff auf Ports                              *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  : 10.10.1991                                     *;
;*    letztes Update : 11.10.1991                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : TASM PLINKPA                                   *;
;*                     ... dann in PLINKP einbinden                   *;
;**********************************************************************;
                                                                        
;== Konstanten =========================================================
                                                                        
KB_PORT   = 60h                   ;Tastatur-Port
INT_CTR   = 20h                   ;Port des Interrupt-Controllers
EOI       = 20h                   ;End-of-Interrupt-Kommando
ESCAPE    = 1                     ;Scan-Code von Escape
                                                                        
;== Das Datensegment ===================================================

DATA   segment word public        ;Turbo-Datensegment
                                                                        
extrn    EinPort                  ;Eingabeport als Turbo-Variable
extrn    AusPort                  ;Ausgabeport als Turbo-Varibale

DATA   ends                       ;Ende des Datensegments

;== Programm ===========================================================
                                                                        
CODE       segment byte public    ;das Turbo-Codesegment
                                                                        
           assume cs:CODE, ds:DATA, es:nothing, ss:nothing
                                                                        
;-- Public-Deklarationen interner Funktionen ---------------------------
                                                                        
public     intr_install           ;ermglicht Aufruf aus Turbo-Programm
public     intr_remove
public     escapedirekt
public     getb
public     putb

;-- Variablen fr die Interrupt-Handler ----------------------------------
;-- (nur ber das Codesegment zugnglich) --------------------------------

key_ptr    dd 0                   ;Zeiger auf Variable fr ESCAPE
tout_ptr   dd 0                   ;Zeiger auf Time-Out-Zhler
escdirekt  db 0                   ;bei Escape kein Time-Out auslsen

;-- die folgenden Variablen nehmen die alten Adressen der Interrupt- ---
;-- Handler auf, die durch neue Interrupt-Handler ersetzt wurden     ---
                                                                        
int9_ptr   equ this dword         ;alter Interrupt-Vektor 9h
int9_ofs   dw 0                   ;Offsetadresse des alten Handlers
int9_seg   dw 0                   ;Segmentadresse des alten Handlers
                                                                        
int1C_ptr  equ this dword         ;alter Interrupt-Vektor 1Ch
int1C_ofs  dw 0                   ;Offsetadresse des alten Handlers
int1C_seg  dw 0                   ;Segmentadresse des alten Handlers
                                                                        
;----------------------------------------------------------------------
;-- GETB : liest ein Byte vom Eingabeport ein
;-- Aufruf von Turbo: getb : BYTE;

getb      proc near

          mov  dx,EinPort         ;Portadresse in DX
          in   al,dx              ;Einlesen von Port
          and  al,0F8h            ;Bits 0-2 ausblenden

          ret                     ;Rcksprung Funktionsergebnis in AL

getb      endp

;----------------------------------------------------------------------
;-- PUTB : gibt ein Byte auf dem Ausgabeport aus 
;-- Aufruf von Turbo: putb( Wert : BYTE );

putb       proc near

Wert       equ byte ptr [bp+4]    ;Die von TP bergebene Variable

           push bp                ;Zugriff auf die Argumente ermglichen
           mov  bp, sp

           mov  al, Wert          ;Wert nach al laden
           mov  dx, AusPort       ;Ausgabeport nach DX laden
           out  dx, al            ;Wert an Port ausgeben

           pop  bp                ;BP restaurieren
           ret  2                 ;Stack aufrumen und Rcksprung

putb       endp

;-----------------------------------------------------------------------
;-- INTR_INSTALL: Installiert die Interrupt-Handler                 ----
;-- Aufruf von Turbo: intr_install( escape_flag, timeout_count : ptr );

intr_install  proc near

sframe0    struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
ret_adr0   dw ?                   ;Rcksprungadresse
toptr      dd ?                   ;FAR-Zeiger auf den Timeout-Zhler
escptr     dd ?                   ;FAR-Zeiger auf das ESCAPE-Flag
sframe0    ends                   ;Ende der Struktur

frame      equ [ bp - bp0 ]
                                                                        
           push bp                ;BP auf dem Stack sichern
           mov  bp,sp             ;SP nach BP bertragen
           push es                ;ES auf dem Stack sichern
                                                                        
	   ;-- die Argumente vom Stack holen und bearbeiten ------------

	   les  si,frame.escptr       ;Zeiger auf ESCAPE-Flag laden
	   mov  word ptr key_ptr,si   ;und in Codesegment-Variablen
	   mov  word ptr key_ptr+2,es ;ablegen

	   les  si,frame.toptr         ;Zeiger auf Timeout-Zhler laden
	   mov  word ptr tout_ptr,si   ;und in Codesegment-Variablen
	   mov  word ptr tout_ptr+2,es ;ablegen

           ;-- die Adressen der umzuleitenden Interrupt-Handler holen --
                                                                        
           mov  ax,3509h          ;Interrupt-Vektor 9h holen
           int  21h               ;DOS-Interrupt aufrufen
	   mov  int9_ofs,bx       ;Adresse des Handlers in den ent-
	   mov  int9_seg,es       ;sprechenden Variablen merken
                                                                        
	   mov  ax,351Ch          ;Interrupt-Vektor 1Ch holen
           int  21h               ;DOS-Interrupt aufrufen
	   mov  int1C_ofs,bx      ;Adresse des Handlers in den ent-
	   mov  int1C_seg,es      ;sprechenden Variablen merken

           ;-- die neuen Intrrupt-Handler installieren -----------------
                                                                        
           push ds                ;Datensegment merken
	   mov  ax,cs             ;DS auf CS setzen
           mov  ds,ax
                                                                        
	   mov  ax,2509h          ;Fkt.nr.: Interrupt 9h setzen
           mov  dx,offset int09   ;DS:DX nimmt Adresse des Handler auf
	   int  21h               ;DOS-Interrupt aufrufen
                                                                        
	   mov  ax,251Ch          ;Fkt.nr.: Interrupt 1Ch setzen
	   mov  dx,offset int1C   ;DS:DX nimmt Adresse des Handler auf
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
	   pop  ds                ;DS wieder vom Stack holen
           pop  es
           pop  bp                ;BP wieder vom Stack holen
           ret  8                 ;zurck zum Aufrufer
                                                                        
intr_install endp

;-----------------------------------------------------------------------
;-- INTR_REMOVE: Deaktiviert die Interrupt-Handler wieder
;-- Aufruf von Turbo: intr_remove;

intr_remove proc near
                                                                        
	   cli                    ;keine Interrupts mehr zulassen
	   push ds                ;DS sichern

           mov  ax,2509h          ;Fkt.nr: Handler fr INT 9 setzen
	   mov  ds,int9_seg       ;Segmentadresse des alten Handler
	   mov  dx,int9_ofs       ;Offsetadresse des alten Handler
           int  21h               ;den alten Handler wieder installieren
                                                                        
	   mov  ax,251Ch          ;Fkt.nr: Handler fr INT 1C setzen
	   mov  ds,int1C_seg      ;Segmentadresse des alten Handler
	   mov  dx,int1C_ofs      ;Offsetadresse des alten Handler
           int  21h               ;den alten Handler wieder installieren
                                                                        
	   pop  ds                ;DS wieder zurckholen
           sti                    ;Interrupts wieder erlauben
                                                                        
           ret                    ;zurck zum Aufrufer
                                                                        
intr_remove endp                  ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- EscapeDirekt: Legt fest, ob bei Escape ein Time-Out ausgelst   ----
;--               werden soll
;-- Aufruf von Pascal: procedure EscapeDirekt( Ausloesen : boolean );

EscapeDirekt proc near

sframe1    struc                  ;Struktur zum Zugriff auf den Stack
bp1        dw ?                   ;nimmt BP auf
ret_adr1   dw ?                   ;Rcksprungadresse
escflag    dw ?                   ;TRUE oder FALSE
sframe1    ends                   ;Ende der Struktur

frame      equ [ bp - bp1 ]

           push bp                ;BP auf dem Stack sichern
           mov  bp,sp             ;SP nach BP bertragen

           mov  al,byte ptr frame.escflag ;Flag laden
           mov  escdirekt,al              ;und in CS-Variable ablegen

	   pop  bp
	   ret  2                  ;zurck zum Aufrufer, Argument vom
                                   ;Stack

EscapeDirekt endp

;-----------------------------------------------------------------------
;-- es folgen die neuen Interrupt-Handler ------------------------------
;-----------------------------------------------------------------------

	   assume CS:CODE, DS:nothing, ES:nothing, SS:nothing

;-- der neue Interrupt 09h-Handler -------------------------------------
                                                                        
int09      proc far

	   push ax                ;AX und auf dem Stack sichern
	   in   al,KB_PORT        ;Scan-Code vom Tastatur-Port holen

	   cmp  al,128            ;Release-Code?
	   jae  i9_end            ;Ja, gar nicht erst testen

	   cmp  al,ESCAPE         ;Nein, aber ESCAPE?
	   jne  i9_end            ;Nein, an alten Handler weiterreichen

	   ;-- es wurde ESCAPE bettigt --------------------------------

	   push ds                ;DS und SI merken
	   push si
	   lds  si,key_ptr        ;Zeiger auf ESCAPE-Variable laden
	   mov  word ptr [si],1   ;ESCAPE-Flag auf 1 setzen
           cmp  escdirekt,0       ;Time-Out-Flag lschen?
           je   i9_1              ;Nein ---> I9_1

	   lds  si,tout_ptr       ;Ja, Zeiger auf Time-Out-Zhler laden
	   mov  word ptr [si],0   ;Zhler auf 0

i9_1:      pop  si                ;DS und SI wieder restaurieren
	   pop  ds

	   mov  al,EOI            ;Ende des Interrupts anzeigen
	   out  INT_CTR,al

	   pop  ax                ;AX zurckholen
	   iret                   ;zurck zum unterbrochenen Programm

i9_end:    pop  ax                ;AX zurckholen
	   jmp  cs:[int9_ptr]     ;zum alten Handler springen

int09      endp

;-- der neue Interrupt 1Ch-Handler -------------------------------------
                                                                        
int1C      proc far
                                                                        
	   push ds                ;DS und SI merken
	   push si
	   lds  si,tout_ptr       ;Zeiger auf Time-Out-Zhler laden
	   cmp  word ptr [si],0   ;Zhler bereits auf 0?
	   je   no_dekr           ;JA ---> nicht mehr dekrementieren

	   dec  word ptr [si]     ;Nein, dekrementieren

no_dekr:   pop  si                ;DS und SI wieder restaurieren
	   pop  ds

	   jmp  cs:[int1C_ptr]    ;weiter zum alten Interrupt-Handler

int1C      endp

;-------------------------------------------------------------------------
                                                                        
CODE       ends                   ;Ende des Codesegments
           end                    ;Ende des Programms
