{**********************************************************************}
{*                      R E C L O C K P . P A S                       *}
{*--------------------------------------------------------------------*}
{*    Aufgabe       : Demonstriert das Record-Locking mit Hilfe der   *}
{*                    Funktionen und Prozeduren aus der NETILFEP-Unit *}
{*--------------------------------------------------------------------*}
{*    Autor         : Michael Tischer                                 *}
{*    entwickelt am : 10.02.1992                                      *}
{*    letztes Update: 13.02.1992                                      *}
{**********************************************************************}

program RecLockP;

uses Crt, Dos,                   { die Crt- und die DOS-Unit einbinden }
     NetFileP;                           { die Netzwerk-Unit einbinden }

const DateiName = 'reclockp.dat';                 { Name der Testdatei }
      SAETZE    = 10;                          { Anzahl der Datenstze }

type  Test     = array[ 1..160 ] of char;    { Datentyp der Test-Datei }
      TestFile = file of Test;

{**********************************************************************}
{* ErzeugeSatz : Erzeugen eines Testdatensatzes                       *}
{* Eingabe     : Zeichen fr den Satz                                 *}
{* Ausgabe     : Testdatensatz                                        *}
{**********************************************************************}

procedure ErzeugeSatz(     Zeichen : char;
                       var Satz    : test );

var i : word;                                        { Schleifenzhler }

begin
  for i := 1 to 160 do
    Satz[ i ] := Zeichen;
end;

{**********************************************************************}
{* OeffneDatei : Vorhandene Netzwerkdatei ffnen. Anderenfalls        *}
{*               Erzeugen einer neuen Testdatei und Fllen der Datei  *}
{*               mit Testdatenstzten                                 *}
{* Eingabe     : Datei                                                *}
{* Ausgabe     : TRUE, wenn die Datei geffnet werden konnte.         *}
{**********************************************************************}

function OeffneDatei( var Datei : testfile ) : boolean;

var i        : word;                                 { Schleifenzhler }
    TestSatz : Test;        { notwendig fr das Erzeugen der Testdaten }

begin
  {-- Datei fr Ein- u. Ausgabe im Deny-None-Modus erffnen -----------}

  NetReset( DateiName, FM_RW or SM_NO, sizeof( Test ), Datei );
  if ( NetError = NE_FileNotFound ) then      { Datei nicht vorhanden? }
    begin

      {-- Datei erzeugen und mit Testdatenstzen fllen ---------------}

      NetRewrite( Dateiname, FM_RW or SM_NO, sizeof( Test ), Datei );
      if ( NetError = 0 ) then                 { Erzeugen ohne Fehler? }
        begin
          if NetLock( Datei, 0, SAETZE ) then   { alle Records sperren }
            begin
              NetSeek( Datei, 0 );   { Zeiger auf den Anfang der Datei }
              for i := 1 to SAETZE do
                begin
                  ErzeugeSatz( chr( ord( 'A' ) - 1 + i ), TestSatz );
                  NetWrite( Datei, TestSatz );   { Testdaten schreiben }
                end;
              OeffneDatei := NetUnlock( Datei, 0, SAETZE );
            end
          else
            OeffneDatei := false;                { Fehler beim Locking }
        end
      else
        OeffneDatei := false;        { Fehler beim Erzeugern der Datei }
    end
  else
    OeffneDatei := ( NetError = 0 );             { ffnen ohne Fehler? }
end;

{**********************************************************************}
{* NetzEdit    : Demonstration der Netzwerkfunktionen                 *}
{* Eingabe     : Datei                                                *}
{* Ausgabe     : Datei                                                *}
{**********************************************************************}

procedure NetzEdit( var Datei : TestFile );

const StatusMsg : array[ false..true ] of string = ( 'frei    ',
                                                     'gesperrt' );

var AktRecord : longint;                       { Aktuelle Recordnummer }
    AktSatz   : Test;                            { Aktueller Datensatz }
    Aktion    : byte;                              { gewnschte Aktion }
    Status    : boolean;                             
    Zeichen   : char;
    Gesperrt  : array [ 0..SAETZE-1 ] of boolean;
    i         : integer;                             { Schleifenzhler }

begin
  {-- Men ausgeben ---------------------------------------------------}

  writeln( #13#10'Mgliche Funktionen' );
  writeln( '  1: Dateizeiger positionieren.' );
  writeln( '  2: Record sperren' );
  writeln( '  3: Record lesen' );
  writeln( '  4: Datensatz ndern' );
  writeln( '  5: Record schreiben' );
  writeln( '  6: Sperre aufheben' );
  writeln( '  7: Ende' );

  {-- Status der Datenstze initialisieren und ausgeben ---------------}

  gotoxy( 58, 4 );
  write( 'Status der Datenstze:' );
  for i := 0 to SAETZE-1 do
    begin
      Gesperrt[i] := false;
      gotoxy( 64, i+5 );
      write( i:2, '   frei' );
    end;

  AktRecord := 0;                                { Aktueller Datensatz }
  NetSeek( Datei, AktRecord );
  ErzeugeSatz( #32, AktSatz );             { Leeren Datensatz erzeugen }

  repeat
    {-- Informationen ausgeben ----------------------------------------}

    gotoxy( 1, 16 );              { Position des Dateizeigers ausgeben }
    writeln( 'Aktueller Record: ',  AktRecord : 4 );
    writeln( 'Status          :    ', StatusMsg[ Gesperrt[AktRecord] ]);
    Writeln( 'Netzwerkstatus  : ', NetError: 4, ' = ',
       copy( NetErrorMsg( NetError ) + '                   ', 1, 30 ) );
    gotoxy( 1, 21 );                          { Testdatensatz ausgeben }
    writeln( 'Aktueller Datensatz:' );
    writeln( AktSatz );

    NetSeek( Datei, AktRecord );           { Dateizeiger positionieren }

    gotoxy( 1, 13 );
    write( 'Auswahl:                            ' );
    gotoxy( 10, 13 );
    readln( Aktion );
    case Aktion of
      1 : begin
           gotoxy( 1, 13 );
           write( 'Neue Datensatznummer: ' );
            repeat
              gotoxy( 22, 13 );
              write( '                      ' );
              gotoxy( 22, 13 );
              readln( AktRecord );
            until ( AktRecord >= 0 ) and (AktRecord < SAETZE );
          end;

      2 : begin
            Status := NetLock( Datei, AktRecord, 1 );
            if ( Status = TRUE ) then
              begin
                Gesperrt[ AktRecord ] := TRUE;
                gotoxy( 64, AktRecord+5 );
                write( AktRecord:2, '   gesperrt' );
              end;
          end;

      3 : NetRead( Datei, AktSatz );                 { Datensatz lesen }

      4 : begin
            gotoxy( 1, 13 );
            write( 'Neues Zeichen: ' );
            readln( Zeichen );
            ErzeugeSatz( Zeichen, AktSatz );
          end;

      5 : NetWrite( Datei, AktSatz );            { Datensatz schreiben }

      6 : begin
            Status := NetUnlock( Datei, AktRecord, 1 );
            if ( Status = TRUE ) then
              begin
                Gesperrt[ AktRecord ] := FALSE;
                gotoxy( 64, AktRecord+5 );
                write( AktRecord:2, '   frei    ' );
              end;
          end;
    end;
  until ( Aktion = 7 );
end;

{**********************************************************************}
{*               H A U P T P R O G R A M M                            *}
{**********************************************************************}

var Datei : TestFile;                                      { Testdatei }

begin
  clrscr;
  writeln( 'RECLOCKP Demonstration des Record-Locking mit DOS ',
           '  (c) 1992 by Michael Tischer' );
  writeln( '====================================================',
           '===========================' );

  if ( ShareInst ) then                  { Share-Programm installiert? }
    begin
      if OeffneDatei( Datei ) then      { Datei erffnet oder erzeugt? }
        begin
          NetzEdit( Datei );    { Demonstration der Netzwerkfunktionen }
          NetClose( Datei );                        { Datei schliessen }
        end
      else
        writeln( #13#10'Fehler beim ffnen der Netzwerkdatei, ' +
                       'Fehlernummer: ', NetError );
      ClrScr;
    end
  else
    writeln( #13#10'Test nicht mglich, SHARE mu installiert sein' );
end.
