{**********************************************************************}
{*                               R T C P                              *}
{*--------------------------------------------------------------------*}
{*    Aufgabe        : stellt zwei Funktionen zur Verfgung, mit denen*}
{*                     Daten in die batteriegepufferte Echtzeituhr    *}
{*                     geschriebne und wieder ausgelesen werden knnen*}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  : 10.07.1987                                     *}
{*    letztes Update : 17.02.1991                                     *}
{**********************************************************************}
                                                                        
program RTCP;
                                                                        
Uses Crt;                                     { die CRT-Unit einbinden }
                                                                        
type Tage = array[1..7] of string[10];  { Array mit Namen der Wochent. }
                                                                        
const RTCAdrPort  = $70;                     { Adress-Register der RTC }
      RTCDtaPort  = $71;                      { Daten-Register der RTC }
                                                                        
      SEKUNDE     =  $00;   { Adressen einiger Speicherstellen der RTC }
      MINUTE      =  $02;
      STUNDE      =  $04;
      TAGDERWOCHE =  $06;
      TAG         =  $07;
      MONAT       =  $08;
      JAHR        =  $09;
      STATUSA     =  $0A;
      STATUSB     =  $0B;
      STATUSC     =  $0C;
      STATUSD     =  $0D;
      DIAGNOSE    =  $0E;
      JAHRHUNDERT =  $32;

      WochenTag   : Tage = ('Sonntag', 'Montag', 'Dienstag', 'Mittwoch',
                            'Donnerstag', 'Freitag', 'Samstag');
                                                                        
{**********************************************************************}
{* RTCRead: den Inhalt einer der Speicherstellen der RTC auslesen     *}
{* Eingabe : ADRESSE = die Adresse der Speicherstelle in der RTC      *}
{* Ausgabe : der Inhalt dieser Speicherstelle                         *}
{* Info    : liegt die Adresse auerhalb des erlaubten Bereichs       *}
{*           (0 bis 63), wird der Wert -1 zurckgeliefert             *}
{**********************************************************************}
                                                                        
function RTCRead(Adresse : integer) : integer;
                                                                        
begin
 if (Adresse < 0) or (Adresse > 63)            { ist die Adresse o.k.? }
  then RTCRead := -1                                           { NEIN! }
  else
   begin
    Port[RTCAdrPort] := Adresse;      { Adresse an die RTC bermitteln }
    RTCRead := Port[RTCDtaPort]                { ihren Inhalt auslesen }
   end
end;
                                                                        
{**********************************************************************}
{* RTCDT: liest eine der Speicherstellen des Datums oder der Zeit aus *}
{*        der RTC aus und setzt das Ergebnis in einen Binr-Wert um,  *}
{*        falls die RTC im dem BCD-Format arbeitet                    *}
{* Eingabe : ADRESSE = die Adresse der Speicherstelle in der RTC      *}
{* Ausgabe : der Inhalt dieser Speicherstelle als Binr-Wert          *}
{* Info    : liegt die Adresse auerhalb des erlaubten Bereichs (0 -  *}
{*           63) wird der Wert -1 zurckgeliefert                     *}
{**********************************************************************}
                                                                        
function RTCDT(Adresse : integer) : integer;
                                                                        
var Wert : integer;           { zum Speichern eines eingelesenen Werts }
                                                                        
begin
 if (RTCRead(STATUSB) and 2 = 0)              { BCD- oder Binr-Modus? }
  then RTCDT := RTCRead(Adresse)                     { ist Binr-Modus }
  else                                                 { ist BCD-Modus }
   begin
    Wert := RTCRead(Adresse);        { Inhalt der Speicherstelle holen }
    RTCDT := (Wert shr 4) * 10 + Wert and 15  { BCD nach Binr wandeln }
   end
end;
                                                                        
{**********************************************************************}
{* RTCWrite: einen Wert in eine der Speicherst. der RTC schreiben     *}
{* Eingabe : ADRESSE = die Adresse der Speicherstelle in der RTC      *}
{*           INHALT  = neuer Wert fr diese Speicherstelle            *}
{* Ausgabe : keine                                                    *}
{* Info    : die Adresse darf im Bereich 0 bis 63 liegen              *}
{**********************************************************************}
                                                                        
procedure RTCWrite(Adresse : integer; { die Adresse der Speicherstelle }
                   Inhalt  : byte);                 { ihr neuer Inhalt }
                                                                        
begin
 Port[RTCAdrPort] := Adresse;         { Adresse an die RTC bermitteln }
 Port[RTCDtaPort] := Inhalt                     { neuen Wert schreiben }
end;
                                                                        
{**********************************************************************}
{*                            HAUPTPROGRAMM                           *}
{**********************************************************************}
                                                                        
begin
 ClrScr;                                          { Bildschirm lschen }
 writeln('RTCP (c) 1987, 91 by Michael Tischer'#13#10);
 writeln('Informationen aus der batteriegepufferten Echtzeituhr');
 writeln('====================================================='#13#10);
 if RTCRead(Diagnose) and 128 = 0 then        { ist die Batterie o.k.? }
  begin                                        { die Batterie ist o.k. }
   writeln('- die Uhr wird im ', (RTCRead(STATUSB) and 2)*6+12,
           ' Stunden-Modus betrieben');
   writeln('- die Uhrzeit: ', RTCDT(STUNDE), ':', RTCDT(MINUTE):2,
           ':', RTCDT(SEKUNDE):2);
   write('- das Datum: ', WochenTag[RTCDT(TAGDERWOCHE)]);
   writeln(', der ',RTCDT(TAG), '.', RTCDT(MONAT), '.',
           RTCDT(JAHRHUNDERT), RTCDT(JAHR));
  end
 else                                  { die Batterie der RTC ist leer }
  write('       ACHTUNG! Die Batterie der Uhr ist leer')
end.
