;**********************************************************************;
;*                         S 3 2 2 0 P A . A S M                      *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : enthlt die Routinen zur Arbeit mit Sprites im *;
;*                     320*200-256-Grafikmodus der VGA-Karte          *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  :  8.09.1990                                     *;
;*    letztes Update : 14.01.1991                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM /mx S3220PA;    oder   TASM -mx S3220PA   *;
;*                     ... dann mit S3220P.PAS linken                 *;
;**********************************************************************;


;== Konstanten =========================================================

SC_INDEX       = 3c4h             ;Indexregister des Sequencer-Ctrl.
SC_MAP_MASK    = 2                ;Nummer des Map-Mask-Registers
SC_MEM_MODE    = 4                ;Nummer des Memory-Mode-Registers

GC_INDEX       = 3ceh             ;Indexregister des Graphics-Ctrl.
GC_GRAPH_MODE  = 5                ;Nummer des Graphics-Mode-Registers

VERT_RETRACE   = 3DAh             ;Input Status-Register #1
PIXX           = 320              ;horizontale Auflsung

;== Datensegment ===========================================================

DATA   segment word public

DATA   ends

;== Programm ===============================================================

CODE       segment byte public    ;das Programmsegment

	   assume cs:code, ds:data

;-- Public-Deklarationen -----------------------------------------------

public     blockmove

;-----------------------------------------------------------------------
;-- BLOCKMOVE: verschiebt eine Gruppe von Pixeln im Video-RAM
;-- Aufruf von TP: blockmove( vonpage : byte; vonx, vony : integer;
;--                           nachpage : byte; nachx, nachy : integer;
;--                           breite, hoehe: byte; bmskp : pointer );

blockmove  proc near

sframe4    struc                  ;Struktur zum Zugriff auf den Stack
bp4        dw ?                   ;nimmt BP auf
additiv    dw ?                   ;lokale variable
restz      dw ?
movec      dw ?
ret_adr4   dw ?                   ;Rcksprungadresse zum Aufrufer
bmskp      dd ?                   ;Zeiger auf Puffer mit der Bit-Mask
hoehe      dw ?                   ;Hhe
breite     dw ?                   ;Breite
nachy      dw ?                   ;nach Y-Ordinate
nachx      dw ?                   ;nach X-Ordinate
nachpage   dw ?                   ;nach Seite
vony       dw ?                   ;von Y-Ordinate
vonx       dw ?                   ;von X-Ordinate
vonpage    dw ?                   ;von Seite
sframe4    ends                   ;Ende der Struktur

frame      equ [ bp - bp4 ]       ;adressiert die Elemente der Struktur

	   sub   sp,6             ;6 Bytes fr lokale Variablen

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  ds
           cld                    ;bei Stringbefehlen aufwrts

	   mov   dx,GC_INDEX      ;aktuellen Write-Mode holen und
	   mov   al,GC_GRAPH_MODE ;Write-Mode 1 einstellen
	   out   dx,al
	   inc   dx
	   in    al,dx
	   push  ax               ;aktuellen Modus auf Stack merken
	   and   al,not 3
	   or    al,1
	   out   dx,al

	   mov   al,4             ;DS auf Anfang der VON-Seite
	   mov   cl,byte ptr frame.vonpage
	   mul   cl
	   or    al,0A0h
	   xchg  ah,al
	   mov   ds,ax

	   mov   al,4             ;ES auf Anfang der NACH-Seite
	   mov   cl,byte ptr frame.nachpage
	   mul   cl
	   or    al,0A0h
	   xchg  ah,al
	   mov   es,ax

	   mov   ax,PIXX / 4      ;SI auf Startposition VON
	   mul   frame.vony
	   mov   si,frame.vonx
	   shr   si,1
	   shr   si,1
	   add   si,ax

	   mov   ax,PIXX / 4      ;DI auf Zielposition NACH
	   mul   frame.nachy
	   mov   di,frame.nachx
	   shr   di,1
	   shr   di,1
	   add   di,ax

	   mov   dh,byte ptr frame.hoehe   ;DH = Zeilen
	   mov   dl,byte ptr frame.breite  ;DL = Bytes
	   shr   dl,1
	   shr   dl,1

	   mov   bx,PIXX / 4      ;BX als Offset zur nchsten Zeile
	   sub   bl,dl
	   xor   ch,ch            ;Hi-Byte des Counters ist immer 0
	   cmp   word ptr frame.bmskp+2,0    ;kein Hintergrund beachten?
	   jne   mt2                         ;Doch, andere Kopierroutine

	   push  dx               ;DX auf dem Stack sichern
	   mov   dx,SC_INDEX      ;Zugriff auf alle Bit-Planes sichern
	   mov   ah,0Fh
	   mov   al,SC_MAP_MASK
	   out   dx,ax
	   pop   dx               ;DX zurckholen

	   ;-- Kopierroutine fr alle vier Bit-Planes, also ohne Berck-
	   ;-- sichtigung des Hintergrundes

mt1:       mov   cl,dl            ;Anzahl Bytes nach CL

	   rep movsb              ;Zeile kopieren
	   add   di,bx            ;DI in nchste Zeile
	   add   si,bx            ;und auch SI
	   dec   dh               ;noch eine Zeile?
	   jne   mt1              ;Ja ---> weiter
	   jmp short mtend        ;Nein, Ausstieg aus der Routine vorb.

	   ;-- Kopierroutine fr individuelle Bit-Planes unter Zuhilfe-
	   ;-- nahme des bergebenen Bit-Mask-Arrays

mt2:       mov   byte ptr frame.restz,dh ;zunchst Variablen merken, die
	   mov   byte ptr frame.movec,dl ;hier auf dem Stack in lokalen
	   mov   frame.additiv,bx        ;Variablen festgeh. werden

	   mov   al,SC_MAP_MASK   ;jetzt permanent MAP-MASK-Register
	   mov   dx,SC_INDEX      ;adressieren
	   out   dx,al
	   inc   dx               ;DX jetzt auf Datenregister

	   push  ds
	   lds   bx,frame.bmskp   ;BX als Zeiger auf Bit-Mask-Array
	   mov   al,[bx]          ;erstes Byte laden
	   xor   ah,ah            ;Anfang mit einem geraden Byte
	   pop   ds

mt3:       mov   cl,byte ptr frame.movec ;Anzahl Bytes nach CL

mt4:       out   dx,al            ;Bit-Mask setzen
	   movsb                  ;4-Bytes kopieren

	   inc   ah               ;Odd/Even-Zhler inkrementieren
	   test  ah,1             ;jetzt wieder ungerade?
	   jne   mt5              ;Ja, Nibble verschieben

	   ;-- wieder auf geradem Byte, nchstes Byte aus Puffer -------

	   inc   bx               ;BX auf das nchste Bit-Masken-Byte
	   push  ds
	   mov   ds,word ptr frame.bmskp+2
	   mov   al,[bx]          ;nchste Byte laden
	   pop   ds
	   loop  mt4              ;nchsten vier Latches
	   jmp   short mt6

mt5:       shr   al,1             ;Bit-Maske des ungeraden Bytes in das
	   shr   al,1             ;Lo-Nibble holen
	   shr   al,1
	   shr   al,1
	   loop  mt4              ;nchsten vier Latches

mt6:       add   di,frame.additiv ;DI in nchste Zeile
	   add   si,frame.additiv ;und auch SI
	   dec   byte ptr frame.restz ;noch eine Zeile?
	   jne   mt3                  ;Ja ---> weiter

mtend:     mov   dx,GC_INDEX      ;wieder den alten Write-Mode ein-
	   pop   ax               ;stellen
	   mov   ah,al
	   mov   al,GC_GRAPH_MODE
	   out   dx,ax

	   pop   ds
	   pop   bp

           add   sp,6             ;SP ber lokale Variablen hinweg
	   ret   20               ;bei Rckkehr Parameter vom Stack
				  ;lschen
blockmove  endp

;== Ende ===============================================================

CODE       ends                   ;Ende des Codesegments
	   end                    ;Ende des Programms


