;**********************************************************************;
;*                         S 3 2 4 0 P A . A S M                      *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : enthlt die Routinen zur Arbeit mit Sprites im *;
;*                     320*400-256-Grafikmodus der VGA-Karte          *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  :  8.09.1990                                     *;
;*    letztes Update : 14.01.1991                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM /mx S3240PA;    oder   TASM -mx S3240PA   *;
;*                     ... dann mit S3240P.PAS linken                 *;
;**********************************************************************;


;== Konstanten =========================================================

SC_INDEX       = 3c4h             ;Indexregister des Sequencer-Ctrl.
SC_MAP_MASK    = 2                ;Nummer des Map-Mask-Registers
SC_MEM_MODE    = 4                ;Nummer des Memory-Mode-Registers

GC_INDEX       = 3ceh             ;Indexregister des Graphics-Ctrl.
GC_READ_MAP    = 4                ;Nummer des Read-Map-Registers

PIXX           = 320              ;horizontale Auflsung

;== Datensegment ===========================================================

DATA   segment word public

DATA   ends

;== Programm ===============================================================

CODE       segment byte public    ;das Programmsegment

	   assume cs:code, ds:data

;-- Public-Deklarationen -----------------------------------------------

public     copybuf2plane
public     copyplane2buf

;-----------------------------------------------------------------------
;-- CopyBuf2Plane: kopiert den Inhalt eines Puffers in einen rechtecki-
;--                gen Bereich einer Bit-Plane
;-- Aufruf von TP: CopyBuf2Plane( bufptr   : pointer;
;--                               nachpage : byte;
;--                               nachx,
;--                               nachy    : integer;
;--                               breite,
;--                               hoehe    : byte;
;--                               bg       : bool  );

copybuf2plane proc near

sfr0    struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
ret_adr0   dw ?                   ;Rcksprungadresse zum Aufrufer
bg         dw ?                   ;Hintergrund beachten
hoehe0     dw ?                   ;Hhe des Bereichs
breite0    dw ?                   ;Breite
nachy      dw ?                   ;nach Y-Ordinate
nachx      dw ?                   ;nach X-Ordinate
nachpage   dw ?                   ;nach Seite
bufptr0    dd ?                   ;Zeiger auf Puffer
sfr0    ends                   ;Ende der Struktur

fr         equ [ bp - bp0 ]          ;adr. die Elemente der Struktur
bfr        equ byte ptr [ bp - bp0 ] ;adr. Stack-Element als Byte

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  ds
           cld                    ;bei Stringbefehlen aufwrts

	   ;-- Segmentadresse fr Zugriff auf Video-RAM berechnen ------

	   mov   ah,0A0h          ;ES auf Anfang NACH-Seite
	   cmp   bfr.nachpage,0   ;ist es Seite 0?
	   je    cv0              ;Ja, AL schon o.k

	   mov   ah,0A8h          ;Nein, Seite 1 bei A800h

cv0:       xor   al,al            ;Lo-Byte ist immer Null
	   mov   es,ax

	   ;-- Offset fr Zielposition in Page berechnen ---------------

	   mov   ax,PIXX / 4      ;DI auf Zielposition NACH
	   mul   fr.nachy
	   mov   di,fr.nachx
           mov   cx,di            ;X-Koordinate fr Plane-Ber. merken
	   shr   di,1
	   shr   di,1
	   add   di,ax

	   ;-- die zu adressierende Bit-Plane einstellen ---------------

	   mov   ah,1             ;Plane-Nummer mu als Bit-Maske
	   and   cl,3             ;eingestellt werden
	   shl   ah,cl
	   mov   dx,SC_INDEX      ;Zugriff auf die zu bearbeitende
	   mov   al,SC_MAP_MASK   ;Bit-Plane sichern
	   out   dx,ax

	   ;-- Zhler fr die Kopierschleife laden ---------------------

	   mov   dh,bfr.hoehe0    ;DH = Zeilen
	   mov   dl,bfr.breite0   ;DL = Bytes
	   mov   bx,PIXX / 4      ;BX als Offset zur nchsten Zeile
	   sub   bl,dl
	   xor   ch,ch            ;Hi-Byte des Counters ist immer 0

	   lds   si,fr.bufptr0    ;DS:SI auf den Puffer setzen

	   cmp   bfr.bg,0         ;kein Hintergrund beachten?
	   jne   cv2              ;doch, andere Kopierroutine

	   ;-- Kopierroutine fr eine Bit-Plane, ohne Bercksichti- ----
	   ;-- gung des Hintergrundes

cv1:       mov   cl,dl            ;Anzahl Bytes nach CL

	   rep movsb              ;Zeile kopieren
	   add   di,bx            ;DI in nchste Zeile
	   dec   dh               ;noch eine Zeile?
	   jne   cv1              ;Ja ---> weiter

	   jmp   short cvend      ;Nein, Puffer volstndig kopiert

	   ;-- Kopierroutine fr individuelle Bit-Planes unter Auswer- -
	   ;-- tung des bergebenen Puffers

cv2:       mov   cl,dl            ;Anzahl Bytes nach CL

cv3:       lodsb                  ;Byte aus Puffer laden
	   cmp   al,255           ;Hintergrund-Byte?
	   je    cv5              ;Ja ---> nicht mitkopieren
	   stosb                  ;Nein, im Video-RAM ablegen
	   loop  cv3              ;nchstes Byte bearbeiten

cv4:       ;-- Zeiger auf Video-RAM in die nchste Zeile schalten -----

	   add   di,bx            ;DI in nchste Zeile
	   dec   dh               ;noch eine Zeile?
	   jne   cv2              ;Ja ---> weiter
	   jmp   short cvend      ;Nein, Puffer volstndig kopiert

cv5:       ;-- Hintergrund-Byte, nicht mitkopieren ---------------------

	   inc   di               ;dieses Byte im VRAM nicht beschr.
	   loop  cv3              ;noch ein Byte in dieser Zeile?
	   jmp   cv4              ;Nein ---> nchste Zeile bearbeiten

cvend:     pop   ds
	   pop   bp

	   ret   16               ;bei Rckkehr Parameter vom Stack
				  ;lschen
copybuf2plane endp

;-----------------------------------------------------------------------
;-- CopyPlane2Buf: Kopiert ein rechtecikgen Teil aus einer Bit-Plane in
;--                einen Puffer
;-- Aufruf von TP: CopyPlane2Buf( bufptr : pointer;
;--                               vonpage: byte;
;--                               vonx,
;--                               vony    : integer;
;--                               breite,
;--                               hoehe   : byte );

copyplane2buf proc near

sfr1    struc                  ;Struktur zum Zugriff auf den Stack
bp1        dw ?                   ;nimmt BP auf
ret_adr1   dw ?                   ;Rcksprungadresse zum Aufrufer
hoehe1     dw ?                   ;Hhe des Bereichs in Punktzeilen
breite1    dw ?                   ;Breite des Bereichs in Punkten
vony       dw ?                   ;von Y-Ordinate
vonx       dw ?                   ;von X-Ordinate
vonpage    dw ?                   ;von Seite
bufptr1    dd ?                   ;Zeiger auf Puffer
sfr1    ends                   ;Ende der Struktur

fr         equ [ bp - bp1 ]          ;adr. die Elemente der Struktur
bfr        equ byte ptr [ bp - bp1 ] ;adr. Stack-Element als Byte

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  ds

	   ;-- Segmentadresse fr Zugriff auf Video-RAM berechnen ------

	   mov   ah,0A0h          ;ES auf Anfang VON-Seite
	   cmp   bfr.vonpage,0   ;ist es Seite 0?
	   je    cc0              ;Ja, AL schon o.k

	   mov   ah,0A8h          ;Nein, Seite 1 bei A800h

cc0:       xor   al,al            ;Lo-Byte ist immer Null
	   mov   ds,ax

	   ;-- Offset in die auszulesende Seite bilden -----------------

	   mov   ax,PIXX / 4      ;SI auf Zielposition VON
	   mul   fr.vony
	   mov   si,fr.vonx
	   mov   cx,si            ;Koordinate auch in CX merken
	   shr   si,1
	   shr   si,1
	   add   si,ax

	   ;-- die zu adressierende Bit-Plane einstellen ---------------

	   and   cl,3             ;Bit-Maske fr die zu adressierende
	   mov   ah,cl            ;Map in AH berechnen
	   mov   al,GC_READ_MAP   ;nach AL die Registernummer
	   mov   dx,GC_INDEX      ;Indexadresse des Graphic-Ctrl laden
	   out   dx,ax            ;Read-Map-Register laden

	   ;-- Zhler fr die Kopierschleife laden ---------------------

	   mov   dh,bfr.hoehe1    ;DH = Zeilen
	   mov   dl,bfr.breite1   ;DL = Bytes
	   mov   bx,PIXX / 4      ;BX als Offset zur nchsten Zeile
	   sub   bl,dl
	   xor   ch,ch            ;Hi-Byte des Counters ist immer 0

	   les   di,fr.bufptr1    ;ES:DI auf den Puffer setzen

	   ;-- Kopierroutine fr eine Bit-Plane, ohne Bercksichti- ----
	   ;-- gung des Hintergrundes

cc1:       mov   cl,dl            ;Anzahl Bytes nach CL

	   rep movsb              ;Zeile kopieren
	   add   si,bx            ;SI in nchste Zeile
	   dec   dh               ;noch eine Zeile?
	   jne   cc1              ;Ja, weiter

	   pop   ds
	   pop   bp

           ret   14

copyplane2buf endp

;== Ende ===============================================================

CODE       ends                   ;Ende des Codesegments
	   end                    ;Ende des Programms
