{***********************************************************************
*                          S 6 4 3 5 P . P A S                         *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Arbeit mit Sprites in den       *
*                     640*350 Punkte EGA- und VGA-Grafikmodi mit 16    *
*                     Farben bei Nutzung von zwei Bildschirmseiten.    *
*                     Das Programm greift dabei auf die Assembler-     *
*                     Routinen aus den Moduln V16COLPA.ASM und         *
*                     S6435PA.ASM zurck.                              *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    :  5.12.1990                                       *
*  letztes Update am: 14.01.1991                                       *
***********************************************************************}

program S6435P;

uses dos, crt;

{-- externe Referenzen auf die Assembler-Routinen ---------------------}

{$L v16colpa}                              { Assembler-Modul einbinden }

procedure init640350; external;
procedure setpix( x, y : integer; farbe : byte ); external;
function  getpix( x, y: integer ) : byte ; external;
procedure setpage( page : integer ); external;
procedure showpage( page : integer ); external;

{$L s6435pa}                               { Assembler-Modul einbinden }

procedure CopyVideo2Buf( bufptr : pointer;
                         page   : byte;
                         vonx,
                         vony   : integer;
                         breite,
                         hoehe  : byte    ); external;

procedure CopyBuf2Video( bufptr : pointer;
                         page   : byte;
                         nachx,
                         nachy  : integer;
                         breite,
                         hoehe  : byte ); external;

procedure MergeAndCopyBuf2Video( spribufptr,
                                 hgbufptr,
                                 andbufptr  : pointer;
                                 page       : byte;
                                 nachx,
                                 nachy      : integer;
                                 breite,
                                 hoehe      : byte ); external;

{-- Konstanten --------------------------------------------------------}

const MAXX = 639;                         { Maximale X- und Y-Ordinate }
      MAXY = 349;

      OUT_LEFT   = 1;       { fr Kollisionsmarkierung in SpriteMove() }
      OUT_TOP    = 2;
      OUT_RIGHT  = 4;
      OUT_BOTTOM = 8;
      OUT_NO     = 0;                              { nirgendwo drauen }

{-- Typdeklarationen --------------------------------------------------}

type PIXBUF = record         { Informationen fr GetVideo und PutVideo }
                breitebyte,             { Breite des Bereichs in Bytes }
                hoehe  : byte;                     { Anzahl der Zeilen }
                pixblen: integer;            { Lnge des Pixel-Puffers }
                pixbptr: pointer;        { Zeiger auf den Pixel-Puffer }
              end;
     PIXPTR = ^PIXBUF;                 { Zeiger auf einen Pixel-Puffer }

     SPLOOK = record                  { Erscheinungsbild eines Sprites }
                breite,                                { Breite gesamt }
                hoehe  : byte;                   { Hhe in Punktzeilen }
                bmskp  : array [0..7] of pointer; { Ptr auf AND-Puffer }
                pixmp  : array [0..7] of PIXPTR; {Zeiger auf Pixel-Def.}
              end;                  { seiner Seite abgelegt wurde      }
     SPLP = ^SPLOOK;                 { Zeiger auf das Erscheinungsbild }

     SPID = record                           { Sprite-Beschreiber (ID) }
              splookp : SPLP;            { Zeiger auf Erscheinungsbild }
              x, y    : array [0..1] of integer;  { Koordinaten S. 0&1 }
              hgptr   : array [0..1] of PIXPTR;   { Zeiger auf Hinter- }
            end;                                  { grund Puffer       }
     SPIP = ^SPID;                     { Zeiger auf Sprite-Beschreiber }

     BYTEAR = array[0..10000] of byte;     { zur Adressierung der ver- }
     BARPTR = ^BYTEAR;                     { schiedenen Puffer         }

     KARTE = ( EGA, VGA, WEDERNOCH );

{***********************************************************************
*  IsEgaVga : Stellt fest, ob eine EGA- oder VGA-Karte installiert ist.*
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : EGA, VGA oder WEDERNOCH                                   *
***********************************************************************}

function IsEgaVga : KARTE;

var Regs : Registers;          { Prozessorregister fr Interruptaufruf }

begin
  Regs.AX := $1a00;                 { Funktion 1Ah gibt es nur bei VGA }
  Intr( $10, Regs );
  if ( Regs.AL = $1a ) then              { ist die Funktion verfgbar? }
    IsEgaVga := VGA
  else
    begin
      Regs.ah := $12;                     { Funktion 12h Unterfunktion }
      Regs.bl := $10;                     { 10h aufrufen               }
      intr($10, Regs);                           { Video-BIOS aufrufen }
      if ( Regs.bl <> $10 ) then IsEgaVga := EGA
                            else IsEgaVga := WEDERNOCH;
    end;
end;

{***********************************************************************
*  PrintChar : Schreibt ein Zeichen im Grafikmodus in den Bildschirm   *
**--------------------------------------------------------------------**
*  Eingabe :   zeichen = das aufzubauende Zeichen                      *
*              x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************}

procedure PrintChar( zeichen : char; x, y : integer; vg, hg : byte );

type ZDEF = array[0..255,0..7] of byte;       { Aufbau des Zeichensatz }
     ZPTR = ^ZDEF;                            { Zeiger auf Zeichensatz }

var  Regs  : Registers;                 { Register zum Interruptaufruf }
     ch    : char;                      { einzelner Punkt des Zeichens }
     i, k,                                           { Schleifenzhler }
     Maske : byte;                  { Bitmaske fr Aufbau des Zeichens }

const fptr : ZPTR = NIL;               { Zeiger auf Zeichensatz im ROM }

begin
  if fptr = NIL then         { Zeiger auf Zeichensatz schon ermittelt? }
    begin                                                       { Nein }
      Regs.AH := $11;                   { Funktion $11 des Video-BIOS, }
      Regs.AL := $30;                   { Unterfunktion $30 aufrufen   }
      Regs.BH := 3;             { Zeiger auf 8*8 Zeichensatz ermitteln }
      intr( $10, Regs );
      fptr := ptr( Regs.ES, Regs.BP );         { Zeiger zusammensetzen }
    end;

  if ( hg = 255 ) then                 { Zeichen transparent zeichnen? }
    for i := 0 to 7 do             { Ja, nur Vordergrund-Punkte setzen }
      begin
        Maske := fptr^[ord(zeichen),i]; {Bitmuster fr eine Zeile holen}
        for k := 0 to 7 do
          begin
            if ( Maske and 128 <> 0 ) then             { Punkt setzen? }
              setpix( x+k, y+i, vg );                             { Ja }
            Maske := Maske shl 1;
          end;
      end
  else                        { Nein, auch Hintergrund bercksichtigen }
    for i := 0 to 7 do                        { die Zeilen durchlaufen }
      begin
        Maske := fptr^[ord(zeichen),i]; {Bitmuster fr eine Zeile holen}
        for k := 0 to 7 do
          begin
            if ( Maske and 128 <> 0 ) then              { Vordergrund? }
              setpix( x+k, y+i, vg )                              { Ja }
            else
              setpix( x+k, y+i, hg );              { Nein, Hintergrund }
            Maske := Maske shl 1;
         end;
      end;
end;

{***********************************************************************
*  PrintString: Schreibt einen String im Grafikmodus in den Bildschirm *
**--------------------------------------------------------------------**
*  Eingabe :   x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*              tstr    = der auszugebende String                       *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************}

procedure PrintString( x, y : integer; vg, hg : byte; tstr : string );

var i : integer;                                     { Schleifenzhler }

begin
  for i := 1 to length( tstr ) do             { den String durchlaufen }
    begin
      PrintChar( tstr[i], x, y, vg, hg );               { und ausgeben }
      inc( x, 8 );                    { Ausgabeposition inkrementieren }
    end;
end;

{***********************************************************************
*  Line: Zieht eine Linie innerhalb nach dem Bresenham-Algorithmus     *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endekoordinate                                   *
*            FARBE  = Farbe der Linienpunkte                           *
***********************************************************************}

procedure Line( x1, y1, x2, y2 : integer; farbe : byte );

var d, dx, dy,
    aincr, bincr,
    xincr, yincr,
    x, y                 : integer;

{-- Unterprozedur, vertauscht zwei Integer-Variablen ------------------}

procedure SwapInt( var i1, i2: integer );

var dummy : integer;

begin
  dummy := i2;
  i2    := i1;
  i1    := dummy;
end;

{-- Hauptprozedur -----------------------------------------------------}

begin
  if ( abs(x2-x1) < abs(y2-y1) ) then   { ber X- oder Y-Achse laufen? }
    begin                                           { ber die Y-Achse }
      if ( y1 > y2 ) then                              { y1 grer y2? }
        begin
          SwapInt( x1, x2 );                   { Ja, X1 mit X2 und     }
          SwapInt( y1, y2 );                   { Y1 mit Y2 vertauschen }
        end;

      if ( x2 > x1 ) then xincr := 1      { Spalteninkrement festlegen }
                     else xincr := -1;

      dy := y2 - y1;
      dx := abs( x2-x1 );
      d  := 2 * dx - dy;
      aincr := 2 * (dx - dy);
      bincr := 2 * dx;
      x := x1;
      y := y1;

      setpix( x, y, farbe );                     { ersten Punkt setzen }
      for y:=y1+1 to y2 do              { Line auf X-Achse durchlaufen }
        begin
          if ( d >= 0 ) then
            begin
              inc( x, xincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          setpix( x, y, farbe );
        end;
    end
  else                                              { ber die X-Achse }
    begin
      if ( x1 > x2 ) then                              { x1 grer x2? }
        begin
          SwapInt( x1, x2 );                   { Ja, X1 mit X2 und     }
          SwapInt( y1, y2 );                   { Y1 mit Y2 vertauschen }
        end;

      if ( y2 > y1 ) then yincr := 1       { Zeileninkrement festlegen }
                     else yincr := -1;

      dx := x2 - x1;
      dy := abs( y2-y1 );
      d  := 2 * dy - dx;
      aincr := 2 * (dy - dx);
      bincr := 2 * dy;
      x := x1;
      y := y1;

      setpix( x, y, farbe );                     { ersten Punkt setzen }
      for x:=x1+1 to x2 do              { Line auf X-Achse durchlaufen }
        begin
          if ( d >= 0 ) then
            begin
              inc( y, yincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          setpix( x, y, farbe );
       end;
    end;
end;

{***********************************************************************
*  GetVideo: Holt den Inhalt eines rechteckigen Bereichs aus dem       *
*            Video-RAM in einen Puffer                                 *
**--------------------------------------------------------------------**
*  Eingabe : PAGE   = Bildschirmseite (0 oder 1)                       *
*            X1, Y1 = Startkoordinate                                  *
*            BREITE = Breite des rechteckigen Bereichs in Punkten      *
*            HOEHE  = Hhe des rechteckigen Bereichs in Punkten        *
*            BUFPTR = Zeiger auf Pixel-Puffer, in dem die Informa-     *
*                     tionen abgelegt werden sollen                    *
*  Ausgabe : Zeiger auf den angelegten Pixel-Puffer mit dem Inhalt     *
*            des angegebenen Bereichs                                  *
*  Info    : Wird fr den Parameter BUFPTR der Wert NIL bergeben,     *
*            wird ein neuer Pixel-Puffer ber den Heap allokiert und   *
*            zurckgeliefert. Dieser Puffer kann bei einem neuer-      *
*            lichen Aufruf wieder angegeben werden, sofern der bis-    *
*            herige Inhalt nicht mehr bentigt wird und die Gre des  *
*            rechteckigen Bereichs gegenber dem vorhergehenden Aufruf *
*            unverndert bleibt.                                       *
*            Der angegebene Bereich mu an einer durch acht teilbaren  *
*            X-Ordinate beginnen und sich ber ein vielfaches von acht *
*            Punkten erstrecken.                                       *
***********************************************************************}

function GetVideo( page : byte; x1, y1 : integer;
                   breite, hoehe : byte; bufptr : PIXPTR ) : PIXPTR;

begin
  if ( bufptr = NIL ) then        { kein Puffer beim Aufruf bergeben? }
    begin                                             { Nein, erzeugen }
      new( bufptr );                             { Pixelpuffer anlegen }
      getmem( bufptr^.pixbptr, (breite*hoehe) div 2 ); { Pix-P. allok. }
      bufptr^.hoehe := hoehe;             { Hhe des Puffers in Zeilen }
      bufptr^.breitebyte := breite div 8;  {Breite einer Zeile in Bytes}
      bufptr^.pixblen := (breite*hoehe) div 2;  {Gesamtlnge des Puffer}
    end;

  CopyVideo2Buf( bufptr^.pixbptr, page, x1, y1, breite div 8, hoehe );
  GetVideo := bufptr;    { Zeiger auf Puffer an Aufrufer zurckliefern }
end;

{***********************************************************************
*  PutVideo: Schreibt den Inhalt eines zuvor ber GetVideo gesicherten *
*            rechteckigen Bildschirmbereichs wieder in den Video-RAM   *
**--------------------------------------------------------------------**
*  Eingabe : BUFPTR = Beim vorherigen Aufruf von Getvideo zurck-      *
*                     gelieferter Zeiger auf den Pixel-Puffer          *
*            PAGE   = Bildschirmseite (0 oder 1)                       *
*            X1, Y1 = Startkoordinate                                  *
*  Info    : Der Pixel-Puffer wird durch diese Prozedur nicht gelscht,*
*            dafr mu die Prozedur FreiPixBuf aufgerufen werden.      *
*            Die angegebene X-Ordinate mu ein Vielfaches von acht     *
*            sein!                                                     *
***********************************************************************}

procedure PutVideo( bufptr : PIXPTR; page : byte; x1, y1 : integer );

begin
  CopyBuf2Video( bufptr^.pixbptr, page, x1, y1,
                 bufptr^.breitebyte, bufptr^.hoehe );
end;

{***********************************************************************
*  FreePixBuf: Lscht einen Pixel-Puffer, der bei einem vorherigen     *
*              Aufruf von GetVideo ber den Heap allokiert wurde       *
**--------------------------------------------------------------------**
*  Eingabe : BUFPTR = Beim vorherigen Aufruf von Getvideo zurck-      *
*                     gelieferter Zeiger auf den Pixel-Puffer          *
***********************************************************************}

procedure FreePixBuf( bufptr : PIXPTR );

begin
  freemem( bufptr^.pixbptr, bufptr^.pixblen );
  dispose( bufptr );
end;

{***********************************************************************
*  CreateSprite: Erzeugt ein Sprite auf der Basis einer bereits kompi- *
*                lierten Punktmusters                                  *
**--------------------------------------------------------------------**
*  Eingabe : SPLOOKP = Zeiger auf die von CompileSprite erzeugte       *
*                      Datenstruktur                                   *
*  Ausgabe : Zeiger auf die erstellte Sprite-Struktur                  *
***********************************************************************}

function CreateSprite( splookp : SPLP ) : SPIP;

var spidp : SPIP;                { Zeiger auf erzeugte Sprite-Struktur }

begin
  new( spidp );           { Speicher fr Sprite-Beschreiber allokieren }
  spidp^.splookp := splookp;                    { Daten in die Sprite- }

  {-- zwei Hintergrund-Puffer erzeugen, indem ein entsprechend groer -}
  {-- Bereich aus dem Video-RAM ber GetVideo gesichert wird          -}

  spidp^.hgptr[0] := GetVideo( 0, 0, 0, splookp^.breite,
                               splookp^.hoehe, NIL );
  spidp^.hgptr[1] := GetVideo( 0, 0, 0, splookp^.breite,
                               splookp^.hoehe, NIL );
  CreateSprite := spidp;    { Zeiger auf Sprite-Struktur zurckliefern }
end;

{***********************************************************************
*  CompileSprite: Erzeugt das Punkt- und Bitmusters eines Sprites,     *
*                 auf der Basis einer zur Laufzeit angegebenen Sprite- *
*                 definition                                           *
**--------------------------------------------------------------------**
*  Eingabe : BUFP    = Zeiger auf Array mit Zeigern auf Strings, die   *
*                      das Zeichenmuster des Sprites enthalten         *
*            HOEHE   = Hoehe des Sprites und damit Anzahl der Strings  *
*  Info    : Im bergebenen Sprite-Muster steht ein Leerzeichen fr    *
*            einen Hintergrund-Punkt, das A fr den Farbcode 0, B fr  *
*            1, C fr 2 usw.                                           *
***********************************************************************}

function CompileSprite( var buf; hoehe : byte ) : SPLP;

type BYPTR  = ^byte;                             { Zeiger auf ein Byte }

var  sbreite,                                     { Breite der Strings }
     gbreite,                                     { Breite der Sprites }
     c,                         { nimmt Zeichen aus C-Sprite-Array auf }
     i, k, l, y,                                  { Schleifenvariablen }
     andc,                  { Punktzhler fr Erstellung der Bit-Maske }
     andm     : byte;                                     { Punktmaske }
     andindex : integer;                         { Index in AND-Puffer }
     splookp  : SPLP;            { Zeiger auf erzeugte Sprite-Struktur }
     lspb     : BYPTR;                   { Laufzeiger in Sprite-Puffer }
     andp,                                     { Zeiger auf AND-Puffer }
     bptr     : BARPTR; {zur Adressierung des Puffers mit dem Erschbild}
     tpix     : PIXPTR;           { Zeiger auf temporren Pixel-Puffer }

{-- Unterprozedur AndBufInit: Initialisiert einen AND-Puffer ----------}

procedure AndBufInit( bufp : BARPTR );

begin
  andp := bufp;                             { Zeiger auf Puffer merken }
  andindex := 0;                      { am Anfang des Puffers beginnen }
  andm := 0;                                { Bit-Maske ist zunchst 0 }
  andc := 0;                            { noch kein Bit im ersten Byte }
end;

{-- Unterprozedur AndBufAppendBit: ein Bit an AND-Puffer anhngen -----}

procedure AndBufAppendBit( bit : byte );

begin
  andm := andm or bit;               { Bit in Bitposition 0 einblenden }
  if andc = 7 then                          { ist das Byte jetzt voll? }
    begin                                                         { Ja }
      andp^[andindex] := andm;                { Byte im Puffer ablegen }
      inc( andindex );                     { nchstes Byte adressieren }
      andm := 0;                                  { Bit-Maske wieder 0 }
      andc := 0;          { im nchsten Byte mit erstem Bit fortfahren }
    end
  else                                  { das Byte ist noch nicht voll }
    begin
      inc( andc );                           { ein Bit mehr bearbeitet }
      andm := andm shl 1;                          { Bit-Maske shiften }
    end;
end;

{-- Unterprozedur AndBufEnd: AND-Puffer schlieen ---------------------}

procedure AndBufEnd;

begin
  if ( andc <> 0 ) then                { letztes Byte noch nicht voll? }
    andp^[andindex] := andm shl (7 - andc);        { Nein, abschlieen }
end;

begin
  {-- Sprite-Look-Struktur erzeugen und mit Daten fllen --------------}

  new( splookp );
  bptr := @buf;                        { Zeiger auf Logo-Puffer setzen }
  sbreite := bptr^[0]; { Lnge der Strings und damit Logo-Breite holen }
  gbreite := ( ( sbreite + 7 + 7 ) div 8 ) * 8;         { Gesamtbreite }
  splookp^.breite := gbreite;                 { Breite und Hhe merken }
  splookp^.hoehe := hoehe;

  setpage( 1 );                          { Sprites in Seite 1 aufbauen }
  showpage( 0 );                               { aber Seite 0 anzeigen }
  tpix := GetVideo( 1, 0, 0, gbreite, hoehe, NIL ); {Hintergrund merken}

  {-- das Sprite acht mal aufbauen und kodieren -----------------------}

  for l := 0 to 7 do 
    begin          { zunchst Hintergrund mit schwarzen Punkten fllen }
      for y := 0 to hoehe-1 do
        Line( 0, y, gbreite-1, y, 0 );

      {-- Speicher fr AND-Puffer anlegen und initialisieren ----------}

      getmem( splookp^.bmskp[ l ], (gbreite*hoehe) div 8 );
      AndBufInit( splookp^.bmskp[ l ] );

      for i := 0 to hoehe-1 do                    { Zeilen durchlaufen }
        begin
          for y := 1 to l do       { AND-Bits fr linken Rand erzeugen }
            AndBufAppendBit( 1 );

          for k := 0 to sbreite-1 do             { Spalten durchlaufen }
            begin
              c := bptr^[i*(sbreite+1)+k+1];             { Farbe holen }
              if ( c = 32 ) then                  { Hintergrund-Punkt? }
                begin                          { Ja, Farbcode 0 setzen }
                  setpix( k+l, i, 0 );
                  AndBufAppendBit( 1 );     { Hintergrund-Punkt bleibt }
                end
              else               { Nein, Farbcode wie angegeben setzen }
                begin
                  setpix( k+l, i, c-ord('@') );
                  AndBufAppendBit( 0 ); { Hintergrund-Punkt ausblenden }
                end;
            end;
          for y := gbreite-sbreite-l downto 1 do    { AND-Bits fr den }
            AndBufAppendBit( 1 );                   { rechten R. anh.  }
        end;
      AndBufEnd;                                { AND-Puffer schlieen }

      {-- Pixel-Muster des Sprites aus Video-RAM holen ----------------}
      splookp^.pixmp[ l ] := GetVideo( 1, 0, 0, gbreite, hoehe, nil );

    end;                          { nchstes der acht Sprites aufbauen }

  PutVideo( tpix, 1, 0, 0 );         { Sprite-Hintergrund in Seite 1   }
  FreePixBuf( tpix );                { restaurieren und Puffer lschen }

  CompileSprite := splookp;   { Zeiger auf Sprite-Puffer zurckliefern }
end;

{***********************************************************************
*  PrintSprite : Sprite in einer bestimmten Seite ausgeben             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in der das Sprite gezeichnet werden soll   *
*                    (0 oder 1)                                        *
***********************************************************************}

procedure PrintSprite( spidp : SPIP; seite : byte );

var x : integer;                              { X-Ordinate des Sprites }

begin
  x := spidp^.x[seite];
  MergeAndCopyBuf2Video( spidp^.splookp^.pixmp[x mod 8]^.pixbptr,
                         spidp^.hgptr[seite]^.pixbptr,
                         spidp^.splookp^.bmskp[x mod 8],
                         seite,
                         x and not(7),
                         spidp^.y[seite],
                         spidp^.splookp^.breite div 8,
                         spidp^.splookp^.hoehe );
end;

{***********************************************************************
*  GetSpriteBg: Hintergrund eines Sprites holen und an der dafr vor-  *
*               gesehenen Position ablegen                             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, aus der der Hintergrund geholt werden soll *
*                    (0 oder 1)                                        *
***********************************************************************}

procedure GetSpriteBg( spidp : SPIP; seite : BYTE );

var dummy : PIXPTR;

begin
  dummy := GetVideo( seite, spidp^.x[seite] and not(7), spidp^.y[seite],
                     spidp^.splookp^.breite, spidp^.splookp^.hoehe,
                     spidp^.hgptr[seite] );
end;

{***********************************************************************
*  RestoreSpriteBg: Zuvor gesicherten Hintergrund eines Sprites wieder *
*                   in die ursprngliche Grafikseite kopieren          *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in die der Hinntergrund zurckkopiert      *
*                    werden soll (0 oder 1)                            *
***********************************************************************}

procedure RestoreSpriteBg( spidp : SPIP; seite : BYTE );

begin
 PutVideo( spidp^.hgptr[seite], seite,
           spidp^.x[seite] and not(7), spidp^.y[seite] );
end;

{***********************************************************************
*  MoveSprite: Sprite in einer Bildschirmseite bewegen                 *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP  = Zeiger auf die Sprite-Struktur                   *
*            SEITE  = Seite, in die der Hintergrund zurckkopiert      *
*                     werden soll (0 oder 1)                           *
*            DELTAX = Bewegungszhler in X- und Y-Richtung             *
*            DELTAY                                                    *
*  Ausgabe : Kollisionsmarkierung, siehe Konstanten OUT_...            *
***********************************************************************}

function MoveSprite( spidp : SPIP; seite : byte;
                     deltax, deltay : integer    ) : byte;

var neux, neuy : integer;                     { neue Sprite-Koordinate }
    out        : byte;                   { Kollision mit Rand anzeigen }

begin
 {-- X-Koordinate verschieben und Kollision mit Rand testen -----------}

 neux := spidp^.x[seite] + deltax;
 if ( neux  < 0 ) then
   begin
     neux := 0 - deltax - spidp^.x[seite];
     out := OUT_LEFT;
   end
 else
   if ( neux > MAXX - spidp^.splookp^.breite ) then
     begin
       neux := (2*(MAXX+1))-neux-2*(spidp^.splookp^.breite);
       out := OUT_RIGHT;
     end
  else
     out := OUT_NO;

 {-- Y-Koordinate verschieben und Kollision mit Rand testen -----------}

 neuy := spidp^.y[seite] + deltay;                      { oberer Rand? }
 if ( neuy < 0 ) then                                   { oberer Rand? }
   begin                                 { Ja, deltay mu negativ sein }
     neuy := 0 - deltay - spidp^.y[seite];
     out := out or OUT_TOP;
   end
 else
   if ( neuy + spidp^.splookp^.hoehe > MAXY+1  ) then  { unterer Rand? }
     begin                               { Ja, deltay mu positiv sein }
       neuy := (2*(MAXY+1))-neuy-2*(spidp^.splookp^.hoehe);
       out := out or OUT_BOTTOM;
     end;

 {-- neue Position nur setzen, wenn gegenber der alten verndert -----}

 if ( neux <> spidp^.x[seite] ) or  ( neuy <> spidp^.y[seite] ) then
   begin                                           { ist neue Position }
     RestoreSpriteBg( spidp, seite );       { Hintergrund zurcksetzen }
     spidp^.x[seite] := neux;                { neue Koordinaten merken }
     spidp^.y[seite] := neuy;
     GetSpriteBg( spidp, seite );            { neuen Hintergrund holen }
     PrintSprite( spidp, seite );  { Sprite in angegeb. Seite zeichnen }
   end;

 MoveSprite := out;
end;

{***********************************************************************
*  SetSprite: Sprite an eine bestimmte Position versetzen              *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP  = Zeiger auf die Sprite-Struktur                   *
*            x0, y0 = Sprite-Koordinate fr die Seite 0                *
*            x1, y1 = Sprite-Koordinate fr die Seite 1                *
*  Info    : Der Aufruf dieser Funktion sollte dem ersten Aufruf von   *
*            MoveSprite() vorausgehen                                  *
***********************************************************************}

procedure SetSprite( spidp : SPIP; x0, y0, x1, y1 : integer );

begin
 spidp^.x[0] := x0;            { Koordinaten in Sprite-Struktur merken }
 spidp^.x[1] := x1;
 spidp^.y[0] := y0;
 spidp^.y[1] := y1;

 GetSpriteBg( spidp, 0 );                { Sprite-Hintergrund in Seite }
 GetSpriteBg( spidp, 1 );                { 0 und 1 holen               }
 PrintSprite( spidp, 0 );                          { Sprite in Seite 0 }
 PrintSprite( spidp, 1 );                          { und 1 zeichnen    }
end;

{***********************************************************************
*  Demo: Demonstriert den Einsatz der verschiedenen Funktionen aus     *
*        diesem Modul                                                  *
***********************************************************************}

procedure Demo;

const GleiterRauf :array [1..20] of string[32] =
                ( '               AA               ',
                  '              AAAA              ',
                  '              AAAA              ',
                  '               AA               ',
                  '             GGBBGG             ',
                  '            GBBCCBBG            ',
                  '           GBBBCCBBBG           ',
                  '          GBBBBBBBBBBG          ',
                  '          GBBBBBBBBBBG          ',
                  ' G       GBBBBBBBBBBBBG       G ',
                  'GCG     GGDBBBBBBBBBBDGG     GCG',
                  'GCG   GGBBBDBBB  BBBDBBBGG   GCG',
                  'GCBGGGBBBBBDBB    BBDBBBBBGGGBCG',
                  'GCBBBBBBBBBBDB    BDBBBBBBBBBBCG',
                  'BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB',
                  'GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ',
                  '  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ',
                  '    GGBBDDDDDGGGGGDDDDDDBBG     ',
                  '      GDDDDGGG    GGGDDDDG      ',
                  '       DDDD          DDDD       '  );

const GleiterRunter :array [1..20] of string[32] =
                ( '       DDDD          DDDD       ',
                  '      GDDDDGGG    GGGDDDDG      ',
                  '    GGBBDDDDDGGGGGDDDDDDBBG     ',
                  '  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ',
                  'GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ',
                  'BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB',
                  'GCBBBBBBBBBBDB    BDBBBBBBBBBBCG',
                  'GCBGGGBBBBBDBB    BBDBBBBBGGGBCG',
                  'GCG   GGBBBDBBB  BBBDBBBGG   GCG',
                  'GCG     GGDBBBBBBBBBBDGG     GCG',
                  ' G       GBBBBBBBBBBBBG       G ',
                  '          GBBBBBBBBBBG          ',
                  '          GBBBBBBBBBBG          ',
                  '           GBBBCCBBBG           ',
                  '            GBBCCBBG            ',
                  '             GGBBGG             ',
                  '               AA               ',
                  '              AAAA              ',
                  '              AAAA              ',
                  '               AA               ' );

  SPRANZ = 6;                                     { Anzahl der Sprites }
  BREITE = 42;               { Breite der Copyright-Meldung in Zeichen }
  HOEHE  = 6;                             { Hhe der Meldung in Zeilen }
  SX     = (MAXX-(BREITE*8)) div 2;                 { Start-Ordinate X }
  SY     = (MAXY-(HOEHE*8)) div 2;                  { Start-Ordinate Y }

type SPRITE = record                      { zur Verwaltung der Sprites }
                spidp : SPIP;                   { Zeiger auf Sprite-Id }
                deltax,              { X-Verschiebung fr Page 0 und 1 }
                deltay : array [0..1] of integer;     { Y-Verschiebung }
              end;

var sprites    : array [1..spranz] of SPRITE;
    page,                                  { aktuell bearbeitete Seite }
    lc,                             { Zeichen fr den Bildschirmaufbau }
    out        : byte;           { nimmt Flags fr Seitenkollision auf }
    x, y, i,                                         { Schleifenzhler }
    dx, dy     : integer;                             { Bewegungswerte }
    gleiterupp,
    gleiterdnp : SPLP;                        { Zeiger auf die Sprites }
    ch         : char;

begin
  Randomize;                         { Zufallsgenerator initialisieren }

  {-- Vorlagen fr die verschiedenen Sprites aufbauen -----------------}

  gleiterupp := CompileSprite( GleiterRauf,   20 );
  gleiterdnp := CompileSprite( GleiterRunter, 20 );

  {-- die ersten beiden Grafikseiten mit Buchstaben fllen ------------}

  for page := 0 to 1 do
    begin
      setpage( page );
      showpage( page );
      lc := 0;
      y := 0;
      while ( y < (MAXY+1)-8 ) do
        begin
          x := 0;
          while ( x < (MAXX+1)-8 ) do
            begin
              PrintChar( chr(lc and 127), x, y, lc and 15, 0 );
              inc( lc );
              inc( x, 8 );
            end;
          inc( y, 12 );
        end;

      {-- Copyright-Meldung ausgeben ----------------------------------}

      Line( SX-1, SY-1, SX+BREITE*8, SY-1, 15 );
      Line( SX+BREITE*8, SY-1, SX+BREITE*8, SY+HOEHE*8,15 );
      Line( SX+BREITE*8, SY+HOEHE*8, SX-1, SY+HOEHE*8, 15 );
      Line( SX-1, SY+HOEHE*8, SX-1, SY-1, 15 );
      PrintString( SX, SY,    15, 4,
                   '                                          '  );
      PrintString( SX, SY+8,  15, 4,
                   ' S6435P.PAS - (c) 1990 by Michael Tischer ' );
      PrintString( SX, SY+16, 15, 4,
                   '                                          '  );
      PrintString( SX, SY+24, 15, 4,
                   '    Sprite-Demo fuer den 640*350 Punkte   '  );
      PrintString( SX, SY+32, 15, 4,
                   '    Grafikmodus von EGA- und VGA-Karten   '  );
      PrintString( SX, SY+40, 15, 4,
                   '                                          '  );
    end;

  {-- die verschiedenen Sprites erzeugen ------------------------------}

  for i := 1 to SPRANZ do
    begin
      sprites[ i ].spidp := CreateSprite( gleiterupp );
      repeat                       { Bewegungswerte der Sprites whlen }
        dx := 0;
        dy := random(10) - 5;
      until ( dx <> 0 ) or ( dy <> 0 );

      sprites[ i ].deltax[0] := dx * 2;
      sprites[ i ].deltay[0] := dy * 2;
      sprites[ i ].deltax[1] := dx * 2;
      sprites[ i ].deltay[1] := dy * 2;

      x := ( ((MAXX+1) div SPRANZ) * (i-1) )
           + (((MAXX+1) div SPRANZ)-40) div 2;
      y := random( (MAXY+1) - 40 );
      SetSprite( sprites[ i ].spidp, x, y, x - dx, y - dy );
    end;

  {-- die Sprites bewegen und an den Seitenrndern reflektieren -------}

  page := 1;                                      { mit Page 1 starten }
  while ( not keypressed ) do   { ein Tastendruck beendet die Schleife }
    begin
      showpage( 1-page );                  { die andere Seite anzeigen }
      { ch := readkey; }                            { auf Taste warten }
      for i := 1 to spranz do                { die Sprites durchlaufen }
        begin          { Sprite bewegen und Kontakt mit Seite abfragen }
          out := MoveSprite( sprites[i].spidp, page,
                             sprites[i].deltax[page],
                             sprites[i].deltay[page] );
          if ( ( out and OUT_TOP ) <> 0 ) or       {Kontakt oben/unten?}
             ( ( out and OUT_BOTTOM ) <> 0 ) then
            begin
              {-- Ja, Bewegungsrichtung umdrehen und neues Erschei- ---}
              {-- nungsbild                                         ---}

              sprites[i].deltay[page] := -sprites[i].deltay[page];
              if ( ( out and OUT_TOP ) <> 0 ) then
                sprites[i].spidp^.splookp := gleiterdnp
              else
                sprites[i].spidp^.splookp := gleiterupp;
            end;
          if ( ( out and OUT_LEFT ) <> 0 ) or    {Kontakt links/rechts?}
             ( ( out and OUT_RIGHT ) <> 0 ) then
            sprites[i].deltax[page] := -sprites[i].deltax[page];
        end;
      page := (page+1) and 1;   { von 1 auf 0 und umgekehrt umschalten }
    end;
  ch := readkey;                                    { auf Taste warten }
end;

{----------------------------------------------------------------------}
{-- H A U P T P R O G R A M M                                      ----}
{----------------------------------------------------------------------}

begin
 if ( IsEgaVga <> WEDERNOCH ) then  { EGA- oder VGA-Karte installiert? }
   begin                                               { Ja, los gehts }
     init640350;                          { Grafikmodus initialisieren }
     Demo;
     Textmode( CO80 );              { wieder in den Textmodus schalten }
   end
 else
   writeln( 'S6435P.PAS - (c) 1990 by MICHAEL TISCHER'#13#10#10 +
            'Achtung! Fr die Ausfhrung dieses Programms wird eine ' +
            'EGA- oder VGA-Karte bentigt.'#13#10 );
end.
