;**********************************************************************;
;*                         S 6 4 3 5 P A . A S M                      *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : enthlt die Routinen zur Arbeit mit Sprites im *;
;*                     640*350 Punkte Grafikmodus der EGA- und VGA-   *;
;*                     Karte                                          *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  :  8.12.1990                                     *;
;*    letztes Update : 14.01.1991                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM /mx S6435PA;    oder   TASM -mx S6435PA   *;
;*                     ... dann mit S6435P.PAS linken                 *;
;**********************************************************************;

;== Konstanten =========================================================

SC_INDEX       = 3c4h             ;Indexregister des Sequencer-Ctrl.
SC_MAP_MASK    = 2                ;Nummer des Map-Mask-Registers
SC_MEM_MODE    = 4                ;Nummer des Memory-Mode-Registers

GC_INDEX       = 3ceh             ;Indexregister des Graphics-Ctrl.
GC_READ_MAP    = 4                ;Nummer des Read-Map-Registers
GC_BIT_MASK    = 8                ;Nummer des Bit-Mask-Registers

PIXX           = 640              ;horizontale Auflsung

;== Datensegment ===========================================================

DATA   segment word public
DATA   ends

;== Programm ===============================================================

CODE       segment byte public    ;das Programmsegment

	   assume cs:code, ds:data

;-- Public-Deklarationen -----------------------------------------------

public     copybuf2video
public     mergeandcopybuf2video
public     copyvideo2buf

;-----------------------------------------------------------------------
;-- CopyBuf2Video: kopiert den Inhalt eines rechteckigen Bereichs aus
;--                dem Video-RAM, der zuvor mit Hilfe von CopyVideo2Buf
;--                gesichert wurde wieder in den Video-RAM
;-- Aufruf von TP: CopyBuf2Plane( bufptr   : pointer;
;--                               nachpage : byte;
;--                               nachx,
;--                               nachy    : integer;
;--                               breite,
;--                               hoehe    : byte );
;-- Info         : siehe CopyVideo2Buf

copybuf2video proc near

sfr0    struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
stofs0     dw ?                   ;lok. Var.: Startoffset in Video-RAM
ret_adr0   dw ?                   ;Rcksprungadresse zum Aufrufer
hoehe0     dw ?                   ;Hhe des Bereichs
breite0    dw ?                   ;Breite
nachy      dw ?                   ;nach Y-Ordinate
nachx      dw ?                   ;nach X-Ordinate
nachpage   dw ?                   ;nach Seite
bufptr0    dd ?                   ;Zeiger auf Puffer
sfr0    ends                   ;Ende der Struktur

fr         equ [ bp - bp0 ]          ;adr. die Elemente der Struktur
bfr        equ byte ptr [ bp - bp0 ] ;adr. Stack-Element als Byte

           sub   sp,2             ;Platz fr lokale Variablen

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  ds

           cld                    ;bei Stringbefehlen aufwrts

	   ;-- Segmentadresse fr Zugriff auf Video-RAM berechnen ------

	   mov   ax,0A000h        ;ES auf Anfang NACH-Seite
	   cmp   bfr.nachpage,0   ;ist es Seite 0?
	   je    cv0              ;Ja, AL schon o.k

	   mov   ax,0A6D6h        ;Nein, Seite 1 bei A6D6h

cv0:       mov   es,ax

	   ;-- Offset fr Zielposition in Page berechnen ---------------

	   mov   ax,PIXX / 8      ;AX auf Zielposition VON
	   mul   fr.nachy
	   mov   bx,fr.nachx
	   shr   bx,1
	   shr   bx,1
           shr   bx,1
	   add   bx,ax
           mov   fr.stofs0,bx     ;Startoffset in lokaler Var. merken

	   lds   si,fr.bufptr0    ;DS:SI auf den Puffer setzen

	   ;-- Zhler fr die Kopierschleife laden ---------------------

	   mov   dl,bfr.breite0   ;DL = Bytes
	   mov   bx,PIXX / 8      ;BX als Offset zur nchsten Zeile
	   sub   bl,dl
	   xor   ch,ch            ;Hi-Byte des Counters ist immer 0

	   ;-- die zu adressierende Bit-Plane einstellen ---------------

	   mov   ah,1             ;Plane-Nummer als Bit-Maske laden
	   mov   al,SC_MAP_MASK   ;die Registernummer nach AL

cv1:	   mov   dx,SC_INDEX      ;Zugriff auf die zu bearbeitende
	   out   dx,ax            ;Plane erschlieen

	   ;-- Kopierroutine fr eine Bit-Plane, ohne Bercksichti- ----
	   ;-- gung des Hintergrundes

           mov   di,fr.stofs0     ;DI auf Startoffset
	   mov   dh,bfr.hoehe0    ;DH = Zeilen
	   mov   dl,bfr.breite0   ;DL = Bytes

cv2:       mov   cl,dl            ;Anzahl Bytes nach CL

	   rep movsb              ;Zeile kopieren
	   add   di,bx            ;DI in nchste Zeile
	   dec   dh               ;noch eine Zeile?
	   jne   cv2              ;Ja ---> weiter

           shl   ah,1             ;auf die nchste Plane umschalten
           test  ah,16            ;alle Planes bearbeitet?
           je    cv1              ;Nein, mit nchster Plane fortfahren

	   mov   ax,(0Fh shl 8)+ SC_MAP_MASK   ;Zugriff auf alle Bit-
           mov   dx,SC_INDEX                   ;Planes erlauben
	   out   dx,ax

           pop   ds               ;DS und BP zurckholen
	   pop   bp

           add   sp,2             ;lokale Variablen entfernen
           ret   14               ;zurck zum Aufrufer, dabei Parameter
                                  ;vom Stack entfernen
copybuf2video endp

;-----------------------------------------------------------------------
;-- MergeAndCopyBuf2Video: verknpft den Inhalt eines Hintergrundpuffers
;--                        mit dem Inhalt eines Sprite-Puffers ber eine
;--                        Bit-Maske und kopiert das Resultat in den
;--                        Video-RAM
;-- Aufruf von TP: MergeAndCopyBuf2Video( spribufptr,
;--                                       hgbufptr,
;--                                       andbufptr  : pointer;
;--                                       page       : byte;
;--                                       nachx,
;--                                       nachy      : integer;
;--                                       breite,
;--                                       hoehe      : byte );
;-- Info         : siehe CopyVideo2Buf

mergeandcopybuf2video proc near

sfr2    struc                  ;Struktur zum Zugriff auf den Stack
bp2        dw ?                   ;nimmt BP auf
andptr2    dd ?                   ;lok. Var.: Zeiger in AND-Puffer
stofs2     dw ?                   ;lok. Var.: Startoffset in Video-RAM
ret_adr2   dw ?                   ;Rcksprungadresse zum Aufrufer
hoehe2     dw ?                   ;Hhe des Bereichs
breite2    dw ?                   ;Breite
nachy2     dw ?                   ;nach Y-Ordinate
nachx2     dw ?                   ;nach X-Ordinate
nachpage2  dw ?                   ;nach Seite
andbufptr  dd ?                   ;Zeiger auf AND-Puffer
hgbufptr   dd ?                   ;Zeiger auf Hintergrundpuffer
spribufptr dd ?                   ;Zeiger auf den Spritepuffer
sfr2    ends                   ;Ende der Struktur

fr         equ [ bp - bp2 ]          ;adr. die Elemente der Struktur
bfr        equ byte ptr [ bp - bp2 ] ;adr. Stack-Element als Byte

           sub   sp,6             ;Platz fr lokale Variablen

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  ds

           cld                    ;bei Stringbefehlen aufwrts

	   ;-- Segmentadresse fr Zugriff auf Video-RAM berechnen ------

	   mov   ax,0A000h        ;ES auf Anfang NACH-Seite
	   cmp   bfr.nachpage2,0  ;ist es Seite 0?
	   je    cm0              ;Ja, AL schon o.k

	   mov   ax,0A6D6h        ;Nein, Seite 1 bei A6D6h

cm0:       mov   es,ax

	   ;-- Offset fr Zielposition in Page berechnen ---------------

	   mov   ax,PIXX / 8      ;AX auf Zielposition VON
	   mul   fr.nachy2
	   mov   bx,fr.nachx2
	   shr   bx,1
	   shr   bx,1
           shr   bx,1
	   add   bx,ax
           mov   fr.stofs2,bx     ;Startoffset in lokaler Var. merken

	   ;-- Zhler fr die Kopierschleife laden ---------------------

	   mov   dl,bfr.breite2   ;DL = Bytes
	   mov   bx,PIXX / 8      ;BX als Offset zur nchsten Zeile
	   sub   bl,dl
	   xor   ch,ch            ;Hi-Byte des Counters ist immer 0

           mov   ax,word ptr fr.andbufptr+2 ;Segmentadr. des AND-Ptr
           mov   word ptr fr.andptr2+2,ax   ;in lok. Var. kopieren

	   ;-- die zu adressierende Bit-Plane einstellen ---------------

	   mov   ah,1             ;Plane-Nummer als Bit-Maske laden
	   
cm1:       mov   al,SC_MAP_MASK   ;die Registernummer nach AL
           mov   dx,SC_INDEX      ;Zugriff auf die zu bearbeitende
	   out   dx,ax            ;Plane erschlieen

	   ;-- Kopierroutine fr eine Bit-Plane, ohne Bercksichti- ----
	   ;-- gung des Hintergrundes

           mov   dx,word ptr fr.andbufptr   ;Offsettadr. des AND-Ptr
           mov   word ptr fr.andptr2,dx     ;in lok. Var. kopieren
           mov   di,fr.stofs2     ;DI auf Startoffset
	   mov   dh,bfr.hoehe2    ;DH = Zeilen
	   mov   dl,bfr.breite2   ;DL = Bytes

cm2:       mov   cl,dl            ;Anzahl Bytes nach CL

cm3:       lds   si,fr.hgbufptr   ;Zeiger auf Hintrgr.-Puffer laden
           lodsb                  ;Byte aus Hintergrund-Puffer laden
           mov   word ptr fr.hgbufptr,si ;inkr. Offset abspeichern

           lds   si,fr.andptr2    ;Zeiger in AND-Puffer laden
           and   al,[si]          ;Hintergrund mit AND-Maske verkn.
           inc   si               ;Offset in AND-Puffer inkrementieren
           mov   word ptr fr.andptr2,si  ;und speichern

           lds   si,fr.spribufptr ;Zeiger auf Sprite-Puffer laden
           or    al,[si]          ;Byte aus Sprite-Puffer ORen
           inc   si               ;Offset in Sprite-Puffer inkrementieren
           mov   word ptr fr.spribufptr,si  ;und speichern

           stosb                  ;Byte in Video-RAM schreiben
           loop  cm3              ;nchstes Byte bearbeiten

	   add   di,bx            ;DI in nchste Zeile
	   dec   dh               ;noch eine Zeile?
	   jne   cm2              ;Ja ---> weiter

           shl   ah,1             ;auf die nchste Plane umschalten
           test  ah,16            ;alle Planes bearbeitet?
           je    cm1              ;Nein, mit nchster Plane fortfahren

	   mov   ax,(0Fh shl 8)+ SC_MAP_MASK   ;Zugriff auf alle Bit-
           mov   dx,SC_INDEX                   ;Planes erlauben
	   out   dx,ax

           pop   ds               ;DS und BP zurckholen
	   pop   bp

           add   sp,6             ;lokale Variablen entfernen
           ret   22               ;zurck zum Aufrufer, dabei Parameter
                                  ;vom Stack entfernen
mergeandcopybuf2video endp

;-----------------------------------------------------------------------
;-- CopyVideo2Buf: Kopiert ein rechtecikgen Teil aus dem Video-RAM in
;--                einen Puffer
;-- Aufruf von TP: CopyVideo2Buf( bufptr : pointer;
;--                               vonpage: byte;
;--                               vonx,
;--                               vony    : integer;
;--                               breite,
;--                               hoehe   : byte );
;-- Info         : In dieser Version der Routine mu der zu kopierende
;--                Bereich an einer durch acht teilbaren Punktspalte
;--                beginnen und sich ber ein vielfaches von acht Punk-
;--                ten erstrecken.
;--                BREITE ist hier die Anzahl der Bytes pro Zeile
;--                einer Bit-Plane!

copyvideo2buf proc near

sfr1    struc                  ;Struktur zum Zugriff auf den Stack
bp1        dw ?                   ;nimmt BP auf
stofs1     dw ?                   ;lok. Var.: Startoffset in Video-RAM
ret_adr1   dw ?                   ;Rcksprungadresse zum Aufrufer
hoehe1     dw ?                   ;Hhe des Bereichs in Punktzeilen
breite1    dw ?                   ;Breite des Bereichs in Punkten
vony       dw ?                   ;von Y-Ordinate
vonx       dw ?                   ;von X-Ordinate
vonpage    dw ?                   ;von Seite
bufptr1    dd ?                   ;Zeiger auf Puffer
sfr1    ends                   ;Ende der Struktur

fr         equ [ bp - bp1 ]          ;adr. die Elemente der Struktur
bfr        equ byte ptr [ bp - bp1 ] ;adr. Stack-Element als Byte

           sub   sp,2             ;Platz fr lokale Variablen

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  ds
           cld                    ;bei Stringbefehlen aufwrts

	   ;-- Segmentadresse fr Zugriff auf Video-RAM berechnen ------

	   mov   ax,0A000h        ;ES auf Anfang VON-Seite
	   cmp   bfr.vonpage,0    ;ist es Seite 0?
	   je    cc0              ;Ja, AL schon o.k

	   mov   ax,0A6D6h        ;Nein, Seite 1 bei A6D6h

cc0:       mov   ds,ax

	   ;-- Offset in die auszulesende Seite bilden -----------------

	   mov   ax,PIXX / 8      ;AX auf Zielposition VON
	   mul   fr.vony
	   mov   bx,fr.vonx
	   shr   bx,1
	   shr   bx,1
           shr   bx,1
	   add   bx,ax
           mov   fr.stofs1,bx     ;Startoffset in lokaler Var. merken

	   les   di,fr.bufptr1    ;ES:DI auf den Puffer setzen

	   ;-- Zhler fr die Kopierschleife laden ---------------------

	   mov   dl,bfr.breite1   ;DL = Bytes
	   mov   bx,PIXX / 8      ;BX als Offset zur nchsten Zeile
	   sub   bl,dl
	   xor   ch,ch            ;Hi-Byte des Counters ist immer 0

	   ;-- die zu adressierende Bit-Plane einstellen ---------------

           xor   ah,ah            ;mit Plane #0 beginnen
           mov   al,GC_READ_MAP   ;nach AL die Registernummer

cc1:       mov   dx,GC_INDEX      ;Indexadresse des Graphic-Ctrl laden
	   out   dx,ax            ;Read-Map-Register laden

	   ;-- Kopierroutine fr eine Bit-Plane, ohne Bercksichti- ----
	   ;-- gung des Hintergrundes

	   mov   dh,bfr.hoehe1    ;DH = Zeilen
	   mov   dl,bfr.breite1   ;DL = Bytes
           mov   si,fr.stofs1     ;Startoffset nach SI laden

cc2:       mov   cl,dl            ;Anzahl Bytes nach CL
	   rep movsb              ;Zeile kopieren
	   add   si,bx            ;SI in nchste Zeile
	   dec   dh               ;noch eine Zeile?
	   jne   cc2              ;Ja, weiter

           inc   ah               ;auf die nchste Plane umschalten
           cmp   ah,4             ;alle Planes bearbeitet?
           jne   cc1              ;Nein, mit nchster Plane fortfahren

	   pop   ds               ;DS und BP zurckholen
	   pop   bp

           add   sp,2             ;lokale Variablen entfernen
           ret   14               ;zurck zum Aufrufer, dabei Parameter
                                  ;vom Stack entfernen
copyvideo2buf endp

;== Ende ===============================================================

CODE       ends                   ;Ende des Codesegments
	   end                    ;Ende des Programms
