{********************************************************************}
{                       S E R I R Q . P A S                          }
{*------------------------------------------------------------------*}
{ Aufgabe        : Demonstriert mittels Chat-Programm die            }
{                  Programierung des seriellen Schnittstellen IRQ's  }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 08.04.1994                                        }
{ letztes Update : 10.10.1994                                        }
{********************************************************************}
{$X+} { Funktionsergebnisse optional }

uses CRT,ARGS,WIN,SERUTIL,IRQUTIL;

{- Globale Variablen ------------------------------------------------}
var
 Screen,
 Remote,
 Local,
 Status   : WINDOW;

 iSerPort,
 iSerIRQ,
 iUART,
 iCom     : Integer;
 c        : Array[0..1] of Char;
 lBaud    : Longint;
 lpOldIRQ : Pointer;
 pSaved   : ^Byte;                           { geretteter Bildschirm }
 s        : String;

{********************************************************************}
{ GetSer : Interruptroutine, ausgelst durch serielle Schnittstelle  }
{*------------------------------------------------------------------*}
{ Info : In dieser Prozedur werden Daten von der seriellen Schnitt-  }
{        stelle entgegengenommen und im "Remote"-Fenster dar-        }
{        gestellt. Diese Prozedur enthlt auerdem das Gerst fr    }
{        alle weiteren Schnittstellenereignisse, die einen IRQ aus-  }
{        lsen.                                                      }
{********************************************************************}
Procedure GetSer; Interrupt;

var bIRQID : Byte;
    c      : String;
    i      : Integer;

Begin
  bIRQID := port[iSerPort + SER_IRQ_ID]; {Schnittstellen-Status holen}
  if ( bIRQID and SER_ID_PENDING ) = 0 then         { steht IRQ an ? }
    Begin { Ja }
      case ( bIRQID and SER_ID_MASK ) of
        SER_ID_RECEIVED:                           { Nach Datenempfamg }
          Begin
            i := 0;
            c := '';
            while ( ser_IsDataAvaiable( iSerPort ) and ( i < 16 ) ) do
              Begin
                c := c + char ( port[iSerPort + SER_RXBUFFER]);
                Inc( i );
              End;
            win_Print( Remote, c );
          End;

        SER_ID_SENT:                               { Nach Datensendung }
          Begin
          End;

        SER_ID_MODEMSTATUS:       { Nach nderung der Leitungszustnde }
          Begin
          End;
      End;
    End;
  irq_SendEOI( iSerIRQ );                        { End Of Interrupt }
End;

{********************************************************************}
{ Hz : Wandelt eine Hex-Zahl in einen Hex-String um                  }
{*------------------------------------------------------------------*}
{ Eingabe : HZ - die Hex-Zahl                                        }
{ Ausgabe : der umgewandelte Hex-String                              }
{********************************************************************}
Function Hex( hz : Word ) : String;

var i  : Integer;
    hs : String;
begin
  hs := '';
  Repeat
    i := hz mod 16;
    hz := hz div 16;
    case i of
      0..9  : hs := chr( i + ord( '0' ) ) + hs;
      10..15: hs := chr( i - 10 + ord( 'A' ) ) + hs;
    end;
  Until hz = 0;
  if Length( hs ) mod 2 <> 0 then
    hs := '0' + hs;
  hex := hs;
End;

{********************************************************************}
{ H A U P T P R O G R A M M                                          }
{********************************************************************}
Begin
  if GetArg( '?', _none, Nil, 1 ) <> 0 then
    Begin
      Writeln( 'Aufruf:' );
      Writeln( 'SERIRQ [-COM:comport] [-BAUD:baudrate]' );
      Writeln( 'comport  = 1 oder 2    (Default: 1 )' );
      Writeln( 'baudrate = 50 - 115200 (Default: 9600)' );
      Halt(0);
    End;

  {-- Kommandozeile auswerten ------------------}
  if GetArg( '-COM:', _int, @iCom, 1 ) <> 0 then
    Begin
      if iCom = 1 then
        Begin                  { Nur COM1 und COM2 sind "standardisiert" }
          iSerPort := SER_COM1;
          iSerIRQ  := SER_IRQ_COM1;
        End
      else
        if iCom = 2 then
          Begin                  { Nur COM1 und COM2 sind "standardisiert" }
            iSerPort := SER_COM2;
            iSerIRQ  := SER_IRQ_COM2;
          End
        else
          Begin
            Writeln( 'Nicht untersttzter Com-Port!' );
            Halt(0);
          End;
    End
  else
    Begin                    { Nur COM1 und COM2 sind "standardisiert" }
      iSerPort := SER_COM1;
      iSerIRQ  := SER_IRQ_COM1;
    End;

  if GetArg( '-BAUD:', _long, @lBaud, 1 ) = 0 then
    lBaud := 9600;                { Maximale Baudrate bei UART 8450A }

  if lBaud > SER_MAXBAUD then
    Begin
      Writeln( 'Baudrate zu hoch!' );
      Writeln( 'Maximum: ',SER_MAXBAUD,' Bd' );
    End;

  {-- Schnittstelle mit den ermittelten Parametern initialisieren --}
  iUART := ser_Init( iSerPort, lBaud, SER_LCR_8BITS or
                                      SER_LCR_1STOPBIT or
                                      SER_LCR_NOPARITY );

  if iUART = NOSER then
    Begin
      Writeln('Keine Schnittstelle!');
      Halt( 0 );
    End;

  if iUART > INS8250 then { bei 14450 FIFO-Puffer anschalten }
    ser_FIFOLevel( iSerPort, SER_FIFO_TRIGGER14 ); 

  win_GetScreenSettings( Screen );               { Bildschirm retten }
  pSaved := win_Save( Screen );
                                        { Ausgabe Fenster einrichten }
  win_Init( Remote, 0, 0, 80, 9, $17, $1f, WIN_SCROLL or WIN_CRLF );
  win_Init( Local,  0,10, 80, 9, $17, $1f, WIN_SCROLL or WIN_CRLF or
                                           WIN_ACTIVE or WIN_HASCURSOR );
  win_Init( Status,  0,20, 80, 2, $17, $1f, WIN_SCROLL or WIN_CRLF );

  win_Clr( Screen );
  win_Clr( Remote );
  win_Clr( Local );
  win_Clr( Status );
  str(lBaud,s);
  win_print( Status, 'COM-Port: $' + Hex( iSerPort ) +
                     ', IRQ: $' + Hex( iSerIRQ ) +
                     ', Baud: '+s+#10 );
  win_print( Status, '<ESC> = Ende        ' );
  win_print( Status, 'Aufruf mit "?" zeigt Optionen    ' );
  win_print( Status, '(c) BHJ, MITI' );

  {-- seriellen Interrupt-Handler installieren. --}
  lpOldIRQ := ser_SetIRQHandler( iSerPort,
                                 iSerIRQ,      
                                 @GetSer,
                                 SER_IER_RECEIVED or SER_IER_SENT );

  {-- Chat-Modus in Gang setzen ----}
  c[0] := #0;
  c[1] := #0;

  win_GotoXY( Local, 0, 0);
  While c[0] <> #27 do
  Begin
    c[0] := ReadKey;
    if c[0] <> #0 then
      Begin       { eigegebenes Zeichen ber Schnittstelle ausgeben... }
        ser_WriteByte( iSerPort, Byte ( c[0] ), $8000, 0, 0 );
        if ( c[0] > #31 ) and ( c[0] < #128 ) then
          win_Print( Local, c[0] ) { ...und in Fenster protokollieren. }
        else
          win_print( Local, '<' + Hex ( Byte ( c[0] ) ) + '>' );
      End
    else
      ReadKey;
  End;
  {--- Alten Interrupt-Handler und Bildschirminhalt restaurieren ----}
  ser_RestoreIRQHandler( iSerPort, iSerIRQ, lpOldIRQ );
  win_Restore( pSaved, TRUE );       

  if iUART > INS8250 then ser_FIFOLevel( iSerPort, 0 ); {FIFO zurcks.}
End.
