{**********************************************************************}
{*                               T A S T P                            *}
{*--------------------------------------------------------------------*}
{*    Aufgabe : stellt eine Funktion zur Verfgung, mit deren Hilfe   *}
{*              ein Zeichen von der Tastatur eingelesen werden kann,  *}
{*              wobei der Status der Umschaltmodi INSERT, CAPS und    *}
{*              NUM auf dem Bildschirm angezeigt wird.                *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  :  8.07.1987                                     *}
{*    letztes Update :  1.01.1992                                     *}
{**********************************************************************}
                                                                        
program TASTP;
                                                                        
Uses Crt, Dos;                   { die CRT- und die DOS-Unit einbinden }
                                                                        
{$V-}                      { die Lngenprfung von Strings unterbleibt }
                                                                        
type FlagText = string[6];         { dient der bergabe des Flag-Namen }
                                                                        
                                                                        
const FZ      = 1;         { Zeile, in der die Flags ausgegeben werden }
      FS      = 65;       { Spalte. ab der die Flags ausgegeben werden }
      FlagVdg = 0;                        { Vordergrundfarbe der Flags }
      FlagHtg = 7;                        { Hintergrundfarbe der Flags }
                                                                        
                                                                        
      {** Bits des BIOS-Tastatur-Status *******************************}
      SCRL =  16;                                    { Scroll-Lock-Bit }
      NUML =  32;                                       { Num-Lock-Bit }
      CAPL =  64;                                      { Caps-Lock-Bit }
      INS  = 128;                                         { Insert-Bit }
                                                                        
      {** Codes einiger Tasten, wie sie GETKEY liefert ****************}
      BEL     = 7;                          { Code des Klingelzeichens }
      BS      = 8;                          { Code der Backspace-Taste }
      TAB     = 9;                          { Code der Tabulator-Taste }
      LF      = 10;                         { Code des Linefeed        }
      CR      = 13;                         { Code der Return-Taste    }
      ESC     = 27;                         { Code der Escape-Taste    }
      F1      = 315;                        { Funktionstasten          }
      F2      = 316;                        
      F3      = 317;                        
      F4      = 318;                        
      F5      = 319;                        
      F6      = 320;                        
      F7      = 321;                        
      F8      = 322;                        
      F9      = 323;                        
      F10     = 324;                        
      CUP     = 328;                        { Cursor-Tasten            }
      CLEFT   = 331;                        
      CRIGHT  = 333;                        
      CDOWN   = 328;                        
                                                                        
var Insert,                                  { Status des INSERT-Flags }
    Num,                                        { Status des NUM-Flags }
    Caps     : boolean;                        { Status des CAPS-Flags }
    VdgFarbe,                              { aktuelle Vordergrundfarbe }
    HtgFarbe,                              { aktuelle Hintergrundfarbe }
    Taste    : integer;                  { Code der eingelesenen Taste }
                                                                        
{**********************************************************************}
{* NegFlag: evtl. Flag negieren und zugehrigen Text ausgeben         *}
{* Eingabe : s.u.                                                     *}
{* Ausgabe : der neue Status des Flags (true = an, false = aus)       *}
{**********************************************************************}
                                                                        
function NegFlag(Flag   : boolean;       { der letzte Status des Flags }
                 FlagReg,     { der aktuelle Status des Flag (0 = aus) }
                 Spalte,              { Spalte fr den Namen des Flags }
                 Zeile  : integer;     { Zeile fr den Namen des Flags }
                 Text   : FlagText) : boolean;        { Name des Flags }
                                                                        
var AktZeile,                                         { aktuelle Zeile }
    AktSpalte : integer;                             { aktuelle Spalte }
                                                                        
begin
 if (Flag and (FlagReg = 0)) or            { testen ob sich der Status }
    (not(Flag) and (FlagReg <> 0)) then    { des Flags gendert hat    }
  begin                                                           { JA }
   AktZeile := WhereY;                         { aktuelle Zeile merken }
   AktSpalte := WhereX;                       { aktuelle Spalte merken }
   gotoxy(Spalte, Zeile);         { Cursor auf Position fr Flag-Namen }
   if FlagReg = 0 then                           { ist Flag jetzt aus? }
    begin                                                         { JA }
     NegFlag := false;              { Ergebnis der Funktion : Flag aus }
     TextColor( Black );                { Vordergrundfarbe ist schwarz }
     TextBackground( Black );           { HIntergrungfarbe ist schwarz }
    end
   else
    begin                                          { Flag ist jetzt an }
     NegFlag:=true;                  { Ergebnis der Funktion : Flag an }
     TextColor(FlagVdg);                { Vordergrundfarbe ist FLAGVDG }
     TextBackground(FlagHtg)            { Hintergrundfarbe ist FLAGHTG }
    end;
   write(Text);                              { Name des Flags ausgeben }
   GotoXY(AktSpalte, AktZeile);   { alte Cursporposition wieder setzen }
   TextColor(VdgFarbe);        { letzte Vordergrundfarbe wieder setzen }
   TextBackground(HtgFarbe)    { letzte Hintergrundfarbe wieder setzen }
  end
  else
   NegFlag := Flag         { Status des Flags hat sich nicht verndert }
end;
                                                                        
{**********************************************************************}
{* GetKey: Einlesen eines Zeichens und Ausgabe des Flag-Status        *}
{* Eingabe : keine                                                    *}
{* Ausgabe : Code der empfangenen Taste < 256 : normale Taste         *}
{*                                     >= 256 : erweiterte Taste      *}
{**********************************************************************}
                                                                        
function GetKey : integer;
                                                                        
var Regs        : Registers;  { Prozessorregister fr Interrupt-Aufruf }
    TasteBereit : boolean;         { gibt an, ob schon Taste empfangen }
                                                                        
begin
 TasteBereit := false;                    { noch keine Taste empfangen }
 repeat
  Regs.ah := $2;             { Funktionsnummer fr Tastatursatus lesen }
  intr($16, Regs );                 { BIOS-Tastatur-Interrupt aufrufen }
                                                                        
  {** Flags dem neuen Status anpassen *********************************}
  Insert := NegFlag(Insert, Regs.al and INS, FS+9, FZ, 'INSERT');
  Caps := NegFlag(Caps, Regs.al and CAPL, FS+3, FZ, ' CAPS ');
  Num := NegFlag(Num, Regs.al and NUML, FS, FZ, 'NUM');
  Regs.ah := $1;                 { Funktionsnummer fr Zeichen bereit? }
  intr($16, Regs);                  { BIOS-Tastatur-Interrupt aufrufen }
  if (Regs.flags and FZero = 0) then      { ist das Zero-Flag gesetzt? }
   begin                                     { JA (Taste steht bereit) }
    TasteBereit := true;                             { Taste empfangen }
    Regs.ah := 0;                 { Funktionsnummer fr Taste einlesen }
    intr($16, Regs );               { BIOS-Tastatur-Interrupt aufrufen }
    if (Regs.al = 0)                        { ist es erweiterte Taste? }
     then GetKey := Regs.ah or $100                               { JA }
     else GetKey := Regs.al;                                    { NEIN }
   end;
 until TasteBereit;             { wiederholen bis eine Taste empfangen }
end;
                                                                        
{**********************************************************************}
{* IniKey: Tastatur-Flags initialisieren                              *}
{* Eingabe : keine                                                    *}
{* Ausgabe : keine                                                    *}
{* Info    : die Tastatur-Flags werden gegenber ihrem aktuellen      *}
{*           Status umgedreht. Dadurch wird erreicht, da ihr aktu-   *}
{*           eller Status beim nchsten Aufruf der GETKEY-Funktion    *}
{*           ausgegeben wird.                                         *}
{**********************************************************************}
                                                                        
procedure IniKey;
                                                                        
var Regs        : Registers;  { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ah := $2;            { Funktionsnummer fr Tastatur-Status lesen }
 intr($16, Regs );                  { BIOS-Tastatur-Interrupt aufrufen }
 if (Regs.al and INS <> 0) then Insert := false         { INSERT-Flag  }
                           else Insert := true;         { setzen       }
 if (Regs.al and CAPL <> 0) then Caps := false          { CAPS-Flag    }
                            else Caps := true;          { setzen       }
 if (Regs.al and NUML <> 0) then Num := false           { NUM-Flag     }
                            else Num := true            { setzen       }
end;
                                                                        
{**********************************************************************}
{* Farbe: setzt Vorder- und Hintergrundfarbe fr Bildschirmausgaben   *}
{* Eingabe : s.u.                                                     *}
{* Ausgabe : keine                                                    *}
{* Var.    : die Farbe wird in den globalen Variablen VDGFARBE und    *}
{*           HTGFARBE abgespeichert.                                  *}
{* Info    : diese Prozedur mu zum Setzen der Farbe aufgerufen       *}
{*           werden, damit nach einer Ausgabe des Status der Tastatur-*}
{*           Flags die aktuelle Text-Farbe wieder gesetzt werden kann,*}
{*           da in TURBO keine Funktion zur Abfrage dieser Farbe      *}
{*           existiert                                                *}
{**********************************************************************}
                                                                        
procedure Farbe(Vordergrund, Hintergrund : integer);
                                                                        
begin
 VdgFarbe := Vordergrund;                    { Vordergrundfarbe merken }
 HtgFarbe := Hintergrund;                    { Hintergrundfarbe merken }
 TextColor(Vordergrund);                     { Vordergrundfarbe setzen }
 TextBackground(Hintergrund)                 { Hintergrundfarbe setzen }
end;
                                                                        
{**********************************************************************}
{*                            HAUPTPROGRAMM                           *}
{**********************************************************************}
                                                                        
begin
 IniKey;                               { Tastatur-Flags initialisieren }
 Farbe(7,0);                              { Farbe ist wei auf schwarz }
 ClrScr;                                          { Bildschirm lschen }
 writeln(#13#10'TASTP (c) 1987, 92 by Michael Tischer');
 writeln(#13#10'Sie knnen nun einige Zeichen eingeben und dabei den '+
         'INSERT-, CAPS- oder NUM-');
 writeln('Modus an- oder ausschalten. Der Status der drei Umschaltmodi'+
         ' wird jederzeit in ');
 writeln('der oberen rechten Bildschirmecke angezeigt.');
 writeln('Die Bettigung der <RETURN> oder der <F1>-Taste beendet das'+
         ' Programm...');
 write(#13#10'Ihre Eingabe: ');
 repeat                                              { Eingabeschleife }
  Taste := GetKey;                                    { Taste einlesen }
  if (Taste < 256) then write(chr(Taste))   { ausgeben (falls normale) }
 until (Taste = CR) or (Taste = F1);      { wiederholen bis F1 oder CR }
 writeln;
end.
