{**********************************************************************}
{*                              T S R P                               *}
{*--------------------------------------------------------------------*}
{*    Aufgabe        : erstellt mit Hilfe eines Assembler-Moduls ein  *}
{*                     TSR-Programm.                                  *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  : 18.08.1988                                     *}
{*    letztes Update : 18.11.1991                                     *}
{**********************************************************************}
                                                                        
program TSRP;

uses DOS, CRT;                   { die DOS- und die CRT-Unit einbinden }
                                                                        
{$M 2048, 0, 5120}   { 2KB fr den Stack und maximal 5 KB fr den Heap }
{$L tsrpa}                             { das Assembler-Modul einbinden }
                                                                        
{-- Deklaration der externen Funktion aus dem Assembler-Modul ---------}

procedure TsrInit( PrzPtr   : word;   { Offsetadresse der TSR-Prozedur }
                   ResPara  : word    { Anzahl der zu res. Paragraphen }
                 ) ; external ;                        { der ID-String }
function  TsrIsInst( i2F_fktnr : byte ) : boolean ; external ;
procedure TsrUnInst; external;                  { TSR-Programm reinst. }
procedure TsrSetPtr( Offset : word ); external;
function  TsrCanUnInst : boolean; external;

{$F+}                { jetzt folgen die FAR-Prozeduren und -Funktionen }
procedure TsrCall ; external;
procedure TsrSetHotKey( KeyMask  : word;    { der Hotkey (siehe CONST) }
                        ScCode   : byte                    { Scan-Code }
                      ) ; external;
{$F-}
                                                                        
{-- Konstanten --------------------------------------------------------}

      {-- Scan-Codes der verschiedenen Tasten -------------------------}

const SC_ESC             = $01;      SC_Y               = $2C;
      SC_1               = $02;      SC_X               = $2D;
      SC_2               = $03;      SC_C               = $2E;
      SC_3               = $04;      SC_V               = $2F;
      SC_4               = $05;      SC_B               = $30;
      SC_5               = $06;      SC_N               = $31;
      SC_6               = $07;      SC_M               = $32;
      SC_7               = $08;      SC_KOMMA           = $33;
      SC_8               = $09;      SC_PUNKT           = $34;
      SC_9               = $0A;      SC_STRICH          = $35;
      SC_0               = $0B;      SC_SHIFT_RECHTS    = $36;
      SC_SCHARFES_S      = $0C;      SC_PRINT_SCREEN    = $37;
      SC_APOSTROPH       = $0D;      SC_ALT             = $38;
      SC_BACKSPACE       = $0E;      SC_SPACE           = $39;
      SC_TAB             = $0F;      SC_CAPS            = $3A;
      SC_Q               = $10;      SC_F1              = $3B;
      SC_W               = $11;      SC_F2              = $3C;
      SC_E               = $12;      SC_F3              = $3D;
      SC_R               = $13;      SC_F4              = $3E;
      SC_T               = $14;      SC_F5              = $3F;
      SC_Z               = $15;      SC_F6              = $40;
      SC_U               = $16;      SC_F7              = $41;
      SC_I               = $17;      SC_F8              = $42;
      SC_O               = $18;      SC_F9              = $43;
      SC_P               = $19;      SC_F10             = $44;
      SC_UE              = $1A;      SC_NUM_LOCK        = $45;
      SC_PLUS            = $1B;      SC_SCROLL_LOCK     = $46;
      SC_RETURN          = $1C;      SC_CURSOR_HOME     = $47;
      SC_CONTROL         = $1D;      SC_CURSOR_UP       = $48;
      SC_A               = $1E;      SC_CURSOR_PG_UP    = $49;
      SC_S               = $1F;      SC_NUM_MINUS       = $4A;
      SC_D               = $20;      SC_CURSOR_LEFT     = $4B;
      SC_F               = $21;      SC_NUM_5           = $4C;
      SC_G               = $22;      SC_CURSOR_RIGHT    = $4D;
      SC_H               = $23;      SC_NUM_PLUS        = $4E;
      SC_J               = $24;      SC_CURSOR_END      = $4F;
      SC_K               = $25;      SC_CURSOR_DOWN     = $50;
      SC_L               = $26;      SC_CURSOR_PG_DOWN  = $51;
      SC_OE              = $27;      SC_INSERT          = $52;
      SC_AE              = $28;      SC_DELETE          = $53;
      SC_GROESSER        = $29;      SC_SYS_REQUEST     = $54;
      SC_SHIFT_LINKS     = $2A;      SC_F11             = $57;
      SC_FIS             = $2B;      SC_F12             = $58;
      SC_NOKEY           = $80;              { keine zustzliche Taste }

      {-- Bitmasken fr die verschiedenen Umschalttasten --------------}

      LSHIFT =     1;                              { Linke SHIFT-Taste }
      RSHIFT =     2;                             { rechte SHIFT-Taste }
      CTRL   =     4;                                     { CTRL-Taste }
      ALT    =     8;                                      { ALT-Taste }
      SYSREQ =  1024;                { SYS-REQ-Taste (nur AT-Tastatur) }
      BREAK  =  4096;                                    { BREAK-Taste }
      NUM    =  8192;                                      { NUM-Taste }
      CAPS   = 16384;                                     { CAPS-Taste }
      INSERT = 32768;                                   { INSERT-Taste }
                                                                        
      I2F_CODE   = $C4;                       { Funktionsnummer INT 2F }
      I2F_FKT_0  = $AA;                  { Code fr INT 2F, Funktion 0 }
      I2F_FKT_1  = $BB;                  { Code fr INT 2F, Funktion 1 }

{-- Typdeklarationen --------------------------------------------------}

type VBuf   = array[1..25, 1..80] of word;  { beschreibt den Bilschirm }
     VPtr   = ^VBuf;               { Zeiger auf einen Bildschirmpuffer }

     {-- Deklaration der Prozedur- und Funktionstypen, mit deren Hilfe-}
     {-- Prozeduren und Funktionen in der bereits installierten Kopie -}
     {-- des TSR aufgerufen werden knnen                             -}

     OAProzT  = procedure;                  { Procedure ohne Argumente }
     SHKProzT = procedure( KeyMask : word;              { TsrSetHotkey }
                           ScCode  : byte );
     PPtrT  = record        { Union zur Erstellung der Prozedur-Zeiger }
                case integer of
                  1 : ( OAProz  : OAProzT  );
                  2 : ( SHKProz : SHKProzT );
              end;

const Call : PPtrT = ( OAProz : TsrCall );

{-- globale Variablen -------------------------------------------------}

var MBuf   : VBuf absolute $B000:0000;      { der monochrome Video-RAM }
    CBuf   : Vbuf absolute $B800:0000;           { der color Video-RAM }
    VioPtr : VPtr;                         { Pointer auf den Video-RAM }
    ATimes : integer;                   { Anzahl der TSR-Aktivierungen }

{**********************************************************************}
{* DispInit: erstellt einen Pointer auf den Video-RAM                 *}
{* Eingabe : keine                                                    *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure DispInit;
                                                                        
var Regs: Registers;                 { nimmt die Prozessorregister auf }
                                                                        
begin
  Regs.ah := $0f;         { Funktionsnr. 15 = Auslesen des Video-Modus }
  Intr($10, Regs);                 { den BIOS-Video-Interrupt aufrufen }
  if Regs.al=7 then                           { monochrome Videokarte? }
    VioPtr := @MBuf     { Ja, Pointer auf monochromen Video-RAM setzen }
  else             { es handelt sich um eine EGA-, VGA- oder CGA-Karte }
    VioPtr := @CBuf;              { Pointer auf color Video-RAM setzen }
end;
                                                                        
{**********************************************************************}
{* SaveScreen: sichert den Bildschirminhalt in einem Puffer           *}
{* Eingabe : SPTR : Zeiger auf den Puffer, in dem der Bildschirm-     *}
{*                  inhalt gesichert werden soll.                     *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure SaveScreen( SPtr : VPtr );
                                                                        
var zeile,                             { die aktuell bearbeitete Zeile }
    spalte : byte;                   { die aktuelle bearbeitete Spalte }
                                                                        
begin
  for zeile:=1 to 25 do          { die 25 Bildschirmzeilen durchlaufen }
    for spalte:=1 to 80 do      { die 80 Bildschirmspalten durchlaufen }
      SPtr^[zeile, spalte] := VioPtr^[zeile, spalte];{ Z.&Attr. merken }
end;
                                                                        
{**********************************************************************}
{* RestoreScreen: kopiert den Inhalt eines Puffers in den Video-RAM   *}
{* Eingabe : BPTR : Zeiger auf den Puffer, dessen Inhalt in den       *}
{*                  Video-RAM kopiert werden soll.                    *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure RestoreScreen( BPtr : VPtr );
                                                                        
var zeile,                             { die aktuell bearbeitete Zeile }
    spalte : byte;                   { die aktuelle bearbeitete Spalte }
                                                                        
begin
  for zeile:=1 to 25 do          { die 25 Bildschirmzeilen durchlaufen }
    for spalte:=1 to 80 do      { die 80 Bildschirmspalten durchlaufen }
      VioPtr^[zeile, spalte] := BPtr^[zeile, spalte];{ Z.& Attr. holen }
end;
                                                                        
{**********************************************************************}
{* ResPara: berechnet die Anzahl der Paragraphen, die fr das Pro-    *}
{*          gramm allokiert werden mssen                             *}
{* Eingabe : keine                                                    *}
{* Ausgabe : die Anzahl der zu reservierenden Paragraphen             *}
{**********************************************************************}
                                                                        
function ResPara : word;
                                                                        
begin
  {-- Anzahl der jeweils auszulagernden Bytes berechnen ---------------}

  {$ifdef VER50}                                          { fr TP 5.0 }
    ResPara := Seg(FreePtr^)+$1000-PrefixSeg; { Anzahl der Paragraphen }
  {$endif}

  {$ifdef VER55}                                          { fr TP 5.5 }
    ResPara := Seg(FreePtr^)+$1000-PrefixSeg; { Anzahl der Paragraphen }
  {$endif}

  {$ifdef VER60}                                          { fr TP 6.0 }
    ResPara := Seg(HeapEnd^)-PrefixSeg;
  {$endif}
end;

{**********************************************************************}
{* ParamGetHotKey: Sucht unter den Kommanodzeilenparametern nach Hot- *}
{*                 key-Befehlen (/T) und wertet diese aus             *}
{* Eingabe : KEYMASK = Variable, die die Key-Maske aufnehmen soll     *}
{*           SCCODE  = Variable, die den Scan-Code aufnehmen soll     *}
{* Ausgabe : TRUE, wenn die Hotkeys bekannt sind, sonst FALSE         *}
{* Info    : - Parameter, die nicht mit /T beginnen, lt die Funktion*}
{*             unangetastet, damit sie von einer anderen Routine aus- *}
{*             gewertet werden knnen                                 *}
{*           - Werden keine /T-Parameter entdeckt, wird in der je-    *}
{*             weiligen Variable 0 bzw. SC_NOKEY zurckgeliefert.     *}
{**********************************************************************}
                                                                        
function ParamGetHotKey( var KeyMask : word;
                         var ScCode  : byte ) : boolean;

type UmschaltT = record
                   Name : string[6];
                   Wert : word;
                 end;

const UTasten : array[ 1..9 ] of UmschaltT =
                 ( ( Name: 'LSHIFT'; Wert : LSHIFT ),
                   ( Name: 'RSHIFT'; Wert : RSHIFT ),
                   ( Name: 'CTRL';   Wert : CTRL   ),
                   ( Name: 'ALT';    Wert : ALT    ),
                   ( Name: 'SYSREQ'; Wert : SYSREQ ),
                   ( Name: 'BREAK';  Wert : BREAK  ),
                   ( Name: 'NUM';    Wert : NUM    ),
                   ( Name: 'CAPS';   Wert : CAPS   ),
                   ( Name: 'INSERT'; Wert : INSERT )
                  );

var i , j,                                           { Schleifenzhler }
    code,                               { zur Umwandlung des Scan-Code }
    dummy : integer;                         { Fehler-Variable fr VAL }
    arg   : string;                     { zur Aufnahme eines Arguments }

begin
  KeyMask := 0;
  ScCode := SC_NOKEY;

  for i := 1 to ParamCount do          { die Kommandozeile durchlaufen }
    begin
      arg := ParamStr(i);                             { Argument holen }
      for j := 1 to length(arg) do          { String in Grobuchstaben }
        arg[j] := upcase(arg[j]);
      if ( arg[1] = '/' ) and ( arg[2] = 'T' ) then
        begin                                 { es ist ein /T-Argument }
          delete( arg, 1, 2 );             { /T aus String eleminieren }
          val( arg, code, dummy );            { Rest nach binr wandln }
          if ( dummy = 0 ) then                       { Umwandlung ok? }
            begin                                                 { Ja }
              if ( code > 0 ) and ( code < 128 ) then { gltiger Code? }
                ScCode := Code                            { Ja, merken }
              else
                begin                                { ungltiger Code }
                  ParamGetHotKey := false;
                  exit;                   { Funktion mit FALSE beenden }
                end;
            end
          else         { keine Zahl, mu Name einer Umschalttaste sein }
            begin
              j := 1;               { Umschalttasten-Array durchsuchen }
              while ( j < 10 ) and ( arg <> UTasten[j].Name ) do
                j := j + 1;
              if ( j < 10 ) then                     { Namen gefunden? }
                KeyMask := KeyMask or UTasten[j].Wert  {Ja, Flag einbl.}
              else
                begin                    { Nein, weder Zahl noch Taste }
                  ParamGetHotKey := false;
                  exit;                   { Funktion mit FALSE beenden }
                end;
            end;
        end;
    end;
  ParamGetHotKey := true;        { wenn die Funktion bis hierin kommt, }
end;                             { waren die Parameter o.k.            }

{**********************************************************************}
{* EndePrz: wird bei der Reinstallation des TSR-Programms vom Assem-  *}
{*          bler-Modul aufgerufen                                     *}
{* Eingabe : keine                                                    *}
{* Ausgabe : keine                                                    *}
{* Info    : diese Prozedur mu sich im Hauptprogramm befinden und    *}
{*           darf nicht durch die $F+-Compiler-Direktive in eine      *}
{*           FAR-Prozedur umgewandelt werden                          *}
{**********************************************************************}
                                                                        
{$F+}

procedure EndePrz;
                                                                        
begin
  TextBackground( Black );                      { dunkeler Hintergrund }
  TextColor( LightGray );                              { Helle Schrift }
  writeln('Das TSR-Programm wurde ', ATimes, ' mal aufgerufen.');
end;

{$F-}
                                                                        
{**********************************************************************}
{* Tsr: diese Prozedur wird von dem Assemblermodul nach der Betti-   *}
{*      gung des Hotkey aufgerufen                                    *}
{* Eingabe : keine                                                    *}
{* Ausgabe : keine                                                    *}
{* Info    : diese Prozedur mu sich im Hauptprogramm befinden und    *}
{*           darf nicht durch die $F+-Compiler-Direktive in eine      *}
{*           FAR-Prozedur umgewandelt werden                          *}
{**********************************************************************}
                                                                        
{$F-}                                   { keine FAR-Prozedur erstellen }
                                                                        
procedure Tsr;
                                                                        
var BufPtr : VPtr;       { nimmt Zeiger auf den allokierten Puffer auf }
    Spalte,                            { die aktuelle Bildschirmspalte }
    Zeile  : byte;                      { die aktuelle Bildschirmzeile }
    Taste  : char;
                                                                        
begin
  while KeyPressed do                     { Tastaturpuffer leer machen }
    Taste := ReadKey;
  inc( ATimes );                         { Aufrufzhler inkrementieren }
  DispInit;                          { Adresse des Video-RAM ermitteln }
  GetMem(BufPtr, SizeOf(VBuf) );                   { Puffer allokieren }
  SaveScreen( BufPtr );                 { den Bildschirminhalt sichern }
  Zeile := WhereY;                    { aktuelle Bildschirmzeile holen }
  Spalte := WhereX;                  { aktuelle Bildschirmspalte holen }
  TextBackground( LightGray );                     { helle Hintergrund }
  TextColor( Black );                                 { dunkle Schrift }
  ClrScr;                            { den gesamten Bildschirm lschen }
  GotoXY(22, 12);
  write('TSRP  -  (c) 1988, 91 by MICHAEL TISCHER');
  GotoXY(30, 14);
  write('Bitte eine Taste drcken ...');
  Taste := ReadKey;                            { auf eine Taste warten }
  RestoreScreen( BufPtr ); { den alten Bildschirminhalt zurckkopieren }
  FreeMem( BufPtr, SizeOf(VBuf) );      { allokierten Puffer freigeben }
  GotoXY( Spalte, Zeile );   { Cursor zurck an ursprngliche Position }
end;
                                                                        
{**********************************************************************}
{**                           HAUPTPROGRAMM                          **}
{**********************************************************************}
                                                                        
var KeyMask : word;
    ScCode : byte;

begin
  writeln('TSRP  -  (c) 1988, 91 by MICHAEL TISCHER');
  if not ParamGetHotKey( KeyMask, ScCode ) then
    begin                     { Fehler in den Kommandozeilenparametern }
      writeln( 'Ungltiger Kommandozeilen-Parameter!');
      exit;
    end;

  {-- Kommandozeilen-Parameter waren o.k. -----------------------------}

  if ( TsrIsInst( I2F_CODE ) = FALSE ) then {Programm bereits install.?}
    begin                                                       { Nein }
      ATimes := 0;           { das Programm wurde noch nicht aktiviert }
      writeln( 'Das TSR-Programm wurde installiert. ' );
      if ( KeyMask = 0 ) and ( ScCode = SC_NOKEY ) then  {keine Param.?}
        begin                                { Nein, Vorgabe ist ALT-H }
          TsrSetHotkey( ALT, SC_H );
          writeln( 'Start: <ALT> + H' );
        end
      else                         { Hotkey des Anwenders installieren }
        TsrSetHotkey( KeyMask, ScCode );
      TsrInit( Ofs(Tsr), ResPara );        { das Programm installieren }
    end
  else                              { Programm war bereits installiert }
    begin
      if ( KeyMask = 0 ) and ( ScCode = SC_NOKEY ) then  {keine Param.?}
        begin                         { Nein, Reinstallation versuchen }
          if TsrCanUnInst then
            begin
              TsrSetPtr(ofs(EndePrz));      { Ende-Prozedur in bereits }
              Call.OAProz;                  { install. Kopie aufrufen  }
              TsrUnInst;
              writeln( 'Das Programm wurde erfolgreich reinstalliert.');
            end
          else
            writeln( 'Das Programm kann nicht reinstalliert werden.');
        end
      else                                   { neuen Hotkey bertragen }
        begin
          writeln( 'Neuer Hotkey installiert' );
          TsrSetPtr(ofs(TsrSetHotKey));
          Call.SHKProz( KeyMask, ScCode );
        end
    end;
end.
