;**********************************************************************;
;*                             T Y P M P A                            *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : Assemblerroutine zur Einbindung in ein Turbo-  *;
;*                     Pascal-Programm, mit deren Hilfe die Type-     *;
;*                     matic-Rate der AT-Tastatur eingestellt werden  *;
;*                     kann.                                          *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  : 27.08.1988                                     *;
;*    letztes Update :  1.03.1992                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM TYPMPA;                                   *;
;*                     LINK TYPMPA                                    *;
;*                     EXE2BIN TYPMPA TYPMPA.BIN                      *;
;*                     ... in INLINE-Befehle umwandeln und in Pro-    *;
;*                         gramm TYPMP.PAS einbinden             -    *;
;**********************************************************************;
                                                                        
;== Konstanten =========================================================
                                                                        
KB_STATUS_P   equ 64h             ;Statusport der Tastatur
KB_DATEN_P    equ 60h             ;Datenport der Tastatur
                                                                        
OB_FULL       equ 1               ;Bit 0 im Status-Port der Tastatur
                                  ;ein Zeichen im Ausgabe-Puffer
IB_FULL       equ 2               ;Bit 1 im Status-Port der Tastatur
                                  ;ein Zeichen im Eingabe-Puffer
                                                                        
ACK_SIGNAL    equ 0fah            ;Acknowledge-Signal der Tastatur
SET_TYPEM     equ 0f3h            ;Set-Typematic Code
                                                                        
MAX_TRY       equ 3               ;Anzahl der erlaubten Fehlversuche
                                                                        
                                                                        
;== Programmcode =======================================================
                                                                        
code      segment para 'CODE'     ;Definition des CODE-Segments
                                                                        
          org  100h
                                                                        
          assume cs:code, ds:code, ss:code, es:code
                                                                        
;-----------------------------------------------------------------------
;-- SET_TYPM: bermittelt die zu setzende Typematic-Rate an den  -------
;--           Tastatur-Controller
;-- Info    : hier wird von einem FAR-CALL-Modell ausgegangen

set_typm   proc near
                                                                        
sframe0    struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
ret_adr0   dd ?                   ;Rcksprungadresse zum Aufrufer
                                  ;(FAR-Adresse)
trate0     dw ?                   ;die zu setzende Typematic-Rate
sframe0    ends                   ;Ende der Struktur
                                                                        
frame      equ [ bp - bp0 ]       ;adressiert die Elemente der Struktur
                                                                        
           ;-- die folgenden 2 Befehle werden bereits von TURBO --------
           ;-- ausgefhrt                                       --------
                                                                        
           ;push bp                ;BP auf dem Stack sichern
           ;mov  bp,sp             ;SP nach BP bertragen
                                                                        
           xor  dl,dl             ;von fehlerhafter bertragung ausgehen
           mov  ah,SET_TYPEM      ;Befehlscode fr Typematic-Rate setzen
           cli                    ;keine Interrupts mehr zulassen
           call send_kb           ;an den Controller senden
           jne  error             ;Fehler? JA --> Error
                                                                        
           mov  ah,byte ptr frame.trate0 ;Typematic-Rate holen
           call send_kb           ;an den Controller senden
           jne  error             ;Fehler? JA --> Error
                                                                        
           inc  dl                ;alles o.k., TRUE zurckliefern
                                                                        
error:     sti                    ;Interrupts wieder erlauben
           mov  [bp-1],dl         ;Return-Wert auf den Stack bringen
           ;pop  bp               ;BP wieder vom Stack holen (TURBO)
           jmp  ende              ;zurck zum Pascal-Programm
                                                                        
set_typm   endp
                                                                        
;-----------------------------------------------------------------------
;-- SEND_KB: sendet ein Byte an den Tastatur-Controller ----------------
;-- Eingabe  : AH = das zu sendende Byte
;-- Ausgabe  : Zero-Flag: 0=Fehler, 1=o.k.
;-- Register : AX und das Flag-Register werden verndert
;-- Info     : diese Routine ist nur zum internen Aufruf innerhalb
;--            dieses Moduls bestimmt
                                                                        
send_kb   proc near
                                                                        
          push cx                 ;alle Register, die innerhalb dieser
          push bx                 ;Routine verndert werden, auf dem
                                  ;Stack sichern
                                                                        
          mov  bl,MAX_TRY         ;maximal MAX_TRY-Fehlversuche
                                                                        
          ;-- warten, bis Controller zur Aufnahme von Daten bereit ist -
                                                                        
skb_1:    xor  cx,cx              ;maximal 65536 Schleifendurchlufe
skb_2:    in   al,KB_STATUS_P     ;Inhalt des Statusport einlesen
          test al,IB_FULL         ;noch ein Zeichen im Input-Buffer?
          loopne skb_2            ;JA --> SKB_2
                                                                        
          ;-- Zeichen an den Controller senden -------------------------
                                                                        
          mov  al,ah              ;Zeichen nach AL holen
          out  KB_DATEN_P,al      ;Zeichen an den Datenport senden
skb_3:    in   al,KB_STATUS_P     ;Inhalt des Statusport einlesen
          test al,OB_FULL         ;Anwort im Output-Buffer?
          loope skb_3             ;NEIN --> SKB_3
                                                                        
          ;-- Antwort des Controllers holen und auswerten --------------
                                                                        
          in   al,KB_DATEN_P      ;Anwort vom Datenport einlesen
          cmp  al,ACK_SIGNAL      ;wurde des Zeichen akzeptiert?
          je   skb_end            ;JA --> alles o.k.
                                                                        
          ;-- das Zeichen wurde nicht akzeptiert -----------------------
                                                                        
          dec  bl                 ;Fehlerzhler dekrementieren
          jne  skb_2              ;noch Fehlerversuche frei?
                                  ;JA --> SKB_2
                                                                        
          or   bl,1               ;NEIN, Zero-Flag auf 0 setzen und
                                  ;damit Fehler anzeigen
                                                                        
skb_end:  pop  bx                 ;die gesicherten Register wieder vom
          pop  cx                 ;Stack holen
          ret                     ;zurck zum Aufrufer
                                                                        
send_kb   endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
ende      label near
                                                                        
;== Ende ===============================================================
                                                                        
code      ends                    ;Ende des CODE-Segments
          end  set_typm
                                                                        
