{***********************************************************************
*                          V 3 2 2 0 P . P A S                         *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Grafikprogrammierung im         *
*                     VGA-320*200-Grafikmodus bei 256 Farben und vier  *
*                     Bildschirmseiten. Das Programm greift dabei auf  *
*                     die Assembler-Routinen aus dem Modul V3220PA.ASM *
*                     zurck.                                          *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    :  8.09.1990                                       *
*  letztes Update am: 14.01.1991                                       *
***********************************************************************}

program V3220P;

uses dos, crt;

{-- Typdeklarationen --------------------------------------------------}

type BPTR = ^byte;

{-- externe Referenzen auf die Assembler-Routinen ---------------------}

{$L v3220pa}                               { Assembler-Modul einbinden }

procedure init320200; external;
procedure setpix( x, y : integer; farbe : byte ); external;
function  getpix( x, y: integer ) : byte ; external;
procedure setpage( page : byte ); external;
procedure showpage( page : byte ); external;

{-- Konstanten --------------------------------------------------------}

const MAXX = 319;                         { Maximale X- und Y-Ordinate }
      MAXY = 199;

{***********************************************************************
*  IsVga : Stellt fest, ob eine VGA-Karte installiert ist.             *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : TRUE oder FALSE                                           *
***********************************************************************}

function IsVga : boolean;

var Regs : Registers;          { Prozessorregister fr Interruptaufruf }

begin
  Regs.AX := $1a00;                 { Funktion 1Ah gibt es nur bei VGA }
  Intr( $10, Regs );
  IsVga := ( Regs.AL = $1a );
end;

{***********************************************************************
*  PrintChar : Schreibt ein Zeichen im Grafikmodus in den Bildschirm   *
**--------------------------------------------------------------------**
*  Eingabe :   zeichen = das aufzubauende Zeichen                      *
*              x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************}

procedure PrintChar( zeichen : char; x, y : integer; vg, hg : byte );

type ZDEF = array[0..255,0..7] of byte;       { Aufbau des Zeichensatz }
     ZPTR = ^ZDEF;                            { Zeiger auf Zeichensatz }

var  Regs  : Registers;                 { Register zum Interruptaufruf }
     ch    : char;                      { einzelner Punkt des Zeichens }
     i, k,                                           { Schleifenzhler }
     Maske : byte;                  { Bitmaske fr Aufbau des Zeichens }

const fptr : ZPTR = NIL;               { Zeiger auf Zeichensatz im ROM }

begin
  if fptr = NIL then         { Zeiger auf Zeichensatz schon ermittelt? }
    begin                                                       { Nein }
      Regs.AH := $11;                   { Funktion $11 des Video-BIOS, }
      Regs.AL := $30;                   { Unterfunktion $30 aufrufen   }
      Regs.BH := 3;             { Zeiger auf 8*8 Zeichensatz ermitteln }
      intr( $10, Regs );
      fptr := ptr( Regs.ES, Regs.BP );         { Zeiger zusammensetzen }
    end;

  if ( hg = 255 ) then                 { Zeichen transparent zeichnen? }
    for i := 0 to 7 do             { Ja, nur Vordergrund-Punkte setzen }
      begin
        Maske := fptr^[ord(zeichen),i]; {Bitmuster fr eine Zeile holen}
        for k := 0 to 7 do
          begin
            if ( Maske and 128 <> 0 ) then             { Punkt setzen? }
              setpix( x+k, y+i, vg );                             { Ja }
            Maske := Maske shl 1;
          end;
      end
  else                        { Nein, auch Hintergrund bercksichtigen }
    for i := 0 to 7 do                        { die Zeilen durchlaufen }
      begin
        Maske := fptr^[ord(zeichen),i]; {Bitmuster fr eine Zeile holen}
        for k := 0 to 7 do
          begin
            if ( Maske and 128 <> 0 ) then              { Vordergrund? }
              setpix( x+k, y+i, vg )                              { Ja }
            else
              setpix( x+k, y+i, hg );              { Nein, Hintergrund }
            Maske := Maske shl 1;
         end;
      end;
end;

{***********************************************************************
*  Line: Zieht eine Linie innerhalb nach dem Bresenham-Algorithmus     *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endekoordinate                                   *
*            FARBE  = Farbe der Linienpunkte                           *
***********************************************************************}

procedure Line( x1, y1, x2, y2 : integer; farbe : byte );

var d, dx, dy,
    aincr, bincr,
    xincr, yincr,
    x, y                 : integer;

{-- Unterprozedur, vertauscht zwei Integer-Variablen ------------------}

procedure SwapInt( var i1, i2: integer );

var dummy : integer;

begin
  dummy := i2;
  i2    := i1;
  i1    := dummy;
end;

{-- Hauptprozedur -----------------------------------------------------}

begin
  if ( abs(x2-x1) < abs(y2-y1) ) then   { ber X- oder Y-Achse laufen? }
    begin                                           { ber die Y-Achse }
      if ( y1 > y2 ) then                              { y1 grer y2? }
        begin
          SwapInt( x1, x2 );                   { Ja, X1 mit X2 und     }
          SwapInt( y1, y2 );                   { Y1 mit Y2 vertauschen }
        end;

      if ( x2 > x1 ) then xincr := 1      { Spalteninkrement festlegen }
                     else xincr := -1;

      dy := y2 - y1;
      dx := abs( x2-x1 );
      d  := 2 * dx - dy;
      aincr := 2 * (dx - dy);
      bincr := 2 * dx;
      x := x1;
      y := y1;

      setpix( x, y, farbe );                     { ersten Punkt setzen }
      for y:=y1+1 to y2 do              { Line auf X-Achse durchlaufen }
        begin
          if ( d >= 0 ) then
            begin
              inc( x, xincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          setpix( x, y, farbe );
        end;
    end
  else                                              { ber die X-Achse }
    begin
      if ( x1 > x2 ) then                              { x1 grer x2? }
        begin
          SwapInt( x1, x2 );                   { Ja, X1 mit X2 und     }
          SwapInt( y1, y2 );                   { Y1 mit Y2 vertauschen }
        end;

      if ( y2 > y1 ) then yincr := 1       { Zeileninkrement festlegen }
                     else yincr := -1;

      dx := x2 - x1;
      dy := abs( y2-y1 );
      d  := 2 * dy - dx;
      aincr := 2 * (dy - dx);
      bincr := 2 * dy;
      x := x1;
      y := y1;

      setpix( x, y, farbe );                     { ersten Punkt setzen }
      for x:=x1+1 to x2 do              { Line auf X-Achse durchlaufen }
        begin
          if ( d >= 0 ) then
            begin
              inc( y, yincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          setpix( x, y, farbe );
       end;
    end;
end;

{***********************************************************************
*  GrafPrint: Gibt einen formatierten String im Grafikbildschirm aus   *
**--------------------------------------------------------------------**
*  Eingabe : X, Y   = Startkoordinate (0 - ...)                        *
*            VG     = Vordergrundfarbe                                 *
*            HG     = Hintergrundfarbe (255 = transparent)             *
*            STRING = String mit Formatinformationen                   *
***********************************************************************}

procedure GrafPrint( x, y : integer; vg, hg : byte; strt : string );

var i : integer;                                     { Schleifenzhler }

begin
 for i:=1 to length( strt ) do
   begin
     printchar( strt[i], x, y, vg, hg );     { ber PrintChar ausgeben }
     inc( x, 8 );                  { X auf die nchste Zeichenposition }
   end;
end;

{***********************************************************************
*  ColorBox: Zeichnet ein Rechteckt und fllt es mit einem Muster aus  *
*            Linien.                                                   *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Koordinate der oberen linken Fensterecke         *
*            X2, Y2 = Koordinate der unteren rechnte Fensterecke       *
*            FARBMAX= grter Farbwert                                 *
*  Info    : Die Farben der Linien werden in einem Zyklus von 0 bis    *
*            FARBMAX gewhlt                                           *
***********************************************************************}

procedure ColorBox( x1, y1, x2, y2 : integer; farbmax : byte );

var x, y,                                            { Schleifenzhler }
    sx, sy : integer;          { Ausgangspunkt fr letzte Farbschleife }

begin
 Line( x1, y1, x1, y2, 15 );                { Rahmen um die BOX ziehen }
 Line( x1, y2, x2, y2, 15 );
 Line( x2, y2, x2, y1, 15 );
 Line( x2, y1, x1, y1, 15 );

 for y := y2-1 downto y1+1 do    { von links unten an den rechten Rand }
   Line( x1+1, y2-1, x2-1, y, y mod farbmax );

 for y := y2-1 downto y1+1 do    { von rechts unten an den linken Rand }
  Line( x2-1, y2-1, x1+1, y, y mod farbmax );

 {-- von der Boxmitte an den oberen Rand ------------------------------}

 sx := x1+ (x2-x1) div 2;
 sy := y1+ (y2-y1) div 2;
 for x := x1+1 to x2-1 do
  Line( sx, sy, x, y1+1, x mod farbmax );
end;

{***********************************************************************
*  DrawAxis: Zeichnet die Achsen am linken und oberen Rand des Bild-   *
*            schirms.                                                  *
**--------------------------------------------------------------------**
*  Eingabe : XSTEP = Schrittweite fr die X-Achse                      *
*            YSTEP = Schrittweite fr die Y-Achse                      *
*            VG    = Vordergrundfarbe                                  *
*            HG    = Hintergrundfarbe (255 = transparent)              *
***********************************************************************}

procedure DrawAxis( stepx, stepy : integer; vg, hg : byte );

var x, y     : integer;                           { Schleifenordinaten }
    ordinate : string[3];

begin
 Line( 0, 0, MAXX, 0, vg );                           { X-Achse ziehen }
 Line( 0, 0, 0, MAXY, vg );                           { Y-Achse ziehen }

 x := stepx;                                       { X-Achse skalieren }
 while ( x < MAXX ) do
   begin
     Line( x, 0, x, 5, vg );
     str( x, ordinate );
     if ( x < 100 ) then
       GrafPrint( x - 8 , 8, vg, hg, ordinate )
     else
       GrafPrint( x - 12, 8, vg, hg, ordinate );
     inc( x, stepx );
  end;

 y := stepy;                                       { Y-Achse skalieren }
 while ( y < MAXY ) do
   begin
     Line( 0, y, 5, y, vg );
     str( y:3, ordinate );
     GrafPrint( 8, y-4, vg, hg, ordinate );
     inc( y, stepy );
  end;
end;

{***********************************************************************
*  Demo: Demonstriert den Einsatz der verschiedenen Funktionen aus     *
*        diesem Modul                                                  *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************}

procedure Demo;

const PAUSE = 100;                     { Pausenzhler in Millisekunden }

var seite : byte;                                       { Seitenzhler }

begin
 for seite := 0 to 4 do
   begin
     setpage( seite );                              { Seite bearbeiten }
     ColorBox( 80, 25, 308, 175, ( seite + 1 ) * 16 ); { Farbbox malen }
     DrawAxis( 30, 20, 15, 255 );                      { Axen zeichnen }
     GrafPrint( 46, MAXY-10, 15, 255,
                'V3220P  -  (c) by MICHAEL TISCHER' );
   end;

 {-- die vier Grafikseiten im Wechsel anzeigen ------------------------}

 for seite := 0 to 50 do                               { 50 Durchlufe }
   begin
     showpage( seite mod 4 );                         { Seite anzeigen }
     delay( PAUSE );                                     { kurze Pause }
   end;
end;

{----------------------------------------------------------------------}
{-- H A U P T P R O G R A M M                                      ----}
{----------------------------------------------------------------------}

begin
 if IsVga then                                { VGA-Karte installiert? }
   begin                                               { Ja, los gehts }
     init320200;                          { Grafikmodus initialisieren }
     Demo;
     repeat until keypressed;
     Textmode( CO80 );              { wieder in den Textmodus schalten }
   end
 else
   writeln( 'V3220P  -  (c) 1990 by MICHAEL TISCHER'#13#10#10 +
            'Achtung! Fr die Ausfhrung dieses Programms wird eine ' +
            'VGA-Karte bentigt.'#13#10 );
end.
