;**********************************************************************;
;*                         V 3 2 2 0 P A . A S M                      *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : enthlt verschiedene Routinen zur Arbeit im    *;
;*                     320*200-256-Grafikmodus der VGA-Karte          *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  :  5.09.1990                                     *;
;*    letztes Update : 14.01.1991                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM /mx V3220PA;    oder   TASM -mx V3220PA   *;
;*                     ... dann in V3220P.PAS einbinden               *;
;**********************************************************************;

;== Konstanten =========================================================

SC_INDEX       = 3c4h             ;Indexregister des Sequencer-Ctrl.
SC_MAP_MASK    = 2                ;Nummer des Map-Mask-Registers
SC_MEM_MODE    = 4                ;Nummer des Memory-Mode-Registers

GC_INDEX       = 3ceh             ;Indexregister des Graphics-Ctrl.
GC_READ_MAP    = 4                ;Nummer des Read-Map-Registers
GC_GRAPH_MODE  = 5                ;Nummer des Graphics-Mode-Registers
GC_MISCELL     = 6                ;Nummer des Miscellaneous-Registers

CRTC_INDEX     = 3d4h             ;Indexregistes des CRT-Controllers
CC_MAX_SCAN    = 9                ;Nummer des Maximum-Scan-Line-Reg.
CC_START_HI    = 0Ch              ;Nummer des Hi-Start-Registers
CC_UNDERLINE   = 14h              ;Nummer des Underline-Registers
CC_MODE_CTRL   = 17h              ;Nummer des Mode-Control-Registers

DAC_WRITE_ADR  = 3C8h             ;DAC-Write-Adresse
DAC_READ_ADR   = 3C7h             ;DAC-Read-Adresse
DAC_DATA       = 3C9h             ;DAC-Datenregister

VERT_RETRACE   = 3DAh             ;Input Status-Register #1

PIXX           = 320              ;horizontale Auflsung

;== Datensegment ===========================================================

DATA   segment word public

vio_seg    dw (?)                 ;Video-Segment mit akt. Seite
				  ;mu zur Laufzeit initialisiert werden

DATA   ends

;== Programm ===============================================================

CODE       segment byte public    ;das Programmsegment

	   assume cs:code, ds:data

;-- Public-Deklarationen -----------------------------------------------

public    init320200              ;320*200-Punkte-Modus initialisieren
public    setpix                  ;Punkt setzen
public    getpix                  ;Farbe eines Punktes ermitteln
public    showpage                ;Seite 0 oder 1 anzeigen
public    setpage                 ;Seite fr setpix und getpix festlegen

;-----------------------------------------------------------------------
;-- INIT320200: initialisiert den 320*200 Punkte-Modus
;-- Aufruf von TP: init320200;

init320200 proc near

	   ;-- zunchst ganz normal den Modus 13h einstellen, damit das
	   ;-- BIOS die meiste Arbeit der Initialisierung bernimmt.
	   ;-- Danach die Register verndern, die sich noch nicht in
	   ;-- der gewnschten Stellung befinden.

	   mov   ax,0013h         ;normalen Modus 13h einstellen
	   int   10h

	   mov   dx,GC_INDEX      ;Auftrennung der Speicheradressen
	   mov   al,GC_GRAPH_MODE ;im Graphics-Mode-Register des
	   out   dx,al            ;Gaphics-Controllers ber das
	   inc   dx               ;Bit #4 abschalten
	   in    al,dx
	   and   al,11101111b
	   out   dx,al
	   dec   dx

	   mov   al,GC_MISCELL    ;und das Ganze noch einmal im
	   out   dx,al            ;Miscellanous-Register ber das
	   inc   dx               ;Bit #1
	   in    al,dx
	   and   al,11111101b
	   out   dx,al

	   mov   dx,SC_INDEX      ;das Memory-Mode-Register im
	   mov   al,SC_MEM_MODE   ;Sequencer-Controlller dahingehend
	   out   dx,al            ;modifizieren, da keine Auftrennung
	   inc   dx               ;der Adressen in verschiedene Bit-
	   in    al,dx            ;Planes mehr erfolgt und die aktuelle
	   and   al,11110111b     ;Plane ber das Bit-Mask-Register
	   or    al,4             ;festgelegt wird
	   out   dx,al

	   mov   ax,0A000h        ;alle vier Bit-Planes komplett mit
	   mov   vio_seg,ax       ;dem Farbcode 00h fllen und damit
	   mov   es,ax            ;den Bildschirm zunchst lschen
	   xor   di,di
	   mov   ax,di
	   mov   cx,8000h
	   rep   stosw

	   mov   dx,CRTC_INDEX    ;Doppelwort-Modus ber das Bit 6
	   mov   al,CC_UNDERLINE  ;im Underline-Register des CRT-
	   out   dx,al            ;Controllers abstellen
	   inc   dx
	   in    al,dx
	   and   al,10111111b
	   out   dx,al
	   dec   dx

	   mov   al,CC_MODE_CTRL  ;ber das Bit 6 im Mode-Control-
	   out   dx,al            ;Register des CRT-Controllers jetzt
	   inc   dx               ;vom Word- in den Byte-Modus wechseln
	   in    al,dx
	   or    al,01000000b
	   out   dx,al

	   ret                    ;zurck zum Aufrufer

init320200 endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- SETPIX: verleiht einem Punkt eine bestimmte Farbe
;-- Aufruf von TP: setpix( x , y : integer; farbe : byte );

setpix     proc near

sframe     struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
ret_adr0   dw ?                   ;Rcksprungadresse zum Aufrufer
farbe      dw ?                   ;Farbe
y0         dw ?                   ;Y-Ordinate
x0         dw ?                   ;X-Ordinate
sframe     ends                   ;Ende der Struktur

frame      equ [ bp - bp0 ]       ;adressiert die Elemente der Struktur

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   mov   ax,PIXX / 4      ;Offset in den Video-RAM berechnen
	   mul   frame.y0         ;und nach DI laden
	   mov   cx,frame.x0
	   mov   bx,cx
	   shr   bx,1
	   shr   bx,1
	   add   ax,bx
	   mov   di,ax

	   and   cl,3             ;Bit-Maske fr die zu adressierende
	   mov   ah,1             ;Map in AH berechnen
	   shl   ah,cl
	   mov   al,SC_MAP_MASK   ;nach AL die Registernummer
	   mov   dx,SC_INDEX      ;Indexadresse des Sequencers laden
	   out   dx,ax            ;Bit-Mask-Register laden

	   mov   ax,vio_seg       ;ES auf den Video-RAM setzen
	   mov   es,ax
	   mov   al,byte ptr frame.farbe  ;Farbe des Punktes laden
	   stosb                  ;und in die gewhlte Bit-Map schreiben

	   pop   bp               ;Register wieder vom Stack holen

	   ret   6                ;zurck zum Aufrufer, dabei Argumente
				  ;vom Stack entfernen

setpix     endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- GETPIX: liefert die Farbe eines Punktes
;-- Aufruf von TP: x := getpix( x , y : integer );

getpix     proc near

sframe1    struc                  ;Struktur zum Zugriff auf den Stack
bp1        dw ?                   ;nimmt BP auf
ret_adr1   dw ?                   ;Rcksprungadresse zum Aufrufer
y1         dw ?                   ;Y-Ordinate
x1         dw ?                   ;X-Ordinate
sframe1    ends                   ;Ende der Struktur

frame      equ [ bp - bp1 ]       ;adressiert die Elemente der Struktur

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   mov   ax,PIXX / 4      ;Offset in den Video-RAM berechnen
	   mul   frame.y1         ;und nach SI laden
	   mov   si,frame.x1
	   mov   cx,si
	   shr   si,1
	   shr   si,1
	   add   si,ax

	   and   cl,3             ;Bit-Maske fr die zu adressierende
	   mov   ah,cl            ;Map in AH berechnen
	   mov   al,GC_READ_MAP   ;nach AL die Registernummer
	   mov   dx,GC_INDEX      ;Indexadresse des Graphic-Ctrl laden
	   out   dx,ax            ;Read-Map-Register laden

	   mov   ax,vio_seg       ;ES auf den Video-RAM setzen
	   mov   es,ax
	   mov   al,es:[si]       ;Farbe des Punktes laden

	   pop   bp               ;Register wieder vom Stack holen

	   ret   4                ;zurck zum Aufrufer, dabei Argumente
				  ;vom Stack entfernen

getpix     endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- SETPAGE: Whlt die Seite aus, auf die bei Funktionsaufrufen von
;--          setpix und getpix zugegriffen werden soll
;-- Aufruf von TP: setpage( page : byte );

setpage    proc near

	   pop   bx               ;Rcksprungadresse vom Stack
	   pop   cx               ;Argument vom Stack

	   push  cx               ;und beides wieder rauf
	   push  bx

	   mov   al,4             ;Hi-Byte der Segadr = Seite * 4 + A0h
	   mul   cl
	   or    al,0A0h

	   mov   byte ptr vio_seg + 1,al ;neue Segmentadresse merken

	   ret   2                ;zurck zum Aufrufer, dabei Argumente
				  ;vom Stack entfernen

setpage    endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- SHOWPAGE: Bringt eine der beiden Bildschirmseiten zur Anzeige
;-- Aufruf von TP: showpage( page : byte );

showpage   proc near

	   pop   bx               ;Rcksprungadresse vom Stack
	   pop   cx               ;Argument vom Stack

	   push  cx               ;und beides wieder rauf
	   push  bx

	   mov   al,64            ;Hi-Byte des Offsets = Seite * 64
	   mul   cl
	   mov   ah,al            ;Hi-Byte des Offsets nach AH

	   ;-- jetzt die neue Startadresse laden -----------------------

	   mov   dx,CRTC_INDEX    ;CRT-Controller adressieren
	   mov   al,CC_START_HI   ;Registernummer nach AL
	   out   dx,ax            ;und ab gehts

	   ;-- wieder auf den Start eines Bildschirmaufbaus warten -----

	   mov   dx,VERT_RETRACE  ;zunchst auf Ende des vertikalen
sp3:       in    al,dx            ;Rcklaufs warten
	   test  al,8
	   jne   sp3

sp4:       in    al,dx            ;dann auf den Start des Rcklaufs
	   test  al,8
	   je    sp4

	   ret   2                ;zurck zum Aufrufer, dabei Argumente
				  ;vom Stack entfernen

showpage   endp                   ;Ende der Prozedur

;== Ende ===============================================================

CODE       ends                   ;Ende des Codesegments
	   end                    ;Ende des Programms


