{***********************************************************************
*                            V D A C P . P A S                         *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Programmierung der DAC-Farb-    *
*                     Register in den 256 Farben Grfikmodi der VGA-    *
*                     Karte. Das Programm greift dabei auf die Assem-  *
*                     bler-Routinen aus dem Modul V3240PA.ASM zurck.  *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    :  2.01.1991                                       *
*  letztes Update am: 14.01.1991                                       *
***********************************************************************}

program VDACP;

uses dos, crt;

{-- Typdeklarationen --------------------------------------------------}

type DACREG = record                     { beschreibt ein DAC-Register }
                case integer of
                 0 : ( Rot, Gruen, Blau : BYTE );{ Farbkomponenten RGB }
                 1 : ( RGB : array[ 1..3] of BYTE );
              end;
     DACARRAY = array [0..255] of DACREG;      { komplette DAC-Tabelle }

{-- externe Referenzen auf die Assembler-Routinen ---------------------}

{$L v3240pa}                               { Assembler-Modul einbinden }

procedure init320400; external;
procedure setpix( x, y : integer; farbe : byte ); external;
function  getpix( x, y: integer ) : byte ; external;
procedure setpage( page : byte ); external;
procedure showpage( page : byte ); external;

{-- Konstanten --------------------------------------------------------}

const MAXX = 319;                         { Maximale X- und Y-Ordinate }
      MAXY = 399;

      BREITE  = 10;               { Breite eines Farbblocks in Punkten }
      HOEHE   = 20;                 { Hhe eines Farbblocks in Punkten }
      ABSTAND = 2;                      { Abstand zwischen den Blcken }
      GBREITE = 16 * BREITE + ( 15 * ABSTAND );         { Gesamtbreite }
      GHOEHE  = 16 * HOEHE + ( 15 * ABSTAND );            { Gesamthhe }
      STARTX  = ( MAXX - GBREITE ) div 2;      { obere linke Blockecke }
      STARTY  = ( MAXY - GHOEHE ) div 2;

{***********************************************************************
*  IsVga : Stellt fest, ob eine VGA-Karte installiert ist.             *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : TRUE oder FALSE                                           *
***********************************************************************}

function IsVga : boolean;

var Regs : Registers;          { Prozessorregister fr Interruptaufruf }

begin
  Regs.AX := $1a00;                 { Funktion 1Ah gibt es nur bei VGA }
  Intr( $10, Regs );
  IsVga := ( Regs.AL = $1a );
end;

{***********************************************************************
*  PrintChar : Schreibt ein Zeichen im Grafikmodus in den Bildschirm   *
**--------------------------------------------------------------------**
*  Eingabe :   zeichen = das aufzubauende Zeichen                      *
*              x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************}

procedure PrintChar( zeichen : char; x, y : integer; vg, hg : byte );

type ZDEF = array[0..255,0..7] of byte;       { Aufbau des Zeichensatz }
     ZPTR = ^ZDEF;                            { Zeiger auf Zeichensatz }

var  Regs  : Registers;                 { Register zum Interruptaufruf }
     ch    : char;                      { einzelner Punkt des Zeichens }
     i, k,                                           { Schleifenzhler }
     Maske : byte;                  { Bitmaske fr Aufbau des Zeichens }

const fptr : ZPTR = NIL;               { Zeiger auf Zeichensatz im ROM }

begin
  if fptr = NIL then         { Zeiger auf Zeichensatz schon ermittelt? }
    begin                                                       { Nein }
      Regs.AH := $11;                   { Funktion $11 des Video-BIOS, }
      Regs.AL := $30;                   { Unterfunktion $30 aufrufen   }
      Regs.BH := 3;             { Zeiger auf 8*8 Zeichensatz ermitteln }
      intr( $10, Regs );
      fptr := ptr( Regs.ES, Regs.BP );         { Zeiger zusammensetzen }
    end;

  if ( hg = 255 ) then                 { Zeichen transparent zeichnen? }
    for i := 0 to 7 do             { Ja, nur Vordergrund-Punkte setzen }
      begin
        Maske := fptr^[ord(zeichen),i]; {Bitmuster fr eine Zeile holen}
        for k := 0 to 7 do
          begin
            if ( Maske and 128 <> 0 ) then             { Punkt setzen? }
              setpix( x+k, y+i, vg );                             { Ja }
            Maske := Maske shl 1;
          end;
      end
  else                        { Nein, auch Hintergrund bercksichtigen }
    for i := 0 to 7 do                        { die Zeilen durchlaufen }
      begin
        Maske := fptr^[ord(zeichen),i]; {Bitmuster fr eine Zeile holen}
        for k := 0 to 7 do
          begin
            if ( Maske and 128 <> 0 ) then              { Vordergrund? }
              setpix( x+k, y+i, vg )                              { Ja }
            else
              setpix( x+k, y+i, hg );              { Nein, Hintergrund }
            Maske := Maske shl 1;
         end;
      end;
end;

{***********************************************************************
*  Line: Zieht eine Linie nach dem Bresenham-Algorithmus               *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endekoordinate                                   *
*            FARBE  = Farbe der Linienpunkte                           *
***********************************************************************}

procedure Line( x1, y1, x2, y2 : integer; farbe : byte );

var d, dx, dy,
    aincr, bincr,
    xincr, yincr,
    x, y                 : integer;

{-- Unterprozedur, vertauscht zwei Integer-Variablen ------------------}

procedure SwapInt( var i1, i2: integer );

var dummy : integer;

begin
  dummy := i2;
  i2    := i1;
  i1    := dummy;
end;

{-- Hauptprozedur -----------------------------------------------------}

begin
  if ( abs(x2-x1) < abs(y2-y1) ) then   { ber X- oder Y-Achse laufen? }
    begin                                           { ber die Y-Achse }
      if ( y1 > y2 ) then                              { y1 grer y2? }
        begin
          SwapInt( x1, x2 );                   { Ja, X1 mit X2 und     }
          SwapInt( y1, y2 );                   { Y1 mit Y2 vertauschen }
        end;

      if ( x2 > x1 ) then xincr := 1      { Spalteninkrement festlegen }
                     else xincr := -1;

      dy := y2 - y1;
      dx := abs( x2-x1 );
      d  := 2 * dx - dy;
      aincr := 2 * (dx - dy);
      bincr := 2 * dx;
      x := x1;
      y := y1;

      setpix( x, y, farbe );                     { ersten Punkt setzen }
      for y:=y1+1 to y2 do              { Line auf X-Achse durchlaufen }
        begin
          if ( d >= 0 ) then
            begin
              inc( x, xincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          setpix( x, y, farbe );
        end;
    end
  else                                              { ber die X-Achse }
    begin
      if ( x1 > x2 ) then                              { x1 grer x2? }
        begin
          SwapInt( x1, x2 );                   { Ja, X1 mit X2 und     }
          SwapInt( y1, y2 );                   { Y1 mit Y2 vertauschen }
        end;

      if ( y2 > y1 ) then yincr := 1       { Zeileninkrement festlegen }
                     else yincr := -1;

      dx := x2 - x1;
      dy := abs( y2-y1 );
      d  := 2 * dy - dx;
      aincr := 2 * (dy - dx);
      bincr := 2 * dy;
      x := x1;
      y := y1;

      setpix( x, y, farbe );                     { ersten Punkt setzen }
      for x:=x1+1 to x2 do              { Line auf X-Achse durchlaufen }
        begin
          if ( d >= 0 ) then
            begin
              inc( y, yincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          setpix( x, y, farbe );
       end;
    end;
end;

{***********************************************************************
*  GrafPrint: Gibt einen formatierten String im Grafikbildschirm aus   *
**--------------------------------------------------------------------**
*  Eingabe : X, Y   = Startkoordinate (0 - ...)                        *
*            VG     = Vordergrundfarbe                                 *
*            HG     = Hintergrundfarbe (255 = transparent)             *
*            STRING = String mit Formatinformationen                   *
***********************************************************************}

procedure GrafPrint( x, y : integer; vg, hg : byte; strt : string );

var i : integer;                                     { Schleifenzhler }

begin
 for i:=1 to length( strt ) do
   begin
     printchar( strt[i], x, y, vg, hg );     { ber PrintChar ausgeben }
     inc( x, 8 );                  { X auf die nchste Zeichenposition }
   end;
end;

{***********************************************************************
*  GetDac: Liefert den Inhalt einer bestimmten Anzahl von DAC-Registern*
**--------------------------------------------------------------------**
*  Eingabe : FIRST = Nummer des ersten DAC-Registers (0-255)           *
*            ANZ   = Anzahl der DAC-Register                           *
*            BUF   = Puffer, in den die Inhalte der DAC-Register ge-   *
*                    laden werden sollen. Dabei mu es sich um eine    *
*                    Variable vom Typ DACREG oder ein Array dieses     *
*                    Typs handeln.                                     *
*  Info    : Der bergebene Puffer mu fr jedes der zu lesenden DAC-  *
*            Register drei Bytes bereithalten, in denen der Rot-, Grn-*
*            und Blauanteil der jeweiligen Farbe festgehalten wird     *
***********************************************************************}

procedure GetDac( First, Anz : integer; var Buf );

var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }

begin
  Regs.AX := $1017;                 { Funktions- und Unterfunktionsnr. }
  Regs.BX := First;                  { Nummer des ersten DAC-Registers }
  Regs.CX := Anz;                    { Anzahl der zu ladenden Register }
  Regs.ES := seg( Buf );                 { Zeiger auf den Puffer laden }
  Regs.DX := ofs( Buf );
  intr( $10, Regs );                   { BIOS-Video-Interrupt aufrufen }
end;

{***********************************************************************
*  SetDac: Ldt eine bestimmten Anzahl von DAC-Registern               *
**--------------------------------------------------------------------**
*  Eingabe : FIRST = Nummer des ersten DAC-Registers (0-255)           *
*            ANZ   = Anzahl der DAC-Register                           *
*            BUF   = Puffer, aus dem die Inhalte der DAC-Register ent- *
*                    nommen werden sollen. Dabei mu es sich um eine   *
*                    Variable vom Typ DACREG oder ein Array dieses     *
*                    Typs handeln.                                     *
*  Info    : Siehe GetDac                                              *
***********************************************************************}

procedure SetDac( First, Anz : integer; var Buf );

var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }

begin
  Regs.AX := $1012;                 { Funktions- und Unterfunktionsnr. }
  Regs.BX := First;                  { Nummer des ersten DAC-Registers }
  Regs.CX := Anz;                    { Anzahl der zu ladenden Register }
  Regs.ES := seg( Buf );                 { Zeiger auf den Puffer laden }
  Regs.DX := ofs( Buf );
  intr( $10, Regs );                   { BIOS-Video-Interrupt aufrufen }
end;

{***********************************************************************
*  PrintDac: Gibt den Inhalt eines DAC-Registers auf dem Bildschirm    *
*            aus und stellt die Farbe im DAC-Register 255 ein          *
**--------------------------------------------------------------------**
*  Eingabe : DREG  = Das DAC-Register                                  *
*            NR    = Die Nummer dieses Registers                       *
*            FARBE = Ausgabefarbe                                      *
***********************************************************************}

procedure PrintDac( DReg : DACREG; Nr, Farbe : BYTE );

var nrstr,                                { String fr Register-Nummer }
    rstr,                                      { String fr Rot-Anteil }
    gstr,                                     { String fr Grn-Anteil }
    bstr : string[3];                         { String fr Blau-Anteil }

begin
  SetDac( 255, 1, DReg );                  { Farbe in DAC-Register 255 }
  str( Nr : 3, nrstr );          { Farben und Nr. in Strings umwandeln }
  str( DReg.Rot : 2, rstr );
  str( DReg.Gruen : 2, gstr );
  str( DReg.Blau : 2, bstr );
  GrafPrint( 60, MAXY-10, Farbe, 0, 'DAC:' + nrstr + '  R:' + rstr +
                                    '  G:' + gstr + '  B:' + bstr);
end;

{***********************************************************************
*  Umrahmen: Zieht einen Rahmen um eines der Farbfelder                *
**--------------------------------------------------------------------**
*  Eingabe : X     = X-Ordinate des Farbfeldes (0-15)                  *
*            Y     = Y-Ordinate des Farbfeldes (0-15)                  *
*            FARBE = Farbe des Rahmens                                 *
*  Info    : Die Linienstrke betrgt unabhngig des Abstands der      *
*            Farbfelder einen Punkt.                                   *
***********************************************************************}

procedure Umrahmen( X, Y, Farbe : BYTE );

var sx, sy,                             { obere linke Ecke des Rahmens }
    ex, ey : integer;                 { untere rechte Ecke des Rahmens }

begin
  sx := STARTX + X * (BREITE + ABSTAND) - 1;       { Eck-Ordinaten des }
  ex := sx + BREITE + 1;                           { Rahmens berechnen }
  sy := STARTY + Y * (HOEHE + ABSTAND) - 1;
  ey := sy + HOEHE + 1;
  Line( sx, sy, ex, sy, Farbe );                       { Rahmen ziehen }
  Line( ex, sy, ex, ey, Farbe );
  Line( ex, ey, sx, ey, Farbe );
  Line( sx, ey, sx, sy, Farbe );
end;

{***********************************************************************
*  ChangeDacReg: Verndert den Inhalt eines DAC-Registers im Speicher  *
*                und in der DAC-Tabelle der Video-Karte und gibt ihn   *
*                anschlieend auf dem Bildschirm aus                   *
**--------------------------------------------------------------------**
*  Eingabe : DREG  = Zu vernderndes DAC-Register                      *
*            NR    = Nummer des DAC-Registers                          *
*            KOMP  = Nummer der zu verndernden Komponente (1-3)       *
*                     1 = Rot, 2 = Gruen, 3 = Blau                     *
*            INKR  = Inkrement fr diese Komponente                    *
***********************************************************************}

procedure ChangeDacReg( var DReg : DACREG; Nr, Komp : BYTE;
                            Inkr : integer );

begin
  inc( DReg.RGB[ Komp ], Inkr );           { Komponente inkrementieren }
  if DReg.RGB[ Komp ] > 63 then                             { ber 63? }
    DReg.RGB[ Komp ] := 0;                      { Jsa, auf Null setzen }
  SetDac( Nr, 1, DReg );                          { DAC-Register laden }
  PrintDac( DReg, Nr, 15 );                    { neuen Inhalt ausgeben }
end;

{***********************************************************************
*  Demo: Demonstriert die Programmierung der DAC-Register und das      *
*        Farbmodell der VGA-Karte                                      *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************}

procedure Demo;

var x,  y,
    ix, jx,
    iy, jy,
    k,  f  : integer;                                { Schleifenzhler }
    ch     : char;                                             { Taste }
    dacbuf : DACARRAY;              { Array mit kompletter DAC-Tabelle }
    DReg   : DACREG;               { das jeweils aktuelle DAC-Register }

begin
  {-- Bildschirm aufbauen ---------------------------------------------}

  SetPage( 0 );                                   { Seite 0 bearbeiten }
  ShowPage( 0 );                                    { Seite 0 anzeigen }
  GetDac( 0, 256, dacbuf );              { komplette DAC-Tabelle laden }

  GrafPrint( 10, 0, 255, 0,
             'VDACP  -  (c) 1991 by MICHAEL TISCHER' );

  {-- den Block aus 16*16 Farbfeldern aufbauen ------------------------}

  iy := STARTY;                    { Ausgangspunkt auf dem Bildschirm  }
  jy := STARTY + HOEHE - 1;
  f  := 0;
  for y := 0 to 15 do                 { die 16 Blockzeilen durchlaufen }
    begin
      ix := STARTX;
      jx := STARTX + BREITE - 1;
      for x := 0 to 15 do           { die 16 Blockspaolten durchlaufen }
        begin
          for k := iy to jy do     { Block aus einzelnen Linien formen }
            Line( ix, k, jx, k, f );
          inc( ix, BREITE + ABSTAND );         { nchster Block rechts }
          inc( jx, BREITE + ABSTAND );
          inc( f );          { Farbe fr nchsten Block inkrementieren }
        end;
      inc( iy, HOEHE + ABSTAND );            { Ausgabepos. fr nchste }
      inc( jy, HOEHE + ABSTAND );            { Blockzeile              }
    end;

  {-- Eingaben des Anwenders abfragen und darauf reagieren ------------}

  ix := 0;         { in der oberen linken Ecke mit der Farbe 0 starten }
  iy := 0;
  jx := 0;
  jy := 0;
  k := 0;
  GetDac( 0, 1, DReg );                                { Farbe 0 holen }
  Umrahmen( 0, 0, 15 );                            { Farbfeld umrahmen }
  PrintDac( DReg, 0, 15 );                       { und Inhalt ausgeben }
  repeat
    ch := ReadKey;                                    { Taste abfragen }
    if ( ch <> #0 ) then                     { keine erweiterte Taste? }
      case ch of                                     { Nein, auswerten }
        'r' : ChangeDacReg( DReg, k, 1, +1 );             { r = Rot +  }
        'g' : ChangeDacReg( DReg, k, 2, +1 );             { g = Grn + }
        'b' : ChangeDacReg( DReg, k, 3, +1 );             { b = Blau + }
        'R' : ChangeDacReg( DReg, k, 1, -1 );             { R = Rot -  }
        'G' : ChangeDacReg( DReg, k, 2, -1 );             { G = Grn - }
        'B' : ChangeDacReg( DReg, k, 3, -1 );             { B = Blau - }
        ' ' : begin           { Space = ursprnglichen Wert einstellen }
                DReg := dacbuf[ k ];
                ChangeDacReg( DReg, k, 1, 0 );
              end;
      end
    else                                    { erweiterter Tastaturcode }
      case ReadKey of
        #72 : if ( iy = 0 ) then                         { Cursor oben }
                jy := 15
              else
                jy := iy - 1;

        #80 : if ( iy = 15 ) then                       { Cursor unten }
                jy := 0
              else
                jy := iy + 1;

        #75 : if ( ix = 0  ) then                       { Cursor links }
                jx := 15
              else
                jx := ix - 1;

        #77 : if ( ix = 15 ) then                      { Cursor rechts }
                jx := 0
              else
                jx := ix + 1;
      end;

    if ( ix <> jx ) or ( iy <> jy ) then       { neue Cursor-Position? }
      begin                                                       { Ja }
        Umrahmen( ix, iy, 0 );       { Rahmen an alter Pos. ausblenden }
        Umrahmen( jx, jy, 15 );           { Rahmen um neue Pos. ziehen }
        ix := jx;                              { neues Farbfeld merken }
        iy := jy;
        k := iy*16+ix;              { Nummer des neuen Felds berechnen }
        GetDac( k, 1, DReg );                     { DAC-Register laden }
        PrintDac( DReg, k, 15 );                  { und ausgeben       }
      end;
  until ch = #13;                   { wiederholen, bis RETURN bettigt }

  SetDac( 0, 256, dacbuf );          { DAC-Tabelle wieder restaurieren }
end;

{----------------------------------------------------------------------}
{-- H A U P T P R O G R A M M                                      ----}
{----------------------------------------------------------------------}

begin
 if IsVga then                                { VGA-Karte installiert? }
   begin                                               { Ja, los gehts }
     init320400;                          { Grafikmodus initialisieren }
     Demo;
     Textmode( CO80 );              { wieder in den Textmodus schalten }
   end
 else
   writeln( 'VDACP  -  (c) 1991 by MICHAEL TISCHER'#13#10#10 +
            'Achtung! Fr die Ausfhrung dieses Programms wird eine ' +
            'VGA-Karte bentigt.'#13#10 );
end.
