{***********************************************************************
*                        V O N O F F P . P A S                         *
**--------------------------------------------------------------------**
*  Aufgabe        : Demonstriert die An- und Abschaltung des Video-    *
*                   bilds bei EGA- und VGA-Karten.                     *
**--------------------------------------------------------------------**
*  Autor          : MICHAEL TISCHER                                    *
*  entwickelt am  :  5.08.1990                                         *
*  letztes Update : 14.01.1991                                         *
***********************************************************************}

program VOnOffP;

uses DOS, CRT;                                       { Units einbinden }

{-- Konstanten --------------------------------------------------------}

const EV_STATC         = $3DA;  { EGA-/VGA-Karten Statusregister Color }
      EV_STATM         = $3BA;   { EGA-/VGA-Karten Statusregister Mono }
      EV_ATTR          = $3C0;   { EGA-/VGA-Karten Attribut-Controller }

procedure CLI; inline( $FA );                { Interrupts unterdrcken }
procedure STI; inline( $FB );             { Interrupts wieder zulassen }

{***********************************************************************
*  ScrOff : Schaltet den Bildschirm bei EGA- und VGA-Karten ab.        *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************}

procedure ScrOff;

var dummy : BYTE;                   { zur Aufnahme des Registerinhalts }

begin
  cli;                                        { keine Interrupts jetzt }
  dummy := port[EV_STATC];                { Reset auf Status-Reg Color }
  dummy := port[EV_STATM];                 { Reset auf Status-Reg Mono }
  port[EV_ATTR] := $00;         { Bit 5 ausblenden, dadurch Verbindung }
                                { zum CRT-Controller unterbinden       }
  sti;                                          { Interrupts wieder an }
end;

{***********************************************************************
*  ScrOn : Schaltet den Bildschirm bei EGA- und VGA-Karten an.         *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************}

procedure ScrOn;

var dummy : BYTE;                   { zur Aufnahme des Registerinhalts }

begin
  cli;                                        { keine Interrupts jetzt }
  dummy := port[EV_STATC];                { Reset auf Status-Reg Color }
  dummy := port[EV_STATM];                 { Reset auf Status-Reg Mono }
  port[EV_ATTR] := $20;         { Bit 5 einblenden, dadurch Verbindung }
                                { zum CRT-Controller wieder herstellen }
  sti;                                          { Interrupts wieder an }
end;

{***********************************************************************
*  IsEgaVga : Stellt fest, ob eine EGA- oder VGA-Karte installiert ist.*
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : TRUE, wenn EGA- oder VGA-KArte, sonst FALSE               *
***********************************************************************}

function IsEgaVga : boolean;

var Regs : Registers;          { Prozessorregister fr Interruptaufruf }

begin
  Regs.AX := $1a00;                 { Funktion 1Ah gibt es nur bei VGA }
  Intr( $10, Regs );
  if ( Regs.AL = $1a ) then              { ist die Funktion verfgbar? }
    IsEgaVGa := TRUE
  else
    begin
      Regs.ah := $12;                     { Funktion 12h Unterfunktion }
      Regs.bl := $10;                     { 10h aufrufen               }
      intr($10, Regs);                           { Video-BIOS aufrufen }
      IsEgaVga := ( Regs.bl <> $10 );
    end;
end;

{**********************************************************************}
{-- H A U P T P R O G R A M M                                        --}
{**********************************************************************}

var ch : char;                                  { nimmt eine Taste auf }

begin
  ClrScr;
  writeln( 'VONOFFP  -  (c) 1990 by MICHAEL TISCHER'#13#10 );
  if IsEgaVga then                              { EGA- oder VGA-Karte? }
    begin                                                  { Ja, do it }
      writeln( 'Achtung! Nach fnf Sekunden wird der Bildschirm '+
               'abgeschaltet. ' );
      writeln( 'Bettigen Sie dann eine beliebige Taste, um ihn wieder'+
            ' anzuschalten... ' );
      Delay( 5000 );                            { fnf Sekunden warten }
      while KeyPressed do   { alle Tasten aus Tastaturpuffer entfernen }
        ch := ReadKey;
      ScrOff;                                         { Bildschirm aus }
      ch := ReadKey;                                { auf Taste warten }
      ScrOn;                                    { Bildschirm wieder an }
      writeln ( #13#10#10#10 + 'Das war`s ...' );
    end
  else                                      { Nein, keine EGA oder VGA }
    writeln( 'Achtung! Dieses Programm bentigt eine ' +
             'EGA- oder VGA-Karte.' );
end.
