{**********************************************************************}
{                        W I N D A P . P A S                           }
{----------------------------------------------------------------------}
{    Aufgabe       : Feststellen ob und in welchem Modus Windows       }
{                    aktiv ist.                                        }
{----------------------------------------------------------------------}
{    Autor         : Michael Tischer                                   }
{    entwickelt am : 22.08.1991                                        }
{    letztes Update: 22.10.1991                                        }
{**********************************************************************}

uses Dos;                                     { die DOS-Unit einbinden }

const MULTIPLEX  = $2F;     { Interrupt-Nummer des Multiplex-Interrupt }
      NO_WIN     = $00;                          { Windows nicht aktiv }
      W_386_X    = $01;                      { Windows /386 V2.X luft }
      W_REAL     = $81;                  { Windows luft im Real-Modus }
      W_STANDARD = $82;              { Windows luft im Standard-Modus }
      W_ENHANCED = $83;           { Windows luft im Erweiterten-Modus }

{**********************************************************************}
{ WINDOWS : stellt fest ob Windows aktiv ist                           }
{ Eingabe : HVERSION = Integer-Variable, in die die Hauptversions-     }
{                      nummer eingetragen wird                         }
{           NVERSION = Integer-Variable, in die die Unterversions-     }
{                      nummer eingetragen wird                         }
{ Ausgabe : Windows-Status, eine der Konstanten NO_WIN, W_386_X,       }
{           W_REAL, W_STANDARD oder W_ENHANCED                         }
{ Info    : Die Versionsnummer kann nur im Enhanced-Modus von Windows  }
{           3 ermittelt werden                                         }
{**********************************************************************}

function windows( var HVersion, NVersion : integer ) : integer;

var regs : registers;                  { Register fr Interrupt-Aufruf }
    Erg  : integer;

{-- Diese Funktion ersetzt den Aufruf von intr( $2F, Regs ) mit -------}
{-- Regs.ax = $1600 (Installationstest Erweiterter-Modus), der  -------}
{-- Aufruf ber die Pascalfunktion liefert falsche Werte        -------}

function int2fcall : integer;

begin
  inline( $b8 / $00 / $16 /                      { mov   ax,1600h      }
          $cd / $2f /                            { int   2Fh           }
          $89 / $46 / $FE );                     { mov   [bp-2], ax    }
  {  An dieser Stelle fgt der Compiler noch "mov ax, [bp-2]" ein, um  }
  {  die lokale Funktionsvariable in das Rckgaberegister zu laden     }
end;

begin
  HVersion := 0;                        { Initialisiere Versionsnummer }
  NVersion := 0;

  {-- Identifiziere Windows x.y im Erweiterten-Modus ------------------}

  erg := int2fcall;              { Installationstest Erweiterter-Modus }

  case ( lo(Erg) ) of
    $01,
    $FF:  begin
            HVersion := 2;                              { Hauptversion }
            NVersion := 0;                    { Nebenversion unbekannt }
            Windows := W_386_X;
          end;
    $00,
    $80:  begin
            regs.ax := $4680;  { Real- u. Standardmodus identifizieren }
            intr( MULTIPLEX, regs );
            if ( regs.al = $80 ) then
              Windows := NO_WIN                  { Windows luft nicht }
            else
              begin
                {-- Windows im Real- oder Standardmodus ---------------}

                regs.ax := $1605;   { Installation DOS-Extender simul. }
                regs.bx := $0000;
                regs.si := $0000;
                regs.cx := $0000;
                regs.es := $0000;
                regs.ds := $0000;
                regs.dx := $0001;
                intr( MULTIPLEX, regs );
                if ( regs.cx = $0000 ) then
                  begin
                    {-- Windows im Real-Modus -------------------------}

                    regs.ax := $1606;
                    intr( MULTIPLEX, regs );
                    Windows := W_REAL;
                  end
                else
                  Windows := W_STANDARD;
              end;
          end;

    {-- Windows im Erweiterten-Modus, ax enthlt Versionsnummer -------}

    else
      begin
        HVersion := lo(Erg);                { Windows Version ausgeben }
        NVersion := hi(Erg);
        Windows := W_ENHANCED;          { Windows im Erweiterten-Modus }
      end;
  end;
end;

{**********************************************************************}
{                     H A U P T P R O G R A M M                        }
{**********************************************************************}

var WindowsAktiv,                                      { Windows-Modus }
    HVer,                                   { Hauptversion von Windows }
    NVer         : integer;                 { Nebenversion von Windows }

begin
  writeln( ' WINDAP  -  (c) 1991 by Michael Tischer ' );
  writeln;
  WindowsAktiv := windows( HVer, NVer );
  case ( WindowsAktiv ) of
    NO_WIN:     writeln( 'Windows nicht aktiv ' );
    W_REAL:     writeln( 'Windows im Real-Modus' );
    W_STANDARD: writeln( 'Windows aktiv im Standard-Modus' );
    W_386_X:    writeln( 'Windows/386 V 2.x aktiv' );
    W_ENHANCED: writeln( 'Windows V ', Hver, '.', NVer,
                         ' aktiv im erweiterten Modus' );
  end;
  halt( WindowsAktiv );
end.
